//javascript/closure/base.js
// Copyright 2006 The Closure Library Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS-IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * @fileoverview Bootstrap for the Google JS Library (Closure).
 *
 * In uncompiled mode base.js will write out Closure's deps file, unless the
 * global <code>CLOSURE_NO_DEPS</code> is set to true.  This allows projects to
 * include their own deps file(s) from different locations.
 *
 * @author pupius@google.com (Dan Pupius)
 * @author arv@google.com (Erik Arvidsson)
 *
 * @provideGoog
 */


/**
 * @define {boolean} Overridden to true by the compiler when --closure_pass
 *     or --mark_as_compiled is specified.
 */
var COMPILED = false;


/**
 * Base namespace for the Closure library.  Checks to see goog is already
 * defined in the current scope before assigning to prevent clobbering if
 * base.js is loaded more than once.
 *
 * @const
 */
var goog = goog || {};


/**
 * Reference to the global context.  In most cases this will be 'window'.
 */
goog.global = this;


/**
 * A hook for overriding the define values in uncompiled mode.
 *
 * In uncompiled mode, {@code CLOSURE_UNCOMPILED_DEFINES} may be defined before
 * loading base.js.  If a key is defined in {@code CLOSURE_UNCOMPILED_DEFINES},
 * {@code goog.define} will use the value instead of the default value.  This
 * allows flags to be overwritten without compilation (this is normally
 * accomplished with the compiler's "define" flag).
 *
 * Example:
 * <pre>
 *   var CLOSURE_UNCOMPILED_DEFINES = {'goog.DEBUG': false};
 * </pre>
 *
 * @type {Object.<string, (string|number|boolean)>|undefined}
 */
goog.global.CLOSURE_UNCOMPILED_DEFINES;


/**
 * A hook for overriding the define values in uncompiled or compiled mode,
 * like CLOSURE_UNCOMPILED_DEFINES but effective in compiled code.  In
 * uncompiled code CLOSURE_UNCOMPILED_DEFINES takes precedence.
 *
 * Also unlike CLOSURE_UNCOMPILED_DEFINES the values must be number, boolean or
 * string literals or the compiler will emit an error.
 *
 * While any @define value may be set, only those set with goog.define will be
 * effective for uncompiled code.
 *
 * Example:
 * <pre>
 *   var CLOSURE_DEFINES = {'goog.DEBUG': false};
 * </pre>
 *
 * @type {Object.<string, (string|number|boolean)>|undefined}
 */
goog.global.CLOSURE_DEFINES;


/**
 * Returns true if the specified value is not undefined.
 * WARNING: Do not use this to test if an object has a property. Use the in
 * operator instead.
 *
 * @param {?} val Variable to test.
 * @return {boolean} Whether variable is defined.
 */
goog.isDef = function(val) {
  // void 0 always evaluates to undefined and hence we do not need to depend on
  // the definition of the global variable named 'undefined'.
  return val !== void 0;
};


/**
 * Builds an object structure for the provided namespace path, ensuring that
 * names that already exist are not overwritten. For example:
 * "a.b.c" -> a = {};a.b={};a.b.c={};
 * Used by goog.provide and goog.exportSymbol.
 * @param {string} name name of the object that this file defines.
 * @param {*=} opt_object the object to expose at the end of the path.
 * @param {Object=} opt_objectToExportTo The object to add the path to; default
 *     is |goog.global|.
 * @private
 */
goog.exportPath_ = function(name, opt_object, opt_objectToExportTo) {
  var parts = name.split('.');
  var cur = opt_objectToExportTo || goog.global;

  // Internet Explorer exhibits strange behavior when throwing errors from
  // methods externed in this manner.  See the testExportSymbolExceptions in
  // base_test.html for an example.
  if (!(parts[0] in cur) && cur.execScript) {
    cur.execScript('var ' + parts[0]);
  }

  // Certain browsers cannot parse code in the form for((a in b); c;);
  // This pattern is produced by the JSCompiler when it collapses the
  // statement above into the conditional loop below. To prevent this from
  // happening, use a for-loop and reserve the init logic as below.

  // Parentheses added to eliminate strict JS warning in Firefox.
  for (var part; parts.length && (part = parts.shift());) {
    if (!parts.length && goog.isDef(opt_object)) {
      // last part and we have an object; use it
      cur[part] = opt_object;
    } else if (cur[part]) {
      cur = cur[part];
    } else {
      cur = cur[part] = {};
    }
  }
};


/**
 * Defines a named value. In uncompiled mode, the value is retreived from
 * CLOSURE_DEFINES or CLOSURE_UNCOMPILED_DEFINES if the object is defined and
 * has the property specified, and otherwise used the defined defaultValue.
 * When compiled the default can be overridden using the compiler
 * options or the value set in the CLOSURE_DEFINES object.
 *
 * @param {string} name The distinguished name to provide.
 * @param {string|number|boolean} defaultValue
 */
goog.define = function(name, defaultValue) {
  var value = defaultValue;
  if (!COMPILED) {
    if (goog.global.CLOSURE_UNCOMPILED_DEFINES &&
        Object.prototype.hasOwnProperty.call(
            goog.global.CLOSURE_UNCOMPILED_DEFINES, name)) {
      value = goog.global.CLOSURE_UNCOMPILED_DEFINES[name];
    } else if (goog.global.CLOSURE_DEFINES &&
        Object.prototype.hasOwnProperty.call(
            goog.global.CLOSURE_DEFINES, name)) {
      value = goog.global.CLOSURE_DEFINES[name];
    }
  }
  goog.exportPath_(name, value);
};


/**
 * @define {boolean} DEBUG is provided as a convenience so that debugging code
 * that should not be included in a production js_binary can be easily stripped
 * by specifying --define goog.DEBUG=false to the JSCompiler. For example, most
 * toString() methods should be declared inside an "if (goog.DEBUG)" conditional
 * because they are generally used for debugging purposes and it is difficult
 * for the JSCompiler to statically determine whether they are used.
 */
goog.DEBUG = true;


/**
 * @define {string} LOCALE defines the locale being used for compilation. It is
 * used to select locale specific data to be compiled in js binary. BUILD rule
 * can specify this value by "--define goog.LOCALE=<locale_name>" as JSCompiler
 * option.
 *
 * Take into account that the locale code format is important. You should use
 * the canonical Unicode format with hyphen as a delimiter. Language must be
 * lowercase, Language Script - Capitalized, Region - UPPERCASE.
 * There are few examples: pt-BR, en, en-US, sr-Latin-BO, zh-Hans-CN.
 *
 * See more info about locale codes here:
 * http://www.unicode.org/reports/tr35/#Unicode_Language_and_Locale_Identifiers
 *
 * For language codes you should use values defined by ISO 693-1. See it here
 * http://www.w3.org/WAI/ER/IG/ert/iso639.htm. There is only one exception from
 * this rule: the Hebrew language. For legacy reasons the old code (iw) should
 * be used instead of the new code (he), see http://wiki/Main/IIISynonyms.
 */
goog.define('goog.LOCALE', 'en');  // default to en


/**
 * @define {boolean} Whether this code is running on trusted sites.
 *
 * On untrusted sites, several native functions can be defined or overridden by
 * external libraries like Prototype, Datejs, and JQuery and setting this flag
 * to false forces closure to use its own implementations when possible.
 *
 * If your JavaScript can be loaded by a third party site and you are wary about
 * relying on non-standard implementations, specify
 * "--define goog.TRUSTED_SITE=false" to the JSCompiler.
 */
goog.define('goog.TRUSTED_SITE', true);


/**
 * @define {boolean} Whether a project is expected to be running in strict mode.
 *
 * This define can be used to trigger alternate implementations compatible with
 * running in EcmaScript Strict mode or warn about unavailable functionality.
 * See https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Functions_and_function_scope/Strict_mode
 */
goog.define('goog.STRICT_MODE_COMPATIBLE', false);


/**
 * Creates object stubs for a namespace.  The presence of one or more
 * goog.provide() calls indicate that the file defines the given
 * objects/namespaces.  Provided objects must not be null or undefined.
 * Build tools also scan for provide/require statements
 * to discern dependencies, build dependency files (see deps.js), etc.
 * @see goog.require
 * @param {string} name Namespace provided by this file in the form
 *     "goog.package.part".
 */
goog.provide = function(name) {
  if (!COMPILED) {
    // Ensure that the same namespace isn't provided twice.
    // A goog.module/goog.provide maps a goog.require to a specific file
    if (goog.isProvided_(name)) {
      throw Error('Namespace "' + name + '" already declared.');
    }
  }

  goog.constructNamespace_(name);
};


/**
 * @param {string} name Namespace provided by this file in the form
 *     "goog.package.part".
 * @param {Object=} opt_obj The object to embed in the namespace.
 */
goog.constructNamespace_ = function(name, opt_obj) {
  if (!COMPILED) {
    delete goog.implicitNamespaces_[name];

    var namespace = name;
    while ((namespace = namespace.substring(0, namespace.lastIndexOf('.')))) {
      if (goog.getObjectByName(namespace)) {
        break;
      }
      goog.implicitNamespaces_[namespace] = true;
    }
  }

  goog.exportPath_(name, opt_obj);
};


/**
 * goog.module serves two purposes:
 * - marks a file that must be loaded as a module
 * - reserves a namespace (it can not also be goog.provided)
 * and has three requirements:
 * - goog.module may not be used in the same file as goog.provide.
 * - goog.module must be the first statement in the file.
 * - only one goog.module is allowed per file.
 * When a goog.module annotated file is loaded, it is loaded enclosed in
 * a strict function closure. This means that:
 * - any variable declared in a goog.module file are private to the file,
 * not global. Although the compiler is expected to inline the module.
 * - The code must obey all the rules of "strict" JavaScript.
 * - the file will be marked as "use strict"
 *
 * NOTE: unlike goog.provide, goog.module does not declare any symbols by
 * itself.
 *
 * @param {string} name Namespace provided by this file in the form
 *     "goog.package.part", is expected but not required.
 */
goog.module = function(name) {
  if (!goog.isString(name) || !name) {
    throw Error('Invalid module identifier');
  }
  if (!goog.isInModuleLoader_()) {
    throw Error('Module ' + name + ' has been loaded incorrectly.');
  }
  if (goog.moduleLoaderState_.moduleName) {
    throw Error('goog.module may only be called once per module.');
  }

  // Store the module name for the loader.
  goog.moduleLoaderState_.moduleName = name;
  if (!COMPILED) {
    // Ensure that the same namespace isn't provided twice.
    // A goog.module/goog.provide maps a goog.require to a specific file
    if (goog.isProvided_(name)) {
      throw Error('Namespace "' + name + '" already declared.');
    }
    delete goog.implicitNamespaces_[name];
  }
};


/**
 * @param {string} name The module identifier.
 * @return {?} The module exports for an already loaded module or null.
 *
 * Note: This is not an alternative to goog.require, it does not
 * indicate a hard dependency, instead it is used to indicate
 * an optional dependency or to access the exports of a module
 * that has already been loaded.
 */
goog.module.get = function(name) {
  return goog.module.getInternal_(name);
};


/**
 * @param {string} name The module identifier.
 * @return {?} The module exports for an already loaded module or null.
 * @private
 */
goog.module.getInternal_ = function(name) {
  if (!COMPILED) {
    if (goog.isProvided_(name)) {
      // goog.require only return a value with-in goog.module files.
      return name in goog.loadedModules_ ?
          goog.loadedModules_[name] :
          goog.getObjectByName(name);
    } else {
      return null;
    }
  }
};


/**
 * @private {{
 *     moduleName:(string|undefined),
 *     declareTestMethods:boolean}|null}
 */
goog.moduleLoaderState_ = null;


/**
 * @private
 * @return {boolean} Whether a goog.module is currently being initialized.
 */
goog.isInModuleLoader_ = function() {
  return goog.moduleLoaderState_ != null;
};


/**
 * Indicate that a module's exports that are known test methods should
 * be copied to the global object.  This makes the test methods visible to
 * test runners that inspect the global object.
 *
 * TODO(johnlenz): Make the test framework aware of goog.module so
 * that this isn't necessary. Alternately combine this with goog.setTestOnly
 * to minimize boiler plate.
 */
goog.module.declareTestMethods = function() {
  if (!goog.isInModuleLoader_()) {
    throw new Error('goog.module.declareTestMethods must be called from ' +
        'within a goog.module');
  }
  goog.moduleLoaderState_.declareTestMethods = true;
};


/**
 * Indicate that a module's exports that are known test methods should
 * be copied to the global object.  This makes the test methods visible to
 * test runners that inspect the global object.
 *
 * TODO(johnlenz): Make the test framework aware of goog.module so
 * that this isn't necessary. Alternately combine this with goog.setTestOnly
 * to minimize boiler plate.
 */
goog.module.declareLegacyNamespace = function() {
  if (!COMPILED && !goog.isInModuleLoader_()) {
    throw new Error('goog.module.declareLegacyNamespace must be called from ' +
        'within a goog.module');
  }
  if (!COMPILED && !goog.moduleLoaderState_.moduleName) {
    throw Error('goog.module must be called prior to ' +
        'goog.module.declareLegacyNamespace.');
  }
  goog.moduleLoaderState_.declareLegacyNamespace = true;
}


/**
 * Marks that the current file should only be used for testing, and never for
 * live code in production.
 *
 * In the case of unit tests, the message may optionally be an exact namespace
 * for the test (e.g. 'goog.stringTest'). The linter will then ignore the extra
 * provide (if not explicitly defined in the code).
 *
 * @param {string=} opt_message Optional message to add to the error that's
 *     raised when used in production code.
 */
goog.setTestOnly = function(opt_message) {
  if (COMPILED && !goog.DEBUG) {
    opt_message = opt_message || '';
    throw Error('Importing test-only code into non-debug environment' +
                (opt_message ? ': ' + opt_message : '.'));
  }
};


/**
 * Forward declares a symbol. This is an indication to the compiler that the
 * symbol may be used in the source yet is not required and may not be provided
 * in compilation.
 *
 * The most common usage of forward declaration is code that takes a type as a
 * function parameter but does not need to require it. By forward declaring
 * instead of requiring, no hard dependency is made, and (if not required
 * elsewhere) the namespace may never be required and thus, not be pulled
 * into the JavaScript binary. If it is required elsewhere, it will be type
 * checked as normal.
 *
 * MOE:begin_intracomment_strip
 * See the detailed proposal at http://b/11534958
 * Design doc at http://go/forwarddeclare
 * Also see apps framework's forward_declarations builddef:
 * http://go/forwarddeclarations
 * MOE:end_intracomment_strip
 *
 * @param {string} name The namespace to forward declare in the form of
 *     "goog.package.part".
 */
goog.forwardDeclare = function(name) {};


if (!COMPILED) {

  /**
   * Check if the given name has been goog.provided. This will return false for
   * names that are available only as implicit namespaces.
   * @param {string} name name of the object to look for.
   * @return {boolean} Whether the name has been provided.
   * @private
   */
  goog.isProvided_ = function(name) {
    return (name in goog.loadedModules_) ||
        (!goog.implicitNamespaces_[name] &&
            goog.isDefAndNotNull(goog.getObjectByName(name)));
  };

  /**
   * Namespaces implicitly defined by goog.provide. For example,
   * goog.provide('goog.events.Event') implicitly declares that 'goog' and
   * 'goog.events' must be namespaces.
   *
   * @type {Object.<string, (boolean|undefined)>}
   * @private
   */
  goog.implicitNamespaces_ = {'goog.module': true};

  // NOTE: We add goog.module as an implicit namespace as goog.module is defined
  // here and because the existing module package has not been moved yet out of
  // the goog.module namespace. This satisifies both the debug loader and
  // ahead-of-time dependency management.
}


/**
 * Returns an object based on its fully qualified external name.  The object
 * is not found if null or undefined.  If you are using a compilation pass that
 * renames property names beware that using this function will not find renamed
 * properties.
 *
 * @param {string} name The fully qualified name.
 * @param {Object=} opt_obj The object within which to look; default is
 *     |goog.global|.
 * @return {?} The value (object or primitive) or, if not found, null.
 */
goog.getObjectByName = function(name, opt_obj) {
  var parts = name.split('.');
  var cur = opt_obj || goog.global;
  for (var part; part = parts.shift(); ) {
    if (goog.isDefAndNotNull(cur[part])) {
      cur = cur[part];
    } else {
      return null;
    }
  }
  return cur;
};


/**
 * Globalizes a whole namespace, such as goog or goog.lang.
 *
 * @param {Object} obj The namespace to globalize.
 * @param {Object=} opt_global The object to add the properties to.
 * @deprecated Properties may be explicitly exported to the global scope, but
 *     this should no longer be done in bulk.
 */
goog.globalize = function(obj, opt_global) {
  var global = opt_global || goog.global;
  for (var x in obj) {
    global[x] = obj[x];
  }
};


/**
 * Adds a dependency from a file to the files it requires.
 * @param {string} relPath The path to the js file.
 * @param {Array.<string>} provides An array of strings with
 *     the names of the objects this file provides.
 * @param {Array.<string>} requires An array of strings with
 *     the names of the objects this file requires.
 * @param {boolean=} opt_isModule Whether this dependency must be loaded as
 *     a module as declared by goog.module.
 */
goog.addDependency = function(relPath, provides, requires, opt_isModule) {
  if (goog.DEPENDENCIES_ENABLED) {
    var provide, require;
    var path = relPath.replace(/\\/g, '/');
    var deps = goog.dependencies_;
    for (var i = 0; provide = provides[i]; i++) {
      deps.nameToPath[provide] = path;
      deps.pathIsModule[path] = !!opt_isModule;
    }
    for (var j = 0; require = requires[j]; j++) {
      if (!(path in deps.requires)) {
        deps.requires[path] = {};
      }
      deps.requires[path][require] = true;
    }
  }
};


// MOE:begin_strip
/**
 * Whether goog.require should throw an exception if it fails.
 * @type {boolean}
 */
goog.useStrictRequires = false;


// MOE:end_strip


// NOTE(nnaze): The debug DOM loader was included in base.js as an original way
// to do "debug-mode" development.  The dependency system can sometimes be
// confusing, as can the debug DOM loader's asynchronous nature.
//
// With the DOM loader, a call to goog.require() is not blocking -- the script
// will not load until some point after the current script.  If a namespace is
// needed at runtime, it needs to be defined in a previous script, or loaded via
// require() with its registered dependencies.
// User-defined namespaces may need their own deps file.  See http://go/js_deps,
// http://go/genjsdeps, or, externally, DepsWriter.
// https://developers.google.com/closure/library/docs/depswriter
//
// Because of legacy clients, the DOM loader can't be easily removed from
// base.js.  Work is being done to make it disableable or replaceable for
// different environments (DOM-less JavaScript interpreters like Rhino or V8,
// for example). See bootstrap/ for more information.


/**
 * @define {boolean} Whether to enable the debug loader.
 *
 * If enabled, a call to goog.require() will attempt to load the namespace by
 * appending a script tag to the DOM (if the namespace has been registered).
 *
 * If disabled, goog.require() will simply assert that the namespace has been
 * provided (and depend on the fact that some outside tool correctly ordered
 * the script).
 */
goog.define('goog.ENABLE_DEBUG_LOADER', true);


/**
 * @param {string} msg
 * @private
 */
goog.logToConsole_ = function(msg) {
  if (goog.global.console) {
    goog.global.console['error'](msg);
  }
};


/**
 * Implements a system for the dynamic resolution of dependencies that works in
 * parallel with the BUILD system. Note that all calls to goog.require will be
 * stripped by the JSCompiler when the --closure_pass option is used.
 * @see goog.provide
 * @param {string} name Namespace to include (as was given in goog.provide()) in
 *     the form "goog.package.part".
 * @return {?} If called within a goog.module file, the associated namespace or
 *     module otherwise null.
 */
goog.require = function(name) {

  // If the object already exists we do not need do do anything.
  if (!COMPILED) {
    if (goog.isProvided_(name)) {
      if (goog.isInModuleLoader_()) {
        return goog.module.getInternal_(name);
      } else {
        return null;
      }
    }

    if (goog.ENABLE_DEBUG_LOADER) {
      var path = goog.getPathFromDeps_(name);
      if (path) {
        goog.included_[path] = true;
        goog.writeScripts_();
        return null;
      }
    }

    var errorMessage = 'goog.require could not find: ' + name;
    goog.logToConsole_(errorMessage);

    // MOE:begin_strip

    // NOTE(nicksantos): We could always throw an error, but this would break
    // legacy users that depended on this failing silently. Instead, the
    // compiler should warn us when there are invalid goog.require calls.
    // For now, we simply give clients a way to turn strict mode on.
    if (goog.useStrictRequires) {
      throw Error(errorMessage);
    }

    return null;

    // In external Closure, always error.
    // MOE:end_strip_and_replace throw Error(errorMessage);
  }
};


/**
 * Path for included scripts.
 * @type {string}
 */
goog.basePath = '';


/**
 * A hook for overriding the base path.
 * @type {string|undefined}
 */
goog.global.CLOSURE_BASE_PATH;


/**
 * Whether to write out Closure's deps file. By default, the deps are written.
 * @type {boolean|undefined}
 */
goog.global.CLOSURE_NO_DEPS;


/**
 * A function to import a single script. This is meant to be overridden when
 * Closure is being run in non-HTML contexts, such as web workers. It's defined
 * in the global scope so that it can be set before base.js is loaded, which
 * allows deps.js to be imported properly.
 *
 * The function is passed the script source, which is a relative URI. It should
 * return true if the script was imported, false otherwise.
 * @type {(function(string): boolean)|undefined}
 */
goog.global.CLOSURE_IMPORT_SCRIPT;


/**
 * Null function used for default values of callbacks, etc.
 * @return {void} Nothing.
 */
goog.nullFunction = function() {};


/**
 * The identity function. Returns its first argument.
 *
 * @param {*=} opt_returnValue The single value that will be returned.
 * @param {...*} var_args Optional trailing arguments. These are ignored.
 * @return {?} The first argument. We can't know the type -- just pass it along
 *      without type.
 * @deprecated Use goog.functions.identity instead.
 */
goog.identityFunction = function(opt_returnValue, var_args) {
  return opt_returnValue;
};


/**
 * When defining a class Foo with an abstract method bar(), you can do:
 * Foo.prototype.bar = goog.abstractMethod
 *
 * Now if a subclass of Foo fails to override bar(), an error will be thrown
 * when bar() is invoked.
 *
 * Note: This does not take the name of the function to override as an argument
 * because that would make it more difficult to obfuscate our JavaScript code.
 *
 * @type {!Function}
 * @throws {Error} when invoked to indicate the method should be overridden.
 */
goog.abstractMethod = function() {
  throw Error('unimplemented abstract method');
};


/**
 * Adds a {@code getInstance} static method that always returns the same
 * instance object.
 * @param {!Function} ctor The constructor for the class to add the static
 *     method to.
 */
goog.addSingletonGetter = function(ctor) {
  ctor.getInstance = function() {
    if (ctor.instance_) {
      return ctor.instance_;
    }
    if (goog.DEBUG) {
      // NOTE: JSCompiler can't optimize away Array#push.
      goog.instantiatedSingletons_[goog.instantiatedSingletons_.length] = ctor;
    }
    return ctor.instance_ = new ctor;
  };
};


/**
 * All singleton classes that have been instantiated, for testing. Don't read
 * it directly, use the {@code goog.testing.singleton} module. The compiler
 * removes this variable if unused.
 * @type {!Array.<!Function>}
 * @private
 */
goog.instantiatedSingletons_ = [];


/**
 * @define {boolean} Whether to load goog.modules using {@code eval} when using
 * the debug loader.  This provides a better debugging experience as the
 * source is unmodified and can be edited using Chrome Workspaces or
 * similiar.  However in some environments the use of {@code eval} is banned
 * so we provide an alternative.
 */
goog.define('goog.LOAD_MODULE_USING_EVAL', true);


/**
 * @define {boolean} Whether the exports of goog.modules should be sealed when
 * possible.
 */
goog.define('goog.SEAL_MODULE_EXPORTS', goog.DEBUG);


/**
 * The registry of initialized modules:
 * the module identifier to module exports map.
 * @private @const {Object.<string, ?>}
 */
goog.loadedModules_ = {};


/**
 * True if goog.dependencies_ is available.
 * @const {boolean}
 */
goog.DEPENDENCIES_ENABLED = !COMPILED && goog.ENABLE_DEBUG_LOADER;


if (goog.DEPENDENCIES_ENABLED) {
  /**
   * Object used to keep track of urls that have already been added. This record
   * allows the prevention of circular dependencies.
   * @type {Object}
   * @private
   */
  goog.included_ = {};


  /**
   * This object is used to keep track of dependencies and other data that is
   * used for loading scripts.
   * @private
   * @type {Object}
   */
  goog.dependencies_ = {
    pathIsModule: {}, // 1 to 1
    nameToPath: {}, // many to 1
    requires: {}, // 1 to many
    // Used when resolving dependencies to prevent us from visiting file twice.
    visited: {},
    written: {} // Used to keep track of script files we have written.
  };


  /**
   * Tries to detect whether is in the context of an HTML document.
   * @return {boolean} True if it looks like HTML document.
   * @private
   */
  goog.inHtmlDocument_ = function() {
    var doc = goog.global.document;
    return typeof doc != 'undefined' &&
           'write' in doc;  // XULDocument misses write.
  };


  /**
   * Tries to detect the base path of base.js script that bootstraps Closure.
   * @private
   */
  goog.findBasePath_ = function() {
    if (goog.global.CLOSURE_BASE_PATH) {
      goog.basePath = goog.global.CLOSURE_BASE_PATH;
      return;
    } else if (!goog.inHtmlDocument_()) {
      return;
    }
    var doc = goog.global.document;
    var scripts = doc.getElementsByTagName('script');
    // Search backwards since the current script is in almost all cases the one
    // that has base.js.
    for (var i = scripts.length - 1; i >= 0; --i) {
      var src = scripts[i].src;
      var qmark = src.lastIndexOf('?');
      var l = qmark == -1 ? src.length : qmark;
      if (src.substr(l - 7, 7) == 'base.js') {
        goog.basePath = src.substr(0, l - 7);
        return;
      }
    }
  };


  /**
   * Imports a script if, and only if, that script hasn't already been imported.
   * (Must be called at execution time)
   * @param {string} src Script source.
   * @param {string=} opt_sourceText The optionally source text to evaluate
   * @private
   */
  goog.importScript_ = function(src, opt_sourceText) {
    var importScript = goog.global.CLOSURE_IMPORT_SCRIPT ||
        goog.writeScriptTag_;
    if (importScript(src, opt_sourceText)) {
      goog.dependencies_.written[src] = true;
    }
  };


  /** @const @private {boolean} */
  goog.IS_OLD_IE_ = goog.global.document &&
      goog.global.document.all && !goog.global.atob;


  /**
   * Given a URL initiate retrieval and execution of the module.
   * @param {string} src Script source URL.
   * @private
   */
  goog.importModule_ = function(src) {
    // In an attempt to keep browsers from timing out loading scripts using
    // synchronous XHRs, put each load in its own script block.
    var bootstrap = 'goog.retrieveAndExecModule_("' + src + '");';

    if (goog.importScript_('', bootstrap)) {
      goog.dependencies_.written[src] = true;
    }
  };


  /** @private {Array.<string>} */
  goog.queuedModules_ = [];


  /**
   * Retrieve and execute a module.
   * @param {string} src Script source URL.
   * @private
   */
  goog.retrieveAndExecModule_ = function(src) {
    // Canonicalize the path, removing any /./ or /../ since Chrome's debugging
    // console doesn't auto-canonicalize XHR loads as it does <script> srcs.
    var separator;
    while ((separator = src.indexOf('/./')) != -1) {
      src = src.substr(0, separator) + src.substr(separator + '/.'.length);
    }
    while ((separator = src.indexOf('/../')) != -1) {
      var previousComponent = src.lastIndexOf('/', separator - 1);
      src = src.substr(0, previousComponent) +
          src.substr(separator + '/..'.length);
    }

    var importScript = goog.global.CLOSURE_IMPORT_SCRIPT ||
        goog.writeScriptTag_;

    var scriptText = null;

    var xhr = new goog.global['XMLHttpRequest']();

    /** @this {Object} */
    xhr.onload = function() {
      scriptText = this.responseText;
    };
    xhr.open('get', src, false);
    xhr.send();

    scriptText = xhr.responseText;

    if (scriptText != null) {
      var execModuleScript = goog.wrapModule_(src, scriptText);
      var isOldIE = goog.IS_OLD_IE_;
      if (isOldIE) {
        goog.queuedModules_.push(execModuleScript);
      } else {
        importScript(src, execModuleScript);
      }
      goog.dependencies_.written[src] = true;
    } else {
      throw new Error('load of ' + src + 'failed');
    }
  };


  /**
   * Return an appropriate module text. Suitable to insert into
   * a script tag (that is unescaped).
   * @param {string} srcUrl
   * @param {string} scriptText
   * @return {string}
   * @private
   */
  goog.wrapModule_ = function(srcUrl, scriptText) {
    if (!goog.LOAD_MODULE_USING_EVAL || !goog.isDef(goog.global.JSON)) {
      return '' +
          'goog.loadModule(function(exports) {' +
          '"use strict";' +
          scriptText +
          '\n' + // terminate any trailing single line comment.
          ';return exports' +
          '});' +
          '\n//# sourceURL=' + srcUrl + '\n';
    } else {
      return '' +
          'goog.loadModule(' +
          goog.global.JSON.stringify(
              scriptText + '\n//# sourceURL=' + srcUrl + '\n') +
          ');';
    }
  };


  /**
   * Load any deferred goog.module loads.
   * @private
   */
  goog.loadQueuedModules_ = function() {
    var count = goog.queuedModules_.length;
    if (count > 0) {
      var queue = goog.queuedModules_;
      goog.queuedModules_ = [];
      for (var i = 0; i < count; i++) {
        var entry = queue[i];
        goog.globalEval(entry);
      }
    }
  };


  /**
   * @param {function(?):?|string} moduleDef The module definition.
   */
  goog.loadModule = function(moduleDef) {
    // NOTE: we allow function definitions to be either in the from
    // of a string to eval (which keeps the original source intact) or
    // in a eval forbidden environment (CSP) we allow a function definition
    // which in its body must call {@code goog.module}, and return the exports
    // of the module.
    try {
      goog.moduleLoaderState_ = {
          moduleName: undefined, declareTestMethods: false};
      var exports;
      if (goog.isFunction(moduleDef)) {
        exports = moduleDef.call(goog.global, {});
      } else if (goog.isString(moduleDef)) {
        exports = goog.loadModuleFromSource_.call(goog.global, moduleDef);
      } else {
        throw Error('Invalid module definition');
      }

      var moduleName = goog.moduleLoaderState_.moduleName;
      if (!goog.isString(moduleName) || !moduleName) {
        throw Error('Invalid module name \"' + moduleName + '\"');
      }

      // Don't seal legacy namespaces as they may be uses as a parent of
      // another namespace
      if (goog.moduleLoaderState_.declareLegacyNamespace) {
        goog.constructNamespace_(moduleName, exports);
      } else if (goog.SEAL_MODULE_EXPORTS && Object.seal) {
        Object.seal(exports);
      }

      goog.loadedModules_[moduleName] = exports;
      if (goog.moduleLoaderState_.declareTestMethods) {
        for (var entry in exports) {
          if (entry.indexOf('test', 0) === 0 ||
              entry == 'tearDown' ||
              entry == 'setup') {
            goog.global[entry] = exports[entry];
          }
        }
      }
    } finally {
      goog.moduleLoaderState_ = null;
    }
  };


  /**
   * @private @const {function(string):?}
   */
  goog.loadModuleFromSource_ = function() {
    // NOTE: we avoid declaring parameters or local variables here to avoid
    // masking globals or leaking values into the module definition.
    'use strict';
    var exports = {};
    eval(arguments[0]);
    return exports;
  };


  /**
   * The default implementation of the import function. Writes a script tag to
   * import the script.
   *
   * @param {string} src The script url.
   * @param {string=} opt_sourceText The optionally source text to evaluate
   * @return {boolean} True if the script was imported, false otherwise.
   * @private
   */
  goog.writeScriptTag_ = function(src, opt_sourceText) {
    if (goog.inHtmlDocument_()) {
      var doc = goog.global.document;

      // If the user tries to require a new symbol after document load,
      // something has gone terribly wrong. Doing a document.write would
      // wipe out the page.
      if (doc.readyState == 'complete') {
        // Certain test frameworks load base.js multiple times, which tries
        // to write deps.js each time. If that happens, just fail silently.
        // These frameworks wipe the page between each load of base.js, so this
        // is OK.
        var isDeps = /\bdeps.js$/.test(src);
        if (isDeps) {
          return false;
        } else {
          throw Error('Cannot write "' + src + '" after document load');
        }
      }

      var isOldIE = goog.IS_OLD_IE_;

      if (opt_sourceText === undefined) {
        if (!isOldIE) {
          doc.write(
              '<script type="text/javascript" src="' +
                  src + '"></' + 'script>');
        } else {
          var state = " onreadystatechange='goog.onScriptLoad_(this, " +
              ++goog.lastNonModuleScriptIndex_ + ")' ";
          doc.write(
              '<script type="text/javascript" src="' +
                  src + '"' + state + '></' + 'script>');
        }
      } else {
        doc.write(
            '<script type="text/javascript">' +
            opt_sourceText +
            '</' + 'script>');
      }
      return true;
    } else {
      return false;
    }
  };


  /** @private {number} */
  goog.lastNonModuleScriptIndex_ = 0;


  /**
   * A readystatechange handler for legacy IE
   * @param {HTMLScriptElement} script
   * @param {number} scriptIndex
   * @return {boolean}
   * @private
   */
  goog.onScriptLoad_ = function(script, scriptIndex) {
    // for now load the modules when we reach the last script,
    // later allow more inter-mingling.
    if (script.readyState == 'complete' &&
        goog.lastNonModuleScriptIndex_ == scriptIndex) {
      goog.loadQueuedModules_();
    }
    return true;
  };

  /**
   * Resolves dependencies based on the dependencies added using addDependency
   * and calls importScript_ in the correct order.
   * @private
   */
  goog.writeScripts_ = function() {
    // The scripts we need to write this time.
    var scripts = [];
    var seenScript = {};
    var deps = goog.dependencies_;

    function visitNode(path) {
      if (path in deps.written) {
        return;
      }

      // We have already visited this one. We can get here if we have cyclic
      // dependencies.
      if (path in deps.visited) {
        if (!(path in seenScript)) {
          seenScript[path] = true;
          scripts.push(path);
        }
        return;
      }

      deps.visited[path] = true;

      if (path in deps.requires) {
        for (var requireName in deps.requires[path]) {
          // If the required name is defined, we assume that it was already
          // bootstrapped by other means.
          if (!goog.isProvided_(requireName)) {
            if (requireName in deps.nameToPath) {
              visitNode(deps.nameToPath[requireName]);
            } else {
              throw Error('Undefined nameToPath for ' + requireName);
            }
          }
        }
      }

      if (!(path in seenScript)) {
        seenScript[path] = true;
        scripts.push(path);
      }
    }

    for (var path in goog.included_) {
      if (!deps.written[path]) {
        visitNode(path);
      }
    }

    // record that we are going to load all these scripts.
    for (var i = 0; i < scripts.length; i++) {
      var path = scripts[i];
      goog.dependencies_.written[path] = true;
    }

    // If a module is loaded synchronously then we need to
    // clear the current inModuleLoader value, and restore it when we are
    // done loading the current "requires".
    var moduleState = goog.moduleLoaderState_;
    goog.moduleLoaderState_ = null;

    var loadingModule = false;
    for (var i = 0; i < scripts.length; i++) {
      var path = scripts[i];
      if (path) {
        if (!deps.pathIsModule[path]) {
          goog.importScript_(goog.basePath + path);
        } else {
          loadingModule = true;
          goog.importModule_(goog.basePath + path);
        }
      } else {
        goog.moduleLoaderState_ = moduleState;
        throw Error('Undefined script input');
      }
    }

    // restore the current "module loading state"
    goog.moduleLoaderState_ = moduleState;
  };


  /**
   * Looks at the dependency rules and tries to determine the script file that
   * fulfills a particular rule.
   * @param {string} rule In the form goog.namespace.Class or project.script.
   * @return {?string} Url corresponding to the rule, or null.
   * @private
   */
  goog.getPathFromDeps_ = function(rule) {
    if (rule in goog.dependencies_.nameToPath) {
      return goog.dependencies_.nameToPath[rule];
    } else {
      return null;
    }
  };

  goog.findBasePath_();

  // Allow projects to manage the deps files themselves.
  if (!goog.global.CLOSURE_NO_DEPS) {
    goog.importScript_(goog.basePath + 'deps.js');
  }
}



//==============================================================================
// Language Enhancements
//==============================================================================


/**
 * This is a "fixed" version of the typeof operator.  It differs from the typeof
 * operator in such a way that null returns 'null' and arrays return 'array'.
 * @param {*} value The value to get the type of.
 * @return {string} The name of the type.
 */
goog.typeOf = function(value) {
  var s = typeof value;
  if (s == 'object') {
    if (value) {
      // Check these first, so we can avoid calling Object.prototype.toString if
      // possible.
      //
      // IE improperly marshals tyepof across execution contexts, but a
      // cross-context object will still return false for "instanceof Object".
      if (value instanceof Array) {
        return 'array';
      } else if (value instanceof Object) {
        return s;
      }

      // HACK: In order to use an Object prototype method on the arbitrary
      //   value, the compiler requires the value be cast to type Object,
      //   even though the ECMA spec explicitly allows it.
      var className = Object.prototype.toString.call(
          /** @type {Object} */ (value));
      // In Firefox 3.6, attempting to access iframe window objects' length
      // property throws an NS_ERROR_FAILURE, so we need to special-case it
      // here.
      if (className == '[object Window]') {
        return 'object';
      }

      // We cannot always use constructor == Array or instanceof Array because
      // different frames have different Array objects. In IE6, if the iframe
      // where the array was created is destroyed, the array loses its
      // prototype. Then dereferencing val.splice here throws an exception, so
      // we can't use goog.isFunction. Calling typeof directly returns 'unknown'
      // so that will work. In this case, this function will return false and
      // most array functions will still work because the array is still
      // array-like (supports length and []) even though it has lost its
      // prototype.
      // Mark Miller noticed that Object.prototype.toString
      // allows access to the unforgeable [[Class]] property.
      //  15.2.4.2 Object.prototype.toString ( )
      //  When the toString method is called, the following steps are taken:
      //      1. Get the [[Class]] property of this object.
      //      2. Compute a string value by concatenating the three strings
      //         "[object ", Result(1), and "]".
      //      3. Return Result(2).
      // and this behavior survives the destruction of the execution context.
      if ((className == '[object Array]' ||
           // In IE all non value types are wrapped as objects across window
           // boundaries (not iframe though) so we have to do object detection
           // for this edge case.
           typeof value.length == 'number' &&
           typeof value.splice != 'undefined' &&
           typeof value.propertyIsEnumerable != 'undefined' &&
           !value.propertyIsEnumerable('splice')

          )) {
        return 'array';
      }
      // HACK: There is still an array case that fails.
      //     function ArrayImpostor() {}
      //     ArrayImpostor.prototype = [];
      //     var impostor = new ArrayImpostor;
      // this can be fixed by getting rid of the fast path
      // (value instanceof Array) and solely relying on
      // (value && Object.prototype.toString.vall(value) === '[object Array]')
      // but that would require many more function calls and is not warranted
      // unless closure code is receiving objects from untrusted sources.

      // IE in cross-window calls does not correctly marshal the function type
      // (it appears just as an object) so we cannot use just typeof val ==
      // 'function'. However, if the object has a call property, it is a
      // function.
      if ((className == '[object Function]' ||
          typeof value.call != 'undefined' &&
          typeof value.propertyIsEnumerable != 'undefined' &&
          !value.propertyIsEnumerable('call'))) {
        return 'function';
      }

    } else {
      return 'null';
    }

  } else if (s == 'function' && typeof value.call == 'undefined') {
    // In Safari typeof nodeList returns 'function', and on Firefox typeof
    // behaves similarly for HTML{Applet,Embed,Object}, Elements and RegExps. We
    // would like to return object for those and we can detect an invalid
    // function by making sure that the function object has a call method.
    return 'object';
  }
  return s;
};


/**
 * Returns true if the specified value is null.
 * @param {?} val Variable to test.
 * @return {boolean} Whether variable is null.
 */
goog.isNull = function(val) {
  return val === null;
};


/**
 * Returns true if the specified value is defined and not null.
 * @param {?} val Variable to test.
 * @return {boolean} Whether variable is defined and not null.
 */
goog.isDefAndNotNull = function(val) {
  // Note that undefined == null.
  return val != null;
};


/**
 * Returns true if the specified value is an array.
 * @param {?} val Variable to test.
 * @return {boolean} Whether variable is an array.
 */
goog.isArray = function(val) {
  return goog.typeOf(val) == 'array';
};


/**
 * Returns true if the object looks like an array. To qualify as array like
 * the value needs to be either a NodeList or an object with a Number length
 * property.
 * @param {?} val Variable to test.
 * @return {boolean} Whether variable is an array.
 */
goog.isArrayLike = function(val) {
  var type = goog.typeOf(val);
  return type == 'array' || type == 'object' && typeof val.length == 'number';
};


/**
 * Returns true if the object looks like a Date. To qualify as Date-like the
 * value needs to be an object and have a getFullYear() function.
 * @param {?} val Variable to test.
 * @return {boolean} Whether variable is a like a Date.
 */
goog.isDateLike = function(val) {
  return goog.isObject(val) && typeof val.getFullYear == 'function';
};


/**
 * Returns true if the specified value is a string.
 * @param {?} val Variable to test.
 * @return {boolean} Whether variable is a string.
 */
goog.isString = function(val) {
  return typeof val == 'string';
};


/**
 * Returns true if the specified value is a boolean.
 * @param {?} val Variable to test.
 * @return {boolean} Whether variable is boolean.
 */
goog.isBoolean = function(val) {
  return typeof val == 'boolean';
};


/**
 * Returns true if the specified value is a number.
 * @param {?} val Variable to test.
 * @return {boolean} Whether variable is a number.
 */
goog.isNumber = function(val) {
  return typeof val == 'number';
};


/**
 * Returns true if the specified value is a function.
 * @param {?} val Variable to test.
 * @return {boolean} Whether variable is a function.
 */
goog.isFunction = function(val) {
  return goog.typeOf(val) == 'function';
};


/**
 * Returns true if the specified value is an object.  This includes arrays and
 * functions.
 * @param {?} val Variable to test.
 * @return {boolean} Whether variable is an object.
 */
goog.isObject = function(val) {
  var type = typeof val;
  return type == 'object' && val != null || type == 'function';
  // return Object(val) === val also works, but is slower, especially if val is
  // not an object.
};


/**
 * Gets a unique ID for an object. This mutates the object so that further calls
 * with the same object as a parameter returns the same value. The unique ID is
 * guaranteed to be unique across the current session amongst objects that are
 * passed into {@code getUid}. There is no guarantee that the ID is unique or
 * consistent across sessions. It is unsafe to generate unique ID for function
 * prototypes.
 *
 * @param {Object} obj The object to get the unique ID for.
 * @return {number} The unique ID for the object.
 */
goog.getUid = function(obj) {
  // TODO(arv): Make the type stricter, do not accept null.

  // In Opera window.hasOwnProperty exists but always returns false so we avoid
  // using it. As a consequence the unique ID generated for BaseClass.prototype
  // and SubClass.prototype will be the same.
  return obj[goog.UID_PROPERTY_] ||
      (obj[goog.UID_PROPERTY_] = ++goog.uidCounter_);
};


/**
 * Whether the given object is alreay assigned a unique ID.
 *
 * This does not modify the object.
 *
 * @param {Object} obj The object to check.
 * @return {boolean} Whether there an assigned unique id for the object.
 */
goog.hasUid = function(obj) {
  return !!obj[goog.UID_PROPERTY_];
};


/**
 * Removes the unique ID from an object. This is useful if the object was
 * previously mutated using {@code goog.getUid} in which case the mutation is
 * undone.
 * @param {Object} obj The object to remove the unique ID field from.
 */
goog.removeUid = function(obj) {
  // TODO(arv): Make the type stricter, do not accept null.

  // In IE, DOM nodes are not instances of Object and throw an exception if we
  // try to delete.  Instead we try to use removeAttribute.
  if ('removeAttribute' in obj) {
    obj.removeAttribute(goog.UID_PROPERTY_);
  }
  /** @preserveTry */
  try {
    delete obj[goog.UID_PROPERTY_];
  } catch (ex) {
  }
};


/**
 * Name for unique ID property. Initialized in a way to help avoid collisions
 * with other closure JavaScript on the same page.
 * @type {string}
 * @private
 */
goog.UID_PROPERTY_ = 'closure_uid_' + ((Math.random() * 1e9) >>> 0);


/**
 * Counter for UID.
 * @type {number}
 * @private
 */
goog.uidCounter_ = 0;


/**
 * Adds a hash code field to an object. The hash code is unique for the
 * given object.
 * @param {Object} obj The object to get the hash code for.
 * @return {number} The hash code for the object.
 * @deprecated Use goog.getUid instead.
 */
goog.getHashCode = goog.getUid;


/**
 * Removes the hash code field from an object.
 * @param {Object} obj The object to remove the field from.
 * @deprecated Use goog.removeUid instead.
 */
goog.removeHashCode = goog.removeUid;


/**
 * Clones a value. The input may be an Object, Array, or basic type. Objects and
 * arrays will be cloned recursively.
 *
 * WARNINGS:
 * <code>goog.cloneObject</code> does not detect reference loops. Objects that
 * refer to themselves will cause infinite recursion.
 *
 * <code>goog.cloneObject</code> is unaware of unique identifiers, and copies
 * UIDs created by <code>getUid</code> into cloned results.
 *
 * @param {*} obj The value to clone.
 * @return {*} A clone of the input value.
 * @deprecated goog.cloneObject is unsafe. Prefer the goog.object methods.
 */
goog.cloneObject = function(obj) {
  var type = goog.typeOf(obj);
  if (type == 'object' || type == 'array') {
    if (obj.clone) {
      return obj.clone();
    }
    var clone = type == 'array' ? [] : {};
    for (var key in obj) {
      clone[key] = goog.cloneObject(obj[key]);
    }
    return clone;
  }

  return obj;
};


/**
 * A native implementation of goog.bind.
 * @param {Function} fn A function to partially apply.
 * @param {Object|undefined} selfObj Specifies the object which this should
 *     point to when the function is run.
 * @param {...*} var_args Additional arguments that are partially applied to the
 *     function.
 * @return {!Function} A partially-applied form of the function bind() was
 *     invoked as a method of.
 * @private
 * @suppress {deprecated} The compiler thinks that Function.prototype.bind is
 *     deprecated because some people have declared a pure-JS version.
 *     Only the pure-JS version is truly deprecated.
 */
goog.bindNative_ = function(fn, selfObj, var_args) {
  return /** @type {!Function} */ (fn.call.apply(fn.bind, arguments));
};


/**
 * A pure-JS implementation of goog.bind.
 * @param {Function} fn A function to partially apply.
 * @param {Object|undefined} selfObj Specifies the object which this should
 *     point to when the function is run.
 * @param {...*} var_args Additional arguments that are partially applied to the
 *     function.
 * @return {!Function} A partially-applied form of the function bind() was
 *     invoked as a method of.
 * @private
 */
goog.bindJs_ = function(fn, selfObj, var_args) {
  if (!fn) {
    throw new Error();
  }

  if (arguments.length > 2) {
    var boundArgs = Array.prototype.slice.call(arguments, 2);
    return function() {
      // Prepend the bound arguments to the current arguments.
      var newArgs = Array.prototype.slice.call(arguments);
      Array.prototype.unshift.apply(newArgs, boundArgs);
      return fn.apply(selfObj, newArgs);
    };

  } else {
    return function() {
      return fn.apply(selfObj, arguments);
    };
  }
};


/**
 * Partially applies this function to a particular 'this object' and zero or
 * more arguments. The result is a new function with some arguments of the first
 * function pre-filled and the value of this 'pre-specified'.
 *
 * Remaining arguments specified at call-time are appended to the pre-specified
 * ones.
 *
 * Also see: {@link #partial}.
 *
 * Usage:
 * <pre>var barMethBound = bind(myFunction, myObj, 'arg1', 'arg2');
 * barMethBound('arg3', 'arg4');</pre>
 *
 * @param {?function(this:T, ...)} fn A function to partially apply.
 * @param {T} selfObj Specifies the object which this should point to when the
 *     function is run.
 * @param {...*} var_args Additional arguments that are partially applied to the
 *     function.
 * @return {!Function} A partially-applied form of the function bind() was
 *     invoked as a method of.
 * @template T
 * @suppress {deprecated} See above.
 */
goog.bind = function(fn, selfObj, var_args) {
  // TODO(nicksantos): narrow the type signature.
  if (Function.prototype.bind &&
      // NOTE(nicksantos): Somebody pulled base.js into the default Chrome
      // extension environment. This means that for Chrome extensions, they get
      // the implementation of Function.prototype.bind that calls goog.bind
      // instead of the native one. Even worse, we don't want to introduce a
      // circular dependency between goog.bind and Function.prototype.bind, so
      // we have to hack this to make sure it works correctly.
      Function.prototype.bind.toString().indexOf('native code') != -1) {
    goog.bind = goog.bindNative_;
  } else {
    goog.bind = goog.bindJs_;
  }
  return goog.bind.apply(null, arguments);
};


/**
 * Like bind(), except that a 'this object' is not required. Useful when the
 * target function is already bound.
 *
 * Usage:
 * var g = partial(f, arg1, arg2);
 * g(arg3, arg4);
 *
 * @param {Function} fn A function to partially apply.
 * @param {...*} var_args Additional arguments that are partially applied to fn.
 * @return {!Function} A partially-applied form of the function bind() was
 *     invoked as a method of.
 */
goog.partial = function(fn, var_args) {
  var args = Array.prototype.slice.call(arguments, 1);
  return function() {
    // Clone the array (with slice()) and append additional arguments
    // to the existing arguments.
    var newArgs = args.slice();
    newArgs.push.apply(newArgs, arguments);
    return fn.apply(this, newArgs);
  };
};


/**
 * Copies all the members of a source object to a target object. This method
 * does not work on all browsers for all objects that contain keys such as
 * toString or hasOwnProperty. Use goog.object.extend for this purpose.
 * @param {Object} target Target.
 * @param {Object} source Source.
 */
goog.mixin = function(target, source) {
  for (var x in source) {
    target[x] = source[x];
  }

  // For IE7 or lower, the for-in-loop does not contain any properties that are
  // not enumerable on the prototype object (for example, isPrototypeOf from
  // Object.prototype) but also it will not include 'replace' on objects that
  // extend String and change 'replace' (not that it is common for anyone to
  // extend anything except Object).
};


/**
 * @return {number} An integer value representing the number of milliseconds
 *     between midnight, January 1, 1970 and the current time.
 */
goog.now = (goog.TRUSTED_SITE && Date.now) || (function() {
  // Unary plus operator converts its operand to a number which in the case of
  // a date is done by calling getTime().
  return +new Date();
});


/**
 * Evals JavaScript in the global scope.  In IE this uses execScript, other
 * browsers use goog.global.eval. If goog.global.eval does not evaluate in the
 * global scope (for example, in Safari), appends a script tag instead.
 * Throws an exception if neither execScript or eval is defined.
 * @param {string} script JavaScript string.
 */
goog.globalEval = function(script) {
  if (goog.global.execScript) {
    goog.global.execScript(script, 'JavaScript');
  } else if (goog.global.eval) {
    // Test to see if eval works
    if (goog.evalWorksForGlobals_ == null) {
      goog.global.eval('var _et_ = 1;');
      if (typeof goog.global['_et_'] != 'undefined') {
        delete goog.global['_et_'];
        goog.evalWorksForGlobals_ = true;
      } else {
        goog.evalWorksForGlobals_ = false;
      }
    }

    if (goog.evalWorksForGlobals_) {
      goog.global.eval(script);
    } else {
      var doc = goog.global.document;
      var scriptElt = doc.createElement('script');
      scriptElt.type = 'text/javascript';
      scriptElt.defer = false;
      // Note(pupius): can't use .innerHTML since "t('<test>')" will fail and
      // .text doesn't work in Safari 2.  Therefore we append a text node.
      scriptElt.appendChild(doc.createTextNode(script));
      doc.body.appendChild(scriptElt);
      doc.body.removeChild(scriptElt);
    }
  } else {
    throw Error('goog.globalEval not available');
  }
};


/**
 * Indicates whether or not we can call 'eval' directly to eval code in the
 * global scope. Set to a Boolean by the first call to goog.globalEval (which
 * empirically tests whether eval works for globals). @see goog.globalEval
 * @type {?boolean}
 * @private
 */
goog.evalWorksForGlobals_ = null;


/**
 * Optional map of CSS class names to obfuscated names used with
 * goog.getCssName().
 * @type {Object|undefined}
 * @private
 * @see goog.setCssNameMapping
 */
goog.cssNameMapping_;


/**
 * Optional obfuscation style for CSS class names. Should be set to either
 * 'BY_WHOLE' or 'BY_PART' if defined.
 * @type {string|undefined}
 * @private
 * @see goog.setCssNameMapping
 */
goog.cssNameMappingStyle_;


/**
 * Handles strings that are intended to be used as CSS class names.
 *
 * This function works in tandem with @see goog.setCssNameMapping.
 *
 * Without any mapping set, the arguments are simple joined with a hyphen and
 * passed through unaltered.
 *
 * When there is a mapping, there are two possible styles in which these
 * mappings are used. In the BY_PART style, each part (i.e. in between hyphens)
 * of the passed in css name is rewritten according to the map. In the BY_WHOLE
 * style, the full css name is looked up in the map directly. If a rewrite is
 * not specified by the map, the compiler will output a warning.
 *
 * When the mapping is passed to the compiler, it will replace calls to
 * goog.getCssName with the strings from the mapping, e.g.
 *     var x = goog.getCssName('foo');
 *     var y = goog.getCssName(this.baseClass, 'active');
 *  becomes:
 *     var x= 'foo';
 *     var y = this.baseClass + '-active';
 *
 * If one argument is passed it will be processed, if two are passed only the
 * modifier will be processed, as it is assumed the first argument was generated
 * as a result of calling goog.getCssName.
 *
 * @param {string} className The class name.
 * @param {string=} opt_modifier A modifier to be appended to the class name.
 * @return {string} The class name or the concatenation of the class name and
 *     the modifier.
 */
goog.getCssName = function(className, opt_modifier) {
  var getMapping = function(cssName) {
    return goog.cssNameMapping_[cssName] || cssName;
  };

  var renameByParts = function(cssName) {
    // Remap all the parts individually.
    var parts = cssName.split('-');
    var mapped = [];
    for (var i = 0; i < parts.length; i++) {
      mapped.push(getMapping(parts[i]));
    }
    return mapped.join('-');
  };

  var rename;
  if (goog.cssNameMapping_) {
    rename = goog.cssNameMappingStyle_ == 'BY_WHOLE' ?
        getMapping : renameByParts;
  } else {
    rename = function(a) {
      return a;
    };
  }

  if (opt_modifier) {
    return className + '-' + rename(opt_modifier);
  } else {
    return rename(className);
  }
};


/**
 * Sets the map to check when returning a value from goog.getCssName(). Example:
 * <pre>
 * goog.setCssNameMapping({
 *   "goog": "a",
 *   "disabled": "b",
 * });
 *
 * var x = goog.getCssName('goog');
 * // The following evaluates to: "a a-b".
 * goog.getCssName('goog') + ' ' + goog.getCssName(x, 'disabled')
 * </pre>
 * When declared as a map of string literals to string literals, the JSCompiler
 * will replace all calls to goog.getCssName() using the supplied map if the
 * --closure_pass flag is set.
 *
 * @param {!Object} mapping A map of strings to strings where keys are possible
 *     arguments to goog.getCssName() and values are the corresponding values
 *     that should be returned.
 * @param {string=} opt_style The style of css name mapping. There are two valid
 *     options: 'BY_PART', and 'BY_WHOLE'.
 * @see goog.getCssName for a description.
 */
goog.setCssNameMapping = function(mapping, opt_style) {
  goog.cssNameMapping_ = mapping;
  goog.cssNameMappingStyle_ = opt_style;
};


/**
 * To use CSS renaming in compiled mode, one of the input files should have a
 * call to goog.setCssNameMapping() with an object literal that the JSCompiler
 * can extract and use to replace all calls to goog.getCssName(). In uncompiled
 * mode, JavaScript code should be loaded before this base.js file that declares
 * a global variable, CLOSURE_CSS_NAME_MAPPING, which is used below. This is
 * to ensure that the mapping is loaded before any calls to goog.getCssName()
 * are made in uncompiled mode.
 *
 * A hook for overriding the CSS name mapping.
 * @type {Object|undefined}
 */
goog.global.CLOSURE_CSS_NAME_MAPPING;


if (!COMPILED && goog.global.CLOSURE_CSS_NAME_MAPPING) {
  // This does not call goog.setCssNameMapping() because the JSCompiler
  // requires that goog.setCssNameMapping() be called with an object literal.
  goog.cssNameMapping_ = goog.global.CLOSURE_CSS_NAME_MAPPING;
}


/**
 * Gets a localized message.
 *
 * This function is a compiler primitive. If you give the compiler a localized
 * message bundle, it will replace the string at compile-time with a localized
 * version, and expand goog.getMsg call to a concatenated string.
 *
 * Messages must be initialized in the form:
 * <code>
 * var MSG_NAME = goog.getMsg('Hello {$placeholder}', {'placeholder': 'world'});
 * </code>
 *
 * @param {string} str Translatable string, places holders in the form {$foo}.
 * @param {Object=} opt_values Map of place holder name to value.
 * @return {string} message with placeholders filled.
 */
goog.getMsg = function(str, opt_values) {
  if (opt_values) {
    str = str.replace(/\{\$([^}]+)}/g, function(match, key) {
      return key in opt_values ? opt_values[key] : match;
    });
  }
  return str;
};


/**
 * Gets a localized message. If the message does not have a translation, gives a
 * fallback message.
 *
 * This is useful when introducing a new message that has not yet been
 * translated into all languages.
 *
 * This function is a compiler primitive. Must be used in the form:
 * <code>var x = goog.getMsgWithFallback(MSG_A, MSG_B);</code>
 * where MSG_A and MSG_B were initialized with goog.getMsg.
 *
 * @param {string} a The preferred message.
 * @param {string} b The fallback message.
 * @return {string} The best translated message.
 */
goog.getMsgWithFallback = function(a, b) {
  return a;
};


/**
 * Exposes an unobfuscated global namespace path for the given object.
 * Note that fields of the exported object *will* be obfuscated, unless they are
 * exported in turn via this function or goog.exportProperty.
 *
 * Also handy for making public items that are defined in anonymous closures.
 *
 * ex. goog.exportSymbol('public.path.Foo', Foo);
 *
 * ex. goog.exportSymbol('public.path.Foo.staticFunction', Foo.staticFunction);
 *     public.path.Foo.staticFunction();
 *
 * ex. goog.exportSymbol('public.path.Foo.prototype.myMethod',
 *                       Foo.prototype.myMethod);
 *     new public.path.Foo().myMethod();
 *
 * @param {string} publicPath Unobfuscated name to export.
 * @param {*} object Object the name should point to.
 * @param {Object=} opt_objectToExportTo The object to add the path to; default
 *     is goog.global.
 */
goog.exportSymbol = function(publicPath, object, opt_objectToExportTo) {
  goog.exportPath_(publicPath, object, opt_objectToExportTo);
};


/**
 * Exports a property unobfuscated into the object's namespace.
 * ex. goog.exportProperty(Foo, 'staticFunction', Foo.staticFunction);
 * ex. goog.exportProperty(Foo.prototype, 'myMethod', Foo.prototype.myMethod);
 * @param {Object} object Object whose static property is being exported.
 * @param {string} publicName Unobfuscated name to export.
 * @param {*} symbol Object the name should point to.
 */
goog.exportProperty = function(object, publicName, symbol) {
  object[publicName] = symbol;
};


/**
 * Inherit the prototype methods from one constructor into another.
 *
 * Usage:
 * <pre>
 * function ParentClass(a, b) { }
 * ParentClass.prototype.foo = function(a) { };
 *
 * function ChildClass(a, b, c) {
 *   ChildClass.base(this, 'constructor', a, b);
 * }
 * goog.inherits(ChildClass, ParentClass);
 *
 * var child = new ChildClass('a', 'b', 'see');
 * child.foo(); // This works.
 * </pre>
 *
 * @param {Function} childCtor Child class.
 * @param {Function} parentCtor Parent class.
 */
goog.inherits = function(childCtor, parentCtor) {
  /** @constructor */
  function tempCtor() {};
  tempCtor.prototype = parentCtor.prototype;
  childCtor.superClass_ = parentCtor.prototype;
  childCtor.prototype = new tempCtor();
  /** @override */
  childCtor.prototype.constructor = childCtor;

  /**
   * Calls superclass constructor/method.
   *
   * This function is only available if you use goog.inherits to
   * express inheritance relationships between classes.
   *
   * NOTE: This is a replacement for goog.base and for superClass_
   * property defined in childCtor.
   *
   * @param {!Object} me Should always be "this".
   * @param {string} methodName The method name to call. Calling
   *     superclass constructor can be done with the special string
   *     'constructor'.
   * @param {...*} var_args The arguments to pass to superclass
   *     method/constructor.
   * @return {*} The return value of the superclass method/constructor.
   */
  childCtor.base = function(me, methodName, var_args) {
    var args = Array.prototype.slice.call(arguments, 2);
    return parentCtor.prototype[methodName].apply(me, args);
  };
};


/**
 * Call up to the superclass.
 *
 * If this is called from a constructor, then this calls the superclass
 * constructor with arguments 1-N.
 *
 * If this is called from a prototype method, then you must pass the name of the
 * method as the second argument to this function. If you do not, you will get a
 * runtime error. This calls the superclass' method with arguments 2-N.
 *
 * This function only works if you use goog.inherits to express inheritance
 * relationships between your classes.
 *
 * This function is a compiler primitive. At compile-time, the compiler will do
 * macro expansion to remove a lot of the extra overhead that this function
 * introduces. The compiler will also enforce a lot of the assumptions that this
 * function makes, and treat it as a compiler error if you break them.
 *
 * @param {!Object} me Should always be "this".
 * @param {*=} opt_methodName The method name if calling a super method.
 * @param {...*} var_args The rest of the arguments.
 * @return {*} The return value of the superclass method.
 * @suppress {es5Strict} This method can not be used in strict mode, but
 *     all Closure Library consumers must depend on this file.
 */
goog.base = function(me, opt_methodName, var_args) {
  var caller = arguments.callee.caller;

  if (goog.STRICT_MODE_COMPATIBLE || (goog.DEBUG && !caller)) {
    throw Error('arguments.caller not defined.  goog.base() cannot be used ' +
                'with strict mode code. See ' +
                'http://www.ecma-international.org/ecma-262/5.1/#sec-C');
  }

  if (caller.superClass_) {
    // This is a constructor. Call the superclass constructor.
    return caller.superClass_.constructor.apply(
        me, Array.prototype.slice.call(arguments, 1));
  }

  var args = Array.prototype.slice.call(arguments, 2);
  var foundCaller = false;
  for (var ctor = me.constructor;
       ctor; ctor = ctor.superClass_ && ctor.superClass_.constructor) {
    if (ctor.prototype[opt_methodName] === caller) {
      foundCaller = true;
    } else if (foundCaller) {
      return ctor.prototype[opt_methodName].apply(me, args);
    }
  }

  // If we did not find the caller in the prototype chain, then one of two
  // things happened:
  // 1) The caller is an instance method.
  // 2) This method was not called by the right caller.
  if (me[opt_methodName] === caller) {
    return me.constructor.prototype[opt_methodName].apply(me, args);
  } else {
    throw Error(
        'goog.base called from a method of one name ' +
        'to a method of a different name');
  }
};


/**
 * Allow for aliasing within scope functions.  This function exists for
 * uncompiled code - in compiled code the calls will be inlined and the aliases
 * applied.  In uncompiled code the function is simply run since the aliases as
 * written are valid JavaScript.
 *
 * MOE:begin_intracomment_strip
 * See the goog.scope document at http://go/goog.scope
 * MOE:end_intracomment_strip
 *
 * @param {function()} fn Function to call.  This function can contain aliases
 *     to namespaces (e.g. "var dom = goog.dom") or classes
 *     (e.g. "var Timer = goog.Timer").
 */
goog.scope = function(fn) {
  fn.call(goog.global);
};


/*
 * To support uncompiled, strict mode bundles that use eval to divide source
 * like so:
 *    eval('someSource;//# sourceUrl sourcefile.js');
 * We need to export the globally defined symbols "goog" and "COMPILED".
 * Exporting "goog" breaks the compiler optimizations, so we required that
 * be defined externally.
 * NOTE: We don't use goog.exportSymbol here because we don't want to trigger
 * extern generation when that compiler option is enabled.
 */
if (!COMPILED) {
  goog.global['COMPILED'] = COMPILED;
}


// MOE:begin_strip

// The section between this token and the end token below will be stripped
// automatically by the open source release scripts.  Please leave in place.

//==============================================================================
// Extending Function
//==============================================================================


/**
 * @define {boolean} Whether to extend Function.prototype.
 *     Use --define='goog.MODIFY_FUNCTION_PROTOTYPES=false' to change.
 */
goog.define('goog.MODIFY_FUNCTION_PROTOTYPES', true);

if (goog.MODIFY_FUNCTION_PROTOTYPES) {
  /**
   * An alias to the {@link goog.bind()} global function. Deprecated: use
   * goog.bind instead.
   *
   * Usage:
   * var g = f.bind(obj, arg1, arg2);
   * g(arg3, arg4);
   *
   * @param {Object|undefined} selfObj Specifies the object to which this should
   *     point when the function is run. If the value is null or undefined, it
   *     will default to the global object.
   * @param {...*} var_args Additional arguments that are partially applied to
   *     fn.
   * @return {!Function} A partially-applied form of the Function on which
   *     bind() was invoked as a method.
   * @suppress {duplicate}
   */
  Function.prototype.bind =
      Function.prototype.bind || function(selfObj, var_args) {
    if (arguments.length > 1) {
      var args = Array.prototype.slice.call(arguments, 1);
      args.unshift(this, selfObj);
      return goog.bind.apply(null, args);
    } else {
      return goog.bind(this, selfObj);
    }
  };


  /**
   * An alias to the {@link goog.partial()} static function.
   *
   * Usage:
   * var g = f.partial(arg1, arg2);
   * g(arg3, arg4);
   *
   * @param {...*} var_args Additional arguments that are partially applied to
   *     fn.
   * @return {!Function} A partially-applied form of the function partial() was
   *     invoked as a method of.
   * @deprecated Use the static function goog.partial instead.
   */
  Function.prototype.partial = function(var_args) {
    var args = Array.prototype.slice.call(arguments);
    args.unshift(this, null);
    return goog.bind.apply(null, args);
  };


  /**
   * Inherit the prototype methods from one constructor into another.
   * @param {Function} parentCtor Parent class.
   * @see goog.inherits
   * @deprecated Use the static function goog.inherits instead.
   */
  Function.prototype.inherits = function(parentCtor) {
    goog.inherits(this, parentCtor);
  };


  /**
   * Mixes in an object's properties and methods into the callee's prototype.
   * Basically mixin based inheritance, thus providing an alternative method for
   * adding properties and methods to a class' prototype.
   *
   * <pre>
   * function X() {}
   * X.mixin({
   *   one: 1,
   *   two: 2,
   *   three: 3,
   *   doit: function() { return this.one + this.two + this.three; }
   * });
   *
   * function Y() {}
   * Y.mixin(X.prototype);
   * Y.prototype.four = 15;
   * Y.prototype.doit2 = function() { return this.doit() + this.four; }
   *
   * // or
   *
   * function Y() {}
   * Y.inherits(X);
   * Y.mixin({
   *   one: 10,
   *   four: 15,
   *   doit2: function() { return this.doit() + this.four; }
   * });
   * </pre>
   *
   * @param {Object} source from which to copy properties.
   * @see goog.mixin
   * @deprecated Use the static function goog.object.extend instead.
   */
  Function.prototype.mixin = function(source) {
    goog.mixin(this.prototype, source);
  };
}

// MOE:end_strip

//==============================================================================
// goog.defineClass implementation
//==============================================================================

/**
 * Creates a restricted form of a Closure "class":
 *   - from the compiler's perspective, the instance returned from the
 *     constructor is sealed (no new properties may be added).  This enables
 *     better checks.
 *   - the compiler will rewrite this definition to a form that is optimal
 *     for type checking and optimization (initially this will be a more
 *     traditional form).
 *
 * @param {Function} superClass The superclass, Object or null.
 * @param {goog.defineClass.ClassDescriptor} def
 *     An object literal describing the
 *     the class.  It may have the following properties:
 *     "constructor": the constructor function
 *     "statics": an object literal containing methods to add to the constructor
 *        as "static" methods or a function that will receive the constructor
 *        function as its only parameter to which static properties can
 *        be added.
 *     all other properties are added to the prototype.
 * @return {!Function} The class constructor.
 */
goog.defineClass = function(superClass, def) {
  // TODO(johnlenz): consider making the superClass an optional parameter.
  var constructor = def.constructor;
  var statics = def.statics;
  // Wrap the constructor prior to setting up the prototype and static methods.
  if (!constructor || constructor == Object.prototype.constructor) {
    constructor = function() {
      throw Error('cannot instantiate an interface (no constructor defined).');
    };
  }

  var cls = goog.defineClass.createSealingConstructor_(constructor, superClass);
  if (superClass) {
    goog.inherits(cls, superClass);
  }

  // Remove all the properties that should not be copied to the prototype.
  delete def.constructor;
  delete def.statics;

  goog.defineClass.applyProperties_(cls.prototype, def);
  if (statics != null) {
    if (statics instanceof Function) {
      statics(cls);
    } else {
      goog.defineClass.applyProperties_(cls, statics);
    }
  }

  return cls;
};


/**
 * @typedef {
 *     !Object|
 *     {constructor:!Function}|
 *     {constructor:!Function, statics:(Object|function(Function):void)}}
 */
goog.defineClass.ClassDescriptor;


/**
 * @define {boolean} Whether the instances returned by
 * goog.defineClass should be sealed when possible.
 */
goog.define('goog.defineClass.SEAL_CLASS_INSTANCES', goog.DEBUG);


/**
 * If goog.defineClass.SEAL_CLASS_INSTANCES is enabled and Object.seal is
 * defined, this function will wrap the constructor in a function that seals the
 * results of the provided constructor function.
 *
 * @param {!Function} ctr The constructor whose results maybe be sealed.
 * @param {Function} superClass The superclass constructor.
 * @return {!Function} The replacement constructor.
 * @private
 */
goog.defineClass.createSealingConstructor_ = function(ctr, superClass) {
  if (goog.defineClass.SEAL_CLASS_INSTANCES &&
      Object.seal instanceof Function) {
    // Don't seal subclasses of unsealable-tagged legacy classes.
    if (superClass && superClass.prototype &&
        superClass.prototype[goog.UNSEALABLE_CONSTRUCTOR_PROPERTY_]) {
      return ctr;
    }
    /** @this {*} */
    var wrappedCtr = function() {
      // Don't seal an instance of a subclass when it calls the constructor of
      // its super class as there is most likely still setup to do.
      var instance = ctr.apply(this, arguments) || this;
      instance[goog.UID_PROPERTY_] = instance[goog.UID_PROPERTY_];
      if (this.constructor === wrappedCtr) {
        Object.seal(instance);
      }
      return instance;
    };
    return wrappedCtr;
  }
  return ctr;
};


// TODO(johnlenz): share these values with the goog.object
/**
 * The names of the fields that are defined on Object.prototype.
 * @type {!Array.<string>}
 * @private
 * @const
 */
goog.defineClass.OBJECT_PROTOTYPE_FIELDS_ = [
  'constructor',
  'hasOwnProperty',
  'isPrototypeOf',
  'propertyIsEnumerable',
  'toLocaleString',
  'toString',
  'valueOf'
];


// TODO(johnlenz): share this function with the goog.object
/**
 * @param {!Object} target The object to add properties to.
 * @param {!Object} source The object to copy properites from.
 * @private
 */
goog.defineClass.applyProperties_ = function(target, source) {
  // TODO(johnlenz): update this to support ES5 getters/setters

  var key;
  for (key in source) {
    if (Object.prototype.hasOwnProperty.call(source, key)) {
      target[key] = source[key];
    }
  }

  // For IE the for-in-loop does not contain any properties that are not
  // enumerable on the prototype object (for example isPrototypeOf from
  // Object.prototype) and it will also not include 'replace' on objects that
  // extend String and change 'replace' (not that it is common for anyone to
  // extend anything except Object).
  for (var i = 0; i < goog.defineClass.OBJECT_PROTOTYPE_FIELDS_.length; i++) {
    key = goog.defineClass.OBJECT_PROTOTYPE_FIELDS_[i];
    if (Object.prototype.hasOwnProperty.call(source, key)) {
      target[key] = source[key];
    }
  }
};


/**
 * Sealing classes breaks the older idiom of assigning properties on the
 * prototype rather than in the constructor.  As such, goog.defineClass
 * must not seal subclasses of these old-style classes until they are fixed.
 * Until then, this marks a class as "broken", instructing defineClass
 * not to seal subclasses.
 * @param {!Function} ctr The legacy constructor to tag as unsealable.
 */
goog.tagUnsealableClass = function(ctr) {
  if (!COMPILED && goog.defineClass.SEAL_CLASS_INSTANCES) {
    ctr.prototype[goog.UNSEALABLE_CONSTRUCTOR_PROPERTY_] = true;
  }
};


/**
 * Name for unsealable tag property.
 * @const @private {string}
 */
goog.UNSEALABLE_CONSTRUCTOR_PROPERTY_ = 'goog_defineClass_legacy_unsealable';

//javascript/closure/deps.js
// Copyright 2014 The Closure Library Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS-IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// This file has been auto-generated by GenJsDeps, please do not edit.

goog.addDependency('../../third_party/javascript/closure/caja/string/html/htmlparser.js', ['goog.string.html.HtmlParser', 'goog.string.html.HtmlParser.EFlags', 'goog.string.html.HtmlParser.Elements', 'goog.string.html.HtmlParser.Entities', 'goog.string.html.HtmlSaxHandler'], [], false);
goog.addDependency('../../third_party/javascript/closure/caja/string/html/htmlsanitizer.js', ['goog.string.html.HtmlSanitizer', 'goog.string.html.HtmlSanitizer.AttributeType', 'goog.string.html.HtmlSanitizer.Attributes', 'goog.string.html.htmlSanitize'], ['goog.string.StringBuffer', 'goog.string.html.HtmlParser', 'goog.string.html.HtmlParser.EFlags', 'goog.string.html.HtmlParser.Elements', 'goog.string.html.HtmlSaxHandler'], false);
goog.addDependency('../../third_party/javascript/closure/dojo/dom/query.js', ['goog.dom.query'], ['goog.array', 'goog.dom', 'goog.functions', 'goog.string', 'goog.userAgent'], false);
goog.addDependency('../../third_party/javascript/closure/jpeg_encoder/jpeg_encoder_basic.js', ['goog.crypt.JpegEncoder'], ['goog.crypt.base64'], false);
goog.addDependency('../../third_party/javascript/closure/loremipsum/text/loremipsum.js', ['goog.text.LoremIpsum'], ['goog.array', 'goog.math', 'goog.string', 'goog.structs.Map', 'goog.structs.Set'], false);
goog.addDependency('../../third_party/javascript/closure/mochikit/async/deferred.js', ['goog.async.Deferred', 'goog.async.Deferred.AlreadyCalledError', 'goog.async.Deferred.CanceledError'], ['goog.Promise', 'goog.Thenable', 'goog.array', 'goog.asserts', 'goog.debug.Error'], false);
goog.addDependency('../../third_party/javascript/closure/mochikit/async/deferredlist.js', ['goog.async.DeferredList'], ['goog.async.Deferred'], false);
goog.addDependency('../../third_party/javascript/closure/osapi/osapi.js', ['goog.osapi'], [], false);
goog.addDependency('../../third_party/javascript/closure/svgpan/svgpan.js', ['svgpan.SvgPan'], ['goog.Disposable', 'goog.events', 'goog.events.EventType', 'goog.events.MouseWheelHandler'], false);
goog.addDependency('a11y/aria/announcer.js', ['goog.a11y.aria.Announcer'], ['goog.Disposable', 'goog.a11y.aria', 'goog.a11y.aria.LivePriority', 'goog.a11y.aria.State', 'goog.dom', 'goog.object'], false);
goog.addDependency('a11y/aria/announcer_test.js', ['goog.a11y.aria.AnnouncerTest'], ['goog.a11y.aria', 'goog.a11y.aria.Announcer', 'goog.a11y.aria.LivePriority', 'goog.a11y.aria.State', 'goog.array', 'goog.dom', 'goog.dom.iframe', 'goog.testing.jsunit'], false);
goog.addDependency('a11y/aria/aria.js', ['goog.a11y.aria'], ['goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.a11y.aria.datatables', 'goog.array', 'goog.asserts', 'goog.dom', 'goog.dom.TagName', 'goog.object', 'goog.string'], false);
goog.addDependency('a11y/aria/aria_test.js', ['goog.a11y.ariaTest'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.dom', 'goog.dom.TagName', 'goog.testing.jsunit'], false);
goog.addDependency('a11y/aria/attributes.js', ['goog.a11y.aria.AutoCompleteValues', 'goog.a11y.aria.CheckedValues', 'goog.a11y.aria.DropEffectValues', 'goog.a11y.aria.ExpandedValues', 'goog.a11y.aria.GrabbedValues', 'goog.a11y.aria.InvalidValues', 'goog.a11y.aria.LivePriority', 'goog.a11y.aria.OrientationValues', 'goog.a11y.aria.PressedValues', 'goog.a11y.aria.RelevantValues', 'goog.a11y.aria.SelectedValues', 'goog.a11y.aria.SortValues', 'goog.a11y.aria.State'], [], false);
goog.addDependency('a11y/aria/datatables.js', ['goog.a11y.aria.datatables'], ['goog.a11y.aria.State', 'goog.object'], false);
goog.addDependency('a11y/aria/roles.js', ['goog.a11y.aria.Role'], [], false);
goog.addDependency('array/array.js', ['goog.array', 'goog.array.ArrayLike'], ['goog.asserts'], false);
goog.addDependency('array/array_test.js', ['goog.arrayTest'], ['goog.array', 'goog.dom', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('asserts/asserts.js', ['goog.asserts', 'goog.asserts.AssertionError'], ['goog.debug.Error', 'goog.dom.NodeType', 'goog.string'], false);
goog.addDependency('asserts/asserts_test.js', ['goog.assertsTest'], ['goog.asserts', 'goog.asserts.AssertionError', 'goog.dom', 'goog.string', 'goog.testing.jsunit'], false);
goog.addDependency('async/animationdelay.js', ['goog.async.AnimationDelay'], ['goog.Disposable', 'goog.events', 'goog.functions'], false);
goog.addDependency('async/animationdelay_test.js', ['goog.async.AnimationDelayTest'], ['goog.async.AnimationDelay', 'goog.testing.AsyncTestCase', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('async/conditionaldelay.js', ['goog.async.ConditionalDelay'], ['goog.Disposable', 'goog.async.Delay'], false);
goog.addDependency('async/conditionaldelay_test.js', ['goog.async.ConditionalDelayTest'], ['goog.async.ConditionalDelay', 'goog.testing.MockClock', 'goog.testing.jsunit'], false);
goog.addDependency('async/delay.js', ['goog.Delay', 'goog.async.Delay'], ['goog.Disposable', 'goog.Timer'], false);
goog.addDependency('async/delay_test.js', ['goog.async.DelayTest'], ['goog.async.Delay', 'goog.testing.MockClock', 'goog.testing.jsunit'], false);
goog.addDependency('async/nexttick.js', ['goog.async.nextTick', 'goog.async.throwException'], ['goog.debug.entryPointRegistry', 'goog.functions', 'goog.labs.userAgent.browser'], false);
goog.addDependency('async/nexttick_test.js', ['goog.async.nextTickTest'], ['goog.async.nextTick', 'goog.debug.ErrorHandler', 'goog.debug.entryPointRegistry', 'goog.testing.AsyncTestCase', 'goog.testing.MockClock', 'goog.testing.jsunit'], false);
goog.addDependency('async/run.js', ['goog.async.run'], ['goog.async.nextTick', 'goog.async.throwException', 'goog.testing.watchers'], false);
goog.addDependency('async/run_test.js', ['goog.async.runTest'], ['goog.async.run', 'goog.testing.MockClock', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('async/throttle.js', ['goog.Throttle', 'goog.async.Throttle'], ['goog.Disposable', 'goog.Timer'], false);
goog.addDependency('async/throttle_test.js', ['goog.async.ThrottleTest'], ['goog.async.Throttle', 'goog.testing.MockClock', 'goog.testing.jsunit'], false);
goog.addDependency('base.js', ['goog'], [], false);
goog.addDependency('base_module_test.js', ['goog.baseModuleTest'], ['goog.Timer', 'goog.test_module', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit'], true);
goog.addDependency('base_test.js', ['an.existing.path', 'dup.base', 'far.out', 'goog.baseTest', 'goog.explicit', 'goog.implicit.explicit', 'goog.test', 'goog.test.name', 'goog.test.name.space', 'goog.xy', 'goog.xy.z', 'ns', 'testDep.bar'], ['goog.Timer', 'goog.functions', 'goog.test_module', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.recordFunction', 'goog.userAgent'], false);
goog.addDependency('color/alpha.js', ['goog.color.alpha'], ['goog.color'], false);
goog.addDependency('color/alpha_test.js', ['goog.color.alphaTest'], ['goog.array', 'goog.color', 'goog.color.alpha', 'goog.testing.jsunit'], false);
goog.addDependency('color/color.js', ['goog.color', 'goog.color.Hsl', 'goog.color.Hsv', 'goog.color.Rgb'], ['goog.color.names', 'goog.math'], false);
goog.addDependency('color/color_test.js', ['goog.colorTest'], ['goog.array', 'goog.color', 'goog.color.names', 'goog.testing.jsunit'], false);
goog.addDependency('color/names.js', ['goog.color.names'], [], false);
goog.addDependency('crypt/aes.js', ['goog.crypt.Aes'], ['goog.asserts', 'goog.crypt.BlockCipher'], false);
goog.addDependency('crypt/aes_test.js', ['goog.crypt.AesTest'], ['goog.crypt', 'goog.crypt.Aes', 'goog.testing.jsunit'], false);
goog.addDependency('crypt/arc4.js', ['goog.crypt.Arc4'], ['goog.asserts'], false);
goog.addDependency('crypt/arc4_test.js', ['goog.crypt.Arc4Test'], ['goog.array', 'goog.crypt.Arc4', 'goog.testing.jsunit'], false);
goog.addDependency('crypt/base64.js', ['goog.crypt.base64'], ['goog.crypt', 'goog.userAgent'], false);
goog.addDependency('crypt/base64_test.js', ['goog.crypt.base64Test'], ['goog.crypt', 'goog.crypt.base64', 'goog.testing.jsunit'], false);
goog.addDependency('crypt/basen.js', ['goog.crypt.baseN'], [], false);
goog.addDependency('crypt/basen_test.js', ['goog.crypt.baseNTest'], ['goog.crypt.baseN', 'goog.testing.jsunit'], false);
goog.addDependency('crypt/blobhasher.js', ['goog.crypt.BlobHasher', 'goog.crypt.BlobHasher.EventType'], ['goog.asserts', 'goog.crypt', 'goog.crypt.Hash', 'goog.events.EventTarget', 'goog.fs', 'goog.log'], false);
goog.addDependency('crypt/blobhasher_test.js', ['goog.crypt.BlobHasherTest'], ['goog.crypt', 'goog.crypt.BlobHasher', 'goog.crypt.Md5', 'goog.events', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit'], false);
goog.addDependency('crypt/blockcipher.js', ['goog.crypt.BlockCipher'], [], false);
goog.addDependency('crypt/bytestring_perf.js', ['goog.crypt.byteArrayToStringPerf'], ['goog.array', 'goog.dom', 'goog.testing.PerformanceTable'], false);
goog.addDependency('crypt/cbc.js', ['goog.crypt.Cbc'], ['goog.array', 'goog.asserts', 'goog.crypt'], false);
goog.addDependency('crypt/cbc_test.js', ['goog.crypt.CbcTest'], ['goog.crypt', 'goog.crypt.Aes', 'goog.crypt.Cbc', 'goog.testing.jsunit'], false);
goog.addDependency('crypt/crypt.js', ['goog.crypt'], ['goog.array', 'goog.asserts'], false);
goog.addDependency('crypt/crypt_test.js', ['goog.cryptTest'], ['goog.crypt', 'goog.string', 'goog.testing.jsunit'], false);
goog.addDependency('crypt/hash.js', ['goog.crypt.Hash'], [], false);
goog.addDependency('crypt/hash32.js', ['goog.crypt.hash32'], ['goog.crypt'], false);
goog.addDependency('crypt/hash32_test.js', ['goog.crypt.hash32Test'], ['goog.crypt.hash32', 'goog.testing.TestCase', 'goog.testing.jsunit'], false);
goog.addDependency('crypt/hashtester.js', ['goog.crypt.hashTester'], ['goog.array', 'goog.crypt', 'goog.testing.PerformanceTable', 'goog.testing.PseudoRandom', 'goog.testing.asserts'], false);
goog.addDependency('crypt/hmac.js', ['goog.crypt.Hmac'], ['goog.crypt.Hash'], false);
goog.addDependency('crypt/hmac_test.js', ['goog.crypt.HmacTest'], ['goog.crypt.Hmac', 'goog.crypt.Sha1', 'goog.crypt.hashTester', 'goog.testing.jsunit'], false);
goog.addDependency('crypt/md5.js', ['goog.crypt.Md5'], ['goog.crypt.Hash'], false);
goog.addDependency('crypt/md5_test.js', ['goog.crypt.Md5Test'], ['goog.crypt', 'goog.crypt.Md5', 'goog.crypt.hashTester', 'goog.testing.jsunit'], false);
goog.addDependency('crypt/pbkdf2.js', ['goog.crypt.pbkdf2'], ['goog.asserts', 'goog.crypt', 'goog.crypt.Hmac', 'goog.crypt.Sha1'], false);
goog.addDependency('crypt/pbkdf2_test.js', ['goog.crypt.pbkdf2Test'], ['goog.crypt', 'goog.crypt.pbkdf2', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('crypt/sha1.js', ['goog.crypt.Sha1'], ['goog.crypt.Hash'], false);
goog.addDependency('crypt/sha1_test.js', ['goog.crypt.Sha1Test'], ['goog.crypt', 'goog.crypt.Sha1', 'goog.crypt.hashTester', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('crypt/sha2.js', ['goog.crypt.Sha2'], ['goog.array', 'goog.asserts', 'goog.crypt.Hash'], false);
goog.addDependency('crypt/sha224.js', ['goog.crypt.Sha224'], ['goog.crypt.Sha2'], false);
goog.addDependency('crypt/sha224_test.js', ['goog.crypt.Sha224Test'], ['goog.crypt', 'goog.crypt.Sha224', 'goog.crypt.hashTester', 'goog.testing.jsunit'], false);
goog.addDependency('crypt/sha256.js', ['goog.crypt.Sha256'], ['goog.crypt.Sha2'], false);
goog.addDependency('crypt/sha256_test.js', ['goog.crypt.Sha256Test'], ['goog.crypt', 'goog.crypt.Sha256', 'goog.crypt.hashTester', 'goog.testing.jsunit'], false);
goog.addDependency('crypt/sha2_64bit.js', ['goog.crypt.Sha2_64bit'], ['goog.array', 'goog.asserts', 'goog.crypt.Hash', 'goog.math.Long'], false);
goog.addDependency('crypt/sha2_64bit_test.js', ['goog.crypt.Sha2_64bit_test'], ['goog.array', 'goog.crypt', 'goog.crypt.Sha384', 'goog.crypt.Sha512', 'goog.crypt.Sha512_256', 'goog.crypt.hashTester', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('crypt/sha384.js', ['goog.crypt.Sha384'], ['goog.crypt.Sha2_64bit'], false);
goog.addDependency('crypt/sha512.js', ['goog.crypt.Sha512'], ['goog.crypt.Sha2_64bit'], false);
goog.addDependency('crypt/sha512_256.js', ['goog.crypt.Sha512_256'], ['goog.crypt.Sha2_64bit'], false);
goog.addDependency('cssom/cssom.js', ['goog.cssom', 'goog.cssom.CssRuleType'], ['goog.array', 'goog.dom'], false);
goog.addDependency('cssom/cssom_test.js', ['goog.cssomTest'], ['goog.array', 'goog.cssom', 'goog.cssom.CssRuleType', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('cssom/iframe/style.js', ['goog.cssom.iframe.style'], ['goog.asserts', 'goog.cssom', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.TagName', 'goog.dom.classlist', 'goog.string', 'goog.style', 'goog.userAgent'], false);
goog.addDependency('cssom/iframe/style_test.js', ['goog.cssom.iframe.styleTest'], ['goog.cssom', 'goog.cssom.iframe.style', 'goog.dom', 'goog.dom.DomHelper', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('datasource/datamanager.js', ['goog.ds.DataManager'], ['goog.ds.BasicNodeList', 'goog.ds.DataNode', 'goog.ds.Expr', 'goog.object', 'goog.string', 'goog.structs', 'goog.structs.Map'], false);
goog.addDependency('datasource/datasource.js', ['goog.ds.BaseDataNode', 'goog.ds.BasicNodeList', 'goog.ds.DataNode', 'goog.ds.DataNodeList', 'goog.ds.EmptyNodeList', 'goog.ds.LoadState', 'goog.ds.SortedNodeList', 'goog.ds.Util', 'goog.ds.logger'], ['goog.array', 'goog.log'], false);
goog.addDependency('datasource/datasource_test.js', ['goog.ds.JsDataSourceTest'], ['goog.dom.xml', 'goog.ds.DataManager', 'goog.ds.JsDataSource', 'goog.ds.SortedNodeList', 'goog.ds.XmlDataSource', 'goog.testing.jsunit'], false);
goog.addDependency('datasource/expr.js', ['goog.ds.Expr'], ['goog.ds.BasicNodeList', 'goog.ds.EmptyNodeList', 'goog.string'], false);
goog.addDependency('datasource/expr_test.js', ['goog.ds.ExprTest'], ['goog.ds.DataManager', 'goog.ds.Expr', 'goog.ds.JsDataSource', 'goog.testing.jsunit'], false);
goog.addDependency('datasource/fastdatanode.js', ['goog.ds.AbstractFastDataNode', 'goog.ds.FastDataNode', 'goog.ds.FastListNode', 'goog.ds.PrimitiveFastDataNode'], ['goog.ds.DataManager', 'goog.ds.EmptyNodeList', 'goog.string'], false);
goog.addDependency('datasource/fastdatanode_test.js', ['goog.ds.FastDataNodeTest'], ['goog.array', 'goog.ds.DataManager', 'goog.ds.Expr', 'goog.ds.FastDataNode', 'goog.testing.jsunit'], false);
goog.addDependency('datasource/jsdatasource.js', ['goog.ds.JsDataSource', 'goog.ds.JsPropertyDataSource'], ['goog.ds.BaseDataNode', 'goog.ds.BasicNodeList', 'goog.ds.DataManager', 'goog.ds.EmptyNodeList', 'goog.ds.LoadState'], false);
goog.addDependency('datasource/jsondatasource.js', ['goog.ds.JsonDataSource'], ['goog.Uri', 'goog.dom', 'goog.ds.DataManager', 'goog.ds.JsDataSource', 'goog.ds.LoadState', 'goog.ds.logger'], false);
goog.addDependency('datasource/jsxmlhttpdatasource.js', ['goog.ds.JsXmlHttpDataSource'], ['goog.Uri', 'goog.ds.DataManager', 'goog.ds.FastDataNode', 'goog.ds.LoadState', 'goog.ds.logger', 'goog.events', 'goog.log', 'goog.net.EventType', 'goog.net.XhrIo'], false);
goog.addDependency('datasource/jsxmlhttpdatasource_test.js', ['goog.ds.JsXmlHttpDataSourceTest'], ['goog.ds.JsXmlHttpDataSource', 'goog.testing.TestQueue', 'goog.testing.jsunit', 'goog.testing.net.XhrIo'], false);
goog.addDependency('datasource/xmldatasource.js', ['goog.ds.XmlDataSource', 'goog.ds.XmlHttpDataSource'], ['goog.Uri', 'goog.dom.NodeType', 'goog.dom.xml', 'goog.ds.BasicNodeList', 'goog.ds.DataManager', 'goog.ds.LoadState', 'goog.ds.logger', 'goog.net.XhrIo', 'goog.string'], false);
goog.addDependency('date/date.js', ['goog.date', 'goog.date.Date', 'goog.date.DateTime', 'goog.date.Interval', 'goog.date.month', 'goog.date.weekDay'], ['goog.asserts', 'goog.date.DateLike', 'goog.i18n.DateTimeSymbols', 'goog.string'], false);
goog.addDependency('date/date_test.js', ['goog.dateTest'], ['goog.array', 'goog.date', 'goog.date.Date', 'goog.date.DateTime', 'goog.date.Interval', 'goog.date.month', 'goog.date.weekDay', 'goog.i18n.DateTimeSymbols', 'goog.testing.ExpectedFailures', 'goog.testing.jsunit', 'goog.userAgent', 'goog.userAgent.platform', 'goog.userAgent.product', 'goog.userAgent.product.isVersion'], false);
goog.addDependency('date/datelike.js', ['goog.date.DateLike'], [], false);
goog.addDependency('date/daterange.js', ['goog.date.DateRange', 'goog.date.DateRange.Iterator', 'goog.date.DateRange.StandardDateRangeKeys'], ['goog.date.Date', 'goog.date.Interval', 'goog.iter.Iterator', 'goog.iter.StopIteration'], false);
goog.addDependency('date/daterange_test.js', ['goog.date.DateRangeTest'], ['goog.date.Date', 'goog.date.DateRange', 'goog.date.Interval', 'goog.i18n.DateTimeSymbols', 'goog.testing.jsunit'], false);
goog.addDependency('date/duration.js', ['goog.date.duration'], ['goog.i18n.DateTimeFormat', 'goog.i18n.MessageFormat'], false);
goog.addDependency('date/duration_test.js', ['goog.date.durationTest'], ['goog.date.duration', 'goog.i18n.DateTimeFormat', 'goog.i18n.DateTimeSymbols', 'goog.i18n.DateTimeSymbols_bn', 'goog.i18n.DateTimeSymbols_en', 'goog.i18n.DateTimeSymbols_fa', 'goog.testing.jsunit'], false);
goog.addDependency('date/relative.js', ['goog.date.relative', 'goog.date.relative.TimeDeltaFormatter', 'goog.date.relative.Unit'], ['goog.i18n.DateTimeFormat'], false);
goog.addDependency('date/relative_test.js', ['goog.date.relativeTest'], ['goog.date.DateTime', 'goog.date.relative', 'goog.i18n.DateTimeFormat', 'goog.testing.jsunit'], false);
goog.addDependency('date/relativewithplurals.js', ['goog.date.relativeWithPlurals'], ['goog.date.relative', 'goog.date.relative.Unit', 'goog.i18n.MessageFormat'], false);
goog.addDependency('date/relativewithplurals_test.js', ['goog.date.relativeWithPluralsTest'], ['goog.date.relative', 'goog.date.relativeTest', 'goog.date.relativeWithPlurals', 'goog.i18n.DateTimeFormat', 'goog.i18n.DateTimeSymbols', 'goog.i18n.DateTimeSymbols_bn', 'goog.i18n.DateTimeSymbols_en', 'goog.i18n.DateTimeSymbols_fa', 'goog.i18n.NumberFormatSymbols', 'goog.i18n.NumberFormatSymbols_bn', 'goog.i18n.NumberFormatSymbols_en', 'goog.i18n.NumberFormatSymbols_fa'], false);
goog.addDependency('date/utcdatetime.js', ['goog.date.UtcDateTime'], ['goog.date', 'goog.date.Date', 'goog.date.DateTime', 'goog.date.Interval'], false);
goog.addDependency('date/utcdatetime_test.js', ['goog.date.UtcDateTimeTest'], ['goog.date.Interval', 'goog.date.UtcDateTime', 'goog.date.month', 'goog.date.weekDay', 'goog.testing.jsunit'], false);
goog.addDependency('db/cursor.js', ['goog.db.Cursor'], ['goog.async.Deferred', 'goog.db.Error', 'goog.debug', 'goog.events.EventTarget'], false);
goog.addDependency('db/db.js', ['goog.db'], ['goog.async.Deferred', 'goog.db.Error', 'goog.db.IndexedDb', 'goog.db.Transaction'], false);
goog.addDependency('db/db_test.js', ['goog.dbTest'], ['goog.Disposable', 'goog.array', 'goog.async.Deferred', 'goog.async.DeferredList', 'goog.db', 'goog.db.Cursor', 'goog.db.Error', 'goog.db.IndexedDb', 'goog.db.KeyRange', 'goog.db.Transaction', 'goog.events', 'goog.object', 'goog.testing.AsyncTestCase', 'goog.testing.PropertyReplacer', 'goog.testing.asserts', 'goog.testing.jsunit', 'goog.userAgent.product', 'goog.userAgent.product.isVersion'], false);
goog.addDependency('db/error.js', ['goog.db.Error', 'goog.db.Error.ErrorCode', 'goog.db.Error.ErrorName', 'goog.db.Error.VersionChangeBlockedError'], ['goog.debug.Error'], false);
goog.addDependency('db/index.js', ['goog.db.Index'], ['goog.async.Deferred', 'goog.db.Cursor', 'goog.db.Error', 'goog.debug'], false);
goog.addDependency('db/indexeddb.js', ['goog.db.IndexedDb'], ['goog.async.Deferred', 'goog.db.Error', 'goog.db.ObjectStore', 'goog.db.Transaction', 'goog.events.Event', 'goog.events.EventHandler', 'goog.events.EventTarget'], false);
goog.addDependency('db/keyrange.js', ['goog.db.KeyRange'], [], false);
goog.addDependency('db/objectstore.js', ['goog.db.ObjectStore'], ['goog.async.Deferred', 'goog.db.Cursor', 'goog.db.Error', 'goog.db.Index', 'goog.debug', 'goog.events'], false);
goog.addDependency('db/old_db_test.js', ['goog.oldDbTest'], ['goog.async.Deferred', 'goog.db', 'goog.db.Cursor', 'goog.db.Error', 'goog.db.IndexedDb', 'goog.db.KeyRange', 'goog.db.Transaction', 'goog.events', 'goog.testing.AsyncTestCase', 'goog.testing.asserts', 'goog.testing.jsunit', 'goog.userAgent.product', 'goog.userAgent.product.isVersion'], false);
goog.addDependency('db/transaction.js', ['goog.db.Transaction', 'goog.db.Transaction.TransactionMode'], ['goog.async.Deferred', 'goog.db.Error', 'goog.db.ObjectStore', 'goog.events.EventHandler', 'goog.events.EventTarget'], false);
goog.addDependency('debug/console.js', ['goog.debug.Console'], ['goog.debug.LogManager', 'goog.debug.Logger.Level', 'goog.debug.TextFormatter'], false);
goog.addDependency('debug/console_test.js', ['goog.debug.ConsoleTest'], ['goog.debug.Console', 'goog.debug.LogRecord', 'goog.debug.Logger', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('debug/debug.js', ['goog.debug'], ['goog.array', 'goog.string', 'goog.structs.Set', 'goog.userAgent'], false);
goog.addDependency('debug/debug_test.js', ['goog.debugTest'], ['goog.debug', 'goog.structs.Set', 'goog.testing.jsunit'], false);
goog.addDependency('debug/debugwindow.js', ['goog.debug.DebugWindow'], ['goog.debug.HtmlFormatter', 'goog.debug.LogManager', 'goog.debug.Logger', 'goog.structs.CircularBuffer', 'goog.userAgent'], false);
goog.addDependency('debug/debugwindow_test.js', ['goog.debug.DebugWindowTest'], ['goog.debug.DebugWindow', 'goog.testing.jsunit'], false);
goog.addDependency('debug/devcss/devcss.js', ['goog.debug.DevCss', 'goog.debug.DevCss.UserAgent'], ['goog.asserts', 'goog.cssom', 'goog.dom.classlist', 'goog.events', 'goog.events.EventType', 'goog.string', 'goog.userAgent'], false);
goog.addDependency('debug/devcss/devcss_test.js', ['goog.debug.DevCssTest'], ['goog.debug.DevCss', 'goog.style', 'goog.testing.jsunit'], false);
goog.addDependency('debug/devcss/devcssrunner.js', ['goog.debug.devCssRunner'], ['goog.debug.DevCss'], false);
goog.addDependency('debug/divconsole.js', ['goog.debug.DivConsole'], ['goog.debug.HtmlFormatter', 'goog.debug.LogManager', 'goog.style'], false);
goog.addDependency('debug/enhanceerror_test.js', ['goog.debugEnhanceErrorTest'], ['goog.debug', 'goog.testing.jsunit'], false);
goog.addDependency('debug/entrypointregistry.js', ['goog.debug.EntryPointMonitor', 'goog.debug.entryPointRegistry'], ['goog.asserts'], false);
goog.addDependency('debug/entrypointregistry_test.js', ['goog.debug.entryPointRegistryTest'], ['goog.debug.ErrorHandler', 'goog.debug.entryPointRegistry', 'goog.testing.jsunit'], false);
goog.addDependency('debug/error.js', ['goog.debug.Error'], [], false);
goog.addDependency('debug/error_test.js', ['goog.debug.ErrorTest'], ['goog.debug.Error', 'goog.testing.ExpectedFailures', 'goog.testing.jsunit', 'goog.userAgent', 'goog.userAgent.product'], false);
goog.addDependency('debug/errorhandler.js', ['goog.debug.ErrorHandler', 'goog.debug.ErrorHandler.ProtectedFunctionError'], ['goog.asserts', 'goog.debug', 'goog.debug.EntryPointMonitor', 'goog.debug.Trace'], false);
goog.addDependency('debug/errorhandler_async_test.js', ['goog.debug.ErrorHandlerAsyncTest'], ['goog.debug.ErrorHandler', 'goog.testing.AsyncTestCase', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('debug/errorhandler_test.js', ['goog.debug.ErrorHandlerTest'], ['goog.debug.ErrorHandler', 'goog.testing.MockControl', 'goog.testing.jsunit'], false);
goog.addDependency('debug/errorhandlerweakdep.js', ['goog.debug.errorHandlerWeakDep'], [], false);
goog.addDependency('debug/errorreporter.js', ['goog.debug.ErrorReporter', 'goog.debug.ErrorReporter.ExceptionEvent'], ['goog.asserts', 'goog.debug', 'goog.debug.ErrorHandler', 'goog.debug.entryPointRegistry', 'goog.events', 'goog.events.Event', 'goog.events.EventTarget', 'goog.log', 'goog.net.XhrIo', 'goog.object', 'goog.string', 'goog.uri.utils', 'goog.userAgent'], false);
goog.addDependency('debug/errorreporter_test.js', ['goog.debug.ErrorReporterTest'], ['goog.debug.ErrorReporter', 'goog.events', 'goog.functions', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('debug/fancywindow.js', ['goog.debug.FancyWindow'], ['goog.array', 'goog.debug.DebugWindow', 'goog.debug.LogManager', 'goog.debug.Logger', 'goog.dom.DomHelper', 'goog.dom.safe', 'goog.html.SafeHtml', 'goog.object', 'goog.string', 'goog.userAgent'], false);
goog.addDependency('debug/formatter.js', ['goog.debug.Formatter', 'goog.debug.HtmlFormatter', 'goog.debug.TextFormatter'], ['goog.debug.RelativeTimeProvider', 'goog.string'], false);
goog.addDependency('debug/fpsdisplay.js', ['goog.debug.FpsDisplay'], ['goog.asserts', 'goog.async.AnimationDelay', 'goog.ui.Component'], false);
goog.addDependency('debug/fpsdisplay_test.js', ['goog.debug.FpsDisplayTest'], ['goog.debug.FpsDisplay', 'goog.testing.AsyncTestCase', 'goog.testing.jsunit'], false);
goog.addDependency('debug/gcdiagnostics.js', ['goog.debug.GcDiagnostics'], ['goog.debug.Trace', 'goog.log', 'goog.userAgent'], false);
goog.addDependency('debug/logbuffer.js', ['goog.debug.LogBuffer'], ['goog.asserts', 'goog.debug.LogRecord'], false);
goog.addDependency('debug/logbuffer_test.js', ['goog.debug.LogBufferTest'], ['goog.debug.LogBuffer', 'goog.debug.Logger', 'goog.testing.jsunit'], false);
goog.addDependency('debug/logger.js', ['goog.debug.LogManager', 'goog.debug.Loggable', 'goog.debug.Logger', 'goog.debug.Logger.Level'], ['goog.array', 'goog.asserts', 'goog.debug', 'goog.debug.LogBuffer', 'goog.debug.LogRecord'], false);
goog.addDependency('debug/logger_test.js', ['goog.debug.LoggerTest'], ['goog.debug.LogManager', 'goog.debug.Logger', 'goog.testing.jsunit'], false);
goog.addDependency('debug/logrecord.js', ['goog.debug.LogRecord'], [], false);
goog.addDependency('debug/logrecordserializer.js', ['goog.debug.logRecordSerializer'], ['goog.debug.LogRecord', 'goog.debug.Logger.Level', 'goog.json', 'goog.object'], false);
goog.addDependency('debug/logrecordserializer_test.js', ['goog.debug.logRecordSerializerTest'], ['goog.debug.LogRecord', 'goog.debug.Logger', 'goog.debug.logRecordSerializer', 'goog.testing.jsunit'], false);
goog.addDependency('debug/relativetimeprovider.js', ['goog.debug.RelativeTimeProvider'], [], false);
goog.addDependency('debug/tracer.js', ['goog.debug.Trace'], ['goog.array', 'goog.iter', 'goog.log', 'goog.structs.Map', 'goog.structs.SimplePool'], false);
goog.addDependency('debug/tracer_test.js', ['goog.debug.TraceTest'], ['goog.debug.Trace', 'goog.testing.jsunit'], false);
goog.addDependency('defineclass_test.js', ['goog.defineClassTest'], ['goog.testing.jsunit'], false);
goog.addDependency('disposable/disposable.js', ['goog.Disposable', 'goog.dispose', 'goog.disposeAll'], ['goog.disposable.IDisposable'], false);
goog.addDependency('disposable/disposable_test.js', ['goog.DisposableTest'], ['goog.Disposable', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('disposable/idisposable.js', ['goog.disposable.IDisposable'], [], false);
goog.addDependency('dom/abstractmultirange.js', ['goog.dom.AbstractMultiRange'], ['goog.array', 'goog.dom', 'goog.dom.AbstractRange'], false);
goog.addDependency('dom/abstractrange.js', ['goog.dom.AbstractRange', 'goog.dom.RangeIterator', 'goog.dom.RangeType'], ['goog.dom', 'goog.dom.NodeType', 'goog.dom.SavedCaretRange', 'goog.dom.TagIterator', 'goog.userAgent'], false);
goog.addDependency('dom/abstractrange_test.js', ['goog.dom.AbstractRangeTest'], ['goog.dom', 'goog.dom.AbstractRange', 'goog.dom.Range', 'goog.testing.jsunit'], false);
goog.addDependency('dom/animationframe/animationframe.js', ['goog.dom.animationFrame', 'goog.dom.animationFrame.Spec', 'goog.dom.animationFrame.State'], ['goog.dom.animationFrame.polyfill'], false);
goog.addDependency('dom/animationframe/polyfill.js', ['goog.dom.animationFrame.polyfill'], [], false);
goog.addDependency('dom/annotate.js', ['goog.dom.annotate', 'goog.dom.annotate.AnnotateFn'], ['goog.array', 'goog.asserts', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.safe', 'goog.html.SafeHtml'], false);
goog.addDependency('dom/annotate_test.js', ['goog.dom.annotateTest'], ['goog.dom', 'goog.dom.annotate', 'goog.html.SafeHtml', 'goog.testing.jsunit'], false);
goog.addDependency('dom/browserfeature.js', ['goog.dom.BrowserFeature'], ['goog.userAgent'], false);
goog.addDependency('dom/browserrange/abstractrange.js', ['goog.dom.browserrange.AbstractRange'], ['goog.array', 'goog.asserts', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.RangeEndpoint', 'goog.dom.TagName', 'goog.dom.TextRangeIterator', 'goog.iter', 'goog.math.Coordinate', 'goog.string', 'goog.string.StringBuffer', 'goog.userAgent'], false);
goog.addDependency('dom/browserrange/browserrange.js', ['goog.dom.browserrange', 'goog.dom.browserrange.Error'], ['goog.dom', 'goog.dom.BrowserFeature', 'goog.dom.NodeType', 'goog.dom.browserrange.GeckoRange', 'goog.dom.browserrange.IeRange', 'goog.dom.browserrange.OperaRange', 'goog.dom.browserrange.W3cRange', 'goog.dom.browserrange.WebKitRange', 'goog.userAgent'], false);
goog.addDependency('dom/browserrange/browserrange_test.js', ['goog.dom.browserrangeTest'], ['goog.dom', 'goog.dom.NodeType', 'goog.dom.Range', 'goog.dom.RangeEndpoint', 'goog.dom.TagName', 'goog.dom.browserrange', 'goog.testing.dom', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('dom/browserrange/geckorange.js', ['goog.dom.browserrange.GeckoRange'], ['goog.dom.browserrange.W3cRange'], false);
goog.addDependency('dom/browserrange/ierange.js', ['goog.dom.browserrange.IeRange'], ['goog.array', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.RangeEndpoint', 'goog.dom.TagName', 'goog.dom.browserrange.AbstractRange', 'goog.log', 'goog.string'], false);
goog.addDependency('dom/browserrange/operarange.js', ['goog.dom.browserrange.OperaRange'], ['goog.dom.browserrange.W3cRange'], false);
goog.addDependency('dom/browserrange/w3crange.js', ['goog.dom.browserrange.W3cRange'], ['goog.array', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.RangeEndpoint', 'goog.dom.browserrange.AbstractRange', 'goog.string', 'goog.userAgent'], false);
goog.addDependency('dom/browserrange/webkitrange.js', ['goog.dom.browserrange.WebKitRange'], ['goog.dom.RangeEndpoint', 'goog.dom.browserrange.W3cRange', 'goog.userAgent'], false);
goog.addDependency('dom/bufferedviewportsizemonitor.js', ['goog.dom.BufferedViewportSizeMonitor'], ['goog.asserts', 'goog.async.Delay', 'goog.events', 'goog.events.EventTarget', 'goog.events.EventType'], false);
goog.addDependency('dom/bufferedviewportsizemonitor_test.js', ['goog.dom.BufferedViewportSizeMonitorTest'], ['goog.dom.BufferedViewportSizeMonitor', 'goog.dom.ViewportSizeMonitor', 'goog.events', 'goog.events.EventType', 'goog.math.Size', 'goog.testing.MockClock', 'goog.testing.events', 'goog.testing.events.Event', 'goog.testing.jsunit'], false);
goog.addDependency('dom/classes.js', ['goog.dom.classes'], ['goog.array'], false);
goog.addDependency('dom/classes_test.js', ['goog.dom.classes_test'], ['goog.dom', 'goog.dom.classes', 'goog.testing.jsunit'], false);
goog.addDependency('dom/classlist.js', ['goog.dom.classlist'], ['goog.array'], false);
goog.addDependency('dom/classlist_test.js', ['goog.dom.classlist_test'], ['goog.dom', 'goog.dom.classlist', 'goog.testing.ExpectedFailures', 'goog.testing.jsunit'], false);
goog.addDependency('dom/controlrange.js', ['goog.dom.ControlRange', 'goog.dom.ControlRangeIterator'], ['goog.array', 'goog.dom', 'goog.dom.AbstractMultiRange', 'goog.dom.AbstractRange', 'goog.dom.RangeIterator', 'goog.dom.RangeType', 'goog.dom.SavedRange', 'goog.dom.TagWalkType', 'goog.dom.TextRange', 'goog.iter.StopIteration', 'goog.userAgent'], false);
goog.addDependency('dom/controlrange_test.js', ['goog.dom.ControlRangeTest'], ['goog.dom', 'goog.dom.ControlRange', 'goog.dom.RangeType', 'goog.dom.TagName', 'goog.dom.TextRange', 'goog.testing.dom', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('dom/dataset.js', ['goog.dom.dataset'], ['goog.string'], false);
goog.addDependency('dom/dataset_test.js', ['goog.dom.datasetTest'], ['goog.dom', 'goog.dom.dataset', 'goog.testing.jsunit'], false);
goog.addDependency('dom/dom.js', ['goog.dom', 'goog.dom.Appendable', 'goog.dom.DomHelper'], ['goog.array', 'goog.asserts', 'goog.dom.BrowserFeature', 'goog.dom.NodeType', 'goog.dom.TagName', 'goog.math.Coordinate', 'goog.math.Size', 'goog.object', 'goog.string', 'goog.userAgent'], false);
goog.addDependency('dom/dom_test.js', ['goog.dom.dom_test'], ['goog.dom', 'goog.dom.BrowserFeature', 'goog.dom.DomHelper', 'goog.dom.NodeType', 'goog.dom.TagName', 'goog.functions', 'goog.object', 'goog.string.Unicode', 'goog.testing.PropertyReplacer', 'goog.testing.asserts', 'goog.userAgent', 'goog.userAgent.product', 'goog.userAgent.product.isVersion'], false);
goog.addDependency('dom/fontsizemonitor.js', ['goog.dom.FontSizeMonitor', 'goog.dom.FontSizeMonitor.EventType'], ['goog.dom', 'goog.events', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.userAgent'], false);
goog.addDependency('dom/fontsizemonitor_test.js', ['goog.dom.FontSizeMonitorTest'], ['goog.dom', 'goog.dom.FontSizeMonitor', 'goog.events', 'goog.events.Event', 'goog.testing.PropertyReplacer', 'goog.testing.events', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('dom/forms.js', ['goog.dom.forms'], ['goog.structs.Map'], false);
goog.addDependency('dom/forms_test.js', ['goog.dom.formsTest'], ['goog.dom', 'goog.dom.forms', 'goog.testing.jsunit'], false);
goog.addDependency('dom/fullscreen.js', ['goog.dom.fullscreen', 'goog.dom.fullscreen.EventType'], ['goog.dom', 'goog.userAgent'], false);
goog.addDependency('dom/iframe.js', ['goog.dom.iframe'], ['goog.dom', 'goog.userAgent'], false);
goog.addDependency('dom/iframe_test.js', ['goog.dom.iframeTest'], ['goog.dom', 'goog.dom.iframe', 'goog.testing.jsunit'], false);
goog.addDependency('dom/iter.js', ['goog.dom.iter.AncestorIterator', 'goog.dom.iter.ChildIterator', 'goog.dom.iter.SiblingIterator'], ['goog.iter.Iterator', 'goog.iter.StopIteration'], false);
goog.addDependency('dom/iter_test.js', ['goog.dom.iterTest'], ['goog.dom', 'goog.dom.NodeType', 'goog.dom.iter.AncestorIterator', 'goog.dom.iter.ChildIterator', 'goog.dom.iter.SiblingIterator', 'goog.testing.dom', 'goog.testing.jsunit'], false);
goog.addDependency('dom/multirange.js', ['goog.dom.MultiRange', 'goog.dom.MultiRangeIterator'], ['goog.array', 'goog.dom.AbstractMultiRange', 'goog.dom.AbstractRange', 'goog.dom.RangeIterator', 'goog.dom.RangeType', 'goog.dom.SavedRange', 'goog.dom.TextRange', 'goog.iter.StopIteration', 'goog.log'], false);
goog.addDependency('dom/multirange_test.js', ['goog.dom.MultiRangeTest'], ['goog.dom', 'goog.dom.MultiRange', 'goog.dom.Range', 'goog.iter', 'goog.testing.jsunit'], false);
goog.addDependency('dom/nodeiterator.js', ['goog.dom.NodeIterator'], ['goog.dom.TagIterator'], false);
goog.addDependency('dom/nodeiterator_test.js', ['goog.dom.NodeIteratorTest'], ['goog.dom', 'goog.dom.NodeIterator', 'goog.testing.dom', 'goog.testing.jsunit'], false);
goog.addDependency('dom/nodeoffset.js', ['goog.dom.NodeOffset'], ['goog.Disposable', 'goog.dom.TagName'], false);
goog.addDependency('dom/nodeoffset_test.js', ['goog.dom.NodeOffsetTest'], ['goog.dom', 'goog.dom.NodeOffset', 'goog.dom.NodeType', 'goog.dom.TagName', 'goog.testing.jsunit'], false);
goog.addDependency('dom/nodetype.js', ['goog.dom.NodeType'], [], false);
goog.addDependency('dom/pattern/abstractpattern.js', ['goog.dom.pattern.AbstractPattern'], ['goog.dom.pattern.MatchType'], false);
goog.addDependency('dom/pattern/allchildren.js', ['goog.dom.pattern.AllChildren'], ['goog.dom.pattern.AbstractPattern', 'goog.dom.pattern.MatchType'], false);
goog.addDependency('dom/pattern/callback/callback.js', ['goog.dom.pattern.callback'], ['goog.dom', 'goog.dom.TagWalkType', 'goog.iter'], false);
goog.addDependency('dom/pattern/callback/counter.js', ['goog.dom.pattern.callback.Counter'], [], false);
goog.addDependency('dom/pattern/callback/test.js', ['goog.dom.pattern.callback.Test'], ['goog.iter.StopIteration'], false);
goog.addDependency('dom/pattern/childmatches.js', ['goog.dom.pattern.ChildMatches'], ['goog.dom.pattern.AllChildren', 'goog.dom.pattern.MatchType'], false);
goog.addDependency('dom/pattern/endtag.js', ['goog.dom.pattern.EndTag'], ['goog.dom.TagWalkType', 'goog.dom.pattern.Tag'], false);
goog.addDependency('dom/pattern/fulltag.js', ['goog.dom.pattern.FullTag'], ['goog.dom.pattern.MatchType', 'goog.dom.pattern.StartTag', 'goog.dom.pattern.Tag'], false);
goog.addDependency('dom/pattern/matcher.js', ['goog.dom.pattern.Matcher'], ['goog.dom.TagIterator', 'goog.dom.pattern.MatchType', 'goog.iter'], false);
goog.addDependency('dom/pattern/matcher_test.js', ['goog.dom.pattern.matcherTest'], ['goog.dom', 'goog.dom.pattern.EndTag', 'goog.dom.pattern.FullTag', 'goog.dom.pattern.Matcher', 'goog.dom.pattern.Repeat', 'goog.dom.pattern.Sequence', 'goog.dom.pattern.StartTag', 'goog.dom.pattern.callback.Counter', 'goog.dom.pattern.callback.Test', 'goog.iter.StopIteration', 'goog.testing.jsunit'], false);
goog.addDependency('dom/pattern/nodetype.js', ['goog.dom.pattern.NodeType'], ['goog.dom.pattern.AbstractPattern', 'goog.dom.pattern.MatchType'], false);
goog.addDependency('dom/pattern/pattern.js', ['goog.dom.pattern', 'goog.dom.pattern.MatchType'], [], false);
goog.addDependency('dom/pattern/pattern_test.js', ['goog.dom.patternTest'], ['goog.dom', 'goog.dom.NodeType', 'goog.dom.TagWalkType', 'goog.dom.pattern.AllChildren', 'goog.dom.pattern.ChildMatches', 'goog.dom.pattern.EndTag', 'goog.dom.pattern.FullTag', 'goog.dom.pattern.MatchType', 'goog.dom.pattern.NodeType', 'goog.dom.pattern.Repeat', 'goog.dom.pattern.Sequence', 'goog.dom.pattern.StartTag', 'goog.dom.pattern.Text', 'goog.testing.jsunit'], false);
goog.addDependency('dom/pattern/repeat.js', ['goog.dom.pattern.Repeat'], ['goog.dom.NodeType', 'goog.dom.pattern.AbstractPattern', 'goog.dom.pattern.MatchType'], false);
goog.addDependency('dom/pattern/sequence.js', ['goog.dom.pattern.Sequence'], ['goog.dom.NodeType', 'goog.dom.pattern.AbstractPattern', 'goog.dom.pattern.MatchType'], false);
goog.addDependency('dom/pattern/starttag.js', ['goog.dom.pattern.StartTag'], ['goog.dom.TagWalkType', 'goog.dom.pattern.Tag'], false);
goog.addDependency('dom/pattern/tag.js', ['goog.dom.pattern.Tag'], ['goog.dom.pattern', 'goog.dom.pattern.AbstractPattern', 'goog.dom.pattern.MatchType', 'goog.object'], false);
goog.addDependency('dom/pattern/text.js', ['goog.dom.pattern.Text'], ['goog.dom.NodeType', 'goog.dom.pattern', 'goog.dom.pattern.AbstractPattern', 'goog.dom.pattern.MatchType'], false);
goog.addDependency('dom/range.js', ['goog.dom.Range'], ['goog.dom', 'goog.dom.AbstractRange', 'goog.dom.BrowserFeature', 'goog.dom.ControlRange', 'goog.dom.MultiRange', 'goog.dom.NodeType', 'goog.dom.TextRange', 'goog.userAgent'], false);
goog.addDependency('dom/range_test.js', ['goog.dom.RangeTest'], ['goog.dom', 'goog.dom.NodeType', 'goog.dom.Range', 'goog.dom.RangeType', 'goog.dom.TagName', 'goog.dom.TextRange', 'goog.dom.browserrange', 'goog.testing.dom', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('dom/rangeendpoint.js', ['goog.dom.RangeEndpoint'], [], false);
goog.addDependency('dom/safe.js', ['goog.dom.safe'], ['goog.html.SafeHtml', 'goog.html.SafeUrl'], false);
goog.addDependency('dom/safe_test.js', ['goog.dom.safeTest'], ['goog.dom.safe', 'goog.html.SafeUrl', 'goog.html.testing', 'goog.string.Const', 'goog.testing.jsunit'], false);
goog.addDependency('dom/savedcaretrange.js', ['goog.dom.SavedCaretRange'], ['goog.array', 'goog.dom', 'goog.dom.SavedRange', 'goog.dom.TagName', 'goog.string'], false);
goog.addDependency('dom/savedcaretrange_test.js', ['goog.dom.SavedCaretRangeTest'], ['goog.dom', 'goog.dom.Range', 'goog.dom.SavedCaretRange', 'goog.testing.dom', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('dom/savedrange.js', ['goog.dom.SavedRange'], ['goog.Disposable', 'goog.log'], false);
goog.addDependency('dom/savedrange_test.js', ['goog.dom.SavedRangeTest'], ['goog.dom', 'goog.dom.Range', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('dom/selection.js', ['goog.dom.selection'], ['goog.string', 'goog.userAgent'], false);
goog.addDependency('dom/selection_test.js', ['goog.dom.selectionTest'], ['goog.dom', 'goog.dom.selection', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('dom/tagiterator.js', ['goog.dom.TagIterator', 'goog.dom.TagWalkType'], ['goog.dom', 'goog.dom.NodeType', 'goog.iter.Iterator', 'goog.iter.StopIteration'], false);
goog.addDependency('dom/tagiterator_test.js', ['goog.dom.TagIteratorTest'], ['goog.dom', 'goog.dom.TagIterator', 'goog.dom.TagWalkType', 'goog.iter', 'goog.iter.StopIteration', 'goog.testing.dom', 'goog.testing.jsunit'], false);
goog.addDependency('dom/tagname.js', ['goog.dom.TagName'], [], false);
goog.addDependency('dom/tagname_test.js', ['goog.dom.TagNameTest'], ['goog.dom.TagName', 'goog.object', 'goog.testing.jsunit'], false);
goog.addDependency('dom/tags.js', ['goog.dom.tags'], ['goog.object'], false);
goog.addDependency('dom/tags_test.js', ['goog.dom.tagsTest'], ['goog.dom.tags', 'goog.testing.jsunit'], false);
goog.addDependency('dom/textrange.js', ['goog.dom.TextRange'], ['goog.array', 'goog.dom', 'goog.dom.AbstractRange', 'goog.dom.RangeType', 'goog.dom.SavedRange', 'goog.dom.TagName', 'goog.dom.TextRangeIterator', 'goog.dom.browserrange', 'goog.string', 'goog.userAgent'], false);
goog.addDependency('dom/textrange_test.js', ['goog.dom.TextRangeTest'], ['goog.dom', 'goog.dom.ControlRange', 'goog.dom.Range', 'goog.dom.TextRange', 'goog.math.Coordinate', 'goog.style', 'goog.testing.ExpectedFailures', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('dom/textrangeiterator.js', ['goog.dom.TextRangeIterator'], ['goog.array', 'goog.dom.NodeType', 'goog.dom.RangeIterator', 'goog.dom.TagName', 'goog.iter.StopIteration'], false);
goog.addDependency('dom/textrangeiterator_test.js', ['goog.dom.TextRangeIteratorTest'], ['goog.dom', 'goog.dom.TagName', 'goog.dom.TextRangeIterator', 'goog.iter.StopIteration', 'goog.testing.dom', 'goog.testing.jsunit'], false);
goog.addDependency('dom/vendor.js', ['goog.dom.vendor'], ['goog.string', 'goog.userAgent'], false);
goog.addDependency('dom/vendor_test.js', ['goog.dom.vendorTest'], ['goog.array', 'goog.dom.vendor', 'goog.labs.userAgent.util', 'goog.testing.MockUserAgent', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.userAgent', 'goog.userAgentTestUtil'], false);
goog.addDependency('dom/viewportsizemonitor.js', ['goog.dom.ViewportSizeMonitor'], ['goog.dom', 'goog.events', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.math.Size'], false);
goog.addDependency('dom/viewportsizemonitor_test.js', ['goog.dom.ViewportSizeMonitorTest'], ['goog.dom.ViewportSizeMonitor', 'goog.events', 'goog.events.Event', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.math.Size', 'goog.testing.MockClock', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit'], false);
goog.addDependency('dom/xml.js', ['goog.dom.xml'], ['goog.dom', 'goog.dom.NodeType'], false);
goog.addDependency('dom/xml_test.js', ['goog.dom.xmlTest'], ['goog.dom.xml', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('editor/browserfeature.js', ['goog.editor.BrowserFeature'], ['goog.editor.defines', 'goog.userAgent', 'goog.userAgent.product', 'goog.userAgent.product.isVersion'], false);
goog.addDependency('editor/browserfeature_test.js', ['goog.editor.BrowserFeatureTest'], ['goog.dom', 'goog.dom.Range', 'goog.editor.BrowserFeature', 'goog.testing.ExpectedFailures', 'goog.testing.jsunit'], false);
goog.addDependency('editor/clicktoeditwrapper.js', ['goog.editor.ClickToEditWrapper'], ['goog.Disposable', 'goog.asserts', 'goog.dom', 'goog.dom.Range', 'goog.dom.TagName', 'goog.editor.BrowserFeature', 'goog.editor.Command', 'goog.editor.Field.EventType', 'goog.editor.range', 'goog.events.BrowserEvent.MouseButton', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.log'], false);
goog.addDependency('editor/clicktoeditwrapper_test.js', ['goog.editor.ClickToEditWrapperTest'], ['goog.dom', 'goog.dom.Range', 'goog.editor.ClickToEditWrapper', 'goog.editor.SeamlessField', 'goog.testing.MockClock', 'goog.testing.events', 'goog.testing.jsunit', 'goog.userAgent', 'goog.userAgent.product'], false);
goog.addDependency('editor/command.js', ['goog.editor.Command'], [], false);
goog.addDependency('editor/contenteditablefield.js', ['goog.editor.ContentEditableField'], ['goog.asserts', 'goog.editor.Field', 'goog.log'], false);
goog.addDependency('editor/contenteditablefield_test.js', ['goog.editor.ContentEditableFieldTest'], ['goog.dom', 'goog.editor.ContentEditableField', 'goog.editor.field_test', 'goog.testing.jsunit'], false);
goog.addDependency('editor/defines.js', ['goog.editor.defines'], [], false);
goog.addDependency('editor/field.js', ['goog.editor.Field', 'goog.editor.Field.EventType'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.array', 'goog.asserts', 'goog.async.Delay', 'goog.dom', 'goog.dom.Range', 'goog.dom.TagName', 'goog.editor.BrowserFeature', 'goog.editor.Command', 'goog.editor.Plugin', 'goog.editor.icontent', 'goog.editor.icontent.FieldFormatInfo', 'goog.editor.icontent.FieldStyleInfo', 'goog.editor.node', 'goog.editor.range', 'goog.events', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.functions', 'goog.log', 'goog.log.Level', 'goog.string', 'goog.string.Unicode', 'goog.style', 'goog.userAgent', 'goog.userAgent.product'], false);
goog.addDependency('editor/field_test.js', ['goog.editor.field_test'], ['goog.dom', 'goog.dom.Range', 'goog.editor.BrowserFeature', 'goog.editor.Field', 'goog.editor.Plugin', 'goog.editor.range', 'goog.events', 'goog.events.BrowserEvent', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.functions', 'goog.testing.LooseMock', 'goog.testing.MockClock', 'goog.testing.dom', 'goog.testing.events', 'goog.testing.events.Event', 'goog.testing.recordFunction', 'goog.userAgent'], false);
goog.addDependency('editor/focus.js', ['goog.editor.focus'], ['goog.dom.selection'], false);
goog.addDependency('editor/focus_test.js', ['goog.editor.focusTest'], ['goog.dom.selection', 'goog.editor.BrowserFeature', 'goog.editor.focus', 'goog.testing.jsunit'], false);
goog.addDependency('editor/icontent.js', ['goog.editor.icontent', 'goog.editor.icontent.FieldFormatInfo', 'goog.editor.icontent.FieldStyleInfo'], ['goog.editor.BrowserFeature', 'goog.style', 'goog.userAgent'], false);
goog.addDependency('editor/icontent_test.js', ['goog.editor.icontentTest'], ['goog.dom', 'goog.editor.BrowserFeature', 'goog.editor.icontent', 'goog.editor.icontent.FieldFormatInfo', 'goog.editor.icontent.FieldStyleInfo', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('editor/link.js', ['goog.editor.Link'], ['goog.array', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.Range', 'goog.dom.TagName', 'goog.editor.BrowserFeature', 'goog.editor.Command', 'goog.editor.node', 'goog.editor.range', 'goog.string', 'goog.string.Unicode', 'goog.uri.utils', 'goog.uri.utils.ComponentIndex'], false);
goog.addDependency('editor/link_test.js', ['goog.editor.LinkTest'], ['goog.dom', 'goog.dom.NodeType', 'goog.dom.Range', 'goog.dom.TagName', 'goog.editor.Link', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('editor/node.js', ['goog.editor.node'], ['goog.dom', 'goog.dom.NodeType', 'goog.dom.TagName', 'goog.dom.iter.ChildIterator', 'goog.dom.iter.SiblingIterator', 'goog.iter', 'goog.object', 'goog.string', 'goog.string.Unicode', 'goog.userAgent'], false);
goog.addDependency('editor/node_test.js', ['goog.editor.nodeTest'], ['goog.array', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.TagName', 'goog.editor.node', 'goog.style', 'goog.testing.ExpectedFailures', 'goog.testing.dom', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('editor/plugin.js', ['goog.editor.Plugin'], ['goog.editor.Command', 'goog.events.EventTarget', 'goog.functions', 'goog.log', 'goog.object', 'goog.reflect'], false);
goog.addDependency('editor/plugin_test.js', ['goog.editor.PluginTest'], ['goog.editor.Field', 'goog.editor.Plugin', 'goog.functions', 'goog.testing.StrictMock', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/abstractbubbleplugin.js', ['goog.editor.plugins.AbstractBubblePlugin'], ['goog.array', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.Range', 'goog.dom.TagName', 'goog.dom.classlist', 'goog.editor.Plugin', 'goog.editor.style', 'goog.events', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.events.actionEventWrapper', 'goog.functions', 'goog.string.Unicode', 'goog.ui.Component', 'goog.ui.editor.Bubble', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/abstractbubbleplugin_test.js', ['goog.editor.plugins.AbstractBubblePluginTest'], ['goog.dom', 'goog.editor.plugins.AbstractBubblePlugin', 'goog.events.BrowserEvent', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.functions', 'goog.style', 'goog.testing.editor.FieldMock', 'goog.testing.editor.TestHelper', 'goog.testing.events', 'goog.testing.events.Event', 'goog.testing.jsunit', 'goog.ui.editor.Bubble', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/abstractdialogplugin.js', ['goog.editor.plugins.AbstractDialogPlugin', 'goog.editor.plugins.AbstractDialogPlugin.EventType'], ['goog.dom', 'goog.dom.Range', 'goog.editor.Field.EventType', 'goog.editor.Plugin', 'goog.editor.range', 'goog.events', 'goog.ui.editor.AbstractDialog.EventType'], false);
goog.addDependency('editor/plugins/abstractdialogplugin_test.js', ['goog.editor.plugins.AbstractDialogPluginTest'], ['goog.dom.SavedRange', 'goog.editor.Field', 'goog.editor.plugins.AbstractDialogPlugin', 'goog.events.Event', 'goog.events.EventHandler', 'goog.functions', 'goog.testing.MockClock', 'goog.testing.MockControl', 'goog.testing.PropertyReplacer', 'goog.testing.editor.FieldMock', 'goog.testing.editor.TestHelper', 'goog.testing.events', 'goog.testing.jsunit', 'goog.testing.mockmatchers.ArgumentMatcher', 'goog.ui.editor.AbstractDialog', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/abstracttabhandler.js', ['goog.editor.plugins.AbstractTabHandler'], ['goog.editor.Plugin', 'goog.events.KeyCodes'], false);
goog.addDependency('editor/plugins/abstracttabhandler_test.js', ['goog.editor.plugins.AbstractTabHandlerTest'], ['goog.editor.Field', 'goog.editor.plugins.AbstractTabHandler', 'goog.events.BrowserEvent', 'goog.events.KeyCodes', 'goog.testing.StrictMock', 'goog.testing.editor.FieldMock', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/basictextformatter.js', ['goog.editor.plugins.BasicTextFormatter', 'goog.editor.plugins.BasicTextFormatter.COMMAND'], ['goog.array', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.Range', 'goog.dom.TagName', 'goog.editor.BrowserFeature', 'goog.editor.Command', 'goog.editor.Link', 'goog.editor.Plugin', 'goog.editor.node', 'goog.editor.range', 'goog.editor.style', 'goog.iter', 'goog.iter.StopIteration', 'goog.log', 'goog.object', 'goog.string', 'goog.string.Unicode', 'goog.style', 'goog.ui.editor.messages', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/basictextformatter_test.js', ['goog.editor.plugins.BasicTextFormatterTest'], ['goog.dom', 'goog.dom.Range', 'goog.dom.TagName', 'goog.editor.BrowserFeature', 'goog.editor.Command', 'goog.editor.Field', 'goog.editor.Plugin', 'goog.editor.plugins.BasicTextFormatter', 'goog.object', 'goog.style', 'goog.testing.ExpectedFailures', 'goog.testing.LooseMock', 'goog.testing.PropertyReplacer', 'goog.testing.editor.FieldMock', 'goog.testing.editor.TestHelper', 'goog.testing.jsunit', 'goog.testing.mockmatchers', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/blockquote.js', ['goog.editor.plugins.Blockquote'], ['goog.dom', 'goog.dom.NodeType', 'goog.dom.TagName', 'goog.dom.classlist', 'goog.editor.BrowserFeature', 'goog.editor.Command', 'goog.editor.Plugin', 'goog.editor.node', 'goog.functions', 'goog.log'], false);
goog.addDependency('editor/plugins/blockquote_test.js', ['goog.editor.plugins.BlockquoteTest'], ['goog.dom', 'goog.dom.Range', 'goog.dom.TagName', 'goog.editor.plugins.Blockquote', 'goog.testing.editor.FieldMock', 'goog.testing.editor.TestHelper', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/emoticons.js', ['goog.editor.plugins.Emoticons'], ['goog.dom.TagName', 'goog.editor.Plugin', 'goog.editor.range', 'goog.functions', 'goog.ui.emoji.Emoji', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/emoticons_test.js', ['goog.editor.plugins.EmoticonsTest'], ['goog.Uri', 'goog.array', 'goog.dom', 'goog.dom.TagName', 'goog.editor.Field', 'goog.editor.plugins.Emoticons', 'goog.testing.jsunit', 'goog.ui.emoji.Emoji', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/enterhandler.js', ['goog.editor.plugins.EnterHandler'], ['goog.dom', 'goog.dom.NodeOffset', 'goog.dom.NodeType', 'goog.dom.Range', 'goog.dom.TagName', 'goog.editor.BrowserFeature', 'goog.editor.Plugin', 'goog.editor.node', 'goog.editor.plugins.Blockquote', 'goog.editor.range', 'goog.editor.style', 'goog.events.KeyCodes', 'goog.functions', 'goog.object', 'goog.string', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/enterhandler_test.js', ['goog.editor.plugins.EnterHandlerTest'], ['goog.dom', 'goog.dom.NodeType', 'goog.dom.Range', 'goog.dom.TagName', 'goog.editor.BrowserFeature', 'goog.editor.Field', 'goog.editor.Plugin', 'goog.editor.plugins.Blockquote', 'goog.editor.plugins.EnterHandler', 'goog.editor.range', 'goog.events', 'goog.events.KeyCodes', 'goog.testing.ExpectedFailures', 'goog.testing.MockClock', 'goog.testing.dom', 'goog.testing.editor.TestHelper', 'goog.testing.events', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/firststrong.js', ['goog.editor.plugins.FirstStrong'], ['goog.dom.NodeType', 'goog.dom.TagIterator', 'goog.dom.TagName', 'goog.editor.Command', 'goog.editor.Plugin', 'goog.editor.node', 'goog.editor.range', 'goog.i18n.bidi', 'goog.i18n.uChar', 'goog.iter', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/firststrong_test.js', ['goog.editor.plugins.FirstStrongTest'], ['goog.dom.Range', 'goog.editor.Command', 'goog.editor.Field', 'goog.editor.plugins.FirstStrong', 'goog.editor.range', 'goog.events.KeyCodes', 'goog.testing.editor.TestHelper', 'goog.testing.events', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/headerformatter.js', ['goog.editor.plugins.HeaderFormatter'], ['goog.editor.Command', 'goog.editor.Plugin', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/headerformatter_test.js', ['goog.editor.plugins.HeaderFormatterTest'], ['goog.dom', 'goog.editor.Command', 'goog.editor.plugins.BasicTextFormatter', 'goog.editor.plugins.HeaderFormatter', 'goog.events.BrowserEvent', 'goog.testing.LooseMock', 'goog.testing.editor.FieldMock', 'goog.testing.editor.TestHelper', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/linkbubble.js', ['goog.editor.plugins.LinkBubble', 'goog.editor.plugins.LinkBubble.Action'], ['goog.array', 'goog.dom', 'goog.dom.TagName', 'goog.editor.Command', 'goog.editor.Link', 'goog.editor.plugins.AbstractBubblePlugin', 'goog.editor.range', 'goog.functions', 'goog.string', 'goog.style', 'goog.ui.editor.messages', 'goog.uri.utils', 'goog.window'], false);
goog.addDependency('editor/plugins/linkbubble_test.js', ['goog.editor.plugins.LinkBubbleTest'], ['goog.dom', 'goog.dom.Range', 'goog.dom.TagName', 'goog.editor.Command', 'goog.editor.Link', 'goog.editor.plugins.LinkBubble', 'goog.events.BrowserEvent', 'goog.events.Event', 'goog.events.EventType', 'goog.string', 'goog.style', 'goog.testing.FunctionMock', 'goog.testing.PropertyReplacer', 'goog.testing.editor.FieldMock', 'goog.testing.editor.TestHelper', 'goog.testing.events', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/linkdialogplugin.js', ['goog.editor.plugins.LinkDialogPlugin'], ['goog.array', 'goog.dom', 'goog.editor.Command', 'goog.editor.plugins.AbstractDialogPlugin', 'goog.events.EventHandler', 'goog.functions', 'goog.ui.editor.AbstractDialog.EventType', 'goog.ui.editor.LinkDialog', 'goog.ui.editor.LinkDialog.EventType', 'goog.ui.editor.LinkDialog.OkEvent', 'goog.uri.utils'], false);
goog.addDependency('editor/plugins/linkdialogplugin_test.js', ['goog.ui.editor.plugins.LinkDialogTest'], ['goog.dom', 'goog.dom.DomHelper', 'goog.dom.NodeType', 'goog.dom.TagName', 'goog.editor.BrowserFeature', 'goog.editor.Command', 'goog.editor.Field', 'goog.editor.Link', 'goog.editor.plugins.LinkDialogPlugin', 'goog.string', 'goog.string.Unicode', 'goog.testing.MockControl', 'goog.testing.editor.FieldMock', 'goog.testing.editor.TestHelper', 'goog.testing.editor.dom', 'goog.testing.events', 'goog.testing.jsunit', 'goog.testing.mockmatchers', 'goog.ui.editor.AbstractDialog', 'goog.ui.editor.LinkDialog', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/linkshortcutplugin.js', ['goog.editor.plugins.LinkShortcutPlugin'], ['goog.editor.Command', 'goog.editor.Link', 'goog.editor.Plugin', 'goog.string'], false);
goog.addDependency('editor/plugins/linkshortcutplugin_test.js', ['goog.editor.plugins.LinkShortcutPluginTest'], ['goog.dom', 'goog.editor.Field', 'goog.editor.plugins.BasicTextFormatter', 'goog.editor.plugins.LinkBubble', 'goog.editor.plugins.LinkShortcutPlugin', 'goog.events.KeyCodes', 'goog.testing.PropertyReplacer', 'goog.testing.dom', 'goog.testing.events', 'goog.testing.jsunit'], false);
goog.addDependency('editor/plugins/listtabhandler.js', ['goog.editor.plugins.ListTabHandler'], ['goog.dom.TagName', 'goog.editor.Command', 'goog.editor.plugins.AbstractTabHandler'], false);
goog.addDependency('editor/plugins/listtabhandler_test.js', ['goog.editor.plugins.ListTabHandlerTest'], ['goog.dom', 'goog.editor.Command', 'goog.editor.plugins.ListTabHandler', 'goog.events.BrowserEvent', 'goog.events.KeyCodes', 'goog.functions', 'goog.testing.StrictMock', 'goog.testing.editor.FieldMock', 'goog.testing.editor.TestHelper', 'goog.testing.jsunit'], false);
goog.addDependency('editor/plugins/loremipsum.js', ['goog.editor.plugins.LoremIpsum'], ['goog.asserts', 'goog.dom', 'goog.editor.Command', 'goog.editor.Plugin', 'goog.editor.node', 'goog.functions'], false);
goog.addDependency('editor/plugins/loremipsum_test.js', ['goog.editor.plugins.LoremIpsumTest'], ['goog.dom', 'goog.editor.Command', 'goog.editor.Field', 'goog.editor.plugins.LoremIpsum', 'goog.string.Unicode', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/removeformatting.js', ['goog.editor.plugins.RemoveFormatting'], ['goog.dom', 'goog.dom.NodeType', 'goog.dom.Range', 'goog.dom.TagName', 'goog.editor.BrowserFeature', 'goog.editor.Plugin', 'goog.editor.node', 'goog.editor.range', 'goog.string', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/removeformatting_test.js', ['goog.editor.plugins.RemoveFormattingTest'], ['goog.dom', 'goog.dom.Range', 'goog.dom.TagName', 'goog.editor.BrowserFeature', 'goog.editor.plugins.RemoveFormatting', 'goog.string', 'goog.testing.ExpectedFailures', 'goog.testing.dom', 'goog.testing.editor.FieldMock', 'goog.testing.editor.TestHelper', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/spacestabhandler.js', ['goog.editor.plugins.SpacesTabHandler'], ['goog.dom', 'goog.dom.TagName', 'goog.editor.plugins.AbstractTabHandler', 'goog.editor.range'], false);
goog.addDependency('editor/plugins/spacestabhandler_test.js', ['goog.editor.plugins.SpacesTabHandlerTest'], ['goog.dom', 'goog.dom.Range', 'goog.editor.plugins.SpacesTabHandler', 'goog.events.BrowserEvent', 'goog.events.KeyCodes', 'goog.functions', 'goog.testing.StrictMock', 'goog.testing.editor.FieldMock', 'goog.testing.editor.TestHelper', 'goog.testing.jsunit'], false);
goog.addDependency('editor/plugins/tableeditor.js', ['goog.editor.plugins.TableEditor'], ['goog.array', 'goog.dom', 'goog.dom.TagName', 'goog.editor.Plugin', 'goog.editor.Table', 'goog.editor.node', 'goog.editor.range', 'goog.object'], false);
goog.addDependency('editor/plugins/tableeditor_test.js', ['goog.editor.plugins.TableEditorTest'], ['goog.dom', 'goog.dom.Range', 'goog.editor.plugins.TableEditor', 'goog.object', 'goog.string', 'goog.testing.ExpectedFailures', 'goog.testing.JsUnitException', 'goog.testing.editor.FieldMock', 'goog.testing.editor.TestHelper', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/tagonenterhandler.js', ['goog.editor.plugins.TagOnEnterHandler'], ['goog.dom', 'goog.dom.NodeType', 'goog.dom.Range', 'goog.dom.TagName', 'goog.editor.Command', 'goog.editor.node', 'goog.editor.plugins.EnterHandler', 'goog.editor.range', 'goog.editor.style', 'goog.events.KeyCodes', 'goog.string', 'goog.style', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/tagonenterhandler_test.js', ['goog.editor.plugins.TagOnEnterHandlerTest'], ['goog.dom', 'goog.dom.NodeType', 'goog.dom.Range', 'goog.dom.TagName', 'goog.editor.BrowserFeature', 'goog.editor.Field', 'goog.editor.Plugin', 'goog.editor.plugins.TagOnEnterHandler', 'goog.events.KeyCodes', 'goog.string.Unicode', 'goog.testing.dom', 'goog.testing.editor.TestHelper', 'goog.testing.events', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('editor/plugins/undoredo.js', ['goog.editor.plugins.UndoRedo'], ['goog.dom', 'goog.dom.NodeOffset', 'goog.dom.Range', 'goog.editor.BrowserFeature', 'goog.editor.Command', 'goog.editor.Field.EventType', 'goog.editor.Plugin', 'goog.editor.node', 'goog.editor.plugins.UndoRedoManager', 'goog.editor.plugins.UndoRedoState', 'goog.events', 'goog.events.EventHandler', 'goog.log'], false);
goog.addDependency('editor/plugins/undoredo_test.js', ['goog.editor.plugins.UndoRedoTest'], ['goog.array', 'goog.dom', 'goog.dom.browserrange', 'goog.editor.Field', 'goog.editor.plugins.LoremIpsum', 'goog.editor.plugins.UndoRedo', 'goog.events', 'goog.functions', 'goog.testing.MockClock', 'goog.testing.PropertyReplacer', 'goog.testing.StrictMock', 'goog.testing.jsunit'], false);
goog.addDependency('editor/plugins/undoredomanager.js', ['goog.editor.plugins.UndoRedoManager', 'goog.editor.plugins.UndoRedoManager.EventType'], ['goog.editor.plugins.UndoRedoState', 'goog.events.EventTarget'], false);
goog.addDependency('editor/plugins/undoredomanager_test.js', ['goog.editor.plugins.UndoRedoManagerTest'], ['goog.editor.plugins.UndoRedoManager', 'goog.editor.plugins.UndoRedoState', 'goog.events', 'goog.testing.StrictMock', 'goog.testing.jsunit'], false);
goog.addDependency('editor/plugins/undoredostate.js', ['goog.editor.plugins.UndoRedoState'], ['goog.events.EventTarget'], false);
goog.addDependency('editor/plugins/undoredostate_test.js', ['goog.editor.plugins.UndoRedoStateTest'], ['goog.editor.plugins.UndoRedoState', 'goog.testing.jsunit'], false);
goog.addDependency('editor/range.js', ['goog.editor.range', 'goog.editor.range.Point'], ['goog.array', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.Range', 'goog.dom.RangeEndpoint', 'goog.dom.SavedCaretRange', 'goog.editor.node', 'goog.editor.style', 'goog.iter', 'goog.userAgent'], false);
goog.addDependency('editor/range_test.js', ['goog.editor.rangeTest'], ['goog.dom', 'goog.dom.Range', 'goog.dom.TagName', 'goog.editor.range', 'goog.editor.range.Point', 'goog.string', 'goog.testing.dom', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('editor/seamlessfield.js', ['goog.editor.SeamlessField'], ['goog.cssom.iframe.style', 'goog.dom', 'goog.dom.Range', 'goog.dom.TagName', 'goog.editor.BrowserFeature', 'goog.editor.Field', 'goog.editor.icontent', 'goog.editor.icontent.FieldFormatInfo', 'goog.editor.icontent.FieldStyleInfo', 'goog.editor.node', 'goog.events', 'goog.events.EventType', 'goog.log', 'goog.style'], false);
goog.addDependency('editor/seamlessfield_test.js', ['goog.editor.seamlessfield_test'], ['goog.dom', 'goog.dom.DomHelper', 'goog.dom.Range', 'goog.editor.BrowserFeature', 'goog.editor.Field', 'goog.editor.SeamlessField', 'goog.events', 'goog.functions', 'goog.style', 'goog.testing.MockClock', 'goog.testing.MockRange', 'goog.testing.jsunit'], false);
goog.addDependency('editor/style.js', ['goog.editor.style'], ['goog.dom', 'goog.dom.NodeType', 'goog.editor.BrowserFeature', 'goog.events.EventType', 'goog.object', 'goog.style', 'goog.userAgent'], false);
goog.addDependency('editor/style_test.js', ['goog.editor.styleTest'], ['goog.dom', 'goog.dom.TagName', 'goog.editor.BrowserFeature', 'goog.editor.style', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.style', 'goog.testing.LooseMock', 'goog.testing.jsunit', 'goog.testing.mockmatchers'], false);
goog.addDependency('editor/table.js', ['goog.editor.Table', 'goog.editor.TableCell', 'goog.editor.TableRow'], ['goog.dom', 'goog.dom.DomHelper', 'goog.dom.NodeType', 'goog.dom.TagName', 'goog.log', 'goog.string.Unicode', 'goog.style'], false);
goog.addDependency('editor/table_test.js', ['goog.editor.TableTest'], ['goog.dom', 'goog.editor.Table', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('events/actioneventwrapper.js', ['goog.events.actionEventWrapper'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.dom', 'goog.events', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.events.EventWrapper', 'goog.events.KeyCodes', 'goog.userAgent'], false);
goog.addDependency('events/actioneventwrapper_test.js', ['goog.events.actionEventWrapperTest'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.events', 'goog.events.EventHandler', 'goog.events.KeyCodes', 'goog.events.actionEventWrapper', 'goog.testing.events', 'goog.testing.jsunit'], false);
goog.addDependency('events/actionhandler.js', ['goog.events.ActionEvent', 'goog.events.ActionHandler', 'goog.events.ActionHandler.EventType', 'goog.events.BeforeActionEvent'], ['goog.events', 'goog.events.BrowserEvent', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.userAgent'], false);
goog.addDependency('events/actionhandler_test.js', ['goog.events.ActionHandlerTest'], ['goog.dom', 'goog.events', 'goog.events.ActionHandler', 'goog.testing.events', 'goog.testing.jsunit'], false);
goog.addDependency('events/browserevent.js', ['goog.events.BrowserEvent', 'goog.events.BrowserEvent.MouseButton'], ['goog.events.BrowserFeature', 'goog.events.Event', 'goog.events.EventType', 'goog.reflect', 'goog.userAgent'], false);
goog.addDependency('events/browserevent_test.js', ['goog.events.BrowserEventTest'], ['goog.events.BrowserEvent', 'goog.events.BrowserFeature', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('events/browserfeature.js', ['goog.events.BrowserFeature'], ['goog.userAgent'], false);
goog.addDependency('events/event.js', ['goog.events.Event', 'goog.events.EventLike'], ['goog.Disposable', 'goog.events.EventId'], false);
goog.addDependency('events/event_test.js', ['goog.events.EventTest'], ['goog.events.Event', 'goog.events.EventId', 'goog.events.EventTarget', 'goog.testing.jsunit'], false);
goog.addDependency('events/eventhandler.js', ['goog.events.EventHandler'], ['goog.Disposable', 'goog.events', 'goog.object'], false);
goog.addDependency('events/eventhandler_test.js', ['goog.events.EventHandlerTest'], ['goog.events', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('events/eventid.js', ['goog.events.EventId'], [], false);
goog.addDependency('events/events.js', ['goog.events', 'goog.events.CaptureSimulationMode', 'goog.events.Key', 'goog.events.ListenableType'], ['goog.asserts', 'goog.debug.entryPointRegistry', 'goog.events.BrowserEvent', 'goog.events.BrowserFeature', 'goog.events.Listenable', 'goog.events.ListenerMap'], false);
goog.addDependency('events/events_test.js', ['goog.eventsTest'], ['goog.asserts.AssertionError', 'goog.debug.EntryPointMonitor', 'goog.debug.ErrorHandler', 'goog.debug.entryPointRegistry', 'goog.dom', 'goog.events', 'goog.events.BrowserFeature', 'goog.events.CaptureSimulationMode', 'goog.events.Event', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.events.Listener', 'goog.functions', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('events/eventtarget.js', ['goog.events.EventTarget'], ['goog.Disposable', 'goog.asserts', 'goog.events', 'goog.events.Event', 'goog.events.Listenable', 'goog.events.ListenerMap', 'goog.object'], false);
goog.addDependency('events/eventtarget_test.js', ['goog.events.EventTargetTest'], ['goog.events.EventTarget', 'goog.events.Listenable', 'goog.events.eventTargetTester', 'goog.events.eventTargetTester.KeyType', 'goog.events.eventTargetTester.UnlistenReturnType', 'goog.testing.jsunit'], false);
goog.addDependency('events/eventtarget_via_googevents_test.js', ['goog.events.EventTargetGoogEventsTest'], ['goog.events', 'goog.events.EventTarget', 'goog.events.eventTargetTester', 'goog.events.eventTargetTester.KeyType', 'goog.events.eventTargetTester.UnlistenReturnType', 'goog.testing', 'goog.testing.jsunit'], false);
goog.addDependency('events/eventtarget_via_w3cinterface_test.js', ['goog.events.EventTargetW3CTest'], ['goog.events.EventTarget', 'goog.events.eventTargetTester', 'goog.events.eventTargetTester.KeyType', 'goog.events.eventTargetTester.UnlistenReturnType', 'goog.testing.jsunit'], false);
goog.addDependency('events/eventtargettester.js', ['goog.events.eventTargetTester', 'goog.events.eventTargetTester.KeyType', 'goog.events.eventTargetTester.UnlistenReturnType'], ['goog.array', 'goog.events', 'goog.events.Event', 'goog.events.EventTarget', 'goog.testing.asserts', 'goog.testing.recordFunction'], false);
goog.addDependency('events/eventtype.js', ['goog.events.EventType'], ['goog.userAgent'], false);
goog.addDependency('events/eventwrapper.js', ['goog.events.EventWrapper'], [], false);
goog.addDependency('events/filedrophandler.js', ['goog.events.FileDropHandler', 'goog.events.FileDropHandler.EventType'], ['goog.array', 'goog.dom', 'goog.events', 'goog.events.BrowserEvent', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.log'], false);
goog.addDependency('events/filedrophandler_test.js', ['goog.events.FileDropHandlerTest'], ['goog.events', 'goog.events.BrowserEvent', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.events.FileDropHandler', 'goog.testing.jsunit'], false);
goog.addDependency('events/focushandler.js', ['goog.events.FocusHandler', 'goog.events.FocusHandler.EventType'], ['goog.events', 'goog.events.BrowserEvent', 'goog.events.EventTarget', 'goog.userAgent'], false);
goog.addDependency('events/imehandler.js', ['goog.events.ImeHandler', 'goog.events.ImeHandler.Event', 'goog.events.ImeHandler.EventType'], ['goog.events.Event', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.userAgent'], false);
goog.addDependency('events/imehandler_test.js', ['goog.events.ImeHandlerTest'], ['goog.array', 'goog.dom', 'goog.events', 'goog.events.ImeHandler', 'goog.events.KeyCodes', 'goog.object', 'goog.string', 'goog.testing.PropertyReplacer', 'goog.testing.events', 'goog.testing.events.Event', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('events/inputhandler.js', ['goog.events.InputHandler', 'goog.events.InputHandler.EventType'], ['goog.Timer', 'goog.dom', 'goog.events.BrowserEvent', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.events.KeyCodes', 'goog.userAgent'], false);
goog.addDependency('events/inputhandler_test.js', ['goog.events.InputHandlerTest'], ['goog.dom', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.events.InputHandler', 'goog.events.KeyCodes', 'goog.testing.events', 'goog.testing.events.Event', 'goog.testing.jsunit', 'goog.testing.recordFunction', 'goog.userAgent'], false);
goog.addDependency('events/keycodes.js', ['goog.events.KeyCodes'], ['goog.userAgent'], false);
goog.addDependency('events/keycodes_test.js', ['goog.events.KeyCodesTest'], ['goog.events.BrowserEvent', 'goog.events.KeyCodes', 'goog.object', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('events/keyhandler.js', ['goog.events.KeyEvent', 'goog.events.KeyHandler', 'goog.events.KeyHandler.EventType'], ['goog.events', 'goog.events.BrowserEvent', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.userAgent'], false);
goog.addDependency('events/keyhandler_test.js', ['goog.events.KeyEventTest'], ['goog.dom', 'goog.events', 'goog.events.BrowserEvent', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.events.KeyHandler', 'goog.testing.events', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('events/keynames.js', ['goog.events.KeyNames'], [], false);
goog.addDependency('events/listenable.js', ['goog.events.Listenable', 'goog.events.ListenableKey'], ['goog.events.EventId'], false);
goog.addDependency('events/listenable_test.js', ['goog.events.ListenableTest'], ['goog.events.Listenable', 'goog.testing.jsunit'], false);
goog.addDependency('events/listener.js', ['goog.events.Listener'], ['goog.events.ListenableKey'], false);
goog.addDependency('events/listenermap.js', ['goog.events.ListenerMap'], ['goog.array', 'goog.events.Listener', 'goog.object'], false);
goog.addDependency('events/listenermap_test.js', ['goog.events.ListenerMapTest'], ['goog.dispose', 'goog.events', 'goog.events.EventId', 'goog.events.EventTarget', 'goog.events.ListenerMap', 'goog.testing.jsunit'], false);
goog.addDependency('events/mousewheelhandler.js', ['goog.events.MouseWheelEvent', 'goog.events.MouseWheelHandler', 'goog.events.MouseWheelHandler.EventType'], ['goog.dom', 'goog.events', 'goog.events.BrowserEvent', 'goog.events.EventTarget', 'goog.math', 'goog.style', 'goog.userAgent'], false);
goog.addDependency('events/mousewheelhandler_test.js', ['goog.events.MouseWheelHandlerTest'], ['goog.dom', 'goog.events', 'goog.events.BrowserEvent', 'goog.events.MouseWheelEvent', 'goog.events.MouseWheelHandler', 'goog.functions', 'goog.string', 'goog.testing.PropertyReplacer', 'goog.testing.events', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('events/onlinehandler.js', ['goog.events.OnlineHandler', 'goog.events.OnlineHandler.EventType'], ['goog.Timer', 'goog.events.BrowserFeature', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.net.NetworkStatusMonitor'], false);
goog.addDependency('events/onlinelistener_test.js', ['goog.events.OnlineHandlerTest'], ['goog.events', 'goog.events.BrowserFeature', 'goog.events.Event', 'goog.events.EventHandler', 'goog.events.OnlineHandler', 'goog.net.NetworkStatusMonitor', 'goog.testing.MockClock', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('events/pastehandler.js', ['goog.events.PasteHandler', 'goog.events.PasteHandler.EventType', 'goog.events.PasteHandler.State'], ['goog.Timer', 'goog.async.ConditionalDelay', 'goog.events.BrowserEvent', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.log', 'goog.userAgent'], false);
goog.addDependency('events/pastehandler_test.js', ['goog.events.PasteHandlerTest'], ['goog.dom', 'goog.events', 'goog.events.BrowserEvent', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.events.PasteHandler', 'goog.testing.MockClock', 'goog.testing.MockUserAgent', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('format/emailaddress.js', ['goog.format.EmailAddress'], ['goog.string'], false);
goog.addDependency('format/emailaddress_test.js', ['goog.format.EmailAddressTest'], ['goog.array', 'goog.format.EmailAddress', 'goog.testing.jsunit'], false);
goog.addDependency('format/format.js', ['goog.format'], ['goog.i18n.GraphemeBreak', 'goog.string', 'goog.userAgent'], false);
goog.addDependency('format/format_test.js', ['goog.formatTest'], ['goog.dom', 'goog.format', 'goog.string', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit'], false);
goog.addDependency('format/htmlprettyprinter.js', ['goog.format.HtmlPrettyPrinter', 'goog.format.HtmlPrettyPrinter.Buffer'], ['goog.object', 'goog.string.StringBuffer'], false);
goog.addDependency('format/htmlprettyprinter_test.js', ['goog.format.HtmlPrettyPrinterTest'], ['goog.format.HtmlPrettyPrinter', 'goog.testing.MockClock', 'goog.testing.jsunit'], false);
goog.addDependency('format/internationalizedemailaddress.js', ['goog.format.InternationalizedEmailAddress'], ['goog.format.EmailAddress', 'goog.string'], false);
goog.addDependency('format/internationalizedemailaddress_test.js', ['goog.format.InternationalizedEmailAddressTest'], ['goog.array', 'goog.format.InternationalizedEmailAddress', 'goog.testing.jsunit'], false);
goog.addDependency('format/jsonprettyprinter.js', ['goog.format.JsonPrettyPrinter', 'goog.format.JsonPrettyPrinter.HtmlDelimiters', 'goog.format.JsonPrettyPrinter.TextDelimiters'], ['goog.json', 'goog.json.Serializer', 'goog.string', 'goog.string.StringBuffer', 'goog.string.format'], false);
goog.addDependency('format/jsonprettyprinter_test.js', ['goog.format.JsonPrettyPrinterTest'], ['goog.format.JsonPrettyPrinter', 'goog.testing.jsunit'], false);
goog.addDependency('fs/entry.js', ['goog.fs.DirectoryEntry', 'goog.fs.DirectoryEntry.Behavior', 'goog.fs.Entry', 'goog.fs.FileEntry'], [], false);
goog.addDependency('fs/entryimpl.js', ['goog.fs.DirectoryEntryImpl', 'goog.fs.EntryImpl', 'goog.fs.FileEntryImpl'], ['goog.array', 'goog.async.Deferred', 'goog.fs.DirectoryEntry', 'goog.fs.Entry', 'goog.fs.Error', 'goog.fs.FileEntry', 'goog.fs.FileWriter', 'goog.functions', 'goog.string'], false);
goog.addDependency('fs/error.js', ['goog.fs.Error', 'goog.fs.Error.ErrorCode'], ['goog.debug.Error', 'goog.object', 'goog.string'], false);
goog.addDependency('fs/filereader.js', ['goog.fs.FileReader', 'goog.fs.FileReader.EventType', 'goog.fs.FileReader.ReadyState'], ['goog.async.Deferred', 'goog.events.EventTarget', 'goog.fs.Error', 'goog.fs.ProgressEvent'], false);
goog.addDependency('fs/filesaver.js', ['goog.fs.FileSaver', 'goog.fs.FileSaver.EventType', 'goog.fs.FileSaver.ProgressEvent', 'goog.fs.FileSaver.ReadyState'], ['goog.events.EventTarget', 'goog.fs.Error', 'goog.fs.ProgressEvent'], false);
goog.addDependency('fs/filesystem.js', ['goog.fs.FileSystem'], [], false);
goog.addDependency('fs/filesystemimpl.js', ['goog.fs.FileSystemImpl'], ['goog.fs.DirectoryEntryImpl', 'goog.fs.FileSystem'], false);
goog.addDependency('fs/filewriter.js', ['goog.fs.FileWriter'], ['goog.fs.Error', 'goog.fs.FileSaver'], false);
goog.addDependency('fs/fs.js', ['goog.fs'], ['goog.array', 'goog.async.Deferred', 'goog.fs.Error', 'goog.fs.FileReader', 'goog.fs.FileSystemImpl', 'goog.userAgent'], false);
goog.addDependency('fs/fs_test.js', ['goog.fsTest'], ['goog.array', 'goog.async.Deferred', 'goog.async.DeferredList', 'goog.dom', 'goog.events', 'goog.fs', 'goog.fs.DirectoryEntry', 'goog.fs.Error', 'goog.fs.FileReader', 'goog.fs.FileSaver', 'goog.string', 'goog.testing.AsyncTestCase', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit'], false);
goog.addDependency('fs/progressevent.js', ['goog.fs.ProgressEvent'], ['goog.events.Event'], false);
goog.addDependency('functions/functions.js', ['goog.functions'], [], false);
goog.addDependency('functions/functions_test.js', ['goog.functionsTest'], ['goog.functions', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('fx/abstractdragdrop.js', ['goog.fx.AbstractDragDrop', 'goog.fx.AbstractDragDrop.EventType', 'goog.fx.DragDropEvent', 'goog.fx.DragDropItem'], ['goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.events', 'goog.events.Event', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.fx.Dragger', 'goog.math.Box', 'goog.math.Coordinate', 'goog.style'], false);
goog.addDependency('fx/abstractdragdrop_test.js', ['goog.fx.AbstractDragDropTest'], ['goog.array', 'goog.events.EventType', 'goog.functions', 'goog.fx.AbstractDragDrop', 'goog.fx.DragDropItem', 'goog.math.Box', 'goog.math.Coordinate', 'goog.style', 'goog.testing.events', 'goog.testing.jsunit'], false);
goog.addDependency('fx/anim/anim.js', ['goog.fx.anim', 'goog.fx.anim.Animated'], ['goog.async.AnimationDelay', 'goog.async.Delay', 'goog.object'], false);
goog.addDependency('fx/anim/anim_test.js', ['goog.fx.animTest'], ['goog.async.AnimationDelay', 'goog.async.Delay', 'goog.events', 'goog.functions', 'goog.fx.Animation', 'goog.fx.anim', 'goog.object', 'goog.testing.MockClock', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.recordFunction', 'goog.userAgent'], false);
goog.addDependency('fx/animation.js', ['goog.fx.Animation', 'goog.fx.Animation.EventType', 'goog.fx.Animation.State', 'goog.fx.AnimationEvent'], ['goog.array', 'goog.events.Event', 'goog.fx.Transition', 'goog.fx.Transition.EventType', 'goog.fx.TransitionBase.State', 'goog.fx.anim', 'goog.fx.anim.Animated'], false);
goog.addDependency('fx/animation_test.js', ['goog.fx.AnimationTest'], ['goog.events', 'goog.fx.Animation', 'goog.testing.MockClock', 'goog.testing.jsunit'], false);
goog.addDependency('fx/animationqueue.js', ['goog.fx.AnimationParallelQueue', 'goog.fx.AnimationQueue', 'goog.fx.AnimationSerialQueue'], ['goog.array', 'goog.asserts', 'goog.events.EventHandler', 'goog.fx.Transition.EventType', 'goog.fx.TransitionBase', 'goog.fx.TransitionBase.State'], false);
goog.addDependency('fx/animationqueue_test.js', ['goog.fx.AnimationQueueTest'], ['goog.events', 'goog.fx.Animation', 'goog.fx.AnimationParallelQueue', 'goog.fx.AnimationSerialQueue', 'goog.fx.Transition', 'goog.fx.anim', 'goog.testing.MockClock', 'goog.testing.jsunit'], false);
goog.addDependency('fx/css3/fx.js', ['goog.fx.css3'], ['goog.fx.css3.Transition'], false);
goog.addDependency('fx/css3/transition.js', ['goog.fx.css3.Transition'], ['goog.Timer', 'goog.asserts', 'goog.fx.TransitionBase', 'goog.style', 'goog.style.transition'], false);
goog.addDependency('fx/css3/transition_test.js', ['goog.fx.css3.TransitionTest'], ['goog.dispose', 'goog.dom', 'goog.events', 'goog.fx.Transition', 'goog.fx.css3.Transition', 'goog.style.transition', 'goog.testing.MockClock', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('fx/cssspriteanimation.js', ['goog.fx.CssSpriteAnimation'], ['goog.fx.Animation'], false);
goog.addDependency('fx/cssspriteanimation_test.js', ['goog.fx.CssSpriteAnimationTest'], ['goog.fx.CssSpriteAnimation', 'goog.math.Box', 'goog.math.Size', 'goog.testing.MockClock', 'goog.testing.jsunit'], false);
goog.addDependency('fx/dom.js', ['goog.fx.dom', 'goog.fx.dom.BgColorTransform', 'goog.fx.dom.ColorTransform', 'goog.fx.dom.Fade', 'goog.fx.dom.FadeIn', 'goog.fx.dom.FadeInAndShow', 'goog.fx.dom.FadeOut', 'goog.fx.dom.FadeOutAndHide', 'goog.fx.dom.PredefinedEffect', 'goog.fx.dom.Resize', 'goog.fx.dom.ResizeHeight', 'goog.fx.dom.ResizeWidth', 'goog.fx.dom.Scroll', 'goog.fx.dom.Slide', 'goog.fx.dom.SlideFrom', 'goog.fx.dom.Swipe'], ['goog.color', 'goog.events', 'goog.fx.Animation', 'goog.fx.Transition', 'goog.style', 'goog.style.bidi'], false);
goog.addDependency('fx/dragdrop.js', ['goog.fx.DragDrop'], ['goog.fx.AbstractDragDrop', 'goog.fx.DragDropItem'], false);
goog.addDependency('fx/dragdropgroup.js', ['goog.fx.DragDropGroup'], ['goog.dom', 'goog.fx.AbstractDragDrop', 'goog.fx.DragDropItem'], false);
goog.addDependency('fx/dragdropgroup_test.js', ['goog.fx.DragDropGroupTest'], ['goog.events', 'goog.fx.DragDropGroup', 'goog.testing.jsunit'], false);
goog.addDependency('fx/dragger.js', ['goog.fx.DragEvent', 'goog.fx.Dragger', 'goog.fx.Dragger.EventType'], ['goog.dom', 'goog.events', 'goog.events.Event', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.math.Coordinate', 'goog.math.Rect', 'goog.style', 'goog.style.bidi', 'goog.userAgent'], false);
goog.addDependency('fx/dragger_test.js', ['goog.fx.DraggerTest'], ['goog.dom', 'goog.events', 'goog.events.BrowserEvent', 'goog.events.Event', 'goog.events.EventType', 'goog.fx.Dragger', 'goog.math.Rect', 'goog.style.bidi', 'goog.testing.StrictMock', 'goog.testing.events', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('fx/draglistgroup.js', ['goog.fx.DragListDirection', 'goog.fx.DragListGroup', 'goog.fx.DragListGroup.EventType', 'goog.fx.DragListGroupEvent'], ['goog.array', 'goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.events', 'goog.events.Event', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.fx.Dragger', 'goog.math.Coordinate', 'goog.string', 'goog.style'], false);
goog.addDependency('fx/draglistgroup_test.js', ['goog.fx.DragListGroupTest'], ['goog.array', 'goog.dom', 'goog.dom.classlist', 'goog.events', 'goog.events.BrowserEvent', 'goog.events.BrowserFeature', 'goog.events.Event', 'goog.events.EventType', 'goog.fx.DragEvent', 'goog.fx.DragListDirection', 'goog.fx.DragListGroup', 'goog.fx.Dragger', 'goog.math.Coordinate', 'goog.object', 'goog.testing.events', 'goog.testing.jsunit'], false);
goog.addDependency('fx/dragscrollsupport.js', ['goog.fx.DragScrollSupport'], ['goog.Disposable', 'goog.Timer', 'goog.dom', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.math.Coordinate', 'goog.style'], false);
goog.addDependency('fx/dragscrollsupport_test.js', ['goog.fx.DragScrollSupportTest'], ['goog.fx.DragScrollSupport', 'goog.math.Coordinate', 'goog.testing.MockClock', 'goog.testing.events', 'goog.testing.jsunit'], false);
goog.addDependency('fx/easing.js', ['goog.fx.easing'], [], false);
goog.addDependency('fx/easing_test.js', ['goog.fx.easingTest'], ['goog.fx.easing', 'goog.testing.jsunit'], false);
goog.addDependency('fx/fx.js', ['goog.fx'], ['goog.asserts', 'goog.fx.Animation', 'goog.fx.Animation.EventType', 'goog.fx.Animation.State', 'goog.fx.AnimationEvent', 'goog.fx.Transition.EventType', 'goog.fx.easing'], false);
goog.addDependency('fx/fx_test.js', ['goog.fxTest'], ['goog.fx.Animation', 'goog.object', 'goog.testing.MockClock', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit'], false);
goog.addDependency('fx/transition.js', ['goog.fx.Transition', 'goog.fx.Transition.EventType'], [], false);
goog.addDependency('fx/transitionbase.js', ['goog.fx.TransitionBase', 'goog.fx.TransitionBase.State'], ['goog.events.EventTarget', 'goog.fx.Transition', 'goog.fx.Transition.EventType'], false);
goog.addDependency('graphics/abstractgraphics.js', ['goog.graphics.AbstractGraphics'], ['goog.dom', 'goog.graphics.Path', 'goog.math.Coordinate', 'goog.math.Size', 'goog.style', 'goog.ui.Component'], false);
goog.addDependency('graphics/affinetransform.js', ['goog.graphics.AffineTransform'], ['goog.math'], false);
goog.addDependency('graphics/canvaselement.js', ['goog.graphics.CanvasEllipseElement', 'goog.graphics.CanvasGroupElement', 'goog.graphics.CanvasImageElement', 'goog.graphics.CanvasPathElement', 'goog.graphics.CanvasRectElement', 'goog.graphics.CanvasTextElement'], ['goog.array', 'goog.dom', 'goog.dom.TagName', 'goog.graphics.EllipseElement', 'goog.graphics.GroupElement', 'goog.graphics.ImageElement', 'goog.graphics.Path', 'goog.graphics.PathElement', 'goog.graphics.RectElement', 'goog.graphics.TextElement', 'goog.math', 'goog.string'], false);
goog.addDependency('graphics/canvasgraphics.js', ['goog.graphics.CanvasGraphics'], ['goog.events.EventType', 'goog.graphics.AbstractGraphics', 'goog.graphics.CanvasEllipseElement', 'goog.graphics.CanvasGroupElement', 'goog.graphics.CanvasImageElement', 'goog.graphics.CanvasPathElement', 'goog.graphics.CanvasRectElement', 'goog.graphics.CanvasTextElement', 'goog.graphics.SolidFill', 'goog.math.Size', 'goog.style'], false);
goog.addDependency('graphics/element.js', ['goog.graphics.Element'], ['goog.asserts', 'goog.events', 'goog.events.EventTarget', 'goog.events.Listenable', 'goog.graphics.AffineTransform', 'goog.math'], false);
goog.addDependency('graphics/ellipseelement.js', ['goog.graphics.EllipseElement'], ['goog.graphics.StrokeAndFillElement'], false);
goog.addDependency('graphics/ext/coordinates.js', ['goog.graphics.ext.coordinates'], ['goog.string'], false);
goog.addDependency('graphics/ext/element.js', ['goog.graphics.ext.Element'], ['goog.events.EventTarget', 'goog.functions', 'goog.graphics.ext.coordinates'], false);
goog.addDependency('graphics/ext/ellipse.js', ['goog.graphics.ext.Ellipse'], ['goog.graphics.ext.StrokeAndFillElement'], false);
goog.addDependency('graphics/ext/ext.js', ['goog.graphics.ext'], ['goog.graphics.ext.Ellipse', 'goog.graphics.ext.Graphics', 'goog.graphics.ext.Group', 'goog.graphics.ext.Image', 'goog.graphics.ext.Rectangle', 'goog.graphics.ext.Shape', 'goog.graphics.ext.coordinates'], false);
goog.addDependency('graphics/ext/graphics.js', ['goog.graphics.ext.Graphics'], ['goog.events.EventType', 'goog.graphics.ext.Group'], false);
goog.addDependency('graphics/ext/group.js', ['goog.graphics.ext.Group'], ['goog.array', 'goog.graphics.ext.Element'], false);
goog.addDependency('graphics/ext/image.js', ['goog.graphics.ext.Image'], ['goog.graphics.ext.Element'], false);
goog.addDependency('graphics/ext/path.js', ['goog.graphics.ext.Path'], ['goog.graphics.AffineTransform', 'goog.graphics.Path', 'goog.math', 'goog.math.Rect'], false);
goog.addDependency('graphics/ext/rectangle.js', ['goog.graphics.ext.Rectangle'], ['goog.graphics.ext.StrokeAndFillElement'], false);
goog.addDependency('graphics/ext/shape.js', ['goog.graphics.ext.Shape'], ['goog.graphics.ext.Path', 'goog.graphics.ext.StrokeAndFillElement', 'goog.math.Rect'], false);
goog.addDependency('graphics/ext/strokeandfillelement.js', ['goog.graphics.ext.StrokeAndFillElement'], ['goog.graphics.ext.Element'], false);
goog.addDependency('graphics/fill.js', ['goog.graphics.Fill'], [], false);
goog.addDependency('graphics/font.js', ['goog.graphics.Font'], [], false);
goog.addDependency('graphics/graphics.js', ['goog.graphics'], ['goog.dom', 'goog.graphics.CanvasGraphics', 'goog.graphics.SvgGraphics', 'goog.graphics.VmlGraphics', 'goog.userAgent'], false);
goog.addDependency('graphics/groupelement.js', ['goog.graphics.GroupElement'], ['goog.graphics.Element'], false);
goog.addDependency('graphics/imageelement.js', ['goog.graphics.ImageElement'], ['goog.graphics.Element'], false);
goog.addDependency('graphics/lineargradient.js', ['goog.graphics.LinearGradient'], ['goog.asserts', 'goog.graphics.Fill'], false);
goog.addDependency('graphics/path.js', ['goog.graphics.Path', 'goog.graphics.Path.Segment'], ['goog.array', 'goog.math'], false);
goog.addDependency('graphics/pathelement.js', ['goog.graphics.PathElement'], ['goog.graphics.StrokeAndFillElement'], false);
goog.addDependency('graphics/paths.js', ['goog.graphics.paths'], ['goog.graphics.Path', 'goog.math.Coordinate'], false);
goog.addDependency('graphics/rectelement.js', ['goog.graphics.RectElement'], ['goog.graphics.StrokeAndFillElement'], false);
goog.addDependency('graphics/solidfill.js', ['goog.graphics.SolidFill'], ['goog.graphics.Fill'], false);
goog.addDependency('graphics/stroke.js', ['goog.graphics.Stroke'], [], false);
goog.addDependency('graphics/strokeandfillelement.js', ['goog.graphics.StrokeAndFillElement'], ['goog.graphics.Element'], false);
goog.addDependency('graphics/svgelement.js', ['goog.graphics.SvgEllipseElement', 'goog.graphics.SvgGroupElement', 'goog.graphics.SvgImageElement', 'goog.graphics.SvgPathElement', 'goog.graphics.SvgRectElement', 'goog.graphics.SvgTextElement'], ['goog.dom', 'goog.graphics.EllipseElement', 'goog.graphics.GroupElement', 'goog.graphics.ImageElement', 'goog.graphics.PathElement', 'goog.graphics.RectElement', 'goog.graphics.TextElement'], false);
goog.addDependency('graphics/svggraphics.js', ['goog.graphics.SvgGraphics'], ['goog.Timer', 'goog.dom', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.graphics.AbstractGraphics', 'goog.graphics.LinearGradient', 'goog.graphics.Path', 'goog.graphics.SolidFill', 'goog.graphics.Stroke', 'goog.graphics.SvgEllipseElement', 'goog.graphics.SvgGroupElement', 'goog.graphics.SvgImageElement', 'goog.graphics.SvgPathElement', 'goog.graphics.SvgRectElement', 'goog.graphics.SvgTextElement', 'goog.math', 'goog.math.Size', 'goog.style', 'goog.userAgent'], false);
goog.addDependency('graphics/textelement.js', ['goog.graphics.TextElement'], ['goog.graphics.StrokeAndFillElement'], false);
goog.addDependency('graphics/vmlelement.js', ['goog.graphics.VmlEllipseElement', 'goog.graphics.VmlGroupElement', 'goog.graphics.VmlImageElement', 'goog.graphics.VmlPathElement', 'goog.graphics.VmlRectElement', 'goog.graphics.VmlTextElement'], ['goog.dom', 'goog.graphics.EllipseElement', 'goog.graphics.GroupElement', 'goog.graphics.ImageElement', 'goog.graphics.PathElement', 'goog.graphics.RectElement', 'goog.graphics.TextElement'], false);
goog.addDependency('graphics/vmlgraphics.js', ['goog.graphics.VmlGraphics'], ['goog.array', 'goog.events', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.graphics.AbstractGraphics', 'goog.graphics.LinearGradient', 'goog.graphics.Path', 'goog.graphics.SolidFill', 'goog.graphics.VmlEllipseElement', 'goog.graphics.VmlGroupElement', 'goog.graphics.VmlImageElement', 'goog.graphics.VmlPathElement', 'goog.graphics.VmlRectElement', 'goog.graphics.VmlTextElement', 'goog.math', 'goog.math.Size', 'goog.string', 'goog.style'], false);
goog.addDependency('history/event.js', ['goog.history.Event'], ['goog.events.Event', 'goog.history.EventType'], false);
goog.addDependency('history/eventtype.js', ['goog.history.EventType'], [], false);
goog.addDependency('history/history.js', ['goog.History', 'goog.History.Event', 'goog.History.EventType'], ['goog.Timer', 'goog.dom', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.history.Event', 'goog.history.EventType', 'goog.labs.userAgent.device', 'goog.memoize', 'goog.string', 'goog.userAgent'], false);
goog.addDependency('history/history_test.js', ['goog.HistoryTest'], ['goog.History', 'goog.dispose', 'goog.dom', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('history/html5history.js', ['goog.history.Html5History', 'goog.history.Html5History.TokenTransformer'], ['goog.asserts', 'goog.events', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.history.Event', 'goog.history.EventType'], false);
goog.addDependency('history/html5history_test.js', ['goog.history.Html5HistoryTest'], ['goog.history.Html5History', 'goog.testing.MockControl', 'goog.testing.jsunit', 'goog.testing.mockmatchers'], false);
goog.addDependency('html/legacyconversions.js', ['goog.html.legacyconversions'], ['goog.html.SafeHtml', 'goog.html.SafeUrl', 'goog.html.TrustedResourceUrl'], false);
goog.addDependency('html/legacyconversions_test.js', ['goog.html.legacyconversionsTest'], ['goog.html.SafeHtml', 'goog.html.SafeUrl', 'goog.html.TrustedResourceUrl', 'goog.html.legacyconversions', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit'], false);
goog.addDependency('html/safehtml.js', ['goog.html.SafeHtml'], ['goog.array', 'goog.asserts', 'goog.dom.tags', 'goog.html.SafeStyle', 'goog.html.SafeUrl', 'goog.i18n.bidi.Dir', 'goog.i18n.bidi.DirectionalString', 'goog.object', 'goog.string', 'goog.string.Const', 'goog.string.TypedString'], false);
goog.addDependency('html/safehtml_test.js', ['goog.html.safeHtmlTest'], ['goog.html.SafeHtml', 'goog.html.SafeStyle', 'goog.html.SafeUrl', 'goog.html.testing', 'goog.i18n.bidi.Dir', 'goog.string.Const', 'goog.testing.jsunit'], false);
goog.addDependency('html/safescript.js', ['goog.html.SafeScript'], ['goog.asserts', 'goog.string.Const', 'goog.string.TypedString'], false);
goog.addDependency('html/safescript_test.js', ['goog.html.safeScriptTest'], ['goog.html.SafeScript', 'goog.string.Const', 'goog.testing.jsunit'], false);
goog.addDependency('html/safestyle.js', ['goog.html.SafeStyle'], ['goog.array', 'goog.asserts', 'goog.string', 'goog.string.Const', 'goog.string.TypedString'], false);
goog.addDependency('html/safestyle_test.js', ['goog.html.safeStyleTest'], ['goog.html.SafeStyle', 'goog.string.Const', 'goog.testing.jsunit'], false);
goog.addDependency('html/safestylesheet.js', ['goog.html.SafeStyleSheet'], ['goog.asserts', 'goog.string', 'goog.string.Const', 'goog.string.TypedString'], false);
goog.addDependency('html/safestylesheet_test.js', ['goog.html.safeStyleSheetTest'], ['goog.html.SafeStyleSheet', 'goog.string', 'goog.string.Const', 'goog.testing.jsunit'], false);
goog.addDependency('html/safeurl.js', ['goog.html.SafeUrl'], ['goog.asserts', 'goog.i18n.bidi.Dir', 'goog.i18n.bidi.DirectionalString', 'goog.string.Const', 'goog.string.TypedString'], false);
goog.addDependency('html/safeurl_test.js', ['goog.html.safeUrlTest'], ['goog.html.SafeUrl', 'goog.i18n.bidi.Dir', 'goog.string.Const', 'goog.testing.jsunit'], false);
goog.addDependency('html/testing.js', ['goog.html.testing'], ['goog.html.SafeHtml', 'goog.html.SafeScript', 'goog.html.SafeStyle', 'goog.html.SafeStyleSheet', 'goog.html.SafeUrl', 'goog.html.TrustedResourceUrl'], false);
goog.addDependency('html/trustedresourceurl.js', ['goog.html.TrustedResourceUrl'], ['goog.asserts', 'goog.i18n.bidi.Dir', 'goog.i18n.bidi.DirectionalString', 'goog.string.Const', 'goog.string.TypedString'], false);
goog.addDependency('html/trustedresourceurl_test.js', ['goog.html.trustedResourceUrlTest'], ['goog.html.TrustedResourceUrl', 'goog.i18n.bidi.Dir', 'goog.string.Const', 'goog.testing.jsunit'], false);
goog.addDependency('html/uncheckedconversions.js', ['goog.html.uncheckedconversions'], ['goog.asserts', 'goog.html.SafeHtml', 'goog.html.SafeScript', 'goog.html.SafeStyle', 'goog.html.SafeStyleSheet', 'goog.html.SafeUrl', 'goog.html.TrustedResourceUrl', 'goog.string', 'goog.string.Const'], false);
goog.addDependency('html/uncheckedconversions_test.js', ['goog.html.uncheckedconversionsTest'], ['goog.html.SafeHtml', 'goog.html.SafeScript', 'goog.html.SafeStyleSheet', 'goog.html.SafeUrl', 'goog.html.TrustedResourceUrl', 'goog.html.uncheckedconversions', 'goog.i18n.bidi.Dir', 'goog.string.Const', 'goog.testing.jsunit'], false);
goog.addDependency('html/utils.js', ['goog.html.utils'], ['goog.string'], false);
goog.addDependency('html/utils_test.js', ['goog.html.UtilsTest'], ['goog.array', 'goog.dom.TagName', 'goog.html.utils', 'goog.object', 'goog.testing.jsunit'], false);
goog.addDependency('i18n/bidi.js', ['goog.i18n.bidi', 'goog.i18n.bidi.Dir', 'goog.i18n.bidi.DirectionalString', 'goog.i18n.bidi.Format'], [], false);
goog.addDependency('i18n/bidi_test.js', ['goog.i18n.bidiTest'], ['goog.i18n.bidi', 'goog.i18n.bidi.Dir', 'goog.testing.jsunit'], false);
goog.addDependency('i18n/bidiformatter.js', ['goog.i18n.BidiFormatter'], ['goog.html.SafeHtml', 'goog.html.legacyconversions', 'goog.i18n.bidi', 'goog.i18n.bidi.Dir', 'goog.i18n.bidi.Format'], false);
goog.addDependency('i18n/bidiformatter_test.js', ['goog.i18n.BidiFormatterTest'], ['goog.html.SafeHtml', 'goog.i18n.BidiFormatter', 'goog.i18n.bidi.Dir', 'goog.i18n.bidi.Format', 'goog.testing.jsunit'], false);
goog.addDependency('i18n/charlistdecompressor.js', ['goog.i18n.CharListDecompressor'], ['goog.array', 'goog.i18n.uChar'], false);
goog.addDependency('i18n/charlistdecompressor_test.js', ['goog.i18n.CharListDecompressorTest'], ['goog.i18n.CharListDecompressor', 'goog.testing.jsunit'], false);
goog.addDependency('i18n/charpickerdata.js', ['goog.i18n.CharPickerData'], [], false);
goog.addDependency('i18n/collation.js', ['goog.i18n.collation'], [], false);
goog.addDependency('i18n/collation_test.js', ['goog.i18n.collationTest'], ['goog.i18n.collation', 'goog.testing.ExpectedFailures', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('i18n/compactnumberformatsymbols.js', ['goog.i18n.CompactNumberFormatSymbols', 'goog.i18n.CompactNumberFormatSymbols_af', 'goog.i18n.CompactNumberFormatSymbols_af_ZA', 'goog.i18n.CompactNumberFormatSymbols_am', 'goog.i18n.CompactNumberFormatSymbols_am_ET', 'goog.i18n.CompactNumberFormatSymbols_ar', 'goog.i18n.CompactNumberFormatSymbols_ar_001', 'goog.i18n.CompactNumberFormatSymbols_az', 'goog.i18n.CompactNumberFormatSymbols_az_Cyrl_AZ', 'goog.i18n.CompactNumberFormatSymbols_az_Latn_AZ', 'goog.i18n.CompactNumberFormatSymbols_bg', 'goog.i18n.CompactNumberFormatSymbols_bg_BG', 'goog.i18n.CompactNumberFormatSymbols_bn', 'goog.i18n.CompactNumberFormatSymbols_bn_BD', 'goog.i18n.CompactNumberFormatSymbols_br', 'goog.i18n.CompactNumberFormatSymbols_br_FR', 'goog.i18n.CompactNumberFormatSymbols_ca', 'goog.i18n.CompactNumberFormatSymbols_ca_AD', 'goog.i18n.CompactNumberFormatSymbols_ca_ES', 'goog.i18n.CompactNumberFormatSymbols_ca_ES_VALENCIA', 'goog.i18n.CompactNumberFormatSymbols_ca_FR', 'goog.i18n.CompactNumberFormatSymbols_ca_IT', 'goog.i18n.CompactNumberFormatSymbols_chr', 'goog.i18n.CompactNumberFormatSymbols_chr_US', 'goog.i18n.CompactNumberFormatSymbols_cs', 'goog.i18n.CompactNumberFormatSymbols_cs_CZ', 'goog.i18n.CompactNumberFormatSymbols_cy', 'goog.i18n.CompactNumberFormatSymbols_cy_GB', 'goog.i18n.CompactNumberFormatSymbols_da', 'goog.i18n.CompactNumberFormatSymbols_da_DK', 'goog.i18n.CompactNumberFormatSymbols_da_GL', 'goog.i18n.CompactNumberFormatSymbols_de', 'goog.i18n.CompactNumberFormatSymbols_de_AT', 'goog.i18n.CompactNumberFormatSymbols_de_BE', 'goog.i18n.CompactNumberFormatSymbols_de_CH', 'goog.i18n.CompactNumberFormatSymbols_de_DE', 'goog.i18n.CompactNumberFormatSymbols_de_LU', 'goog.i18n.CompactNumberFormatSymbols_el', 'goog.i18n.CompactNumberFormatSymbols_el_GR', 'goog.i18n.CompactNumberFormatSymbols_en', 'goog.i18n.CompactNumberFormatSymbols_en_001', 'goog.i18n.CompactNumberFormatSymbols_en_AS', 'goog.i18n.CompactNumberFormatSymbols_en_AU', 'goog.i18n.CompactNumberFormatSymbols_en_DG', 'goog.i18n.CompactNumberFormatSymbols_en_FM', 'goog.i18n.CompactNumberFormatSymbols_en_GB', 'goog.i18n.CompactNumberFormatSymbols_en_GU', 'goog.i18n.CompactNumberFormatSymbols_en_IE', 'goog.i18n.CompactNumberFormatSymbols_en_IN', 'goog.i18n.CompactNumberFormatSymbols_en_IO', 'goog.i18n.CompactNumberFormatSymbols_en_MH', 'goog.i18n.CompactNumberFormatSymbols_en_MP', 'goog.i18n.CompactNumberFormatSymbols_en_PR', 'goog.i18n.CompactNumberFormatSymbols_en_PW', 'goog.i18n.CompactNumberFormatSymbols_en_SG', 'goog.i18n.CompactNumberFormatSymbols_en_TC', 'goog.i18n.CompactNumberFormatSymbols_en_UM', 'goog.i18n.CompactNumberFormatSymbols_en_US', 'goog.i18n.CompactNumberFormatSymbols_en_VG', 'goog.i18n.CompactNumberFormatSymbols_en_VI', 'goog.i18n.CompactNumberFormatSymbols_en_ZA', 'goog.i18n.CompactNumberFormatSymbols_en_ZW', 'goog.i18n.CompactNumberFormatSymbols_es', 'goog.i18n.CompactNumberFormatSymbols_es_419', 'goog.i18n.CompactNumberFormatSymbols_es_EA', 'goog.i18n.CompactNumberFormatSymbols_es_ES', 'goog.i18n.CompactNumberFormatSymbols_es_IC', 'goog.i18n.CompactNumberFormatSymbols_et', 'goog.i18n.CompactNumberFormatSymbols_et_EE', 'goog.i18n.CompactNumberFormatSymbols_eu', 'goog.i18n.CompactNumberFormatSymbols_eu_ES', 'goog.i18n.CompactNumberFormatSymbols_fa', 'goog.i18n.CompactNumberFormatSymbols_fa_IR', 'goog.i18n.CompactNumberFormatSymbols_fi', 'goog.i18n.CompactNumberFormatSymbols_fi_FI', 'goog.i18n.CompactNumberFormatSymbols_fil', 'goog.i18n.CompactNumberFormatSymbols_fil_PH', 'goog.i18n.CompactNumberFormatSymbols_fr', 'goog.i18n.CompactNumberFormatSymbols_fr_BL', 'goog.i18n.CompactNumberFormatSymbols_fr_CA', 'goog.i18n.CompactNumberFormatSymbols_fr_FR', 'goog.i18n.CompactNumberFormatSymbols_fr_GF', 'goog.i18n.CompactNumberFormatSymbols_fr_GP', 'goog.i18n.CompactNumberFormatSymbols_fr_MC', 'goog.i18n.CompactNumberFormatSymbols_fr_MF', 'goog.i18n.CompactNumberFormatSymbols_fr_MQ', 'goog.i18n.CompactNumberFormatSymbols_fr_PM', 'goog.i18n.CompactNumberFormatSymbols_fr_RE', 'goog.i18n.CompactNumberFormatSymbols_fr_YT', 'goog.i18n.CompactNumberFormatSymbols_ga', 'goog.i18n.CompactNumberFormatSymbols_ga_IE', 'goog.i18n.CompactNumberFormatSymbols_gl', 'goog.i18n.CompactNumberFormatSymbols_gl_ES', 'goog.i18n.CompactNumberFormatSymbols_gsw', 'goog.i18n.CompactNumberFormatSymbols_gsw_CH', 'goog.i18n.CompactNumberFormatSymbols_gsw_LI', 'goog.i18n.CompactNumberFormatSymbols_gu', 'goog.i18n.CompactNumberFormatSymbols_gu_IN', 'goog.i18n.CompactNumberFormatSymbols_haw', 'goog.i18n.CompactNumberFormatSymbols_haw_US', 'goog.i18n.CompactNumberFormatSymbols_he', 'goog.i18n.CompactNumberFormatSymbols_he_IL', 'goog.i18n.CompactNumberFormatSymbols_hi', 'goog.i18n.CompactNumberFormatSymbols_hi_IN', 'goog.i18n.CompactNumberFormatSymbols_hr', 'goog.i18n.CompactNumberFormatSymbols_hr_HR', 'goog.i18n.CompactNumberFormatSymbols_hu', 'goog.i18n.CompactNumberFormatSymbols_hu_HU', 'goog.i18n.CompactNumberFormatSymbols_hy', 'goog.i18n.CompactNumberFormatSymbols_hy_AM', 'goog.i18n.CompactNumberFormatSymbols_id', 'goog.i18n.CompactNumberFormatSymbols_id_ID', 'goog.i18n.CompactNumberFormatSymbols_in', 'goog.i18n.CompactNumberFormatSymbols_is', 'goog.i18n.CompactNumberFormatSymbols_is_IS', 'goog.i18n.CompactNumberFormatSymbols_it', 'goog.i18n.CompactNumberFormatSymbols_it_IT', 'goog.i18n.CompactNumberFormatSymbols_it_SM', 'goog.i18n.CompactNumberFormatSymbols_iw', 'goog.i18n.CompactNumberFormatSymbols_ja', 'goog.i18n.CompactNumberFormatSymbols_ja_JP', 'goog.i18n.CompactNumberFormatSymbols_ka', 'goog.i18n.CompactNumberFormatSymbols_ka_GE', 'goog.i18n.CompactNumberFormatSymbols_kk', 'goog.i18n.CompactNumberFormatSymbols_kk_Cyrl_KZ', 'goog.i18n.CompactNumberFormatSymbols_km', 'goog.i18n.CompactNumberFormatSymbols_km_KH', 'goog.i18n.CompactNumberFormatSymbols_kn', 'goog.i18n.CompactNumberFormatSymbols_kn_IN', 'goog.i18n.CompactNumberFormatSymbols_ko', 'goog.i18n.CompactNumberFormatSymbols_ko_KR', 'goog.i18n.CompactNumberFormatSymbols_ky', 'goog.i18n.CompactNumberFormatSymbols_ky_Cyrl_KG', 'goog.i18n.CompactNumberFormatSymbols_ln', 'goog.i18n.CompactNumberFormatSymbols_ln_CD', 'goog.i18n.CompactNumberFormatSymbols_lo', 'goog.i18n.CompactNumberFormatSymbols_lo_LA', 'goog.i18n.CompactNumberFormatSymbols_lt', 'goog.i18n.CompactNumberFormatSymbols_lt_LT', 'goog.i18n.CompactNumberFormatSymbols_lv', 'goog.i18n.CompactNumberFormatSymbols_lv_LV', 'goog.i18n.CompactNumberFormatSymbols_mk', 'goog.i18n.CompactNumberFormatSymbols_mk_MK', 'goog.i18n.CompactNumberFormatSymbols_ml', 'goog.i18n.CompactNumberFormatSymbols_ml_IN', 'goog.i18n.CompactNumberFormatSymbols_mn', 'goog.i18n.CompactNumberFormatSymbols_mn_Cyrl_MN', 'goog.i18n.CompactNumberFormatSymbols_mr', 'goog.i18n.CompactNumberFormatSymbols_mr_IN', 'goog.i18n.CompactNumberFormatSymbols_ms', 'goog.i18n.CompactNumberFormatSymbols_ms_Latn_MY', 'goog.i18n.CompactNumberFormatSymbols_mt', 'goog.i18n.CompactNumberFormatSymbols_mt_MT', 'goog.i18n.CompactNumberFormatSymbols_my', 'goog.i18n.CompactNumberFormatSymbols_my_MM', 'goog.i18n.CompactNumberFormatSymbols_nb', 'goog.i18n.CompactNumberFormatSymbols_nb_NO', 'goog.i18n.CompactNumberFormatSymbols_nb_SJ', 'goog.i18n.CompactNumberFormatSymbols_ne', 'goog.i18n.CompactNumberFormatSymbols_ne_NP', 'goog.i18n.CompactNumberFormatSymbols_nl', 'goog.i18n.CompactNumberFormatSymbols_nl_NL', 'goog.i18n.CompactNumberFormatSymbols_no', 'goog.i18n.CompactNumberFormatSymbols_no_NO', 'goog.i18n.CompactNumberFormatSymbols_or', 'goog.i18n.CompactNumberFormatSymbols_or_IN', 'goog.i18n.CompactNumberFormatSymbols_pa', 'goog.i18n.CompactNumberFormatSymbols_pa_Guru_IN', 'goog.i18n.CompactNumberFormatSymbols_pl', 'goog.i18n.CompactNumberFormatSymbols_pl_PL', 'goog.i18n.CompactNumberFormatSymbols_pt', 'goog.i18n.CompactNumberFormatSymbols_pt_BR', 'goog.i18n.CompactNumberFormatSymbols_pt_PT', 'goog.i18n.CompactNumberFormatSymbols_ro', 'goog.i18n.CompactNumberFormatSymbols_ro_RO', 'goog.i18n.CompactNumberFormatSymbols_ru', 'goog.i18n.CompactNumberFormatSymbols_ru_RU', 'goog.i18n.CompactNumberFormatSymbols_si', 'goog.i18n.CompactNumberFormatSymbols_si_LK', 'goog.i18n.CompactNumberFormatSymbols_sk', 'goog.i18n.CompactNumberFormatSymbols_sk_SK', 'goog.i18n.CompactNumberFormatSymbols_sl', 'goog.i18n.CompactNumberFormatSymbols_sl_SI', 'goog.i18n.CompactNumberFormatSymbols_sq', 'goog.i18n.CompactNumberFormatSymbols_sq_AL', 'goog.i18n.CompactNumberFormatSymbols_sr', 'goog.i18n.CompactNumberFormatSymbols_sr_Cyrl_RS', 'goog.i18n.CompactNumberFormatSymbols_sv', 'goog.i18n.CompactNumberFormatSymbols_sv_SE', 'goog.i18n.CompactNumberFormatSymbols_sw', 'goog.i18n.CompactNumberFormatSymbols_sw_TZ', 'goog.i18n.CompactNumberFormatSymbols_ta', 'goog.i18n.CompactNumberFormatSymbols_ta_IN', 'goog.i18n.CompactNumberFormatSymbols_te', 'goog.i18n.CompactNumberFormatSymbols_te_IN', 'goog.i18n.CompactNumberFormatSymbols_th', 'goog.i18n.CompactNumberFormatSymbols_th_TH', 'goog.i18n.CompactNumberFormatSymbols_tl', 'goog.i18n.CompactNumberFormatSymbols_tr', 'goog.i18n.CompactNumberFormatSymbols_tr_TR', 'goog.i18n.CompactNumberFormatSymbols_uk', 'goog.i18n.CompactNumberFormatSymbols_uk_UA', 'goog.i18n.CompactNumberFormatSymbols_ur', 'goog.i18n.CompactNumberFormatSymbols_ur_PK', 'goog.i18n.CompactNumberFormatSymbols_uz', 'goog.i18n.CompactNumberFormatSymbols_uz_Latn_UZ', 'goog.i18n.CompactNumberFormatSymbols_vi', 'goog.i18n.CompactNumberFormatSymbols_vi_VN', 'goog.i18n.CompactNumberFormatSymbols_zh', 'goog.i18n.CompactNumberFormatSymbols_zh_CN', 'goog.i18n.CompactNumberFormatSymbols_zh_HK', 'goog.i18n.CompactNumberFormatSymbols_zh_Hans_CN', 'goog.i18n.CompactNumberFormatSymbols_zh_TW', 'goog.i18n.CompactNumberFormatSymbols_zu', 'goog.i18n.CompactNumberFormatSymbols_zu_ZA'], [], false);
goog.addDependency('i18n/compactnumberformatsymbols_ext.js', ['goog.i18n.CompactNumberFormatSymbolsExt', 'goog.i18n.CompactNumberFormatSymbols_aa', 'goog.i18n.CompactNumberFormatSymbols_aa_DJ', 'goog.i18n.CompactNumberFormatSymbols_aa_ER', 'goog.i18n.CompactNumberFormatSymbols_aa_ET', 'goog.i18n.CompactNumberFormatSymbols_af_NA', 'goog.i18n.CompactNumberFormatSymbols_agq', 'goog.i18n.CompactNumberFormatSymbols_agq_CM', 'goog.i18n.CompactNumberFormatSymbols_ak', 'goog.i18n.CompactNumberFormatSymbols_ak_GH', 'goog.i18n.CompactNumberFormatSymbols_ar_AE', 'goog.i18n.CompactNumberFormatSymbols_ar_BH', 'goog.i18n.CompactNumberFormatSymbols_ar_DJ', 'goog.i18n.CompactNumberFormatSymbols_ar_DZ', 'goog.i18n.CompactNumberFormatSymbols_ar_EG', 'goog.i18n.CompactNumberFormatSymbols_ar_EH', 'goog.i18n.CompactNumberFormatSymbols_ar_ER', 'goog.i18n.CompactNumberFormatSymbols_ar_IL', 'goog.i18n.CompactNumberFormatSymbols_ar_IQ', 'goog.i18n.CompactNumberFormatSymbols_ar_JO', 'goog.i18n.CompactNumberFormatSymbols_ar_KM', 'goog.i18n.CompactNumberFormatSymbols_ar_KW', 'goog.i18n.CompactNumberFormatSymbols_ar_LB', 'goog.i18n.CompactNumberFormatSymbols_ar_LY', 'goog.i18n.CompactNumberFormatSymbols_ar_MA', 'goog.i18n.CompactNumberFormatSymbols_ar_MR', 'goog.i18n.CompactNumberFormatSymbols_ar_OM', 'goog.i18n.CompactNumberFormatSymbols_ar_PS', 'goog.i18n.CompactNumberFormatSymbols_ar_QA', 'goog.i18n.CompactNumberFormatSymbols_ar_SA', 'goog.i18n.CompactNumberFormatSymbols_ar_SD', 'goog.i18n.CompactNumberFormatSymbols_ar_SO', 'goog.i18n.CompactNumberFormatSymbols_ar_SS', 'goog.i18n.CompactNumberFormatSymbols_ar_SY', 'goog.i18n.CompactNumberFormatSymbols_ar_TD', 'goog.i18n.CompactNumberFormatSymbols_ar_TN', 'goog.i18n.CompactNumberFormatSymbols_ar_YE', 'goog.i18n.CompactNumberFormatSymbols_as', 'goog.i18n.CompactNumberFormatSymbols_as_IN', 'goog.i18n.CompactNumberFormatSymbols_asa', 'goog.i18n.CompactNumberFormatSymbols_asa_TZ', 'goog.i18n.CompactNumberFormatSymbols_ast', 'goog.i18n.CompactNumberFormatSymbols_ast_ES', 'goog.i18n.CompactNumberFormatSymbols_az_Cyrl', 'goog.i18n.CompactNumberFormatSymbols_az_Latn', 'goog.i18n.CompactNumberFormatSymbols_bas', 'goog.i18n.CompactNumberFormatSymbols_bas_CM', 'goog.i18n.CompactNumberFormatSymbols_be', 'goog.i18n.CompactNumberFormatSymbols_be_BY', 'goog.i18n.CompactNumberFormatSymbols_bem', 'goog.i18n.CompactNumberFormatSymbols_bem_ZM', 'goog.i18n.CompactNumberFormatSymbols_bez', 'goog.i18n.CompactNumberFormatSymbols_bez_TZ', 'goog.i18n.CompactNumberFormatSymbols_bm', 'goog.i18n.CompactNumberFormatSymbols_bm_ML', 'goog.i18n.CompactNumberFormatSymbols_bn_IN', 'goog.i18n.CompactNumberFormatSymbols_bo', 'goog.i18n.CompactNumberFormatSymbols_bo_CN', 'goog.i18n.CompactNumberFormatSymbols_bo_IN', 'goog.i18n.CompactNumberFormatSymbols_brx', 'goog.i18n.CompactNumberFormatSymbols_brx_IN', 'goog.i18n.CompactNumberFormatSymbols_bs', 'goog.i18n.CompactNumberFormatSymbols_bs_Cyrl', 'goog.i18n.CompactNumberFormatSymbols_bs_Cyrl_BA', 'goog.i18n.CompactNumberFormatSymbols_bs_Latn', 'goog.i18n.CompactNumberFormatSymbols_bs_Latn_BA', 'goog.i18n.CompactNumberFormatSymbols_byn', 'goog.i18n.CompactNumberFormatSymbols_byn_ER', 'goog.i18n.CompactNumberFormatSymbols_cgg', 'goog.i18n.CompactNumberFormatSymbols_cgg_UG', 'goog.i18n.CompactNumberFormatSymbols_ckb', 'goog.i18n.CompactNumberFormatSymbols_ckb_Arab', 'goog.i18n.CompactNumberFormatSymbols_ckb_Arab_IQ', 'goog.i18n.CompactNumberFormatSymbols_ckb_Arab_IR', 'goog.i18n.CompactNumberFormatSymbols_ckb_IQ', 'goog.i18n.CompactNumberFormatSymbols_ckb_IR', 'goog.i18n.CompactNumberFormatSymbols_ckb_Latn', 'goog.i18n.CompactNumberFormatSymbols_ckb_Latn_IQ', 'goog.i18n.CompactNumberFormatSymbols_dav', 'goog.i18n.CompactNumberFormatSymbols_dav_KE', 'goog.i18n.CompactNumberFormatSymbols_de_LI', 'goog.i18n.CompactNumberFormatSymbols_dje', 'goog.i18n.CompactNumberFormatSymbols_dje_NE', 'goog.i18n.CompactNumberFormatSymbols_dua', 'goog.i18n.CompactNumberFormatSymbols_dua_CM', 'goog.i18n.CompactNumberFormatSymbols_dyo', 'goog.i18n.CompactNumberFormatSymbols_dyo_SN', 'goog.i18n.CompactNumberFormatSymbols_dz', 'goog.i18n.CompactNumberFormatSymbols_dz_BT', 'goog.i18n.CompactNumberFormatSymbols_ebu', 'goog.i18n.CompactNumberFormatSymbols_ebu_KE', 'goog.i18n.CompactNumberFormatSymbols_ee', 'goog.i18n.CompactNumberFormatSymbols_ee_GH', 'goog.i18n.CompactNumberFormatSymbols_ee_TG', 'goog.i18n.CompactNumberFormatSymbols_el_CY', 'goog.i18n.CompactNumberFormatSymbols_en_150', 'goog.i18n.CompactNumberFormatSymbols_en_AG', 'goog.i18n.CompactNumberFormatSymbols_en_AI', 'goog.i18n.CompactNumberFormatSymbols_en_BB', 'goog.i18n.CompactNumberFormatSymbols_en_BE', 'goog.i18n.CompactNumberFormatSymbols_en_BM', 'goog.i18n.CompactNumberFormatSymbols_en_BS', 'goog.i18n.CompactNumberFormatSymbols_en_BW', 'goog.i18n.CompactNumberFormatSymbols_en_BZ', 'goog.i18n.CompactNumberFormatSymbols_en_CA', 'goog.i18n.CompactNumberFormatSymbols_en_CC', 'goog.i18n.CompactNumberFormatSymbols_en_CK', 'goog.i18n.CompactNumberFormatSymbols_en_CM', 'goog.i18n.CompactNumberFormatSymbols_en_CX', 'goog.i18n.CompactNumberFormatSymbols_en_DM', 'goog.i18n.CompactNumberFormatSymbols_en_ER', 'goog.i18n.CompactNumberFormatSymbols_en_FJ', 'goog.i18n.CompactNumberFormatSymbols_en_FK', 'goog.i18n.CompactNumberFormatSymbols_en_GD', 'goog.i18n.CompactNumberFormatSymbols_en_GG', 'goog.i18n.CompactNumberFormatSymbols_en_GH', 'goog.i18n.CompactNumberFormatSymbols_en_GI', 'goog.i18n.CompactNumberFormatSymbols_en_GM', 'goog.i18n.CompactNumberFormatSymbols_en_GY', 'goog.i18n.CompactNumberFormatSymbols_en_HK', 'goog.i18n.CompactNumberFormatSymbols_en_IM', 'goog.i18n.CompactNumberFormatSymbols_en_JE', 'goog.i18n.CompactNumberFormatSymbols_en_JM', 'goog.i18n.CompactNumberFormatSymbols_en_KE', 'goog.i18n.CompactNumberFormatSymbols_en_KI', 'goog.i18n.CompactNumberFormatSymbols_en_KN', 'goog.i18n.CompactNumberFormatSymbols_en_KY', 'goog.i18n.CompactNumberFormatSymbols_en_LC', 'goog.i18n.CompactNumberFormatSymbols_en_LR', 'goog.i18n.CompactNumberFormatSymbols_en_LS', 'goog.i18n.CompactNumberFormatSymbols_en_MG', 'goog.i18n.CompactNumberFormatSymbols_en_MO', 'goog.i18n.CompactNumberFormatSymbols_en_MS', 'goog.i18n.CompactNumberFormatSymbols_en_MT', 'goog.i18n.CompactNumberFormatSymbols_en_MU', 'goog.i18n.CompactNumberFormatSymbols_en_MW', 'goog.i18n.CompactNumberFormatSymbols_en_NA', 'goog.i18n.CompactNumberFormatSymbols_en_NF', 'goog.i18n.CompactNumberFormatSymbols_en_NG', 'goog.i18n.CompactNumberFormatSymbols_en_NR', 'goog.i18n.CompactNumberFormatSymbols_en_NU', 'goog.i18n.CompactNumberFormatSymbols_en_NZ', 'goog.i18n.CompactNumberFormatSymbols_en_PG', 'goog.i18n.CompactNumberFormatSymbols_en_PH', 'goog.i18n.CompactNumberFormatSymbols_en_PK', 'goog.i18n.CompactNumberFormatSymbols_en_PN', 'goog.i18n.CompactNumberFormatSymbols_en_RW', 'goog.i18n.CompactNumberFormatSymbols_en_SB', 'goog.i18n.CompactNumberFormatSymbols_en_SC', 'goog.i18n.CompactNumberFormatSymbols_en_SD', 'goog.i18n.CompactNumberFormatSymbols_en_SH', 'goog.i18n.CompactNumberFormatSymbols_en_SL', 'goog.i18n.CompactNumberFormatSymbols_en_SS', 'goog.i18n.CompactNumberFormatSymbols_en_SX', 'goog.i18n.CompactNumberFormatSymbols_en_SZ', 'goog.i18n.CompactNumberFormatSymbols_en_TK', 'goog.i18n.CompactNumberFormatSymbols_en_TO', 'goog.i18n.CompactNumberFormatSymbols_en_TT', 'goog.i18n.CompactNumberFormatSymbols_en_TV', 'goog.i18n.CompactNumberFormatSymbols_en_TZ', 'goog.i18n.CompactNumberFormatSymbols_en_UG', 'goog.i18n.CompactNumberFormatSymbols_en_VC', 'goog.i18n.CompactNumberFormatSymbols_en_VU', 'goog.i18n.CompactNumberFormatSymbols_en_WS', 'goog.i18n.CompactNumberFormatSymbols_en_ZM', 'goog.i18n.CompactNumberFormatSymbols_eo', 'goog.i18n.CompactNumberFormatSymbols_eo_001', 'goog.i18n.CompactNumberFormatSymbols_es_AR', 'goog.i18n.CompactNumberFormatSymbols_es_BO', 'goog.i18n.CompactNumberFormatSymbols_es_CL', 'goog.i18n.CompactNumberFormatSymbols_es_CO', 'goog.i18n.CompactNumberFormatSymbols_es_CR', 'goog.i18n.CompactNumberFormatSymbols_es_CU', 'goog.i18n.CompactNumberFormatSymbols_es_DO', 'goog.i18n.CompactNumberFormatSymbols_es_EC', 'goog.i18n.CompactNumberFormatSymbols_es_GQ', 'goog.i18n.CompactNumberFormatSymbols_es_GT', 'goog.i18n.CompactNumberFormatSymbols_es_HN', 'goog.i18n.CompactNumberFormatSymbols_es_MX', 'goog.i18n.CompactNumberFormatSymbols_es_NI', 'goog.i18n.CompactNumberFormatSymbols_es_PA', 'goog.i18n.CompactNumberFormatSymbols_es_PE', 'goog.i18n.CompactNumberFormatSymbols_es_PH', 'goog.i18n.CompactNumberFormatSymbols_es_PR', 'goog.i18n.CompactNumberFormatSymbols_es_PY', 'goog.i18n.CompactNumberFormatSymbols_es_SV', 'goog.i18n.CompactNumberFormatSymbols_es_US', 'goog.i18n.CompactNumberFormatSymbols_es_UY', 'goog.i18n.CompactNumberFormatSymbols_es_VE', 'goog.i18n.CompactNumberFormatSymbols_ewo', 'goog.i18n.CompactNumberFormatSymbols_ewo_CM', 'goog.i18n.CompactNumberFormatSymbols_fa_AF', 'goog.i18n.CompactNumberFormatSymbols_ff', 'goog.i18n.CompactNumberFormatSymbols_ff_CM', 'goog.i18n.CompactNumberFormatSymbols_ff_GN', 'goog.i18n.CompactNumberFormatSymbols_ff_MR', 'goog.i18n.CompactNumberFormatSymbols_ff_SN', 'goog.i18n.CompactNumberFormatSymbols_fo', 'goog.i18n.CompactNumberFormatSymbols_fo_FO', 'goog.i18n.CompactNumberFormatSymbols_fr_BE', 'goog.i18n.CompactNumberFormatSymbols_fr_BF', 'goog.i18n.CompactNumberFormatSymbols_fr_BI', 'goog.i18n.CompactNumberFormatSymbols_fr_BJ', 'goog.i18n.CompactNumberFormatSymbols_fr_CD', 'goog.i18n.CompactNumberFormatSymbols_fr_CF', 'goog.i18n.CompactNumberFormatSymbols_fr_CG', 'goog.i18n.CompactNumberFormatSymbols_fr_CH', 'goog.i18n.CompactNumberFormatSymbols_fr_CI', 'goog.i18n.CompactNumberFormatSymbols_fr_CM', 'goog.i18n.CompactNumberFormatSymbols_fr_DJ', 'goog.i18n.CompactNumberFormatSymbols_fr_DZ', 'goog.i18n.CompactNumberFormatSymbols_fr_GA', 'goog.i18n.CompactNumberFormatSymbols_fr_GN', 'goog.i18n.CompactNumberFormatSymbols_fr_GQ', 'goog.i18n.CompactNumberFormatSymbols_fr_HT', 'goog.i18n.CompactNumberFormatSymbols_fr_KM', 'goog.i18n.CompactNumberFormatSymbols_fr_LU', 'goog.i18n.CompactNumberFormatSymbols_fr_MA', 'goog.i18n.CompactNumberFormatSymbols_fr_MG', 'goog.i18n.CompactNumberFormatSymbols_fr_ML', 'goog.i18n.CompactNumberFormatSymbols_fr_MR', 'goog.i18n.CompactNumberFormatSymbols_fr_MU', 'goog.i18n.CompactNumberFormatSymbols_fr_NC', 'goog.i18n.CompactNumberFormatSymbols_fr_NE', 'goog.i18n.CompactNumberFormatSymbols_fr_PF', 'goog.i18n.CompactNumberFormatSymbols_fr_RW', 'goog.i18n.CompactNumberFormatSymbols_fr_SC', 'goog.i18n.CompactNumberFormatSymbols_fr_SN', 'goog.i18n.CompactNumberFormatSymbols_fr_SY', 'goog.i18n.CompactNumberFormatSymbols_fr_TD', 'goog.i18n.CompactNumberFormatSymbols_fr_TG', 'goog.i18n.CompactNumberFormatSymbols_fr_TN', 'goog.i18n.CompactNumberFormatSymbols_fr_VU', 'goog.i18n.CompactNumberFormatSymbols_fr_WF', 'goog.i18n.CompactNumberFormatSymbols_fur', 'goog.i18n.CompactNumberFormatSymbols_fur_IT', 'goog.i18n.CompactNumberFormatSymbols_fy', 'goog.i18n.CompactNumberFormatSymbols_fy_NL', 'goog.i18n.CompactNumberFormatSymbols_gd', 'goog.i18n.CompactNumberFormatSymbols_gd_GB', 'goog.i18n.CompactNumberFormatSymbols_guz', 'goog.i18n.CompactNumberFormatSymbols_guz_KE', 'goog.i18n.CompactNumberFormatSymbols_gv', 'goog.i18n.CompactNumberFormatSymbols_gv_IM', 'goog.i18n.CompactNumberFormatSymbols_ha', 'goog.i18n.CompactNumberFormatSymbols_ha_Latn', 'goog.i18n.CompactNumberFormatSymbols_ha_Latn_GH', 'goog.i18n.CompactNumberFormatSymbols_ha_Latn_NE', 'goog.i18n.CompactNumberFormatSymbols_ha_Latn_NG', 'goog.i18n.CompactNumberFormatSymbols_hr_BA', 'goog.i18n.CompactNumberFormatSymbols_ia', 'goog.i18n.CompactNumberFormatSymbols_ia_FR', 'goog.i18n.CompactNumberFormatSymbols_ig', 'goog.i18n.CompactNumberFormatSymbols_ig_NG', 'goog.i18n.CompactNumberFormatSymbols_ii', 'goog.i18n.CompactNumberFormatSymbols_ii_CN', 'goog.i18n.CompactNumberFormatSymbols_it_CH', 'goog.i18n.CompactNumberFormatSymbols_jgo', 'goog.i18n.CompactNumberFormatSymbols_jgo_CM', 'goog.i18n.CompactNumberFormatSymbols_jmc', 'goog.i18n.CompactNumberFormatSymbols_jmc_TZ', 'goog.i18n.CompactNumberFormatSymbols_kab', 'goog.i18n.CompactNumberFormatSymbols_kab_DZ', 'goog.i18n.CompactNumberFormatSymbols_kam', 'goog.i18n.CompactNumberFormatSymbols_kam_KE', 'goog.i18n.CompactNumberFormatSymbols_kde', 'goog.i18n.CompactNumberFormatSymbols_kde_TZ', 'goog.i18n.CompactNumberFormatSymbols_kea', 'goog.i18n.CompactNumberFormatSymbols_kea_CV', 'goog.i18n.CompactNumberFormatSymbols_khq', 'goog.i18n.CompactNumberFormatSymbols_khq_ML', 'goog.i18n.CompactNumberFormatSymbols_ki', 'goog.i18n.CompactNumberFormatSymbols_ki_KE', 'goog.i18n.CompactNumberFormatSymbols_kk_Cyrl', 'goog.i18n.CompactNumberFormatSymbols_kkj', 'goog.i18n.CompactNumberFormatSymbols_kkj_CM', 'goog.i18n.CompactNumberFormatSymbols_kl', 'goog.i18n.CompactNumberFormatSymbols_kl_GL', 'goog.i18n.CompactNumberFormatSymbols_kln', 'goog.i18n.CompactNumberFormatSymbols_kln_KE', 'goog.i18n.CompactNumberFormatSymbols_ko_KP', 'goog.i18n.CompactNumberFormatSymbols_kok', 'goog.i18n.CompactNumberFormatSymbols_kok_IN', 'goog.i18n.CompactNumberFormatSymbols_ks', 'goog.i18n.CompactNumberFormatSymbols_ks_Arab', 'goog.i18n.CompactNumberFormatSymbols_ks_Arab_IN', 'goog.i18n.CompactNumberFormatSymbols_ksb', 'goog.i18n.CompactNumberFormatSymbols_ksb_TZ', 'goog.i18n.CompactNumberFormatSymbols_ksf', 'goog.i18n.CompactNumberFormatSymbols_ksf_CM', 'goog.i18n.CompactNumberFormatSymbols_ksh', 'goog.i18n.CompactNumberFormatSymbols_ksh_DE', 'goog.i18n.CompactNumberFormatSymbols_kw', 'goog.i18n.CompactNumberFormatSymbols_kw_GB', 'goog.i18n.CompactNumberFormatSymbols_ky_Cyrl', 'goog.i18n.CompactNumberFormatSymbols_lag', 'goog.i18n.CompactNumberFormatSymbols_lag_TZ', 'goog.i18n.CompactNumberFormatSymbols_lg', 'goog.i18n.CompactNumberFormatSymbols_lg_UG', 'goog.i18n.CompactNumberFormatSymbols_lkt', 'goog.i18n.CompactNumberFormatSymbols_lkt_US', 'goog.i18n.CompactNumberFormatSymbols_ln_AO', 'goog.i18n.CompactNumberFormatSymbols_ln_CF', 'goog.i18n.CompactNumberFormatSymbols_ln_CG', 'goog.i18n.CompactNumberFormatSymbols_lu', 'goog.i18n.CompactNumberFormatSymbols_lu_CD', 'goog.i18n.CompactNumberFormatSymbols_luo', 'goog.i18n.CompactNumberFormatSymbols_luo_KE', 'goog.i18n.CompactNumberFormatSymbols_luy', 'goog.i18n.CompactNumberFormatSymbols_luy_KE', 'goog.i18n.CompactNumberFormatSymbols_mas', 'goog.i18n.CompactNumberFormatSymbols_mas_KE', 'goog.i18n.CompactNumberFormatSymbols_mas_TZ', 'goog.i18n.CompactNumberFormatSymbols_mer', 'goog.i18n.CompactNumberFormatSymbols_mer_KE', 'goog.i18n.CompactNumberFormatSymbols_mfe', 'goog.i18n.CompactNumberFormatSymbols_mfe_MU', 'goog.i18n.CompactNumberFormatSymbols_mg', 'goog.i18n.CompactNumberFormatSymbols_mg_MG', 'goog.i18n.CompactNumberFormatSymbols_mgh', 'goog.i18n.CompactNumberFormatSymbols_mgh_MZ', 'goog.i18n.CompactNumberFormatSymbols_mgo', 'goog.i18n.CompactNumberFormatSymbols_mgo_CM', 'goog.i18n.CompactNumberFormatSymbols_mn_Cyrl', 'goog.i18n.CompactNumberFormatSymbols_ms_Latn', 'goog.i18n.CompactNumberFormatSymbols_ms_Latn_BN', 'goog.i18n.CompactNumberFormatSymbols_ms_Latn_SG', 'goog.i18n.CompactNumberFormatSymbols_mua', 'goog.i18n.CompactNumberFormatSymbols_mua_CM', 'goog.i18n.CompactNumberFormatSymbols_naq', 'goog.i18n.CompactNumberFormatSymbols_naq_NA', 'goog.i18n.CompactNumberFormatSymbols_nd', 'goog.i18n.CompactNumberFormatSymbols_nd_ZW', 'goog.i18n.CompactNumberFormatSymbols_ne_IN', 'goog.i18n.CompactNumberFormatSymbols_nl_AW', 'goog.i18n.CompactNumberFormatSymbols_nl_BE', 'goog.i18n.CompactNumberFormatSymbols_nl_BQ', 'goog.i18n.CompactNumberFormatSymbols_nl_CW', 'goog.i18n.CompactNumberFormatSymbols_nl_SR', 'goog.i18n.CompactNumberFormatSymbols_nl_SX', 'goog.i18n.CompactNumberFormatSymbols_nmg', 'goog.i18n.CompactNumberFormatSymbols_nmg_CM', 'goog.i18n.CompactNumberFormatSymbols_nn', 'goog.i18n.CompactNumberFormatSymbols_nn_NO', 'goog.i18n.CompactNumberFormatSymbols_nnh', 'goog.i18n.CompactNumberFormatSymbols_nnh_CM', 'goog.i18n.CompactNumberFormatSymbols_nr', 'goog.i18n.CompactNumberFormatSymbols_nr_ZA', 'goog.i18n.CompactNumberFormatSymbols_nso', 'goog.i18n.CompactNumberFormatSymbols_nso_ZA', 'goog.i18n.CompactNumberFormatSymbols_nus', 'goog.i18n.CompactNumberFormatSymbols_nus_SD', 'goog.i18n.CompactNumberFormatSymbols_nyn', 'goog.i18n.CompactNumberFormatSymbols_nyn_UG', 'goog.i18n.CompactNumberFormatSymbols_om', 'goog.i18n.CompactNumberFormatSymbols_om_ET', 'goog.i18n.CompactNumberFormatSymbols_om_KE', 'goog.i18n.CompactNumberFormatSymbols_os', 'goog.i18n.CompactNumberFormatSymbols_os_GE', 'goog.i18n.CompactNumberFormatSymbols_os_RU', 'goog.i18n.CompactNumberFormatSymbols_pa_Arab', 'goog.i18n.CompactNumberFormatSymbols_pa_Arab_PK', 'goog.i18n.CompactNumberFormatSymbols_pa_Guru', 'goog.i18n.CompactNumberFormatSymbols_ps', 'goog.i18n.CompactNumberFormatSymbols_ps_AF', 'goog.i18n.CompactNumberFormatSymbols_pt_AO', 'goog.i18n.CompactNumberFormatSymbols_pt_CV', 'goog.i18n.CompactNumberFormatSymbols_pt_GW', 'goog.i18n.CompactNumberFormatSymbols_pt_MO', 'goog.i18n.CompactNumberFormatSymbols_pt_MZ', 'goog.i18n.CompactNumberFormatSymbols_pt_ST', 'goog.i18n.CompactNumberFormatSymbols_pt_TL', 'goog.i18n.CompactNumberFormatSymbols_rm', 'goog.i18n.CompactNumberFormatSymbols_rm_CH', 'goog.i18n.CompactNumberFormatSymbols_rn', 'goog.i18n.CompactNumberFormatSymbols_rn_BI', 'goog.i18n.CompactNumberFormatSymbols_ro_MD', 'goog.i18n.CompactNumberFormatSymbols_rof', 'goog.i18n.CompactNumberFormatSymbols_rof_TZ', 'goog.i18n.CompactNumberFormatSymbols_ru_BY', 'goog.i18n.CompactNumberFormatSymbols_ru_KG', 'goog.i18n.CompactNumberFormatSymbols_ru_KZ', 'goog.i18n.CompactNumberFormatSymbols_ru_MD', 'goog.i18n.CompactNumberFormatSymbols_ru_UA', 'goog.i18n.CompactNumberFormatSymbols_rw', 'goog.i18n.CompactNumberFormatSymbols_rw_RW', 'goog.i18n.CompactNumberFormatSymbols_rwk', 'goog.i18n.CompactNumberFormatSymbols_rwk_TZ', 'goog.i18n.CompactNumberFormatSymbols_sah', 'goog.i18n.CompactNumberFormatSymbols_sah_RU', 'goog.i18n.CompactNumberFormatSymbols_saq', 'goog.i18n.CompactNumberFormatSymbols_saq_KE', 'goog.i18n.CompactNumberFormatSymbols_sbp', 'goog.i18n.CompactNumberFormatSymbols_sbp_TZ', 'goog.i18n.CompactNumberFormatSymbols_se', 'goog.i18n.CompactNumberFormatSymbols_se_FI', 'goog.i18n.CompactNumberFormatSymbols_se_NO', 'goog.i18n.CompactNumberFormatSymbols_seh', 'goog.i18n.CompactNumberFormatSymbols_seh_MZ', 'goog.i18n.CompactNumberFormatSymbols_ses', 'goog.i18n.CompactNumberFormatSymbols_ses_ML', 'goog.i18n.CompactNumberFormatSymbols_sg', 'goog.i18n.CompactNumberFormatSymbols_sg_CF', 'goog.i18n.CompactNumberFormatSymbols_shi', 'goog.i18n.CompactNumberFormatSymbols_shi_Latn', 'goog.i18n.CompactNumberFormatSymbols_shi_Latn_MA', 'goog.i18n.CompactNumberFormatSymbols_shi_Tfng', 'goog.i18n.CompactNumberFormatSymbols_shi_Tfng_MA', 'goog.i18n.CompactNumberFormatSymbols_sn', 'goog.i18n.CompactNumberFormatSymbols_sn_ZW', 'goog.i18n.CompactNumberFormatSymbols_so', 'goog.i18n.CompactNumberFormatSymbols_so_DJ', 'goog.i18n.CompactNumberFormatSymbols_so_ET', 'goog.i18n.CompactNumberFormatSymbols_so_KE', 'goog.i18n.CompactNumberFormatSymbols_so_SO', 'goog.i18n.CompactNumberFormatSymbols_sq_MK', 'goog.i18n.CompactNumberFormatSymbols_sq_XK', 'goog.i18n.CompactNumberFormatSymbols_sr_Cyrl', 'goog.i18n.CompactNumberFormatSymbols_sr_Cyrl_BA', 'goog.i18n.CompactNumberFormatSymbols_sr_Cyrl_ME', 'goog.i18n.CompactNumberFormatSymbols_sr_Cyrl_XK', 'goog.i18n.CompactNumberFormatSymbols_sr_Latn', 'goog.i18n.CompactNumberFormatSymbols_sr_Latn_BA', 'goog.i18n.CompactNumberFormatSymbols_sr_Latn_ME', 'goog.i18n.CompactNumberFormatSymbols_sr_Latn_RS', 'goog.i18n.CompactNumberFormatSymbols_sr_Latn_XK', 'goog.i18n.CompactNumberFormatSymbols_ss', 'goog.i18n.CompactNumberFormatSymbols_ss_SZ', 'goog.i18n.CompactNumberFormatSymbols_ss_ZA', 'goog.i18n.CompactNumberFormatSymbols_ssy', 'goog.i18n.CompactNumberFormatSymbols_ssy_ER', 'goog.i18n.CompactNumberFormatSymbols_st', 'goog.i18n.CompactNumberFormatSymbols_st_LS', 'goog.i18n.CompactNumberFormatSymbols_st_ZA', 'goog.i18n.CompactNumberFormatSymbols_sv_AX', 'goog.i18n.CompactNumberFormatSymbols_sv_FI', 'goog.i18n.CompactNumberFormatSymbols_sw_KE', 'goog.i18n.CompactNumberFormatSymbols_sw_UG', 'goog.i18n.CompactNumberFormatSymbols_swc', 'goog.i18n.CompactNumberFormatSymbols_swc_CD', 'goog.i18n.CompactNumberFormatSymbols_ta_LK', 'goog.i18n.CompactNumberFormatSymbols_ta_MY', 'goog.i18n.CompactNumberFormatSymbols_ta_SG', 'goog.i18n.CompactNumberFormatSymbols_teo', 'goog.i18n.CompactNumberFormatSymbols_teo_KE', 'goog.i18n.CompactNumberFormatSymbols_teo_UG', 'goog.i18n.CompactNumberFormatSymbols_tg', 'goog.i18n.CompactNumberFormatSymbols_tg_Cyrl', 'goog.i18n.CompactNumberFormatSymbols_tg_Cyrl_TJ', 'goog.i18n.CompactNumberFormatSymbols_ti', 'goog.i18n.CompactNumberFormatSymbols_ti_ER', 'goog.i18n.CompactNumberFormatSymbols_ti_ET', 'goog.i18n.CompactNumberFormatSymbols_tig', 'goog.i18n.CompactNumberFormatSymbols_tig_ER', 'goog.i18n.CompactNumberFormatSymbols_tn', 'goog.i18n.CompactNumberFormatSymbols_tn_BW', 'goog.i18n.CompactNumberFormatSymbols_tn_ZA', 'goog.i18n.CompactNumberFormatSymbols_to', 'goog.i18n.CompactNumberFormatSymbols_to_TO', 'goog.i18n.CompactNumberFormatSymbols_tr_CY', 'goog.i18n.CompactNumberFormatSymbols_ts', 'goog.i18n.CompactNumberFormatSymbols_ts_ZA', 'goog.i18n.CompactNumberFormatSymbols_twq', 'goog.i18n.CompactNumberFormatSymbols_twq_NE', 'goog.i18n.CompactNumberFormatSymbols_tzm', 'goog.i18n.CompactNumberFormatSymbols_tzm_Latn', 'goog.i18n.CompactNumberFormatSymbols_tzm_Latn_MA', 'goog.i18n.CompactNumberFormatSymbols_ug', 'goog.i18n.CompactNumberFormatSymbols_ug_Arab', 'goog.i18n.CompactNumberFormatSymbols_ug_Arab_CN', 'goog.i18n.CompactNumberFormatSymbols_ur_IN', 'goog.i18n.CompactNumberFormatSymbols_uz_Arab', 'goog.i18n.CompactNumberFormatSymbols_uz_Arab_AF', 'goog.i18n.CompactNumberFormatSymbols_uz_Cyrl', 'goog.i18n.CompactNumberFormatSymbols_uz_Cyrl_UZ', 'goog.i18n.CompactNumberFormatSymbols_uz_Latn', 'goog.i18n.CompactNumberFormatSymbols_vai', 'goog.i18n.CompactNumberFormatSymbols_vai_Latn', 'goog.i18n.CompactNumberFormatSymbols_vai_Latn_LR', 'goog.i18n.CompactNumberFormatSymbols_vai_Vaii', 'goog.i18n.CompactNumberFormatSymbols_vai_Vaii_LR', 'goog.i18n.CompactNumberFormatSymbols_ve', 'goog.i18n.CompactNumberFormatSymbols_ve_ZA', 'goog.i18n.CompactNumberFormatSymbols_vo', 'goog.i18n.CompactNumberFormatSymbols_vo_001', 'goog.i18n.CompactNumberFormatSymbols_vun', 'goog.i18n.CompactNumberFormatSymbols_vun_TZ', 'goog.i18n.CompactNumberFormatSymbols_wae', 'goog.i18n.CompactNumberFormatSymbols_wae_CH', 'goog.i18n.CompactNumberFormatSymbols_wal', 'goog.i18n.CompactNumberFormatSymbols_wal_ET', 'goog.i18n.CompactNumberFormatSymbols_xh', 'goog.i18n.CompactNumberFormatSymbols_xh_ZA', 'goog.i18n.CompactNumberFormatSymbols_xog', 'goog.i18n.CompactNumberFormatSymbols_xog_UG', 'goog.i18n.CompactNumberFormatSymbols_yav', 'goog.i18n.CompactNumberFormatSymbols_yav_CM', 'goog.i18n.CompactNumberFormatSymbols_yo', 'goog.i18n.CompactNumberFormatSymbols_yo_BJ', 'goog.i18n.CompactNumberFormatSymbols_yo_NG', 'goog.i18n.CompactNumberFormatSymbols_zgh', 'goog.i18n.CompactNumberFormatSymbols_zgh_MA', 'goog.i18n.CompactNumberFormatSymbols_zh_Hans', 'goog.i18n.CompactNumberFormatSymbols_zh_Hans_HK', 'goog.i18n.CompactNumberFormatSymbols_zh_Hans_MO', 'goog.i18n.CompactNumberFormatSymbols_zh_Hans_SG', 'goog.i18n.CompactNumberFormatSymbols_zh_Hant', 'goog.i18n.CompactNumberFormatSymbols_zh_Hant_HK', 'goog.i18n.CompactNumberFormatSymbols_zh_Hant_MO', 'goog.i18n.CompactNumberFormatSymbols_zh_Hant_TW'], [], false);
goog.addDependency('i18n/currency.js', ['goog.i18n.currency', 'goog.i18n.currency.CurrencyInfo', 'goog.i18n.currency.CurrencyInfoTier2'], [], false);
goog.addDependency('i18n/currency_test.js', ['goog.i18n.currencyTest'], ['goog.i18n.NumberFormat', 'goog.i18n.currency', 'goog.i18n.currency.CurrencyInfo', 'goog.object', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit'], false);
goog.addDependency('i18n/currencycodemap.js', ['goog.i18n.currencyCodeMap', 'goog.i18n.currencyCodeMapTier2'], [], false);
goog.addDependency('i18n/datetimeformat.js', ['goog.i18n.DateTimeFormat', 'goog.i18n.DateTimeFormat.Format'], ['goog.asserts', 'goog.date', 'goog.i18n.DateTimeSymbols', 'goog.i18n.TimeZone', 'goog.string'], false);
goog.addDependency('i18n/datetimeformat_test.js', ['goog.i18n.DateTimeFormatTest'], ['goog.date.Date', 'goog.date.DateTime', 'goog.i18n.DateTimeFormat', 'goog.i18n.DateTimePatterns', 'goog.i18n.DateTimePatterns_de', 'goog.i18n.DateTimePatterns_en', 'goog.i18n.DateTimePatterns_fa', 'goog.i18n.DateTimePatterns_fr', 'goog.i18n.DateTimePatterns_ja', 'goog.i18n.DateTimePatterns_sv', 'goog.i18n.DateTimeSymbols', 'goog.i18n.DateTimeSymbols_ar_AE', 'goog.i18n.DateTimeSymbols_ar_SA', 'goog.i18n.DateTimeSymbols_bn_BD', 'goog.i18n.DateTimeSymbols_de', 'goog.i18n.DateTimeSymbols_en', 'goog.i18n.DateTimeSymbols_en_GB', 'goog.i18n.DateTimeSymbols_en_IE', 'goog.i18n.DateTimeSymbols_en_IN', 'goog.i18n.DateTimeSymbols_en_US', 'goog.i18n.DateTimeSymbols_fa', 'goog.i18n.DateTimeSymbols_fr', 'goog.i18n.DateTimeSymbols_fr_DJ', 'goog.i18n.DateTimeSymbols_he_IL', 'goog.i18n.DateTimeSymbols_ja', 'goog.i18n.DateTimeSymbols_ro_RO', 'goog.i18n.DateTimeSymbols_sv', 'goog.i18n.TimeZone', 'goog.testing.jsunit'], false);
goog.addDependency('i18n/datetimeparse.js', ['goog.i18n.DateTimeParse'], ['goog.date', 'goog.i18n.DateTimeFormat', 'goog.i18n.DateTimeSymbols'], false);
goog.addDependency('i18n/datetimeparse_test.js', ['goog.i18n.DateTimeParseTest'], ['goog.date.Date', 'goog.i18n.DateTimeFormat', 'goog.i18n.DateTimeParse', 'goog.i18n.DateTimeSymbols', 'goog.i18n.DateTimeSymbols_en', 'goog.i18n.DateTimeSymbols_fa', 'goog.i18n.DateTimeSymbols_fr', 'goog.i18n.DateTimeSymbols_pl', 'goog.i18n.DateTimeSymbols_zh', 'goog.testing.ExpectedFailures', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('i18n/datetimepatterns.js', ['goog.i18n.DateTimePatterns', 'goog.i18n.DateTimePatterns_af', 'goog.i18n.DateTimePatterns_am', 'goog.i18n.DateTimePatterns_ar', 'goog.i18n.DateTimePatterns_az', 'goog.i18n.DateTimePatterns_bg', 'goog.i18n.DateTimePatterns_bn', 'goog.i18n.DateTimePatterns_br', 'goog.i18n.DateTimePatterns_ca', 'goog.i18n.DateTimePatterns_chr', 'goog.i18n.DateTimePatterns_cs', 'goog.i18n.DateTimePatterns_cy', 'goog.i18n.DateTimePatterns_da', 'goog.i18n.DateTimePatterns_de', 'goog.i18n.DateTimePatterns_de_AT', 'goog.i18n.DateTimePatterns_de_CH', 'goog.i18n.DateTimePatterns_el', 'goog.i18n.DateTimePatterns_en', 'goog.i18n.DateTimePatterns_en_AU', 'goog.i18n.DateTimePatterns_en_GB', 'goog.i18n.DateTimePatterns_en_IE', 'goog.i18n.DateTimePatterns_en_IN', 'goog.i18n.DateTimePatterns_en_SG', 'goog.i18n.DateTimePatterns_en_US', 'goog.i18n.DateTimePatterns_en_ZA', 'goog.i18n.DateTimePatterns_es', 'goog.i18n.DateTimePatterns_es_419', 'goog.i18n.DateTimePatterns_es_ES', 'goog.i18n.DateTimePatterns_et', 'goog.i18n.DateTimePatterns_eu', 'goog.i18n.DateTimePatterns_fa', 'goog.i18n.DateTimePatterns_fi', 'goog.i18n.DateTimePatterns_fil', 'goog.i18n.DateTimePatterns_fr', 'goog.i18n.DateTimePatterns_fr_CA', 'goog.i18n.DateTimePatterns_ga', 'goog.i18n.DateTimePatterns_gl', 'goog.i18n.DateTimePatterns_gsw', 'goog.i18n.DateTimePatterns_gu', 'goog.i18n.DateTimePatterns_haw', 'goog.i18n.DateTimePatterns_he', 'goog.i18n.DateTimePatterns_hi', 'goog.i18n.DateTimePatterns_hr', 'goog.i18n.DateTimePatterns_hu', 'goog.i18n.DateTimePatterns_hy', 'goog.i18n.DateTimePatterns_id', 'goog.i18n.DateTimePatterns_in', 'goog.i18n.DateTimePatterns_is', 'goog.i18n.DateTimePatterns_it', 'goog.i18n.DateTimePatterns_iw', 'goog.i18n.DateTimePatterns_ja', 'goog.i18n.DateTimePatterns_ka', 'goog.i18n.DateTimePatterns_kk', 'goog.i18n.DateTimePatterns_km', 'goog.i18n.DateTimePatterns_kn', 'goog.i18n.DateTimePatterns_ko', 'goog.i18n.DateTimePatterns_ky', 'goog.i18n.DateTimePatterns_ln', 'goog.i18n.DateTimePatterns_lo', 'goog.i18n.DateTimePatterns_lt', 'goog.i18n.DateTimePatterns_lv', 'goog.i18n.DateTimePatterns_mk', 'goog.i18n.DateTimePatterns_ml', 'goog.i18n.DateTimePatterns_mn', 'goog.i18n.DateTimePatterns_mo', 'goog.i18n.DateTimePatterns_mr', 'goog.i18n.DateTimePatterns_ms', 'goog.i18n.DateTimePatterns_mt', 'goog.i18n.DateTimePatterns_my', 'goog.i18n.DateTimePatterns_nb', 'goog.i18n.DateTimePatterns_ne', 'goog.i18n.DateTimePatterns_nl', 'goog.i18n.DateTimePatterns_no', 'goog.i18n.DateTimePatterns_no_NO', 'goog.i18n.DateTimePatterns_or', 'goog.i18n.DateTimePatterns_pa', 'goog.i18n.DateTimePatterns_pl', 'goog.i18n.DateTimePatterns_pt', 'goog.i18n.DateTimePatterns_pt_BR', 'goog.i18n.DateTimePatterns_pt_PT', 'goog.i18n.DateTimePatterns_ro', 'goog.i18n.DateTimePatterns_ru', 'goog.i18n.DateTimePatterns_sh', 'goog.i18n.DateTimePatterns_si', 'goog.i18n.DateTimePatterns_sk', 'goog.i18n.DateTimePatterns_sl', 'goog.i18n.DateTimePatterns_sq', 'goog.i18n.DateTimePatterns_sr', 'goog.i18n.DateTimePatterns_sv', 'goog.i18n.DateTimePatterns_sw', 'goog.i18n.DateTimePatterns_ta', 'goog.i18n.DateTimePatterns_te', 'goog.i18n.DateTimePatterns_th', 'goog.i18n.DateTimePatterns_tl', 'goog.i18n.DateTimePatterns_tr', 'goog.i18n.DateTimePatterns_uk', 'goog.i18n.DateTimePatterns_ur', 'goog.i18n.DateTimePatterns_uz', 'goog.i18n.DateTimePatterns_vi', 'goog.i18n.DateTimePatterns_zh', 'goog.i18n.DateTimePatterns_zh_CN', 'goog.i18n.DateTimePatterns_zh_HK', 'goog.i18n.DateTimePatterns_zh_TW', 'goog.i18n.DateTimePatterns_zu'], [], false);
goog.addDependency('i18n/datetimepatternsext.js', ['goog.i18n.DateTimePatternsExt', 'goog.i18n.DateTimePatterns_af_NA', 'goog.i18n.DateTimePatterns_af_ZA', 'goog.i18n.DateTimePatterns_agq', 'goog.i18n.DateTimePatterns_agq_CM', 'goog.i18n.DateTimePatterns_ak', 'goog.i18n.DateTimePatterns_ak_GH', 'goog.i18n.DateTimePatterns_am_ET', 'goog.i18n.DateTimePatterns_ar_001', 'goog.i18n.DateTimePatterns_ar_AE', 'goog.i18n.DateTimePatterns_ar_BH', 'goog.i18n.DateTimePatterns_ar_DJ', 'goog.i18n.DateTimePatterns_ar_DZ', 'goog.i18n.DateTimePatterns_ar_EG', 'goog.i18n.DateTimePatterns_ar_EH', 'goog.i18n.DateTimePatterns_ar_ER', 'goog.i18n.DateTimePatterns_ar_IL', 'goog.i18n.DateTimePatterns_ar_IQ', 'goog.i18n.DateTimePatterns_ar_JO', 'goog.i18n.DateTimePatterns_ar_KM', 'goog.i18n.DateTimePatterns_ar_KW', 'goog.i18n.DateTimePatterns_ar_LB', 'goog.i18n.DateTimePatterns_ar_LY', 'goog.i18n.DateTimePatterns_ar_MA', 'goog.i18n.DateTimePatterns_ar_MR', 'goog.i18n.DateTimePatterns_ar_OM', 'goog.i18n.DateTimePatterns_ar_PS', 'goog.i18n.DateTimePatterns_ar_QA', 'goog.i18n.DateTimePatterns_ar_SA', 'goog.i18n.DateTimePatterns_ar_SD', 'goog.i18n.DateTimePatterns_ar_SO', 'goog.i18n.DateTimePatterns_ar_SS', 'goog.i18n.DateTimePatterns_ar_SY', 'goog.i18n.DateTimePatterns_ar_TD', 'goog.i18n.DateTimePatterns_ar_TN', 'goog.i18n.DateTimePatterns_ar_YE', 'goog.i18n.DateTimePatterns_as', 'goog.i18n.DateTimePatterns_as_IN', 'goog.i18n.DateTimePatterns_asa', 'goog.i18n.DateTimePatterns_asa_TZ', 'goog.i18n.DateTimePatterns_az_Cyrl', 'goog.i18n.DateTimePatterns_az_Cyrl_AZ', 'goog.i18n.DateTimePatterns_az_Latn', 'goog.i18n.DateTimePatterns_az_Latn_AZ', 'goog.i18n.DateTimePatterns_bas', 'goog.i18n.DateTimePatterns_bas_CM', 'goog.i18n.DateTimePatterns_be', 'goog.i18n.DateTimePatterns_be_BY', 'goog.i18n.DateTimePatterns_bem', 'goog.i18n.DateTimePatterns_bem_ZM', 'goog.i18n.DateTimePatterns_bez', 'goog.i18n.DateTimePatterns_bez_TZ', 'goog.i18n.DateTimePatterns_bg_BG', 'goog.i18n.DateTimePatterns_bm', 'goog.i18n.DateTimePatterns_bm_ML', 'goog.i18n.DateTimePatterns_bn_BD', 'goog.i18n.DateTimePatterns_bn_IN', 'goog.i18n.DateTimePatterns_bo', 'goog.i18n.DateTimePatterns_bo_CN', 'goog.i18n.DateTimePatterns_bo_IN', 'goog.i18n.DateTimePatterns_br_FR', 'goog.i18n.DateTimePatterns_brx', 'goog.i18n.DateTimePatterns_brx_IN', 'goog.i18n.DateTimePatterns_bs', 'goog.i18n.DateTimePatterns_bs_Cyrl', 'goog.i18n.DateTimePatterns_bs_Cyrl_BA', 'goog.i18n.DateTimePatterns_bs_Latn', 'goog.i18n.DateTimePatterns_bs_Latn_BA', 'goog.i18n.DateTimePatterns_ca_AD', 'goog.i18n.DateTimePatterns_ca_ES', 'goog.i18n.DateTimePatterns_ca_FR', 'goog.i18n.DateTimePatterns_ca_IT', 'goog.i18n.DateTimePatterns_cgg', 'goog.i18n.DateTimePatterns_cgg_UG', 'goog.i18n.DateTimePatterns_chr_US', 'goog.i18n.DateTimePatterns_cs_CZ', 'goog.i18n.DateTimePatterns_cy_GB', 'goog.i18n.DateTimePatterns_da_DK', 'goog.i18n.DateTimePatterns_da_GL', 'goog.i18n.DateTimePatterns_dav', 'goog.i18n.DateTimePatterns_dav_KE', 'goog.i18n.DateTimePatterns_de_BE', 'goog.i18n.DateTimePatterns_de_DE', 'goog.i18n.DateTimePatterns_de_LI', 'goog.i18n.DateTimePatterns_de_LU', 'goog.i18n.DateTimePatterns_dje', 'goog.i18n.DateTimePatterns_dje_NE', 'goog.i18n.DateTimePatterns_dua', 'goog.i18n.DateTimePatterns_dua_CM', 'goog.i18n.DateTimePatterns_dyo', 'goog.i18n.DateTimePatterns_dyo_SN', 'goog.i18n.DateTimePatterns_dz', 'goog.i18n.DateTimePatterns_dz_BT', 'goog.i18n.DateTimePatterns_ebu', 'goog.i18n.DateTimePatterns_ebu_KE', 'goog.i18n.DateTimePatterns_ee', 'goog.i18n.DateTimePatterns_ee_GH', 'goog.i18n.DateTimePatterns_ee_TG', 'goog.i18n.DateTimePatterns_el_CY', 'goog.i18n.DateTimePatterns_el_GR', 'goog.i18n.DateTimePatterns_en_001', 'goog.i18n.DateTimePatterns_en_150', 'goog.i18n.DateTimePatterns_en_AG', 'goog.i18n.DateTimePatterns_en_AI', 'goog.i18n.DateTimePatterns_en_AS', 'goog.i18n.DateTimePatterns_en_BB', 'goog.i18n.DateTimePatterns_en_BE', 'goog.i18n.DateTimePatterns_en_BM', 'goog.i18n.DateTimePatterns_en_BS', 'goog.i18n.DateTimePatterns_en_BW', 'goog.i18n.DateTimePatterns_en_BZ', 'goog.i18n.DateTimePatterns_en_CA', 'goog.i18n.DateTimePatterns_en_CC', 'goog.i18n.DateTimePatterns_en_CK', 'goog.i18n.DateTimePatterns_en_CM', 'goog.i18n.DateTimePatterns_en_CX', 'goog.i18n.DateTimePatterns_en_DG', 'goog.i18n.DateTimePatterns_en_DM', 'goog.i18n.DateTimePatterns_en_ER', 'goog.i18n.DateTimePatterns_en_FJ', 'goog.i18n.DateTimePatterns_en_FK', 'goog.i18n.DateTimePatterns_en_FM', 'goog.i18n.DateTimePatterns_en_GD', 'goog.i18n.DateTimePatterns_en_GG', 'goog.i18n.DateTimePatterns_en_GH', 'goog.i18n.DateTimePatterns_en_GI', 'goog.i18n.DateTimePatterns_en_GM', 'goog.i18n.DateTimePatterns_en_GU', 'goog.i18n.DateTimePatterns_en_GY', 'goog.i18n.DateTimePatterns_en_HK', 'goog.i18n.DateTimePatterns_en_IM', 'goog.i18n.DateTimePatterns_en_IO', 'goog.i18n.DateTimePatterns_en_JE', 'goog.i18n.DateTimePatterns_en_JM', 'goog.i18n.DateTimePatterns_en_KE', 'goog.i18n.DateTimePatterns_en_KI', 'goog.i18n.DateTimePatterns_en_KN', 'goog.i18n.DateTimePatterns_en_KY', 'goog.i18n.DateTimePatterns_en_LC', 'goog.i18n.DateTimePatterns_en_LR', 'goog.i18n.DateTimePatterns_en_LS', 'goog.i18n.DateTimePatterns_en_MG', 'goog.i18n.DateTimePatterns_en_MH', 'goog.i18n.DateTimePatterns_en_MO', 'goog.i18n.DateTimePatterns_en_MP', 'goog.i18n.DateTimePatterns_en_MS', 'goog.i18n.DateTimePatterns_en_MT', 'goog.i18n.DateTimePatterns_en_MU', 'goog.i18n.DateTimePatterns_en_MW', 'goog.i18n.DateTimePatterns_en_NA', 'goog.i18n.DateTimePatterns_en_NF', 'goog.i18n.DateTimePatterns_en_NG', 'goog.i18n.DateTimePatterns_en_NR', 'goog.i18n.DateTimePatterns_en_NU', 'goog.i18n.DateTimePatterns_en_NZ', 'goog.i18n.DateTimePatterns_en_PG', 'goog.i18n.DateTimePatterns_en_PH', 'goog.i18n.DateTimePatterns_en_PK', 'goog.i18n.DateTimePatterns_en_PN', 'goog.i18n.DateTimePatterns_en_PR', 'goog.i18n.DateTimePatterns_en_PW', 'goog.i18n.DateTimePatterns_en_RW', 'goog.i18n.DateTimePatterns_en_SB', 'goog.i18n.DateTimePatterns_en_SC', 'goog.i18n.DateTimePatterns_en_SD', 'goog.i18n.DateTimePatterns_en_SH', 'goog.i18n.DateTimePatterns_en_SL', 'goog.i18n.DateTimePatterns_en_SS', 'goog.i18n.DateTimePatterns_en_SX', 'goog.i18n.DateTimePatterns_en_SZ', 'goog.i18n.DateTimePatterns_en_TC', 'goog.i18n.DateTimePatterns_en_TK', 'goog.i18n.DateTimePatterns_en_TO', 'goog.i18n.DateTimePatterns_en_TT', 'goog.i18n.DateTimePatterns_en_TV', 'goog.i18n.DateTimePatterns_en_TZ', 'goog.i18n.DateTimePatterns_en_UG', 'goog.i18n.DateTimePatterns_en_UM', 'goog.i18n.DateTimePatterns_en_US_POSIX', 'goog.i18n.DateTimePatterns_en_VC', 'goog.i18n.DateTimePatterns_en_VG', 'goog.i18n.DateTimePatterns_en_VI', 'goog.i18n.DateTimePatterns_en_VU', 'goog.i18n.DateTimePatterns_en_WS', 'goog.i18n.DateTimePatterns_en_ZM', 'goog.i18n.DateTimePatterns_en_ZW', 'goog.i18n.DateTimePatterns_eo', 'goog.i18n.DateTimePatterns_es_AR', 'goog.i18n.DateTimePatterns_es_BO', 'goog.i18n.DateTimePatterns_es_CL', 'goog.i18n.DateTimePatterns_es_CO', 'goog.i18n.DateTimePatterns_es_CR', 'goog.i18n.DateTimePatterns_es_CU', 'goog.i18n.DateTimePatterns_es_DO', 'goog.i18n.DateTimePatterns_es_EA', 'goog.i18n.DateTimePatterns_es_EC', 'goog.i18n.DateTimePatterns_es_GQ', 'goog.i18n.DateTimePatterns_es_GT', 'goog.i18n.DateTimePatterns_es_HN', 'goog.i18n.DateTimePatterns_es_IC', 'goog.i18n.DateTimePatterns_es_MX', 'goog.i18n.DateTimePatterns_es_NI', 'goog.i18n.DateTimePatterns_es_PA', 'goog.i18n.DateTimePatterns_es_PE', 'goog.i18n.DateTimePatterns_es_PH', 'goog.i18n.DateTimePatterns_es_PR', 'goog.i18n.DateTimePatterns_es_PY', 'goog.i18n.DateTimePatterns_es_SV', 'goog.i18n.DateTimePatterns_es_US', 'goog.i18n.DateTimePatterns_es_UY', 'goog.i18n.DateTimePatterns_es_VE', 'goog.i18n.DateTimePatterns_et_EE', 'goog.i18n.DateTimePatterns_eu_ES', 'goog.i18n.DateTimePatterns_ewo', 'goog.i18n.DateTimePatterns_ewo_CM', 'goog.i18n.DateTimePatterns_fa_AF', 'goog.i18n.DateTimePatterns_fa_IR', 'goog.i18n.DateTimePatterns_ff', 'goog.i18n.DateTimePatterns_ff_SN', 'goog.i18n.DateTimePatterns_fi_FI', 'goog.i18n.DateTimePatterns_fil_PH', 'goog.i18n.DateTimePatterns_fo', 'goog.i18n.DateTimePatterns_fo_FO', 'goog.i18n.DateTimePatterns_fr_BE', 'goog.i18n.DateTimePatterns_fr_BF', 'goog.i18n.DateTimePatterns_fr_BI', 'goog.i18n.DateTimePatterns_fr_BJ', 'goog.i18n.DateTimePatterns_fr_BL', 'goog.i18n.DateTimePatterns_fr_CD', 'goog.i18n.DateTimePatterns_fr_CF', 'goog.i18n.DateTimePatterns_fr_CG', 'goog.i18n.DateTimePatterns_fr_CH', 'goog.i18n.DateTimePatterns_fr_CI', 'goog.i18n.DateTimePatterns_fr_CM', 'goog.i18n.DateTimePatterns_fr_DJ', 'goog.i18n.DateTimePatterns_fr_DZ', 'goog.i18n.DateTimePatterns_fr_FR', 'goog.i18n.DateTimePatterns_fr_GA', 'goog.i18n.DateTimePatterns_fr_GF', 'goog.i18n.DateTimePatterns_fr_GN', 'goog.i18n.DateTimePatterns_fr_GP', 'goog.i18n.DateTimePatterns_fr_GQ', 'goog.i18n.DateTimePatterns_fr_HT', 'goog.i18n.DateTimePatterns_fr_KM', 'goog.i18n.DateTimePatterns_fr_LU', 'goog.i18n.DateTimePatterns_fr_MA', 'goog.i18n.DateTimePatterns_fr_MC', 'goog.i18n.DateTimePatterns_fr_MF', 'goog.i18n.DateTimePatterns_fr_MG', 'goog.i18n.DateTimePatterns_fr_ML', 'goog.i18n.DateTimePatterns_fr_MQ', 'goog.i18n.DateTimePatterns_fr_MR', 'goog.i18n.DateTimePatterns_fr_MU', 'goog.i18n.DateTimePatterns_fr_NC', 'goog.i18n.DateTimePatterns_fr_NE', 'goog.i18n.DateTimePatterns_fr_PF', 'goog.i18n.DateTimePatterns_fr_PM', 'goog.i18n.DateTimePatterns_fr_RE', 'goog.i18n.DateTimePatterns_fr_RW', 'goog.i18n.DateTimePatterns_fr_SC', 'goog.i18n.DateTimePatterns_fr_SN', 'goog.i18n.DateTimePatterns_fr_SY', 'goog.i18n.DateTimePatterns_fr_TD', 'goog.i18n.DateTimePatterns_fr_TG', 'goog.i18n.DateTimePatterns_fr_TN', 'goog.i18n.DateTimePatterns_fr_VU', 'goog.i18n.DateTimePatterns_fr_WF', 'goog.i18n.DateTimePatterns_fr_YT', 'goog.i18n.DateTimePatterns_ga_IE', 'goog.i18n.DateTimePatterns_gl_ES', 'goog.i18n.DateTimePatterns_gsw_CH', 'goog.i18n.DateTimePatterns_gsw_LI', 'goog.i18n.DateTimePatterns_gu_IN', 'goog.i18n.DateTimePatterns_guz', 'goog.i18n.DateTimePatterns_guz_KE', 'goog.i18n.DateTimePatterns_gv', 'goog.i18n.DateTimePatterns_gv_IM', 'goog.i18n.DateTimePatterns_ha', 'goog.i18n.DateTimePatterns_ha_Latn', 'goog.i18n.DateTimePatterns_ha_Latn_GH', 'goog.i18n.DateTimePatterns_ha_Latn_NE', 'goog.i18n.DateTimePatterns_ha_Latn_NG', 'goog.i18n.DateTimePatterns_haw_US', 'goog.i18n.DateTimePatterns_he_IL', 'goog.i18n.DateTimePatterns_hi_IN', 'goog.i18n.DateTimePatterns_hr_BA', 'goog.i18n.DateTimePatterns_hr_HR', 'goog.i18n.DateTimePatterns_hu_HU', 'goog.i18n.DateTimePatterns_hy_AM', 'goog.i18n.DateTimePatterns_id_ID', 'goog.i18n.DateTimePatterns_ig', 'goog.i18n.DateTimePatterns_ig_NG', 'goog.i18n.DateTimePatterns_ii', 'goog.i18n.DateTimePatterns_ii_CN', 'goog.i18n.DateTimePatterns_is_IS', 'goog.i18n.DateTimePatterns_it_CH', 'goog.i18n.DateTimePatterns_it_IT', 'goog.i18n.DateTimePatterns_it_SM', 'goog.i18n.DateTimePatterns_ja_JP', 'goog.i18n.DateTimePatterns_jgo', 'goog.i18n.DateTimePatterns_jgo_CM', 'goog.i18n.DateTimePatterns_jmc', 'goog.i18n.DateTimePatterns_jmc_TZ', 'goog.i18n.DateTimePatterns_ka_GE', 'goog.i18n.DateTimePatterns_kab', 'goog.i18n.DateTimePatterns_kab_DZ', 'goog.i18n.DateTimePatterns_kam', 'goog.i18n.DateTimePatterns_kam_KE', 'goog.i18n.DateTimePatterns_kde', 'goog.i18n.DateTimePatterns_kde_TZ', 'goog.i18n.DateTimePatterns_kea', 'goog.i18n.DateTimePatterns_kea_CV', 'goog.i18n.DateTimePatterns_khq', 'goog.i18n.DateTimePatterns_khq_ML', 'goog.i18n.DateTimePatterns_ki', 'goog.i18n.DateTimePatterns_ki_KE', 'goog.i18n.DateTimePatterns_kk_Cyrl', 'goog.i18n.DateTimePatterns_kk_Cyrl_KZ', 'goog.i18n.DateTimePatterns_kkj', 'goog.i18n.DateTimePatterns_kkj_CM', 'goog.i18n.DateTimePatterns_kl', 'goog.i18n.DateTimePatterns_kl_GL', 'goog.i18n.DateTimePatterns_kln', 'goog.i18n.DateTimePatterns_kln_KE', 'goog.i18n.DateTimePatterns_km_KH', 'goog.i18n.DateTimePatterns_kn_IN', 'goog.i18n.DateTimePatterns_ko_KP', 'goog.i18n.DateTimePatterns_ko_KR', 'goog.i18n.DateTimePatterns_kok', 'goog.i18n.DateTimePatterns_kok_IN', 'goog.i18n.DateTimePatterns_ks', 'goog.i18n.DateTimePatterns_ks_Arab', 'goog.i18n.DateTimePatterns_ks_Arab_IN', 'goog.i18n.DateTimePatterns_ksb', 'goog.i18n.DateTimePatterns_ksb_TZ', 'goog.i18n.DateTimePatterns_ksf', 'goog.i18n.DateTimePatterns_ksf_CM', 'goog.i18n.DateTimePatterns_kw', 'goog.i18n.DateTimePatterns_kw_GB', 'goog.i18n.DateTimePatterns_ky_Cyrl', 'goog.i18n.DateTimePatterns_ky_Cyrl_KG', 'goog.i18n.DateTimePatterns_lag', 'goog.i18n.DateTimePatterns_lag_TZ', 'goog.i18n.DateTimePatterns_lg', 'goog.i18n.DateTimePatterns_lg_UG', 'goog.i18n.DateTimePatterns_lkt', 'goog.i18n.DateTimePatterns_lkt_US', 'goog.i18n.DateTimePatterns_ln_AO', 'goog.i18n.DateTimePatterns_ln_CD', 'goog.i18n.DateTimePatterns_ln_CF', 'goog.i18n.DateTimePatterns_ln_CG', 'goog.i18n.DateTimePatterns_lo_LA', 'goog.i18n.DateTimePatterns_lt_LT', 'goog.i18n.DateTimePatterns_lu', 'goog.i18n.DateTimePatterns_lu_CD', 'goog.i18n.DateTimePatterns_luo', 'goog.i18n.DateTimePatterns_luo_KE', 'goog.i18n.DateTimePatterns_luy', 'goog.i18n.DateTimePatterns_luy_KE', 'goog.i18n.DateTimePatterns_lv_LV', 'goog.i18n.DateTimePatterns_mas', 'goog.i18n.DateTimePatterns_mas_KE', 'goog.i18n.DateTimePatterns_mas_TZ', 'goog.i18n.DateTimePatterns_mer', 'goog.i18n.DateTimePatterns_mer_KE', 'goog.i18n.DateTimePatterns_mfe', 'goog.i18n.DateTimePatterns_mfe_MU', 'goog.i18n.DateTimePatterns_mg', 'goog.i18n.DateTimePatterns_mg_MG', 'goog.i18n.DateTimePatterns_mgh', 'goog.i18n.DateTimePatterns_mgh_MZ', 'goog.i18n.DateTimePatterns_mgo', 'goog.i18n.DateTimePatterns_mgo_CM', 'goog.i18n.DateTimePatterns_mk_MK', 'goog.i18n.DateTimePatterns_ml_IN', 'goog.i18n.DateTimePatterns_mn_Cyrl', 'goog.i18n.DateTimePatterns_mn_Cyrl_MN', 'goog.i18n.DateTimePatterns_mr_IN', 'goog.i18n.DateTimePatterns_ms_Latn', 'goog.i18n.DateTimePatterns_ms_Latn_BN', 'goog.i18n.DateTimePatterns_ms_Latn_MY', 'goog.i18n.DateTimePatterns_ms_Latn_SG', 'goog.i18n.DateTimePatterns_mt_MT', 'goog.i18n.DateTimePatterns_mua', 'goog.i18n.DateTimePatterns_mua_CM', 'goog.i18n.DateTimePatterns_my_MM', 'goog.i18n.DateTimePatterns_naq', 'goog.i18n.DateTimePatterns_naq_NA', 'goog.i18n.DateTimePatterns_nb_NO', 'goog.i18n.DateTimePatterns_nb_SJ', 'goog.i18n.DateTimePatterns_nd', 'goog.i18n.DateTimePatterns_nd_ZW', 'goog.i18n.DateTimePatterns_ne_IN', 'goog.i18n.DateTimePatterns_ne_NP', 'goog.i18n.DateTimePatterns_nl_AW', 'goog.i18n.DateTimePatterns_nl_BE', 'goog.i18n.DateTimePatterns_nl_BQ', 'goog.i18n.DateTimePatterns_nl_CW', 'goog.i18n.DateTimePatterns_nl_NL', 'goog.i18n.DateTimePatterns_nl_SR', 'goog.i18n.DateTimePatterns_nl_SX', 'goog.i18n.DateTimePatterns_nmg', 'goog.i18n.DateTimePatterns_nmg_CM', 'goog.i18n.DateTimePatterns_nn', 'goog.i18n.DateTimePatterns_nn_NO', 'goog.i18n.DateTimePatterns_nnh', 'goog.i18n.DateTimePatterns_nnh_CM', 'goog.i18n.DateTimePatterns_nus', 'goog.i18n.DateTimePatterns_nus_SD', 'goog.i18n.DateTimePatterns_nyn', 'goog.i18n.DateTimePatterns_nyn_UG', 'goog.i18n.DateTimePatterns_om', 'goog.i18n.DateTimePatterns_om_ET', 'goog.i18n.DateTimePatterns_om_KE', 'goog.i18n.DateTimePatterns_or_IN', 'goog.i18n.DateTimePatterns_pa_Arab', 'goog.i18n.DateTimePatterns_pa_Arab_PK', 'goog.i18n.DateTimePatterns_pa_Guru', 'goog.i18n.DateTimePatterns_pa_Guru_IN', 'goog.i18n.DateTimePatterns_pl_PL', 'goog.i18n.DateTimePatterns_ps', 'goog.i18n.DateTimePatterns_ps_AF', 'goog.i18n.DateTimePatterns_pt_AO', 'goog.i18n.DateTimePatterns_pt_CV', 'goog.i18n.DateTimePatterns_pt_GW', 'goog.i18n.DateTimePatterns_pt_MO', 'goog.i18n.DateTimePatterns_pt_MZ', 'goog.i18n.DateTimePatterns_pt_ST', 'goog.i18n.DateTimePatterns_pt_TL', 'goog.i18n.DateTimePatterns_rm', 'goog.i18n.DateTimePatterns_rm_CH', 'goog.i18n.DateTimePatterns_rn', 'goog.i18n.DateTimePatterns_rn_BI', 'goog.i18n.DateTimePatterns_ro_MD', 'goog.i18n.DateTimePatterns_ro_RO', 'goog.i18n.DateTimePatterns_rof', 'goog.i18n.DateTimePatterns_rof_TZ', 'goog.i18n.DateTimePatterns_ru_BY', 'goog.i18n.DateTimePatterns_ru_KG', 'goog.i18n.DateTimePatterns_ru_KZ', 'goog.i18n.DateTimePatterns_ru_MD', 'goog.i18n.DateTimePatterns_ru_RU', 'goog.i18n.DateTimePatterns_ru_UA', 'goog.i18n.DateTimePatterns_rw', 'goog.i18n.DateTimePatterns_rw_RW', 'goog.i18n.DateTimePatterns_rwk', 'goog.i18n.DateTimePatterns_rwk_TZ', 'goog.i18n.DateTimePatterns_saq', 'goog.i18n.DateTimePatterns_saq_KE', 'goog.i18n.DateTimePatterns_sbp', 'goog.i18n.DateTimePatterns_sbp_TZ', 'goog.i18n.DateTimePatterns_seh', 'goog.i18n.DateTimePatterns_seh_MZ', 'goog.i18n.DateTimePatterns_ses', 'goog.i18n.DateTimePatterns_ses_ML', 'goog.i18n.DateTimePatterns_sg', 'goog.i18n.DateTimePatterns_sg_CF', 'goog.i18n.DateTimePatterns_shi', 'goog.i18n.DateTimePatterns_shi_Latn', 'goog.i18n.DateTimePatterns_shi_Latn_MA', 'goog.i18n.DateTimePatterns_shi_Tfng', 'goog.i18n.DateTimePatterns_shi_Tfng_MA', 'goog.i18n.DateTimePatterns_si_LK', 'goog.i18n.DateTimePatterns_sk_SK', 'goog.i18n.DateTimePatterns_sl_SI', 'goog.i18n.DateTimePatterns_sn', 'goog.i18n.DateTimePatterns_sn_ZW', 'goog.i18n.DateTimePatterns_so', 'goog.i18n.DateTimePatterns_so_DJ', 'goog.i18n.DateTimePatterns_so_ET', 'goog.i18n.DateTimePatterns_so_KE', 'goog.i18n.DateTimePatterns_so_SO', 'goog.i18n.DateTimePatterns_sq_AL', 'goog.i18n.DateTimePatterns_sq_MK', 'goog.i18n.DateTimePatterns_sq_XK', 'goog.i18n.DateTimePatterns_sr_Cyrl', 'goog.i18n.DateTimePatterns_sr_Cyrl_BA', 'goog.i18n.DateTimePatterns_sr_Cyrl_ME', 'goog.i18n.DateTimePatterns_sr_Cyrl_RS', 'goog.i18n.DateTimePatterns_sr_Cyrl_XK', 'goog.i18n.DateTimePatterns_sr_Latn', 'goog.i18n.DateTimePatterns_sr_Latn_BA', 'goog.i18n.DateTimePatterns_sr_Latn_ME', 'goog.i18n.DateTimePatterns_sr_Latn_RS', 'goog.i18n.DateTimePatterns_sr_Latn_XK', 'goog.i18n.DateTimePatterns_sv_AX', 'goog.i18n.DateTimePatterns_sv_FI', 'goog.i18n.DateTimePatterns_sv_SE', 'goog.i18n.DateTimePatterns_sw_KE', 'goog.i18n.DateTimePatterns_sw_TZ', 'goog.i18n.DateTimePatterns_sw_UG', 'goog.i18n.DateTimePatterns_swc', 'goog.i18n.DateTimePatterns_swc_CD', 'goog.i18n.DateTimePatterns_ta_IN', 'goog.i18n.DateTimePatterns_ta_LK', 'goog.i18n.DateTimePatterns_ta_MY', 'goog.i18n.DateTimePatterns_ta_SG', 'goog.i18n.DateTimePatterns_te_IN', 'goog.i18n.DateTimePatterns_teo', 'goog.i18n.DateTimePatterns_teo_KE', 'goog.i18n.DateTimePatterns_teo_UG', 'goog.i18n.DateTimePatterns_th_TH', 'goog.i18n.DateTimePatterns_ti', 'goog.i18n.DateTimePatterns_ti_ER', 'goog.i18n.DateTimePatterns_ti_ET', 'goog.i18n.DateTimePatterns_to', 'goog.i18n.DateTimePatterns_to_TO', 'goog.i18n.DateTimePatterns_tr_CY', 'goog.i18n.DateTimePatterns_tr_TR', 'goog.i18n.DateTimePatterns_twq', 'goog.i18n.DateTimePatterns_twq_NE', 'goog.i18n.DateTimePatterns_tzm', 'goog.i18n.DateTimePatterns_tzm_Latn', 'goog.i18n.DateTimePatterns_tzm_Latn_MA', 'goog.i18n.DateTimePatterns_ug', 'goog.i18n.DateTimePatterns_ug_Arab', 'goog.i18n.DateTimePatterns_ug_Arab_CN', 'goog.i18n.DateTimePatterns_uk_UA', 'goog.i18n.DateTimePatterns_ur_IN', 'goog.i18n.DateTimePatterns_ur_PK', 'goog.i18n.DateTimePatterns_uz_Arab', 'goog.i18n.DateTimePatterns_uz_Arab_AF', 'goog.i18n.DateTimePatterns_uz_Cyrl', 'goog.i18n.DateTimePatterns_uz_Cyrl_UZ', 'goog.i18n.DateTimePatterns_uz_Latn', 'goog.i18n.DateTimePatterns_uz_Latn_UZ', 'goog.i18n.DateTimePatterns_vai', 'goog.i18n.DateTimePatterns_vai_Latn', 'goog.i18n.DateTimePatterns_vai_Latn_LR', 'goog.i18n.DateTimePatterns_vai_Vaii', 'goog.i18n.DateTimePatterns_vai_Vaii_LR', 'goog.i18n.DateTimePatterns_vi_VN', 'goog.i18n.DateTimePatterns_vun', 'goog.i18n.DateTimePatterns_vun_TZ', 'goog.i18n.DateTimePatterns_xog', 'goog.i18n.DateTimePatterns_xog_UG', 'goog.i18n.DateTimePatterns_yav', 'goog.i18n.DateTimePatterns_yav_CM', 'goog.i18n.DateTimePatterns_yo', 'goog.i18n.DateTimePatterns_yo_BJ', 'goog.i18n.DateTimePatterns_yo_NG', 'goog.i18n.DateTimePatterns_zgh', 'goog.i18n.DateTimePatterns_zgh_MA', 'goog.i18n.DateTimePatterns_zh_Hans', 'goog.i18n.DateTimePatterns_zh_Hans_CN', 'goog.i18n.DateTimePatterns_zh_Hans_HK', 'goog.i18n.DateTimePatterns_zh_Hans_MO', 'goog.i18n.DateTimePatterns_zh_Hans_SG', 'goog.i18n.DateTimePatterns_zh_Hant', 'goog.i18n.DateTimePatterns_zh_Hant_HK', 'goog.i18n.DateTimePatterns_zh_Hant_MO', 'goog.i18n.DateTimePatterns_zh_Hant_TW', 'goog.i18n.DateTimePatterns_zu_ZA'], ['goog.i18n.DateTimePatterns'], false);
goog.addDependency('i18n/datetimesymbols.js', ['goog.i18n.DateTimeSymbols', 'goog.i18n.DateTimeSymbols_af', 'goog.i18n.DateTimeSymbols_am', 'goog.i18n.DateTimeSymbols_ar', 'goog.i18n.DateTimeSymbols_az', 'goog.i18n.DateTimeSymbols_bg', 'goog.i18n.DateTimeSymbols_bn', 'goog.i18n.DateTimeSymbols_br', 'goog.i18n.DateTimeSymbols_ca', 'goog.i18n.DateTimeSymbols_chr', 'goog.i18n.DateTimeSymbols_cs', 'goog.i18n.DateTimeSymbols_cy', 'goog.i18n.DateTimeSymbols_da', 'goog.i18n.DateTimeSymbols_de', 'goog.i18n.DateTimeSymbols_de_AT', 'goog.i18n.DateTimeSymbols_de_CH', 'goog.i18n.DateTimeSymbols_el', 'goog.i18n.DateTimeSymbols_en', 'goog.i18n.DateTimeSymbols_en_AU', 'goog.i18n.DateTimeSymbols_en_GB', 'goog.i18n.DateTimeSymbols_en_IE', 'goog.i18n.DateTimeSymbols_en_IN', 'goog.i18n.DateTimeSymbols_en_ISO', 'goog.i18n.DateTimeSymbols_en_SG', 'goog.i18n.DateTimeSymbols_en_US', 'goog.i18n.DateTimeSymbols_en_ZA', 'goog.i18n.DateTimeSymbols_es', 'goog.i18n.DateTimeSymbols_es_419', 'goog.i18n.DateTimeSymbols_es_ES', 'goog.i18n.DateTimeSymbols_et', 'goog.i18n.DateTimeSymbols_eu', 'goog.i18n.DateTimeSymbols_fa', 'goog.i18n.DateTimeSymbols_fi', 'goog.i18n.DateTimeSymbols_fil', 'goog.i18n.DateTimeSymbols_fr', 'goog.i18n.DateTimeSymbols_fr_CA', 'goog.i18n.DateTimeSymbols_ga', 'goog.i18n.DateTimeSymbols_gl', 'goog.i18n.DateTimeSymbols_gsw', 'goog.i18n.DateTimeSymbols_gu', 'goog.i18n.DateTimeSymbols_haw', 'goog.i18n.DateTimeSymbols_he', 'goog.i18n.DateTimeSymbols_hi', 'goog.i18n.DateTimeSymbols_hr', 'goog.i18n.DateTimeSymbols_hu', 'goog.i18n.DateTimeSymbols_hy', 'goog.i18n.DateTimeSymbols_id', 'goog.i18n.DateTimeSymbols_in', 'goog.i18n.DateTimeSymbols_is', 'goog.i18n.DateTimeSymbols_it', 'goog.i18n.DateTimeSymbols_iw', 'goog.i18n.DateTimeSymbols_ja', 'goog.i18n.DateTimeSymbols_ka', 'goog.i18n.DateTimeSymbols_kk', 'goog.i18n.DateTimeSymbols_km', 'goog.i18n.DateTimeSymbols_kn', 'goog.i18n.DateTimeSymbols_ko', 'goog.i18n.DateTimeSymbols_ky', 'goog.i18n.DateTimeSymbols_ln', 'goog.i18n.DateTimeSymbols_lo', 'goog.i18n.DateTimeSymbols_lt', 'goog.i18n.DateTimeSymbols_lv', 'goog.i18n.DateTimeSymbols_mk', 'goog.i18n.DateTimeSymbols_ml', 'goog.i18n.DateTimeSymbols_mn', 'goog.i18n.DateTimeSymbols_mr', 'goog.i18n.DateTimeSymbols_ms', 'goog.i18n.DateTimeSymbols_mt', 'goog.i18n.DateTimeSymbols_my', 'goog.i18n.DateTimeSymbols_nb', 'goog.i18n.DateTimeSymbols_ne', 'goog.i18n.DateTimeSymbols_nl', 'goog.i18n.DateTimeSymbols_no', 'goog.i18n.DateTimeSymbols_no_NO', 'goog.i18n.DateTimeSymbols_or', 'goog.i18n.DateTimeSymbols_pa', 'goog.i18n.DateTimeSymbols_pl', 'goog.i18n.DateTimeSymbols_pt', 'goog.i18n.DateTimeSymbols_pt_BR', 'goog.i18n.DateTimeSymbols_pt_PT', 'goog.i18n.DateTimeSymbols_ro', 'goog.i18n.DateTimeSymbols_ru', 'goog.i18n.DateTimeSymbols_si', 'goog.i18n.DateTimeSymbols_sk', 'goog.i18n.DateTimeSymbols_sl', 'goog.i18n.DateTimeSymbols_sq', 'goog.i18n.DateTimeSymbols_sr', 'goog.i18n.DateTimeSymbols_sv', 'goog.i18n.DateTimeSymbols_sw', 'goog.i18n.DateTimeSymbols_ta', 'goog.i18n.DateTimeSymbols_te', 'goog.i18n.DateTimeSymbols_th', 'goog.i18n.DateTimeSymbols_tl', 'goog.i18n.DateTimeSymbols_tr', 'goog.i18n.DateTimeSymbols_uk', 'goog.i18n.DateTimeSymbols_ur', 'goog.i18n.DateTimeSymbols_uz', 'goog.i18n.DateTimeSymbols_vi', 'goog.i18n.DateTimeSymbols_zh', 'goog.i18n.DateTimeSymbols_zh_CN', 'goog.i18n.DateTimeSymbols_zh_HK', 'goog.i18n.DateTimeSymbols_zh_TW', 'goog.i18n.DateTimeSymbols_zu'], [], false);
goog.addDependency('i18n/datetimesymbolsext.js', ['goog.i18n.DateTimeSymbolsExt', 'goog.i18n.DateTimeSymbols_aa', 'goog.i18n.DateTimeSymbols_aa_DJ', 'goog.i18n.DateTimeSymbols_aa_ER', 'goog.i18n.DateTimeSymbols_aa_ET', 'goog.i18n.DateTimeSymbols_af_NA', 'goog.i18n.DateTimeSymbols_af_ZA', 'goog.i18n.DateTimeSymbols_agq', 'goog.i18n.DateTimeSymbols_agq_CM', 'goog.i18n.DateTimeSymbols_ak', 'goog.i18n.DateTimeSymbols_ak_GH', 'goog.i18n.DateTimeSymbols_am_ET', 'goog.i18n.DateTimeSymbols_ar_001', 'goog.i18n.DateTimeSymbols_ar_AE', 'goog.i18n.DateTimeSymbols_ar_BH', 'goog.i18n.DateTimeSymbols_ar_DJ', 'goog.i18n.DateTimeSymbols_ar_DZ', 'goog.i18n.DateTimeSymbols_ar_EG', 'goog.i18n.DateTimeSymbols_ar_EH', 'goog.i18n.DateTimeSymbols_ar_ER', 'goog.i18n.DateTimeSymbols_ar_IL', 'goog.i18n.DateTimeSymbols_ar_IQ', 'goog.i18n.DateTimeSymbols_ar_JO', 'goog.i18n.DateTimeSymbols_ar_KM', 'goog.i18n.DateTimeSymbols_ar_KW', 'goog.i18n.DateTimeSymbols_ar_LB', 'goog.i18n.DateTimeSymbols_ar_LY', 'goog.i18n.DateTimeSymbols_ar_MA', 'goog.i18n.DateTimeSymbols_ar_MR', 'goog.i18n.DateTimeSymbols_ar_OM', 'goog.i18n.DateTimeSymbols_ar_PS', 'goog.i18n.DateTimeSymbols_ar_QA', 'goog.i18n.DateTimeSymbols_ar_SA', 'goog.i18n.DateTimeSymbols_ar_SD', 'goog.i18n.DateTimeSymbols_ar_SO', 'goog.i18n.DateTimeSymbols_ar_SS', 'goog.i18n.DateTimeSymbols_ar_SY', 'goog.i18n.DateTimeSymbols_ar_TD', 'goog.i18n.DateTimeSymbols_ar_TN', 'goog.i18n.DateTimeSymbols_ar_YE', 'goog.i18n.DateTimeSymbols_as', 'goog.i18n.DateTimeSymbols_as_IN', 'goog.i18n.DateTimeSymbols_asa', 'goog.i18n.DateTimeSymbols_asa_TZ', 'goog.i18n.DateTimeSymbols_ast', 'goog.i18n.DateTimeSymbols_ast_ES', 'goog.i18n.DateTimeSymbols_az_Cyrl', 'goog.i18n.DateTimeSymbols_az_Cyrl_AZ', 'goog.i18n.DateTimeSymbols_az_Latn', 'goog.i18n.DateTimeSymbols_az_Latn_AZ', 'goog.i18n.DateTimeSymbols_bas', 'goog.i18n.DateTimeSymbols_bas_CM', 'goog.i18n.DateTimeSymbols_be', 'goog.i18n.DateTimeSymbols_be_BY', 'goog.i18n.DateTimeSymbols_bem', 'goog.i18n.DateTimeSymbols_bem_ZM', 'goog.i18n.DateTimeSymbols_bez', 'goog.i18n.DateTimeSymbols_bez_TZ', 'goog.i18n.DateTimeSymbols_bg_BG', 'goog.i18n.DateTimeSymbols_bm', 'goog.i18n.DateTimeSymbols_bm_ML', 'goog.i18n.DateTimeSymbols_bn_BD', 'goog.i18n.DateTimeSymbols_bn_IN', 'goog.i18n.DateTimeSymbols_bo', 'goog.i18n.DateTimeSymbols_bo_CN', 'goog.i18n.DateTimeSymbols_bo_IN', 'goog.i18n.DateTimeSymbols_br_FR', 'goog.i18n.DateTimeSymbols_brx', 'goog.i18n.DateTimeSymbols_brx_IN', 'goog.i18n.DateTimeSymbols_bs', 'goog.i18n.DateTimeSymbols_bs_Cyrl', 'goog.i18n.DateTimeSymbols_bs_Cyrl_BA', 'goog.i18n.DateTimeSymbols_bs_Latn', 'goog.i18n.DateTimeSymbols_bs_Latn_BA', 'goog.i18n.DateTimeSymbols_byn', 'goog.i18n.DateTimeSymbols_byn_ER', 'goog.i18n.DateTimeSymbols_ca_AD', 'goog.i18n.DateTimeSymbols_ca_ES', 'goog.i18n.DateTimeSymbols_ca_ES_VALENCIA', 'goog.i18n.DateTimeSymbols_ca_FR', 'goog.i18n.DateTimeSymbols_ca_IT', 'goog.i18n.DateTimeSymbols_cgg', 'goog.i18n.DateTimeSymbols_cgg_UG', 'goog.i18n.DateTimeSymbols_chr_US', 'goog.i18n.DateTimeSymbols_ckb', 'goog.i18n.DateTimeSymbols_ckb_Arab', 'goog.i18n.DateTimeSymbols_ckb_Arab_IQ', 'goog.i18n.DateTimeSymbols_ckb_Arab_IR', 'goog.i18n.DateTimeSymbols_ckb_IQ', 'goog.i18n.DateTimeSymbols_ckb_IR', 'goog.i18n.DateTimeSymbols_ckb_Latn', 'goog.i18n.DateTimeSymbols_ckb_Latn_IQ', 'goog.i18n.DateTimeSymbols_cs_CZ', 'goog.i18n.DateTimeSymbols_cy_GB', 'goog.i18n.DateTimeSymbols_da_DK', 'goog.i18n.DateTimeSymbols_da_GL', 'goog.i18n.DateTimeSymbols_dav', 'goog.i18n.DateTimeSymbols_dav_KE', 'goog.i18n.DateTimeSymbols_de_BE', 'goog.i18n.DateTimeSymbols_de_DE', 'goog.i18n.DateTimeSymbols_de_LI', 'goog.i18n.DateTimeSymbols_de_LU', 'goog.i18n.DateTimeSymbols_dje', 'goog.i18n.DateTimeSymbols_dje_NE', 'goog.i18n.DateTimeSymbols_dua', 'goog.i18n.DateTimeSymbols_dua_CM', 'goog.i18n.DateTimeSymbols_dyo', 'goog.i18n.DateTimeSymbols_dyo_SN', 'goog.i18n.DateTimeSymbols_dz', 'goog.i18n.DateTimeSymbols_dz_BT', 'goog.i18n.DateTimeSymbols_ebu', 'goog.i18n.DateTimeSymbols_ebu_KE', 'goog.i18n.DateTimeSymbols_ee', 'goog.i18n.DateTimeSymbols_ee_GH', 'goog.i18n.DateTimeSymbols_ee_TG', 'goog.i18n.DateTimeSymbols_el_CY', 'goog.i18n.DateTimeSymbols_el_GR', 'goog.i18n.DateTimeSymbols_en_001', 'goog.i18n.DateTimeSymbols_en_150', 'goog.i18n.DateTimeSymbols_en_AG', 'goog.i18n.DateTimeSymbols_en_AI', 'goog.i18n.DateTimeSymbols_en_AS', 'goog.i18n.DateTimeSymbols_en_BB', 'goog.i18n.DateTimeSymbols_en_BE', 'goog.i18n.DateTimeSymbols_en_BM', 'goog.i18n.DateTimeSymbols_en_BS', 'goog.i18n.DateTimeSymbols_en_BW', 'goog.i18n.DateTimeSymbols_en_BZ', 'goog.i18n.DateTimeSymbols_en_CA', 'goog.i18n.DateTimeSymbols_en_CC', 'goog.i18n.DateTimeSymbols_en_CK', 'goog.i18n.DateTimeSymbols_en_CM', 'goog.i18n.DateTimeSymbols_en_CX', 'goog.i18n.DateTimeSymbols_en_DG', 'goog.i18n.DateTimeSymbols_en_DM', 'goog.i18n.DateTimeSymbols_en_ER', 'goog.i18n.DateTimeSymbols_en_FJ', 'goog.i18n.DateTimeSymbols_en_FK', 'goog.i18n.DateTimeSymbols_en_FM', 'goog.i18n.DateTimeSymbols_en_GD', 'goog.i18n.DateTimeSymbols_en_GG', 'goog.i18n.DateTimeSymbols_en_GH', 'goog.i18n.DateTimeSymbols_en_GI', 'goog.i18n.DateTimeSymbols_en_GM', 'goog.i18n.DateTimeSymbols_en_GU', 'goog.i18n.DateTimeSymbols_en_GY', 'goog.i18n.DateTimeSymbols_en_HK', 'goog.i18n.DateTimeSymbols_en_IM', 'goog.i18n.DateTimeSymbols_en_IO', 'goog.i18n.DateTimeSymbols_en_JE', 'goog.i18n.DateTimeSymbols_en_JM', 'goog.i18n.DateTimeSymbols_en_KE', 'goog.i18n.DateTimeSymbols_en_KI', 'goog.i18n.DateTimeSymbols_en_KN', 'goog.i18n.DateTimeSymbols_en_KY', 'goog.i18n.DateTimeSymbols_en_LC', 'goog.i18n.DateTimeSymbols_en_LR', 'goog.i18n.DateTimeSymbols_en_LS', 'goog.i18n.DateTimeSymbols_en_MG', 'goog.i18n.DateTimeSymbols_en_MH', 'goog.i18n.DateTimeSymbols_en_MO', 'goog.i18n.DateTimeSymbols_en_MP', 'goog.i18n.DateTimeSymbols_en_MS', 'goog.i18n.DateTimeSymbols_en_MT', 'goog.i18n.DateTimeSymbols_en_MU', 'goog.i18n.DateTimeSymbols_en_MW', 'goog.i18n.DateTimeSymbols_en_NA', 'goog.i18n.DateTimeSymbols_en_NF', 'goog.i18n.DateTimeSymbols_en_NG', 'goog.i18n.DateTimeSymbols_en_NR', 'goog.i18n.DateTimeSymbols_en_NU', 'goog.i18n.DateTimeSymbols_en_NZ', 'goog.i18n.DateTimeSymbols_en_PG', 'goog.i18n.DateTimeSymbols_en_PH', 'goog.i18n.DateTimeSymbols_en_PK', 'goog.i18n.DateTimeSymbols_en_PN', 'goog.i18n.DateTimeSymbols_en_PR', 'goog.i18n.DateTimeSymbols_en_PW', 'goog.i18n.DateTimeSymbols_en_RW', 'goog.i18n.DateTimeSymbols_en_SB', 'goog.i18n.DateTimeSymbols_en_SC', 'goog.i18n.DateTimeSymbols_en_SD', 'goog.i18n.DateTimeSymbols_en_SH', 'goog.i18n.DateTimeSymbols_en_SL', 'goog.i18n.DateTimeSymbols_en_SS', 'goog.i18n.DateTimeSymbols_en_SX', 'goog.i18n.DateTimeSymbols_en_SZ', 'goog.i18n.DateTimeSymbols_en_TC', 'goog.i18n.DateTimeSymbols_en_TK', 'goog.i18n.DateTimeSymbols_en_TO', 'goog.i18n.DateTimeSymbols_en_TT', 'goog.i18n.DateTimeSymbols_en_TV', 'goog.i18n.DateTimeSymbols_en_TZ', 'goog.i18n.DateTimeSymbols_en_UG', 'goog.i18n.DateTimeSymbols_en_UM', 'goog.i18n.DateTimeSymbols_en_VC', 'goog.i18n.DateTimeSymbols_en_VG', 'goog.i18n.DateTimeSymbols_en_VI', 'goog.i18n.DateTimeSymbols_en_VU', 'goog.i18n.DateTimeSymbols_en_WS', 'goog.i18n.DateTimeSymbols_en_ZM', 'goog.i18n.DateTimeSymbols_en_ZW', 'goog.i18n.DateTimeSymbols_eo', 'goog.i18n.DateTimeSymbols_eo_001', 'goog.i18n.DateTimeSymbols_es_AR', 'goog.i18n.DateTimeSymbols_es_BO', 'goog.i18n.DateTimeSymbols_es_CL', 'goog.i18n.DateTimeSymbols_es_CO', 'goog.i18n.DateTimeSymbols_es_CR', 'goog.i18n.DateTimeSymbols_es_CU', 'goog.i18n.DateTimeSymbols_es_DO', 'goog.i18n.DateTimeSymbols_es_EA', 'goog.i18n.DateTimeSymbols_es_EC', 'goog.i18n.DateTimeSymbols_es_GQ', 'goog.i18n.DateTimeSymbols_es_GT', 'goog.i18n.DateTimeSymbols_es_HN', 'goog.i18n.DateTimeSymbols_es_IC', 'goog.i18n.DateTimeSymbols_es_MX', 'goog.i18n.DateTimeSymbols_es_NI', 'goog.i18n.DateTimeSymbols_es_PA', 'goog.i18n.DateTimeSymbols_es_PE', 'goog.i18n.DateTimeSymbols_es_PH', 'goog.i18n.DateTimeSymbols_es_PR', 'goog.i18n.DateTimeSymbols_es_PY', 'goog.i18n.DateTimeSymbols_es_SV', 'goog.i18n.DateTimeSymbols_es_US', 'goog.i18n.DateTimeSymbols_es_UY', 'goog.i18n.DateTimeSymbols_es_VE', 'goog.i18n.DateTimeSymbols_et_EE', 'goog.i18n.DateTimeSymbols_eu_ES', 'goog.i18n.DateTimeSymbols_ewo', 'goog.i18n.DateTimeSymbols_ewo_CM', 'goog.i18n.DateTimeSymbols_fa_AF', 'goog.i18n.DateTimeSymbols_fa_IR', 'goog.i18n.DateTimeSymbols_ff', 'goog.i18n.DateTimeSymbols_ff_CM', 'goog.i18n.DateTimeSymbols_ff_GN', 'goog.i18n.DateTimeSymbols_ff_MR', 'goog.i18n.DateTimeSymbols_ff_SN', 'goog.i18n.DateTimeSymbols_fi_FI', 'goog.i18n.DateTimeSymbols_fil_PH', 'goog.i18n.DateTimeSymbols_fo', 'goog.i18n.DateTimeSymbols_fo_FO', 'goog.i18n.DateTimeSymbols_fr_BE', 'goog.i18n.DateTimeSymbols_fr_BF', 'goog.i18n.DateTimeSymbols_fr_BI', 'goog.i18n.DateTimeSymbols_fr_BJ', 'goog.i18n.DateTimeSymbols_fr_BL', 'goog.i18n.DateTimeSymbols_fr_CD', 'goog.i18n.DateTimeSymbols_fr_CF', 'goog.i18n.DateTimeSymbols_fr_CG', 'goog.i18n.DateTimeSymbols_fr_CH', 'goog.i18n.DateTimeSymbols_fr_CI', 'goog.i18n.DateTimeSymbols_fr_CM', 'goog.i18n.DateTimeSymbols_fr_DJ', 'goog.i18n.DateTimeSymbols_fr_DZ', 'goog.i18n.DateTimeSymbols_fr_FR', 'goog.i18n.DateTimeSymbols_fr_GA', 'goog.i18n.DateTimeSymbols_fr_GF', 'goog.i18n.DateTimeSymbols_fr_GN', 'goog.i18n.DateTimeSymbols_fr_GP', 'goog.i18n.DateTimeSymbols_fr_GQ', 'goog.i18n.DateTimeSymbols_fr_HT', 'goog.i18n.DateTimeSymbols_fr_KM', 'goog.i18n.DateTimeSymbols_fr_LU', 'goog.i18n.DateTimeSymbols_fr_MA', 'goog.i18n.DateTimeSymbols_fr_MC', 'goog.i18n.DateTimeSymbols_fr_MF', 'goog.i18n.DateTimeSymbols_fr_MG', 'goog.i18n.DateTimeSymbols_fr_ML', 'goog.i18n.DateTimeSymbols_fr_MQ', 'goog.i18n.DateTimeSymbols_fr_MR', 'goog.i18n.DateTimeSymbols_fr_MU', 'goog.i18n.DateTimeSymbols_fr_NC', 'goog.i18n.DateTimeSymbols_fr_NE', 'goog.i18n.DateTimeSymbols_fr_PF', 'goog.i18n.DateTimeSymbols_fr_PM', 'goog.i18n.DateTimeSymbols_fr_RE', 'goog.i18n.DateTimeSymbols_fr_RW', 'goog.i18n.DateTimeSymbols_fr_SC', 'goog.i18n.DateTimeSymbols_fr_SN', 'goog.i18n.DateTimeSymbols_fr_SY', 'goog.i18n.DateTimeSymbols_fr_TD', 'goog.i18n.DateTimeSymbols_fr_TG', 'goog.i18n.DateTimeSymbols_fr_TN', 'goog.i18n.DateTimeSymbols_fr_VU', 'goog.i18n.DateTimeSymbols_fr_WF', 'goog.i18n.DateTimeSymbols_fr_YT', 'goog.i18n.DateTimeSymbols_fur', 'goog.i18n.DateTimeSymbols_fur_IT', 'goog.i18n.DateTimeSymbols_fy', 'goog.i18n.DateTimeSymbols_fy_NL', 'goog.i18n.DateTimeSymbols_ga_IE', 'goog.i18n.DateTimeSymbols_gd', 'goog.i18n.DateTimeSymbols_gd_GB', 'goog.i18n.DateTimeSymbols_gl_ES', 'goog.i18n.DateTimeSymbols_gsw_CH', 'goog.i18n.DateTimeSymbols_gsw_LI', 'goog.i18n.DateTimeSymbols_gu_IN', 'goog.i18n.DateTimeSymbols_guz', 'goog.i18n.DateTimeSymbols_guz_KE', 'goog.i18n.DateTimeSymbols_gv', 'goog.i18n.DateTimeSymbols_gv_IM', 'goog.i18n.DateTimeSymbols_ha', 'goog.i18n.DateTimeSymbols_ha_Latn', 'goog.i18n.DateTimeSymbols_ha_Latn_GH', 'goog.i18n.DateTimeSymbols_ha_Latn_NE', 'goog.i18n.DateTimeSymbols_ha_Latn_NG', 'goog.i18n.DateTimeSymbols_haw_US', 'goog.i18n.DateTimeSymbols_he_IL', 'goog.i18n.DateTimeSymbols_hi_IN', 'goog.i18n.DateTimeSymbols_hr_BA', 'goog.i18n.DateTimeSymbols_hr_HR', 'goog.i18n.DateTimeSymbols_hu_HU', 'goog.i18n.DateTimeSymbols_hy_AM', 'goog.i18n.DateTimeSymbols_ia', 'goog.i18n.DateTimeSymbols_ia_FR', 'goog.i18n.DateTimeSymbols_id_ID', 'goog.i18n.DateTimeSymbols_ig', 'goog.i18n.DateTimeSymbols_ig_NG', 'goog.i18n.DateTimeSymbols_ii', 'goog.i18n.DateTimeSymbols_ii_CN', 'goog.i18n.DateTimeSymbols_is_IS', 'goog.i18n.DateTimeSymbols_it_CH', 'goog.i18n.DateTimeSymbols_it_IT', 'goog.i18n.DateTimeSymbols_it_SM', 'goog.i18n.DateTimeSymbols_ja_JP', 'goog.i18n.DateTimeSymbols_jgo', 'goog.i18n.DateTimeSymbols_jgo_CM', 'goog.i18n.DateTimeSymbols_jmc', 'goog.i18n.DateTimeSymbols_jmc_TZ', 'goog.i18n.DateTimeSymbols_ka_GE', 'goog.i18n.DateTimeSymbols_kab', 'goog.i18n.DateTimeSymbols_kab_DZ', 'goog.i18n.DateTimeSymbols_kam', 'goog.i18n.DateTimeSymbols_kam_KE', 'goog.i18n.DateTimeSymbols_kde', 'goog.i18n.DateTimeSymbols_kde_TZ', 'goog.i18n.DateTimeSymbols_kea', 'goog.i18n.DateTimeSymbols_kea_CV', 'goog.i18n.DateTimeSymbols_khq', 'goog.i18n.DateTimeSymbols_khq_ML', 'goog.i18n.DateTimeSymbols_ki', 'goog.i18n.DateTimeSymbols_ki_KE', 'goog.i18n.DateTimeSymbols_kk_Cyrl', 'goog.i18n.DateTimeSymbols_kk_Cyrl_KZ', 'goog.i18n.DateTimeSymbols_kkj', 'goog.i18n.DateTimeSymbols_kkj_CM', 'goog.i18n.DateTimeSymbols_kl', 'goog.i18n.DateTimeSymbols_kl_GL', 'goog.i18n.DateTimeSymbols_kln', 'goog.i18n.DateTimeSymbols_kln_KE', 'goog.i18n.DateTimeSymbols_km_KH', 'goog.i18n.DateTimeSymbols_kn_IN', 'goog.i18n.DateTimeSymbols_ko_KP', 'goog.i18n.DateTimeSymbols_ko_KR', 'goog.i18n.DateTimeSymbols_kok', 'goog.i18n.DateTimeSymbols_kok_IN', 'goog.i18n.DateTimeSymbols_ks', 'goog.i18n.DateTimeSymbols_ks_Arab', 'goog.i18n.DateTimeSymbols_ks_Arab_IN', 'goog.i18n.DateTimeSymbols_ksb', 'goog.i18n.DateTimeSymbols_ksb_TZ', 'goog.i18n.DateTimeSymbols_ksf', 'goog.i18n.DateTimeSymbols_ksf_CM', 'goog.i18n.DateTimeSymbols_ksh', 'goog.i18n.DateTimeSymbols_ksh_DE', 'goog.i18n.DateTimeSymbols_kw', 'goog.i18n.DateTimeSymbols_kw_GB', 'goog.i18n.DateTimeSymbols_ky_Cyrl', 'goog.i18n.DateTimeSymbols_ky_Cyrl_KG', 'goog.i18n.DateTimeSymbols_lag', 'goog.i18n.DateTimeSymbols_lag_TZ', 'goog.i18n.DateTimeSymbols_lg', 'goog.i18n.DateTimeSymbols_lg_UG', 'goog.i18n.DateTimeSymbols_lkt', 'goog.i18n.DateTimeSymbols_lkt_US', 'goog.i18n.DateTimeSymbols_ln_AO', 'goog.i18n.DateTimeSymbols_ln_CD', 'goog.i18n.DateTimeSymbols_ln_CF', 'goog.i18n.DateTimeSymbols_ln_CG', 'goog.i18n.DateTimeSymbols_lo_LA', 'goog.i18n.DateTimeSymbols_lt_LT', 'goog.i18n.DateTimeSymbols_lu', 'goog.i18n.DateTimeSymbols_lu_CD', 'goog.i18n.DateTimeSymbols_luo', 'goog.i18n.DateTimeSymbols_luo_KE', 'goog.i18n.DateTimeSymbols_luy', 'goog.i18n.DateTimeSymbols_luy_KE', 'goog.i18n.DateTimeSymbols_lv_LV', 'goog.i18n.DateTimeSymbols_mas', 'goog.i18n.DateTimeSymbols_mas_KE', 'goog.i18n.DateTimeSymbols_mas_TZ', 'goog.i18n.DateTimeSymbols_mer', 'goog.i18n.DateTimeSymbols_mer_KE', 'goog.i18n.DateTimeSymbols_mfe', 'goog.i18n.DateTimeSymbols_mfe_MU', 'goog.i18n.DateTimeSymbols_mg', 'goog.i18n.DateTimeSymbols_mg_MG', 'goog.i18n.DateTimeSymbols_mgh', 'goog.i18n.DateTimeSymbols_mgh_MZ', 'goog.i18n.DateTimeSymbols_mgo', 'goog.i18n.DateTimeSymbols_mgo_CM', 'goog.i18n.DateTimeSymbols_mk_MK', 'goog.i18n.DateTimeSymbols_ml_IN', 'goog.i18n.DateTimeSymbols_mn_Cyrl', 'goog.i18n.DateTimeSymbols_mn_Cyrl_MN', 'goog.i18n.DateTimeSymbols_mr_IN', 'goog.i18n.DateTimeSymbols_ms_Latn', 'goog.i18n.DateTimeSymbols_ms_Latn_BN', 'goog.i18n.DateTimeSymbols_ms_Latn_MY', 'goog.i18n.DateTimeSymbols_ms_Latn_SG', 'goog.i18n.DateTimeSymbols_mt_MT', 'goog.i18n.DateTimeSymbols_mua', 'goog.i18n.DateTimeSymbols_mua_CM', 'goog.i18n.DateTimeSymbols_my_MM', 'goog.i18n.DateTimeSymbols_naq', 'goog.i18n.DateTimeSymbols_naq_NA', 'goog.i18n.DateTimeSymbols_nb_NO', 'goog.i18n.DateTimeSymbols_nb_SJ', 'goog.i18n.DateTimeSymbols_nd', 'goog.i18n.DateTimeSymbols_nd_ZW', 'goog.i18n.DateTimeSymbols_ne_IN', 'goog.i18n.DateTimeSymbols_ne_NP', 'goog.i18n.DateTimeSymbols_nl_AW', 'goog.i18n.DateTimeSymbols_nl_BE', 'goog.i18n.DateTimeSymbols_nl_BQ', 'goog.i18n.DateTimeSymbols_nl_CW', 'goog.i18n.DateTimeSymbols_nl_NL', 'goog.i18n.DateTimeSymbols_nl_SR', 'goog.i18n.DateTimeSymbols_nl_SX', 'goog.i18n.DateTimeSymbols_nmg', 'goog.i18n.DateTimeSymbols_nmg_CM', 'goog.i18n.DateTimeSymbols_nn', 'goog.i18n.DateTimeSymbols_nn_NO', 'goog.i18n.DateTimeSymbols_nnh', 'goog.i18n.DateTimeSymbols_nnh_CM', 'goog.i18n.DateTimeSymbols_nr', 'goog.i18n.DateTimeSymbols_nr_ZA', 'goog.i18n.DateTimeSymbols_nso', 'goog.i18n.DateTimeSymbols_nso_ZA', 'goog.i18n.DateTimeSymbols_nus', 'goog.i18n.DateTimeSymbols_nus_SD', 'goog.i18n.DateTimeSymbols_nyn', 'goog.i18n.DateTimeSymbols_nyn_UG', 'goog.i18n.DateTimeSymbols_om', 'goog.i18n.DateTimeSymbols_om_ET', 'goog.i18n.DateTimeSymbols_om_KE', 'goog.i18n.DateTimeSymbols_or_IN', 'goog.i18n.DateTimeSymbols_os', 'goog.i18n.DateTimeSymbols_os_GE', 'goog.i18n.DateTimeSymbols_os_RU', 'goog.i18n.DateTimeSymbols_pa_Arab', 'goog.i18n.DateTimeSymbols_pa_Arab_PK', 'goog.i18n.DateTimeSymbols_pa_Guru', 'goog.i18n.DateTimeSymbols_pa_Guru_IN', 'goog.i18n.DateTimeSymbols_pl_PL', 'goog.i18n.DateTimeSymbols_ps', 'goog.i18n.DateTimeSymbols_ps_AF', 'goog.i18n.DateTimeSymbols_pt_AO', 'goog.i18n.DateTimeSymbols_pt_CV', 'goog.i18n.DateTimeSymbols_pt_GW', 'goog.i18n.DateTimeSymbols_pt_MO', 'goog.i18n.DateTimeSymbols_pt_MZ', 'goog.i18n.DateTimeSymbols_pt_ST', 'goog.i18n.DateTimeSymbols_pt_TL', 'goog.i18n.DateTimeSymbols_rm', 'goog.i18n.DateTimeSymbols_rm_CH', 'goog.i18n.DateTimeSymbols_rn', 'goog.i18n.DateTimeSymbols_rn_BI', 'goog.i18n.DateTimeSymbols_ro_MD', 'goog.i18n.DateTimeSymbols_ro_RO', 'goog.i18n.DateTimeSymbols_rof', 'goog.i18n.DateTimeSymbols_rof_TZ', 'goog.i18n.DateTimeSymbols_ru_BY', 'goog.i18n.DateTimeSymbols_ru_KG', 'goog.i18n.DateTimeSymbols_ru_KZ', 'goog.i18n.DateTimeSymbols_ru_MD', 'goog.i18n.DateTimeSymbols_ru_RU', 'goog.i18n.DateTimeSymbols_ru_UA', 'goog.i18n.DateTimeSymbols_rw', 'goog.i18n.DateTimeSymbols_rw_RW', 'goog.i18n.DateTimeSymbols_rwk', 'goog.i18n.DateTimeSymbols_rwk_TZ', 'goog.i18n.DateTimeSymbols_sah', 'goog.i18n.DateTimeSymbols_sah_RU', 'goog.i18n.DateTimeSymbols_saq', 'goog.i18n.DateTimeSymbols_saq_KE', 'goog.i18n.DateTimeSymbols_sbp', 'goog.i18n.DateTimeSymbols_sbp_TZ', 'goog.i18n.DateTimeSymbols_se', 'goog.i18n.DateTimeSymbols_se_FI', 'goog.i18n.DateTimeSymbols_se_NO', 'goog.i18n.DateTimeSymbols_seh', 'goog.i18n.DateTimeSymbols_seh_MZ', 'goog.i18n.DateTimeSymbols_ses', 'goog.i18n.DateTimeSymbols_ses_ML', 'goog.i18n.DateTimeSymbols_sg', 'goog.i18n.DateTimeSymbols_sg_CF', 'goog.i18n.DateTimeSymbols_shi', 'goog.i18n.DateTimeSymbols_shi_Latn', 'goog.i18n.DateTimeSymbols_shi_Latn_MA', 'goog.i18n.DateTimeSymbols_shi_Tfng', 'goog.i18n.DateTimeSymbols_shi_Tfng_MA', 'goog.i18n.DateTimeSymbols_si_LK', 'goog.i18n.DateTimeSymbols_sk_SK', 'goog.i18n.DateTimeSymbols_sl_SI', 'goog.i18n.DateTimeSymbols_sn', 'goog.i18n.DateTimeSymbols_sn_ZW', 'goog.i18n.DateTimeSymbols_so', 'goog.i18n.DateTimeSymbols_so_DJ', 'goog.i18n.DateTimeSymbols_so_ET', 'goog.i18n.DateTimeSymbols_so_KE', 'goog.i18n.DateTimeSymbols_so_SO', 'goog.i18n.DateTimeSymbols_sq_AL', 'goog.i18n.DateTimeSymbols_sq_MK', 'goog.i18n.DateTimeSymbols_sq_XK', 'goog.i18n.DateTimeSymbols_sr_Cyrl', 'goog.i18n.DateTimeSymbols_sr_Cyrl_BA', 'goog.i18n.DateTimeSymbols_sr_Cyrl_ME', 'goog.i18n.DateTimeSymbols_sr_Cyrl_RS', 'goog.i18n.DateTimeSymbols_sr_Cyrl_XK', 'goog.i18n.DateTimeSymbols_sr_Latn', 'goog.i18n.DateTimeSymbols_sr_Latn_BA', 'goog.i18n.DateTimeSymbols_sr_Latn_ME', 'goog.i18n.DateTimeSymbols_sr_Latn_RS', 'goog.i18n.DateTimeSymbols_sr_Latn_XK', 'goog.i18n.DateTimeSymbols_ss', 'goog.i18n.DateTimeSymbols_ss_SZ', 'goog.i18n.DateTimeSymbols_ss_ZA', 'goog.i18n.DateTimeSymbols_ssy', 'goog.i18n.DateTimeSymbols_ssy_ER', 'goog.i18n.DateTimeSymbols_st', 'goog.i18n.DateTimeSymbols_st_LS', 'goog.i18n.DateTimeSymbols_st_ZA', 'goog.i18n.DateTimeSymbols_sv_AX', 'goog.i18n.DateTimeSymbols_sv_FI', 'goog.i18n.DateTimeSymbols_sv_SE', 'goog.i18n.DateTimeSymbols_sw_KE', 'goog.i18n.DateTimeSymbols_sw_TZ', 'goog.i18n.DateTimeSymbols_sw_UG', 'goog.i18n.DateTimeSymbols_swc', 'goog.i18n.DateTimeSymbols_swc_CD', 'goog.i18n.DateTimeSymbols_ta_IN', 'goog.i18n.DateTimeSymbols_ta_LK', 'goog.i18n.DateTimeSymbols_ta_MY', 'goog.i18n.DateTimeSymbols_ta_SG', 'goog.i18n.DateTimeSymbols_te_IN', 'goog.i18n.DateTimeSymbols_teo', 'goog.i18n.DateTimeSymbols_teo_KE', 'goog.i18n.DateTimeSymbols_teo_UG', 'goog.i18n.DateTimeSymbols_tg', 'goog.i18n.DateTimeSymbols_tg_Cyrl', 'goog.i18n.DateTimeSymbols_tg_Cyrl_TJ', 'goog.i18n.DateTimeSymbols_th_TH', 'goog.i18n.DateTimeSymbols_ti', 'goog.i18n.DateTimeSymbols_ti_ER', 'goog.i18n.DateTimeSymbols_ti_ET', 'goog.i18n.DateTimeSymbols_tig', 'goog.i18n.DateTimeSymbols_tig_ER', 'goog.i18n.DateTimeSymbols_tn', 'goog.i18n.DateTimeSymbols_tn_BW', 'goog.i18n.DateTimeSymbols_tn_ZA', 'goog.i18n.DateTimeSymbols_to', 'goog.i18n.DateTimeSymbols_to_TO', 'goog.i18n.DateTimeSymbols_tr_CY', 'goog.i18n.DateTimeSymbols_tr_TR', 'goog.i18n.DateTimeSymbols_ts', 'goog.i18n.DateTimeSymbols_ts_ZA', 'goog.i18n.DateTimeSymbols_twq', 'goog.i18n.DateTimeSymbols_twq_NE', 'goog.i18n.DateTimeSymbols_tzm', 'goog.i18n.DateTimeSymbols_tzm_Latn', 'goog.i18n.DateTimeSymbols_tzm_Latn_MA', 'goog.i18n.DateTimeSymbols_ug', 'goog.i18n.DateTimeSymbols_ug_Arab', 'goog.i18n.DateTimeSymbols_ug_Arab_CN', 'goog.i18n.DateTimeSymbols_uk_UA', 'goog.i18n.DateTimeSymbols_ur_IN', 'goog.i18n.DateTimeSymbols_ur_PK', 'goog.i18n.DateTimeSymbols_uz_Arab', 'goog.i18n.DateTimeSymbols_uz_Arab_AF', 'goog.i18n.DateTimeSymbols_uz_Cyrl', 'goog.i18n.DateTimeSymbols_uz_Cyrl_UZ', 'goog.i18n.DateTimeSymbols_uz_Latn', 'goog.i18n.DateTimeSymbols_uz_Latn_UZ', 'goog.i18n.DateTimeSymbols_vai', 'goog.i18n.DateTimeSymbols_vai_Latn', 'goog.i18n.DateTimeSymbols_vai_Latn_LR', 'goog.i18n.DateTimeSymbols_vai_Vaii', 'goog.i18n.DateTimeSymbols_vai_Vaii_LR', 'goog.i18n.DateTimeSymbols_ve', 'goog.i18n.DateTimeSymbols_ve_ZA', 'goog.i18n.DateTimeSymbols_vi_VN', 'goog.i18n.DateTimeSymbols_vo', 'goog.i18n.DateTimeSymbols_vo_001', 'goog.i18n.DateTimeSymbols_vun', 'goog.i18n.DateTimeSymbols_vun_TZ', 'goog.i18n.DateTimeSymbols_wae', 'goog.i18n.DateTimeSymbols_wae_CH', 'goog.i18n.DateTimeSymbols_wal', 'goog.i18n.DateTimeSymbols_wal_ET', 'goog.i18n.DateTimeSymbols_xh', 'goog.i18n.DateTimeSymbols_xh_ZA', 'goog.i18n.DateTimeSymbols_xog', 'goog.i18n.DateTimeSymbols_xog_UG', 'goog.i18n.DateTimeSymbols_yav', 'goog.i18n.DateTimeSymbols_yav_CM', 'goog.i18n.DateTimeSymbols_yo', 'goog.i18n.DateTimeSymbols_yo_BJ', 'goog.i18n.DateTimeSymbols_yo_NG', 'goog.i18n.DateTimeSymbols_zgh', 'goog.i18n.DateTimeSymbols_zgh_MA', 'goog.i18n.DateTimeSymbols_zh_Hans', 'goog.i18n.DateTimeSymbols_zh_Hans_CN', 'goog.i18n.DateTimeSymbols_zh_Hans_HK', 'goog.i18n.DateTimeSymbols_zh_Hans_MO', 'goog.i18n.DateTimeSymbols_zh_Hans_SG', 'goog.i18n.DateTimeSymbols_zh_Hant', 'goog.i18n.DateTimeSymbols_zh_Hant_HK', 'goog.i18n.DateTimeSymbols_zh_Hant_MO', 'goog.i18n.DateTimeSymbols_zh_Hant_TW', 'goog.i18n.DateTimeSymbols_zu_ZA'], ['goog.i18n.DateTimeSymbols'], false);
goog.addDependency('i18n/graphemebreak.js', ['goog.i18n.GraphemeBreak'], ['goog.structs.InversionMap'], false);
goog.addDependency('i18n/graphemebreak_test.js', ['goog.i18n.GraphemeBreakTest'], ['goog.i18n.GraphemeBreak', 'goog.testing.jsunit'], false);
goog.addDependency('i18n/messageformat.js', ['goog.i18n.MessageFormat'], ['goog.asserts', 'goog.i18n.NumberFormat', 'goog.i18n.ordinalRules', 'goog.i18n.pluralRules'], false);
goog.addDependency('i18n/messageformat_test.js', ['goog.i18n.MessageFormatTest'], ['goog.i18n.MessageFormat', 'goog.i18n.NumberFormatSymbols_hr', 'goog.i18n.pluralRules', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit'], false);
goog.addDependency('i18n/mime.js', ['goog.i18n.mime', 'goog.i18n.mime.encode'], ['goog.array'], false);
goog.addDependency('i18n/mime_test.js', ['goog.i18n.mime.encodeTest'], ['goog.i18n.mime.encode', 'goog.testing.jsunit'], false);
goog.addDependency('i18n/numberformat.js', ['goog.i18n.NumberFormat', 'goog.i18n.NumberFormat.CurrencyStyle', 'goog.i18n.NumberFormat.Format'], ['goog.asserts', 'goog.i18n.CompactNumberFormatSymbols', 'goog.i18n.NumberFormatSymbols', 'goog.i18n.currency', 'goog.math'], false);
goog.addDependency('i18n/numberformat_test.js', ['goog.i18n.NumberFormatTest'], ['goog.i18n.CompactNumberFormatSymbols', 'goog.i18n.CompactNumberFormatSymbols_de', 'goog.i18n.CompactNumberFormatSymbols_en', 'goog.i18n.CompactNumberFormatSymbols_fr', 'goog.i18n.NumberFormat', 'goog.i18n.NumberFormatSymbols', 'goog.i18n.NumberFormatSymbols_de', 'goog.i18n.NumberFormatSymbols_en', 'goog.i18n.NumberFormatSymbols_fr', 'goog.i18n.NumberFormatSymbols_pl', 'goog.i18n.NumberFormatSymbols_ro', 'goog.testing.ExpectedFailures', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.userAgent', 'goog.userAgent.product', 'goog.userAgent.product.isVersion'], false);
goog.addDependency('i18n/numberformatsymbols.js', ['goog.i18n.NumberFormatSymbols', 'goog.i18n.NumberFormatSymbols_af', 'goog.i18n.NumberFormatSymbols_af_ZA', 'goog.i18n.NumberFormatSymbols_am', 'goog.i18n.NumberFormatSymbols_am_ET', 'goog.i18n.NumberFormatSymbols_ar', 'goog.i18n.NumberFormatSymbols_ar_001', 'goog.i18n.NumberFormatSymbols_az', 'goog.i18n.NumberFormatSymbols_az_Cyrl_AZ', 'goog.i18n.NumberFormatSymbols_az_Latn_AZ', 'goog.i18n.NumberFormatSymbols_bg', 'goog.i18n.NumberFormatSymbols_bg_BG', 'goog.i18n.NumberFormatSymbols_bn', 'goog.i18n.NumberFormatSymbols_bn_BD', 'goog.i18n.NumberFormatSymbols_br', 'goog.i18n.NumberFormatSymbols_br_FR', 'goog.i18n.NumberFormatSymbols_ca', 'goog.i18n.NumberFormatSymbols_ca_AD', 'goog.i18n.NumberFormatSymbols_ca_ES', 'goog.i18n.NumberFormatSymbols_ca_ES_VALENCIA', 'goog.i18n.NumberFormatSymbols_ca_FR', 'goog.i18n.NumberFormatSymbols_ca_IT', 'goog.i18n.NumberFormatSymbols_chr', 'goog.i18n.NumberFormatSymbols_chr_US', 'goog.i18n.NumberFormatSymbols_cs', 'goog.i18n.NumberFormatSymbols_cs_CZ', 'goog.i18n.NumberFormatSymbols_cy', 'goog.i18n.NumberFormatSymbols_cy_GB', 'goog.i18n.NumberFormatSymbols_da', 'goog.i18n.NumberFormatSymbols_da_DK', 'goog.i18n.NumberFormatSymbols_da_GL', 'goog.i18n.NumberFormatSymbols_de', 'goog.i18n.NumberFormatSymbols_de_AT', 'goog.i18n.NumberFormatSymbols_de_BE', 'goog.i18n.NumberFormatSymbols_de_CH', 'goog.i18n.NumberFormatSymbols_de_DE', 'goog.i18n.NumberFormatSymbols_de_LU', 'goog.i18n.NumberFormatSymbols_el', 'goog.i18n.NumberFormatSymbols_el_GR', 'goog.i18n.NumberFormatSymbols_en', 'goog.i18n.NumberFormatSymbols_en_001', 'goog.i18n.NumberFormatSymbols_en_AS', 'goog.i18n.NumberFormatSymbols_en_AU', 'goog.i18n.NumberFormatSymbols_en_DG', 'goog.i18n.NumberFormatSymbols_en_FM', 'goog.i18n.NumberFormatSymbols_en_GB', 'goog.i18n.NumberFormatSymbols_en_GU', 'goog.i18n.NumberFormatSymbols_en_IE', 'goog.i18n.NumberFormatSymbols_en_IN', 'goog.i18n.NumberFormatSymbols_en_IO', 'goog.i18n.NumberFormatSymbols_en_MH', 'goog.i18n.NumberFormatSymbols_en_MP', 'goog.i18n.NumberFormatSymbols_en_PR', 'goog.i18n.NumberFormatSymbols_en_PW', 'goog.i18n.NumberFormatSymbols_en_SG', 'goog.i18n.NumberFormatSymbols_en_TC', 'goog.i18n.NumberFormatSymbols_en_UM', 'goog.i18n.NumberFormatSymbols_en_US', 'goog.i18n.NumberFormatSymbols_en_VG', 'goog.i18n.NumberFormatSymbols_en_VI', 'goog.i18n.NumberFormatSymbols_en_ZA', 'goog.i18n.NumberFormatSymbols_en_ZW', 'goog.i18n.NumberFormatSymbols_es', 'goog.i18n.NumberFormatSymbols_es_419', 'goog.i18n.NumberFormatSymbols_es_EA', 'goog.i18n.NumberFormatSymbols_es_ES', 'goog.i18n.NumberFormatSymbols_es_IC', 'goog.i18n.NumberFormatSymbols_et', 'goog.i18n.NumberFormatSymbols_et_EE', 'goog.i18n.NumberFormatSymbols_eu', 'goog.i18n.NumberFormatSymbols_eu_ES', 'goog.i18n.NumberFormatSymbols_fa', 'goog.i18n.NumberFormatSymbols_fa_IR', 'goog.i18n.NumberFormatSymbols_fi', 'goog.i18n.NumberFormatSymbols_fi_FI', 'goog.i18n.NumberFormatSymbols_fil', 'goog.i18n.NumberFormatSymbols_fil_PH', 'goog.i18n.NumberFormatSymbols_fr', 'goog.i18n.NumberFormatSymbols_fr_BL', 'goog.i18n.NumberFormatSymbols_fr_CA', 'goog.i18n.NumberFormatSymbols_fr_FR', 'goog.i18n.NumberFormatSymbols_fr_GF', 'goog.i18n.NumberFormatSymbols_fr_GP', 'goog.i18n.NumberFormatSymbols_fr_MC', 'goog.i18n.NumberFormatSymbols_fr_MF', 'goog.i18n.NumberFormatSymbols_fr_MQ', 'goog.i18n.NumberFormatSymbols_fr_PM', 'goog.i18n.NumberFormatSymbols_fr_RE', 'goog.i18n.NumberFormatSymbols_fr_YT', 'goog.i18n.NumberFormatSymbols_ga', 'goog.i18n.NumberFormatSymbols_ga_IE', 'goog.i18n.NumberFormatSymbols_gl', 'goog.i18n.NumberFormatSymbols_gl_ES', 'goog.i18n.NumberFormatSymbols_gsw', 'goog.i18n.NumberFormatSymbols_gsw_CH', 'goog.i18n.NumberFormatSymbols_gsw_LI', 'goog.i18n.NumberFormatSymbols_gu', 'goog.i18n.NumberFormatSymbols_gu_IN', 'goog.i18n.NumberFormatSymbols_haw', 'goog.i18n.NumberFormatSymbols_haw_US', 'goog.i18n.NumberFormatSymbols_he', 'goog.i18n.NumberFormatSymbols_he_IL', 'goog.i18n.NumberFormatSymbols_hi', 'goog.i18n.NumberFormatSymbols_hi_IN', 'goog.i18n.NumberFormatSymbols_hr', 'goog.i18n.NumberFormatSymbols_hr_HR', 'goog.i18n.NumberFormatSymbols_hu', 'goog.i18n.NumberFormatSymbols_hu_HU', 'goog.i18n.NumberFormatSymbols_hy', 'goog.i18n.NumberFormatSymbols_hy_AM', 'goog.i18n.NumberFormatSymbols_id', 'goog.i18n.NumberFormatSymbols_id_ID', 'goog.i18n.NumberFormatSymbols_in', 'goog.i18n.NumberFormatSymbols_is', 'goog.i18n.NumberFormatSymbols_is_IS', 'goog.i18n.NumberFormatSymbols_it', 'goog.i18n.NumberFormatSymbols_it_IT', 'goog.i18n.NumberFormatSymbols_it_SM', 'goog.i18n.NumberFormatSymbols_iw', 'goog.i18n.NumberFormatSymbols_ja', 'goog.i18n.NumberFormatSymbols_ja_JP', 'goog.i18n.NumberFormatSymbols_ka', 'goog.i18n.NumberFormatSymbols_ka_GE', 'goog.i18n.NumberFormatSymbols_kk', 'goog.i18n.NumberFormatSymbols_kk_Cyrl_KZ', 'goog.i18n.NumberFormatSymbols_km', 'goog.i18n.NumberFormatSymbols_km_KH', 'goog.i18n.NumberFormatSymbols_kn', 'goog.i18n.NumberFormatSymbols_kn_IN', 'goog.i18n.NumberFormatSymbols_ko', 'goog.i18n.NumberFormatSymbols_ko_KR', 'goog.i18n.NumberFormatSymbols_ky', 'goog.i18n.NumberFormatSymbols_ky_Cyrl_KG', 'goog.i18n.NumberFormatSymbols_ln', 'goog.i18n.NumberFormatSymbols_ln_CD', 'goog.i18n.NumberFormatSymbols_lo', 'goog.i18n.NumberFormatSymbols_lo_LA', 'goog.i18n.NumberFormatSymbols_lt', 'goog.i18n.NumberFormatSymbols_lt_LT', 'goog.i18n.NumberFormatSymbols_lv', 'goog.i18n.NumberFormatSymbols_lv_LV', 'goog.i18n.NumberFormatSymbols_mk', 'goog.i18n.NumberFormatSymbols_mk_MK', 'goog.i18n.NumberFormatSymbols_ml', 'goog.i18n.NumberFormatSymbols_ml_IN', 'goog.i18n.NumberFormatSymbols_mn', 'goog.i18n.NumberFormatSymbols_mn_Cyrl_MN', 'goog.i18n.NumberFormatSymbols_mr', 'goog.i18n.NumberFormatSymbols_mr_IN', 'goog.i18n.NumberFormatSymbols_ms', 'goog.i18n.NumberFormatSymbols_ms_Latn_MY', 'goog.i18n.NumberFormatSymbols_mt', 'goog.i18n.NumberFormatSymbols_mt_MT', 'goog.i18n.NumberFormatSymbols_my', 'goog.i18n.NumberFormatSymbols_my_MM', 'goog.i18n.NumberFormatSymbols_nb', 'goog.i18n.NumberFormatSymbols_nb_NO', 'goog.i18n.NumberFormatSymbols_nb_SJ', 'goog.i18n.NumberFormatSymbols_ne', 'goog.i18n.NumberFormatSymbols_ne_NP', 'goog.i18n.NumberFormatSymbols_nl', 'goog.i18n.NumberFormatSymbols_nl_NL', 'goog.i18n.NumberFormatSymbols_no', 'goog.i18n.NumberFormatSymbols_no_NO', 'goog.i18n.NumberFormatSymbols_or', 'goog.i18n.NumberFormatSymbols_or_IN', 'goog.i18n.NumberFormatSymbols_pa', 'goog.i18n.NumberFormatSymbols_pa_Guru_IN', 'goog.i18n.NumberFormatSymbols_pl', 'goog.i18n.NumberFormatSymbols_pl_PL', 'goog.i18n.NumberFormatSymbols_pt', 'goog.i18n.NumberFormatSymbols_pt_BR', 'goog.i18n.NumberFormatSymbols_pt_PT', 'goog.i18n.NumberFormatSymbols_ro', 'goog.i18n.NumberFormatSymbols_ro_RO', 'goog.i18n.NumberFormatSymbols_ru', 'goog.i18n.NumberFormatSymbols_ru_RU', 'goog.i18n.NumberFormatSymbols_si', 'goog.i18n.NumberFormatSymbols_si_LK', 'goog.i18n.NumberFormatSymbols_sk', 'goog.i18n.NumberFormatSymbols_sk_SK', 'goog.i18n.NumberFormatSymbols_sl', 'goog.i18n.NumberFormatSymbols_sl_SI', 'goog.i18n.NumberFormatSymbols_sq', 'goog.i18n.NumberFormatSymbols_sq_AL', 'goog.i18n.NumberFormatSymbols_sr', 'goog.i18n.NumberFormatSymbols_sr_Cyrl_RS', 'goog.i18n.NumberFormatSymbols_sv', 'goog.i18n.NumberFormatSymbols_sv_SE', 'goog.i18n.NumberFormatSymbols_sw', 'goog.i18n.NumberFormatSymbols_sw_TZ', 'goog.i18n.NumberFormatSymbols_ta', 'goog.i18n.NumberFormatSymbols_ta_IN', 'goog.i18n.NumberFormatSymbols_te', 'goog.i18n.NumberFormatSymbols_te_IN', 'goog.i18n.NumberFormatSymbols_th', 'goog.i18n.NumberFormatSymbols_th_TH', 'goog.i18n.NumberFormatSymbols_tl', 'goog.i18n.NumberFormatSymbols_tr', 'goog.i18n.NumberFormatSymbols_tr_TR', 'goog.i18n.NumberFormatSymbols_uk', 'goog.i18n.NumberFormatSymbols_uk_UA', 'goog.i18n.NumberFormatSymbols_ur', 'goog.i18n.NumberFormatSymbols_ur_PK', 'goog.i18n.NumberFormatSymbols_uz', 'goog.i18n.NumberFormatSymbols_uz_Latn_UZ', 'goog.i18n.NumberFormatSymbols_vi', 'goog.i18n.NumberFormatSymbols_vi_VN', 'goog.i18n.NumberFormatSymbols_zh', 'goog.i18n.NumberFormatSymbols_zh_CN', 'goog.i18n.NumberFormatSymbols_zh_HK', 'goog.i18n.NumberFormatSymbols_zh_Hans_CN', 'goog.i18n.NumberFormatSymbols_zh_TW', 'goog.i18n.NumberFormatSymbols_zu', 'goog.i18n.NumberFormatSymbols_zu_ZA'], [], false);
goog.addDependency('i18n/numberformatsymbolsext.js', ['goog.i18n.NumberFormatSymbolsExt', 'goog.i18n.NumberFormatSymbols_aa', 'goog.i18n.NumberFormatSymbols_aa_DJ', 'goog.i18n.NumberFormatSymbols_aa_ER', 'goog.i18n.NumberFormatSymbols_aa_ET', 'goog.i18n.NumberFormatSymbols_af_NA', 'goog.i18n.NumberFormatSymbols_agq', 'goog.i18n.NumberFormatSymbols_agq_CM', 'goog.i18n.NumberFormatSymbols_ak', 'goog.i18n.NumberFormatSymbols_ak_GH', 'goog.i18n.NumberFormatSymbols_ar_AE', 'goog.i18n.NumberFormatSymbols_ar_BH', 'goog.i18n.NumberFormatSymbols_ar_DJ', 'goog.i18n.NumberFormatSymbols_ar_DZ', 'goog.i18n.NumberFormatSymbols_ar_EG', 'goog.i18n.NumberFormatSymbols_ar_EH', 'goog.i18n.NumberFormatSymbols_ar_ER', 'goog.i18n.NumberFormatSymbols_ar_IL', 'goog.i18n.NumberFormatSymbols_ar_IQ', 'goog.i18n.NumberFormatSymbols_ar_JO', 'goog.i18n.NumberFormatSymbols_ar_KM', 'goog.i18n.NumberFormatSymbols_ar_KW', 'goog.i18n.NumberFormatSymbols_ar_LB', 'goog.i18n.NumberFormatSymbols_ar_LY', 'goog.i18n.NumberFormatSymbols_ar_MA', 'goog.i18n.NumberFormatSymbols_ar_MR', 'goog.i18n.NumberFormatSymbols_ar_OM', 'goog.i18n.NumberFormatSymbols_ar_PS', 'goog.i18n.NumberFormatSymbols_ar_QA', 'goog.i18n.NumberFormatSymbols_ar_SA', 'goog.i18n.NumberFormatSymbols_ar_SD', 'goog.i18n.NumberFormatSymbols_ar_SO', 'goog.i18n.NumberFormatSymbols_ar_SS', 'goog.i18n.NumberFormatSymbols_ar_SY', 'goog.i18n.NumberFormatSymbols_ar_TD', 'goog.i18n.NumberFormatSymbols_ar_TN', 'goog.i18n.NumberFormatSymbols_ar_YE', 'goog.i18n.NumberFormatSymbols_as', 'goog.i18n.NumberFormatSymbols_as_IN', 'goog.i18n.NumberFormatSymbols_asa', 'goog.i18n.NumberFormatSymbols_asa_TZ', 'goog.i18n.NumberFormatSymbols_ast', 'goog.i18n.NumberFormatSymbols_ast_ES', 'goog.i18n.NumberFormatSymbols_az_Cyrl', 'goog.i18n.NumberFormatSymbols_az_Latn', 'goog.i18n.NumberFormatSymbols_bas', 'goog.i18n.NumberFormatSymbols_bas_CM', 'goog.i18n.NumberFormatSymbols_be', 'goog.i18n.NumberFormatSymbols_be_BY', 'goog.i18n.NumberFormatSymbols_bem', 'goog.i18n.NumberFormatSymbols_bem_ZM', 'goog.i18n.NumberFormatSymbols_bez', 'goog.i18n.NumberFormatSymbols_bez_TZ', 'goog.i18n.NumberFormatSymbols_bm', 'goog.i18n.NumberFormatSymbols_bm_ML', 'goog.i18n.NumberFormatSymbols_bn_IN', 'goog.i18n.NumberFormatSymbols_bo', 'goog.i18n.NumberFormatSymbols_bo_CN', 'goog.i18n.NumberFormatSymbols_bo_IN', 'goog.i18n.NumberFormatSymbols_brx', 'goog.i18n.NumberFormatSymbols_brx_IN', 'goog.i18n.NumberFormatSymbols_bs', 'goog.i18n.NumberFormatSymbols_bs_Cyrl', 'goog.i18n.NumberFormatSymbols_bs_Cyrl_BA', 'goog.i18n.NumberFormatSymbols_bs_Latn', 'goog.i18n.NumberFormatSymbols_bs_Latn_BA', 'goog.i18n.NumberFormatSymbols_byn', 'goog.i18n.NumberFormatSymbols_byn_ER', 'goog.i18n.NumberFormatSymbols_cgg', 'goog.i18n.NumberFormatSymbols_cgg_UG', 'goog.i18n.NumberFormatSymbols_ckb', 'goog.i18n.NumberFormatSymbols_ckb_Arab', 'goog.i18n.NumberFormatSymbols_ckb_Arab_IQ', 'goog.i18n.NumberFormatSymbols_ckb_Arab_IR', 'goog.i18n.NumberFormatSymbols_ckb_IQ', 'goog.i18n.NumberFormatSymbols_ckb_IR', 'goog.i18n.NumberFormatSymbols_ckb_Latn', 'goog.i18n.NumberFormatSymbols_ckb_Latn_IQ', 'goog.i18n.NumberFormatSymbols_dav', 'goog.i18n.NumberFormatSymbols_dav_KE', 'goog.i18n.NumberFormatSymbols_de_LI', 'goog.i18n.NumberFormatSymbols_dje', 'goog.i18n.NumberFormatSymbols_dje_NE', 'goog.i18n.NumberFormatSymbols_dua', 'goog.i18n.NumberFormatSymbols_dua_CM', 'goog.i18n.NumberFormatSymbols_dyo', 'goog.i18n.NumberFormatSymbols_dyo_SN', 'goog.i18n.NumberFormatSymbols_dz', 'goog.i18n.NumberFormatSymbols_dz_BT', 'goog.i18n.NumberFormatSymbols_ebu', 'goog.i18n.NumberFormatSymbols_ebu_KE', 'goog.i18n.NumberFormatSymbols_ee', 'goog.i18n.NumberFormatSymbols_ee_GH', 'goog.i18n.NumberFormatSymbols_ee_TG', 'goog.i18n.NumberFormatSymbols_el_CY', 'goog.i18n.NumberFormatSymbols_en_150', 'goog.i18n.NumberFormatSymbols_en_AG', 'goog.i18n.NumberFormatSymbols_en_AI', 'goog.i18n.NumberFormatSymbols_en_BB', 'goog.i18n.NumberFormatSymbols_en_BE', 'goog.i18n.NumberFormatSymbols_en_BM', 'goog.i18n.NumberFormatSymbols_en_BS', 'goog.i18n.NumberFormatSymbols_en_BW', 'goog.i18n.NumberFormatSymbols_en_BZ', 'goog.i18n.NumberFormatSymbols_en_CA', 'goog.i18n.NumberFormatSymbols_en_CC', 'goog.i18n.NumberFormatSymbols_en_CK', 'goog.i18n.NumberFormatSymbols_en_CM', 'goog.i18n.NumberFormatSymbols_en_CX', 'goog.i18n.NumberFormatSymbols_en_DM', 'goog.i18n.NumberFormatSymbols_en_ER', 'goog.i18n.NumberFormatSymbols_en_FJ', 'goog.i18n.NumberFormatSymbols_en_FK', 'goog.i18n.NumberFormatSymbols_en_GD', 'goog.i18n.NumberFormatSymbols_en_GG', 'goog.i18n.NumberFormatSymbols_en_GH', 'goog.i18n.NumberFormatSymbols_en_GI', 'goog.i18n.NumberFormatSymbols_en_GM', 'goog.i18n.NumberFormatSymbols_en_GY', 'goog.i18n.NumberFormatSymbols_en_HK', 'goog.i18n.NumberFormatSymbols_en_IM', 'goog.i18n.NumberFormatSymbols_en_JE', 'goog.i18n.NumberFormatSymbols_en_JM', 'goog.i18n.NumberFormatSymbols_en_KE', 'goog.i18n.NumberFormatSymbols_en_KI', 'goog.i18n.NumberFormatSymbols_en_KN', 'goog.i18n.NumberFormatSymbols_en_KY', 'goog.i18n.NumberFormatSymbols_en_LC', 'goog.i18n.NumberFormatSymbols_en_LR', 'goog.i18n.NumberFormatSymbols_en_LS', 'goog.i18n.NumberFormatSymbols_en_MG', 'goog.i18n.NumberFormatSymbols_en_MO', 'goog.i18n.NumberFormatSymbols_en_MS', 'goog.i18n.NumberFormatSymbols_en_MT', 'goog.i18n.NumberFormatSymbols_en_MU', 'goog.i18n.NumberFormatSymbols_en_MW', 'goog.i18n.NumberFormatSymbols_en_NA', 'goog.i18n.NumberFormatSymbols_en_NF', 'goog.i18n.NumberFormatSymbols_en_NG', 'goog.i18n.NumberFormatSymbols_en_NR', 'goog.i18n.NumberFormatSymbols_en_NU', 'goog.i18n.NumberFormatSymbols_en_NZ', 'goog.i18n.NumberFormatSymbols_en_PG', 'goog.i18n.NumberFormatSymbols_en_PH', 'goog.i18n.NumberFormatSymbols_en_PK', 'goog.i18n.NumberFormatSymbols_en_PN', 'goog.i18n.NumberFormatSymbols_en_RW', 'goog.i18n.NumberFormatSymbols_en_SB', 'goog.i18n.NumberFormatSymbols_en_SC', 'goog.i18n.NumberFormatSymbols_en_SD', 'goog.i18n.NumberFormatSymbols_en_SH', 'goog.i18n.NumberFormatSymbols_en_SL', 'goog.i18n.NumberFormatSymbols_en_SS', 'goog.i18n.NumberFormatSymbols_en_SX', 'goog.i18n.NumberFormatSymbols_en_SZ', 'goog.i18n.NumberFormatSymbols_en_TK', 'goog.i18n.NumberFormatSymbols_en_TO', 'goog.i18n.NumberFormatSymbols_en_TT', 'goog.i18n.NumberFormatSymbols_en_TV', 'goog.i18n.NumberFormatSymbols_en_TZ', 'goog.i18n.NumberFormatSymbols_en_UG', 'goog.i18n.NumberFormatSymbols_en_VC', 'goog.i18n.NumberFormatSymbols_en_VU', 'goog.i18n.NumberFormatSymbols_en_WS', 'goog.i18n.NumberFormatSymbols_en_ZM', 'goog.i18n.NumberFormatSymbols_eo', 'goog.i18n.NumberFormatSymbols_eo_001', 'goog.i18n.NumberFormatSymbols_es_AR', 'goog.i18n.NumberFormatSymbols_es_BO', 'goog.i18n.NumberFormatSymbols_es_CL', 'goog.i18n.NumberFormatSymbols_es_CO', 'goog.i18n.NumberFormatSymbols_es_CR', 'goog.i18n.NumberFormatSymbols_es_CU', 'goog.i18n.NumberFormatSymbols_es_DO', 'goog.i18n.NumberFormatSymbols_es_EC', 'goog.i18n.NumberFormatSymbols_es_GQ', 'goog.i18n.NumberFormatSymbols_es_GT', 'goog.i18n.NumberFormatSymbols_es_HN', 'goog.i18n.NumberFormatSymbols_es_MX', 'goog.i18n.NumberFormatSymbols_es_NI', 'goog.i18n.NumberFormatSymbols_es_PA', 'goog.i18n.NumberFormatSymbols_es_PE', 'goog.i18n.NumberFormatSymbols_es_PH', 'goog.i18n.NumberFormatSymbols_es_PR', 'goog.i18n.NumberFormatSymbols_es_PY', 'goog.i18n.NumberFormatSymbols_es_SV', 'goog.i18n.NumberFormatSymbols_es_US', 'goog.i18n.NumberFormatSymbols_es_UY', 'goog.i18n.NumberFormatSymbols_es_VE', 'goog.i18n.NumberFormatSymbols_ewo', 'goog.i18n.NumberFormatSymbols_ewo_CM', 'goog.i18n.NumberFormatSymbols_fa_AF', 'goog.i18n.NumberFormatSymbols_ff', 'goog.i18n.NumberFormatSymbols_ff_CM', 'goog.i18n.NumberFormatSymbols_ff_GN', 'goog.i18n.NumberFormatSymbols_ff_MR', 'goog.i18n.NumberFormatSymbols_ff_SN', 'goog.i18n.NumberFormatSymbols_fo', 'goog.i18n.NumberFormatSymbols_fo_FO', 'goog.i18n.NumberFormatSymbols_fr_BE', 'goog.i18n.NumberFormatSymbols_fr_BF', 'goog.i18n.NumberFormatSymbols_fr_BI', 'goog.i18n.NumberFormatSymbols_fr_BJ', 'goog.i18n.NumberFormatSymbols_fr_CD', 'goog.i18n.NumberFormatSymbols_fr_CF', 'goog.i18n.NumberFormatSymbols_fr_CG', 'goog.i18n.NumberFormatSymbols_fr_CH', 'goog.i18n.NumberFormatSymbols_fr_CI', 'goog.i18n.NumberFormatSymbols_fr_CM', 'goog.i18n.NumberFormatSymbols_fr_DJ', 'goog.i18n.NumberFormatSymbols_fr_DZ', 'goog.i18n.NumberFormatSymbols_fr_GA', 'goog.i18n.NumberFormatSymbols_fr_GN', 'goog.i18n.NumberFormatSymbols_fr_GQ', 'goog.i18n.NumberFormatSymbols_fr_HT', 'goog.i18n.NumberFormatSymbols_fr_KM', 'goog.i18n.NumberFormatSymbols_fr_LU', 'goog.i18n.NumberFormatSymbols_fr_MA', 'goog.i18n.NumberFormatSymbols_fr_MG', 'goog.i18n.NumberFormatSymbols_fr_ML', 'goog.i18n.NumberFormatSymbols_fr_MR', 'goog.i18n.NumberFormatSymbols_fr_MU', 'goog.i18n.NumberFormatSymbols_fr_NC', 'goog.i18n.NumberFormatSymbols_fr_NE', 'goog.i18n.NumberFormatSymbols_fr_PF', 'goog.i18n.NumberFormatSymbols_fr_RW', 'goog.i18n.NumberFormatSymbols_fr_SC', 'goog.i18n.NumberFormatSymbols_fr_SN', 'goog.i18n.NumberFormatSymbols_fr_SY', 'goog.i18n.NumberFormatSymbols_fr_TD', 'goog.i18n.NumberFormatSymbols_fr_TG', 'goog.i18n.NumberFormatSymbols_fr_TN', 'goog.i18n.NumberFormatSymbols_fr_VU', 'goog.i18n.NumberFormatSymbols_fr_WF', 'goog.i18n.NumberFormatSymbols_fur', 'goog.i18n.NumberFormatSymbols_fur_IT', 'goog.i18n.NumberFormatSymbols_fy', 'goog.i18n.NumberFormatSymbols_fy_NL', 'goog.i18n.NumberFormatSymbols_gd', 'goog.i18n.NumberFormatSymbols_gd_GB', 'goog.i18n.NumberFormatSymbols_guz', 'goog.i18n.NumberFormatSymbols_guz_KE', 'goog.i18n.NumberFormatSymbols_gv', 'goog.i18n.NumberFormatSymbols_gv_IM', 'goog.i18n.NumberFormatSymbols_ha', 'goog.i18n.NumberFormatSymbols_ha_Latn', 'goog.i18n.NumberFormatSymbols_ha_Latn_GH', 'goog.i18n.NumberFormatSymbols_ha_Latn_NE', 'goog.i18n.NumberFormatSymbols_ha_Latn_NG', 'goog.i18n.NumberFormatSymbols_hr_BA', 'goog.i18n.NumberFormatSymbols_ia', 'goog.i18n.NumberFormatSymbols_ia_FR', 'goog.i18n.NumberFormatSymbols_ig', 'goog.i18n.NumberFormatSymbols_ig_NG', 'goog.i18n.NumberFormatSymbols_ii', 'goog.i18n.NumberFormatSymbols_ii_CN', 'goog.i18n.NumberFormatSymbols_it_CH', 'goog.i18n.NumberFormatSymbols_jgo', 'goog.i18n.NumberFormatSymbols_jgo_CM', 'goog.i18n.NumberFormatSymbols_jmc', 'goog.i18n.NumberFormatSymbols_jmc_TZ', 'goog.i18n.NumberFormatSymbols_kab', 'goog.i18n.NumberFormatSymbols_kab_DZ', 'goog.i18n.NumberFormatSymbols_kam', 'goog.i18n.NumberFormatSymbols_kam_KE', 'goog.i18n.NumberFormatSymbols_kde', 'goog.i18n.NumberFormatSymbols_kde_TZ', 'goog.i18n.NumberFormatSymbols_kea', 'goog.i18n.NumberFormatSymbols_kea_CV', 'goog.i18n.NumberFormatSymbols_khq', 'goog.i18n.NumberFormatSymbols_khq_ML', 'goog.i18n.NumberFormatSymbols_ki', 'goog.i18n.NumberFormatSymbols_ki_KE', 'goog.i18n.NumberFormatSymbols_kk_Cyrl', 'goog.i18n.NumberFormatSymbols_kkj', 'goog.i18n.NumberFormatSymbols_kkj_CM', 'goog.i18n.NumberFormatSymbols_kl', 'goog.i18n.NumberFormatSymbols_kl_GL', 'goog.i18n.NumberFormatSymbols_kln', 'goog.i18n.NumberFormatSymbols_kln_KE', 'goog.i18n.NumberFormatSymbols_ko_KP', 'goog.i18n.NumberFormatSymbols_kok', 'goog.i18n.NumberFormatSymbols_kok_IN', 'goog.i18n.NumberFormatSymbols_ks', 'goog.i18n.NumberFormatSymbols_ks_Arab', 'goog.i18n.NumberFormatSymbols_ks_Arab_IN', 'goog.i18n.NumberFormatSymbols_ksb', 'goog.i18n.NumberFormatSymbols_ksb_TZ', 'goog.i18n.NumberFormatSymbols_ksf', 'goog.i18n.NumberFormatSymbols_ksf_CM', 'goog.i18n.NumberFormatSymbols_ksh', 'goog.i18n.NumberFormatSymbols_ksh_DE', 'goog.i18n.NumberFormatSymbols_kw', 'goog.i18n.NumberFormatSymbols_kw_GB', 'goog.i18n.NumberFormatSymbols_ky_Cyrl', 'goog.i18n.NumberFormatSymbols_lag', 'goog.i18n.NumberFormatSymbols_lag_TZ', 'goog.i18n.NumberFormatSymbols_lg', 'goog.i18n.NumberFormatSymbols_lg_UG', 'goog.i18n.NumberFormatSymbols_lkt', 'goog.i18n.NumberFormatSymbols_lkt_US', 'goog.i18n.NumberFormatSymbols_ln_AO', 'goog.i18n.NumberFormatSymbols_ln_CF', 'goog.i18n.NumberFormatSymbols_ln_CG', 'goog.i18n.NumberFormatSymbols_lu', 'goog.i18n.NumberFormatSymbols_lu_CD', 'goog.i18n.NumberFormatSymbols_luo', 'goog.i18n.NumberFormatSymbols_luo_KE', 'goog.i18n.NumberFormatSymbols_luy', 'goog.i18n.NumberFormatSymbols_luy_KE', 'goog.i18n.NumberFormatSymbols_mas', 'goog.i18n.NumberFormatSymbols_mas_KE', 'goog.i18n.NumberFormatSymbols_mas_TZ', 'goog.i18n.NumberFormatSymbols_mer', 'goog.i18n.NumberFormatSymbols_mer_KE', 'goog.i18n.NumberFormatSymbols_mfe', 'goog.i18n.NumberFormatSymbols_mfe_MU', 'goog.i18n.NumberFormatSymbols_mg', 'goog.i18n.NumberFormatSymbols_mg_MG', 'goog.i18n.NumberFormatSymbols_mgh', 'goog.i18n.NumberFormatSymbols_mgh_MZ', 'goog.i18n.NumberFormatSymbols_mgo', 'goog.i18n.NumberFormatSymbols_mgo_CM', 'goog.i18n.NumberFormatSymbols_mn_Cyrl', 'goog.i18n.NumberFormatSymbols_ms_Latn', 'goog.i18n.NumberFormatSymbols_ms_Latn_BN', 'goog.i18n.NumberFormatSymbols_ms_Latn_SG', 'goog.i18n.NumberFormatSymbols_mua', 'goog.i18n.NumberFormatSymbols_mua_CM', 'goog.i18n.NumberFormatSymbols_naq', 'goog.i18n.NumberFormatSymbols_naq_NA', 'goog.i18n.NumberFormatSymbols_nd', 'goog.i18n.NumberFormatSymbols_nd_ZW', 'goog.i18n.NumberFormatSymbols_ne_IN', 'goog.i18n.NumberFormatSymbols_nl_AW', 'goog.i18n.NumberFormatSymbols_nl_BE', 'goog.i18n.NumberFormatSymbols_nl_BQ', 'goog.i18n.NumberFormatSymbols_nl_CW', 'goog.i18n.NumberFormatSymbols_nl_SR', 'goog.i18n.NumberFormatSymbols_nl_SX', 'goog.i18n.NumberFormatSymbols_nmg', 'goog.i18n.NumberFormatSymbols_nmg_CM', 'goog.i18n.NumberFormatSymbols_nn', 'goog.i18n.NumberFormatSymbols_nn_NO', 'goog.i18n.NumberFormatSymbols_nnh', 'goog.i18n.NumberFormatSymbols_nnh_CM', 'goog.i18n.NumberFormatSymbols_nr', 'goog.i18n.NumberFormatSymbols_nr_ZA', 'goog.i18n.NumberFormatSymbols_nso', 'goog.i18n.NumberFormatSymbols_nso_ZA', 'goog.i18n.NumberFormatSymbols_nus', 'goog.i18n.NumberFormatSymbols_nus_SD', 'goog.i18n.NumberFormatSymbols_nyn', 'goog.i18n.NumberFormatSymbols_nyn_UG', 'goog.i18n.NumberFormatSymbols_om', 'goog.i18n.NumberFormatSymbols_om_ET', 'goog.i18n.NumberFormatSymbols_om_KE', 'goog.i18n.NumberFormatSymbols_os', 'goog.i18n.NumberFormatSymbols_os_GE', 'goog.i18n.NumberFormatSymbols_os_RU', 'goog.i18n.NumberFormatSymbols_pa_Arab', 'goog.i18n.NumberFormatSymbols_pa_Arab_PK', 'goog.i18n.NumberFormatSymbols_pa_Guru', 'goog.i18n.NumberFormatSymbols_ps', 'goog.i18n.NumberFormatSymbols_ps_AF', 'goog.i18n.NumberFormatSymbols_pt_AO', 'goog.i18n.NumberFormatSymbols_pt_CV', 'goog.i18n.NumberFormatSymbols_pt_GW', 'goog.i18n.NumberFormatSymbols_pt_MO', 'goog.i18n.NumberFormatSymbols_pt_MZ', 'goog.i18n.NumberFormatSymbols_pt_ST', 'goog.i18n.NumberFormatSymbols_pt_TL', 'goog.i18n.NumberFormatSymbols_rm', 'goog.i18n.NumberFormatSymbols_rm_CH', 'goog.i18n.NumberFormatSymbols_rn', 'goog.i18n.NumberFormatSymbols_rn_BI', 'goog.i18n.NumberFormatSymbols_ro_MD', 'goog.i18n.NumberFormatSymbols_rof', 'goog.i18n.NumberFormatSymbols_rof_TZ', 'goog.i18n.NumberFormatSymbols_ru_BY', 'goog.i18n.NumberFormatSymbols_ru_KG', 'goog.i18n.NumberFormatSymbols_ru_KZ', 'goog.i18n.NumberFormatSymbols_ru_MD', 'goog.i18n.NumberFormatSymbols_ru_UA', 'goog.i18n.NumberFormatSymbols_rw', 'goog.i18n.NumberFormatSymbols_rw_RW', 'goog.i18n.NumberFormatSymbols_rwk', 'goog.i18n.NumberFormatSymbols_rwk_TZ', 'goog.i18n.NumberFormatSymbols_sah', 'goog.i18n.NumberFormatSymbols_sah_RU', 'goog.i18n.NumberFormatSymbols_saq', 'goog.i18n.NumberFormatSymbols_saq_KE', 'goog.i18n.NumberFormatSymbols_sbp', 'goog.i18n.NumberFormatSymbols_sbp_TZ', 'goog.i18n.NumberFormatSymbols_se', 'goog.i18n.NumberFormatSymbols_se_FI', 'goog.i18n.NumberFormatSymbols_se_NO', 'goog.i18n.NumberFormatSymbols_seh', 'goog.i18n.NumberFormatSymbols_seh_MZ', 'goog.i18n.NumberFormatSymbols_ses', 'goog.i18n.NumberFormatSymbols_ses_ML', 'goog.i18n.NumberFormatSymbols_sg', 'goog.i18n.NumberFormatSymbols_sg_CF', 'goog.i18n.NumberFormatSymbols_shi', 'goog.i18n.NumberFormatSymbols_shi_Latn', 'goog.i18n.NumberFormatSymbols_shi_Latn_MA', 'goog.i18n.NumberFormatSymbols_shi_Tfng', 'goog.i18n.NumberFormatSymbols_shi_Tfng_MA', 'goog.i18n.NumberFormatSymbols_sn', 'goog.i18n.NumberFormatSymbols_sn_ZW', 'goog.i18n.NumberFormatSymbols_so', 'goog.i18n.NumberFormatSymbols_so_DJ', 'goog.i18n.NumberFormatSymbols_so_ET', 'goog.i18n.NumberFormatSymbols_so_KE', 'goog.i18n.NumberFormatSymbols_so_SO', 'goog.i18n.NumberFormatSymbols_sq_MK', 'goog.i18n.NumberFormatSymbols_sq_XK', 'goog.i18n.NumberFormatSymbols_sr_Cyrl', 'goog.i18n.NumberFormatSymbols_sr_Cyrl_BA', 'goog.i18n.NumberFormatSymbols_sr_Cyrl_ME', 'goog.i18n.NumberFormatSymbols_sr_Cyrl_XK', 'goog.i18n.NumberFormatSymbols_sr_Latn', 'goog.i18n.NumberFormatSymbols_sr_Latn_BA', 'goog.i18n.NumberFormatSymbols_sr_Latn_ME', 'goog.i18n.NumberFormatSymbols_sr_Latn_RS', 'goog.i18n.NumberFormatSymbols_sr_Latn_XK', 'goog.i18n.NumberFormatSymbols_ss', 'goog.i18n.NumberFormatSymbols_ss_SZ', 'goog.i18n.NumberFormatSymbols_ss_ZA', 'goog.i18n.NumberFormatSymbols_ssy', 'goog.i18n.NumberFormatSymbols_ssy_ER', 'goog.i18n.NumberFormatSymbols_st', 'goog.i18n.NumberFormatSymbols_st_LS', 'goog.i18n.NumberFormatSymbols_st_ZA', 'goog.i18n.NumberFormatSymbols_sv_AX', 'goog.i18n.NumberFormatSymbols_sv_FI', 'goog.i18n.NumberFormatSymbols_sw_KE', 'goog.i18n.NumberFormatSymbols_sw_UG', 'goog.i18n.NumberFormatSymbols_swc', 'goog.i18n.NumberFormatSymbols_swc_CD', 'goog.i18n.NumberFormatSymbols_ta_LK', 'goog.i18n.NumberFormatSymbols_ta_MY', 'goog.i18n.NumberFormatSymbols_ta_SG', 'goog.i18n.NumberFormatSymbols_teo', 'goog.i18n.NumberFormatSymbols_teo_KE', 'goog.i18n.NumberFormatSymbols_teo_UG', 'goog.i18n.NumberFormatSymbols_tg', 'goog.i18n.NumberFormatSymbols_tg_Cyrl', 'goog.i18n.NumberFormatSymbols_tg_Cyrl_TJ', 'goog.i18n.NumberFormatSymbols_ti', 'goog.i18n.NumberFormatSymbols_ti_ER', 'goog.i18n.NumberFormatSymbols_ti_ET', 'goog.i18n.NumberFormatSymbols_tig', 'goog.i18n.NumberFormatSymbols_tig_ER', 'goog.i18n.NumberFormatSymbols_tn', 'goog.i18n.NumberFormatSymbols_tn_BW', 'goog.i18n.NumberFormatSymbols_tn_ZA', 'goog.i18n.NumberFormatSymbols_to', 'goog.i18n.NumberFormatSymbols_to_TO', 'goog.i18n.NumberFormatSymbols_tr_CY', 'goog.i18n.NumberFormatSymbols_ts', 'goog.i18n.NumberFormatSymbols_ts_ZA', 'goog.i18n.NumberFormatSymbols_twq', 'goog.i18n.NumberFormatSymbols_twq_NE', 'goog.i18n.NumberFormatSymbols_tzm', 'goog.i18n.NumberFormatSymbols_tzm_Latn', 'goog.i18n.NumberFormatSymbols_tzm_Latn_MA', 'goog.i18n.NumberFormatSymbols_ug', 'goog.i18n.NumberFormatSymbols_ug_Arab', 'goog.i18n.NumberFormatSymbols_ug_Arab_CN', 'goog.i18n.NumberFormatSymbols_ur_IN', 'goog.i18n.NumberFormatSymbols_uz_Arab', 'goog.i18n.NumberFormatSymbols_uz_Arab_AF', 'goog.i18n.NumberFormatSymbols_uz_Cyrl', 'goog.i18n.NumberFormatSymbols_uz_Cyrl_UZ', 'goog.i18n.NumberFormatSymbols_uz_Latn', 'goog.i18n.NumberFormatSymbols_vai', 'goog.i18n.NumberFormatSymbols_vai_Latn', 'goog.i18n.NumberFormatSymbols_vai_Latn_LR', 'goog.i18n.NumberFormatSymbols_vai_Vaii', 'goog.i18n.NumberFormatSymbols_vai_Vaii_LR', 'goog.i18n.NumberFormatSymbols_ve', 'goog.i18n.NumberFormatSymbols_ve_ZA', 'goog.i18n.NumberFormatSymbols_vo', 'goog.i18n.NumberFormatSymbols_vo_001', 'goog.i18n.NumberFormatSymbols_vun', 'goog.i18n.NumberFormatSymbols_vun_TZ', 'goog.i18n.NumberFormatSymbols_wae', 'goog.i18n.NumberFormatSymbols_wae_CH', 'goog.i18n.NumberFormatSymbols_wal', 'goog.i18n.NumberFormatSymbols_wal_ET', 'goog.i18n.NumberFormatSymbols_xh', 'goog.i18n.NumberFormatSymbols_xh_ZA', 'goog.i18n.NumberFormatSymbols_xog', 'goog.i18n.NumberFormatSymbols_xog_UG', 'goog.i18n.NumberFormatSymbols_yav', 'goog.i18n.NumberFormatSymbols_yav_CM', 'goog.i18n.NumberFormatSymbols_yo', 'goog.i18n.NumberFormatSymbols_yo_BJ', 'goog.i18n.NumberFormatSymbols_yo_NG', 'goog.i18n.NumberFormatSymbols_zgh', 'goog.i18n.NumberFormatSymbols_zgh_MA', 'goog.i18n.NumberFormatSymbols_zh_Hans', 'goog.i18n.NumberFormatSymbols_zh_Hans_HK', 'goog.i18n.NumberFormatSymbols_zh_Hans_MO', 'goog.i18n.NumberFormatSymbols_zh_Hans_SG', 'goog.i18n.NumberFormatSymbols_zh_Hant', 'goog.i18n.NumberFormatSymbols_zh_Hant_HK', 'goog.i18n.NumberFormatSymbols_zh_Hant_MO', 'goog.i18n.NumberFormatSymbols_zh_Hant_TW'], ['goog.i18n.NumberFormatSymbols'], false);
goog.addDependency('i18n/ordinalrules.js', ['goog.i18n.ordinalRules'], [], false);
goog.addDependency('i18n/pluralrules.js', ['goog.i18n.pluralRules'], [], false);
goog.addDependency('i18n/pluralrules_test.js', ['goog.i18n.pluralRulesTest'], ['goog.i18n.pluralRules', 'goog.testing.jsunit'], false);
goog.addDependency('i18n/timezone.js', ['goog.i18n.TimeZone'], ['goog.array', 'goog.date.DateLike', 'goog.string'], false);
goog.addDependency('i18n/timezone_test.js', ['goog.i18n.TimeZoneTest'], ['goog.i18n.TimeZone', 'goog.testing.jsunit'], false);
goog.addDependency('i18n/uchar.js', ['goog.i18n.uChar'], [], false);
goog.addDependency('i18n/uchar/localnamefetcher.js', ['goog.i18n.uChar.LocalNameFetcher'], ['goog.i18n.uChar', 'goog.i18n.uChar.NameFetcher', 'goog.log'], false);
goog.addDependency('i18n/uchar/localnamefetcher_test.js', ['goog.i18n.uChar.LocalNameFetcherTest'], ['goog.i18n.uChar.LocalNameFetcher', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('i18n/uchar/namefetcher.js', ['goog.i18n.uChar.NameFetcher'], [], false);
goog.addDependency('i18n/uchar/remotenamefetcher.js', ['goog.i18n.uChar.RemoteNameFetcher'], ['goog.Disposable', 'goog.Uri', 'goog.i18n.uChar', 'goog.i18n.uChar.NameFetcher', 'goog.log', 'goog.net.XhrIo', 'goog.structs.Map'], false);
goog.addDependency('i18n/uchar/remotenamefetcher_test.js', ['goog.i18n.uChar.RemoteNameFetcherTest'], ['goog.i18n.uChar.RemoteNameFetcher', 'goog.net.XhrIo', 'goog.testing.jsunit', 'goog.testing.net.XhrIo', 'goog.testing.recordFunction'], false);
goog.addDependency('i18n/uchar_test.js', ['goog.i18n.uCharTest'], ['goog.i18n.uChar', 'goog.testing.jsunit'], false);
goog.addDependency('iter/iter.js', ['goog.iter', 'goog.iter.Iterable', 'goog.iter.Iterator', 'goog.iter.StopIteration'], ['goog.array', 'goog.asserts', 'goog.functions', 'goog.math'], false);
goog.addDependency('iter/iter_test.js', ['goog.iterTest'], ['goog.iter', 'goog.iter.Iterator', 'goog.iter.StopIteration', 'goog.testing.jsunit'], false);
goog.addDependency('json/evaljsonprocessor.js', ['goog.json.EvalJsonProcessor'], ['goog.json', 'goog.json.Processor', 'goog.json.Serializer'], false);
goog.addDependency('json/hybrid.js', ['goog.json.hybrid'], ['goog.asserts', 'goog.json'], false);
goog.addDependency('json/hybrid_test.js', ['goog.json.hybridTest'], ['goog.json', 'goog.json.hybrid', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.recordFunction', 'goog.userAgent'], false);
goog.addDependency('json/hybridjsonprocessor.js', ['goog.json.HybridJsonProcessor'], ['goog.json.Processor', 'goog.json.hybrid'], false);
goog.addDependency('json/hybridjsonprocessor_test.js', ['goog.json.HybridJsonProcessorTest'], ['goog.json.HybridJsonProcessor', 'goog.json.hybrid', 'goog.testing.jsunit'], false);
goog.addDependency('json/json.js', ['goog.json', 'goog.json.Replacer', 'goog.json.Reviver', 'goog.json.Serializer'], [], false);
goog.addDependency('json/json_perf.js', ['goog.jsonPerf'], ['goog.dom', 'goog.json', 'goog.math', 'goog.string', 'goog.testing.PerformanceTable', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit'], false);
goog.addDependency('json/json_test.js', ['goog.jsonTest'], ['goog.functions', 'goog.json', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('json/nativejsonprocessor.js', ['goog.json.NativeJsonProcessor'], ['goog.asserts', 'goog.json', 'goog.json.Processor'], false);
goog.addDependency('json/processor.js', ['goog.json.Processor'], ['goog.string.Parser', 'goog.string.Stringifier'], false);
goog.addDependency('json/processor_test.js', ['goog.json.processorTest'], ['goog.json.EvalJsonProcessor', 'goog.json.NativeJsonProcessor', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('labs/dom/pagevisibilitymonitor.js', ['goog.labs.dom.PageVisibilityEvent', 'goog.labs.dom.PageVisibilityMonitor', 'goog.labs.dom.PageVisibilityState'], ['goog.dom', 'goog.dom.vendor', 'goog.events', 'goog.events.Event', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.memoize'], false);
goog.addDependency('labs/dom/pagevisibilitymonitor_test.js', ['goog.labs.dom.PageVisibilityMonitorTest'], ['goog.events', 'goog.functions', 'goog.labs.dom.PageVisibilityMonitor', 'goog.testing.PropertyReplacer', 'goog.testing.events', 'goog.testing.events.Event', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('labs/events/nondisposableeventtarget.js', ['goog.labs.events.NonDisposableEventTarget'], ['goog.array', 'goog.asserts', 'goog.events.Event', 'goog.events.Listenable', 'goog.events.ListenerMap', 'goog.object'], false);
goog.addDependency('labs/events/nondisposableeventtarget_test.js', ['goog.labs.events.NonDisposableEventTargetTest'], ['goog.events.Listenable', 'goog.events.eventTargetTester', 'goog.events.eventTargetTester.KeyType', 'goog.events.eventTargetTester.UnlistenReturnType', 'goog.labs.events.NonDisposableEventTarget', 'goog.testing.jsunit'], false);
goog.addDependency('labs/events/nondisposableeventtarget_via_googevents_test.js', ['goog.labs.events.NonDisposableEventTargetGoogEventsTest'], ['goog.events', 'goog.events.eventTargetTester', 'goog.events.eventTargetTester.KeyType', 'goog.events.eventTargetTester.UnlistenReturnType', 'goog.labs.events.NonDisposableEventTarget', 'goog.testing', 'goog.testing.jsunit'], false);
goog.addDependency('labs/events/touch.js', ['goog.labs.events.touch', 'goog.labs.events.touch.TouchData'], ['goog.array', 'goog.asserts', 'goog.events.EventType', 'goog.string'], false);
goog.addDependency('labs/events/touch_test.js', ['goog.labs.events.touchTest'], ['goog.labs.events.touch', 'goog.testing.jsunit'], false);
goog.addDependency('labs/format/csv.js', ['goog.labs.format.csv', 'goog.labs.format.csv.ParseError', 'goog.labs.format.csv.Token'], ['goog.array', 'goog.asserts', 'goog.debug.Error', 'goog.object', 'goog.string', 'goog.string.newlines'], false);
goog.addDependency('labs/format/csv_test.js', ['goog.labs.format.csvTest'], ['goog.labs.format.csv', 'goog.labs.format.csv.ParseError', 'goog.object', 'goog.testing.asserts', 'goog.testing.jsunit'], false);
goog.addDependency('labs/html/attribute_rewriter.js', ['goog.labs.html.AttributeRewriter', 'goog.labs.html.AttributeValue', 'goog.labs.html.attributeRewriterPresubmitWorkaround'], [], false);
goog.addDependency('labs/html/sanitizer.js', ['goog.labs.html.Sanitizer'], ['goog.asserts', 'goog.html.SafeUrl', 'goog.labs.html.attributeRewriterPresubmitWorkaround', 'goog.labs.html.scrubber', 'goog.object', 'goog.string'], false);
goog.addDependency('labs/html/sanitizer_test.js', ['goog.labs.html.SanitizerTest'], ['goog.html.SafeUrl', 'goog.labs.html.Sanitizer', 'goog.string', 'goog.string.Const', 'goog.testing.jsunit'], false);
goog.addDependency('labs/html/scrubber.js', ['goog.labs.html.scrubber'], ['goog.array', 'goog.dom.tags', 'goog.labs.html.attributeRewriterPresubmitWorkaround', 'goog.string'], false);
goog.addDependency('labs/html/scrubber_test.js', ['goog.html.ScrubberTest'], ['goog.labs.html.scrubber', 'goog.object', 'goog.string', 'goog.testing.jsunit'], false);
goog.addDependency('labs/i18n/listformat.js', ['goog.labs.i18n.GenderInfo', 'goog.labs.i18n.GenderInfo.Gender', 'goog.labs.i18n.ListFormat'], ['goog.asserts', 'goog.labs.i18n.ListFormatSymbols'], false);
goog.addDependency('labs/i18n/listformat_test.js', ['goog.labs.i18n.ListFormatTest'], ['goog.labs.i18n.GenderInfo', 'goog.labs.i18n.ListFormat', 'goog.labs.i18n.ListFormatSymbols', 'goog.labs.i18n.ListFormatSymbols_el', 'goog.labs.i18n.ListFormatSymbols_en', 'goog.labs.i18n.ListFormatSymbols_fr', 'goog.labs.i18n.ListFormatSymbols_ml', 'goog.labs.i18n.ListFormatSymbols_zu', 'goog.testing.jsunit'], false);
goog.addDependency('labs/i18n/listsymbols.js', ['goog.labs.i18n.ListFormatSymbols', 'goog.labs.i18n.ListFormatSymbols_af', 'goog.labs.i18n.ListFormatSymbols_am', 'goog.labs.i18n.ListFormatSymbols_ar', 'goog.labs.i18n.ListFormatSymbols_az', 'goog.labs.i18n.ListFormatSymbols_bg', 'goog.labs.i18n.ListFormatSymbols_bn', 'goog.labs.i18n.ListFormatSymbols_br', 'goog.labs.i18n.ListFormatSymbols_ca', 'goog.labs.i18n.ListFormatSymbols_chr', 'goog.labs.i18n.ListFormatSymbols_cs', 'goog.labs.i18n.ListFormatSymbols_cy', 'goog.labs.i18n.ListFormatSymbols_da', 'goog.labs.i18n.ListFormatSymbols_de', 'goog.labs.i18n.ListFormatSymbols_de_AT', 'goog.labs.i18n.ListFormatSymbols_de_CH', 'goog.labs.i18n.ListFormatSymbols_el', 'goog.labs.i18n.ListFormatSymbols_en', 'goog.labs.i18n.ListFormatSymbols_en_AU', 'goog.labs.i18n.ListFormatSymbols_en_GB', 'goog.labs.i18n.ListFormatSymbols_en_IE', 'goog.labs.i18n.ListFormatSymbols_en_IN', 'goog.labs.i18n.ListFormatSymbols_en_ISO', 'goog.labs.i18n.ListFormatSymbols_en_SG', 'goog.labs.i18n.ListFormatSymbols_en_US', 'goog.labs.i18n.ListFormatSymbols_en_ZA', 'goog.labs.i18n.ListFormatSymbols_es', 'goog.labs.i18n.ListFormatSymbols_es_419', 'goog.labs.i18n.ListFormatSymbols_es_ES', 'goog.labs.i18n.ListFormatSymbols_et', 'goog.labs.i18n.ListFormatSymbols_eu', 'goog.labs.i18n.ListFormatSymbols_fa', 'goog.labs.i18n.ListFormatSymbols_fi', 'goog.labs.i18n.ListFormatSymbols_fil', 'goog.labs.i18n.ListFormatSymbols_fr', 'goog.labs.i18n.ListFormatSymbols_fr_CA', 'goog.labs.i18n.ListFormatSymbols_ga', 'goog.labs.i18n.ListFormatSymbols_gl', 'goog.labs.i18n.ListFormatSymbols_gsw', 'goog.labs.i18n.ListFormatSymbols_gu', 'goog.labs.i18n.ListFormatSymbols_haw', 'goog.labs.i18n.ListFormatSymbols_he', 'goog.labs.i18n.ListFormatSymbols_hi', 'goog.labs.i18n.ListFormatSymbols_hr', 'goog.labs.i18n.ListFormatSymbols_hu', 'goog.labs.i18n.ListFormatSymbols_hy', 'goog.labs.i18n.ListFormatSymbols_id', 'goog.labs.i18n.ListFormatSymbols_in', 'goog.labs.i18n.ListFormatSymbols_is', 'goog.labs.i18n.ListFormatSymbols_it', 'goog.labs.i18n.ListFormatSymbols_iw', 'goog.labs.i18n.ListFormatSymbols_ja', 'goog.labs.i18n.ListFormatSymbols_ka', 'goog.labs.i18n.ListFormatSymbols_kk', 'goog.labs.i18n.ListFormatSymbols_km', 'goog.labs.i18n.ListFormatSymbols_kn', 'goog.labs.i18n.ListFormatSymbols_ko', 'goog.labs.i18n.ListFormatSymbols_ky', 'goog.labs.i18n.ListFormatSymbols_ln', 'goog.labs.i18n.ListFormatSymbols_lo', 'goog.labs.i18n.ListFormatSymbols_lt', 'goog.labs.i18n.ListFormatSymbols_lv', 'goog.labs.i18n.ListFormatSymbols_mk', 'goog.labs.i18n.ListFormatSymbols_ml', 'goog.labs.i18n.ListFormatSymbols_mn', 'goog.labs.i18n.ListFormatSymbols_mo', 'goog.labs.i18n.ListFormatSymbols_mr', 'goog.labs.i18n.ListFormatSymbols_ms', 'goog.labs.i18n.ListFormatSymbols_mt', 'goog.labs.i18n.ListFormatSymbols_my', 'goog.labs.i18n.ListFormatSymbols_nb', 'goog.labs.i18n.ListFormatSymbols_ne', 'goog.labs.i18n.ListFormatSymbols_nl', 'goog.labs.i18n.ListFormatSymbols_no', 'goog.labs.i18n.ListFormatSymbols_no_NO', 'goog.labs.i18n.ListFormatSymbols_or', 'goog.labs.i18n.ListFormatSymbols_pa', 'goog.labs.i18n.ListFormatSymbols_pl', 'goog.labs.i18n.ListFormatSymbols_pt', 'goog.labs.i18n.ListFormatSymbols_pt_BR', 'goog.labs.i18n.ListFormatSymbols_pt_PT', 'goog.labs.i18n.ListFormatSymbols_ro', 'goog.labs.i18n.ListFormatSymbols_ru', 'goog.labs.i18n.ListFormatSymbols_sh', 'goog.labs.i18n.ListFormatSymbols_si', 'goog.labs.i18n.ListFormatSymbols_sk', 'goog.labs.i18n.ListFormatSymbols_sl', 'goog.labs.i18n.ListFormatSymbols_sq', 'goog.labs.i18n.ListFormatSymbols_sr', 'goog.labs.i18n.ListFormatSymbols_sv', 'goog.labs.i18n.ListFormatSymbols_sw', 'goog.labs.i18n.ListFormatSymbols_ta', 'goog.labs.i18n.ListFormatSymbols_te', 'goog.labs.i18n.ListFormatSymbols_th', 'goog.labs.i18n.ListFormatSymbols_tl', 'goog.labs.i18n.ListFormatSymbols_tr', 'goog.labs.i18n.ListFormatSymbols_uk', 'goog.labs.i18n.ListFormatSymbols_ur', 'goog.labs.i18n.ListFormatSymbols_uz', 'goog.labs.i18n.ListFormatSymbols_vi', 'goog.labs.i18n.ListFormatSymbols_zh', 'goog.labs.i18n.ListFormatSymbols_zh_CN', 'goog.labs.i18n.ListFormatSymbols_zh_HK', 'goog.labs.i18n.ListFormatSymbols_zh_TW', 'goog.labs.i18n.ListFormatSymbols_zu'], [], false);
goog.addDependency('labs/i18n/listsymbolsext.js', ['goog.labs.i18n.ListFormatSymbolsExt', 'goog.labs.i18n.ListFormatSymbols_af_NA', 'goog.labs.i18n.ListFormatSymbols_af_ZA', 'goog.labs.i18n.ListFormatSymbols_agq', 'goog.labs.i18n.ListFormatSymbols_agq_CM', 'goog.labs.i18n.ListFormatSymbols_ak', 'goog.labs.i18n.ListFormatSymbols_ak_GH', 'goog.labs.i18n.ListFormatSymbols_am_ET', 'goog.labs.i18n.ListFormatSymbols_ar_001', 'goog.labs.i18n.ListFormatSymbols_ar_AE', 'goog.labs.i18n.ListFormatSymbols_ar_BH', 'goog.labs.i18n.ListFormatSymbols_ar_DJ', 'goog.labs.i18n.ListFormatSymbols_ar_DZ', 'goog.labs.i18n.ListFormatSymbols_ar_EG', 'goog.labs.i18n.ListFormatSymbols_ar_EH', 'goog.labs.i18n.ListFormatSymbols_ar_ER', 'goog.labs.i18n.ListFormatSymbols_ar_IL', 'goog.labs.i18n.ListFormatSymbols_ar_IQ', 'goog.labs.i18n.ListFormatSymbols_ar_JO', 'goog.labs.i18n.ListFormatSymbols_ar_KM', 'goog.labs.i18n.ListFormatSymbols_ar_KW', 'goog.labs.i18n.ListFormatSymbols_ar_LB', 'goog.labs.i18n.ListFormatSymbols_ar_LY', 'goog.labs.i18n.ListFormatSymbols_ar_MA', 'goog.labs.i18n.ListFormatSymbols_ar_MR', 'goog.labs.i18n.ListFormatSymbols_ar_OM', 'goog.labs.i18n.ListFormatSymbols_ar_PS', 'goog.labs.i18n.ListFormatSymbols_ar_QA', 'goog.labs.i18n.ListFormatSymbols_ar_SA', 'goog.labs.i18n.ListFormatSymbols_ar_SD', 'goog.labs.i18n.ListFormatSymbols_ar_SO', 'goog.labs.i18n.ListFormatSymbols_ar_SS', 'goog.labs.i18n.ListFormatSymbols_ar_SY', 'goog.labs.i18n.ListFormatSymbols_ar_TD', 'goog.labs.i18n.ListFormatSymbols_ar_TN', 'goog.labs.i18n.ListFormatSymbols_ar_YE', 'goog.labs.i18n.ListFormatSymbols_as', 'goog.labs.i18n.ListFormatSymbols_as_IN', 'goog.labs.i18n.ListFormatSymbols_asa', 'goog.labs.i18n.ListFormatSymbols_asa_TZ', 'goog.labs.i18n.ListFormatSymbols_az_Cyrl', 'goog.labs.i18n.ListFormatSymbols_az_Cyrl_AZ', 'goog.labs.i18n.ListFormatSymbols_az_Latn', 'goog.labs.i18n.ListFormatSymbols_az_Latn_AZ', 'goog.labs.i18n.ListFormatSymbols_bas', 'goog.labs.i18n.ListFormatSymbols_bas_CM', 'goog.labs.i18n.ListFormatSymbols_be', 'goog.labs.i18n.ListFormatSymbols_be_BY', 'goog.labs.i18n.ListFormatSymbols_bem', 'goog.labs.i18n.ListFormatSymbols_bem_ZM', 'goog.labs.i18n.ListFormatSymbols_bez', 'goog.labs.i18n.ListFormatSymbols_bez_TZ', 'goog.labs.i18n.ListFormatSymbols_bg_BG', 'goog.labs.i18n.ListFormatSymbols_bm', 'goog.labs.i18n.ListFormatSymbols_bm_ML', 'goog.labs.i18n.ListFormatSymbols_bn_BD', 'goog.labs.i18n.ListFormatSymbols_bn_IN', 'goog.labs.i18n.ListFormatSymbols_bo', 'goog.labs.i18n.ListFormatSymbols_bo_CN', 'goog.labs.i18n.ListFormatSymbols_bo_IN', 'goog.labs.i18n.ListFormatSymbols_br_FR', 'goog.labs.i18n.ListFormatSymbols_brx', 'goog.labs.i18n.ListFormatSymbols_brx_IN', 'goog.labs.i18n.ListFormatSymbols_bs', 'goog.labs.i18n.ListFormatSymbols_bs_Cyrl', 'goog.labs.i18n.ListFormatSymbols_bs_Cyrl_BA', 'goog.labs.i18n.ListFormatSymbols_bs_Latn', 'goog.labs.i18n.ListFormatSymbols_bs_Latn_BA', 'goog.labs.i18n.ListFormatSymbols_ca_AD', 'goog.labs.i18n.ListFormatSymbols_ca_ES', 'goog.labs.i18n.ListFormatSymbols_ca_FR', 'goog.labs.i18n.ListFormatSymbols_ca_IT', 'goog.labs.i18n.ListFormatSymbols_cgg', 'goog.labs.i18n.ListFormatSymbols_cgg_UG', 'goog.labs.i18n.ListFormatSymbols_chr_US', 'goog.labs.i18n.ListFormatSymbols_cs_CZ', 'goog.labs.i18n.ListFormatSymbols_cy_GB', 'goog.labs.i18n.ListFormatSymbols_da_DK', 'goog.labs.i18n.ListFormatSymbols_da_GL', 'goog.labs.i18n.ListFormatSymbols_dav', 'goog.labs.i18n.ListFormatSymbols_dav_KE', 'goog.labs.i18n.ListFormatSymbols_de_BE', 'goog.labs.i18n.ListFormatSymbols_de_DE', 'goog.labs.i18n.ListFormatSymbols_de_LI', 'goog.labs.i18n.ListFormatSymbols_de_LU', 'goog.labs.i18n.ListFormatSymbols_dje', 'goog.labs.i18n.ListFormatSymbols_dje_NE', 'goog.labs.i18n.ListFormatSymbols_dua', 'goog.labs.i18n.ListFormatSymbols_dua_CM', 'goog.labs.i18n.ListFormatSymbols_dyo', 'goog.labs.i18n.ListFormatSymbols_dyo_SN', 'goog.labs.i18n.ListFormatSymbols_dz', 'goog.labs.i18n.ListFormatSymbols_dz_BT', 'goog.labs.i18n.ListFormatSymbols_ebu', 'goog.labs.i18n.ListFormatSymbols_ebu_KE', 'goog.labs.i18n.ListFormatSymbols_ee', 'goog.labs.i18n.ListFormatSymbols_ee_GH', 'goog.labs.i18n.ListFormatSymbols_ee_TG', 'goog.labs.i18n.ListFormatSymbols_el_CY', 'goog.labs.i18n.ListFormatSymbols_el_GR', 'goog.labs.i18n.ListFormatSymbols_en_001', 'goog.labs.i18n.ListFormatSymbols_en_150', 'goog.labs.i18n.ListFormatSymbols_en_AG', 'goog.labs.i18n.ListFormatSymbols_en_AI', 'goog.labs.i18n.ListFormatSymbols_en_AS', 'goog.labs.i18n.ListFormatSymbols_en_BB', 'goog.labs.i18n.ListFormatSymbols_en_BE', 'goog.labs.i18n.ListFormatSymbols_en_BM', 'goog.labs.i18n.ListFormatSymbols_en_BS', 'goog.labs.i18n.ListFormatSymbols_en_BW', 'goog.labs.i18n.ListFormatSymbols_en_BZ', 'goog.labs.i18n.ListFormatSymbols_en_CA', 'goog.labs.i18n.ListFormatSymbols_en_CC', 'goog.labs.i18n.ListFormatSymbols_en_CK', 'goog.labs.i18n.ListFormatSymbols_en_CM', 'goog.labs.i18n.ListFormatSymbols_en_CX', 'goog.labs.i18n.ListFormatSymbols_en_DG', 'goog.labs.i18n.ListFormatSymbols_en_DM', 'goog.labs.i18n.ListFormatSymbols_en_ER', 'goog.labs.i18n.ListFormatSymbols_en_FJ', 'goog.labs.i18n.ListFormatSymbols_en_FK', 'goog.labs.i18n.ListFormatSymbols_en_FM', 'goog.labs.i18n.ListFormatSymbols_en_GD', 'goog.labs.i18n.ListFormatSymbols_en_GG', 'goog.labs.i18n.ListFormatSymbols_en_GH', 'goog.labs.i18n.ListFormatSymbols_en_GI', 'goog.labs.i18n.ListFormatSymbols_en_GM', 'goog.labs.i18n.ListFormatSymbols_en_GU', 'goog.labs.i18n.ListFormatSymbols_en_GY', 'goog.labs.i18n.ListFormatSymbols_en_HK', 'goog.labs.i18n.ListFormatSymbols_en_IM', 'goog.labs.i18n.ListFormatSymbols_en_IO', 'goog.labs.i18n.ListFormatSymbols_en_JE', 'goog.labs.i18n.ListFormatSymbols_en_JM', 'goog.labs.i18n.ListFormatSymbols_en_KE', 'goog.labs.i18n.ListFormatSymbols_en_KI', 'goog.labs.i18n.ListFormatSymbols_en_KN', 'goog.labs.i18n.ListFormatSymbols_en_KY', 'goog.labs.i18n.ListFormatSymbols_en_LC', 'goog.labs.i18n.ListFormatSymbols_en_LR', 'goog.labs.i18n.ListFormatSymbols_en_LS', 'goog.labs.i18n.ListFormatSymbols_en_MG', 'goog.labs.i18n.ListFormatSymbols_en_MH', 'goog.labs.i18n.ListFormatSymbols_en_MO', 'goog.labs.i18n.ListFormatSymbols_en_MP', 'goog.labs.i18n.ListFormatSymbols_en_MS', 'goog.labs.i18n.ListFormatSymbols_en_MT', 'goog.labs.i18n.ListFormatSymbols_en_MU', 'goog.labs.i18n.ListFormatSymbols_en_MW', 'goog.labs.i18n.ListFormatSymbols_en_NA', 'goog.labs.i18n.ListFormatSymbols_en_NF', 'goog.labs.i18n.ListFormatSymbols_en_NG', 'goog.labs.i18n.ListFormatSymbols_en_NR', 'goog.labs.i18n.ListFormatSymbols_en_NU', 'goog.labs.i18n.ListFormatSymbols_en_NZ', 'goog.labs.i18n.ListFormatSymbols_en_PG', 'goog.labs.i18n.ListFormatSymbols_en_PH', 'goog.labs.i18n.ListFormatSymbols_en_PK', 'goog.labs.i18n.ListFormatSymbols_en_PN', 'goog.labs.i18n.ListFormatSymbols_en_PR', 'goog.labs.i18n.ListFormatSymbols_en_PW', 'goog.labs.i18n.ListFormatSymbols_en_RW', 'goog.labs.i18n.ListFormatSymbols_en_SB', 'goog.labs.i18n.ListFormatSymbols_en_SC', 'goog.labs.i18n.ListFormatSymbols_en_SD', 'goog.labs.i18n.ListFormatSymbols_en_SH', 'goog.labs.i18n.ListFormatSymbols_en_SL', 'goog.labs.i18n.ListFormatSymbols_en_SS', 'goog.labs.i18n.ListFormatSymbols_en_SX', 'goog.labs.i18n.ListFormatSymbols_en_SZ', 'goog.labs.i18n.ListFormatSymbols_en_TC', 'goog.labs.i18n.ListFormatSymbols_en_TK', 'goog.labs.i18n.ListFormatSymbols_en_TO', 'goog.labs.i18n.ListFormatSymbols_en_TT', 'goog.labs.i18n.ListFormatSymbols_en_TV', 'goog.labs.i18n.ListFormatSymbols_en_TZ', 'goog.labs.i18n.ListFormatSymbols_en_UG', 'goog.labs.i18n.ListFormatSymbols_en_UM', 'goog.labs.i18n.ListFormatSymbols_en_US_POSIX', 'goog.labs.i18n.ListFormatSymbols_en_VC', 'goog.labs.i18n.ListFormatSymbols_en_VG', 'goog.labs.i18n.ListFormatSymbols_en_VI', 'goog.labs.i18n.ListFormatSymbols_en_VU', 'goog.labs.i18n.ListFormatSymbols_en_WS', 'goog.labs.i18n.ListFormatSymbols_en_ZM', 'goog.labs.i18n.ListFormatSymbols_en_ZW', 'goog.labs.i18n.ListFormatSymbols_eo', 'goog.labs.i18n.ListFormatSymbols_es_AR', 'goog.labs.i18n.ListFormatSymbols_es_BO', 'goog.labs.i18n.ListFormatSymbols_es_CL', 'goog.labs.i18n.ListFormatSymbols_es_CO', 'goog.labs.i18n.ListFormatSymbols_es_CR', 'goog.labs.i18n.ListFormatSymbols_es_CU', 'goog.labs.i18n.ListFormatSymbols_es_DO', 'goog.labs.i18n.ListFormatSymbols_es_EA', 'goog.labs.i18n.ListFormatSymbols_es_EC', 'goog.labs.i18n.ListFormatSymbols_es_GQ', 'goog.labs.i18n.ListFormatSymbols_es_GT', 'goog.labs.i18n.ListFormatSymbols_es_HN', 'goog.labs.i18n.ListFormatSymbols_es_IC', 'goog.labs.i18n.ListFormatSymbols_es_MX', 'goog.labs.i18n.ListFormatSymbols_es_NI', 'goog.labs.i18n.ListFormatSymbols_es_PA', 'goog.labs.i18n.ListFormatSymbols_es_PE', 'goog.labs.i18n.ListFormatSymbols_es_PH', 'goog.labs.i18n.ListFormatSymbols_es_PR', 'goog.labs.i18n.ListFormatSymbols_es_PY', 'goog.labs.i18n.ListFormatSymbols_es_SV', 'goog.labs.i18n.ListFormatSymbols_es_US', 'goog.labs.i18n.ListFormatSymbols_es_UY', 'goog.labs.i18n.ListFormatSymbols_es_VE', 'goog.labs.i18n.ListFormatSymbols_et_EE', 'goog.labs.i18n.ListFormatSymbols_eu_ES', 'goog.labs.i18n.ListFormatSymbols_ewo', 'goog.labs.i18n.ListFormatSymbols_ewo_CM', 'goog.labs.i18n.ListFormatSymbols_fa_AF', 'goog.labs.i18n.ListFormatSymbols_fa_IR', 'goog.labs.i18n.ListFormatSymbols_ff', 'goog.labs.i18n.ListFormatSymbols_ff_SN', 'goog.labs.i18n.ListFormatSymbols_fi_FI', 'goog.labs.i18n.ListFormatSymbols_fil_PH', 'goog.labs.i18n.ListFormatSymbols_fo', 'goog.labs.i18n.ListFormatSymbols_fo_FO', 'goog.labs.i18n.ListFormatSymbols_fr_BE', 'goog.labs.i18n.ListFormatSymbols_fr_BF', 'goog.labs.i18n.ListFormatSymbols_fr_BI', 'goog.labs.i18n.ListFormatSymbols_fr_BJ', 'goog.labs.i18n.ListFormatSymbols_fr_BL', 'goog.labs.i18n.ListFormatSymbols_fr_CD', 'goog.labs.i18n.ListFormatSymbols_fr_CF', 'goog.labs.i18n.ListFormatSymbols_fr_CG', 'goog.labs.i18n.ListFormatSymbols_fr_CH', 'goog.labs.i18n.ListFormatSymbols_fr_CI', 'goog.labs.i18n.ListFormatSymbols_fr_CM', 'goog.labs.i18n.ListFormatSymbols_fr_DJ', 'goog.labs.i18n.ListFormatSymbols_fr_DZ', 'goog.labs.i18n.ListFormatSymbols_fr_FR', 'goog.labs.i18n.ListFormatSymbols_fr_GA', 'goog.labs.i18n.ListFormatSymbols_fr_GF', 'goog.labs.i18n.ListFormatSymbols_fr_GN', 'goog.labs.i18n.ListFormatSymbols_fr_GP', 'goog.labs.i18n.ListFormatSymbols_fr_GQ', 'goog.labs.i18n.ListFormatSymbols_fr_HT', 'goog.labs.i18n.ListFormatSymbols_fr_KM', 'goog.labs.i18n.ListFormatSymbols_fr_LU', 'goog.labs.i18n.ListFormatSymbols_fr_MA', 'goog.labs.i18n.ListFormatSymbols_fr_MC', 'goog.labs.i18n.ListFormatSymbols_fr_MF', 'goog.labs.i18n.ListFormatSymbols_fr_MG', 'goog.labs.i18n.ListFormatSymbols_fr_ML', 'goog.labs.i18n.ListFormatSymbols_fr_MQ', 'goog.labs.i18n.ListFormatSymbols_fr_MR', 'goog.labs.i18n.ListFormatSymbols_fr_MU', 'goog.labs.i18n.ListFormatSymbols_fr_NC', 'goog.labs.i18n.ListFormatSymbols_fr_NE', 'goog.labs.i18n.ListFormatSymbols_fr_PF', 'goog.labs.i18n.ListFormatSymbols_fr_PM', 'goog.labs.i18n.ListFormatSymbols_fr_RE', 'goog.labs.i18n.ListFormatSymbols_fr_RW', 'goog.labs.i18n.ListFormatSymbols_fr_SC', 'goog.labs.i18n.ListFormatSymbols_fr_SN', 'goog.labs.i18n.ListFormatSymbols_fr_SY', 'goog.labs.i18n.ListFormatSymbols_fr_TD', 'goog.labs.i18n.ListFormatSymbols_fr_TG', 'goog.labs.i18n.ListFormatSymbols_fr_TN', 'goog.labs.i18n.ListFormatSymbols_fr_VU', 'goog.labs.i18n.ListFormatSymbols_fr_WF', 'goog.labs.i18n.ListFormatSymbols_fr_YT', 'goog.labs.i18n.ListFormatSymbols_ga_IE', 'goog.labs.i18n.ListFormatSymbols_gl_ES', 'goog.labs.i18n.ListFormatSymbols_gsw_CH', 'goog.labs.i18n.ListFormatSymbols_gsw_LI', 'goog.labs.i18n.ListFormatSymbols_gu_IN', 'goog.labs.i18n.ListFormatSymbols_guz', 'goog.labs.i18n.ListFormatSymbols_guz_KE', 'goog.labs.i18n.ListFormatSymbols_gv', 'goog.labs.i18n.ListFormatSymbols_gv_IM', 'goog.labs.i18n.ListFormatSymbols_ha', 'goog.labs.i18n.ListFormatSymbols_ha_Latn', 'goog.labs.i18n.ListFormatSymbols_ha_Latn_GH', 'goog.labs.i18n.ListFormatSymbols_ha_Latn_NE', 'goog.labs.i18n.ListFormatSymbols_ha_Latn_NG', 'goog.labs.i18n.ListFormatSymbols_haw_US', 'goog.labs.i18n.ListFormatSymbols_he_IL', 'goog.labs.i18n.ListFormatSymbols_hi_IN', 'goog.labs.i18n.ListFormatSymbols_hr_BA', 'goog.labs.i18n.ListFormatSymbols_hr_HR', 'goog.labs.i18n.ListFormatSymbols_hu_HU', 'goog.labs.i18n.ListFormatSymbols_hy_AM', 'goog.labs.i18n.ListFormatSymbols_id_ID', 'goog.labs.i18n.ListFormatSymbols_ig', 'goog.labs.i18n.ListFormatSymbols_ig_NG', 'goog.labs.i18n.ListFormatSymbols_ii', 'goog.labs.i18n.ListFormatSymbols_ii_CN', 'goog.labs.i18n.ListFormatSymbols_is_IS', 'goog.labs.i18n.ListFormatSymbols_it_CH', 'goog.labs.i18n.ListFormatSymbols_it_IT', 'goog.labs.i18n.ListFormatSymbols_it_SM', 'goog.labs.i18n.ListFormatSymbols_ja_JP', 'goog.labs.i18n.ListFormatSymbols_jgo', 'goog.labs.i18n.ListFormatSymbols_jgo_CM', 'goog.labs.i18n.ListFormatSymbols_jmc', 'goog.labs.i18n.ListFormatSymbols_jmc_TZ', 'goog.labs.i18n.ListFormatSymbols_ka_GE', 'goog.labs.i18n.ListFormatSymbols_kab', 'goog.labs.i18n.ListFormatSymbols_kab_DZ', 'goog.labs.i18n.ListFormatSymbols_kam', 'goog.labs.i18n.ListFormatSymbols_kam_KE', 'goog.labs.i18n.ListFormatSymbols_kde', 'goog.labs.i18n.ListFormatSymbols_kde_TZ', 'goog.labs.i18n.ListFormatSymbols_kea', 'goog.labs.i18n.ListFormatSymbols_kea_CV', 'goog.labs.i18n.ListFormatSymbols_khq', 'goog.labs.i18n.ListFormatSymbols_khq_ML', 'goog.labs.i18n.ListFormatSymbols_ki', 'goog.labs.i18n.ListFormatSymbols_ki_KE', 'goog.labs.i18n.ListFormatSymbols_kk_Cyrl', 'goog.labs.i18n.ListFormatSymbols_kk_Cyrl_KZ', 'goog.labs.i18n.ListFormatSymbols_kkj', 'goog.labs.i18n.ListFormatSymbols_kkj_CM', 'goog.labs.i18n.ListFormatSymbols_kl', 'goog.labs.i18n.ListFormatSymbols_kl_GL', 'goog.labs.i18n.ListFormatSymbols_kln', 'goog.labs.i18n.ListFormatSymbols_kln_KE', 'goog.labs.i18n.ListFormatSymbols_km_KH', 'goog.labs.i18n.ListFormatSymbols_kn_IN', 'goog.labs.i18n.ListFormatSymbols_ko_KP', 'goog.labs.i18n.ListFormatSymbols_ko_KR', 'goog.labs.i18n.ListFormatSymbols_kok', 'goog.labs.i18n.ListFormatSymbols_kok_IN', 'goog.labs.i18n.ListFormatSymbols_ks', 'goog.labs.i18n.ListFormatSymbols_ks_Arab', 'goog.labs.i18n.ListFormatSymbols_ks_Arab_IN', 'goog.labs.i18n.ListFormatSymbols_ksb', 'goog.labs.i18n.ListFormatSymbols_ksb_TZ', 'goog.labs.i18n.ListFormatSymbols_ksf', 'goog.labs.i18n.ListFormatSymbols_ksf_CM', 'goog.labs.i18n.ListFormatSymbols_kw', 'goog.labs.i18n.ListFormatSymbols_kw_GB', 'goog.labs.i18n.ListFormatSymbols_ky_Cyrl', 'goog.labs.i18n.ListFormatSymbols_ky_Cyrl_KG', 'goog.labs.i18n.ListFormatSymbols_lag', 'goog.labs.i18n.ListFormatSymbols_lag_TZ', 'goog.labs.i18n.ListFormatSymbols_lg', 'goog.labs.i18n.ListFormatSymbols_lg_UG', 'goog.labs.i18n.ListFormatSymbols_lkt', 'goog.labs.i18n.ListFormatSymbols_lkt_US', 'goog.labs.i18n.ListFormatSymbols_ln_AO', 'goog.labs.i18n.ListFormatSymbols_ln_CD', 'goog.labs.i18n.ListFormatSymbols_ln_CF', 'goog.labs.i18n.ListFormatSymbols_ln_CG', 'goog.labs.i18n.ListFormatSymbols_lo_LA', 'goog.labs.i18n.ListFormatSymbols_lt_LT', 'goog.labs.i18n.ListFormatSymbols_lu', 'goog.labs.i18n.ListFormatSymbols_lu_CD', 'goog.labs.i18n.ListFormatSymbols_luo', 'goog.labs.i18n.ListFormatSymbols_luo_KE', 'goog.labs.i18n.ListFormatSymbols_luy', 'goog.labs.i18n.ListFormatSymbols_luy_KE', 'goog.labs.i18n.ListFormatSymbols_lv_LV', 'goog.labs.i18n.ListFormatSymbols_mas', 'goog.labs.i18n.ListFormatSymbols_mas_KE', 'goog.labs.i18n.ListFormatSymbols_mas_TZ', 'goog.labs.i18n.ListFormatSymbols_mer', 'goog.labs.i18n.ListFormatSymbols_mer_KE', 'goog.labs.i18n.ListFormatSymbols_mfe', 'goog.labs.i18n.ListFormatSymbols_mfe_MU', 'goog.labs.i18n.ListFormatSymbols_mg', 'goog.labs.i18n.ListFormatSymbols_mg_MG', 'goog.labs.i18n.ListFormatSymbols_mgh', 'goog.labs.i18n.ListFormatSymbols_mgh_MZ', 'goog.labs.i18n.ListFormatSymbols_mgo', 'goog.labs.i18n.ListFormatSymbols_mgo_CM', 'goog.labs.i18n.ListFormatSymbols_mk_MK', 'goog.labs.i18n.ListFormatSymbols_ml_IN', 'goog.labs.i18n.ListFormatSymbols_mn_Cyrl', 'goog.labs.i18n.ListFormatSymbols_mn_Cyrl_MN', 'goog.labs.i18n.ListFormatSymbols_mr_IN', 'goog.labs.i18n.ListFormatSymbols_ms_Latn', 'goog.labs.i18n.ListFormatSymbols_ms_Latn_BN', 'goog.labs.i18n.ListFormatSymbols_ms_Latn_MY', 'goog.labs.i18n.ListFormatSymbols_ms_Latn_SG', 'goog.labs.i18n.ListFormatSymbols_mt_MT', 'goog.labs.i18n.ListFormatSymbols_mua', 'goog.labs.i18n.ListFormatSymbols_mua_CM', 'goog.labs.i18n.ListFormatSymbols_my_MM', 'goog.labs.i18n.ListFormatSymbols_naq', 'goog.labs.i18n.ListFormatSymbols_naq_NA', 'goog.labs.i18n.ListFormatSymbols_nb_NO', 'goog.labs.i18n.ListFormatSymbols_nb_SJ', 'goog.labs.i18n.ListFormatSymbols_nd', 'goog.labs.i18n.ListFormatSymbols_nd_ZW', 'goog.labs.i18n.ListFormatSymbols_ne_IN', 'goog.labs.i18n.ListFormatSymbols_ne_NP', 'goog.labs.i18n.ListFormatSymbols_nl_AW', 'goog.labs.i18n.ListFormatSymbols_nl_BE', 'goog.labs.i18n.ListFormatSymbols_nl_BQ', 'goog.labs.i18n.ListFormatSymbols_nl_CW', 'goog.labs.i18n.ListFormatSymbols_nl_NL', 'goog.labs.i18n.ListFormatSymbols_nl_SR', 'goog.labs.i18n.ListFormatSymbols_nl_SX', 'goog.labs.i18n.ListFormatSymbols_nmg', 'goog.labs.i18n.ListFormatSymbols_nmg_CM', 'goog.labs.i18n.ListFormatSymbols_nn', 'goog.labs.i18n.ListFormatSymbols_nn_NO', 'goog.labs.i18n.ListFormatSymbols_nnh', 'goog.labs.i18n.ListFormatSymbols_nnh_CM', 'goog.labs.i18n.ListFormatSymbols_nus', 'goog.labs.i18n.ListFormatSymbols_nus_SD', 'goog.labs.i18n.ListFormatSymbols_nyn', 'goog.labs.i18n.ListFormatSymbols_nyn_UG', 'goog.labs.i18n.ListFormatSymbols_om', 'goog.labs.i18n.ListFormatSymbols_om_ET', 'goog.labs.i18n.ListFormatSymbols_om_KE', 'goog.labs.i18n.ListFormatSymbols_or_IN', 'goog.labs.i18n.ListFormatSymbols_pa_Arab', 'goog.labs.i18n.ListFormatSymbols_pa_Arab_PK', 'goog.labs.i18n.ListFormatSymbols_pa_Guru', 'goog.labs.i18n.ListFormatSymbols_pa_Guru_IN', 'goog.labs.i18n.ListFormatSymbols_pl_PL', 'goog.labs.i18n.ListFormatSymbols_ps', 'goog.labs.i18n.ListFormatSymbols_ps_AF', 'goog.labs.i18n.ListFormatSymbols_pt_AO', 'goog.labs.i18n.ListFormatSymbols_pt_CV', 'goog.labs.i18n.ListFormatSymbols_pt_GW', 'goog.labs.i18n.ListFormatSymbols_pt_MO', 'goog.labs.i18n.ListFormatSymbols_pt_MZ', 'goog.labs.i18n.ListFormatSymbols_pt_ST', 'goog.labs.i18n.ListFormatSymbols_pt_TL', 'goog.labs.i18n.ListFormatSymbols_rm', 'goog.labs.i18n.ListFormatSymbols_rm_CH', 'goog.labs.i18n.ListFormatSymbols_rn', 'goog.labs.i18n.ListFormatSymbols_rn_BI', 'goog.labs.i18n.ListFormatSymbols_ro_MD', 'goog.labs.i18n.ListFormatSymbols_ro_RO', 'goog.labs.i18n.ListFormatSymbols_rof', 'goog.labs.i18n.ListFormatSymbols_rof_TZ', 'goog.labs.i18n.ListFormatSymbols_ru_BY', 'goog.labs.i18n.ListFormatSymbols_ru_KG', 'goog.labs.i18n.ListFormatSymbols_ru_KZ', 'goog.labs.i18n.ListFormatSymbols_ru_MD', 'goog.labs.i18n.ListFormatSymbols_ru_RU', 'goog.labs.i18n.ListFormatSymbols_ru_UA', 'goog.labs.i18n.ListFormatSymbols_rw', 'goog.labs.i18n.ListFormatSymbols_rw_RW', 'goog.labs.i18n.ListFormatSymbols_rwk', 'goog.labs.i18n.ListFormatSymbols_rwk_TZ', 'goog.labs.i18n.ListFormatSymbols_saq', 'goog.labs.i18n.ListFormatSymbols_saq_KE', 'goog.labs.i18n.ListFormatSymbols_sbp', 'goog.labs.i18n.ListFormatSymbols_sbp_TZ', 'goog.labs.i18n.ListFormatSymbols_seh', 'goog.labs.i18n.ListFormatSymbols_seh_MZ', 'goog.labs.i18n.ListFormatSymbols_ses', 'goog.labs.i18n.ListFormatSymbols_ses_ML', 'goog.labs.i18n.ListFormatSymbols_sg', 'goog.labs.i18n.ListFormatSymbols_sg_CF', 'goog.labs.i18n.ListFormatSymbols_shi', 'goog.labs.i18n.ListFormatSymbols_shi_Latn', 'goog.labs.i18n.ListFormatSymbols_shi_Latn_MA', 'goog.labs.i18n.ListFormatSymbols_shi_Tfng', 'goog.labs.i18n.ListFormatSymbols_shi_Tfng_MA', 'goog.labs.i18n.ListFormatSymbols_si_LK', 'goog.labs.i18n.ListFormatSymbols_sk_SK', 'goog.labs.i18n.ListFormatSymbols_sl_SI', 'goog.labs.i18n.ListFormatSymbols_sn', 'goog.labs.i18n.ListFormatSymbols_sn_ZW', 'goog.labs.i18n.ListFormatSymbols_so', 'goog.labs.i18n.ListFormatSymbols_so_DJ', 'goog.labs.i18n.ListFormatSymbols_so_ET', 'goog.labs.i18n.ListFormatSymbols_so_KE', 'goog.labs.i18n.ListFormatSymbols_so_SO', 'goog.labs.i18n.ListFormatSymbols_sq_AL', 'goog.labs.i18n.ListFormatSymbols_sq_MK', 'goog.labs.i18n.ListFormatSymbols_sq_XK', 'goog.labs.i18n.ListFormatSymbols_sr_Cyrl', 'goog.labs.i18n.ListFormatSymbols_sr_Cyrl_BA', 'goog.labs.i18n.ListFormatSymbols_sr_Cyrl_ME', 'goog.labs.i18n.ListFormatSymbols_sr_Cyrl_RS', 'goog.labs.i18n.ListFormatSymbols_sr_Cyrl_XK', 'goog.labs.i18n.ListFormatSymbols_sr_Latn', 'goog.labs.i18n.ListFormatSymbols_sr_Latn_BA', 'goog.labs.i18n.ListFormatSymbols_sr_Latn_ME', 'goog.labs.i18n.ListFormatSymbols_sr_Latn_RS', 'goog.labs.i18n.ListFormatSymbols_sr_Latn_XK', 'goog.labs.i18n.ListFormatSymbols_sv_AX', 'goog.labs.i18n.ListFormatSymbols_sv_FI', 'goog.labs.i18n.ListFormatSymbols_sv_SE', 'goog.labs.i18n.ListFormatSymbols_sw_KE', 'goog.labs.i18n.ListFormatSymbols_sw_TZ', 'goog.labs.i18n.ListFormatSymbols_sw_UG', 'goog.labs.i18n.ListFormatSymbols_swc', 'goog.labs.i18n.ListFormatSymbols_swc_CD', 'goog.labs.i18n.ListFormatSymbols_ta_IN', 'goog.labs.i18n.ListFormatSymbols_ta_LK', 'goog.labs.i18n.ListFormatSymbols_ta_MY', 'goog.labs.i18n.ListFormatSymbols_ta_SG', 'goog.labs.i18n.ListFormatSymbols_te_IN', 'goog.labs.i18n.ListFormatSymbols_teo', 'goog.labs.i18n.ListFormatSymbols_teo_KE', 'goog.labs.i18n.ListFormatSymbols_teo_UG', 'goog.labs.i18n.ListFormatSymbols_th_TH', 'goog.labs.i18n.ListFormatSymbols_ti', 'goog.labs.i18n.ListFormatSymbols_ti_ER', 'goog.labs.i18n.ListFormatSymbols_ti_ET', 'goog.labs.i18n.ListFormatSymbols_to', 'goog.labs.i18n.ListFormatSymbols_to_TO', 'goog.labs.i18n.ListFormatSymbols_tr_CY', 'goog.labs.i18n.ListFormatSymbols_tr_TR', 'goog.labs.i18n.ListFormatSymbols_twq', 'goog.labs.i18n.ListFormatSymbols_twq_NE', 'goog.labs.i18n.ListFormatSymbols_tzm', 'goog.labs.i18n.ListFormatSymbols_tzm_Latn', 'goog.labs.i18n.ListFormatSymbols_tzm_Latn_MA', 'goog.labs.i18n.ListFormatSymbols_ug', 'goog.labs.i18n.ListFormatSymbols_ug_Arab', 'goog.labs.i18n.ListFormatSymbols_ug_Arab_CN', 'goog.labs.i18n.ListFormatSymbols_uk_UA', 'goog.labs.i18n.ListFormatSymbols_ur_IN', 'goog.labs.i18n.ListFormatSymbols_ur_PK', 'goog.labs.i18n.ListFormatSymbols_uz_Arab', 'goog.labs.i18n.ListFormatSymbols_uz_Arab_AF', 'goog.labs.i18n.ListFormatSymbols_uz_Cyrl', 'goog.labs.i18n.ListFormatSymbols_uz_Cyrl_UZ', 'goog.labs.i18n.ListFormatSymbols_uz_Latn', 'goog.labs.i18n.ListFormatSymbols_uz_Latn_UZ', 'goog.labs.i18n.ListFormatSymbols_vai', 'goog.labs.i18n.ListFormatSymbols_vai_Latn', 'goog.labs.i18n.ListFormatSymbols_vai_Latn_LR', 'goog.labs.i18n.ListFormatSymbols_vai_Vaii', 'goog.labs.i18n.ListFormatSymbols_vai_Vaii_LR', 'goog.labs.i18n.ListFormatSymbols_vi_VN', 'goog.labs.i18n.ListFormatSymbols_vun', 'goog.labs.i18n.ListFormatSymbols_vun_TZ', 'goog.labs.i18n.ListFormatSymbols_xog', 'goog.labs.i18n.ListFormatSymbols_xog_UG', 'goog.labs.i18n.ListFormatSymbols_yav', 'goog.labs.i18n.ListFormatSymbols_yav_CM', 'goog.labs.i18n.ListFormatSymbols_yo', 'goog.labs.i18n.ListFormatSymbols_yo_BJ', 'goog.labs.i18n.ListFormatSymbols_yo_NG', 'goog.labs.i18n.ListFormatSymbols_zgh', 'goog.labs.i18n.ListFormatSymbols_zgh_MA', 'goog.labs.i18n.ListFormatSymbols_zh_Hans', 'goog.labs.i18n.ListFormatSymbols_zh_Hans_CN', 'goog.labs.i18n.ListFormatSymbols_zh_Hans_HK', 'goog.labs.i18n.ListFormatSymbols_zh_Hans_MO', 'goog.labs.i18n.ListFormatSymbols_zh_Hans_SG', 'goog.labs.i18n.ListFormatSymbols_zh_Hant', 'goog.labs.i18n.ListFormatSymbols_zh_Hant_HK', 'goog.labs.i18n.ListFormatSymbols_zh_Hant_MO', 'goog.labs.i18n.ListFormatSymbols_zh_Hant_TW', 'goog.labs.i18n.ListFormatSymbols_zu_ZA'], ['goog.labs.i18n.ListFormatSymbols'], false);
goog.addDependency('labs/mock/mock.js', ['goog.labs.mock', 'goog.labs.mock.VerificationError'], ['goog.array', 'goog.asserts', 'goog.debug', 'goog.debug.Error', 'goog.functions', 'goog.object'], false);
goog.addDependency('labs/mock/mock_test.js', ['goog.labs.mockTest'], ['goog.array', 'goog.labs.mock', 'goog.labs.mock.VerificationError', 'goog.labs.testing.AnythingMatcher', 'goog.labs.testing.GreaterThanMatcher', 'goog.string', 'goog.testing.jsunit'], false);
goog.addDependency('labs/net/image.js', ['goog.labs.net.image'], ['goog.Promise', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.net.EventType', 'goog.userAgent'], false);
goog.addDependency('labs/net/image_test.js', ['goog.labs.net.imageTest'], ['goog.events', 'goog.labs.net.image', 'goog.string', 'goog.testing.AsyncTestCase', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('labs/net/webchannel.js', ['goog.net.WebChannel'], ['goog.events', 'goog.events.Event'], false);
goog.addDependency('labs/net/webchannel/basetestchannel.js', ['goog.labs.net.webChannel.BaseTestChannel'], ['goog.labs.net.webChannel.Channel', 'goog.labs.net.webChannel.ChannelRequest', 'goog.labs.net.webChannel.requestStats', 'goog.labs.net.webChannel.requestStats.Stat'], false);
goog.addDependency('labs/net/webchannel/channel.js', ['goog.labs.net.webChannel.Channel'], [], false);
goog.addDependency('labs/net/webchannel/channelrequest.js', ['goog.labs.net.webChannel.ChannelRequest'], ['goog.Timer', 'goog.async.Throttle', 'goog.events.EventHandler', 'goog.labs.net.webChannel.requestStats', 'goog.labs.net.webChannel.requestStats.ServerReachability', 'goog.labs.net.webChannel.requestStats.Stat', 'goog.net.ErrorCode', 'goog.net.EventType', 'goog.net.XmlHttp', 'goog.object', 'goog.uri.utils.StandardQueryParam', 'goog.userAgent'], false);
goog.addDependency('labs/net/webchannel/channelrequest_test.js', ['goog.labs.net.webChannel.channelRequestTest'], ['goog.Uri', 'goog.functions', 'goog.labs.net.webChannel.ChannelRequest', 'goog.labs.net.webChannel.WebChannelDebug', 'goog.labs.net.webChannel.requestStats', 'goog.labs.net.webChannel.requestStats.ServerReachability', 'goog.testing.MockClock', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.net.XhrIo', 'goog.testing.recordFunction'], false);
goog.addDependency('labs/net/webchannel/connectionstate.js', ['goog.labs.net.webChannel.ConnectionState'], [], false);
goog.addDependency('labs/net/webchannel/forwardchannelrequestpool.js', ['goog.labs.net.webChannel.ForwardChannelRequestPool'], ['goog.array', 'goog.string', 'goog.structs.Set'], false);
goog.addDependency('labs/net/webchannel/forwardchannelrequestpool_test.js', ['goog.labs.net.webChannel.forwardChannelRequestPoolTest'], ['goog.labs.net.webChannel.ChannelRequest', 'goog.labs.net.webChannel.ForwardChannelRequestPool', 'goog.testing.PropertyReplacer', 'goog.testing.asserts', 'goog.testing.jsunit'], false);
goog.addDependency('labs/net/webchannel/netutils.js', ['goog.labs.net.webChannel.netUtils'], ['goog.Uri', 'goog.labs.net.webChannel.WebChannelDebug'], false);
goog.addDependency('labs/net/webchannel/requeststats.js', ['goog.labs.net.webChannel.requestStats', 'goog.labs.net.webChannel.requestStats.Event', 'goog.labs.net.webChannel.requestStats.ServerReachability', 'goog.labs.net.webChannel.requestStats.ServerReachabilityEvent', 'goog.labs.net.webChannel.requestStats.Stat', 'goog.labs.net.webChannel.requestStats.StatEvent', 'goog.labs.net.webChannel.requestStats.TimingEvent'], ['goog.events.Event', 'goog.events.EventTarget'], false);
goog.addDependency('labs/net/webchannel/webchannelbase.js', ['goog.labs.net.webChannel.WebChannelBase'], ['goog.Uri', 'goog.array', 'goog.asserts', 'goog.debug.TextFormatter', 'goog.json', 'goog.labs.net.webChannel.BaseTestChannel', 'goog.labs.net.webChannel.Channel', 'goog.labs.net.webChannel.ChannelRequest', 'goog.labs.net.webChannel.ConnectionState', 'goog.labs.net.webChannel.ForwardChannelRequestPool', 'goog.labs.net.webChannel.WebChannelDebug', 'goog.labs.net.webChannel.Wire', 'goog.labs.net.webChannel.WireV8', 'goog.labs.net.webChannel.netUtils', 'goog.labs.net.webChannel.requestStats', 'goog.labs.net.webChannel.requestStats.Stat', 'goog.log', 'goog.net.XhrIo', 'goog.object', 'goog.string', 'goog.structs', 'goog.structs.CircularBuffer'], false);
goog.addDependency('labs/net/webchannel/webchannelbase_test.js', ['goog.labs.net.webChannel.webChannelBaseTest'], ['goog.Timer', 'goog.array', 'goog.dom', 'goog.functions', 'goog.json', 'goog.labs.net.webChannel.ChannelRequest', 'goog.labs.net.webChannel.ForwardChannelRequestPool', 'goog.labs.net.webChannel.WebChannelBase', 'goog.labs.net.webChannel.WebChannelBaseTransport', 'goog.labs.net.webChannel.WebChannelDebug', 'goog.labs.net.webChannel.Wire', 'goog.labs.net.webChannel.netUtils', 'goog.labs.net.webChannel.requestStats', 'goog.labs.net.webChannel.requestStats.Stat', 'goog.structs.Map', 'goog.testing.MockClock', 'goog.testing.PropertyReplacer', 'goog.testing.asserts', 'goog.testing.jsunit'], false);
goog.addDependency('labs/net/webchannel/webchannelbasetransport.js', ['goog.labs.net.webChannel.WebChannelBaseTransport'], ['goog.asserts', 'goog.events.EventTarget', 'goog.labs.net.webChannel.WebChannelBase', 'goog.log', 'goog.net.WebChannel', 'goog.net.WebChannelTransport', 'goog.string.path'], false);
goog.addDependency('labs/net/webchannel/webchannelbasetransport_test.js', ['goog.labs.net.webChannel.webChannelBaseTransportTest'], ['goog.events', 'goog.labs.net.webChannel.WebChannelBaseTransport', 'goog.net.WebChannel', 'goog.testing.jsunit'], false);
goog.addDependency('labs/net/webchannel/webchanneldebug.js', ['goog.labs.net.webChannel.WebChannelDebug'], ['goog.json', 'goog.log'], false);
goog.addDependency('labs/net/webchannel/wire.js', ['goog.labs.net.webChannel.Wire'], [], false);
goog.addDependency('labs/net/webchannel/wirev8.js', ['goog.labs.net.webChannel.WireV8'], ['goog.asserts', 'goog.json', 'goog.json.EvalJsonProcessor', 'goog.structs'], false);
goog.addDependency('labs/net/webchannel/wirev8_test.js', ['goog.labs.net.webChannel.WireV8Test'], ['goog.labs.net.webChannel.WireV8', 'goog.testing.jsunit'], false);
goog.addDependency('labs/net/webchanneltransport.js', ['goog.net.WebChannelTransport'], [], false);
goog.addDependency('labs/net/webchanneltransportfactory.js', ['goog.net.createWebChannelTransport'], ['goog.functions', 'goog.labs.net.webChannel.WebChannelBaseTransport'], false);
goog.addDependency('labs/net/xhr.js', ['goog.labs.net.xhr', 'goog.labs.net.xhr.Error', 'goog.labs.net.xhr.HttpError', 'goog.labs.net.xhr.Options', 'goog.labs.net.xhr.PostData', 'goog.labs.net.xhr.TimeoutError'], ['goog.Promise', 'goog.debug.Error', 'goog.json', 'goog.net.HttpStatus', 'goog.net.XmlHttp', 'goog.string', 'goog.uri.utils'], false);
goog.addDependency('labs/net/xhr_test.js', ['goog.labs.net.xhrTest'], ['goog.Promise', 'goog.labs.net.xhr', 'goog.net.XmlHttp', 'goog.string', 'goog.testing.AsyncTestCase', 'goog.testing.MockClock', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('labs/object/object.js', ['goog.labs.object'], [], false);
goog.addDependency('labs/object/object_test.js', ['goog.labs.objectTest'], ['goog.labs.object', 'goog.testing.jsunit'], false);
goog.addDependency('labs/pubsub/broadcastpubsub.js', ['goog.labs.pubsub.BroadcastPubSub'], ['goog.Disposable', 'goog.Timer', 'goog.array', 'goog.async.run', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.json', 'goog.log', 'goog.math', 'goog.pubsub.PubSub', 'goog.storage.Storage', 'goog.storage.mechanism.HTML5LocalStorage', 'goog.string', 'goog.userAgent'], false);
goog.addDependency('labs/pubsub/broadcastpubsub_test.js', ['goog.labs.pubsub.BroadcastPubSubTest'], ['goog.array', 'goog.debug.Logger', 'goog.json', 'goog.labs.pubsub.BroadcastPubSub', 'goog.storage.Storage', 'goog.structs.Map', 'goog.testing.MockClock', 'goog.testing.MockControl', 'goog.testing.events', 'goog.testing.events.Event', 'goog.testing.jsunit', 'goog.testing.mockmatchers', 'goog.testing.mockmatchers.ArgumentMatcher', 'goog.testing.recordFunction', 'goog.userAgent'], false);
goog.addDependency('labs/storage/boundedcollectablestorage.js', ['goog.labs.storage.BoundedCollectableStorage'], ['goog.array', 'goog.asserts', 'goog.iter', 'goog.storage.CollectableStorage', 'goog.storage.ErrorCode', 'goog.storage.ExpiringStorage'], false);
goog.addDependency('labs/storage/boundedcollectablestorage_test.js', ['goog.labs.storage.BoundedCollectableStorageTest'], ['goog.labs.storage.BoundedCollectableStorage', 'goog.storage.collectableStorageTester', 'goog.storage.storage_test', 'goog.testing.MockClock', 'goog.testing.jsunit', 'goog.testing.storage.FakeMechanism'], false);
goog.addDependency('labs/structs/map.js', ['goog.labs.structs.Map'], ['goog.array', 'goog.asserts', 'goog.labs.object', 'goog.object'], false);
goog.addDependency('labs/structs/map_perf.js', ['goog.labs.structs.mapPerf'], ['goog.dom', 'goog.labs.structs.Map', 'goog.structs.Map', 'goog.testing.PerformanceTable', 'goog.testing.jsunit'], false);
goog.addDependency('labs/structs/map_test.js', ['goog.labs.structs.MapTest'], ['goog.labs.structs.Map', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit'], false);
goog.addDependency('labs/structs/multimap.js', ['goog.labs.structs.Multimap'], ['goog.array', 'goog.labs.object', 'goog.labs.structs.Map'], false);
goog.addDependency('labs/structs/multimap_test.js', ['goog.labs.structs.MultimapTest'], ['goog.labs.structs.Map', 'goog.labs.structs.Multimap', 'goog.testing.jsunit'], false);
goog.addDependency('labs/style/pixeldensitymonitor.js', ['goog.labs.style.PixelDensityMonitor', 'goog.labs.style.PixelDensityMonitor.Density', 'goog.labs.style.PixelDensityMonitor.EventType'], ['goog.events', 'goog.events.EventTarget'], false);
goog.addDependency('labs/style/pixeldensitymonitor_test.js', ['goog.labs.style.PixelDensityMonitorTest'], ['goog.array', 'goog.dom.DomHelper', 'goog.events', 'goog.labs.style.PixelDensityMonitor', 'goog.testing.MockControl', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('labs/testing/assertthat.js', ['goog.labs.testing.MatcherError', 'goog.labs.testing.assertThat'], ['goog.asserts', 'goog.debug.Error', 'goog.labs.testing.Matcher'], false);
goog.addDependency('labs/testing/assertthat_test.js', ['goog.labs.testing.assertThatTest'], ['goog.labs.testing.MatcherError', 'goog.labs.testing.assertThat', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('labs/testing/decoratormatcher.js', ['goog.labs.testing.AnythingMatcher'], ['goog.labs.testing.Matcher'], false);
goog.addDependency('labs/testing/decoratormatcher_test.js', ['goog.labs.testing.decoratorMatcherTest'], ['goog.labs.testing.AnythingMatcher', 'goog.labs.testing.GreaterThanMatcher', 'goog.labs.testing.MatcherError', 'goog.labs.testing.assertThat', 'goog.testing.jsunit'], false);
goog.addDependency('labs/testing/dictionarymatcher.js', ['goog.labs.testing.HasEntriesMatcher', 'goog.labs.testing.HasEntryMatcher', 'goog.labs.testing.HasKeyMatcher', 'goog.labs.testing.HasValueMatcher'], ['goog.array', 'goog.asserts', 'goog.labs.testing.Matcher', 'goog.string'], false);
goog.addDependency('labs/testing/dictionarymatcher_test.js', ['goog.labs.testing.dictionaryMatcherTest'], ['goog.labs.testing.HasEntryMatcher', 'goog.labs.testing.MatcherError', 'goog.labs.testing.assertThat', 'goog.testing.jsunit'], false);
goog.addDependency('labs/testing/environment.js', ['goog.labs.testing.Environment'], ['goog.array', 'goog.debug.Console', 'goog.testing.MockClock', 'goog.testing.MockControl', 'goog.testing.TestCase', 'goog.testing.jsunit'], false);
goog.addDependency('labs/testing/environment_test.js', ['goog.labs.testing.environmentTest'], ['goog.labs.testing.Environment', 'goog.testing.MockControl', 'goog.testing.TestCase', 'goog.testing.jsunit'], false);
goog.addDependency('labs/testing/environment_usage_test.js', ['goog.labs.testing.environmentUsageTest'], ['goog.labs.testing.Environment'], false);
goog.addDependency('labs/testing/logicmatcher.js', ['goog.labs.testing.AllOfMatcher', 'goog.labs.testing.AnyOfMatcher', 'goog.labs.testing.IsNotMatcher'], ['goog.array', 'goog.labs.testing.Matcher'], false);
goog.addDependency('labs/testing/logicmatcher_test.js', ['goog.labs.testing.logicMatcherTest'], ['goog.labs.testing.AllOfMatcher', 'goog.labs.testing.GreaterThanMatcher', 'goog.labs.testing.MatcherError', 'goog.labs.testing.assertThat', 'goog.testing.jsunit'], false);
goog.addDependency('labs/testing/matcher.js', ['goog.labs.testing.Matcher'], [], false);
goog.addDependency('labs/testing/numbermatcher.js', ['goog.labs.testing.CloseToMatcher', 'goog.labs.testing.EqualToMatcher', 'goog.labs.testing.GreaterThanEqualToMatcher', 'goog.labs.testing.GreaterThanMatcher', 'goog.labs.testing.LessThanEqualToMatcher', 'goog.labs.testing.LessThanMatcher'], ['goog.asserts', 'goog.labs.testing.Matcher'], false);
goog.addDependency('labs/testing/numbermatcher_test.js', ['goog.labs.testing.numberMatcherTest'], ['goog.labs.testing.LessThanMatcher', 'goog.labs.testing.MatcherError', 'goog.labs.testing.assertThat', 'goog.testing.jsunit'], false);
goog.addDependency('labs/testing/objectmatcher.js', ['goog.labs.testing.HasPropertyMatcher', 'goog.labs.testing.InstanceOfMatcher', 'goog.labs.testing.IsNullMatcher', 'goog.labs.testing.IsNullOrUndefinedMatcher', 'goog.labs.testing.IsUndefinedMatcher', 'goog.labs.testing.ObjectEqualsMatcher'], ['goog.labs.testing.Matcher', 'goog.string'], false);
goog.addDependency('labs/testing/objectmatcher_test.js', ['goog.labs.testing.objectMatcherTest'], ['goog.labs.testing.MatcherError', 'goog.labs.testing.ObjectEqualsMatcher', 'goog.labs.testing.assertThat', 'goog.testing.jsunit'], false);
goog.addDependency('labs/testing/stringmatcher.js', ['goog.labs.testing.ContainsStringMatcher', 'goog.labs.testing.EndsWithMatcher', 'goog.labs.testing.EqualToIgnoringCaseMatcher', 'goog.labs.testing.EqualToIgnoringWhitespaceMatcher', 'goog.labs.testing.EqualsMatcher', 'goog.labs.testing.RegexMatcher', 'goog.labs.testing.StartsWithMatcher', 'goog.labs.testing.StringContainsInOrderMatcher'], ['goog.asserts', 'goog.labs.testing.Matcher', 'goog.string'], false);
goog.addDependency('labs/testing/stringmatcher_test.js', ['goog.labs.testing.stringMatcherTest'], ['goog.labs.testing.MatcherError', 'goog.labs.testing.StringContainsInOrderMatcher', 'goog.labs.testing.assertThat', 'goog.testing.jsunit'], false);
goog.addDependency('labs/useragent/browser.js', ['goog.labs.userAgent.browser'], ['goog.array', 'goog.labs.userAgent.util', 'goog.object', 'goog.string'], false);
goog.addDependency('labs/useragent/browser_test.js', ['goog.labs.userAgent.browserTest'], ['goog.labs.userAgent.browser', 'goog.labs.userAgent.testAgents', 'goog.labs.userAgent.util', 'goog.testing.jsunit'], false);
goog.addDependency('labs/useragent/device.js', ['goog.labs.userAgent.device'], ['goog.labs.userAgent.util'], false);
goog.addDependency('labs/useragent/device_test.js', ['goog.labs.userAgent.deviceTest'], ['goog.labs.userAgent.device', 'goog.labs.userAgent.testAgents', 'goog.labs.userAgent.util', 'goog.testing.jsunit'], false);
goog.addDependency('labs/useragent/engine.js', ['goog.labs.userAgent.engine'], ['goog.array', 'goog.labs.userAgent.util', 'goog.string'], false);
goog.addDependency('labs/useragent/engine_test.js', ['goog.labs.userAgent.engineTest'], ['goog.labs.userAgent.engine', 'goog.labs.userAgent.testAgents', 'goog.labs.userAgent.util', 'goog.testing.jsunit'], false);
goog.addDependency('labs/useragent/platform.js', ['goog.labs.userAgent.platform'], ['goog.labs.userAgent.util', 'goog.string'], false);
goog.addDependency('labs/useragent/platform_test.js', ['goog.labs.userAgent.platformTest'], ['goog.labs.userAgent.platform', 'goog.labs.userAgent.testAgents', 'goog.labs.userAgent.util', 'goog.testing.jsunit'], false);
goog.addDependency('labs/useragent/test_agents.js', ['goog.labs.userAgent.testAgents'], [], false);
goog.addDependency('labs/useragent/util.js', ['goog.labs.userAgent.util'], ['goog.string'], false);
goog.addDependency('labs/useragent/util_test.js', ['goog.labs.userAgent.utilTest'], ['goog.functions', 'goog.labs.userAgent.testAgents', 'goog.labs.userAgent.util', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit'], false);
goog.addDependency('locale/countries.js', ['goog.locale.countries'], [], false);
goog.addDependency('locale/countrylanguagenames_test.js', ['goog.locale.countryLanguageNamesTest'], ['goog.locale', 'goog.testing.jsunit'], false);
goog.addDependency('locale/defaultlocalenameconstants.js', ['goog.locale.defaultLocaleNameConstants'], [], false);
goog.addDependency('locale/genericfontnames.js', ['goog.locale.genericFontNames'], [], false);
goog.addDependency('locale/genericfontnames_test.js', ['goog.locale.genericFontNamesTest'], ['goog.locale.genericFontNames', 'goog.testing.jsunit'], false);
goog.addDependency('locale/genericfontnamesdata.js', ['goog.locale.genericFontNamesData'], [], false);
goog.addDependency('locale/locale.js', ['goog.locale'], ['goog.locale.nativeNameConstants'], false);
goog.addDependency('locale/nativenameconstants.js', ['goog.locale.nativeNameConstants'], [], false);
goog.addDependency('locale/scriptToLanguages.js', ['goog.locale.scriptToLanguages'], ['goog.locale'], false);
goog.addDependency('locale/timezonedetection.js', ['goog.locale.timeZoneDetection'], ['goog.locale', 'goog.locale.TimeZoneFingerprint'], false);
goog.addDependency('locale/timezonedetection_test.js', ['goog.locale.timeZoneDetectionTest'], ['goog.locale.timeZoneDetection', 'goog.testing.jsunit'], false);
goog.addDependency('locale/timezonefingerprint.js', ['goog.locale.TimeZoneFingerprint'], [], false);
goog.addDependency('locale/timezonelist.js', ['goog.locale.TimeZoneList'], ['goog.locale'], false);
goog.addDependency('locale/timezonelist_test.js', ['goog.locale.TimeZoneListTest'], ['goog.locale', 'goog.locale.TimeZoneList', 'goog.testing.jsunit'], false);
goog.addDependency('log/log.js', ['goog.log', 'goog.log.Level', 'goog.log.LogRecord', 'goog.log.Logger'], ['goog.debug', 'goog.debug.LogManager', 'goog.debug.LogRecord', 'goog.debug.Logger'], false);
goog.addDependency('log/log_test.js', ['goog.logTest'], ['goog.debug.LogManager', 'goog.log', 'goog.log.Level', 'goog.testing.jsunit'], false);
goog.addDependency('math/affinetransform.js', ['goog.math.AffineTransform'], ['goog.math'], false);
goog.addDependency('math/affinetransform_test.js', ['goog.math.AffineTransformTest'], ['goog.array', 'goog.math', 'goog.math.AffineTransform', 'goog.testing.jsunit'], false);
goog.addDependency('math/bezier.js', ['goog.math.Bezier'], ['goog.math', 'goog.math.Coordinate'], false);
goog.addDependency('math/bezier_test.js', ['goog.math.BezierTest'], ['goog.math', 'goog.math.Bezier', 'goog.math.Coordinate', 'goog.testing.jsunit'], false);
goog.addDependency('math/box.js', ['goog.math.Box'], ['goog.math.Coordinate'], false);
goog.addDependency('math/box_test.js', ['goog.math.BoxTest'], ['goog.math.Box', 'goog.math.Coordinate', 'goog.testing.jsunit'], false);
goog.addDependency('math/coordinate.js', ['goog.math.Coordinate'], ['goog.math'], false);
goog.addDependency('math/coordinate3.js', ['goog.math.Coordinate3'], [], false);
goog.addDependency('math/coordinate3_test.js', ['goog.math.Coordinate3Test'], ['goog.math.Coordinate3', 'goog.testing.jsunit'], false);
goog.addDependency('math/coordinate_test.js', ['goog.math.CoordinateTest'], ['goog.math.Coordinate', 'goog.testing.jsunit'], false);
goog.addDependency('math/exponentialbackoff.js', ['goog.math.ExponentialBackoff'], ['goog.asserts'], false);
goog.addDependency('math/exponentialbackoff_test.js', ['goog.math.ExponentialBackoffTest'], ['goog.math.ExponentialBackoff', 'goog.testing.jsunit'], false);
goog.addDependency('math/integer.js', ['goog.math.Integer'], [], false);
goog.addDependency('math/integer_test.js', ['goog.math.IntegerTest'], ['goog.math.Integer', 'goog.testing.jsunit'], false);
goog.addDependency('math/interpolator/interpolator1.js', ['goog.math.interpolator.Interpolator1'], [], false);
goog.addDependency('math/interpolator/linear1.js', ['goog.math.interpolator.Linear1'], ['goog.array', 'goog.math', 'goog.math.interpolator.Interpolator1'], false);
goog.addDependency('math/interpolator/linear1_test.js', ['goog.math.interpolator.Linear1Test'], ['goog.math.interpolator.Linear1', 'goog.testing.jsunit'], false);
goog.addDependency('math/interpolator/pchip1.js', ['goog.math.interpolator.Pchip1'], ['goog.math', 'goog.math.interpolator.Spline1'], false);
goog.addDependency('math/interpolator/pchip1_test.js', ['goog.math.interpolator.Pchip1Test'], ['goog.math.interpolator.Pchip1', 'goog.testing.jsunit'], false);
goog.addDependency('math/interpolator/spline1.js', ['goog.math.interpolator.Spline1'], ['goog.array', 'goog.math', 'goog.math.interpolator.Interpolator1', 'goog.math.tdma'], false);
goog.addDependency('math/interpolator/spline1_test.js', ['goog.math.interpolator.Spline1Test'], ['goog.math.interpolator.Spline1', 'goog.testing.jsunit'], false);
goog.addDependency('math/line.js', ['goog.math.Line'], ['goog.math', 'goog.math.Coordinate'], false);
goog.addDependency('math/line_test.js', ['goog.math.LineTest'], ['goog.math.Coordinate', 'goog.math.Line', 'goog.testing.jsunit'], false);
goog.addDependency('math/long.js', ['goog.math.Long'], [], false);
goog.addDependency('math/long_test.js', ['goog.math.LongTest'], ['goog.math.Long', 'goog.testing.jsunit'], false);
goog.addDependency('math/math.js', ['goog.math'], ['goog.array', 'goog.asserts'], false);
goog.addDependency('math/math_test.js', ['goog.mathTest'], ['goog.math', 'goog.testing.jsunit'], false);
goog.addDependency('math/matrix.js', ['goog.math.Matrix'], ['goog.array', 'goog.math', 'goog.math.Size', 'goog.string'], false);
goog.addDependency('math/matrix_test.js', ['goog.math.MatrixTest'], ['goog.math.Matrix', 'goog.testing.jsunit'], false);
goog.addDependency('math/path.js', ['goog.math.Path', 'goog.math.Path.Segment'], ['goog.array', 'goog.math'], false);
goog.addDependency('math/path_test.js', ['goog.math.PathTest'], ['goog.array', 'goog.math.AffineTransform', 'goog.math.Path', 'goog.testing.jsunit'], false);
goog.addDependency('math/paths.js', ['goog.math.paths'], ['goog.math.Coordinate', 'goog.math.Path'], false);
goog.addDependency('math/paths_test.js', ['goog.math.pathsTest'], ['goog.math.Coordinate', 'goog.math.paths', 'goog.testing.jsunit'], false);
goog.addDependency('math/range.js', ['goog.math.Range'], ['goog.asserts'], false);
goog.addDependency('math/range_test.js', ['goog.math.RangeTest'], ['goog.math.Range', 'goog.testing.jsunit'], false);
goog.addDependency('math/rangeset.js', ['goog.math.RangeSet'], ['goog.array', 'goog.iter.Iterator', 'goog.iter.StopIteration', 'goog.math.Range'], false);
goog.addDependency('math/rangeset_test.js', ['goog.math.RangeSetTest'], ['goog.iter', 'goog.math.Range', 'goog.math.RangeSet', 'goog.testing.jsunit'], false);
goog.addDependency('math/rect.js', ['goog.math.Rect'], ['goog.math.Box', 'goog.math.Coordinate', 'goog.math.Size'], false);
goog.addDependency('math/rect_test.js', ['goog.math.RectTest'], ['goog.math.Box', 'goog.math.Coordinate', 'goog.math.Rect', 'goog.math.Size', 'goog.testing.jsunit'], false);
goog.addDependency('math/size.js', ['goog.math.Size'], [], false);
goog.addDependency('math/size_test.js', ['goog.math.SizeTest'], ['goog.math.Size', 'goog.testing.jsunit'], false);
goog.addDependency('math/tdma.js', ['goog.math.tdma'], [], false);
goog.addDependency('math/tdma_test.js', ['goog.math.tdmaTest'], ['goog.math.tdma', 'goog.testing.jsunit'], false);
goog.addDependency('math/vec2.js', ['goog.math.Vec2'], ['goog.math', 'goog.math.Coordinate'], false);
goog.addDependency('math/vec2_test.js', ['goog.math.Vec2Test'], ['goog.math.Vec2', 'goog.testing.jsunit'], false);
goog.addDependency('math/vec3.js', ['goog.math.Vec3'], ['goog.math', 'goog.math.Coordinate3'], false);
goog.addDependency('math/vec3_test.js', ['goog.math.Vec3Test'], ['goog.math.Coordinate3', 'goog.math.Vec3', 'goog.testing.jsunit'], false);
goog.addDependency('memoize/memoize.js', ['goog.memoize'], [], false);
goog.addDependency('memoize/memoize_test.js', ['goog.memoizeTest'], ['goog.memoize', 'goog.testing.jsunit'], false);
goog.addDependency('messaging/abstractchannel.js', ['goog.messaging.AbstractChannel'], ['goog.Disposable', 'goog.debug', 'goog.json', 'goog.log', 'goog.messaging.MessageChannel'], false);
goog.addDependency('messaging/abstractchannel_test.js', ['goog.messaging.AbstractChannelTest'], ['goog.messaging.AbstractChannel', 'goog.testing.MockControl', 'goog.testing.async.MockControl', 'goog.testing.jsunit'], false);
goog.addDependency('messaging/bufferedchannel.js', ['goog.messaging.BufferedChannel'], ['goog.Timer', 'goog.Uri', 'goog.debug.Error', 'goog.events', 'goog.log', 'goog.messaging.MessageChannel', 'goog.messaging.MultiChannel'], false);
goog.addDependency('messaging/bufferedchannel_test.js', ['goog.messaging.BufferedChannelTest'], ['goog.debug.Console', 'goog.dom', 'goog.log', 'goog.log.Level', 'goog.messaging.BufferedChannel', 'goog.testing.MockClock', 'goog.testing.MockControl', 'goog.testing.async.MockControl', 'goog.testing.jsunit', 'goog.testing.messaging.MockMessageChannel'], false);
goog.addDependency('messaging/deferredchannel.js', ['goog.messaging.DeferredChannel'], ['goog.Disposable', 'goog.async.Deferred', 'goog.messaging.MessageChannel'], false);
goog.addDependency('messaging/deferredchannel_test.js', ['goog.messaging.DeferredChannelTest'], ['goog.async.Deferred', 'goog.messaging.DeferredChannel', 'goog.testing.MockControl', 'goog.testing.async.MockControl', 'goog.testing.jsunit', 'goog.testing.messaging.MockMessageChannel'], false);
goog.addDependency('messaging/loggerclient.js', ['goog.messaging.LoggerClient'], ['goog.Disposable', 'goog.debug', 'goog.debug.LogManager', 'goog.debug.Logger'], false);
goog.addDependency('messaging/loggerclient_test.js', ['goog.messaging.LoggerClientTest'], ['goog.debug', 'goog.debug.Logger', 'goog.messaging.LoggerClient', 'goog.testing.MockControl', 'goog.testing.jsunit', 'goog.testing.messaging.MockMessageChannel'], false);
goog.addDependency('messaging/loggerserver.js', ['goog.messaging.LoggerServer'], ['goog.Disposable', 'goog.log'], false);
goog.addDependency('messaging/loggerserver_test.js', ['goog.messaging.LoggerServerTest'], ['goog.debug.LogManager', 'goog.debug.Logger', 'goog.log', 'goog.log.Level', 'goog.messaging.LoggerServer', 'goog.testing.MockControl', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.messaging.MockMessageChannel'], false);
goog.addDependency('messaging/messagechannel.js', ['goog.messaging.MessageChannel'], [], false);
goog.addDependency('messaging/messaging.js', ['goog.messaging'], ['goog.messaging.MessageChannel'], false);
goog.addDependency('messaging/messaging_test.js', ['goog.testing.messaging.MockMessageChannelTest'], ['goog.messaging', 'goog.testing.MockControl', 'goog.testing.jsunit', 'goog.testing.messaging.MockMessageChannel'], false);
goog.addDependency('messaging/multichannel.js', ['goog.messaging.MultiChannel', 'goog.messaging.MultiChannel.VirtualChannel'], ['goog.Disposable', 'goog.events.EventHandler', 'goog.log', 'goog.messaging.MessageChannel', 'goog.object'], false);
goog.addDependency('messaging/multichannel_test.js', ['goog.messaging.MultiChannelTest'], ['goog.messaging.MultiChannel', 'goog.testing.MockControl', 'goog.testing.jsunit', 'goog.testing.messaging.MockMessageChannel', 'goog.testing.mockmatchers.IgnoreArgument'], false);
goog.addDependency('messaging/portcaller.js', ['goog.messaging.PortCaller'], ['goog.Disposable', 'goog.async.Deferred', 'goog.messaging.DeferredChannel', 'goog.messaging.PortChannel', 'goog.messaging.PortNetwork', 'goog.object'], false);
goog.addDependency('messaging/portcaller_test.js', ['goog.messaging.PortCallerTest'], ['goog.events.EventTarget', 'goog.messaging.PortCaller', 'goog.messaging.PortNetwork', 'goog.testing.MockControl', 'goog.testing.jsunit', 'goog.testing.messaging.MockMessageChannel'], false);
goog.addDependency('messaging/portchannel.js', ['goog.messaging.PortChannel'], ['goog.Timer', 'goog.array', 'goog.async.Deferred', 'goog.debug', 'goog.dom', 'goog.dom.DomHelper', 'goog.events', 'goog.events.EventType', 'goog.json', 'goog.log', 'goog.messaging.AbstractChannel', 'goog.messaging.DeferredChannel', 'goog.object', 'goog.string', 'goog.userAgent'], false);
goog.addDependency('messaging/portnetwork.js', ['goog.messaging.PortNetwork'], [], false);
goog.addDependency('messaging/portoperator.js', ['goog.messaging.PortOperator'], ['goog.Disposable', 'goog.asserts', 'goog.log', 'goog.messaging.PortChannel', 'goog.messaging.PortNetwork', 'goog.object'], false);
goog.addDependency('messaging/portoperator_test.js', ['goog.messaging.PortOperatorTest'], ['goog.messaging.PortNetwork', 'goog.messaging.PortOperator', 'goog.testing.MockControl', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.messaging.MockMessageChannel', 'goog.testing.messaging.MockMessagePort'], false);
goog.addDependency('messaging/respondingchannel.js', ['goog.messaging.RespondingChannel'], ['goog.Disposable', 'goog.log', 'goog.messaging.MessageChannel', 'goog.messaging.MultiChannel', 'goog.messaging.MultiChannel.VirtualChannel'], false);
goog.addDependency('messaging/respondingchannel_test.js', ['goog.messaging.RespondingChannelTest'], ['goog.messaging.RespondingChannel', 'goog.testing.MockControl', 'goog.testing.jsunit', 'goog.testing.messaging.MockMessageChannel'], false);
goog.addDependency('messaging/testdata/portchannel_worker.js', ['goog.messaging.testdata.portchannel_worker'], ['goog.messaging.PortChannel'], false);
goog.addDependency('messaging/testdata/portnetwork_worker1.js', ['goog.messaging.testdata.portnetwork_worker1'], ['goog.messaging.PortCaller', 'goog.messaging.PortChannel'], false);
goog.addDependency('messaging/testdata/portnetwork_worker2.js', ['goog.messaging.testdata.portnetwork_worker2'], ['goog.messaging.PortCaller', 'goog.messaging.PortChannel'], false);
goog.addDependency('module/abstractmoduleloader.js', ['goog.module.AbstractModuleLoader'], ['goog.module'], false);
goog.addDependency('module/basemodule.js', ['goog.module.BaseModule'], ['goog.Disposable', 'goog.module'], false);
goog.addDependency('module/loader.js', ['goog.module.Loader'], ['goog.Timer', 'goog.array', 'goog.dom', 'goog.module', 'goog.object'], false);
goog.addDependency('module/module.js', ['goog.module'], [], false);
goog.addDependency('module/moduleinfo.js', ['goog.module.ModuleInfo'], ['goog.Disposable', 'goog.functions', 'goog.module', 'goog.module.BaseModule', 'goog.module.ModuleLoadCallback'], false);
goog.addDependency('module/moduleinfo_test.js', ['goog.module.ModuleInfoTest'], ['goog.module.BaseModule', 'goog.module.ModuleInfo', 'goog.testing.jsunit'], false);
goog.addDependency('module/moduleloadcallback.js', ['goog.module.ModuleLoadCallback'], ['goog.debug.entryPointRegistry', 'goog.debug.errorHandlerWeakDep', 'goog.module'], false);
goog.addDependency('module/moduleloadcallback_test.js', ['goog.module.ModuleLoadCallbackTest'], ['goog.debug.ErrorHandler', 'goog.debug.entryPointRegistry', 'goog.functions', 'goog.module.ModuleLoadCallback', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('module/moduleloader.js', ['goog.module.ModuleLoader'], ['goog.Timer', 'goog.array', 'goog.events', 'goog.events.Event', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.log', 'goog.module.AbstractModuleLoader', 'goog.net.BulkLoader', 'goog.net.EventType', 'goog.net.jsloader', 'goog.userAgent.product'], false);
goog.addDependency('module/moduleloader_test.js', ['goog.module.ModuleLoaderTest'], ['goog.array', 'goog.dom', 'goog.functions', 'goog.module.ModuleLoader', 'goog.module.ModuleManager', 'goog.module.ModuleManager.CallbackType', 'goog.object', 'goog.testing.AsyncTestCase', 'goog.testing.PropertyReplacer', 'goog.testing.events.EventObserver', 'goog.testing.jsunit', 'goog.testing.recordFunction', 'goog.userAgent.product'], false);
goog.addDependency('module/modulemanager.js', ['goog.module.ModuleManager', 'goog.module.ModuleManager.CallbackType', 'goog.module.ModuleManager.FailureType'], ['goog.Disposable', 'goog.array', 'goog.asserts', 'goog.async.Deferred', 'goog.debug.Trace', 'goog.dispose', 'goog.log', 'goog.module', 'goog.module.ModuleInfo', 'goog.module.ModuleLoadCallback', 'goog.object'], false);
goog.addDependency('module/modulemanager_test.js', ['goog.module.ModuleManagerTest'], ['goog.array', 'goog.functions', 'goog.module.BaseModule', 'goog.module.ModuleManager', 'goog.testing', 'goog.testing.MockClock', 'goog.testing.jsunit', 'goog.testing.recordFunction', 'goog.userAgent'], false);
goog.addDependency('module/testdata/modA_1.js', ['goog.module.testdata.modA_1'], [], false);
goog.addDependency('module/testdata/modA_2.js', ['goog.module.testdata.modA_2'], ['goog.module.ModuleManager'], false);
goog.addDependency('module/testdata/modB_1.js', ['goog.module.testdata.modB_1'], ['goog.module.ModuleManager'], false);
goog.addDependency('net/browserchannel.js', ['goog.net.BrowserChannel', 'goog.net.BrowserChannel.Error', 'goog.net.BrowserChannel.Event', 'goog.net.BrowserChannel.Handler', 'goog.net.BrowserChannel.LogSaver', 'goog.net.BrowserChannel.QueuedMap', 'goog.net.BrowserChannel.ServerReachability', 'goog.net.BrowserChannel.ServerReachabilityEvent', 'goog.net.BrowserChannel.Stat', 'goog.net.BrowserChannel.StatEvent', 'goog.net.BrowserChannel.State', 'goog.net.BrowserChannel.TimingEvent'], ['goog.Uri', 'goog.array', 'goog.asserts', 'goog.debug.TextFormatter', 'goog.events.Event', 'goog.events.EventTarget', 'goog.json', 'goog.json.EvalJsonProcessor', 'goog.log', 'goog.net.BrowserTestChannel', 'goog.net.ChannelDebug', 'goog.net.ChannelRequest', 'goog.net.XhrIo', 'goog.net.tmpnetwork', 'goog.object', 'goog.string', 'goog.structs', 'goog.structs.CircularBuffer'], false);
goog.addDependency('net/browserchannel_test.js', ['goog.net.BrowserChannelTest'], ['goog.Timer', 'goog.array', 'goog.dom', 'goog.functions', 'goog.json', 'goog.net.BrowserChannel', 'goog.net.ChannelDebug', 'goog.net.ChannelRequest', 'goog.net.tmpnetwork', 'goog.structs.Map', 'goog.testing.MockClock', 'goog.testing.PropertyReplacer', 'goog.testing.asserts', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('net/browsertestchannel.js', ['goog.net.BrowserTestChannel'], ['goog.json.EvalJsonProcessor', 'goog.net.ChannelRequest', 'goog.net.ChannelRequest.Error', 'goog.net.tmpnetwork', 'goog.string.Parser', 'goog.userAgent'], false);
goog.addDependency('net/bulkloader.js', ['goog.net.BulkLoader'], ['goog.events.EventHandler', 'goog.events.EventTarget', 'goog.log', 'goog.net.BulkLoaderHelper', 'goog.net.EventType', 'goog.net.XhrIo'], false);
goog.addDependency('net/bulkloader_test.js', ['goog.net.BulkLoaderTest'], ['goog.events.Event', 'goog.events.EventHandler', 'goog.net.BulkLoader', 'goog.net.EventType', 'goog.testing.MockClock', 'goog.testing.jsunit'], false);
goog.addDependency('net/bulkloaderhelper.js', ['goog.net.BulkLoaderHelper'], ['goog.Disposable', 'goog.log'], false);
goog.addDependency('net/channeldebug.js', ['goog.net.ChannelDebug'], ['goog.json', 'goog.log'], false);
goog.addDependency('net/channelrequest.js', ['goog.net.ChannelRequest', 'goog.net.ChannelRequest.Error'], ['goog.Timer', 'goog.async.Throttle', 'goog.events.EventHandler', 'goog.net.ErrorCode', 'goog.net.EventType', 'goog.net.XmlHttp', 'goog.object', 'goog.userAgent'], false);
goog.addDependency('net/channelrequest_test.js', ['goog.net.ChannelRequestTest'], ['goog.Uri', 'goog.functions', 'goog.net.BrowserChannel', 'goog.net.ChannelDebug', 'goog.net.ChannelRequest', 'goog.testing.MockClock', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.net.XhrIo', 'goog.testing.recordFunction'], false);
goog.addDependency('net/cookies.js', ['goog.net.Cookies', 'goog.net.cookies'], [], false);
goog.addDependency('net/cookies_test.js', ['goog.net.cookiesTest'], ['goog.array', 'goog.net.cookies', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit'], false);
goog.addDependency('net/corsxmlhttpfactory.js', ['goog.net.CorsXmlHttpFactory', 'goog.net.IeCorsXhrAdapter'], ['goog.net.HttpStatus', 'goog.net.XhrLike', 'goog.net.XmlHttp', 'goog.net.XmlHttpFactory'], false);
goog.addDependency('net/corsxmlhttpfactory_test.js', ['goog.net.CorsXmlHttpFactoryTest'], ['goog.net.CorsXmlHttpFactory', 'goog.net.IeCorsXhrAdapter', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('net/crossdomainrpc.js', ['goog.net.CrossDomainRpc'], ['goog.Uri', 'goog.dom', 'goog.events', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.json', 'goog.log', 'goog.net.EventType', 'goog.net.HttpStatus', 'goog.string', 'goog.userAgent'], false);
goog.addDependency('net/crossdomainrpc_test.js', ['goog.net.CrossDomainRpcTest'], ['goog.log', 'goog.log.Level', 'goog.net.CrossDomainRpc', 'goog.testing.AsyncTestCase', 'goog.testing.jsunit', 'goog.userAgent', 'goog.userAgent.product'], false);
goog.addDependency('net/errorcode.js', ['goog.net.ErrorCode'], [], false);
goog.addDependency('net/eventtype.js', ['goog.net.EventType'], [], false);
goog.addDependency('net/filedownloader.js', ['goog.net.FileDownloader', 'goog.net.FileDownloader.Error'], ['goog.Disposable', 'goog.asserts', 'goog.async.Deferred', 'goog.crypt.hash32', 'goog.debug.Error', 'goog.events', 'goog.events.EventHandler', 'goog.fs', 'goog.fs.DirectoryEntry', 'goog.fs.Error', 'goog.fs.FileSaver', 'goog.net.EventType', 'goog.net.XhrIo', 'goog.net.XhrIoPool', 'goog.object'], false);
goog.addDependency('net/filedownloader_test.js', ['goog.net.FileDownloaderTest'], ['goog.fs.Error', 'goog.net.ErrorCode', 'goog.net.FileDownloader', 'goog.net.XhrIo', 'goog.testing.AsyncTestCase', 'goog.testing.PropertyReplacer', 'goog.testing.fs', 'goog.testing.fs.FileSystem', 'goog.testing.jsunit', 'goog.testing.net.XhrIoPool'], false);
goog.addDependency('net/httpstatus.js', ['goog.net.HttpStatus'], [], false);
goog.addDependency('net/iframe_xhr_test.js', ['goog.net.iframeXhrTest'], ['goog.Timer', 'goog.debug.Console', 'goog.debug.LogManager', 'goog.debug.Logger', 'goog.events', 'goog.net.IframeIo', 'goog.net.XhrIo', 'goog.testing.AsyncTestCase', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('net/iframeio.js', ['goog.net.IframeIo', 'goog.net.IframeIo.IncrementalDataEvent'], ['goog.Timer', 'goog.Uri', 'goog.asserts', 'goog.debug', 'goog.dom', 'goog.events', 'goog.events.Event', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.json', 'goog.log', 'goog.log.Level', 'goog.net.ErrorCode', 'goog.net.EventType', 'goog.reflect', 'goog.string', 'goog.structs', 'goog.userAgent'], false);
goog.addDependency('net/iframeio_different_base_test.js', ['goog.net.iframeIoDifferentBaseTest'], ['goog.events', 'goog.net.EventType', 'goog.net.IframeIo', 'goog.testing.AsyncTestCase', 'goog.testing.jsunit'], false);
goog.addDependency('net/iframeio_test.js', ['goog.net.IframeIoTest'], ['goog.debug', 'goog.debug.DivConsole', 'goog.debug.LogManager', 'goog.dom', 'goog.events', 'goog.events.EventType', 'goog.log', 'goog.log.Level', 'goog.net.IframeIo', 'goog.testing.events', 'goog.testing.events.Event', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('net/iframeloadmonitor.js', ['goog.net.IframeLoadMonitor'], ['goog.dom', 'goog.events', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.userAgent'], false);
goog.addDependency('net/iframeloadmonitor_test.js', ['goog.net.IframeLoadMonitorTest'], ['goog.dom', 'goog.events', 'goog.net.IframeLoadMonitor', 'goog.testing.AsyncTestCase', 'goog.testing.jsunit'], false);
goog.addDependency('net/imageloader.js', ['goog.net.ImageLoader'], ['goog.array', 'goog.dom', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.net.EventType', 'goog.object', 'goog.userAgent'], false);
goog.addDependency('net/imageloader_test.js', ['goog.net.ImageLoaderTest'], ['goog.Timer', 'goog.array', 'goog.dispose', 'goog.events', 'goog.events.Event', 'goog.events.EventType', 'goog.net.EventType', 'goog.net.ImageLoader', 'goog.object', 'goog.string', 'goog.testing.AsyncTestCase', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('net/ipaddress.js', ['goog.net.IpAddress', 'goog.net.Ipv4Address', 'goog.net.Ipv6Address'], ['goog.array', 'goog.math.Integer', 'goog.object', 'goog.string'], false);
goog.addDependency('net/ipaddress_test.js', ['goog.net.IpAddressTest'], ['goog.math.Integer', 'goog.net.IpAddress', 'goog.net.Ipv4Address', 'goog.net.Ipv6Address', 'goog.testing.jsunit'], false);
goog.addDependency('net/jsloader.js', ['goog.net.jsloader', 'goog.net.jsloader.Error', 'goog.net.jsloader.ErrorCode', 'goog.net.jsloader.Options'], ['goog.array', 'goog.async.Deferred', 'goog.debug.Error', 'goog.dom', 'goog.dom.TagName'], false);
goog.addDependency('net/jsloader_test.js', ['goog.net.jsloaderTest'], ['goog.array', 'goog.dom', 'goog.net.jsloader', 'goog.net.jsloader.ErrorCode', 'goog.testing.AsyncTestCase', 'goog.testing.jsunit'], false);
goog.addDependency('net/jsonp.js', ['goog.net.Jsonp'], ['goog.Uri', 'goog.net.jsloader'], false);
goog.addDependency('net/jsonp_test.js', ['goog.net.JsonpTest'], ['goog.net.Jsonp', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.recordFunction', 'goog.userAgent'], false);
goog.addDependency('net/mockiframeio.js', ['goog.net.MockIFrameIo'], ['goog.events.EventTarget', 'goog.json', 'goog.net.ErrorCode', 'goog.net.EventType', 'goog.net.IframeIo'], false);
goog.addDependency('net/multiiframeloadmonitor.js', ['goog.net.MultiIframeLoadMonitor'], ['goog.events', 'goog.net.IframeLoadMonitor'], false);
goog.addDependency('net/multiiframeloadmonitor_test.js', ['goog.net.MultiIframeLoadMonitorTest'], ['goog.dom', 'goog.net.IframeLoadMonitor', 'goog.net.MultiIframeLoadMonitor', 'goog.testing.AsyncTestCase', 'goog.testing.jsunit'], false);
goog.addDependency('net/networkstatusmonitor.js', ['goog.net.NetworkStatusMonitor'], ['goog.events.Listenable'], false);
goog.addDependency('net/networktester.js', ['goog.net.NetworkTester'], ['goog.Timer', 'goog.Uri', 'goog.log'], false);
goog.addDependency('net/networktester_test.js', ['goog.net.NetworkTesterTest'], ['goog.Uri', 'goog.net.NetworkTester', 'goog.testing.MockClock', 'goog.testing.jsunit'], false);
goog.addDependency('net/testdata/jsloader_test1.js', ['goog.net.testdata.jsloader_test1'], [], false);
goog.addDependency('net/testdata/jsloader_test2.js', ['goog.net.testdata.jsloader_test2'], [], false);
goog.addDependency('net/testdata/jsloader_test3.js', ['goog.net.testdata.jsloader_test3'], [], false);
goog.addDependency('net/testdata/jsloader_test4.js', ['goog.net.testdata.jsloader_test4'], [], false);
goog.addDependency('net/tmpnetwork.js', ['goog.net.tmpnetwork'], ['goog.Uri', 'goog.net.ChannelDebug'], false);
goog.addDependency('net/websocket.js', ['goog.net.WebSocket', 'goog.net.WebSocket.ErrorEvent', 'goog.net.WebSocket.EventType', 'goog.net.WebSocket.MessageEvent'], ['goog.Timer', 'goog.asserts', 'goog.debug.entryPointRegistry', 'goog.events', 'goog.events.Event', 'goog.events.EventTarget', 'goog.log'], false);
goog.addDependency('net/websocket_test.js', ['goog.net.WebSocketTest'], ['goog.debug.EntryPointMonitor', 'goog.debug.ErrorHandler', 'goog.debug.entryPointRegistry', 'goog.events', 'goog.functions', 'goog.net.WebSocket', 'goog.testing.MockClock', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('net/wrapperxmlhttpfactory.js', ['goog.net.WrapperXmlHttpFactory'], ['goog.net.XhrLike', 'goog.net.XmlHttpFactory'], false);
goog.addDependency('net/xhrio.js', ['goog.net.XhrIo', 'goog.net.XhrIo.ResponseType'], ['goog.Timer', 'goog.array', 'goog.debug.entryPointRegistry', 'goog.events.EventTarget', 'goog.json', 'goog.log', 'goog.net.ErrorCode', 'goog.net.EventType', 'goog.net.HttpStatus', 'goog.net.XmlHttp', 'goog.object', 'goog.string', 'goog.structs', 'goog.structs.Map', 'goog.uri.utils', 'goog.userAgent'], false);
goog.addDependency('net/xhrio_test.js', ['goog.net.XhrIoTest'], ['goog.Uri', 'goog.debug.EntryPointMonitor', 'goog.debug.ErrorHandler', 'goog.debug.entryPointRegistry', 'goog.events', 'goog.functions', 'goog.net.EventType', 'goog.net.WrapperXmlHttpFactory', 'goog.net.XhrIo', 'goog.net.XmlHttp', 'goog.object', 'goog.string', 'goog.testing.MockClock', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.net.XhrIo', 'goog.testing.recordFunction'], false);
goog.addDependency('net/xhriopool.js', ['goog.net.XhrIoPool'], ['goog.net.XhrIo', 'goog.structs.PriorityPool'], false);
goog.addDependency('net/xhrlike.js', ['goog.net.XhrLike'], [], false);
goog.addDependency('net/xhrmanager.js', ['goog.net.XhrManager', 'goog.net.XhrManager.Event', 'goog.net.XhrManager.Request'], ['goog.events', 'goog.events.Event', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.net.ErrorCode', 'goog.net.EventType', 'goog.net.XhrIo', 'goog.net.XhrIoPool', 'goog.structs.Map'], false);
goog.addDependency('net/xhrmanager_test.js', ['goog.net.XhrManagerTest'], ['goog.events', 'goog.net.EventType', 'goog.net.XhrIo', 'goog.net.XhrManager', 'goog.testing.jsunit', 'goog.testing.net.XhrIoPool', 'goog.testing.recordFunction'], false);
goog.addDependency('net/xmlhttp.js', ['goog.net.DefaultXmlHttpFactory', 'goog.net.XmlHttp', 'goog.net.XmlHttp.OptionType', 'goog.net.XmlHttp.ReadyState', 'goog.net.XmlHttpDefines'], ['goog.asserts', 'goog.net.WrapperXmlHttpFactory', 'goog.net.XmlHttpFactory'], false);
goog.addDependency('net/xmlhttpfactory.js', ['goog.net.XmlHttpFactory'], ['goog.net.XhrLike'], false);
goog.addDependency('net/xpc/crosspagechannel.js', ['goog.net.xpc.CrossPageChannel'], ['goog.Uri', 'goog.async.Deferred', 'goog.async.Delay', 'goog.dispose', 'goog.dom', 'goog.events', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.json', 'goog.log', 'goog.messaging.AbstractChannel', 'goog.net.xpc', 'goog.net.xpc.CfgFields', 'goog.net.xpc.ChannelStates', 'goog.net.xpc.CrossPageChannelRole', 'goog.net.xpc.DirectTransport', 'goog.net.xpc.FrameElementMethodTransport', 'goog.net.xpc.IframePollingTransport', 'goog.net.xpc.IframeRelayTransport', 'goog.net.xpc.NativeMessagingTransport', 'goog.net.xpc.NixTransport', 'goog.net.xpc.TransportTypes', 'goog.net.xpc.UriCfgFields', 'goog.string', 'goog.uri.utils', 'goog.userAgent'], false);
goog.addDependency('net/xpc/crosspagechannel_test.js', ['goog.net.xpc.CrossPageChannelTest'], ['goog.Disposable', 'goog.Uri', 'goog.async.Deferred', 'goog.dom', 'goog.log', 'goog.log.Level', 'goog.net.xpc', 'goog.net.xpc.CfgFields', 'goog.net.xpc.CrossPageChannel', 'goog.net.xpc.CrossPageChannelRole', 'goog.net.xpc.TransportTypes', 'goog.object', 'goog.testing.AsyncTestCase', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit'], false);
goog.addDependency('net/xpc/crosspagechannelrole.js', ['goog.net.xpc.CrossPageChannelRole'], [], false);
goog.addDependency('net/xpc/directtransport.js', ['goog.net.xpc.DirectTransport'], ['goog.Timer', 'goog.async.Deferred', 'goog.events.EventHandler', 'goog.log', 'goog.net.xpc', 'goog.net.xpc.CfgFields', 'goog.net.xpc.CrossPageChannelRole', 'goog.net.xpc.Transport', 'goog.net.xpc.TransportTypes', 'goog.object'], false);
goog.addDependency('net/xpc/directtransport_test.js', ['goog.net.xpc.DirectTransportTest'], ['goog.dom', 'goog.log', 'goog.log.Level', 'goog.net.xpc', 'goog.net.xpc.CfgFields', 'goog.net.xpc.CrossPageChannel', 'goog.net.xpc.CrossPageChannelRole', 'goog.net.xpc.TransportTypes', 'goog.testing.AsyncTestCase', 'goog.testing.jsunit'], false);
goog.addDependency('net/xpc/frameelementmethodtransport.js', ['goog.net.xpc.FrameElementMethodTransport'], ['goog.log', 'goog.net.xpc', 'goog.net.xpc.CrossPageChannelRole', 'goog.net.xpc.Transport', 'goog.net.xpc.TransportTypes'], false);
goog.addDependency('net/xpc/iframepollingtransport.js', ['goog.net.xpc.IframePollingTransport', 'goog.net.xpc.IframePollingTransport.Receiver', 'goog.net.xpc.IframePollingTransport.Sender'], ['goog.array', 'goog.dom', 'goog.log', 'goog.log.Level', 'goog.net.xpc', 'goog.net.xpc.CfgFields', 'goog.net.xpc.CrossPageChannelRole', 'goog.net.xpc.Transport', 'goog.net.xpc.TransportTypes', 'goog.userAgent'], false);
goog.addDependency('net/xpc/iframepollingtransport_test.js', ['goog.net.xpc.IframePollingTransportTest'], ['goog.Timer', 'goog.dom', 'goog.dom.TagName', 'goog.functions', 'goog.net.xpc.CfgFields', 'goog.net.xpc.CrossPageChannel', 'goog.net.xpc.CrossPageChannelRole', 'goog.net.xpc.TransportTypes', 'goog.object', 'goog.testing.MockClock', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('net/xpc/iframerelaytransport.js', ['goog.net.xpc.IframeRelayTransport'], ['goog.dom', 'goog.dom.safe', 'goog.events', 'goog.html.SafeHtml', 'goog.log', 'goog.log.Level', 'goog.net.xpc', 'goog.net.xpc.CfgFields', 'goog.net.xpc.Transport', 'goog.net.xpc.TransportTypes', 'goog.string', 'goog.string.Const', 'goog.userAgent'], false);
goog.addDependency('net/xpc/nativemessagingtransport.js', ['goog.net.xpc.NativeMessagingTransport'], ['goog.Timer', 'goog.asserts', 'goog.async.Deferred', 'goog.events', 'goog.events.EventHandler', 'goog.log', 'goog.net.xpc', 'goog.net.xpc.CrossPageChannelRole', 'goog.net.xpc.Transport', 'goog.net.xpc.TransportTypes'], false);
goog.addDependency('net/xpc/nativemessagingtransport_test.js', ['goog.net.xpc.NativeMessagingTransportTest'], ['goog.dom', 'goog.events', 'goog.net.xpc', 'goog.net.xpc.CfgFields', 'goog.net.xpc.CrossPageChannel', 'goog.net.xpc.CrossPageChannelRole', 'goog.net.xpc.NativeMessagingTransport', 'goog.testing.jsunit'], false);
goog.addDependency('net/xpc/nixtransport.js', ['goog.net.xpc.NixTransport'], ['goog.log', 'goog.net.xpc', 'goog.net.xpc.CfgFields', 'goog.net.xpc.CrossPageChannelRole', 'goog.net.xpc.Transport', 'goog.net.xpc.TransportTypes', 'goog.reflect'], false);
goog.addDependency('net/xpc/relay.js', ['goog.net.xpc.relay'], [], false);
goog.addDependency('net/xpc/transport.js', ['goog.net.xpc.Transport'], ['goog.Disposable', 'goog.dom', 'goog.net.xpc.TransportNames'], false);
goog.addDependency('net/xpc/xpc.js', ['goog.net.xpc', 'goog.net.xpc.CfgFields', 'goog.net.xpc.ChannelStates', 'goog.net.xpc.TransportNames', 'goog.net.xpc.TransportTypes', 'goog.net.xpc.UriCfgFields'], ['goog.log'], false);
goog.addDependency('object/object.js', ['goog.object'], ['goog.array'], false);
goog.addDependency('object/object_test.js', ['goog.objectTest'], ['goog.functions', 'goog.object', 'goog.testing.jsunit'], false);
goog.addDependency('positioning/absoluteposition.js', ['goog.positioning.AbsolutePosition'], ['goog.math.Box', 'goog.math.Coordinate', 'goog.math.Size', 'goog.positioning', 'goog.positioning.AbstractPosition'], false);
goog.addDependency('positioning/abstractposition.js', ['goog.positioning.AbstractPosition'], ['goog.math.Box', 'goog.math.Size', 'goog.positioning.Corner'], false);
goog.addDependency('positioning/anchoredposition.js', ['goog.positioning.AnchoredPosition'], ['goog.math.Box', 'goog.positioning', 'goog.positioning.AbstractPosition'], false);
goog.addDependency('positioning/anchoredposition_test.js', ['goog.positioning.AnchoredPositionTest'], ['goog.dom', 'goog.positioning.AnchoredPosition', 'goog.positioning.Corner', 'goog.positioning.Overflow', 'goog.style', 'goog.testing.jsunit'], false);
goog.addDependency('positioning/anchoredviewportposition.js', ['goog.positioning.AnchoredViewportPosition'], ['goog.math.Box', 'goog.positioning', 'goog.positioning.AnchoredPosition', 'goog.positioning.Corner', 'goog.positioning.Overflow', 'goog.positioning.OverflowStatus'], false);
goog.addDependency('positioning/anchoredviewportposition_test.js', ['goog.positioning.AnchoredViewportPositionTest'], ['goog.dom', 'goog.math.Box', 'goog.positioning.AnchoredViewportPosition', 'goog.positioning.Corner', 'goog.positioning.OverflowStatus', 'goog.style', 'goog.testing.jsunit'], false);
goog.addDependency('positioning/clientposition.js', ['goog.positioning.ClientPosition'], ['goog.asserts', 'goog.math.Box', 'goog.math.Coordinate', 'goog.math.Size', 'goog.positioning', 'goog.positioning.AbstractPosition', 'goog.style'], false);
goog.addDependency('positioning/clientposition_test.js', ['goog.positioning.clientPositionTest'], ['goog.dom', 'goog.positioning.ClientPosition', 'goog.style', 'goog.testing.jsunit'], false);
goog.addDependency('positioning/menuanchoredposition.js', ['goog.positioning.MenuAnchoredPosition'], ['goog.math.Box', 'goog.math.Size', 'goog.positioning', 'goog.positioning.AnchoredViewportPosition', 'goog.positioning.Corner', 'goog.positioning.Overflow'], false);
goog.addDependency('positioning/menuanchoredposition_test.js', ['goog.positioning.MenuAnchoredPositionTest'], ['goog.dom', 'goog.positioning.Corner', 'goog.positioning.MenuAnchoredPosition', 'goog.testing.jsunit'], false);
goog.addDependency('positioning/positioning.js', ['goog.positioning', 'goog.positioning.Corner', 'goog.positioning.CornerBit', 'goog.positioning.Overflow', 'goog.positioning.OverflowStatus'], ['goog.asserts', 'goog.dom', 'goog.dom.TagName', 'goog.math.Coordinate', 'goog.math.Rect', 'goog.math.Size', 'goog.style', 'goog.style.bidi'], false);
goog.addDependency('positioning/positioning_test.js', ['goog.positioningTest'], ['goog.dom', 'goog.dom.DomHelper', 'goog.math.Box', 'goog.math.Coordinate', 'goog.math.Size', 'goog.positioning', 'goog.positioning.Corner', 'goog.positioning.Overflow', 'goog.positioning.OverflowStatus', 'goog.style', 'goog.testing.ExpectedFailures', 'goog.testing.jsunit', 'goog.userAgent', 'goog.userAgent.product'], false);
goog.addDependency('positioning/viewportclientposition.js', ['goog.positioning.ViewportClientPosition'], ['goog.math.Box', 'goog.math.Coordinate', 'goog.math.Size', 'goog.positioning.ClientPosition'], false);
goog.addDependency('positioning/viewportclientposition_test.js', ['goog.positioning.ViewportClientPositionTest'], ['goog.dom', 'goog.positioning.Corner', 'goog.positioning.Overflow', 'goog.positioning.ViewportClientPosition', 'goog.style', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('positioning/viewportposition.js', ['goog.positioning.ViewportPosition'], ['goog.math.Box', 'goog.math.Coordinate', 'goog.math.Size', 'goog.positioning.AbstractPosition'], false);
goog.addDependency('promise/promise.js', ['goog.Promise'], ['goog.Thenable', 'goog.asserts', 'goog.async.run', 'goog.async.throwException', 'goog.debug.Error', 'goog.promise.Resolver'], false);
goog.addDependency('promise/promise_test.js', ['goog.PromiseTest'], ['goog.Promise', 'goog.Thenable', 'goog.functions', 'goog.testing.AsyncTestCase', 'goog.testing.MockClock', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('promise/resolver.js', ['goog.promise.Resolver'], [], false);
goog.addDependency('promise/testsuiteadapter.js', ['goog.promise.testSuiteAdapter'], ['goog.Promise'], false);
goog.addDependency('promise/thenable.js', ['goog.Thenable'], [], false);
goog.addDependency('proto/proto.js', ['goog.proto'], ['goog.proto.Serializer'], false);
goog.addDependency('proto/serializer.js', ['goog.proto.Serializer'], ['goog.json.Serializer', 'goog.string'], false);
goog.addDependency('proto/serializer_test.js', ['goog.protoTest'], ['goog.proto', 'goog.testing.jsunit'], false);
goog.addDependency('proto2/descriptor.js', ['goog.proto2.Descriptor', 'goog.proto2.Metadata'], ['goog.array', 'goog.asserts', 'goog.object', 'goog.string'], false);
goog.addDependency('proto2/descriptor_test.js', ['goog.proto2.DescriptorTest'], ['goog.proto2.Descriptor', 'goog.proto2.Message', 'goog.testing.jsunit'], false);
goog.addDependency('proto2/fielddescriptor.js', ['goog.proto2.FieldDescriptor'], ['goog.asserts', 'goog.string'], false);
goog.addDependency('proto2/fielddescriptor_test.js', ['goog.proto2.FieldDescriptorTest'], ['goog.proto2.FieldDescriptor', 'goog.proto2.Message', 'goog.testing.jsunit'], false);
goog.addDependency('proto2/lazydeserializer.js', ['goog.proto2.LazyDeserializer'], ['goog.asserts', 'goog.proto2.Message', 'goog.proto2.Serializer'], false);
goog.addDependency('proto2/message.js', ['goog.proto2.Message'], ['goog.asserts', 'goog.proto2.Descriptor', 'goog.proto2.FieldDescriptor'], false);
goog.addDependency('proto2/message_test.js', ['goog.proto2.MessageTest'], ['goog.testing.jsunit', 'proto2.TestAllTypes', 'proto2.TestAllTypes.NestedEnum', 'proto2.TestAllTypes.NestedMessage', 'proto2.TestAllTypes.OptionalGroup', 'proto2.TestAllTypes.RepeatedGroup'], false);
goog.addDependency('proto2/objectserializer.js', ['goog.proto2.ObjectSerializer'], ['goog.asserts', 'goog.proto2.FieldDescriptor', 'goog.proto2.Serializer', 'goog.string'], false);
goog.addDependency('proto2/objectserializer_test.js', ['goog.proto2.ObjectSerializerTest'], ['goog.proto2.ObjectSerializer', 'goog.proto2.Serializer', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'proto2.TestAllTypes'], false);
goog.addDependency('proto2/package_test.pb.js', ['someprotopackage.TestPackageTypes'], ['goog.proto2.Message', 'proto2.TestAllTypes'], false);
goog.addDependency('proto2/pbliteserializer.js', ['goog.proto2.PbLiteSerializer'], ['goog.asserts', 'goog.proto2.FieldDescriptor', 'goog.proto2.LazyDeserializer', 'goog.proto2.Serializer'], false);
goog.addDependency('proto2/pbliteserializer_test.js', ['goog.proto2.PbLiteSerializerTest'], ['goog.proto2.PbLiteSerializer', 'goog.testing.jsunit', 'proto2.TestAllTypes'], false);
goog.addDependency('proto2/proto_test.js', ['goog.proto2.messageTest'], ['goog.proto2.FieldDescriptor', 'goog.testing.jsunit', 'proto2.TestAllTypes', 'someprotopackage.TestPackageTypes'], false);
goog.addDependency('proto2/serializer.js', ['goog.proto2.Serializer'], ['goog.asserts', 'goog.proto2.FieldDescriptor', 'goog.proto2.Message'], false);
goog.addDependency('proto2/test.pb.js', ['proto2.TestAllTypes', 'proto2.TestAllTypes.NestedEnum', 'proto2.TestAllTypes.NestedMessage', 'proto2.TestAllTypes.OptionalGroup', 'proto2.TestAllTypes.RepeatedGroup', 'proto2.TestDefaultChild', 'proto2.TestDefaultParent'], ['goog.proto2.Message'], false);
goog.addDependency('proto2/textformatserializer.js', ['goog.proto2.TextFormatSerializer'], ['goog.array', 'goog.asserts', 'goog.json', 'goog.math', 'goog.object', 'goog.proto2.FieldDescriptor', 'goog.proto2.Message', 'goog.proto2.Serializer', 'goog.string'], false);
goog.addDependency('proto2/textformatserializer_test.js', ['goog.proto2.TextFormatSerializerTest'], ['goog.proto2.ObjectSerializer', 'goog.proto2.TextFormatSerializer', 'goog.testing.jsunit', 'proto2.TestAllTypes'], false);
goog.addDependency('proto2/util.js', ['goog.proto2.Util'], ['goog.asserts'], false);
goog.addDependency('pubsub/pubsub.js', ['goog.pubsub.PubSub'], ['goog.Disposable', 'goog.array'], false);
goog.addDependency('pubsub/pubsub_test.js', ['goog.pubsub.PubSubTest'], ['goog.array', 'goog.pubsub.PubSub', 'goog.testing.jsunit'], false);
goog.addDependency('pubsub/topicid.js', ['goog.pubsub.TopicId'], [], false);
goog.addDependency('pubsub/typedpubsub.js', ['goog.pubsub.TypedPubSub'], ['goog.Disposable', 'goog.pubsub.PubSub'], false);
goog.addDependency('pubsub/typedpubsub_test.js', ['goog.pubsub.TypedPubSubTest'], ['goog.array', 'goog.pubsub.TopicId', 'goog.pubsub.TypedPubSub', 'goog.testing.jsunit'], false);
goog.addDependency('reflect/reflect.js', ['goog.reflect'], [], false);
goog.addDependency('result/deferredadaptor.js', ['goog.result.DeferredAdaptor'], ['goog.async.Deferred', 'goog.result', 'goog.result.Result'], false);
goog.addDependency('result/dependentresult.js', ['goog.result.DependentResult'], ['goog.result.Result'], false);
goog.addDependency('result/result_interface.js', ['goog.result.Result'], ['goog.Thenable'], false);
goog.addDependency('result/resultutil.js', ['goog.result'], ['goog.array', 'goog.result.DependentResult', 'goog.result.Result', 'goog.result.SimpleResult'], false);
goog.addDependency('result/simpleresult.js', ['goog.result.SimpleResult', 'goog.result.SimpleResult.StateError'], ['goog.Promise', 'goog.Thenable', 'goog.debug.Error', 'goog.result.Result'], false);
goog.addDependency('soy/data.js', ['goog.soy.data.SanitizedContent', 'goog.soy.data.SanitizedContentKind'], ['goog.html.SafeHtml', 'goog.html.uncheckedconversions', 'goog.string.Const'], false);
goog.addDependency('soy/data_test.js', ['goog.soy.dataTest'], ['goog.html.SafeHtml', 'goog.soy.testHelper', 'goog.testing.jsunit'], false);
goog.addDependency('soy/renderer.js', ['goog.soy.InjectedDataSupplier', 'goog.soy.Renderer'], ['goog.asserts', 'goog.dom', 'goog.soy', 'goog.soy.data.SanitizedContent', 'goog.soy.data.SanitizedContentKind'], false);
goog.addDependency('soy/renderer_test.js', ['goog.soy.RendererTest'], ['goog.dom', 'goog.dom.NodeType', 'goog.dom.TagName', 'goog.html.SafeHtml', 'goog.i18n.bidi.Dir', 'goog.soy.Renderer', 'goog.soy.data.SanitizedContentKind', 'goog.soy.testHelper', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('soy/soy.js', ['goog.soy'], ['goog.asserts', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.TagName', 'goog.soy.data.SanitizedContent', 'goog.soy.data.SanitizedContentKind', 'goog.string'], false);
goog.addDependency('soy/soy_test.js', ['goog.soyTest'], ['goog.dom', 'goog.dom.NodeType', 'goog.dom.TagName', 'goog.functions', 'goog.soy', 'goog.soy.testHelper', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit'], false);
goog.addDependency('soy/soy_testhelper.js', ['goog.soy.testHelper'], ['goog.dom', 'goog.dom.TagName', 'goog.i18n.bidi.Dir', 'goog.soy.data.SanitizedContent', 'goog.soy.data.SanitizedContentKind', 'goog.string', 'goog.userAgent'], false);
goog.addDependency('spell/spellcheck.js', ['goog.spell.SpellCheck', 'goog.spell.SpellCheck.WordChangedEvent'], ['goog.Timer', 'goog.events.EventTarget', 'goog.structs.Set'], false);
goog.addDependency('spell/spellcheck_test.js', ['goog.spell.SpellCheckTest'], ['goog.spell.SpellCheck', 'goog.testing.jsunit'], false);
goog.addDependency('stats/basicstat.js', ['goog.stats.BasicStat'], ['goog.array', 'goog.iter', 'goog.log', 'goog.object', 'goog.string.format', 'goog.structs.CircularBuffer'], false);
goog.addDependency('stats/basicstat_test.js', ['goog.stats.BasicStatTest'], ['goog.array', 'goog.stats.BasicStat', 'goog.string.format', 'goog.testing.PseudoRandom', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('storage/collectablestorage.js', ['goog.storage.CollectableStorage'], ['goog.array', 'goog.iter', 'goog.storage.ErrorCode', 'goog.storage.ExpiringStorage', 'goog.storage.RichStorage'], false);
goog.addDependency('storage/collectablestorage_test.js', ['goog.storage.CollectableStorageTest'], ['goog.storage.CollectableStorage', 'goog.storage.collectableStorageTester', 'goog.storage.storage_test', 'goog.testing.MockClock', 'goog.testing.jsunit', 'goog.testing.storage.FakeMechanism'], false);
goog.addDependency('storage/collectablestoragetester.js', ['goog.storage.collectableStorageTester'], ['goog.testing.asserts'], false);
goog.addDependency('storage/encryptedstorage.js', ['goog.storage.EncryptedStorage'], ['goog.crypt', 'goog.crypt.Arc4', 'goog.crypt.Sha1', 'goog.crypt.base64', 'goog.json', 'goog.json.Serializer', 'goog.storage.CollectableStorage', 'goog.storage.ErrorCode', 'goog.storage.RichStorage', 'goog.storage.RichStorage.Wrapper', 'goog.storage.mechanism.IterableMechanism'], false);
goog.addDependency('storage/encryptedstorage_test.js', ['goog.storage.EncryptedStorageTest'], ['goog.json', 'goog.storage.EncryptedStorage', 'goog.storage.ErrorCode', 'goog.storage.RichStorage', 'goog.storage.collectableStorageTester', 'goog.storage.storage_test', 'goog.testing.MockClock', 'goog.testing.PseudoRandom', 'goog.testing.jsunit', 'goog.testing.storage.FakeMechanism'], false);
goog.addDependency('storage/errorcode.js', ['goog.storage.ErrorCode'], [], false);
goog.addDependency('storage/expiringstorage.js', ['goog.storage.ExpiringStorage'], ['goog.storage.RichStorage', 'goog.storage.RichStorage.Wrapper', 'goog.storage.mechanism.Mechanism'], false);
goog.addDependency('storage/expiringstorage_test.js', ['goog.storage.ExpiringStorageTest'], ['goog.storage.ExpiringStorage', 'goog.storage.storage_test', 'goog.testing.MockClock', 'goog.testing.jsunit', 'goog.testing.storage.FakeMechanism'], false);
goog.addDependency('storage/mechanism/errorcode.js', ['goog.storage.mechanism.ErrorCode'], [], false);
goog.addDependency('storage/mechanism/errorhandlingmechanism.js', ['goog.storage.mechanism.ErrorHandlingMechanism'], ['goog.storage.mechanism.Mechanism'], false);
goog.addDependency('storage/mechanism/errorhandlingmechanism_test.js', ['goog.storage.mechanism.ErrorHandlingMechanismTest'], ['goog.storage.mechanism.ErrorHandlingMechanism', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('storage/mechanism/html5localstorage.js', ['goog.storage.mechanism.HTML5LocalStorage'], ['goog.storage.mechanism.HTML5WebStorage'], false);
goog.addDependency('storage/mechanism/html5localstorage_test.js', ['goog.storage.mechanism.HTML5LocalStorageTest'], ['goog.storage.mechanism.HTML5LocalStorage', 'goog.storage.mechanism.mechanismSeparationTester', 'goog.storage.mechanism.mechanismSharingTester', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('storage/mechanism/html5sessionstorage.js', ['goog.storage.mechanism.HTML5SessionStorage'], ['goog.storage.mechanism.HTML5WebStorage'], false);
goog.addDependency('storage/mechanism/html5sessionstorage_test.js', ['goog.storage.mechanism.HTML5SessionStorageTest'], ['goog.storage.mechanism.HTML5SessionStorage', 'goog.storage.mechanism.mechanismSeparationTester', 'goog.storage.mechanism.mechanismSharingTester', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('storage/mechanism/html5webstorage.js', ['goog.storage.mechanism.HTML5WebStorage'], ['goog.asserts', 'goog.iter.Iterator', 'goog.iter.StopIteration', 'goog.storage.mechanism.ErrorCode', 'goog.storage.mechanism.IterableMechanism'], false);
goog.addDependency('storage/mechanism/html5webstorage_test.js', ['goog.storage.mechanism.HTML5MockStorage', 'goog.storage.mechanism.HTML5WebStorageTest', 'goog.storage.mechanism.MockThrowableStorage'], ['goog.storage.mechanism.ErrorCode', 'goog.storage.mechanism.HTML5WebStorage', 'goog.testing.jsunit'], false);
goog.addDependency('storage/mechanism/ieuserdata.js', ['goog.storage.mechanism.IEUserData'], ['goog.asserts', 'goog.iter.Iterator', 'goog.iter.StopIteration', 'goog.storage.mechanism.ErrorCode', 'goog.storage.mechanism.IterableMechanism', 'goog.structs.Map', 'goog.userAgent'], false);
goog.addDependency('storage/mechanism/ieuserdata_test.js', ['goog.storage.mechanism.IEUserDataTest'], ['goog.storage.mechanism.IEUserData', 'goog.storage.mechanism.mechanismSeparationTester', 'goog.storage.mechanism.mechanismSharingTester', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('storage/mechanism/iterablemechanism.js', ['goog.storage.mechanism.IterableMechanism'], ['goog.array', 'goog.asserts', 'goog.iter', 'goog.iter.Iterator', 'goog.storage.mechanism.Mechanism'], false);
goog.addDependency('storage/mechanism/iterablemechanismtester.js', ['goog.storage.mechanism.iterableMechanismTester'], ['goog.iter.Iterator', 'goog.storage.mechanism.IterableMechanism', 'goog.testing.asserts'], false);
goog.addDependency('storage/mechanism/mechanism.js', ['goog.storage.mechanism.Mechanism'], [], false);
goog.addDependency('storage/mechanism/mechanismfactory.js', ['goog.storage.mechanism.mechanismfactory'], ['goog.storage.mechanism.HTML5LocalStorage', 'goog.storage.mechanism.HTML5SessionStorage', 'goog.storage.mechanism.IEUserData', 'goog.storage.mechanism.IterableMechanism', 'goog.storage.mechanism.PrefixedMechanism'], false);
goog.addDependency('storage/mechanism/mechanismfactory_test.js', ['goog.storage.mechanism.mechanismfactoryTest'], ['goog.storage.mechanism.mechanismfactory', 'goog.testing.jsunit'], false);
goog.addDependency('storage/mechanism/mechanismseparationtester.js', ['goog.storage.mechanism.mechanismSeparationTester'], ['goog.iter.Iterator', 'goog.storage.mechanism.IterableMechanism', 'goog.testing.asserts'], false);
goog.addDependency('storage/mechanism/mechanismsharingtester.js', ['goog.storage.mechanism.mechanismSharingTester'], ['goog.iter.Iterator', 'goog.storage.mechanism.IterableMechanism', 'goog.testing.asserts'], false);
goog.addDependency('storage/mechanism/mechanismtester.js', ['goog.storage.mechanism.mechanismTester'], ['goog.storage.mechanism.ErrorCode', 'goog.storage.mechanism.HTML5LocalStorage', 'goog.storage.mechanism.Mechanism', 'goog.testing.asserts', 'goog.userAgent.product', 'goog.userAgent.product.isVersion'], false);
goog.addDependency('storage/mechanism/prefixedmechanism.js', ['goog.storage.mechanism.PrefixedMechanism'], ['goog.iter.Iterator', 'goog.storage.mechanism.IterableMechanism'], false);
goog.addDependency('storage/mechanism/prefixedmechanism_test.js', ['goog.storage.mechanism.PrefixedMechanismTest'], ['goog.storage.mechanism.HTML5LocalStorage', 'goog.storage.mechanism.PrefixedMechanism', 'goog.storage.mechanism.mechanismSeparationTester', 'goog.storage.mechanism.mechanismSharingTester', 'goog.testing.jsunit'], false);
goog.addDependency('storage/richstorage.js', ['goog.storage.RichStorage', 'goog.storage.RichStorage.Wrapper'], ['goog.storage.ErrorCode', 'goog.storage.Storage', 'goog.storage.mechanism.Mechanism'], false);
goog.addDependency('storage/richstorage_test.js', ['goog.storage.RichStorageTest'], ['goog.storage.ErrorCode', 'goog.storage.RichStorage', 'goog.storage.storage_test', 'goog.testing.jsunit', 'goog.testing.storage.FakeMechanism'], false);
goog.addDependency('storage/storage.js', ['goog.storage.Storage'], ['goog.json', 'goog.storage.ErrorCode'], false);
goog.addDependency('storage/storage_test.js', ['goog.storage.storage_test'], ['goog.storage.Storage', 'goog.structs.Map', 'goog.testing.asserts'], false);
goog.addDependency('string/const.js', ['goog.string.Const'], ['goog.asserts', 'goog.string.TypedString'], false);
goog.addDependency('string/const_test.js', ['goog.string.constTest'], ['goog.string.Const', 'goog.testing.jsunit'], false);
goog.addDependency('string/linkify.js', ['goog.string.linkify'], ['goog.string'], false);
goog.addDependency('string/linkify_test.js', ['goog.string.linkifyTest'], ['goog.string', 'goog.string.linkify', 'goog.testing.dom', 'goog.testing.jsunit'], false);
goog.addDependency('string/newlines.js', ['goog.string.newlines', 'goog.string.newlines.Line'], ['goog.array'], false);
goog.addDependency('string/newlines_test.js', ['goog.string.newlinesTest'], ['goog.string.newlines', 'goog.testing.jsunit'], false);
goog.addDependency('string/parser.js', ['goog.string.Parser'], [], false);
goog.addDependency('string/path.js', ['goog.string.path'], ['goog.array', 'goog.string'], false);
goog.addDependency('string/path_test.js', ['goog.string.pathTest'], ['goog.string.path', 'goog.testing.jsunit'], false);
goog.addDependency('string/string.js', ['goog.string', 'goog.string.Unicode'], [], false);
goog.addDependency('string/string_test.js', ['goog.stringTest'], ['goog.functions', 'goog.object', 'goog.string', 'goog.string.Unicode', 'goog.testing.MockControl', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit'], false);
goog.addDependency('string/stringbuffer.js', ['goog.string.StringBuffer'], [], false);
goog.addDependency('string/stringbuffer_test.js', ['goog.string.StringBufferTest'], ['goog.string.StringBuffer', 'goog.testing.jsunit'], false);
goog.addDependency('string/stringformat.js', ['goog.string.format'], ['goog.string'], false);
goog.addDependency('string/stringformat_test.js', ['goog.string.formatTest'], ['goog.string.format', 'goog.testing.jsunit'], false);
goog.addDependency('string/stringifier.js', ['goog.string.Stringifier'], [], false);
goog.addDependency('string/typedstring.js', ['goog.string.TypedString'], [], false);
goog.addDependency('structs/avltree.js', ['goog.structs.AvlTree', 'goog.structs.AvlTree.Node'], ['goog.structs.Collection'], false);
goog.addDependency('structs/avltree_test.js', ['goog.structs.AvlTreeTest'], ['goog.array', 'goog.structs.AvlTree', 'goog.testing.jsunit'], false);
goog.addDependency('structs/circularbuffer.js', ['goog.structs.CircularBuffer'], [], false);
goog.addDependency('structs/circularbuffer_test.js', ['goog.structs.CircularBufferTest'], ['goog.structs.CircularBuffer', 'goog.testing.jsunit'], false);
goog.addDependency('structs/collection.js', ['goog.structs.Collection'], [], false);
goog.addDependency('structs/collection_test.js', ['goog.structs.CollectionTest'], ['goog.structs.AvlTree', 'goog.structs.Set', 'goog.testing.jsunit'], false);
goog.addDependency('structs/heap.js', ['goog.structs.Heap'], ['goog.array', 'goog.object', 'goog.structs.Node'], false);
goog.addDependency('structs/heap_test.js', ['goog.structs.HeapTest'], ['goog.structs', 'goog.structs.Heap', 'goog.testing.jsunit'], false);
goog.addDependency('structs/inversionmap.js', ['goog.structs.InversionMap'], ['goog.array'], false);
goog.addDependency('structs/inversionmap_test.js', ['goog.structs.InversionMapTest'], ['goog.structs.InversionMap', 'goog.testing.jsunit'], false);
goog.addDependency('structs/linkedmap.js', ['goog.structs.LinkedMap'], ['goog.structs.Map'], false);
goog.addDependency('structs/linkedmap_test.js', ['goog.structs.LinkedMapTest'], ['goog.structs.LinkedMap', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('structs/map.js', ['goog.structs.Map'], ['goog.iter.Iterator', 'goog.iter.StopIteration', 'goog.object'], false);
goog.addDependency('structs/map_test.js', ['goog.structs.MapTest'], ['goog.iter', 'goog.structs', 'goog.structs.Map', 'goog.testing.jsunit'], false);
goog.addDependency('structs/node.js', ['goog.structs.Node'], [], false);
goog.addDependency('structs/pool.js', ['goog.structs.Pool'], ['goog.Disposable', 'goog.structs.Queue', 'goog.structs.Set'], false);
goog.addDependency('structs/pool_test.js', ['goog.structs.PoolTest'], ['goog.structs.Pool', 'goog.testing.MockClock', 'goog.testing.jsunit'], false);
goog.addDependency('structs/prioritypool.js', ['goog.structs.PriorityPool'], ['goog.structs.Pool', 'goog.structs.PriorityQueue'], false);
goog.addDependency('structs/prioritypool_test.js', ['goog.structs.PriorityPoolTest'], ['goog.structs.PriorityPool', 'goog.testing.MockClock', 'goog.testing.jsunit'], false);
goog.addDependency('structs/priorityqueue.js', ['goog.structs.PriorityQueue'], ['goog.structs.Heap'], false);
goog.addDependency('structs/priorityqueue_test.js', ['goog.structs.PriorityQueueTest'], ['goog.structs', 'goog.structs.PriorityQueue', 'goog.testing.jsunit'], false);
goog.addDependency('structs/quadtree.js', ['goog.structs.QuadTree', 'goog.structs.QuadTree.Node', 'goog.structs.QuadTree.Point'], ['goog.math.Coordinate'], false);
goog.addDependency('structs/quadtree_test.js', ['goog.structs.QuadTreeTest'], ['goog.structs', 'goog.structs.QuadTree', 'goog.testing.jsunit'], false);
goog.addDependency('structs/queue.js', ['goog.structs.Queue'], ['goog.array'], false);
goog.addDependency('structs/queue_test.js', ['goog.structs.QueueTest'], ['goog.structs.Queue', 'goog.testing.jsunit'], false);
goog.addDependency('structs/set.js', ['goog.structs.Set'], ['goog.structs', 'goog.structs.Collection', 'goog.structs.Map'], false);
goog.addDependency('structs/set_test.js', ['goog.structs.SetTest'], ['goog.iter', 'goog.structs', 'goog.structs.Set', 'goog.testing.jsunit'], false);
goog.addDependency('structs/simplepool.js', ['goog.structs.SimplePool'], ['goog.Disposable'], false);
goog.addDependency('structs/stringset.js', ['goog.structs.StringSet'], ['goog.asserts', 'goog.iter'], false);
goog.addDependency('structs/stringset_test.js', ['goog.structs.StringSetTest'], ['goog.array', 'goog.iter', 'goog.structs.StringSet', 'goog.testing.asserts', 'goog.testing.jsunit'], false);
goog.addDependency('structs/structs.js', ['goog.structs'], ['goog.array', 'goog.object'], false);
goog.addDependency('structs/structs_test.js', ['goog.structsTest'], ['goog.array', 'goog.structs', 'goog.structs.Map', 'goog.structs.Set', 'goog.testing.jsunit'], false);
goog.addDependency('structs/treenode.js', ['goog.structs.TreeNode'], ['goog.array', 'goog.asserts', 'goog.structs.Node'], false);
goog.addDependency('structs/treenode_test.js', ['goog.structs.TreeNodeTest'], ['goog.structs.TreeNode', 'goog.testing.jsunit'], false);
goog.addDependency('structs/trie.js', ['goog.structs.Trie'], ['goog.object', 'goog.structs'], false);
goog.addDependency('structs/trie_test.js', ['goog.structs.TrieTest'], ['goog.object', 'goog.structs', 'goog.structs.Trie', 'goog.testing.jsunit'], false);
goog.addDependency('structs/weak/weak.js', ['goog.structs.weak'], ['goog.userAgent'], false);
goog.addDependency('structs/weak/weak_test.js', ['goog.structs.weakTest'], ['goog.array', 'goog.structs.weak', 'goog.testing.jsunit'], false);
goog.addDependency('style/bidi.js', ['goog.style.bidi'], ['goog.dom', 'goog.style', 'goog.userAgent'], false);
goog.addDependency('style/bidi_test.js', ['goog.style.bidiTest'], ['goog.dom', 'goog.style', 'goog.style.bidi', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('style/cursor.js', ['goog.style.cursor'], ['goog.userAgent'], false);
goog.addDependency('style/cursor_test.js', ['goog.style.cursorTest'], ['goog.style.cursor', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('style/style.js', ['goog.style'], ['goog.array', 'goog.asserts', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.vendor', 'goog.math.Box', 'goog.math.Coordinate', 'goog.math.Rect', 'goog.math.Size', 'goog.object', 'goog.string', 'goog.userAgent'], false);
goog.addDependency('style/style_test.js', ['goog.style_test'], ['goog.array', 'goog.color', 'goog.dom', 'goog.events.BrowserEvent', 'goog.labs.userAgent.util', 'goog.math.Box', 'goog.math.Coordinate', 'goog.math.Rect', 'goog.math.Size', 'goog.object', 'goog.string', 'goog.style', 'goog.testing.ExpectedFailures', 'goog.testing.MockUserAgent', 'goog.testing.asserts', 'goog.testing.jsunit', 'goog.userAgent', 'goog.userAgent.product', 'goog.userAgentTestUtil', 'goog.userAgentTestUtil.UserAgents'], false);
goog.addDependency('style/style_webkit_scrollbars_test.js', ['goog.style.webkitScrollbarsTest'], ['goog.asserts', 'goog.style', 'goog.styleScrollbarTester', 'goog.testing.ExpectedFailures', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('style/stylescrollbartester.js', ['goog.styleScrollbarTester'], ['goog.dom', 'goog.style', 'goog.testing.asserts'], false);
goog.addDependency('style/transform.js', ['goog.style.transform'], ['goog.functions', 'goog.math.Coordinate', 'goog.style', 'goog.userAgent', 'goog.userAgent.product.isVersion'], false);
goog.addDependency('style/transform_test.js', ['goog.style.transformTest'], ['goog.dom', 'goog.style.transform', 'goog.testing.jsunit', 'goog.userAgent', 'goog.userAgent.product.isVersion'], false);
goog.addDependency('style/transition.js', ['goog.style.transition', 'goog.style.transition.Css3Property'], ['goog.array', 'goog.asserts', 'goog.dom.safe', 'goog.dom.vendor', 'goog.functions', 'goog.html.SafeHtml', 'goog.style', 'goog.userAgent'], false);
goog.addDependency('style/transition_test.js', ['goog.style.transitionTest'], ['goog.style', 'goog.style.transition', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('test_module.js', ['goog.test_module'], ['goog.test_module_dep'], true);
goog.addDependency('test_module_dep.js', ['goog.test_module_dep'], [], true);
goog.addDependency('testing/asserts.js', ['goog.testing.JsUnitException', 'goog.testing.asserts'], ['goog.testing.stacktrace'], false);
goog.addDependency('testing/asserts_test.js', ['goog.testing.assertsTest'], ['goog.array', 'goog.dom', 'goog.iter.Iterator', 'goog.iter.StopIteration', 'goog.labs.userAgent.browser', 'goog.string', 'goog.structs.Map', 'goog.structs.Set', 'goog.testing.asserts', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('testing/async/mockcontrol.js', ['goog.testing.async.MockControl'], ['goog.asserts', 'goog.async.Deferred', 'goog.debug', 'goog.testing.asserts', 'goog.testing.mockmatchers.IgnoreArgument'], false);
goog.addDependency('testing/async/mockcontrol_test.js', ['goog.testing.async.MockControlTest'], ['goog.async.Deferred', 'goog.testing.MockControl', 'goog.testing.asserts', 'goog.testing.async.MockControl', 'goog.testing.jsunit'], false);
goog.addDependency('testing/asynctestcase.js', ['goog.testing.AsyncTestCase', 'goog.testing.AsyncTestCase.ControlBreakingException'], ['goog.testing.TestCase', 'goog.testing.TestCase.Test', 'goog.testing.asserts'], false);
goog.addDependency('testing/asynctestcase_async_test.js', ['goog.testing.AsyncTestCaseAsyncTest'], ['goog.testing.AsyncTestCase', 'goog.testing.jsunit'], false);
goog.addDependency('testing/asynctestcase_noasync_test.js', ['goog.testing.AsyncTestCaseSyncTest'], ['goog.testing.AsyncTestCase', 'goog.testing.jsunit'], false);
goog.addDependency('testing/asynctestcase_test.js', ['goog.testing.AsyncTestCaseTest'], ['goog.debug.Error', 'goog.testing.AsyncTestCase', 'goog.testing.asserts', 'goog.testing.jsunit'], false);
goog.addDependency('testing/benchmark.js', ['goog.testing.benchmark'], ['goog.dom', 'goog.dom.TagName', 'goog.testing.PerformanceTable', 'goog.testing.PerformanceTimer', 'goog.testing.TestCase'], false);
goog.addDependency('testing/continuationtestcase.js', ['goog.testing.ContinuationTestCase', 'goog.testing.ContinuationTestCase.Step', 'goog.testing.ContinuationTestCase.Test'], ['goog.array', 'goog.events.EventHandler', 'goog.testing.TestCase', 'goog.testing.TestCase.Test', 'goog.testing.asserts'], false);
goog.addDependency('testing/continuationtestcase_test.js', ['goog.testing.ContinuationTestCaseTest'], ['goog.events', 'goog.events.EventTarget', 'goog.testing.ContinuationTestCase', 'goog.testing.MockClock', 'goog.testing.PropertyReplacer', 'goog.testing.TestCase', 'goog.testing.jsunit'], false);
goog.addDependency('testing/deferredtestcase.js', ['goog.testing.DeferredTestCase'], ['goog.async.Deferred', 'goog.testing.AsyncTestCase', 'goog.testing.TestCase'], false);
goog.addDependency('testing/deferredtestcase_test.js', ['goog.testing.DeferredTestCaseTest'], ['goog.async.Deferred', 'goog.testing.DeferredTestCase', 'goog.testing.TestCase', 'goog.testing.TestRunner', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('testing/dom.js', ['goog.testing.dom'], ['goog.array', 'goog.asserts', 'goog.dom', 'goog.dom.NodeIterator', 'goog.dom.NodeType', 'goog.dom.TagIterator', 'goog.dom.TagName', 'goog.dom.classlist', 'goog.iter', 'goog.object', 'goog.string', 'goog.style', 'goog.testing.asserts', 'goog.userAgent'], false);
goog.addDependency('testing/dom_test.js', ['goog.testing.domTest'], ['goog.dom', 'goog.dom.TagName', 'goog.testing.dom', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('testing/editor/dom.js', ['goog.testing.editor.dom'], ['goog.dom.NodeType', 'goog.dom.TagIterator', 'goog.dom.TagWalkType', 'goog.iter', 'goog.string', 'goog.testing.asserts'], false);
goog.addDependency('testing/editor/dom_test.js', ['goog.testing.editor.domTest'], ['goog.dom', 'goog.dom.TagName', 'goog.functions', 'goog.testing.editor.dom', 'goog.testing.jsunit'], false);
goog.addDependency('testing/editor/fieldmock.js', ['goog.testing.editor.FieldMock'], ['goog.dom', 'goog.dom.Range', 'goog.editor.Field', 'goog.testing.LooseMock', 'goog.testing.mockmatchers'], false);
goog.addDependency('testing/editor/testhelper.js', ['goog.testing.editor.TestHelper'], ['goog.Disposable', 'goog.dom', 'goog.dom.Range', 'goog.editor.BrowserFeature', 'goog.editor.node', 'goog.editor.plugins.AbstractBubblePlugin', 'goog.testing.dom'], false);
goog.addDependency('testing/editor/testhelper_test.js', ['goog.testing.editor.TestHelperTest'], ['goog.dom', 'goog.dom.TagName', 'goog.editor.node', 'goog.testing.editor.TestHelper', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('testing/events/eventobserver.js', ['goog.testing.events.EventObserver'], ['goog.array'], false);
goog.addDependency('testing/events/eventobserver_test.js', ['goog.testing.events.EventObserverTest'], ['goog.array', 'goog.events', 'goog.events.Event', 'goog.events.EventTarget', 'goog.testing.events.EventObserver', 'goog.testing.jsunit'], false);
goog.addDependency('testing/events/events.js', ['goog.testing.events', 'goog.testing.events.Event'], ['goog.Disposable', 'goog.asserts', 'goog.dom.NodeType', 'goog.events', 'goog.events.BrowserEvent', 'goog.events.BrowserFeature', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.object', 'goog.style', 'goog.userAgent'], false);
goog.addDependency('testing/events/events_test.js', ['goog.testing.eventsTest'], ['goog.array', 'goog.dom', 'goog.events', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.math.Coordinate', 'goog.string', 'goog.style', 'goog.testing.PropertyReplacer', 'goog.testing.events', 'goog.testing.jsunit', 'goog.testing.recordFunction', 'goog.userAgent'], false);
goog.addDependency('testing/events/matchers.js', ['goog.testing.events.EventMatcher'], ['goog.events.Event', 'goog.testing.mockmatchers.ArgumentMatcher'], false);
goog.addDependency('testing/events/matchers_test.js', ['goog.testing.events.EventMatcherTest'], ['goog.events.Event', 'goog.testing.events.EventMatcher', 'goog.testing.jsunit'], false);
goog.addDependency('testing/events/onlinehandler.js', ['goog.testing.events.OnlineHandler'], ['goog.events.EventTarget', 'goog.net.NetworkStatusMonitor'], false);
goog.addDependency('testing/events/onlinehandler_test.js', ['goog.testing.events.OnlineHandlerTest'], ['goog.events', 'goog.net.NetworkStatusMonitor', 'goog.testing.events.EventObserver', 'goog.testing.events.OnlineHandler', 'goog.testing.jsunit'], false);
goog.addDependency('testing/expectedfailures.js', ['goog.testing.ExpectedFailures'], ['goog.debug.DivConsole', 'goog.dom', 'goog.dom.TagName', 'goog.events', 'goog.events.EventType', 'goog.log', 'goog.style', 'goog.testing.JsUnitException', 'goog.testing.TestCase', 'goog.testing.asserts'], false);
goog.addDependency('testing/expectedfailures_test.js', ['goog.testing.ExpectedFailuresTest'], ['goog.debug.Logger', 'goog.testing.ExpectedFailures', 'goog.testing.JsUnitException', 'goog.testing.jsunit'], false);
goog.addDependency('testing/fs/blob.js', ['goog.testing.fs.Blob'], ['goog.crypt.base64'], false);
goog.addDependency('testing/fs/blob_test.js', ['goog.testing.fs.BlobTest'], ['goog.testing.fs.Blob', 'goog.testing.jsunit'], false);
goog.addDependency('testing/fs/directoryentry_test.js', ['goog.testing.fs.DirectoryEntryTest'], ['goog.array', 'goog.fs.DirectoryEntry', 'goog.fs.Error', 'goog.testing.AsyncTestCase', 'goog.testing.MockClock', 'goog.testing.fs.FileSystem', 'goog.testing.jsunit'], false);
goog.addDependency('testing/fs/entry.js', ['goog.testing.fs.DirectoryEntry', 'goog.testing.fs.Entry', 'goog.testing.fs.FileEntry'], ['goog.Timer', 'goog.array', 'goog.asserts', 'goog.async.Deferred', 'goog.fs.DirectoryEntry', 'goog.fs.DirectoryEntryImpl', 'goog.fs.Entry', 'goog.fs.Error', 'goog.fs.FileEntry', 'goog.functions', 'goog.object', 'goog.string', 'goog.testing.fs.File', 'goog.testing.fs.FileWriter'], false);
goog.addDependency('testing/fs/entry_test.js', ['goog.testing.fs.EntryTest'], ['goog.fs.DirectoryEntry', 'goog.fs.Error', 'goog.testing.AsyncTestCase', 'goog.testing.MockClock', 'goog.testing.fs.FileSystem', 'goog.testing.jsunit'], false);
goog.addDependency('testing/fs/file.js', ['goog.testing.fs.File'], ['goog.testing.fs.Blob'], false);
goog.addDependency('testing/fs/fileentry_test.js', ['goog.testing.fs.FileEntryTest'], ['goog.testing.AsyncTestCase', 'goog.testing.MockClock', 'goog.testing.fs.FileEntry', 'goog.testing.fs.FileSystem', 'goog.testing.jsunit'], false);
goog.addDependency('testing/fs/filereader.js', ['goog.testing.fs.FileReader'], ['goog.Timer', 'goog.events.EventTarget', 'goog.fs.Error', 'goog.fs.FileReader', 'goog.testing.fs.ProgressEvent'], false);
goog.addDependency('testing/fs/filereader_test.js', ['goog.testing.fs.FileReaderTest'], ['goog.Timer', 'goog.async.Deferred', 'goog.events', 'goog.fs.Error', 'goog.fs.FileReader', 'goog.fs.FileSaver', 'goog.testing.AsyncTestCase', 'goog.testing.fs.FileReader', 'goog.testing.fs.FileSystem', 'goog.testing.jsunit'], false);
goog.addDependency('testing/fs/filesystem.js', ['goog.testing.fs.FileSystem'], ['goog.fs.FileSystem', 'goog.testing.fs.DirectoryEntry'], false);
goog.addDependency('testing/fs/filewriter.js', ['goog.testing.fs.FileWriter'], ['goog.Timer', 'goog.events.EventTarget', 'goog.fs.Error', 'goog.fs.FileSaver', 'goog.string', 'goog.testing.fs.ProgressEvent'], false);
goog.addDependency('testing/fs/filewriter_test.js', ['goog.testing.fs.FileWriterTest'], ['goog.async.Deferred', 'goog.events', 'goog.fs.Error', 'goog.fs.FileSaver', 'goog.testing.AsyncTestCase', 'goog.testing.MockClock', 'goog.testing.fs.Blob', 'goog.testing.fs.FileSystem', 'goog.testing.jsunit'], false);
goog.addDependency('testing/fs/fs.js', ['goog.testing.fs'], ['goog.Timer', 'goog.array', 'goog.async.Deferred', 'goog.fs', 'goog.testing.fs.Blob', 'goog.testing.fs.FileSystem'], false);
goog.addDependency('testing/fs/fs_test.js', ['goog.testing.fsTest'], ['goog.testing.AsyncTestCase', 'goog.testing.fs', 'goog.testing.fs.Blob', 'goog.testing.jsunit'], false);
goog.addDependency('testing/fs/integration_test.js', ['goog.testing.fs.integrationTest'], ['goog.async.Deferred', 'goog.async.DeferredList', 'goog.events', 'goog.fs', 'goog.fs.DirectoryEntry', 'goog.fs.Error', 'goog.fs.FileSaver', 'goog.testing.AsyncTestCase', 'goog.testing.PropertyReplacer', 'goog.testing.fs', 'goog.testing.jsunit'], false);
goog.addDependency('testing/fs/progressevent.js', ['goog.testing.fs.ProgressEvent'], ['goog.events.Event'], false);
goog.addDependency('testing/functionmock.js', ['goog.testing', 'goog.testing.FunctionMock', 'goog.testing.GlobalFunctionMock', 'goog.testing.MethodMock'], ['goog.object', 'goog.testing.LooseMock', 'goog.testing.Mock', 'goog.testing.MockInterface', 'goog.testing.PropertyReplacer', 'goog.testing.StrictMock'], false);
goog.addDependency('testing/functionmock_test.js', ['goog.testing.FunctionMockTest'], ['goog.array', 'goog.string', 'goog.testing', 'goog.testing.FunctionMock', 'goog.testing.Mock', 'goog.testing.StrictMock', 'goog.testing.asserts', 'goog.testing.jsunit', 'goog.testing.mockmatchers'], false);
goog.addDependency('testing/graphics.js', ['goog.testing.graphics'], ['goog.graphics.Path.Segment', 'goog.testing.asserts'], false);
goog.addDependency('testing/i18n/asserts.js', ['goog.testing.i18n.asserts'], ['goog.testing.jsunit'], false);
goog.addDependency('testing/i18n/asserts_test.js', ['goog.testing.i18n.assertsTest'], ['goog.testing.ExpectedFailures', 'goog.testing.i18n.asserts'], false);
goog.addDependency('testing/jsunit.js', ['goog.testing.jsunit'], ['goog.testing.TestCase', 'goog.testing.TestRunner'], false);
goog.addDependency('testing/loosemock.js', ['goog.testing.LooseExpectationCollection', 'goog.testing.LooseMock'], ['goog.array', 'goog.structs.Map', 'goog.testing.Mock'], false);
goog.addDependency('testing/loosemock_test.js', ['goog.testing.LooseMockTest'], ['goog.testing.LooseMock', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.mockmatchers'], false);
goog.addDependency('testing/messaging/mockmessagechannel.js', ['goog.testing.messaging.MockMessageChannel'], ['goog.messaging.AbstractChannel', 'goog.testing.asserts'], false);
goog.addDependency('testing/messaging/mockmessageevent.js', ['goog.testing.messaging.MockMessageEvent'], ['goog.events.BrowserEvent', 'goog.events.EventType', 'goog.testing.events'], false);
goog.addDependency('testing/messaging/mockmessageport.js', ['goog.testing.messaging.MockMessagePort'], ['goog.events.EventTarget'], false);
goog.addDependency('testing/messaging/mockportnetwork.js', ['goog.testing.messaging.MockPortNetwork'], ['goog.messaging.PortNetwork', 'goog.testing.messaging.MockMessageChannel'], false);
goog.addDependency('testing/mock.js', ['goog.testing.Mock', 'goog.testing.MockExpectation'], ['goog.array', 'goog.object', 'goog.testing.JsUnitException', 'goog.testing.MockInterface', 'goog.testing.mockmatchers'], false);
goog.addDependency('testing/mock_test.js', ['goog.testing.MockTest'], ['goog.array', 'goog.testing', 'goog.testing.Mock', 'goog.testing.MockControl', 'goog.testing.MockExpectation', 'goog.testing.jsunit'], false);
goog.addDependency('testing/mockclassfactory.js', ['goog.testing.MockClassFactory', 'goog.testing.MockClassRecord'], ['goog.array', 'goog.object', 'goog.testing.LooseMock', 'goog.testing.StrictMock', 'goog.testing.TestCase', 'goog.testing.mockmatchers'], false);
goog.addDependency('testing/mockclassfactory_test.js', ['fake.BaseClass', 'fake.ChildClass', 'goog.testing.MockClassFactoryTest'], ['goog.testing', 'goog.testing.MockClassFactory', 'goog.testing.jsunit'], false);
goog.addDependency('testing/mockclock.js', ['goog.testing.MockClock'], ['goog.Disposable', 'goog.async.run', 'goog.testing.PropertyReplacer', 'goog.testing.events', 'goog.testing.events.Event', 'goog.testing.watchers'], false);
goog.addDependency('testing/mockclock_test.js', ['goog.testing.MockClockTest'], ['goog.Promise', 'goog.Timer', 'goog.events', 'goog.functions', 'goog.testing.MockClock', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.recordFunction'], false);
goog.addDependency('testing/mockcontrol.js', ['goog.testing.MockControl'], ['goog.array', 'goog.testing', 'goog.testing.LooseMock', 'goog.testing.StrictMock'], false);
goog.addDependency('testing/mockcontrol_test.js', ['goog.testing.MockControlTest'], ['goog.testing.Mock', 'goog.testing.MockControl', 'goog.testing.jsunit'], false);
goog.addDependency('testing/mockinterface.js', ['goog.testing.MockInterface'], [], false);
goog.addDependency('testing/mockmatchers.js', ['goog.testing.mockmatchers', 'goog.testing.mockmatchers.ArgumentMatcher', 'goog.testing.mockmatchers.IgnoreArgument', 'goog.testing.mockmatchers.InstanceOf', 'goog.testing.mockmatchers.ObjectEquals', 'goog.testing.mockmatchers.RegexpMatch', 'goog.testing.mockmatchers.SaveArgument', 'goog.testing.mockmatchers.TypeOf'], ['goog.array', 'goog.dom', 'goog.testing.asserts'], false);
goog.addDependency('testing/mockmatchers_test.js', ['goog.testing.mockmatchersTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.testing.mockmatchers', 'goog.testing.mockmatchers.ArgumentMatcher'], false);
goog.addDependency('testing/mockrandom.js', ['goog.testing.MockRandom'], ['goog.Disposable'], false);
goog.addDependency('testing/mockrandom_test.js', ['goog.testing.MockRandomTest'], ['goog.testing.MockRandom', 'goog.testing.jsunit'], false);
goog.addDependency('testing/mockrange.js', ['goog.testing.MockRange'], ['goog.dom.AbstractRange', 'goog.testing.LooseMock'], false);
goog.addDependency('testing/mockrange_test.js', ['goog.testing.MockRangeTest'], ['goog.testing.MockRange', 'goog.testing.jsunit'], false);
goog.addDependency('testing/mockstorage.js', ['goog.testing.MockStorage'], ['goog.structs.Map'], false);
goog.addDependency('testing/mockstorage_test.js', ['goog.testing.MockStorageTest'], ['goog.testing.MockStorage', 'goog.testing.jsunit'], false);
goog.addDependency('testing/mockuseragent.js', ['goog.testing.MockUserAgent'], ['goog.Disposable', 'goog.labs.userAgent.util', 'goog.testing.PropertyReplacer', 'goog.userAgent'], false);
goog.addDependency('testing/mockuseragent_test.js', ['goog.testing.MockUserAgentTest'], ['goog.dispose', 'goog.testing.MockUserAgent', 'goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('testing/multitestrunner.js', ['goog.testing.MultiTestRunner', 'goog.testing.MultiTestRunner.TestFrame'], ['goog.Timer', 'goog.array', 'goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.events.EventHandler', 'goog.functions', 'goog.string', 'goog.ui.Component', 'goog.ui.ServerChart', 'goog.ui.TableSorter'], false);
goog.addDependency('testing/net/xhrio.js', ['goog.testing.net.XhrIo'], ['goog.array', 'goog.dom.xml', 'goog.events', 'goog.events.EventTarget', 'goog.json', 'goog.net.ErrorCode', 'goog.net.EventType', 'goog.net.HttpStatus', 'goog.net.XhrIo', 'goog.net.XmlHttp', 'goog.object', 'goog.structs.Map'], false);
goog.addDependency('testing/net/xhrio_test.js', ['goog.testing.net.XhrIoTest'], ['goog.dom.xml', 'goog.events', 'goog.events.Event', 'goog.net.ErrorCode', 'goog.net.EventType', 'goog.net.XmlHttp', 'goog.object', 'goog.testing.MockControl', 'goog.testing.asserts', 'goog.testing.jsunit', 'goog.testing.mockmatchers.InstanceOf', 'goog.testing.net.XhrIo'], false);
goog.addDependency('testing/net/xhriopool.js', ['goog.testing.net.XhrIoPool'], ['goog.net.XhrIoPool', 'goog.testing.net.XhrIo'], false);
goog.addDependency('testing/objectpropertystring.js', ['goog.testing.ObjectPropertyString'], [], false);
goog.addDependency('testing/performancetable.js', ['goog.testing.PerformanceTable'], ['goog.dom', 'goog.testing.PerformanceTimer'], false);
goog.addDependency('testing/performancetimer.js', ['goog.testing.PerformanceTimer', 'goog.testing.PerformanceTimer.Task'], ['goog.array', 'goog.async.Deferred', 'goog.math'], false);
goog.addDependency('testing/performancetimer_test.js', ['goog.testing.PerformanceTimerTest'], ['goog.async.Deferred', 'goog.dom', 'goog.math', 'goog.testing.MockClock', 'goog.testing.PerformanceTimer', 'goog.testing.jsunit'], false);
goog.addDependency('testing/propertyreplacer.js', ['goog.testing.PropertyReplacer'], ['goog.testing.ObjectPropertyString', 'goog.userAgent'], false);
goog.addDependency('testing/propertyreplacer_test.js', ['goog.testing.PropertyReplacerTest'], ['goog.testing.PropertyReplacer', 'goog.testing.asserts', 'goog.testing.jsunit'], false);
goog.addDependency('testing/proto2/proto2.js', ['goog.testing.proto2'], ['goog.proto2.Message', 'goog.proto2.ObjectSerializer', 'goog.testing.asserts'], false);
goog.addDependency('testing/proto2/proto2_test.js', ['goog.testing.proto2Test'], ['goog.testing.jsunit', 'goog.testing.proto2', 'proto2.TestAllTypes'], false);
goog.addDependency('testing/pseudorandom.js', ['goog.testing.PseudoRandom'], ['goog.Disposable'], false);
goog.addDependency('testing/pseudorandom_test.js', ['goog.testing.PseudoRandomTest'], ['goog.testing.PseudoRandom', 'goog.testing.jsunit'], false);
goog.addDependency('testing/recordfunction.js', ['goog.testing.FunctionCall', 'goog.testing.recordConstructor', 'goog.testing.recordFunction'], ['goog.testing.asserts'], false);
goog.addDependency('testing/recordfunction_test.js', ['goog.testing.recordFunctionTest'], ['goog.functions', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.recordConstructor', 'goog.testing.recordFunction'], false);
goog.addDependency('testing/shardingtestcase.js', ['goog.testing.ShardingTestCase'], ['goog.asserts', 'goog.testing.TestCase'], false);
goog.addDependency('testing/shardingtestcase_test.js', ['goog.testing.ShardingTestCaseTest'], ['goog.testing.ShardingTestCase', 'goog.testing.TestCase', 'goog.testing.asserts', 'goog.testing.jsunit'], false);
goog.addDependency('testing/singleton.js', ['goog.testing.singleton'], [], false);
goog.addDependency('testing/singleton_test.js', ['goog.testing.singletonTest'], ['goog.testing.asserts', 'goog.testing.jsunit', 'goog.testing.singleton'], false);
goog.addDependency('testing/stacktrace.js', ['goog.testing.stacktrace', 'goog.testing.stacktrace.Frame'], [], false);
goog.addDependency('testing/stacktrace_test.js', ['goog.testing.stacktraceTest'], ['goog.functions', 'goog.string', 'goog.testing.ExpectedFailures', 'goog.testing.PropertyReplacer', 'goog.testing.StrictMock', 'goog.testing.asserts', 'goog.testing.jsunit', 'goog.testing.stacktrace', 'goog.testing.stacktrace.Frame', 'goog.userAgent'], false);
goog.addDependency('testing/storage/fakemechanism.js', ['goog.testing.storage.FakeMechanism'], ['goog.storage.mechanism.IterableMechanism', 'goog.structs.Map'], false);
goog.addDependency('testing/strictmock.js', ['goog.testing.StrictMock'], ['goog.array', 'goog.testing.Mock'], false);
goog.addDependency('testing/strictmock_test.js', ['goog.testing.StrictMockTest'], ['goog.testing.StrictMock', 'goog.testing.jsunit'], false);
goog.addDependency('testing/style/layoutasserts.js', ['goog.testing.style.layoutasserts'], ['goog.style', 'goog.testing.asserts', 'goog.testing.style'], false);
goog.addDependency('testing/style/layoutasserts_test.js', ['goog.testing.style.layoutassertsTest'], ['goog.dom', 'goog.style', 'goog.testing.jsunit', 'goog.testing.style.layoutasserts'], false);
goog.addDependency('testing/style/style.js', ['goog.testing.style'], ['goog.dom', 'goog.math.Rect', 'goog.style'], false);
goog.addDependency('testing/style/style_test.js', ['goog.testing.styleTest'], ['goog.dom', 'goog.style', 'goog.testing.jsunit', 'goog.testing.style'], false);
goog.addDependency('testing/testcase.js', ['goog.testing.TestCase', 'goog.testing.TestCase.Error', 'goog.testing.TestCase.Order', 'goog.testing.TestCase.Result', 'goog.testing.TestCase.Test'], ['goog.object', 'goog.testing.asserts', 'goog.testing.stacktrace'], false);
goog.addDependency('testing/testcase_test.js', ['goog.testing.TestCaseTest'], ['goog.testing.TestCase', 'goog.testing.jsunit'], false);
goog.addDependency('testing/testqueue.js', ['goog.testing.TestQueue'], [], false);
goog.addDependency('testing/testrunner.js', ['goog.testing.TestRunner'], ['goog.testing.TestCase'], false);
goog.addDependency('testing/ui/rendererasserts.js', ['goog.testing.ui.rendererasserts'], ['goog.testing.asserts'], false);
goog.addDependency('testing/ui/rendererasserts_test.js', ['goog.testing.ui.rendererassertsTest'], ['goog.testing.asserts', 'goog.testing.jsunit', 'goog.testing.ui.rendererasserts', 'goog.ui.ControlRenderer'], false);
goog.addDependency('testing/ui/rendererharness.js', ['goog.testing.ui.RendererHarness'], ['goog.Disposable', 'goog.dom.NodeType', 'goog.testing.asserts', 'goog.testing.dom'], false);
goog.addDependency('testing/ui/style.js', ['goog.testing.ui.style'], ['goog.array', 'goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.testing.asserts'], false);
goog.addDependency('testing/ui/style_test.js', ['goog.testing.ui.styleTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.testing.ui.style'], false);
goog.addDependency('testing/watchers.js', ['goog.testing.watchers'], [], false);
goog.addDependency('timer/timer.js', ['goog.Timer'], ['goog.Promise', 'goog.events.EventTarget'], false);
goog.addDependency('timer/timer_test.js', ['goog.TimerTest'], ['goog.Promise', 'goog.Timer', 'goog.events', 'goog.testing.MockClock', 'goog.testing.jsunit'], false);
goog.addDependency('tweak/entries.js', ['goog.tweak.BaseEntry', 'goog.tweak.BasePrimitiveSetting', 'goog.tweak.BaseSetting', 'goog.tweak.BooleanGroup', 'goog.tweak.BooleanInGroupSetting', 'goog.tweak.BooleanSetting', 'goog.tweak.ButtonAction', 'goog.tweak.NumericSetting', 'goog.tweak.StringSetting'], ['goog.array', 'goog.asserts', 'goog.log', 'goog.object'], false);
goog.addDependency('tweak/entries_test.js', ['goog.tweak.BaseEntryTest'], ['goog.testing.MockControl', 'goog.testing.jsunit', 'goog.tweak.testhelpers'], false);
goog.addDependency('tweak/registry.js', ['goog.tweak.Registry'], ['goog.asserts', 'goog.log', 'goog.object', 'goog.string', 'goog.tweak.BaseEntry', 'goog.uri.utils'], false);
goog.addDependency('tweak/registry_test.js', ['goog.tweak.RegistryTest'], ['goog.asserts.AssertionError', 'goog.testing.jsunit', 'goog.tweak', 'goog.tweak.testhelpers'], false);
goog.addDependency('tweak/testhelpers.js', ['goog.tweak.testhelpers'], ['goog.tweak', 'goog.tweak.BooleanGroup', 'goog.tweak.BooleanInGroupSetting', 'goog.tweak.BooleanSetting', 'goog.tweak.ButtonAction', 'goog.tweak.NumericSetting', 'goog.tweak.Registry', 'goog.tweak.StringSetting'], false);
goog.addDependency('tweak/tweak.js', ['goog.tweak', 'goog.tweak.ConfigParams'], ['goog.asserts', 'goog.tweak.BaseSetting', 'goog.tweak.BooleanGroup', 'goog.tweak.BooleanInGroupSetting', 'goog.tweak.BooleanSetting', 'goog.tweak.ButtonAction', 'goog.tweak.NumericSetting', 'goog.tweak.Registry', 'goog.tweak.StringSetting'], false);
goog.addDependency('tweak/tweakui.js', ['goog.tweak.EntriesPanel', 'goog.tweak.TweakUi'], ['goog.array', 'goog.asserts', 'goog.dom.DomHelper', 'goog.object', 'goog.style', 'goog.tweak', 'goog.ui.Zippy', 'goog.userAgent'], false);
goog.addDependency('tweak/tweakui_test.js', ['goog.tweak.TweakUiTest'], ['goog.dom', 'goog.string', 'goog.testing.jsunit', 'goog.tweak', 'goog.tweak.TweakUi', 'goog.tweak.testhelpers'], false);
goog.addDependency('ui/abstractspellchecker.js', ['goog.ui.AbstractSpellChecker', 'goog.ui.AbstractSpellChecker.AsyncResult'], ['goog.a11y.aria', 'goog.array', 'goog.asserts', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.classlist', 'goog.dom.selection', 'goog.events', 'goog.events.Event', 'goog.events.EventType', 'goog.math.Coordinate', 'goog.spell.SpellCheck', 'goog.structs.Set', 'goog.style', 'goog.ui.Component', 'goog.ui.MenuItem', 'goog.ui.MenuSeparator', 'goog.ui.PopupMenu'], false);
goog.addDependency('ui/ac/ac.js', ['goog.ui.ac'], ['goog.ui.ac.ArrayMatcher', 'goog.ui.ac.AutoComplete', 'goog.ui.ac.InputHandler', 'goog.ui.ac.Renderer'], false);
goog.addDependency('ui/ac/ac_test.js', ['goog.ui.acTest'], ['goog.array', 'goog.asserts', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.classlist', 'goog.dom.selection', 'goog.events', 'goog.events.BrowserEvent', 'goog.events.Event', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.style', 'goog.testing.MockClock', 'goog.testing.jsunit', 'goog.ui.ac', 'goog.userAgent'], false);
goog.addDependency('ui/ac/arraymatcher.js', ['goog.ui.ac.ArrayMatcher'], ['goog.string'], false);
goog.addDependency('ui/ac/arraymatcher_test.js', ['goog.ui.ac.ArrayMatcherTest'], ['goog.testing.jsunit', 'goog.ui.ac.ArrayMatcher'], false);
goog.addDependency('ui/ac/autocomplete.js', ['goog.ui.ac.AutoComplete', 'goog.ui.ac.AutoComplete.EventType'], ['goog.array', 'goog.asserts', 'goog.events', 'goog.events.EventTarget', 'goog.object'], false);
goog.addDependency('ui/ac/autocomplete_test.js', ['goog.ui.ac.AutoCompleteTest'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.dom', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.string', 'goog.testing.MockControl', 'goog.testing.events', 'goog.testing.jsunit', 'goog.testing.mockmatchers', 'goog.ui.ac.AutoComplete', 'goog.ui.ac.InputHandler', 'goog.ui.ac.RenderOptions', 'goog.ui.ac.Renderer'], false);
goog.addDependency('ui/ac/cachingmatcher.js', ['goog.ui.ac.CachingMatcher'], ['goog.array', 'goog.async.Throttle', 'goog.ui.ac.ArrayMatcher', 'goog.ui.ac.RenderOptions'], false);
goog.addDependency('ui/ac/cachingmatcher_test.js', ['goog.ui.ac.CachingMatcherTest'], ['goog.testing.MockControl', 'goog.testing.jsunit', 'goog.testing.mockmatchers', 'goog.ui.ac.CachingMatcher'], false);
goog.addDependency('ui/ac/inputhandler.js', ['goog.ui.ac.InputHandler'], ['goog.Disposable', 'goog.Timer', 'goog.a11y.aria', 'goog.dom', 'goog.dom.selection', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.events.KeyHandler', 'goog.string', 'goog.userAgent', 'goog.userAgent.product'], false);
goog.addDependency('ui/ac/inputhandler_test.js', ['goog.ui.ac.InputHandlerTest'], ['goog.dom.selection', 'goog.events.BrowserEvent', 'goog.events.Event', 'goog.events.EventTarget', 'goog.events.KeyCodes', 'goog.functions', 'goog.object', 'goog.testing.MockClock', 'goog.testing.jsunit', 'goog.ui.ac.InputHandler', 'goog.userAgent'], false);
goog.addDependency('ui/ac/remote.js', ['goog.ui.ac.Remote'], ['goog.ui.ac.AutoComplete', 'goog.ui.ac.InputHandler', 'goog.ui.ac.RemoteArrayMatcher', 'goog.ui.ac.Renderer'], false);
goog.addDependency('ui/ac/remotearraymatcher.js', ['goog.ui.ac.RemoteArrayMatcher'], ['goog.Disposable', 'goog.Uri', 'goog.events', 'goog.json', 'goog.net.EventType', 'goog.net.XhrIo'], false);
goog.addDependency('ui/ac/remotearraymatcher_test.js', ['goog.ui.ac.RemoteArrayMatcherTest'], ['goog.json', 'goog.net.XhrIo', 'goog.testing.MockControl', 'goog.testing.jsunit', 'goog.testing.net.XhrIo', 'goog.ui.ac.RemoteArrayMatcher'], false);
goog.addDependency('ui/ac/renderer.js', ['goog.ui.ac.Renderer', 'goog.ui.ac.Renderer.CustomRenderer'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.array', 'goog.asserts', 'goog.dispose', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.classlist', 'goog.events', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.fx.dom.FadeInAndShow', 'goog.fx.dom.FadeOutAndHide', 'goog.positioning', 'goog.positioning.Corner', 'goog.positioning.Overflow', 'goog.string', 'goog.style', 'goog.ui.IdGenerator', 'goog.ui.ac.AutoComplete'], false);
goog.addDependency('ui/ac/renderer_test.js', ['goog.ui.ac.RendererTest'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.dom', 'goog.dom.classlist', 'goog.events', 'goog.fx.dom.FadeInAndShow', 'goog.fx.dom.FadeOutAndHide', 'goog.string', 'goog.style', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.ui.ac.AutoComplete', 'goog.ui.ac.Renderer'], false);
goog.addDependency('ui/ac/renderoptions.js', ['goog.ui.ac.RenderOptions'], [], false);
goog.addDependency('ui/ac/richinputhandler.js', ['goog.ui.ac.RichInputHandler'], ['goog.ui.ac.InputHandler'], false);
goog.addDependency('ui/ac/richremote.js', ['goog.ui.ac.RichRemote'], ['goog.ui.ac.AutoComplete', 'goog.ui.ac.Remote', 'goog.ui.ac.Renderer', 'goog.ui.ac.RichInputHandler', 'goog.ui.ac.RichRemoteArrayMatcher'], false);
goog.addDependency('ui/ac/richremotearraymatcher.js', ['goog.ui.ac.RichRemoteArrayMatcher'], ['goog.json', 'goog.ui.ac.RemoteArrayMatcher'], false);
goog.addDependency('ui/activitymonitor.js', ['goog.ui.ActivityMonitor'], ['goog.array', 'goog.asserts', 'goog.dom', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.events.EventType'], false);
goog.addDependency('ui/activitymonitor_test.js', ['goog.ui.ActivityMonitorTest'], ['goog.dom', 'goog.events', 'goog.events.Event', 'goog.events.EventType', 'goog.testing.MockClock', 'goog.testing.PropertyReplacer', 'goog.testing.events', 'goog.testing.jsunit', 'goog.testing.recordFunction', 'goog.ui.ActivityMonitor'], false);
goog.addDependency('ui/advancedtooltip.js', ['goog.ui.AdvancedTooltip'], ['goog.events', 'goog.events.EventType', 'goog.math.Box', 'goog.math.Coordinate', 'goog.style', 'goog.ui.Tooltip', 'goog.userAgent'], false);
goog.addDependency('ui/advancedtooltip_test.js', ['goog.ui.AdvancedTooltipTest'], ['goog.dom', 'goog.events.Event', 'goog.events.EventType', 'goog.math.Box', 'goog.math.Coordinate', 'goog.style', 'goog.testing.MockClock', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.AdvancedTooltip', 'goog.ui.Tooltip', 'goog.userAgent'], false);
goog.addDependency('ui/animatedzippy.js', ['goog.ui.AnimatedZippy'], ['goog.dom', 'goog.events', 'goog.fx.Animation', 'goog.fx.Transition', 'goog.fx.easing', 'goog.ui.Zippy', 'goog.ui.ZippyEvent'], false);
goog.addDependency('ui/animatedzippy_test.js', ['goog.ui.AnimatedZippyTest'], ['goog.a11y.aria', 'goog.a11y.aria.State', 'goog.asserts', 'goog.dom', 'goog.events', 'goog.functions', 'goog.fx.Animation', 'goog.fx.Transition', 'goog.testing.PropertyReplacer', 'goog.testing.asserts', 'goog.testing.jsunit', 'goog.ui.AnimatedZippy', 'goog.ui.Zippy'], false);
goog.addDependency('ui/attachablemenu.js', ['goog.ui.AttachableMenu'], ['goog.a11y.aria', 'goog.a11y.aria.State', 'goog.array', 'goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.events.Event', 'goog.events.KeyCodes', 'goog.string', 'goog.style', 'goog.ui.ItemEvent', 'goog.ui.MenuBase', 'goog.ui.PopupBase', 'goog.userAgent'], false);
goog.addDependency('ui/bidiinput.js', ['goog.ui.BidiInput'], ['goog.dom', 'goog.events', 'goog.events.InputHandler', 'goog.i18n.bidi', 'goog.ui.Component'], false);
goog.addDependency('ui/bidiinput_test.js', ['goog.ui.BidiInputTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.ui.BidiInput'], false);
goog.addDependency('ui/bubble.js', ['goog.ui.Bubble'], ['goog.Timer', 'goog.events', 'goog.events.EventType', 'goog.math.Box', 'goog.positioning', 'goog.positioning.AbsolutePosition', 'goog.positioning.AnchoredPosition', 'goog.positioning.Corner', 'goog.positioning.CornerBit', 'goog.style', 'goog.ui.Component', 'goog.ui.Popup'], false);
goog.addDependency('ui/button.js', ['goog.ui.Button', 'goog.ui.Button.Side'], ['goog.events.EventType', 'goog.events.KeyCodes', 'goog.events.KeyHandler', 'goog.ui.ButtonRenderer', 'goog.ui.ButtonSide', 'goog.ui.Component', 'goog.ui.Control', 'goog.ui.NativeButtonRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/button_test.js', ['goog.ui.ButtonTest'], ['goog.dom', 'goog.dom.classlist', 'goog.events', 'goog.events.Event', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.events.KeyHandler', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.Button', 'goog.ui.ButtonRenderer', 'goog.ui.ButtonSide', 'goog.ui.Component', 'goog.ui.NativeButtonRenderer'], false);
goog.addDependency('ui/buttonrenderer.js', ['goog.ui.ButtonRenderer'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.asserts', 'goog.ui.ButtonSide', 'goog.ui.Component', 'goog.ui.ControlRenderer'], false);
goog.addDependency('ui/buttonrenderer_test.js', ['goog.ui.ButtonRendererTest'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.dom', 'goog.dom.classlist', 'goog.testing.ExpectedFailures', 'goog.testing.jsunit', 'goog.testing.ui.rendererasserts', 'goog.ui.Button', 'goog.ui.ButtonRenderer', 'goog.ui.ButtonSide', 'goog.ui.Component'], false);
goog.addDependency('ui/buttonside.js', ['goog.ui.ButtonSide'], [], false);
goog.addDependency('ui/charcounter.js', ['goog.ui.CharCounter', 'goog.ui.CharCounter.Display'], ['goog.dom', 'goog.events', 'goog.events.EventTarget', 'goog.events.InputHandler'], false);
goog.addDependency('ui/charcounter_test.js', ['goog.ui.CharCounterTest'], ['goog.dom', 'goog.testing.asserts', 'goog.testing.jsunit', 'goog.ui.CharCounter', 'goog.userAgent'], false);
goog.addDependency('ui/charpicker.js', ['goog.ui.CharPicker'], ['goog.a11y.aria', 'goog.a11y.aria.State', 'goog.array', 'goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.events', 'goog.events.Event', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.events.InputHandler', 'goog.events.KeyCodes', 'goog.events.KeyHandler', 'goog.i18n.CharListDecompressor', 'goog.i18n.uChar', 'goog.structs.Set', 'goog.style', 'goog.ui.Button', 'goog.ui.Component', 'goog.ui.ContainerScroller', 'goog.ui.FlatButtonRenderer', 'goog.ui.HoverCard', 'goog.ui.LabelInput', 'goog.ui.Menu', 'goog.ui.MenuButton', 'goog.ui.MenuItem', 'goog.ui.Tooltip'], false);
goog.addDependency('ui/charpicker_test.js', ['goog.ui.CharPickerTest'], ['goog.a11y.aria', 'goog.a11y.aria.State', 'goog.dispose', 'goog.dom', 'goog.events.Event', 'goog.events.EventType', 'goog.i18n.CharPickerData', 'goog.i18n.uChar.NameFetcher', 'goog.testing.MockControl', 'goog.testing.events', 'goog.testing.jsunit', 'goog.testing.mockmatchers', 'goog.ui.CharPicker', 'goog.ui.FlatButtonRenderer'], false);
goog.addDependency('ui/checkbox.js', ['goog.ui.Checkbox', 'goog.ui.Checkbox.State'], ['goog.a11y.aria', 'goog.a11y.aria.State', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.string', 'goog.ui.CheckboxRenderer', 'goog.ui.Component', 'goog.ui.Control', 'goog.ui.registry'], false);
goog.addDependency('ui/checkbox_test.js', ['goog.ui.CheckboxTest'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.dom', 'goog.dom.classlist', 'goog.events', 'goog.events.KeyCodes', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.Checkbox', 'goog.ui.CheckboxRenderer', 'goog.ui.Component', 'goog.ui.ControlRenderer', 'goog.ui.decorate'], false);
goog.addDependency('ui/checkboxmenuitem.js', ['goog.ui.CheckBoxMenuItem'], ['goog.ui.MenuItem', 'goog.ui.registry'], false);
goog.addDependency('ui/checkboxrenderer.js', ['goog.ui.CheckboxRenderer'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.array', 'goog.asserts', 'goog.dom.classlist', 'goog.object', 'goog.ui.ControlRenderer'], false);
goog.addDependency('ui/colorbutton.js', ['goog.ui.ColorButton'], ['goog.ui.Button', 'goog.ui.ColorButtonRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/colorbutton_test.js', ['goog.ui.ColorButtonTest'], ['goog.array', 'goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.testing.jsunit', 'goog.ui.ColorButton', 'goog.ui.decorate'], false);
goog.addDependency('ui/colorbuttonrenderer.js', ['goog.ui.ColorButtonRenderer'], ['goog.asserts', 'goog.dom.classlist', 'goog.functions', 'goog.ui.ColorMenuButtonRenderer'], false);
goog.addDependency('ui/colormenubutton.js', ['goog.ui.ColorMenuButton'], ['goog.array', 'goog.object', 'goog.ui.ColorMenuButtonRenderer', 'goog.ui.ColorPalette', 'goog.ui.Component', 'goog.ui.Menu', 'goog.ui.MenuButton', 'goog.ui.registry'], false);
goog.addDependency('ui/colormenubuttonrenderer.js', ['goog.ui.ColorMenuButtonRenderer'], ['goog.asserts', 'goog.color', 'goog.dom.classlist', 'goog.ui.MenuButtonRenderer', 'goog.userAgent'], false);
goog.addDependency('ui/colormenubuttonrenderer_test.js', ['goog.ui.ColorMenuButtonTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.testing.ui.RendererHarness', 'goog.testing.ui.rendererasserts', 'goog.ui.ColorMenuButton', 'goog.ui.ColorMenuButtonRenderer', 'goog.userAgent'], false);
goog.addDependency('ui/colorpalette.js', ['goog.ui.ColorPalette'], ['goog.array', 'goog.color', 'goog.style', 'goog.ui.Palette', 'goog.ui.PaletteRenderer'], false);
goog.addDependency('ui/colorpalette_test.js', ['goog.ui.ColorPaletteTest'], ['goog.color', 'goog.testing.jsunit', 'goog.ui.ColorPalette'], false);
goog.addDependency('ui/colorpicker.js', ['goog.ui.ColorPicker', 'goog.ui.ColorPicker.EventType'], ['goog.ui.ColorPalette', 'goog.ui.Component'], false);
goog.addDependency('ui/colorsplitbehavior.js', ['goog.ui.ColorSplitBehavior'], ['goog.ui.ColorMenuButton', 'goog.ui.SplitBehavior'], false);
goog.addDependency('ui/combobox.js', ['goog.ui.ComboBox', 'goog.ui.ComboBoxItem'], ['goog.Timer', 'goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.events.EventType', 'goog.events.InputHandler', 'goog.events.KeyCodes', 'goog.events.KeyHandler', 'goog.log', 'goog.positioning.Corner', 'goog.positioning.MenuAnchoredPosition', 'goog.string', 'goog.style', 'goog.ui.Component', 'goog.ui.ItemEvent', 'goog.ui.LabelInput', 'goog.ui.Menu', 'goog.ui.MenuItem', 'goog.ui.MenuSeparator', 'goog.ui.registry', 'goog.userAgent'], false);
goog.addDependency('ui/combobox_test.js', ['goog.ui.ComboBoxTest'], ['goog.dom', 'goog.dom.TagName', 'goog.dom.classlist', 'goog.events.KeyCodes', 'goog.testing.MockClock', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.ComboBox', 'goog.ui.ComboBoxItem', 'goog.ui.Component', 'goog.ui.ControlRenderer', 'goog.ui.LabelInput', 'goog.ui.Menu', 'goog.ui.MenuItem'], false);
goog.addDependency('ui/component.js', ['goog.ui.Component', 'goog.ui.Component.Error', 'goog.ui.Component.EventType', 'goog.ui.Component.State'], ['goog.array', 'goog.asserts', 'goog.dom', 'goog.dom.NodeType', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.object', 'goog.style', 'goog.ui.IdGenerator'], false);
goog.addDependency('ui/component_test.js', ['goog.ui.ComponentTest'], ['goog.dom', 'goog.dom.DomHelper', 'goog.dom.NodeType', 'goog.dom.TagName', 'goog.events.EventTarget', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.ui.Component'], false);
goog.addDependency('ui/container.js', ['goog.ui.Container', 'goog.ui.Container.EventType', 'goog.ui.Container.Orientation'], ['goog.a11y.aria', 'goog.a11y.aria.State', 'goog.asserts', 'goog.dom', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.events.KeyHandler', 'goog.object', 'goog.style', 'goog.ui.Component', 'goog.ui.ContainerRenderer', 'goog.ui.Control'], false);
goog.addDependency('ui/container_test.js', ['goog.ui.ContainerTest'], ['goog.a11y.aria', 'goog.dom', 'goog.dom.classlist', 'goog.events', 'goog.events.Event', 'goog.events.KeyCodes', 'goog.events.KeyEvent', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.Component', 'goog.ui.Container', 'goog.ui.Control'], false);
goog.addDependency('ui/containerrenderer.js', ['goog.ui.ContainerRenderer'], ['goog.a11y.aria', 'goog.array', 'goog.asserts', 'goog.dom.NodeType', 'goog.dom.classlist', 'goog.string', 'goog.style', 'goog.ui.registry', 'goog.userAgent'], false);
goog.addDependency('ui/containerrenderer_test.js', ['goog.ui.ContainerRendererTest'], ['goog.dom', 'goog.style', 'goog.testing.ExpectedFailures', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.testing.ui.rendererasserts', 'goog.ui.Container', 'goog.ui.ContainerRenderer', 'goog.userAgent'], false);
goog.addDependency('ui/containerscroller.js', ['goog.ui.ContainerScroller'], ['goog.Disposable', 'goog.Timer', 'goog.events.EventHandler', 'goog.style', 'goog.ui.Component', 'goog.ui.Container'], false);
goog.addDependency('ui/containerscroller_test.js', ['goog.ui.ContainerScrollerTest'], ['goog.dom', 'goog.testing.MockClock', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.Container', 'goog.ui.ContainerScroller'], false);
goog.addDependency('ui/control.js', ['goog.ui.Control'], ['goog.array', 'goog.dom', 'goog.events.Event', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.events.KeyHandler', 'goog.string', 'goog.ui.Component', 'goog.ui.ControlContent', 'goog.ui.ControlRenderer', 'goog.ui.decorate', 'goog.ui.registry', 'goog.userAgent'], false);
goog.addDependency('ui/control_test.js', ['goog.ui.ControlTest'], ['goog.a11y.aria', 'goog.a11y.aria.State', 'goog.array', 'goog.dom', 'goog.dom.classlist', 'goog.events', 'goog.events.BrowserEvent', 'goog.events.KeyCodes', 'goog.object', 'goog.string', 'goog.style', 'goog.testing.ExpectedFailures', 'goog.testing.events', 'goog.testing.events.Event', 'goog.testing.jsunit', 'goog.ui.Component', 'goog.ui.Control', 'goog.ui.ControlRenderer', 'goog.ui.registry', 'goog.userAgent'], false);
goog.addDependency('ui/controlcontent.js', ['goog.ui.ControlContent'], [], false);
goog.addDependency('ui/controlrenderer.js', ['goog.ui.ControlRenderer'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.array', 'goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.object', 'goog.string', 'goog.style', 'goog.ui.Component', 'goog.userAgent'], false);
goog.addDependency('ui/controlrenderer_test.js', ['goog.ui.ControlRendererTest'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.classlist', 'goog.object', 'goog.style', 'goog.testing.ExpectedFailures', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.ui.Component', 'goog.ui.Control', 'goog.ui.ControlRenderer', 'goog.userAgent'], false);
goog.addDependency('ui/cookieeditor.js', ['goog.ui.CookieEditor'], ['goog.asserts', 'goog.dom', 'goog.dom.TagName', 'goog.events.EventType', 'goog.net.cookies', 'goog.string', 'goog.style', 'goog.ui.Component'], false);
goog.addDependency('ui/cookieeditor_test.js', ['goog.ui.CookieEditorTest'], ['goog.dom', 'goog.events.Event', 'goog.events.EventType', 'goog.net.cookies', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.CookieEditor'], false);
goog.addDependency('ui/css3buttonrenderer.js', ['goog.ui.Css3ButtonRenderer'], ['goog.asserts', 'goog.dom.TagName', 'goog.dom.classlist', 'goog.ui.Button', 'goog.ui.ButtonRenderer', 'goog.ui.Component', 'goog.ui.INLINE_BLOCK_CLASSNAME', 'goog.ui.registry'], false);
goog.addDependency('ui/css3menubuttonrenderer.js', ['goog.ui.Css3MenuButtonRenderer'], ['goog.dom', 'goog.dom.TagName', 'goog.ui.INLINE_BLOCK_CLASSNAME', 'goog.ui.MenuButton', 'goog.ui.MenuButtonRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/cssnames.js', ['goog.ui.INLINE_BLOCK_CLASSNAME'], [], false);
goog.addDependency('ui/custombutton.js', ['goog.ui.CustomButton'], ['goog.ui.Button', 'goog.ui.CustomButtonRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/custombuttonrenderer.js', ['goog.ui.CustomButtonRenderer'], ['goog.a11y.aria.Role', 'goog.asserts', 'goog.dom.NodeType', 'goog.dom.classlist', 'goog.string', 'goog.ui.ButtonRenderer', 'goog.ui.INLINE_BLOCK_CLASSNAME'], false);
goog.addDependency('ui/customcolorpalette.js', ['goog.ui.CustomColorPalette'], ['goog.color', 'goog.dom', 'goog.dom.classlist', 'goog.ui.ColorPalette', 'goog.ui.Component'], false);
goog.addDependency('ui/customcolorpalette_test.js', ['goog.ui.CustomColorPaletteTest'], ['goog.dom.TagName', 'goog.dom.classlist', 'goog.testing.jsunit', 'goog.ui.CustomColorPalette'], false);
goog.addDependency('ui/datepicker.js', ['goog.ui.DatePicker', 'goog.ui.DatePicker.Events', 'goog.ui.DatePickerEvent'], ['goog.a11y.aria', 'goog.asserts', 'goog.date.Date', 'goog.date.DateRange', 'goog.date.Interval', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.classlist', 'goog.events.Event', 'goog.events.EventType', 'goog.events.KeyHandler', 'goog.i18n.DateTimeFormat', 'goog.i18n.DateTimePatterns', 'goog.i18n.DateTimeSymbols', 'goog.style', 'goog.ui.Component', 'goog.ui.DefaultDatePickerRenderer', 'goog.ui.IdGenerator'], false);
goog.addDependency('ui/datepicker_test.js', ['goog.ui.DatePickerTest'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.date.Date', 'goog.date.DateRange', 'goog.dom', 'goog.dom.classlist', 'goog.events', 'goog.i18n.DateTimeSymbols', 'goog.i18n.DateTimeSymbols_en_US', 'goog.i18n.DateTimeSymbols_zh_HK', 'goog.style', 'goog.testing.jsunit', 'goog.ui.DatePicker'], false);
goog.addDependency('ui/datepickerrenderer.js', ['goog.ui.DatePickerRenderer'], [], false);
goog.addDependency('ui/decorate.js', ['goog.ui.decorate'], ['goog.ui.registry'], false);
goog.addDependency('ui/decorate_test.js', ['goog.ui.decorateTest'], ['goog.testing.jsunit', 'goog.ui.decorate', 'goog.ui.registry'], false);
goog.addDependency('ui/defaultdatepickerrenderer.js', ['goog.ui.DefaultDatePickerRenderer'], ['goog.dom', 'goog.dom.TagName', 'goog.ui.DatePickerRenderer'], false);
goog.addDependency('ui/dialog.js', ['goog.ui.Dialog', 'goog.ui.Dialog.ButtonSet', 'goog.ui.Dialog.ButtonSet.DefaultButtons', 'goog.ui.Dialog.DefaultButtonCaptions', 'goog.ui.Dialog.DefaultButtonKeys', 'goog.ui.Dialog.Event', 'goog.ui.Dialog.EventType'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.asserts', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.TagName', 'goog.dom.classlist', 'goog.dom.safe', 'goog.events', 'goog.events.Event', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.fx.Dragger', 'goog.html.SafeHtml', 'goog.html.legacyconversions', 'goog.math.Rect', 'goog.string', 'goog.structs.Map', 'goog.style', 'goog.ui.ModalPopup'], false);
goog.addDependency('ui/dialog_test.js', ['goog.ui.DialogTest'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.dom', 'goog.dom.TagName', 'goog.dom.classlist', 'goog.events', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.fx.css3', 'goog.html.SafeHtml', 'goog.html.testing', 'goog.style', 'goog.testing.MockClock', 'goog.testing.PropertyReplacer', 'goog.testing.events', 'goog.testing.jsunit', 'goog.testing.recordFunction', 'goog.ui.Dialog', 'goog.userAgent'], false);
goog.addDependency('ui/dimensionpicker.js', ['goog.ui.DimensionPicker'], ['goog.events.EventType', 'goog.events.KeyCodes', 'goog.math.Size', 'goog.ui.Component', 'goog.ui.Control', 'goog.ui.DimensionPickerRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/dimensionpicker_test.js', ['goog.ui.DimensionPickerTest'], ['goog.dom', 'goog.dom.TagName', 'goog.events.KeyCodes', 'goog.math.Size', 'goog.testing.jsunit', 'goog.testing.ui.rendererasserts', 'goog.ui.DimensionPicker', 'goog.ui.DimensionPickerRenderer'], false);
goog.addDependency('ui/dimensionpickerrenderer.js', ['goog.ui.DimensionPickerRenderer'], ['goog.a11y.aria.Announcer', 'goog.a11y.aria.LivePriority', 'goog.dom', 'goog.dom.TagName', 'goog.i18n.bidi', 'goog.style', 'goog.ui.ControlRenderer', 'goog.userAgent'], false);
goog.addDependency('ui/dimensionpickerrenderer_test.js', ['goog.ui.DimensionPickerRendererTest'], ['goog.a11y.aria.LivePriority', 'goog.array', 'goog.testing.jsunit', 'goog.testing.recordFunction', 'goog.ui.DimensionPicker', 'goog.ui.DimensionPickerRenderer'], false);
goog.addDependency('ui/dragdropdetector.js', ['goog.ui.DragDropDetector', 'goog.ui.DragDropDetector.EventType', 'goog.ui.DragDropDetector.ImageDropEvent', 'goog.ui.DragDropDetector.LinkDropEvent'], ['goog.dom', 'goog.dom.TagName', 'goog.events.Event', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.math.Coordinate', 'goog.string', 'goog.style', 'goog.userAgent'], false);
goog.addDependency('ui/drilldownrow.js', ['goog.ui.DrilldownRow'], ['goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.ui.Component'], false);
goog.addDependency('ui/drilldownrow_test.js', ['goog.ui.DrilldownRowTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.ui.DrilldownRow'], false);
goog.addDependency('ui/editor/abstractdialog.js', ['goog.ui.editor.AbstractDialog', 'goog.ui.editor.AbstractDialog.Builder', 'goog.ui.editor.AbstractDialog.EventType'], ['goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.events.EventTarget', 'goog.string', 'goog.ui.Dialog', 'goog.ui.PopupBase'], false);
goog.addDependency('ui/editor/abstractdialog_test.js', ['goog.ui.editor.AbstractDialogTest'], ['goog.dom', 'goog.dom.DomHelper', 'goog.dom.classlist', 'goog.events.Event', 'goog.events.EventHandler', 'goog.events.KeyCodes', 'goog.testing.MockControl', 'goog.testing.events', 'goog.testing.jsunit', 'goog.testing.mockmatchers.ArgumentMatcher', 'goog.ui.editor.AbstractDialog', 'goog.userAgent'], false);
goog.addDependency('ui/editor/bubble.js', ['goog.ui.editor.Bubble'], ['goog.asserts', 'goog.dom', 'goog.dom.TagName', 'goog.dom.ViewportSizeMonitor', 'goog.dom.classlist', 'goog.editor.style', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.functions', 'goog.log', 'goog.math.Box', 'goog.object', 'goog.positioning', 'goog.positioning.Corner', 'goog.positioning.Overflow', 'goog.positioning.OverflowStatus', 'goog.string', 'goog.style', 'goog.ui.Component', 'goog.ui.PopupBase', 'goog.userAgent'], false);
goog.addDependency('ui/editor/bubble_test.js', ['goog.ui.editor.BubbleTest'], ['goog.dom', 'goog.dom.TagName', 'goog.events', 'goog.events.EventType', 'goog.positioning.Corner', 'goog.positioning.OverflowStatus', 'goog.string', 'goog.style', 'goog.testing.editor.TestHelper', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.Component', 'goog.ui.editor.Bubble'], false);
goog.addDependency('ui/editor/defaulttoolbar.js', ['goog.ui.editor.ButtonDescriptor', 'goog.ui.editor.DefaultToolbar'], ['goog.asserts', 'goog.dom', 'goog.dom.TagName', 'goog.dom.classlist', 'goog.editor.Command', 'goog.style', 'goog.ui.editor.ToolbarFactory', 'goog.ui.editor.messages', 'goog.userAgent'], false);
goog.addDependency('ui/editor/linkdialog.js', ['goog.ui.editor.LinkDialog', 'goog.ui.editor.LinkDialog.BeforeTestLinkEvent', 'goog.ui.editor.LinkDialog.EventType', 'goog.ui.editor.LinkDialog.OkEvent'], ['goog.dom', 'goog.dom.TagName', 'goog.dom.safe', 'goog.editor.BrowserFeature', 'goog.editor.Link', 'goog.editor.focus', 'goog.editor.node', 'goog.events.Event', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.events.InputHandler', 'goog.html.SafeHtml', 'goog.string', 'goog.string.Unicode', 'goog.style', 'goog.ui.Button', 'goog.ui.Component', 'goog.ui.LinkButtonRenderer', 'goog.ui.editor.AbstractDialog', 'goog.ui.editor.TabPane', 'goog.ui.editor.messages', 'goog.userAgent', 'goog.window'], false);
goog.addDependency('ui/editor/linkdialog_test.js', ['goog.ui.editor.LinkDialogTest'], ['goog.dom', 'goog.dom.DomHelper', 'goog.dom.TagName', 'goog.editor.BrowserFeature', 'goog.editor.Link', 'goog.events', 'goog.events.Event', 'goog.events.EventHandler', 'goog.events.InputHandler', 'goog.style', 'goog.testing.MockControl', 'goog.testing.PropertyReplacer', 'goog.testing.dom', 'goog.testing.events', 'goog.testing.jsunit', 'goog.testing.mockmatchers', 'goog.testing.mockmatchers.ArgumentMatcher', 'goog.ui.editor.AbstractDialog', 'goog.ui.editor.LinkDialog', 'goog.ui.editor.messages', 'goog.userAgent'], false);
goog.addDependency('ui/editor/messages.js', ['goog.ui.editor.messages'], ['goog.html.uncheckedconversions', 'goog.string.Const'], false);
goog.addDependency('ui/editor/tabpane.js', ['goog.ui.editor.TabPane'], ['goog.asserts', 'goog.dom.TagName', 'goog.dom.classlist', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.style', 'goog.ui.Component', 'goog.ui.Control', 'goog.ui.Tab', 'goog.ui.TabBar'], false);
goog.addDependency('ui/editor/toolbarcontroller.js', ['goog.ui.editor.ToolbarController'], ['goog.editor.Field', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.ui.Component'], false);
goog.addDependency('ui/editor/toolbarfactory.js', ['goog.ui.editor.ToolbarFactory'], ['goog.array', 'goog.dom', 'goog.dom.TagName', 'goog.string', 'goog.string.Unicode', 'goog.style', 'goog.ui.Component', 'goog.ui.Container', 'goog.ui.Option', 'goog.ui.Toolbar', 'goog.ui.ToolbarButton', 'goog.ui.ToolbarColorMenuButton', 'goog.ui.ToolbarMenuButton', 'goog.ui.ToolbarRenderer', 'goog.ui.ToolbarSelect', 'goog.userAgent'], false);
goog.addDependency('ui/editor/toolbarfactory_test.js', ['goog.ui.editor.ToolbarFactoryTest'], ['goog.dom', 'goog.testing.ExpectedFailures', 'goog.testing.editor.TestHelper', 'goog.testing.jsunit', 'goog.ui.editor.ToolbarFactory', 'goog.userAgent'], false);
goog.addDependency('ui/emoji/emoji.js', ['goog.ui.emoji.Emoji'], [], false);
goog.addDependency('ui/emoji/emojipalette.js', ['goog.ui.emoji.EmojiPalette'], ['goog.events.EventType', 'goog.net.ImageLoader', 'goog.ui.Palette', 'goog.ui.emoji.Emoji', 'goog.ui.emoji.EmojiPaletteRenderer'], false);
goog.addDependency('ui/emoji/emojipaletterenderer.js', ['goog.ui.emoji.EmojiPaletteRenderer'], ['goog.a11y.aria', 'goog.asserts', 'goog.dom.NodeType', 'goog.dom.classlist', 'goog.style', 'goog.ui.PaletteRenderer', 'goog.ui.emoji.Emoji'], false);
goog.addDependency('ui/emoji/emojipicker.js', ['goog.ui.emoji.EmojiPicker'], ['goog.log', 'goog.style', 'goog.ui.Component', 'goog.ui.TabPane', 'goog.ui.emoji.Emoji', 'goog.ui.emoji.EmojiPalette', 'goog.ui.emoji.EmojiPaletteRenderer', 'goog.ui.emoji.ProgressiveEmojiPaletteRenderer'], false);
goog.addDependency('ui/emoji/emojipicker_test.js', ['goog.ui.emoji.EmojiPickerTest'], ['goog.dom.classlist', 'goog.events.EventHandler', 'goog.style', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.Component', 'goog.ui.emoji.Emoji', 'goog.ui.emoji.EmojiPicker', 'goog.ui.emoji.SpriteInfo'], false);
goog.addDependency('ui/emoji/fast_nonprogressive_emojipicker_test.js', ['goog.ui.emoji.FastNonProgressiveEmojiPickerTest'], ['goog.dom.classlist', 'goog.events', 'goog.events.EventType', 'goog.net.EventType', 'goog.style', 'goog.testing.AsyncTestCase', 'goog.testing.jsunit', 'goog.ui.emoji.Emoji', 'goog.ui.emoji.EmojiPicker', 'goog.ui.emoji.SpriteInfo', 'goog.userAgent'], false);
goog.addDependency('ui/emoji/fast_progressive_emojipicker_test.js', ['goog.ui.emoji.FastProgressiveEmojiPickerTest'], ['goog.dom.classlist', 'goog.events', 'goog.events.EventType', 'goog.net.EventType', 'goog.style', 'goog.testing.AsyncTestCase', 'goog.testing.jsunit', 'goog.ui.emoji.Emoji', 'goog.ui.emoji.EmojiPicker', 'goog.ui.emoji.SpriteInfo'], false);
goog.addDependency('ui/emoji/popupemojipicker.js', ['goog.ui.emoji.PopupEmojiPicker'], ['goog.events.EventType', 'goog.positioning.AnchoredPosition', 'goog.positioning.Corner', 'goog.ui.Component', 'goog.ui.Popup', 'goog.ui.emoji.EmojiPicker'], false);
goog.addDependency('ui/emoji/popupemojipicker_test.js', ['goog.ui.emoji.PopupEmojiPickerTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.ui.emoji.PopupEmojiPicker'], false);
goog.addDependency('ui/emoji/progressiveemojipaletterenderer.js', ['goog.ui.emoji.ProgressiveEmojiPaletteRenderer'], ['goog.style', 'goog.ui.emoji.EmojiPaletteRenderer'], false);
goog.addDependency('ui/emoji/spriteinfo.js', ['goog.ui.emoji.SpriteInfo'], [], false);
goog.addDependency('ui/emoji/spriteinfo_test.js', ['goog.ui.emoji.SpriteInfoTest'], ['goog.testing.jsunit', 'goog.ui.emoji.SpriteInfo'], false);
goog.addDependency('ui/filteredmenu.js', ['goog.ui.FilteredMenu'], ['goog.a11y.aria', 'goog.a11y.aria.AutoCompleteValues', 'goog.a11y.aria.State', 'goog.dom', 'goog.events', 'goog.events.EventType', 'goog.events.InputHandler', 'goog.events.KeyCodes', 'goog.object', 'goog.string', 'goog.style', 'goog.ui.Component', 'goog.ui.FilterObservingMenuItem', 'goog.ui.Menu', 'goog.ui.MenuItem', 'goog.userAgent'], false);
goog.addDependency('ui/filteredmenu_test.js', ['goog.ui.FilteredMenuTest'], ['goog.a11y.aria', 'goog.a11y.aria.AutoCompleteValues', 'goog.a11y.aria.State', 'goog.dom', 'goog.events', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.math.Rect', 'goog.style', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.FilteredMenu', 'goog.ui.MenuItem'], false);
goog.addDependency('ui/filterobservingmenuitem.js', ['goog.ui.FilterObservingMenuItem'], ['goog.ui.FilterObservingMenuItemRenderer', 'goog.ui.MenuItem', 'goog.ui.registry'], false);
goog.addDependency('ui/filterobservingmenuitemrenderer.js', ['goog.ui.FilterObservingMenuItemRenderer'], ['goog.ui.MenuItemRenderer'], false);
goog.addDependency('ui/flatbuttonrenderer.js', ['goog.ui.FlatButtonRenderer'], ['goog.a11y.aria.Role', 'goog.asserts', 'goog.dom.classlist', 'goog.ui.Button', 'goog.ui.ButtonRenderer', 'goog.ui.INLINE_BLOCK_CLASSNAME', 'goog.ui.registry'], false);
goog.addDependency('ui/flatmenubuttonrenderer.js', ['goog.ui.FlatMenuButtonRenderer'], ['goog.dom', 'goog.style', 'goog.ui.FlatButtonRenderer', 'goog.ui.INLINE_BLOCK_CLASSNAME', 'goog.ui.Menu', 'goog.ui.MenuButton', 'goog.ui.MenuRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/formpost.js', ['goog.ui.FormPost'], ['goog.array', 'goog.dom.TagName', 'goog.dom.safe', 'goog.html.SafeHtml', 'goog.ui.Component'], false);
goog.addDependency('ui/formpost_test.js', ['goog.ui.FormPostTest'], ['goog.array', 'goog.dom', 'goog.dom.TagName', 'goog.object', 'goog.testing.jsunit', 'goog.ui.FormPost', 'goog.userAgent.product', 'goog.userAgent.product.isVersion'], false);
goog.addDependency('ui/gauge.js', ['goog.ui.Gauge', 'goog.ui.GaugeColoredRange'], ['goog.a11y.aria', 'goog.asserts', 'goog.events', 'goog.fx.Animation', 'goog.fx.Transition', 'goog.fx.easing', 'goog.graphics', 'goog.graphics.Font', 'goog.graphics.Path', 'goog.graphics.SolidFill', 'goog.math', 'goog.ui.Component', 'goog.ui.GaugeTheme'], false);
goog.addDependency('ui/gaugetheme.js', ['goog.ui.GaugeTheme'], ['goog.graphics.LinearGradient', 'goog.graphics.SolidFill', 'goog.graphics.Stroke'], false);
goog.addDependency('ui/hovercard.js', ['goog.ui.HoverCard', 'goog.ui.HoverCard.EventType', 'goog.ui.HoverCard.TriggerEvent'], ['goog.array', 'goog.dom', 'goog.events', 'goog.events.Event', 'goog.events.EventType', 'goog.ui.AdvancedTooltip', 'goog.ui.PopupBase', 'goog.ui.Tooltip'], false);
goog.addDependency('ui/hovercard_test.js', ['goog.ui.HoverCardTest'], ['goog.dom', 'goog.events', 'goog.math.Coordinate', 'goog.style', 'goog.testing.MockClock', 'goog.testing.events', 'goog.testing.events.Event', 'goog.testing.jsunit', 'goog.ui.HoverCard'], false);
goog.addDependency('ui/hsvapalette.js', ['goog.ui.HsvaPalette'], ['goog.array', 'goog.color.alpha', 'goog.dom.TagName', 'goog.events', 'goog.events.EventType', 'goog.style', 'goog.ui.Component', 'goog.ui.HsvPalette'], false);
goog.addDependency('ui/hsvapalette_test.js', ['goog.ui.HsvaPaletteTest'], ['goog.color.alpha', 'goog.dom.TagName', 'goog.dom.classlist', 'goog.events', 'goog.events.Event', 'goog.math.Coordinate', 'goog.style', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.ui.HsvaPalette', 'goog.userAgent'], false);
goog.addDependency('ui/hsvpalette.js', ['goog.ui.HsvPalette'], ['goog.color', 'goog.dom.TagName', 'goog.events', 'goog.events.EventType', 'goog.events.InputHandler', 'goog.style', 'goog.style.bidi', 'goog.ui.Component', 'goog.userAgent'], false);
goog.addDependency('ui/hsvpalette_test.js', ['goog.ui.HsvPaletteTest'], ['goog.color', 'goog.dom.TagName', 'goog.dom.classlist', 'goog.events', 'goog.events.Event', 'goog.math.Coordinate', 'goog.style', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.ui.Component', 'goog.ui.HsvPalette', 'goog.userAgent'], false);
goog.addDependency('ui/idgenerator.js', ['goog.ui.IdGenerator'], [], false);
goog.addDependency('ui/idletimer.js', ['goog.ui.IdleTimer'], ['goog.Timer', 'goog.events', 'goog.events.EventTarget', 'goog.structs.Set', 'goog.ui.ActivityMonitor'], false);
goog.addDependency('ui/idletimer_test.js', ['goog.ui.IdleTimerTest'], ['goog.events', 'goog.testing.MockClock', 'goog.testing.jsunit', 'goog.ui.IdleTimer', 'goog.ui.MockActivityMonitor'], false);
goog.addDependency('ui/iframemask.js', ['goog.ui.IframeMask'], ['goog.Disposable', 'goog.Timer', 'goog.dom', 'goog.dom.iframe', 'goog.events.EventHandler', 'goog.style'], false);
goog.addDependency('ui/iframemask_test.js', ['goog.ui.IframeMaskTest'], ['goog.dom', 'goog.dom.TagName', 'goog.dom.iframe', 'goog.structs.Pool', 'goog.style', 'goog.testing.MockClock', 'goog.testing.StrictMock', 'goog.testing.jsunit', 'goog.ui.IframeMask', 'goog.ui.Popup', 'goog.ui.PopupBase', 'goog.userAgent'], false);
goog.addDependency('ui/imagelessbuttonrenderer.js', ['goog.ui.ImagelessButtonRenderer'], ['goog.dom.classlist', 'goog.ui.Button', 'goog.ui.Component', 'goog.ui.CustomButtonRenderer', 'goog.ui.INLINE_BLOCK_CLASSNAME', 'goog.ui.registry'], false);
goog.addDependency('ui/imagelessmenubuttonrenderer.js', ['goog.ui.ImagelessMenuButtonRenderer'], ['goog.dom', 'goog.dom.TagName', 'goog.dom.classlist', 'goog.ui.INLINE_BLOCK_CLASSNAME', 'goog.ui.MenuButton', 'goog.ui.MenuButtonRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/inputdatepicker.js', ['goog.ui.InputDatePicker'], ['goog.date.DateTime', 'goog.dom', 'goog.string', 'goog.ui.Component', 'goog.ui.DatePicker', 'goog.ui.PopupBase', 'goog.ui.PopupDatePicker'], false);
goog.addDependency('ui/inputdatepicker_test.js', ['goog.ui.InputDatePickerTest'], ['goog.dom', 'goog.i18n.DateTimeFormat', 'goog.i18n.DateTimeParse', 'goog.testing.jsunit', 'goog.ui.InputDatePicker'], false);
goog.addDependency('ui/itemevent.js', ['goog.ui.ItemEvent'], ['goog.events.Event'], false);
goog.addDependency('ui/keyboardshortcuthandler.js', ['goog.ui.KeyboardShortcutEvent', 'goog.ui.KeyboardShortcutHandler', 'goog.ui.KeyboardShortcutHandler.EventType'], ['goog.Timer', 'goog.events', 'goog.events.Event', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.events.KeyNames', 'goog.object', 'goog.userAgent'], false);
goog.addDependency('ui/keyboardshortcuthandler_test.js', ['goog.ui.KeyboardShortcutHandlerTest'], ['goog.dom', 'goog.events', 'goog.events.BrowserEvent', 'goog.events.KeyCodes', 'goog.testing.MockClock', 'goog.testing.PropertyReplacer', 'goog.testing.StrictMock', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.KeyboardShortcutHandler', 'goog.userAgent'], false);
goog.addDependency('ui/labelinput.js', ['goog.ui.LabelInput'], ['goog.Timer', 'goog.a11y.aria', 'goog.a11y.aria.State', 'goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.ui.Component', 'goog.userAgent'], false);
goog.addDependency('ui/labelinput_test.js', ['goog.ui.LabelInputTest'], ['goog.a11y.aria', 'goog.a11y.aria.State', 'goog.dom', 'goog.dom.classlist', 'goog.events.EventType', 'goog.testing.MockClock', 'goog.testing.events', 'goog.testing.events.Event', 'goog.testing.jsunit', 'goog.ui.LabelInput', 'goog.userAgent'], false);
goog.addDependency('ui/linkbuttonrenderer.js', ['goog.ui.LinkButtonRenderer'], ['goog.ui.Button', 'goog.ui.FlatButtonRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/media/flashobject.js', ['goog.ui.media.FlashObject', 'goog.ui.media.FlashObject.ScriptAccessLevel', 'goog.ui.media.FlashObject.Wmodes'], ['goog.asserts', 'goog.events.Event', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.log', 'goog.object', 'goog.string', 'goog.structs.Map', 'goog.style', 'goog.ui.Component', 'goog.userAgent', 'goog.userAgent.flash'], false);
goog.addDependency('ui/media/flashobject_test.js', ['goog.ui.media.FlashObjectTest'], ['goog.dom', 'goog.dom.DomHelper', 'goog.events', 'goog.events.Event', 'goog.events.EventType', 'goog.testing.MockControl', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.media.FlashObject'], false);
goog.addDependency('ui/media/flickr.js', ['goog.ui.media.FlickrSet', 'goog.ui.media.FlickrSetModel'], ['goog.ui.media.FlashObject', 'goog.ui.media.Media', 'goog.ui.media.MediaModel', 'goog.ui.media.MediaRenderer'], false);
goog.addDependency('ui/media/flickr_test.js', ['goog.ui.media.FlickrSetTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.ui.media.FlashObject', 'goog.ui.media.FlickrSet', 'goog.ui.media.FlickrSetModel', 'goog.ui.media.Media'], false);
goog.addDependency('ui/media/googlevideo.js', ['goog.ui.media.GoogleVideo', 'goog.ui.media.GoogleVideoModel'], ['goog.string', 'goog.ui.media.FlashObject', 'goog.ui.media.Media', 'goog.ui.media.MediaModel', 'goog.ui.media.MediaRenderer'], false);
goog.addDependency('ui/media/googlevideo_test.js', ['goog.ui.media.GoogleVideoTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.ui.media.FlashObject', 'goog.ui.media.GoogleVideo', 'goog.ui.media.GoogleVideoModel', 'goog.ui.media.Media'], false);
goog.addDependency('ui/media/media.js', ['goog.ui.media.Media', 'goog.ui.media.MediaRenderer'], ['goog.asserts', 'goog.style', 'goog.ui.Component', 'goog.ui.Control', 'goog.ui.ControlRenderer'], false);
goog.addDependency('ui/media/media_test.js', ['goog.ui.media.MediaTest'], ['goog.dom', 'goog.math.Size', 'goog.testing.jsunit', 'goog.ui.ControlRenderer', 'goog.ui.media.Media', 'goog.ui.media.MediaModel', 'goog.ui.media.MediaRenderer'], false);
goog.addDependency('ui/media/mediamodel.js', ['goog.ui.media.MediaModel', 'goog.ui.media.MediaModel.Category', 'goog.ui.media.MediaModel.Credit', 'goog.ui.media.MediaModel.Credit.Role', 'goog.ui.media.MediaModel.Credit.Scheme', 'goog.ui.media.MediaModel.Medium', 'goog.ui.media.MediaModel.MimeType', 'goog.ui.media.MediaModel.Player', 'goog.ui.media.MediaModel.SubTitle', 'goog.ui.media.MediaModel.Thumbnail'], ['goog.array'], false);
goog.addDependency('ui/media/mediamodel_test.js', ['goog.ui.media.MediaModelTest'], ['goog.testing.jsunit', 'goog.ui.media.MediaModel'], false);
goog.addDependency('ui/media/mp3.js', ['goog.ui.media.Mp3'], ['goog.string', 'goog.ui.media.FlashObject', 'goog.ui.media.Media', 'goog.ui.media.MediaRenderer'], false);
goog.addDependency('ui/media/mp3_test.js', ['goog.ui.media.Mp3Test'], ['goog.dom', 'goog.testing.jsunit', 'goog.ui.media.FlashObject', 'goog.ui.media.Media', 'goog.ui.media.MediaModel', 'goog.ui.media.Mp3'], false);
goog.addDependency('ui/media/photo.js', ['goog.ui.media.Photo'], ['goog.ui.media.Media', 'goog.ui.media.MediaRenderer'], false);
goog.addDependency('ui/media/photo_test.js', ['goog.ui.media.PhotoTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.ui.media.MediaModel', 'goog.ui.media.Photo'], false);
goog.addDependency('ui/media/picasa.js', ['goog.ui.media.PicasaAlbum', 'goog.ui.media.PicasaAlbumModel'], ['goog.ui.media.FlashObject', 'goog.ui.media.Media', 'goog.ui.media.MediaModel', 'goog.ui.media.MediaRenderer'], false);
goog.addDependency('ui/media/picasa_test.js', ['goog.ui.media.PicasaTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.ui.media.FlashObject', 'goog.ui.media.Media', 'goog.ui.media.PicasaAlbum', 'goog.ui.media.PicasaAlbumModel'], false);
goog.addDependency('ui/media/vimeo.js', ['goog.ui.media.Vimeo', 'goog.ui.media.VimeoModel'], ['goog.string', 'goog.ui.media.FlashObject', 'goog.ui.media.Media', 'goog.ui.media.MediaModel', 'goog.ui.media.MediaRenderer'], false);
goog.addDependency('ui/media/vimeo_test.js', ['goog.ui.media.VimeoTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.ui.media.FlashObject', 'goog.ui.media.Media', 'goog.ui.media.Vimeo', 'goog.ui.media.VimeoModel'], false);
goog.addDependency('ui/media/youtube.js', ['goog.ui.media.Youtube', 'goog.ui.media.YoutubeModel'], ['goog.string', 'goog.ui.Component', 'goog.ui.media.FlashObject', 'goog.ui.media.Media', 'goog.ui.media.MediaModel', 'goog.ui.media.MediaRenderer'], false);
goog.addDependency('ui/media/youtube_test.js', ['goog.ui.media.YoutubeTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.ui.media.FlashObject', 'goog.ui.media.Youtube', 'goog.ui.media.YoutubeModel'], false);
goog.addDependency('ui/menu.js', ['goog.ui.Menu', 'goog.ui.Menu.EventType'], ['goog.math.Coordinate', 'goog.string', 'goog.style', 'goog.ui.Component.EventType', 'goog.ui.Component.State', 'goog.ui.Container', 'goog.ui.Container.Orientation', 'goog.ui.MenuHeader', 'goog.ui.MenuItem', 'goog.ui.MenuRenderer', 'goog.ui.MenuSeparator'], false);
goog.addDependency('ui/menu_test.js', ['goog.ui.MenuTest'], ['goog.dom', 'goog.events', 'goog.math.Coordinate', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.Component', 'goog.ui.Menu'], false);
goog.addDependency('ui/menubar.js', ['goog.ui.menuBar'], ['goog.ui.Container', 'goog.ui.MenuBarRenderer'], false);
goog.addDependency('ui/menubardecorator.js', ['goog.ui.menuBarDecorator'], ['goog.ui.MenuBarRenderer', 'goog.ui.menuBar', 'goog.ui.registry'], false);
goog.addDependency('ui/menubarrenderer.js', ['goog.ui.MenuBarRenderer'], ['goog.a11y.aria.Role', 'goog.ui.Container', 'goog.ui.ContainerRenderer'], false);
goog.addDependency('ui/menubase.js', ['goog.ui.MenuBase'], ['goog.events.EventHandler', 'goog.events.EventType', 'goog.events.KeyHandler', 'goog.ui.Popup'], false);
goog.addDependency('ui/menubutton.js', ['goog.ui.MenuButton'], ['goog.Timer', 'goog.a11y.aria', 'goog.a11y.aria.State', 'goog.asserts', 'goog.dom', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.events.KeyHandler', 'goog.math.Box', 'goog.math.Rect', 'goog.positioning', 'goog.positioning.Corner', 'goog.positioning.MenuAnchoredPosition', 'goog.positioning.Overflow', 'goog.style', 'goog.ui.Button', 'goog.ui.Component', 'goog.ui.IdGenerator', 'goog.ui.Menu', 'goog.ui.MenuButtonRenderer', 'goog.ui.MenuItem', 'goog.ui.MenuRenderer', 'goog.ui.registry', 'goog.userAgent', 'goog.userAgent.product'], false);
goog.addDependency('ui/menubutton_test.js', ['goog.ui.MenuButtonTest'], ['goog.Timer', 'goog.a11y.aria', 'goog.a11y.aria.State', 'goog.dom', 'goog.events', 'goog.events.Event', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.events.KeyHandler', 'goog.positioning', 'goog.positioning.Corner', 'goog.positioning.MenuAnchoredPosition', 'goog.positioning.Overflow', 'goog.style', 'goog.testing.ExpectedFailures', 'goog.testing.PropertyReplacer', 'goog.testing.events', 'goog.testing.jsunit', 'goog.testing.recordFunction', 'goog.ui.Component', 'goog.ui.Menu', 'goog.ui.MenuButton', 'goog.ui.MenuItem', 'goog.ui.SubMenu', 'goog.userAgent', 'goog.userAgent.product', 'goog.userAgent.product.isVersion'], false);
goog.addDependency('ui/menubuttonrenderer.js', ['goog.ui.MenuButtonRenderer'], ['goog.dom', 'goog.style', 'goog.ui.CustomButtonRenderer', 'goog.ui.INLINE_BLOCK_CLASSNAME', 'goog.ui.Menu', 'goog.ui.MenuRenderer'], false);
goog.addDependency('ui/menubuttonrenderer_test.js', ['goog.ui.MenuButtonRendererTest'], ['goog.a11y.aria', 'goog.a11y.aria.State', 'goog.dom', 'goog.dom.classlist', 'goog.testing.jsunit', 'goog.testing.ui.rendererasserts', 'goog.ui.MenuButton', 'goog.ui.MenuButtonRenderer', 'goog.userAgent'], false);
goog.addDependency('ui/menuheader.js', ['goog.ui.MenuHeader'], ['goog.ui.Component', 'goog.ui.Control', 'goog.ui.MenuHeaderRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/menuheaderrenderer.js', ['goog.ui.MenuHeaderRenderer'], ['goog.ui.ControlRenderer'], false);
goog.addDependency('ui/menuitem.js', ['goog.ui.MenuItem'], ['goog.a11y.aria.Role', 'goog.array', 'goog.dom', 'goog.dom.classlist', 'goog.math.Coordinate', 'goog.string', 'goog.ui.Component', 'goog.ui.Control', 'goog.ui.MenuItemRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/menuitem_test.js', ['goog.ui.MenuItemTest'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.array', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.classlist', 'goog.events.KeyCodes', 'goog.math.Coordinate', 'goog.testing.events', 'goog.testing.jsunit', 'goog.testing.recordFunction', 'goog.ui.Component', 'goog.ui.MenuItem', 'goog.ui.MenuItemRenderer'], false);
goog.addDependency('ui/menuitemrenderer.js', ['goog.ui.MenuItemRenderer'], ['goog.a11y.aria.Role', 'goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.ui.Component', 'goog.ui.ControlRenderer'], false);
goog.addDependency('ui/menuitemrenderer_test.js', ['goog.ui.MenuItemRendererTest'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.dom', 'goog.dom.classlist', 'goog.testing.jsunit', 'goog.testing.ui.rendererasserts', 'goog.ui.Component', 'goog.ui.MenuItem', 'goog.ui.MenuItemRenderer'], false);
goog.addDependency('ui/menurenderer.js', ['goog.ui.MenuRenderer'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.asserts', 'goog.dom', 'goog.ui.ContainerRenderer', 'goog.ui.Separator'], false);
goog.addDependency('ui/menuseparator.js', ['goog.ui.MenuSeparator'], ['goog.ui.MenuSeparatorRenderer', 'goog.ui.Separator', 'goog.ui.registry'], false);
goog.addDependency('ui/menuseparatorrenderer.js', ['goog.ui.MenuSeparatorRenderer'], ['goog.dom', 'goog.dom.classlist', 'goog.ui.ControlContent', 'goog.ui.ControlRenderer'], false);
goog.addDependency('ui/menuseparatorrenderer_test.js', ['goog.ui.MenuSeparatorRendererTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.ui.MenuSeparator', 'goog.ui.MenuSeparatorRenderer'], false);
goog.addDependency('ui/mockactivitymonitor.js', ['goog.ui.MockActivityMonitor'], ['goog.events.EventType', 'goog.ui.ActivityMonitor'], false);
goog.addDependency('ui/mockactivitymonitor_test.js', ['goog.ui.MockActivityMonitorTest'], ['goog.events', 'goog.functions', 'goog.testing.jsunit', 'goog.testing.recordFunction', 'goog.ui.ActivityMonitor', 'goog.ui.MockActivityMonitor'], false);
goog.addDependency('ui/modalpopup.js', ['goog.ui.ModalPopup'], ['goog.Timer', 'goog.a11y.aria', 'goog.a11y.aria.State', 'goog.asserts', 'goog.dom', 'goog.dom.TagName', 'goog.dom.classlist', 'goog.dom.iframe', 'goog.events', 'goog.events.EventType', 'goog.events.FocusHandler', 'goog.fx.Transition', 'goog.string', 'goog.style', 'goog.ui.Component', 'goog.ui.PopupBase', 'goog.userAgent'], false);
goog.addDependency('ui/modalpopup_test.js', ['goog.ui.ModalPopupTest'], ['goog.a11y.aria', 'goog.a11y.aria.State', 'goog.dispose', 'goog.dom', 'goog.events', 'goog.events.EventTarget', 'goog.fx.Transition', 'goog.fx.css3', 'goog.string', 'goog.style', 'goog.testing.MockClock', 'goog.testing.jsunit', 'goog.ui.ModalPopup', 'goog.ui.PopupBase'], false);
goog.addDependency('ui/nativebuttonrenderer.js', ['goog.ui.NativeButtonRenderer'], ['goog.asserts', 'goog.dom.classlist', 'goog.events.EventType', 'goog.ui.ButtonRenderer', 'goog.ui.Component'], false);
goog.addDependency('ui/nativebuttonrenderer_test.js', ['goog.ui.NativeButtonRendererTest'], ['goog.dom', 'goog.dom.classlist', 'goog.events', 'goog.testing.ExpectedFailures', 'goog.testing.events', 'goog.testing.jsunit', 'goog.testing.ui.rendererasserts', 'goog.ui.Button', 'goog.ui.Component', 'goog.ui.NativeButtonRenderer', 'goog.userAgent'], false);
goog.addDependency('ui/option.js', ['goog.ui.Option'], ['goog.ui.Component', 'goog.ui.MenuItem', 'goog.ui.registry'], false);
goog.addDependency('ui/palette.js', ['goog.ui.Palette'], ['goog.array', 'goog.dom', 'goog.events', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.math.Size', 'goog.ui.Component', 'goog.ui.Control', 'goog.ui.PaletteRenderer', 'goog.ui.SelectionModel'], false);
goog.addDependency('ui/palette_test.js', ['goog.ui.PaletteTest'], ['goog.a11y.aria', 'goog.dom', 'goog.events', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.events.KeyEvent', 'goog.testing.events.Event', 'goog.testing.jsunit', 'goog.testing.recordFunction', 'goog.ui.Component', 'goog.ui.Container', 'goog.ui.Palette'], false);
goog.addDependency('ui/paletterenderer.js', ['goog.ui.PaletteRenderer'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.array', 'goog.asserts', 'goog.dom', 'goog.dom.NodeIterator', 'goog.dom.NodeType', 'goog.dom.TagName', 'goog.dom.classlist', 'goog.iter', 'goog.style', 'goog.ui.ControlRenderer', 'goog.userAgent'], false);
goog.addDependency('ui/paletterenderer_test.js', ['goog.ui.PaletteRendererTest'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.array', 'goog.dom', 'goog.testing.jsunit', 'goog.ui.Palette', 'goog.ui.PaletteRenderer'], false);
goog.addDependency('ui/plaintextspellchecker.js', ['goog.ui.PlainTextSpellChecker'], ['goog.Timer', 'goog.a11y.aria', 'goog.asserts', 'goog.dom', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.events.KeyHandler', 'goog.spell.SpellCheck', 'goog.style', 'goog.ui.AbstractSpellChecker', 'goog.ui.Component', 'goog.userAgent'], false);
goog.addDependency('ui/plaintextspellchecker_test.js', ['goog.ui.PlainTextSpellCheckerTest'], ['goog.Timer', 'goog.dom', 'goog.events.KeyCodes', 'goog.spell.SpellCheck', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.PlainTextSpellChecker'], false);
goog.addDependency('ui/popup.js', ['goog.ui.Popup', 'goog.ui.Popup.AbsolutePosition', 'goog.ui.Popup.AnchoredPosition', 'goog.ui.Popup.AnchoredViewPortPosition', 'goog.ui.Popup.ClientPosition', 'goog.ui.Popup.Corner', 'goog.ui.Popup.Overflow', 'goog.ui.Popup.ViewPortClientPosition', 'goog.ui.Popup.ViewPortPosition'], ['goog.math.Box', 'goog.positioning.AbsolutePosition', 'goog.positioning.AnchoredPosition', 'goog.positioning.AnchoredViewportPosition', 'goog.positioning.ClientPosition', 'goog.positioning.Corner', 'goog.positioning.Overflow', 'goog.positioning.ViewportClientPosition', 'goog.positioning.ViewportPosition', 'goog.style', 'goog.ui.PopupBase'], false);
goog.addDependency('ui/popup_test.js', ['goog.ui.PopupTest'], ['goog.positioning.AnchoredPosition', 'goog.positioning.Corner', 'goog.style', 'goog.testing.jsunit', 'goog.ui.Popup', 'goog.userAgent'], false);
goog.addDependency('ui/popupbase.js', ['goog.ui.PopupBase', 'goog.ui.PopupBase.EventType', 'goog.ui.PopupBase.Type'], ['goog.Timer', 'goog.array', 'goog.dom', 'goog.events', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.fx.Transition', 'goog.style', 'goog.userAgent'], false);
goog.addDependency('ui/popupbase_test.js', ['goog.ui.PopupBaseTest'], ['goog.dom', 'goog.events', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.fx.Transition', 'goog.fx.css3', 'goog.testing.MockClock', 'goog.testing.events', 'goog.testing.events.Event', 'goog.testing.jsunit', 'goog.ui.PopupBase'], false);
goog.addDependency('ui/popupcolorpicker.js', ['goog.ui.PopupColorPicker'], ['goog.asserts', 'goog.dom.classlist', 'goog.events.EventType', 'goog.positioning.AnchoredPosition', 'goog.positioning.Corner', 'goog.ui.ColorPicker', 'goog.ui.Component', 'goog.ui.Popup'], false);
goog.addDependency('ui/popupcolorpicker_test.js', ['goog.ui.PopupColorPickerTest'], ['goog.dom', 'goog.events', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.ColorPicker', 'goog.ui.PopupColorPicker'], false);
goog.addDependency('ui/popupdatepicker.js', ['goog.ui.PopupDatePicker'], ['goog.events.EventType', 'goog.positioning.AnchoredPosition', 'goog.positioning.Corner', 'goog.positioning.Overflow', 'goog.style', 'goog.ui.Component', 'goog.ui.DatePicker', 'goog.ui.Popup', 'goog.ui.PopupBase'], false);
goog.addDependency('ui/popupdatepicker_test.js', ['goog.ui.PopupDatePickerTest'], ['goog.events', 'goog.testing.jsunit', 'goog.testing.recordFunction', 'goog.ui.PopupBase', 'goog.ui.PopupDatePicker'], false);
goog.addDependency('ui/popupmenu.js', ['goog.ui.PopupMenu'], ['goog.events.EventType', 'goog.positioning.AnchoredViewportPosition', 'goog.positioning.Corner', 'goog.positioning.MenuAnchoredPosition', 'goog.positioning.Overflow', 'goog.positioning.ViewportClientPosition', 'goog.structs.Map', 'goog.style', 'goog.ui.Component', 'goog.ui.Menu', 'goog.ui.PopupBase', 'goog.userAgent'], false);
goog.addDependency('ui/popupmenu_test.js', ['goog.ui.PopupMenuTest'], ['goog.dom', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.math.Box', 'goog.math.Coordinate', 'goog.positioning.Corner', 'goog.style', 'goog.testing.jsunit', 'goog.ui.Menu', 'goog.ui.MenuItem', 'goog.ui.PopupMenu'], false);
goog.addDependency('ui/progressbar.js', ['goog.ui.ProgressBar', 'goog.ui.ProgressBar.Orientation'], ['goog.a11y.aria', 'goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.events', 'goog.events.EventType', 'goog.ui.Component', 'goog.ui.RangeModel', 'goog.userAgent'], false);
goog.addDependency('ui/prompt.js', ['goog.ui.Prompt'], ['goog.Timer', 'goog.dom', 'goog.events', 'goog.events.EventType', 'goog.functions', 'goog.html.SafeHtml', 'goog.html.legacyconversions', 'goog.ui.Component', 'goog.ui.Dialog', 'goog.userAgent'], false);
goog.addDependency('ui/prompt_test.js', ['goog.ui.PromptTest'], ['goog.dom.selection', 'goog.events.InputHandler', 'goog.events.KeyCodes', 'goog.functions', 'goog.string', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.BidiInput', 'goog.ui.Dialog', 'goog.ui.Prompt', 'goog.userAgent', 'goog.userAgent.product'], false);
goog.addDependency('ui/rangemodel.js', ['goog.ui.RangeModel'], ['goog.events.EventTarget', 'goog.ui.Component'], false);
goog.addDependency('ui/rangemodel_test.js', ['goog.ui.RangeModelTest'], ['goog.testing.jsunit', 'goog.ui.RangeModel'], false);
goog.addDependency('ui/ratings.js', ['goog.ui.Ratings', 'goog.ui.Ratings.EventType'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.asserts', 'goog.dom.classlist', 'goog.events.EventType', 'goog.ui.Component'], false);
goog.addDependency('ui/registry.js', ['goog.ui.registry'], ['goog.asserts', 'goog.dom.classlist'], false);
goog.addDependency('ui/registry_test.js', ['goog.ui.registryTest'], ['goog.object', 'goog.testing.jsunit', 'goog.ui.registry'], false);
goog.addDependency('ui/richtextspellchecker.js', ['goog.ui.RichTextSpellChecker'], ['goog.Timer', 'goog.asserts', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.Range', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.events.KeyHandler', 'goog.math.Coordinate', 'goog.spell.SpellCheck', 'goog.string.StringBuffer', 'goog.style', 'goog.ui.AbstractSpellChecker', 'goog.ui.Component', 'goog.ui.PopupMenu'], false);
goog.addDependency('ui/richtextspellchecker_test.js', ['goog.ui.RichTextSpellCheckerTest'], ['goog.dom.Range', 'goog.dom.classlist', 'goog.events.KeyCodes', 'goog.object', 'goog.spell.SpellCheck', 'goog.testing.MockClock', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.RichTextSpellChecker'], false);
goog.addDependency('ui/roundedpanel.js', ['goog.ui.BaseRoundedPanel', 'goog.ui.CssRoundedPanel', 'goog.ui.GraphicsRoundedPanel', 'goog.ui.RoundedPanel', 'goog.ui.RoundedPanel.Corner'], ['goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.graphics', 'goog.graphics.Path', 'goog.graphics.SolidFill', 'goog.graphics.Stroke', 'goog.math', 'goog.math.Coordinate', 'goog.style', 'goog.ui.Component', 'goog.userAgent'], false);
goog.addDependency('ui/roundedpanel_test.js', ['goog.ui.RoundedPanelTest'], ['goog.testing.jsunit', 'goog.ui.CssRoundedPanel', 'goog.ui.GraphicsRoundedPanel', 'goog.ui.RoundedPanel', 'goog.userAgent'], false);
goog.addDependency('ui/roundedtabrenderer.js', ['goog.ui.RoundedTabRenderer'], ['goog.dom', 'goog.ui.Tab', 'goog.ui.TabBar', 'goog.ui.TabRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/scrollfloater.js', ['goog.ui.ScrollFloater', 'goog.ui.ScrollFloater.EventType'], ['goog.array', 'goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.events.EventType', 'goog.style', 'goog.ui.Component', 'goog.userAgent'], false);
goog.addDependency('ui/scrollfloater_test.js', ['goog.ui.ScrollFloaterTest'], ['goog.dom', 'goog.events', 'goog.style', 'goog.testing.jsunit', 'goog.ui.ScrollFloater'], false);
goog.addDependency('ui/select.js', ['goog.ui.Select'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.array', 'goog.events.EventType', 'goog.ui.Component', 'goog.ui.IdGenerator', 'goog.ui.MenuButton', 'goog.ui.MenuItem', 'goog.ui.MenuRenderer', 'goog.ui.SelectionModel', 'goog.ui.registry'], false);
goog.addDependency('ui/select_test.js', ['goog.ui.SelectTest'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.dom', 'goog.events', 'goog.testing.jsunit', 'goog.testing.recordFunction', 'goog.ui.Component', 'goog.ui.CustomButtonRenderer', 'goog.ui.Menu', 'goog.ui.MenuItem', 'goog.ui.Select', 'goog.ui.Separator'], false);
goog.addDependency('ui/selectionmenubutton.js', ['goog.ui.SelectionMenuButton', 'goog.ui.SelectionMenuButton.SelectionState'], ['goog.events.EventType', 'goog.style', 'goog.ui.Component', 'goog.ui.MenuButton', 'goog.ui.MenuItem', 'goog.ui.registry'], false);
goog.addDependency('ui/selectionmenubutton_test.js', ['goog.ui.SelectionMenuButtonTest'], ['goog.dom', 'goog.events', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.Component', 'goog.ui.SelectionMenuButton'], false);
goog.addDependency('ui/selectionmodel.js', ['goog.ui.SelectionModel'], ['goog.array', 'goog.events.EventTarget', 'goog.events.EventType'], false);
goog.addDependency('ui/selectionmodel_test.js', ['goog.ui.SelectionModelTest'], ['goog.array', 'goog.testing.jsunit', 'goog.testing.recordFunction', 'goog.ui.SelectionModel'], false);
goog.addDependency('ui/separator.js', ['goog.ui.Separator'], ['goog.a11y.aria', 'goog.asserts', 'goog.ui.Component', 'goog.ui.Control', 'goog.ui.MenuSeparatorRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/serverchart.js', ['goog.ui.ServerChart', 'goog.ui.ServerChart.AxisDisplayType', 'goog.ui.ServerChart.ChartType', 'goog.ui.ServerChart.EncodingType', 'goog.ui.ServerChart.Event', 'goog.ui.ServerChart.LegendPosition', 'goog.ui.ServerChart.MaximumValue', 'goog.ui.ServerChart.MultiAxisAlignment', 'goog.ui.ServerChart.MultiAxisType', 'goog.ui.ServerChart.UriParam', 'goog.ui.ServerChart.UriTooLongEvent'], ['goog.Uri', 'goog.array', 'goog.asserts', 'goog.events.Event', 'goog.string', 'goog.ui.Component'], false);
goog.addDependency('ui/serverchart_test.js', ['goog.ui.ServerChartTest'], ['goog.Uri', 'goog.events', 'goog.testing.jsunit', 'goog.ui.ServerChart'], false);
goog.addDependency('ui/slider.js', ['goog.ui.Slider', 'goog.ui.Slider.Orientation'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.dom', 'goog.ui.SliderBase'], false);
goog.addDependency('ui/sliderbase.js', ['goog.ui.SliderBase', 'goog.ui.SliderBase.AnimationFactory', 'goog.ui.SliderBase.Orientation'], ['goog.Timer', 'goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.array', 'goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.events', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.events.KeyHandler', 'goog.events.MouseWheelHandler', 'goog.functions', 'goog.fx.AnimationParallelQueue', 'goog.fx.Dragger', 'goog.fx.Transition', 'goog.fx.dom.ResizeHeight', 'goog.fx.dom.ResizeWidth', 'goog.fx.dom.Slide', 'goog.math', 'goog.math.Coordinate', 'goog.style', 'goog.style.bidi', 'goog.ui.Component', 'goog.ui.RangeModel'], false);
goog.addDependency('ui/sliderbase_test.js', ['goog.ui.SliderBaseTest'], ['goog.a11y.aria', 'goog.a11y.aria.State', 'goog.dom', 'goog.dom.classlist', 'goog.events', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.fx.Animation', 'goog.math.Coordinate', 'goog.style', 'goog.style.bidi', 'goog.testing.MockClock', 'goog.testing.MockControl', 'goog.testing.events', 'goog.testing.jsunit', 'goog.testing.mockmatchers', 'goog.testing.recordFunction', 'goog.ui.Component', 'goog.ui.SliderBase', 'goog.userAgent'], false);
goog.addDependency('ui/splitbehavior.js', ['goog.ui.SplitBehavior', 'goog.ui.SplitBehavior.DefaultHandlers'], ['goog.Disposable', 'goog.asserts', 'goog.dispose', 'goog.dom', 'goog.dom.NodeType', 'goog.dom.classlist', 'goog.events.EventHandler', 'goog.ui.ButtonSide', 'goog.ui.Component', 'goog.ui.decorate', 'goog.ui.registry'], false);
goog.addDependency('ui/splitbehavior_test.js', ['goog.ui.SplitBehaviorTest'], ['goog.array', 'goog.dom', 'goog.events', 'goog.events.Event', 'goog.testing.jsunit', 'goog.ui.Component', 'goog.ui.CustomButton', 'goog.ui.Menu', 'goog.ui.MenuButton', 'goog.ui.MenuItem', 'goog.ui.SplitBehavior', 'goog.ui.decorate'], false);
goog.addDependency('ui/splitpane.js', ['goog.ui.SplitPane', 'goog.ui.SplitPane.Orientation'], ['goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.events.EventType', 'goog.fx.Dragger', 'goog.math.Rect', 'goog.math.Size', 'goog.style', 'goog.ui.Component', 'goog.userAgent'], false);
goog.addDependency('ui/splitpane_test.js', ['goog.ui.SplitPaneTest'], ['goog.dom', 'goog.dom.classlist', 'goog.events', 'goog.math.Size', 'goog.style', 'goog.testing.events', 'goog.testing.jsunit', 'goog.testing.recordFunction', 'goog.ui.Component', 'goog.ui.SplitPane'], false);
goog.addDependency('ui/style/app/buttonrenderer.js', ['goog.ui.style.app.ButtonRenderer'], ['goog.dom.classlist', 'goog.ui.Button', 'goog.ui.CustomButtonRenderer', 'goog.ui.INLINE_BLOCK_CLASSNAME', 'goog.ui.registry'], false);
goog.addDependency('ui/style/app/buttonrenderer_test.js', ['goog.ui.style.app.ButtonRendererTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.testing.ui.style', 'goog.ui.Button', 'goog.ui.Component', 'goog.ui.style.app.ButtonRenderer', 'goog.userAgent'], false);
goog.addDependency('ui/style/app/menubuttonrenderer.js', ['goog.ui.style.app.MenuButtonRenderer'], ['goog.a11y.aria.Role', 'goog.array', 'goog.dom', 'goog.style', 'goog.ui.Menu', 'goog.ui.MenuRenderer', 'goog.ui.style.app.ButtonRenderer'], false);
goog.addDependency('ui/style/app/menubuttonrenderer_test.js', ['goog.ui.style.app.MenuButtonRendererTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.testing.ui.style', 'goog.ui.Component', 'goog.ui.MenuButton', 'goog.ui.style.app.MenuButtonRenderer'], false);
goog.addDependency('ui/style/app/primaryactionbuttonrenderer.js', ['goog.ui.style.app.PrimaryActionButtonRenderer'], ['goog.ui.Button', 'goog.ui.registry', 'goog.ui.style.app.ButtonRenderer'], false);
goog.addDependency('ui/style/app/primaryactionbuttonrenderer_test.js', ['goog.ui.style.app.PrimaryActionButtonRendererTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.testing.ui.style', 'goog.ui.Button', 'goog.ui.Component', 'goog.ui.style.app.PrimaryActionButtonRenderer'], false);
goog.addDependency('ui/submenu.js', ['goog.ui.SubMenu'], ['goog.Timer', 'goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.events.KeyCodes', 'goog.positioning.AnchoredViewportPosition', 'goog.positioning.Corner', 'goog.style', 'goog.ui.Component', 'goog.ui.Menu', 'goog.ui.MenuItem', 'goog.ui.SubMenuRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/submenu_test.js', ['goog.ui.SubMenuTest'], ['goog.dom', 'goog.dom.classlist', 'goog.events', 'goog.events.Event', 'goog.events.KeyCodes', 'goog.functions', 'goog.positioning', 'goog.positioning.Overflow', 'goog.style', 'goog.testing.MockClock', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.Component', 'goog.ui.Menu', 'goog.ui.MenuItem', 'goog.ui.SubMenu', 'goog.ui.SubMenuRenderer'], false);
goog.addDependency('ui/submenurenderer.js', ['goog.ui.SubMenuRenderer'], ['goog.a11y.aria', 'goog.a11y.aria.State', 'goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.style', 'goog.ui.Menu', 'goog.ui.MenuItemRenderer'], false);
goog.addDependency('ui/tab.js', ['goog.ui.Tab'], ['goog.ui.Component', 'goog.ui.Control', 'goog.ui.TabRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/tab_test.js', ['goog.ui.TabTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.ui.Component', 'goog.ui.Tab', 'goog.ui.TabRenderer'], false);
goog.addDependency('ui/tabbar.js', ['goog.ui.TabBar', 'goog.ui.TabBar.Location'], ['goog.ui.Component.EventType', 'goog.ui.Container', 'goog.ui.Container.Orientation', 'goog.ui.Tab', 'goog.ui.TabBarRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/tabbar_test.js', ['goog.ui.TabBarTest'], ['goog.dom', 'goog.events', 'goog.events.Event', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.testing.jsunit', 'goog.ui.Component', 'goog.ui.Container', 'goog.ui.Tab', 'goog.ui.TabBar', 'goog.ui.TabBarRenderer'], false);
goog.addDependency('ui/tabbarrenderer.js', ['goog.ui.TabBarRenderer'], ['goog.a11y.aria.Role', 'goog.object', 'goog.ui.ContainerRenderer'], false);
goog.addDependency('ui/tabbarrenderer_test.js', ['goog.ui.TabBarRendererTest'], ['goog.a11y.aria.Role', 'goog.dom', 'goog.dom.classlist', 'goog.testing.jsunit', 'goog.testing.ui.rendererasserts', 'goog.ui.Container', 'goog.ui.TabBar', 'goog.ui.TabBarRenderer'], false);
goog.addDependency('ui/tablesorter.js', ['goog.ui.TableSorter', 'goog.ui.TableSorter.EventType'], ['goog.array', 'goog.dom', 'goog.dom.TagName', 'goog.dom.classlist', 'goog.events.EventType', 'goog.functions', 'goog.ui.Component'], false);
goog.addDependency('ui/tablesorter_test.js', ['goog.ui.TableSorterTest'], ['goog.array', 'goog.dom', 'goog.dom.classlist', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.TableSorter'], false);
goog.addDependency('ui/tabpane.js', ['goog.ui.TabPane', 'goog.ui.TabPane.Events', 'goog.ui.TabPane.TabLocation', 'goog.ui.TabPane.TabPage', 'goog.ui.TabPaneEvent'], ['goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.events', 'goog.events.Event', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.style'], false);
goog.addDependency('ui/tabpane_test.js', ['goog.ui.TabPaneTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.ui.TabPane'], false);
goog.addDependency('ui/tabrenderer.js', ['goog.ui.TabRenderer'], ['goog.a11y.aria.Role', 'goog.ui.Component', 'goog.ui.ControlRenderer'], false);
goog.addDependency('ui/tabrenderer_test.js', ['goog.ui.TabRendererTest'], ['goog.a11y.aria.Role', 'goog.dom', 'goog.dom.classlist', 'goog.testing.dom', 'goog.testing.jsunit', 'goog.testing.ui.rendererasserts', 'goog.ui.Tab', 'goog.ui.TabRenderer'], false);
goog.addDependency('ui/textarea.js', ['goog.ui.Textarea', 'goog.ui.Textarea.EventType'], ['goog.asserts', 'goog.dom', 'goog.dom.classlist', 'goog.events.EventType', 'goog.style', 'goog.ui.Control', 'goog.ui.TextareaRenderer', 'goog.userAgent'], false);
goog.addDependency('ui/textarea_test.js', ['goog.ui.TextareaTest'], ['goog.dom', 'goog.dom.classlist', 'goog.events', 'goog.style', 'goog.testing.ExpectedFailures', 'goog.testing.events.EventObserver', 'goog.testing.jsunit', 'goog.ui.Textarea', 'goog.ui.TextareaRenderer', 'goog.userAgent', 'goog.userAgent.product'], false);
goog.addDependency('ui/textarearenderer.js', ['goog.ui.TextareaRenderer'], ['goog.dom.TagName', 'goog.ui.Component', 'goog.ui.ControlRenderer'], false);
goog.addDependency('ui/togglebutton.js', ['goog.ui.ToggleButton'], ['goog.ui.Button', 'goog.ui.Component', 'goog.ui.CustomButtonRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/toolbar.js', ['goog.ui.Toolbar'], ['goog.ui.Container', 'goog.ui.ToolbarRenderer'], false);
goog.addDependency('ui/toolbar_test.js', ['goog.ui.ToolbarTest'], ['goog.a11y.aria', 'goog.dom', 'goog.events.EventType', 'goog.testing.events', 'goog.testing.events.Event', 'goog.testing.jsunit', 'goog.ui.Toolbar', 'goog.ui.ToolbarMenuButton'], false);
goog.addDependency('ui/toolbarbutton.js', ['goog.ui.ToolbarButton'], ['goog.ui.Button', 'goog.ui.ToolbarButtonRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/toolbarbuttonrenderer.js', ['goog.ui.ToolbarButtonRenderer'], ['goog.ui.CustomButtonRenderer'], false);
goog.addDependency('ui/toolbarcolormenubutton.js', ['goog.ui.ToolbarColorMenuButton'], ['goog.ui.ColorMenuButton', 'goog.ui.ToolbarColorMenuButtonRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/toolbarcolormenubuttonrenderer.js', ['goog.ui.ToolbarColorMenuButtonRenderer'], ['goog.asserts', 'goog.dom.classlist', 'goog.ui.ColorMenuButtonRenderer', 'goog.ui.MenuButtonRenderer', 'goog.ui.ToolbarMenuButtonRenderer'], false);
goog.addDependency('ui/toolbarcolormenubuttonrenderer_test.js', ['goog.ui.ToolbarColorMenuButtonRendererTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.testing.ui.RendererHarness', 'goog.testing.ui.rendererasserts', 'goog.ui.ToolbarColorMenuButton', 'goog.ui.ToolbarColorMenuButtonRenderer'], false);
goog.addDependency('ui/toolbarmenubutton.js', ['goog.ui.ToolbarMenuButton'], ['goog.ui.MenuButton', 'goog.ui.ToolbarMenuButtonRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/toolbarmenubuttonrenderer.js', ['goog.ui.ToolbarMenuButtonRenderer'], ['goog.ui.MenuButtonRenderer'], false);
goog.addDependency('ui/toolbarrenderer.js', ['goog.ui.ToolbarRenderer'], ['goog.a11y.aria.Role', 'goog.ui.Container', 'goog.ui.ContainerRenderer', 'goog.ui.Separator', 'goog.ui.ToolbarSeparatorRenderer'], false);
goog.addDependency('ui/toolbarselect.js', ['goog.ui.ToolbarSelect'], ['goog.ui.Select', 'goog.ui.ToolbarMenuButtonRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/toolbarseparator.js', ['goog.ui.ToolbarSeparator'], ['goog.ui.Separator', 'goog.ui.ToolbarSeparatorRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/toolbarseparatorrenderer.js', ['goog.ui.ToolbarSeparatorRenderer'], ['goog.asserts', 'goog.dom.classlist', 'goog.ui.INLINE_BLOCK_CLASSNAME', 'goog.ui.MenuSeparatorRenderer'], false);
goog.addDependency('ui/toolbartogglebutton.js', ['goog.ui.ToolbarToggleButton'], ['goog.ui.ToggleButton', 'goog.ui.ToolbarButtonRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/tooltip.js', ['goog.ui.Tooltip', 'goog.ui.Tooltip.CursorTooltipPosition', 'goog.ui.Tooltip.ElementTooltipPosition', 'goog.ui.Tooltip.State'], ['goog.Timer', 'goog.array', 'goog.dom', 'goog.dom.safe', 'goog.events', 'goog.events.EventType', 'goog.html.legacyconversions', 'goog.math.Box', 'goog.math.Coordinate', 'goog.positioning', 'goog.positioning.AnchoredPosition', 'goog.positioning.Corner', 'goog.positioning.Overflow', 'goog.positioning.OverflowStatus', 'goog.positioning.ViewportPosition', 'goog.structs.Set', 'goog.style', 'goog.ui.Popup', 'goog.ui.PopupBase'], false);
goog.addDependency('ui/tooltip_test.js', ['goog.ui.TooltipTest'], ['goog.dom', 'goog.events.Event', 'goog.events.EventHandler', 'goog.events.EventType', 'goog.html.testing', 'goog.math.Coordinate', 'goog.positioning.AbsolutePosition', 'goog.style', 'goog.testing.MockClock', 'goog.testing.PropertyReplacer', 'goog.testing.TestQueue', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.PopupBase', 'goog.ui.Tooltip', 'goog.userAgent'], false);
goog.addDependency('ui/tree/basenode.js', ['goog.ui.tree.BaseNode', 'goog.ui.tree.BaseNode.EventType'], ['goog.Timer', 'goog.a11y.aria', 'goog.asserts', 'goog.dom.safe', 'goog.events.Event', 'goog.events.KeyCodes', 'goog.html.SafeHtml', 'goog.html.SafeStyle', 'goog.html.legacyconversions', 'goog.string', 'goog.string.StringBuffer', 'goog.style', 'goog.ui.Component'], false);
goog.addDependency('ui/tree/basenode_test.js', ['goog.ui.tree.BaseNodeTest'], ['goog.dom', 'goog.dom.classlist', 'goog.html.legacyconversions', 'goog.html.testing', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.ui.Component', 'goog.ui.tree.BaseNode', 'goog.ui.tree.TreeControl', 'goog.ui.tree.TreeNode'], false);
goog.addDependency('ui/tree/treecontrol.js', ['goog.ui.tree.TreeControl'], ['goog.a11y.aria', 'goog.asserts', 'goog.dom.classlist', 'goog.events.EventType', 'goog.events.FocusHandler', 'goog.events.KeyHandler', 'goog.html.SafeHtml', 'goog.log', 'goog.ui.tree.BaseNode', 'goog.ui.tree.TreeNode', 'goog.ui.tree.TypeAhead', 'goog.userAgent'], false);
goog.addDependency('ui/tree/treecontrol_test.js', ['goog.ui.tree.TreeControlTest'], ['goog.dom', 'goog.testing.jsunit', 'goog.ui.tree.TreeControl'], false);
goog.addDependency('ui/tree/treenode.js', ['goog.ui.tree.TreeNode'], ['goog.ui.tree.BaseNode'], false);
goog.addDependency('ui/tree/typeahead.js', ['goog.ui.tree.TypeAhead', 'goog.ui.tree.TypeAhead.Offset'], ['goog.array', 'goog.events.KeyCodes', 'goog.string', 'goog.structs.Trie'], false);
goog.addDependency('ui/tree/typeahead_test.js', ['goog.ui.tree.TypeAheadTest'], ['goog.dom', 'goog.events.KeyCodes', 'goog.testing.jsunit', 'goog.ui.tree.TreeControl', 'goog.ui.tree.TypeAhead'], false);
goog.addDependency('ui/tristatemenuitem.js', ['goog.ui.TriStateMenuItem', 'goog.ui.TriStateMenuItem.State'], ['goog.dom.classlist', 'goog.ui.Component', 'goog.ui.MenuItem', 'goog.ui.TriStateMenuItemRenderer', 'goog.ui.registry'], false);
goog.addDependency('ui/tristatemenuitemrenderer.js', ['goog.ui.TriStateMenuItemRenderer'], ['goog.asserts', 'goog.dom.classlist', 'goog.ui.MenuItemRenderer'], false);
goog.addDependency('ui/twothumbslider.js', ['goog.ui.TwoThumbSlider'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.dom', 'goog.ui.SliderBase'], false);
goog.addDependency('ui/twothumbslider_test.js', ['goog.ui.TwoThumbSliderTest'], ['goog.testing.jsunit', 'goog.ui.SliderBase', 'goog.ui.TwoThumbSlider'], false);
goog.addDependency('ui/zippy.js', ['goog.ui.Zippy', 'goog.ui.Zippy.Events', 'goog.ui.ZippyEvent'], ['goog.a11y.aria', 'goog.a11y.aria.Role', 'goog.a11y.aria.State', 'goog.dom', 'goog.dom.classlist', 'goog.events.Event', 'goog.events.EventHandler', 'goog.events.EventTarget', 'goog.events.EventType', 'goog.events.KeyCodes', 'goog.style'], false);
goog.addDependency('ui/zippy_test.js', ['goog.ui.ZippyTest'], ['goog.a11y.aria', 'goog.dom', 'goog.dom.classlist', 'goog.events', 'goog.object', 'goog.testing.events', 'goog.testing.jsunit', 'goog.ui.Zippy'], false);
goog.addDependency('uri/uri.js', ['goog.Uri', 'goog.Uri.QueryData'], ['goog.array', 'goog.string', 'goog.structs', 'goog.structs.Map', 'goog.uri.utils', 'goog.uri.utils.ComponentIndex', 'goog.uri.utils.StandardQueryParam'], false);
goog.addDependency('uri/uri_test.js', ['goog.UriTest'], ['goog.Uri', 'goog.testing.jsunit'], false);
goog.addDependency('uri/utils.js', ['goog.uri.utils', 'goog.uri.utils.ComponentIndex', 'goog.uri.utils.QueryArray', 'goog.uri.utils.QueryValue', 'goog.uri.utils.StandardQueryParam'], ['goog.asserts', 'goog.string', 'goog.userAgent'], false);
goog.addDependency('uri/utils_test.js', ['goog.uri.utilsTest'], ['goog.functions', 'goog.string', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.uri.utils'], false);
goog.addDependency('useragent/adobereader.js', ['goog.userAgent.adobeReader'], ['goog.string', 'goog.userAgent'], false);
goog.addDependency('useragent/adobereader_test.js', ['goog.userAgent.adobeReaderTest'], ['goog.testing.jsunit', 'goog.userAgent.adobeReader'], false);
goog.addDependency('useragent/flash.js', ['goog.userAgent.flash'], ['goog.string'], false);
goog.addDependency('useragent/flash_test.js', ['goog.userAgent.flashTest'], ['goog.testing.jsunit', 'goog.userAgent.flash'], false);
goog.addDependency('useragent/iphoto.js', ['goog.userAgent.iphoto'], ['goog.string', 'goog.userAgent'], false);
goog.addDependency('useragent/jscript.js', ['goog.userAgent.jscript'], ['goog.string'], false);
goog.addDependency('useragent/jscript_test.js', ['goog.userAgent.jscriptTest'], ['goog.testing.jsunit', 'goog.userAgent.jscript'], false);
goog.addDependency('useragent/keyboard.js', ['goog.userAgent.keyboard'], ['goog.userAgent', 'goog.userAgent.product'], false);
goog.addDependency('useragent/keyboard_test.js', ['goog.userAgent.keyboardTest'], ['goog.labs.userAgent.testAgents', 'goog.labs.userAgent.util', 'goog.testing.MockUserAgent', 'goog.testing.jsunit', 'goog.userAgent.keyboard', 'goog.userAgentTestUtil'], false);
goog.addDependency('useragent/picasa.js', ['goog.userAgent.picasa'], ['goog.string', 'goog.userAgent'], false);
goog.addDependency('useragent/platform.js', ['goog.userAgent.platform'], ['goog.userAgent'], false);
goog.addDependency('useragent/platform_test.js', ['goog.userAgent.platformTest'], ['goog.testing.MockUserAgent', 'goog.testing.jsunit', 'goog.userAgent', 'goog.userAgent.platform', 'goog.userAgentTestUtil'], false);
goog.addDependency('useragent/product.js', ['goog.userAgent.product'], ['goog.userAgent'], false);
goog.addDependency('useragent/product_isversion.js', ['goog.userAgent.product.isVersion'], ['goog.userAgent.product'], false);
goog.addDependency('useragent/product_test.js', ['goog.userAgent.productTest'], ['goog.array', 'goog.labs.userAgent.util', 'goog.testing.MockUserAgent', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.userAgent', 'goog.userAgent.product', 'goog.userAgent.product.isVersion', 'goog.userAgentTestUtil'], false);
goog.addDependency('useragent/useragent.js', ['goog.userAgent'], ['goog.labs.userAgent.browser', 'goog.labs.userAgent.engine', 'goog.labs.userAgent.util', 'goog.string'], false);
goog.addDependency('useragent/useragent_quirks_test.js', ['goog.userAgentQuirksTest'], ['goog.testing.jsunit', 'goog.userAgent'], false);
goog.addDependency('useragent/useragent_test.js', ['goog.userAgentTest'], ['goog.array', 'goog.labs.userAgent.testAgents', 'goog.labs.userAgent.util', 'goog.testing.PropertyReplacer', 'goog.testing.jsunit', 'goog.userAgent', 'goog.userAgentTestUtil'], false);
goog.addDependency('useragent/useragenttestutil.js', ['goog.userAgentTestUtil', 'goog.userAgentTestUtil.UserAgents'], ['goog.labs.userAgent.browser', 'goog.labs.userAgent.engine', 'goog.userAgent', 'goog.userAgent.keyboard', 'goog.userAgent.platform', 'goog.userAgent.product', 'goog.userAgent.product.isVersion'], false);
goog.addDependency('vec/float32array.js', ['goog.vec.Float32Array'], [], false);
goog.addDependency('vec/float64array.js', ['goog.vec.Float64Array'], [], false);
goog.addDependency('vec/mat3.js', ['goog.vec.Mat3'], ['goog.vec'], false);
goog.addDependency('vec/mat3d.js', ['goog.vec.mat3d', 'goog.vec.mat3d.Type'], ['goog.vec'], false);
goog.addDependency('vec/mat3f.js', ['goog.vec.mat3f', 'goog.vec.mat3f.Type'], ['goog.vec'], false);
goog.addDependency('vec/mat4.js', ['goog.vec.Mat4'], ['goog.vec', 'goog.vec.Vec3', 'goog.vec.Vec4'], false);
goog.addDependency('vec/mat4d.js', ['goog.vec.mat4d', 'goog.vec.mat4d.Type'], ['goog.vec', 'goog.vec.vec3d', 'goog.vec.vec4d'], false);
goog.addDependency('vec/mat4f.js', ['goog.vec.mat4f', 'goog.vec.mat4f.Type'], ['goog.vec', 'goog.vec.vec3f', 'goog.vec.vec4f'], false);
goog.addDependency('vec/matrix3.js', ['goog.vec.Matrix3'], [], false);
goog.addDependency('vec/matrix4.js', ['goog.vec.Matrix4'], ['goog.vec', 'goog.vec.Vec3', 'goog.vec.Vec4'], false);
goog.addDependency('vec/quaternion.js', ['goog.vec.Quaternion'], ['goog.vec', 'goog.vec.Vec3', 'goog.vec.Vec4'], false);
goog.addDependency('vec/ray.js', ['goog.vec.Ray'], ['goog.vec.Vec3'], false);
goog.addDependency('vec/vec.js', ['goog.vec', 'goog.vec.AnyType', 'goog.vec.ArrayType', 'goog.vec.Float32', 'goog.vec.Float64', 'goog.vec.Number'], ['goog.vec.Float32Array', 'goog.vec.Float64Array'], false);
goog.addDependency('vec/vec2.js', ['goog.vec.Vec2'], ['goog.vec'], false);
goog.addDependency('vec/vec2d.js', ['goog.vec.vec2d', 'goog.vec.vec2d.Type'], ['goog.vec'], false);
goog.addDependency('vec/vec2f.js', ['goog.vec.vec2f', 'goog.vec.vec2f.Type'], ['goog.vec'], false);
goog.addDependency('vec/vec3.js', ['goog.vec.Vec3'], ['goog.vec'], false);
goog.addDependency('vec/vec3d.js', ['goog.vec.vec3d', 'goog.vec.vec3d.Type'], ['goog.vec'], false);
goog.addDependency('vec/vec3f.js', ['goog.vec.vec3f', 'goog.vec.vec3f.Type'], ['goog.vec'], false);
goog.addDependency('vec/vec4.js', ['goog.vec.Vec4'], ['goog.vec'], false);
goog.addDependency('vec/vec4d.js', ['goog.vec.vec4d', 'goog.vec.vec4d.Type'], ['goog.vec'], false);
goog.addDependency('vec/vec4f.js', ['goog.vec.vec4f', 'goog.vec.vec4f.Type'], ['goog.vec'], false);
goog.addDependency('webgl/webgl.js', ['goog.webgl'], [], false);
goog.addDependency('window/window.js', ['goog.window'], ['goog.string', 'goog.userAgent'], false);
goog.addDependency('window/window_test.js', ['goog.windowTest'], ['goog.dom', 'goog.events', 'goog.string', 'goog.testing.AsyncTestCase', 'goog.testing.jsunit', 'goog.window'], false);

//javascript/closure/debug/error.js
// Copyright 2009 The Closure Library Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS-IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * @fileoverview Provides a base class for custom Error objects such that the
 * stack is correctly maintained.
 *
 * You should never need to throw goog.debug.Error(msg) directly, Error(msg) is
 * sufficient.
 *
 * @author pupius@google.com (Daniel Pupius)
 */

goog.provide('goog.debug.Error');



/**
 * Base class for custom error objects.
 * @param {*=} opt_msg The message associated with the error.
 * @constructor
 * @extends {Error}
 */
goog.debug.Error = function(opt_msg) {

  // Attempt to ensure there is a stack trace.
  if (Error.captureStackTrace) {
    Error.captureStackTrace(this, goog.debug.Error);
  } else {
    var stack = new Error().stack;
    if (stack) {
      this.stack = stack;
    }
  }

  if (opt_msg) {
    this.message = String(opt_msg);
  }
};
goog.inherits(goog.debug.Error, Error);


/** @override */
goog.debug.Error.prototype.name = 'CustomError';

//javascript/closure/dom/nodetype.js
// Copyright 2006 The Closure Library Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS-IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * @fileoverview Definition of goog.dom.NodeType.
 */

goog.provide('goog.dom.NodeType');


/**
 * Constants for the nodeType attribute in the Node interface.
 *
 * These constants match those specified in the Node interface. These are
 * usually present on the Node object in recent browsers, but not in older
 * browsers (specifically, early IEs) and thus are given here.
 *
 * In some browsers (early IEs), these are not defined on the Node object,
 * so they are provided here.
 *
 * See http://www.w3.org/TR/DOM-Level-2-Core/core.html#ID-1950641247
 * @enum {number}
 */
goog.dom.NodeType = {
  ELEMENT: 1,
  ATTRIBUTE: 2,
  TEXT: 3,
  CDATA_SECTION: 4,
  ENTITY_REFERENCE: 5,
  ENTITY: 6,
  PROCESSING_INSTRUCTION: 7,
  COMMENT: 8,
  DOCUMENT: 9,
  DOCUMENT_TYPE: 10,
  DOCUMENT_FRAGMENT: 11,
  NOTATION: 12
};

//javascript/closure/string/string.js
// Copyright 2006 The Closure Library Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS-IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * @fileoverview Utilities for string manipulation.
 * @author pupius@google.com (Daniel Pupius)
 * @author arv@google.com (Erik Arvidsson)
 */


/**
 * Namespace for string utilities
 */
goog.provide('goog.string');
goog.provide('goog.string.Unicode');


/**
 * @define {boolean} Enables HTML escaping of lowercase letter "e" which helps
 * with detection of double-escaping as this letter is frequently used.
 */
goog.define('goog.string.DETECT_DOUBLE_ESCAPING', false);


/**
 * Common Unicode string characters.
 * @enum {string}
 */
goog.string.Unicode = {
  NBSP: '\xa0'
};


/**
 * Fast prefix-checker.
 * @param {string} str The string to check.
 * @param {string} prefix A string to look for at the start of {@code str}.
 * @return {boolean} True if {@code str} begins with {@code prefix}.
 */
goog.string.startsWith = function(str, prefix) {
  return str.lastIndexOf(prefix, 0) == 0;
};


/**
 * Fast suffix-checker.
 * @param {string} str The string to check.
 * @param {string} suffix A string to look for at the end of {@code str}.
 * @return {boolean} True if {@code str} ends with {@code suffix}.
 */
goog.string.endsWith = function(str, suffix) {
  var l = str.length - suffix.length;
  return l >= 0 && str.indexOf(suffix, l) == l;
};


/**
 * Case-insensitive prefix-checker.
 * @param {string} str The string to check.
 * @param {string} prefix  A string to look for at the end of {@code str}.
 * @return {boolean} True if {@code str} begins with {@code prefix} (ignoring
 *     case).
 */
goog.string.caseInsensitiveStartsWith = function(str, prefix) {
  return goog.string.caseInsensitiveCompare(
      prefix, str.substr(0, prefix.length)) == 0;
};


/**
 * Case-insensitive suffix-checker.
 * @param {string} str The string to check.
 * @param {string} suffix A string to look for at the end of {@code str}.
 * @return {boolean} True if {@code str} ends with {@code suffix} (ignoring
 *     case).
 */
goog.string.caseInsensitiveEndsWith = function(str, suffix) {
  return goog.string.caseInsensitiveCompare(
      suffix, str.substr(str.length - suffix.length, suffix.length)) == 0;
};


/**
 * Case-insensitive equality checker.
 * @param {string} str1 First string to check.
 * @param {string} str2 Second string to check.
 * @return {boolean} True if {@code str1} and {@code str2} are the same string,
 *     ignoring case.
 */
goog.string.caseInsensitiveEquals = function(str1, str2) {
  return str1.toLowerCase() == str2.toLowerCase();
};


/**
 * Does simple python-style string substitution.
 * subs("foo%s hot%s", "bar", "dog") becomes "foobar hotdog".
 * @param {string} str The string containing the pattern.
 * @param {...*} var_args The items to substitute into the pattern.
 * @return {string} A copy of {@code str} in which each occurrence of
 *     {@code %s} has been replaced an argument from {@code var_args}.
 */
goog.string.subs = function(str, var_args) {
  var splitParts = str.split('%s');
  var returnString = '';

  var subsArguments = Array.prototype.slice.call(arguments, 1);
  while (subsArguments.length &&
         // Replace up to the last split part. We are inserting in the
         // positions between split parts.
         splitParts.length > 1) {
    returnString += splitParts.shift() + subsArguments.shift();
  }

  return returnString + splitParts.join('%s'); // Join unused '%s'
};


/**
 * Converts multiple whitespace chars (spaces, non-breaking-spaces, new lines
 * and tabs) to a single space, and strips leading and trailing whitespace.
 * @param {string} str Input string.
 * @return {string} A copy of {@code str} with collapsed whitespace.
 */
goog.string.collapseWhitespace = function(str) {
  // Since IE doesn't include non-breaking-space (0xa0) in their \s character
  // class (as required by section 7.2 of the ECMAScript spec), we explicitly
  // include it in the regexp to enforce consistent cross-browser behavior.
  return str.replace(/[\s\xa0]+/g, ' ').replace(/^\s+|\s+$/g, '');
};


/**
 * Checks if a string is empty or contains only whitespaces.
 * @param {string} str The string to check.
 * @return {boolean} Whether {@code str} is empty or whitespace only.
 */
goog.string.isEmptyOrWhitespace = function(str) {
  // testing length == 0 first is actually slower in all browsers (about the
  // same in Opera).
  // Since IE doesn't include non-breaking-space (0xa0) in their \s character
  // class (as required by section 7.2 of the ECMAScript spec), we explicitly
  // include it in the regexp to enforce consistent cross-browser behavior.
  return /^[\s\xa0]*$/.test(str);
};


/**
 * Checks if a string is empty.
 * @param {string} str The string to check.
 * @return {boolean} Whether {@code str} is empty.
 */
goog.string.isEmptyString = function(str) {
  return str.length == 0;
};


/**
 * Checks if a string is empty or contains only whitespaces.
 *
 * TODO(kenobi): Deprecate this when clients have been switched over to
 * goog.string.isEmptyOrWhitespace.
 *
 * @param {string} str The string to check.
 * @return {boolean} Whether {@code str} is empty or whitespace only.
 */
goog.string.isEmpty = goog.string.isEmptyOrWhitespace;


/**
 * Checks if a string is null, undefined, empty or contains only whitespaces.
 * @param {*} str The string to check.
 * @return {boolean} Whether {@code str} is null, undefined, empty, or
 *     whitespace only.
 */
goog.string.isEmptyOrWhitespaceSafe = function(str) {
  return goog.string.isEmptyOrWhitespace(goog.string.makeSafe(str));
};


/**
 * Checks if a string is null, undefined, empty or contains only whitespaces.
 *
 * TODO(kenobi): Deprecate this when clients have been switched over to
 * goog.string.isEmptyOrWhitespaceSafe.
 *
 * @param {*} str The string to check.
 * @return {boolean} Whether {@code str} is null, undefined, empty, or
 *     whitespace only.
 */
goog.string.isEmptySafe = goog.string.isEmptyOrWhitespaceSafe;


/**
 * Checks if a string is all breaking whitespace.
 * @param {string} str The string to check.
 * @return {boolean} Whether the string is all breaking whitespace.
 */
goog.string.isBreakingWhitespace = function(str) {
  return !/[^\t\n\r ]/.test(str);
};


/**
 * Checks if a string contains all letters.
 * @param {string} str string to check.
 * @return {boolean} True if {@code str} consists entirely of letters.
 */
goog.string.isAlpha = function(str) {
  return !/[^a-zA-Z]/.test(str);
};


/**
 * Checks if a string contains only numbers.
 * @param {*} str string to check. If not a string, it will be
 *     casted to one.
 * @return {boolean} True if {@code str} is numeric.
 */
goog.string.isNumeric = function(str) {
  return !/[^0-9]/.test(str);
};


/**
 * Checks if a string contains only numbers or letters.
 * @param {string} str string to check.
 * @return {boolean} True if {@code str} is alphanumeric.
 */
goog.string.isAlphaNumeric = function(str) {
  return !/[^a-zA-Z0-9]/.test(str);
};


/**
 * Checks if a character is a space character.
 * @param {string} ch Character to check.
 * @return {boolean} True if {code ch} is a space.
 */
goog.string.isSpace = function(ch) {
  return ch == ' ';
};


/**
 * Checks if a character is a valid unicode character.
 * @param {string} ch Character to check.
 * @return {boolean} True if {code ch} is a valid unicode character.
 */
goog.string.isUnicodeChar = function(ch) {
  return ch.length == 1 && ch >= ' ' && ch <= '~' ||
         ch >= '\u0080' && ch <= '\uFFFD';
};


/**
 * Takes a string and replaces newlines with a space. Multiple lines are
 * replaced with a single space.
 * @param {string} str The string from which to strip newlines.
 * @return {string} A copy of {@code str} stripped of newlines.
 */
goog.string.stripNewlines = function(str) {
  return str.replace(/(\r\n|\r|\n)+/g, ' ');
};


/**
 * Replaces Windows and Mac new lines with unix style: \r or \r\n with \n.
 * @param {string} str The string to in which to canonicalize newlines.
 * @return {string} {@code str} A copy of {@code} with canonicalized newlines.
 */
goog.string.canonicalizeNewlines = function(str) {
  return str.replace(/(\r\n|\r|\n)/g, '\n');
};


/**
 * Normalizes whitespace in a string, replacing all whitespace chars with
 * a space.
 * @param {string} str The string in which to normalize whitespace.
 * @return {string} A copy of {@code str} with all whitespace normalized.
 */
goog.string.normalizeWhitespace = function(str) {
  return str.replace(/\xa0|\s/g, ' ');
};


/**
 * Normalizes spaces in a string, replacing all consecutive spaces and tabs
 * with a single space. Replaces non-breaking space with a space.
 * @param {string} str The string in which to normalize spaces.
 * @return {string} A copy of {@code str} with all consecutive spaces and tabs
 *    replaced with a single space.
 */
goog.string.normalizeSpaces = function(str) {
  return str.replace(/\xa0|[ \t]+/g, ' ');
};


/**
 * Removes the breaking spaces from the left and right of the string and
 * collapses the sequences of breaking spaces in the middle into single spaces.
 * The original and the result strings render the same way in HTML.
 * @param {string} str A string in which to collapse spaces.
 * @return {string} Copy of the string with normalized breaking spaces.
 */
goog.string.collapseBreakingSpaces = function(str) {
  return str.replace(/[\t\r\n ]+/g, ' ').replace(
      /^[\t\r\n ]+|[\t\r\n ]+$/g, '');
};


/**
 * Trims white spaces to the left and right of a string.
 * @param {string} str The string to trim.
 * @return {string} A trimmed copy of {@code str}.
 */
goog.string.trim = (goog.TRUSTED_SITE && String.prototype.trim) ?
    function(str) {
      return str.trim();
    } :
    function(str) {
      // Since IE doesn't include non-breaking-space (0xa0) in their \s
      // character class (as required by section 7.2 of the ECMAScript spec),
      // we explicitly include it in the regexp to enforce consistent
      // cross-browser behavior.
      return str.replace(/^[\s\xa0]+|[\s\xa0]+$/g, '');
    };


/**
 * Trims whitespaces at the left end of a string.
 * @param {string} str The string to left trim.
 * @return {string} A trimmed copy of {@code str}.
 */
goog.string.trimLeft = function(str) {
  // Since IE doesn't include non-breaking-space (0xa0) in their \s character
  // class (as required by section 7.2 of the ECMAScript spec), we explicitly
  // include it in the regexp to enforce consistent cross-browser behavior.
  return str.replace(/^[\s\xa0]+/, '');
};


/**
 * Trims whitespaces at the right end of a string.
 * @param {string} str The string to right trim.
 * @return {string} A trimmed copy of {@code str}.
 */
goog.string.trimRight = function(str) {
  // Since IE doesn't include non-breaking-space (0xa0) in their \s character
  // class (as required by section 7.2 of the ECMAScript spec), we explicitly
  // include it in the regexp to enforce consistent cross-browser behavior.
  return str.replace(/[\s\xa0]+$/, '');
};


/**
 * A string comparator that ignores case.
 * -1 = str1 less than str2
 *  0 = str1 equals str2
 *  1 = str1 greater than str2
 *
 * @param {string} str1 The string to compare.
 * @param {string} str2 The string to compare {@code str1} to.
 * @return {number} The comparator result, as described above.
 */
goog.string.caseInsensitiveCompare = function(str1, str2) {
  var test1 = String(str1).toLowerCase();
  var test2 = String(str2).toLowerCase();

  if (test1 < test2) {
    return -1;
  } else if (test1 == test2) {
    return 0;
  } else {
    return 1;
  }
};


/**
 * Regular expression used for splitting a string into substrings of fractional
 * numbers, integers, and non-numeric characters.
 * @type {RegExp}
 * @private
 */
goog.string.numerateCompareRegExp_ = /(\.\d+)|(\d+)|(\D+)/g;


/**
 * String comparison function that handles numbers in a way humans might expect.
 * Using this function, the string "File 2.jpg" sorts before "File 10.jpg". The
 * comparison is mostly case-insensitive, though strings that are identical
 * except for case are sorted with the upper-case strings before lower-case.
 *
 * This comparison function is significantly slower (about 500x) than either
 * the default or the case-insensitive compare. It should not be used in
 * time-critical code, but should be fast enough to sort several hundred short
 * strings (like filenames) with a reasonable delay.
 *
 * @param {string} str1 The string to compare in a numerically sensitive way.
 * @param {string} str2 The string to compare {@code str1} to.
 * @return {number} less than 0 if str1 < str2, 0 if str1 == str2, greater than
 *     0 if str1 > str2.
 */
goog.string.numerateCompare = function(str1, str2) {
  if (str1 == str2) {
    return 0;
  }
  if (!str1) {
    return -1;
  }
  if (!str2) {
    return 1;
  }

  // Using match to split the entire string ahead of time turns out to be faster
  // for most inputs than using RegExp.exec or iterating over each character.
  var tokens1 = str1.toLowerCase().match(goog.string.numerateCompareRegExp_);
  var tokens2 = str2.toLowerCase().match(goog.string.numerateCompareRegExp_);

  var count = Math.min(tokens1.length, tokens2.length);

  for (var i = 0; i < count; i++) {
    var a = tokens1[i];
    var b = tokens2[i];

    // Compare pairs of tokens, returning if one token sorts before the other.
    if (a != b) {

      // Only if both tokens are integers is a special comparison required.
      // Decimal numbers are sorted as strings (e.g., '.09' < '.1').
      var num1 = parseInt(a, 10);
      if (!isNaN(num1)) {
        var num2 = parseInt(b, 10);
        if (!isNaN(num2) && num1 - num2) {
          return num1 - num2;
        }
      }
      return a < b ? -1 : 1;
    }
  }

  // If one string is a substring of the other, the shorter string sorts first.
  if (tokens1.length != tokens2.length) {
    return tokens1.length - tokens2.length;
  }

  // The two strings must be equivalent except for case (perfect equality is
  // tested at the head of the function.) Revert to default ASCII-betical string
  // comparison to stablize the sort.
  return str1 < str2 ? -1 : 1;
};


/**
 * URL-encodes a string
 * @param {*} str The string to url-encode.
 * @return {string} An encoded copy of {@code str} that is safe for urls.
 *     Note that '#', ':', and other characters used to delimit portions
 *     of URLs *will* be encoded.
 */
goog.string.urlEncode = function(str) {
  return encodeURIComponent(String(str));
};


/**
 * URL-decodes the string. We need to specially handle '+'s because
 * the javascript library doesn't convert them to spaces.
 * @param {string} str The string to url decode.
 * @return {string} The decoded {@code str}.
 */
goog.string.urlDecode = function(str) {
  return decodeURIComponent(str.replace(/\+/g, ' '));
};


/**
 * Converts \n to <br>s or <br />s.
 * @param {string} str The string in which to convert newlines.
 * @param {boolean=} opt_xml Whether to use XML compatible tags.
 * @return {string} A copy of {@code str} with converted newlines.
 */
goog.string.newLineToBr = function(str, opt_xml) {
  return str.replace(/(\r\n|\r|\n)/g, opt_xml ? '<br />' : '<br>');
};


/**
 * Escapes double quote '"' and single quote '\'' characters in addition to
 * '&', '<', and '>' so that a string can be included in an HTML tag attribute
 * value within double or single quotes.
 *
 * It should be noted that > doesn't need to be escaped for the HTML or XML to
 * be valid, but it has been decided to escape it for consistency with other
 * implementations.
 *
 * With goog.string.DETECT_DOUBLE_ESCAPING, this function escapes also the
 * lowercase letter "e".
 *
 * NOTE(pupius):
 * HtmlEscape is often called during the generation of large blocks of HTML.
 * Using statics for the regular expressions and strings is an optimization
 * that can more than half the amount of time IE spends in this function for
 * large apps, since strings and regexes both contribute to GC allocations.
 *
 * Testing for the presence of a character before escaping increases the number
 * of function calls, but actually provides a speed increase for the average
 * case -- since the average case often doesn't require the escaping of all 4
 * characters and indexOf() is much cheaper than replace().
 * The worst case does suffer slightly from the additional calls, therefore the
 * opt_isLikelyToContainHtmlChars option has been included for situations
 * where all 4 HTML entities are very likely to be present and need escaping.
 *
 * Some benchmarks (times tended to fluctuate +-0.05ms):
 *                                     FireFox                     IE6
 * (no chars / average (mix of cases) / all 4 chars)
 * no checks                     0.13 / 0.22 / 0.22         0.23 / 0.53 / 0.80
 * indexOf                       0.08 / 0.17 / 0.26         0.22 / 0.54 / 0.84
 * indexOf + re test             0.07 / 0.17 / 0.28         0.19 / 0.50 / 0.85
 *
 * An additional advantage of checking if replace actually needs to be called
 * is a reduction in the number of object allocations, so as the size of the
 * application grows the difference between the various methods would increase.
 *
 * @param {string} str string to be escaped.
 * @param {boolean=} opt_isLikelyToContainHtmlChars Don't perform a check to see
 *     if the character needs replacing - use this option if you expect each of
 *     the characters to appear often. Leave false if you expect few html
 *     characters to occur in your strings, such as if you are escaping HTML.
 * @return {string} An escaped copy of {@code str}.
 */
goog.string.htmlEscape = function(str, opt_isLikelyToContainHtmlChars) {

  if (opt_isLikelyToContainHtmlChars) {
    str = str.replace(goog.string.AMP_RE_, '&amp;')
          .replace(goog.string.LT_RE_, '&lt;')
          .replace(goog.string.GT_RE_, '&gt;')
          .replace(goog.string.QUOT_RE_, '&quot;')
          .replace(goog.string.SINGLE_QUOTE_RE_, '&#39;')
          .replace(goog.string.NULL_RE_, '&#0;');
    if (goog.string.DETECT_DOUBLE_ESCAPING) {
      str = str.replace(goog.string.E_RE_, '&#101;');
    }
    return str;

  } else {
    // quick test helps in the case when there are no chars to replace, in
    // worst case this makes barely a difference to the time taken
    if (!goog.string.ALL_RE_.test(str)) return str;

    // str.indexOf is faster than regex.test in this case
    if (str.indexOf('&') != -1) {
      str = str.replace(goog.string.AMP_RE_, '&amp;');
    }
    if (str.indexOf('<') != -1) {
      str = str.replace(goog.string.LT_RE_, '&lt;');
    }
    if (str.indexOf('>') != -1) {
      str = str.replace(goog.string.GT_RE_, '&gt;');
    }
    if (str.indexOf('"') != -1) {
      str = str.replace(goog.string.QUOT_RE_, '&quot;');
    }
    if (str.indexOf('\'') != -1) {
      str = str.replace(goog.string.SINGLE_QUOTE_RE_, '&#39;');
    }
    if (str.indexOf('\x00') != -1) {
      str = str.replace(goog.string.NULL_RE_, '&#0;');
    }
    if (goog.string.DETECT_DOUBLE_ESCAPING && str.indexOf('e') != -1) {
      str = str.replace(goog.string.E_RE_, '&#101;');
    }
    return str;
  }
};


/**
 * Regular expression that matches an ampersand, for use in escaping.
 * @const {!RegExp}
 * @private
 */
goog.string.AMP_RE_ = /&/g;


/**
 * Regular expression that matches a less than sign, for use in escaping.
 * @const {!RegExp}
 * @private
 */
goog.string.LT_RE_ = /</g;


/**
 * Regular expression that matches a greater than sign, for use in escaping.
 * @const {!RegExp}
 * @private
 */
goog.string.GT_RE_ = />/g;


/**
 * Regular expression that matches a double quote, for use in escaping.
 * @const {!RegExp}
 * @private
 */
goog.string.QUOT_RE_ = /"/g;


/**
 * Regular expression that matches a single quote, for use in escaping.
 * @const {!RegExp}
 * @private
 */
goog.string.SINGLE_QUOTE_RE_ = /'/g;


/**
 * Regular expression that matches null character, for use in escaping.
 * @const {!RegExp}
 * @private
 */
goog.string.NULL_RE_ = /\x00/g;


/**
 * Regular expression that matches a lowercase letter "e", for use in escaping.
 * @const {!RegExp}
 * @private
 */
goog.string.E_RE_ = /e/g;


/**
 * Regular expression that matches any character that needs to be escaped.
 * @const {!RegExp}
 * @private
 */
goog.string.ALL_RE_ = (goog.string.DETECT_DOUBLE_ESCAPING ?
    /[\x00&<>"'e]/ :
    /[\x00&<>"']/);


/**
 * Unescapes an HTML string.
 *
 * @param {string} str The string to unescape.
 * @return {string} An unescaped copy of {@code str}.
 */
goog.string.unescapeEntities = function(str) {
  if (goog.string.contains(str, '&')) {
    // We are careful not to use a DOM if we do not have one. We use the []
    // notation so that the JSCompiler will not complain about these objects and
    // fields in the case where we have no DOM.
    if ('document' in goog.global) {
      return goog.string.unescapeEntitiesUsingDom_(str);
    } else {
      // Fall back on pure XML entities
      return goog.string.unescapePureXmlEntities_(str);
    }
  }
  return str;
};


/**
 * Unescapes a HTML string using the provided document.
 *
 * @param {string} str The string to unescape.
 * @param {!Document} document A document to use in escaping the string.
 * @return {string} An unescaped copy of {@code str}.
 */
goog.string.unescapeEntitiesWithDocument = function(str, document) {
  if (goog.string.contains(str, '&')) {
    return goog.string.unescapeEntitiesUsingDom_(str, document);
  }
  return str;
};


/**
 * Unescapes an HTML string using a DOM to resolve non-XML, non-numeric
 * entities. This function is XSS-safe and whitespace-preserving.
 * @private
 * @param {string} str The string to unescape.
 * @param {Document=} opt_document An optional document to use for creating
 *     elements. If this is not specified then the default window.document
 *     will be used.
 * @return {string} The unescaped {@code str} string.
 */
goog.string.unescapeEntitiesUsingDom_ = function(str, opt_document) {
  var seen = {'&amp;': '&', '&lt;': '<', '&gt;': '>', '&quot;': '"'};
  var div;
  if (opt_document) {
    div = opt_document.createElement('div');
  } else {
    div = goog.global.document.createElement('div');
  }
  // Match as many valid entity characters as possible. If the actual entity
  // happens to be shorter, it will still work as innerHTML will return the
  // trailing characters unchanged. Since the entity characters do not include
  // open angle bracket, there is no chance of XSS from the innerHTML use.
  // Since no whitespace is passed to innerHTML, whitespace is preserved.
  return str.replace(goog.string.HTML_ENTITY_PATTERN_, function(s, entity) {
    // Check for cached entity.
    var value = seen[s];
    if (value) {
      return value;
    }
    // Check for numeric entity.
    if (entity.charAt(0) == '#') {
      // Prefix with 0 so that hex entities (e.g. &#x10) parse as hex numbers.
      var n = Number('0' + entity.substr(1));
      if (!isNaN(n)) {
        value = String.fromCharCode(n);
      }
    }
    // Fall back to innerHTML otherwise.
    if (!value) {
      // Append a non-entity character to avoid a bug in Webkit that parses
      // an invalid entity at the end of innerHTML text as the empty string.
      div.innerHTML = s + ' ';
      // Then remove the trailing character from the result.
      value = div.firstChild.nodeValue.slice(0, -1);
    }
    // Cache and return.
    return seen[s] = value;
  });
};


/**
 * Unescapes XML entities.
 * @private
 * @param {string} str The string to unescape.
 * @return {string} An unescaped copy of {@code str}.
 */
goog.string.unescapePureXmlEntities_ = function(str) {
  return str.replace(/&([^;]+);/g, function(s, entity) {
    switch (entity) {
      case 'amp':
        return '&';
      case 'lt':
        return '<';
      case 'gt':
        return '>';
      case 'quot':
        return '"';
      default:
        if (entity.charAt(0) == '#') {
          // Prefix with 0 so that hex entities (e.g. &#x10) parse as hex.
          var n = Number('0' + entity.substr(1));
          if (!isNaN(n)) {
            return String.fromCharCode(n);
          }
        }
        // For invalid entities we just return the entity
        return s;
    }
  });
};


/**
 * Regular expression that matches an HTML entity.
 * See also HTML5: Tokenization / Tokenizing character references.
 * @private
 * @type {!RegExp}
 */
goog.string.HTML_ENTITY_PATTERN_ = /&([^;\s<&]+);?/g;


/**
 * Do escaping of whitespace to preserve spatial formatting. We use character
 * entity #160 to make it safer for xml.
 * @param {string} str The string in which to escape whitespace.
 * @param {boolean=} opt_xml Whether to use XML compatible tags.
 * @return {string} An escaped copy of {@code str}.
 */
goog.string.whitespaceEscape = function(str, opt_xml) {
  // This doesn't use goog.string.preserveSpaces for backwards compatibility.
  return goog.string.newLineToBr(str.replace(/  /g, ' &#160;'), opt_xml);
};


/**
 * Preserve spaces that would be otherwise collapsed in HTML by replacing them
 * with non-breaking space Unicode characters.
 * @param {string} str The string in which to preserve whitespace.
 * @return {string} A copy of {@code str} with preserved whitespace.
 */
goog.string.preserveSpaces = function(str) {
  return str.replace(/(^|[\n ]) /g, '$1' + goog.string.Unicode.NBSP);
};


/**
 * Strip quote characters around a string.  The second argument is a string of
 * characters to treat as quotes.  This can be a single character or a string of
 * multiple character and in that case each of those are treated as possible
 * quote characters. For example:
 *
 * <pre>
 * goog.string.stripQuotes('"abc"', '"`') --> 'abc'
 * goog.string.stripQuotes('`abc`', '"`') --> 'abc'
 * </pre>
 *
 * @param {string} str The string to strip.
 * @param {string} quoteChars The quote characters to strip.
 * @return {string} A copy of {@code str} without the quotes.
 */
goog.string.stripQuotes = function(str, quoteChars) {
  var length = quoteChars.length;
  for (var i = 0; i < length; i++) {
    var quoteChar = length == 1 ? quoteChars : quoteChars.charAt(i);
    if (str.charAt(0) == quoteChar && str.charAt(str.length - 1) == quoteChar) {
      return str.substring(1, str.length - 1);
    }
  }
  return str;
};


/**
 * Truncates a string to a certain length and adds '...' if necessary.  The
 * length also accounts for the ellipsis, so a maximum length of 10 and a string
 * 'Hello World!' produces 'Hello W...'.
 * @param {string} str The string to truncate.
 * @param {number} chars Max number of characters.
 * @param {boolean=} opt_protectEscapedCharacters Whether to protect escaped
 *     characters from being cut off in the middle.
 * @return {string} The truncated {@code str} string.
 */
goog.string.truncate = function(str, chars, opt_protectEscapedCharacters) {
  if (opt_protectEscapedCharacters) {
    str = goog.string.unescapeEntities(str);
  }

  if (str.length > chars) {
    str = str.substring(0, chars - 3) + '...';
  }

  if (opt_protectEscapedCharacters) {
    str = goog.string.htmlEscape(str);
  }

  return str;
};


/**
 * Truncate a string in the middle, adding "..." if necessary,
 * and favoring the beginning of the string.
 * @param {string} str The string to truncate the middle of.
 * @param {number} chars Max number of characters.
 * @param {boolean=} opt_protectEscapedCharacters Whether to protect escaped
 *     characters from being cutoff in the middle.
 * @param {number=} opt_trailingChars Optional number of trailing characters to
 *     leave at the end of the string, instead of truncating as close to the
 *     middle as possible.
 * @return {string} A truncated copy of {@code str}.
 */
goog.string.truncateMiddle = function(str, chars,
    opt_protectEscapedCharacters, opt_trailingChars) {
  if (opt_protectEscapedCharacters) {
    str = goog.string.unescapeEntities(str);
  }

  if (opt_trailingChars && str.length > chars) {
    if (opt_trailingChars > chars) {
      opt_trailingChars = chars;
    }
    var endPoint = str.length - opt_trailingChars;
    var startPoint = chars - opt_trailingChars;
    str = str.substring(0, startPoint) + '...' + str.substring(endPoint);
  } else if (str.length > chars) {
    // Favor the beginning of the string:
    var half = Math.floor(chars / 2);
    var endPos = str.length - half;
    half += chars % 2;
    str = str.substring(0, half) + '...' + str.substring(endPos);
  }

  if (opt_protectEscapedCharacters) {
    str = goog.string.htmlEscape(str);
  }

  return str;
};


/**
 * Special chars that need to be escaped for goog.string.quote.
 * @private
 * @type {Object}
 */
goog.string.specialEscapeChars_ = {
  '\0': '\\0',
  '\b': '\\b',
  '\f': '\\f',
  '\n': '\\n',
  '\r': '\\r',
  '\t': '\\t',
  '\x0B': '\\x0B', // '\v' is not supported in JScript
  '"': '\\"',
  '\\': '\\\\'
};


/**
 * Character mappings used internally for goog.string.escapeChar.
 * @private
 * @type {Object}
 */
goog.string.jsEscapeCache_ = {
  '\'': '\\\''
};


/**
 * Encloses a string in double quotes and escapes characters so that the
 * string is a valid JS string.
 * @param {string} s The string to quote.
 * @return {string} A copy of {@code s} surrounded by double quotes.
 */
goog.string.quote = function(s) {
  s = String(s);
  if (s.quote) {
    return s.quote();
  } else {
    var sb = ['"'];
    for (var i = 0; i < s.length; i++) {
      var ch = s.charAt(i);
      var cc = ch.charCodeAt(0);
      sb[i + 1] = goog.string.specialEscapeChars_[ch] ||
          ((cc > 31 && cc < 127) ? ch : goog.string.escapeChar(ch));
    }
    sb.push('"');
    return sb.join('');
  }
};


/**
 * Takes a string and returns the escaped string for that character.
 * @param {string} str The string to escape.
 * @return {string} An escaped string representing {@code str}.
 */
goog.string.escapeString = function(str) {
  var sb = [];
  for (var i = 0; i < str.length; i++) {
    sb[i] = goog.string.escapeChar(str.charAt(i));
  }
  return sb.join('');
};


/**
 * Takes a character and returns the escaped string for that character. For
 * example escapeChar(String.fromCharCode(15)) -> "\\x0E".
 * @param {string} c The character to escape.
 * @return {string} An escaped string representing {@code c}.
 */
goog.string.escapeChar = function(c) {
  if (c in goog.string.jsEscapeCache_) {
    return goog.string.jsEscapeCache_[c];
  }

  if (c in goog.string.specialEscapeChars_) {
    return goog.string.jsEscapeCache_[c] = goog.string.specialEscapeChars_[c];
  }

  var rv = c;
  var cc = c.charCodeAt(0);
  if (cc > 31 && cc < 127) {
    rv = c;
  } else {
    // tab is 9 but handled above
    if (cc < 256) {
      rv = '\\x';
      if (cc < 16 || cc > 256) {
        rv += '0';
      }
    } else {
      rv = '\\u';
      if (cc < 4096) { // \u1000
        rv += '0';
      }
    }
    rv += cc.toString(16).toUpperCase();
  }

  return goog.string.jsEscapeCache_[c] = rv;
};


/**
 * Determines whether a string contains a substring.
 * @param {string} str The string to search.
 * @param {string} subString The substring to search for.
 * @return {boolean} Whether {@code str} contains {@code subString}.
 */
goog.string.contains = function(str, subString) {
  return str.indexOf(subString) != -1;
};


/**
 * Determines whether a string contains a substring, ignoring case.
 * @param {string} str The string to search.
 * @param {string} subString The substring to search for.
 * @return {boolean} Whether {@code str} contains {@code subString}.
 */
goog.string.caseInsensitiveContains = function(str, subString) {
  return goog.string.contains(str.toLowerCase(), subString.toLowerCase());
};


/**
 * Returns the non-overlapping occurrences of ss in s.
 * If either s or ss evalutes to false, then returns zero.
 * @param {string} s The string to look in.
 * @param {string} ss The string to look for.
 * @return {number} Number of occurrences of ss in s.
 */
goog.string.countOf = function(s, ss) {
  return s && ss ? s.split(ss).length - 1 : 0;
};


/**
 * Removes a substring of a specified length at a specific
 * index in a string.
 * @param {string} s The base string from which to remove.
 * @param {number} index The index at which to remove the substring.
 * @param {number} stringLength The length of the substring to remove.
 * @return {string} A copy of {@code s} with the substring removed or the full
 *     string if nothing is removed or the input is invalid.
 */
goog.string.removeAt = function(s, index, stringLength) {
  var resultStr = s;
  // If the index is greater or equal to 0 then remove substring
  if (index >= 0 && index < s.length && stringLength > 0) {
    resultStr = s.substr(0, index) +
        s.substr(index + stringLength, s.length - index - stringLength);
  }
  return resultStr;
};


/**
 *  Removes the first occurrence of a substring from a string.
 *  @param {string} s The base string from which to remove.
 *  @param {string} ss The string to remove.
 *  @return {string} A copy of {@code s} with {@code ss} removed or the full
 *      string if nothing is removed.
 */
goog.string.remove = function(s, ss) {
  var re = new RegExp(goog.string.regExpEscape(ss), '');
  return s.replace(re, '');
};


/**
 *  Removes all occurrences of a substring from a string.
 *  @param {string} s The base string from which to remove.
 *  @param {string} ss The string to remove.
 *  @return {string} A copy of {@code s} with {@code ss} removed or the full
 *      string if nothing is removed.
 */
goog.string.removeAll = function(s, ss) {
  var re = new RegExp(goog.string.regExpEscape(ss), 'g');
  return s.replace(re, '');
};


/**
 * Escapes characters in the string that are not safe to use in a RegExp.
 * @param {*} s The string to escape. If not a string, it will be casted
 *     to one.
 * @return {string} A RegExp safe, escaped copy of {@code s}.
 */
goog.string.regExpEscape = function(s) {
  return String(s).replace(/([-()\[\]{}+?*.$\^|,:#<!\\])/g, '\\$1').
      replace(/\x08/g, '\\x08');
};


/**
 * Repeats a string n times.
 * @param {string} string The string to repeat.
 * @param {number} length The number of times to repeat.
 * @return {string} A string containing {@code length} repetitions of
 *     {@code string}.
 */
goog.string.repeat = function(string, length) {
  return new Array(length + 1).join(string);
};


/**
 * Pads number to given length and optionally rounds it to a given precision.
 * For example:
 * <pre>padNumber(1.25, 2, 3) -> '01.250'
 * padNumber(1.25, 2) -> '01.25'
 * padNumber(1.25, 2, 1) -> '01.3'
 * padNumber(1.25, 0) -> '1.25'</pre>
 *
 * @param {number} num The number to pad.
 * @param {number} length The desired length.
 * @param {number=} opt_precision The desired precision.
 * @return {string} {@code num} as a string with the given options.
 */
goog.string.padNumber = function(num, length, opt_precision) {
  var s = goog.isDef(opt_precision) ? num.toFixed(opt_precision) : String(num);
  var index = s.indexOf('.');
  if (index == -1) {
    index = s.length;
  }
  return goog.string.repeat('0', Math.max(0, length - index)) + s;
};


/**
 * Returns a string representation of the given object, with
 * null and undefined being returned as the empty string.
 *
 * @param {*} obj The object to convert.
 * @return {string} A string representation of the {@code obj}.
 */
goog.string.makeSafe = function(obj) {
  return obj == null ? '' : String(obj);
};


/**
 * Concatenates string expressions. This is useful
 * since some browsers are very inefficient when it comes to using plus to
 * concat strings. Be careful when using null and undefined here since
 * these will not be included in the result. If you need to represent these
 * be sure to cast the argument to a String first.
 * For example:
 * <pre>buildString('a', 'b', 'c', 'd') -> 'abcd'
 * buildString(null, undefined) -> ''
 * </pre>
 * @param {...*} var_args A list of strings to concatenate. If not a string,
 *     it will be casted to one.
 * @return {string} The concatenation of {@code var_args}.
 */
goog.string.buildString = function(var_args) {
  return Array.prototype.join.call(arguments, '');
};


/**
 * Returns a string with at least 64-bits of randomness.
 *
 * Doesn't trust Javascript's random function entirely. Uses a combination of
 * random and current timestamp, and then encodes the string in base-36 to
 * make it shorter.
 *
 * @return {string} A random string, e.g. sn1s7vb4gcic.
 */
goog.string.getRandomString = function() {
  var x = 2147483648;
  return Math.floor(Math.random() * x).toString(36) +
         Math.abs(Math.floor(Math.random() * x) ^ goog.now()).toString(36);
};


/**
 * Compares two version numbers.
 *
 * @param {string|number} version1 Version of first item.
 * @param {string|number} version2 Version of second item.
 *
 * @return {number}  1 if {@code version1} is higher.
 *                   0 if arguments are equal.
 *                  -1 if {@code version2} is higher.
 */
goog.string.compareVersions = function(version1, version2) {
  var order = 0;
  // Trim leading and trailing whitespace and split the versions into
  // subversions.
  var v1Subs = goog.string.trim(String(version1)).split('.');
  var v2Subs = goog.string.trim(String(version2)).split('.');
  var subCount = Math.max(v1Subs.length, v2Subs.length);

  // Iterate over the subversions, as long as they appear to be equivalent.
  for (var subIdx = 0; order == 0 && subIdx < subCount; subIdx++) {
    var v1Sub = v1Subs[subIdx] || '';
    var v2Sub = v2Subs[subIdx] || '';

    // Split the subversions into pairs of numbers and qualifiers (like 'b').
    // Two different RegExp objects are needed because they are both using
    // the 'g' flag.
    var v1CompParser = new RegExp('(\\d*)(\\D*)', 'g');
    var v2CompParser = new RegExp('(\\d*)(\\D*)', 'g');
    do {
      var v1Comp = v1CompParser.exec(v1Sub) || ['', '', ''];
      var v2Comp = v2CompParser.exec(v2Sub) || ['', '', ''];
      // Break if there are no more matches.
      if (v1Comp[0].length == 0 && v2Comp[0].length == 0) {
        break;
      }

      // Parse the numeric part of the subversion. A missing number is
      // equivalent to 0.
      var v1CompNum = v1Comp[1].length == 0 ? 0 : parseInt(v1Comp[1], 10);
      var v2CompNum = v2Comp[1].length == 0 ? 0 : parseInt(v2Comp[1], 10);

      // Compare the subversion components. The number has the highest
      // precedence. Next, if the numbers are equal, a subversion without any
      // qualifier is always higher than a subversion with any qualifier. Next,
      // the qualifiers are compared as strings.
      order = goog.string.compareElements_(v1CompNum, v2CompNum) ||
          goog.string.compareElements_(v1Comp[2].length == 0,
              v2Comp[2].length == 0) ||
          goog.string.compareElements_(v1Comp[2], v2Comp[2]);
      // Stop as soon as an inequality is discovered.
    } while (order == 0);
  }

  return order;
};


/**
 * Compares elements of a version number.
 *
 * @param {string|number|boolean} left An element from a version number.
 * @param {string|number|boolean} right An element from a version number.
 *
 * @return {number}  1 if {@code left} is higher.
 *                   0 if arguments are equal.
 *                  -1 if {@code right} is higher.
 * @private
 */
goog.string.compareElements_ = function(left, right) {
  if (left < right) {
    return -1;
  } else if (left > right) {
    return 1;
  }
  return 0;
};


/**
 * Maximum value of #goog.string.hashCode, exclusive. 2^32.
 * @type {number}
 * @private
 */
goog.string.HASHCODE_MAX_ = 0x100000000;


/**
 * String hash function similar to java.lang.String.hashCode().
 * The hash code for a string is computed as
 * s[0] * 31 ^ (n - 1) + s[1] * 31 ^ (n - 2) + ... + s[n - 1],
 * where s[i] is the ith character of the string and n is the length of
 * the string. We mod the result to make it between 0 (inclusive) and 2^32
 * (exclusive).
 * @param {string} str A string.
 * @return {number} Hash value for {@code str}, between 0 (inclusive) and 2^32
 *  (exclusive). The empty string returns 0.
 */
goog.string.hashCode = function(str) {
  var result = 0;
  for (var i = 0; i < str.length; ++i) {
    result = 31 * result + str.charCodeAt(i);
    // Normalize to 4 byte range, 0 ... 2^32.
    result %= goog.string.HASHCODE_MAX_;
  }
  return result;
};


/**
 * The most recent unique ID. |0 is equivalent to Math.floor in this case.
 * @type {number}
 * @private
 */
goog.string.uniqueStringCounter_ = Math.random() * 0x80000000 | 0;


/**
 * Generates and returns a string which is unique in the current document.
 * This is useful, for example, to create unique IDs for DOM elements.
 * @return {string} A unique id.
 */
goog.string.createUniqueString = function() {
  return 'goog_' + goog.string.uniqueStringCounter_++;
};


/**
 * Converts the supplied string to a number, which may be Infinity or NaN.
 * This function strips whitespace: (toNumber(' 123') === 123)
 * This function accepts scientific notation: (toNumber('1e1') === 10)
 *
 * This is better than Javascript's built-in conversions because, sadly:
 *     (Number(' ') === 0) and (parseFloat('123a') === 123)
 *
 * @param {string} str The string to convert.
 * @return {number} The number the supplied string represents, or NaN.
 */
goog.string.toNumber = function(str) {
  var num = Number(str);
  if (num == 0 && goog.string.isEmpty(str)) {
    return NaN;
  }
  return num;
};


/**
 * Returns whether the given string is lower camel case (e.g. "isFooBar").
 *
 * Note that this assumes the string is entirely letters.
 * @see http://en.wikipedia.org/wiki/CamelCase#Variations_and_synonyms
 *
 * @param {string} str String to test.
 * @return {boolean} Whether the string is lower camel case.
 */
goog.string.isLowerCamelCase = function(str) {
  return /^[a-z]+([A-Z][a-z]*)*$/.test(str);
};


/**
 * Returns whether the given string is upper camel case (e.g. "FooBarBaz").
 *
 * Note that this assumes the string is entirely letters.
 * @see http://en.wikipedia.org/wiki/CamelCase#Variations_and_synonyms
 *
 * @param {string} str String to test.
 * @return {boolean} Whether the string is upper camel case.
 */
goog.string.isUpperCamelCase = function(str) {
  return /^([A-Z][a-z]*)+$/.test(str);
};


/**
 * Converts a string from selector-case to camelCase (e.g. from
 * "multi-part-string" to "multiPartString"), useful for converting
 * CSS selectors and HTML dataset keys to their equivalent JS properties.
 * @param {string} str The string in selector-case form.
 * @return {string} The string in camelCase form.
 */
goog.string.toCamelCase = function(str) {
  return String(str).replace(/\-([a-z])/g, function(all, match) {
    return match.toUpperCase();
  });
};


/**
 * Converts a string from camelCase to selector-case (e.g. from
 * "multiPartString" to "multi-part-string"), useful for converting JS
 * style and dataset properties to equivalent CSS selectors and HTML keys.
 * @param {string} str The string in camelCase form.
 * @return {string} The string in selector-case form.
 */
goog.string.toSelectorCase = function(str) {
  return String(str).replace(/([A-Z])/g, '-$1').toLowerCase();
};


/**
 * Converts a string into TitleCase. First character of the string is always
 * capitalized in addition to the first letter of every subsequent word.
 * Words are delimited by one or more whitespaces by default. Custom delimiters
 * can optionally be specified to replace the default, which doesn't preserve
 * whitespace delimiters and instead must be explicitly included if needed.
 *
 * Default delimiter => " ":
 *    goog.string.toTitleCase('oneTwoThree')    => 'OneTwoThree'
 *    goog.string.toTitleCase('one two three')  => 'One Two Three'
 *    goog.string.toTitleCase('  one   two   ') => '  One   Two   '
 *    goog.string.toTitleCase('one_two_three')  => 'One_two_three'
 *    goog.string.toTitleCase('one-two-three')  => 'One-two-three'
 *
 * Custom delimiter => "_-.":
 *    goog.string.toTitleCase('oneTwoThree', '_-.')       => 'OneTwoThree'
 *    goog.string.toTitleCase('one two three', '_-.')     => 'One two three'
 *    goog.string.toTitleCase('  one   two   ', '_-.')    => '  one   two   '
 *    goog.string.toTitleCase('one_two_three', '_-.')     => 'One_Two_Three'
 *    goog.string.toTitleCase('one-two-three', '_-.')     => 'One-Two-Three'
 *    goog.string.toTitleCase('one...two...three', '_-.') => 'One...Two...Three'
 *    goog.string.toTitleCase('one. two. three', '_-.')   => 'One. two. three'
 *    goog.string.toTitleCase('one-two.three', '_-.')     => 'One-Two.Three'
 *
 * @param {string} str String value in camelCase form.
 * @param {string=} opt_delimiters Custom delimiter character set used to
 *      distinguish words in the string value. Each character represents a
 *      single delimiter. When provided, default whitespace delimiter is
 *      overridden and must be explicitly included if needed.
 * @return {string} String value in TitleCase form.
 */
goog.string.toTitleCase = function(str, opt_delimiters) {
  var delimiters = goog.isString(opt_delimiters) ?
      goog.string.regExpEscape(opt_delimiters) : '\\s';

  // For IE8, we need to prevent using an empty character set. Otherwise,
  // incorrect matching will occur.
  delimiters = delimiters ? '|[' + delimiters + ']+' : '';

  var regexp = new RegExp('(^' + delimiters + ')([a-z])', 'g');
  return str.replace(regexp, function(all, p1, p2) {
    return p1 + p2.toUpperCase();
  });
};


/**
 * Parse a string in decimal or hexidecimal ('0xFFFF') form.
 *
 * To parse a particular radix, please use parseInt(string, radix) directly. See
 * https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/parseInt
 *
 * This is a wrapper for the built-in parseInt function that will only parse
 * numbers as base 10 or base 16.  Some JS implementations assume strings
 * starting with "0" are intended to be octal. ES3 allowed but discouraged
 * this behavior. ES5 forbids it.  This function emulates the ES5 behavior.
 *
 * For more information, see Mozilla JS Reference: http://goo.gl/8RiFj
 *
 * @param {string|number|null|undefined} value The value to be parsed.
 * @return {number} The number, parsed. If the string failed to parse, this
 *     will be NaN.
 */
goog.string.parseInt = function(value) {
  // Force finite numbers to strings.
  if (isFinite(value)) {
    value = String(value);
  }

  if (goog.isString(value)) {
    // If the string starts with '0x' or '-0x', parse as hex.
    return /^\s*-?0x/i.test(value) ?
        parseInt(value, 16) : parseInt(value, 10);
  }

  return NaN;
};


/**
 * Splits a string on a separator a limited number of times.
 *
 * This implementation is more similar to Python or Java, where the limit
 * parameter specifies the maximum number of splits rather than truncating
 * the number of results.
 *
 * See http://docs.python.org/2/library/stdtypes.html#str.split
 * See JavaDoc: http://goo.gl/F2AsY
 * See Mozilla reference: http://goo.gl/dZdZs
 *
 * @param {string} str String to split.
 * @param {string} separator The separator.
 * @param {number} limit The limit to the number of splits. The resulting array
 *     will have a maximum length of limit+1.  Negative numbers are the same
 *     as zero.
 * @return {!Array.<string>} The string, split.
 */

goog.string.splitLimit = function(str, separator, limit) {
  var parts = str.split(separator);
  var returnVal = [];

  // Only continue doing this while we haven't hit the limit and we have
  // parts left.
  while (limit > 0 && parts.length) {
    returnVal.push(parts.shift());
    limit--;
  }

  // If there are remaining parts, append them to the end.
  if (parts.length) {
    returnVal.push(parts.join(separator));
  }

  return returnVal;
};

//javascript/closure/asserts/asserts.js
// Copyright 2008 The Closure Library Authors. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS-IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**
 * @fileoverview Utilities to check the preconditions, postconditions and
 * invariants runtime.
 *
 * Methods in this package should be given special treatment by the compiler
 * for type-inference. For example, <code>goog.asserts.assert(foo)</code>
 * will restrict <code>foo</code> to a truthy value.
 *
 * The compiler has an option to disable asserts. So code like:
 * <code>
 * var x = goog.asserts.assert(foo()); goog.asserts.assert(bar());
 * </code>
 * will be transformed into:
 * <code>
 * var x = foo();
 * </code>
 * The compiler will leave in foo() (because its return value is used),
 * but it will remove bar() because it assumes it does not have side-effects.
 *
 * @author pallosp@google.com (Peter Pallos)
 * @author agrieve@google.com (Andrew Grieve)
 */

goog.provide('goog.asserts');
goog.provide('goog.asserts.AssertionError');

goog.require('goog.debug.Error');
goog.require('goog.dom.NodeType');
goog.require('goog.string');


/**
 * @define {boolean} Whether to strip out asserts or to leave them in.
 */
goog.define('goog.asserts.ENABLE_ASSERTS', goog.DEBUG);



/**
 * Error object for failed assertions.
 * @param {string} messagePattern The pattern that was used to form message.
 * @param {!Array.<*>} messageArgs The items to substitute into the pattern.
 * @constructor
 * @extends {goog.debug.Error}
 * @final
 */
goog.asserts.AssertionError = function(messagePattern, messageArgs) {
  messageArgs.unshift(messagePattern);
  goog.debug.Error.call(this, goog.string.subs.apply(null, messageArgs));
  // Remove the messagePattern afterwards to avoid permenantly modifying the
  // passed in array.
  messageArgs.shift();

  /**
   * The message pattern used to format the error message. Error handlers can
   * use this to uniquely identify the assertion.
   * @type {string}
   */
  this.messagePattern = messagePattern;
};
goog.inherits(goog.asserts.AssertionError, goog.debug.Error);


/** @override */
goog.asserts.AssertionError.prototype.name = 'AssertionError';


/**
 * The default error handler.
 * @param {!goog.asserts.AssertionError} e The exception to be handled.
 */
goog.asserts.DEFAULT_ERROR_HANDLER = function(e) { throw e; };


/**
 * The handler responsible for throwing or logging assertion errors.
 * @private {function(!goog.asserts.AssertionError)}
 */
goog.asserts.errorHandler_ = goog.asserts.DEFAULT_ERROR_HANDLER;


/**
 * Throws an exception with the given message and "Assertion failed" prefixed
 * onto it.
 * @param {string} defaultMessage The message to use if givenMessage is empty.
 * @param {Array.<*>} defaultArgs The substitution arguments for defaultMessage.
 * @param {string|undefined} givenMessage Message supplied by the caller.
 * @param {Array.<*>} givenArgs The substitution arguments for givenMessage.
 * @throws {goog.asserts.AssertionError} When the value is not a number.
 * @private
 */
goog.asserts.doAssertFailure_ =
    function(defaultMessage, defaultArgs, givenMessage, givenArgs) {
  var message = 'Assertion failed';
  if (givenMessage) {
    message += ': ' + givenMessage;
    var args = givenArgs;
  } else if (defaultMessage) {
    message += ': ' + defaultMessage;
    args = defaultArgs;
  }
  // The '' + works around an Opera 10 bug in the unit tests. Without it,
  // a stack trace is added to var message above. With this, a stack trace is
  // not added until this line (it causes the extra garbage to be added after
  // the assertion message instead of in the middle of it).
  var e = new goog.asserts.AssertionError('' + message, args || []);
  goog.asserts.errorHandler_(e);
};


/**
 * Sets a custom error handler that can be used to customize the behavior of
 * assertion failures, for example by turning all assertion failures into log
 * messages.
 * @param {function(goog.asserts.AssertionError)} errorHandler
 */
goog.asserts.setErrorHandler = function(errorHandler) {
  if (goog.asserts.ENABLE_ASSERTS) {
    goog.asserts.errorHandler_ = errorHandler;
  }
};


/**
 * Checks if the condition evaluates to true if goog.asserts.ENABLE_ASSERTS is
 * true.
 * @template T
 * @param {T} condition The condition to check.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @return {!T} The value of the condition.
 * @throws {goog.asserts.AssertionError} When the condition evaluates to false.
 */
goog.asserts.assert = function(condition, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS && !condition) {
    goog.asserts.doAssertFailure_('', null, opt_message,
        Array.prototype.slice.call(arguments, 2));
  }
  return condition;
};


/**
 * Fails if goog.asserts.ENABLE_ASSERTS is true. This function is useful in case
 * when we want to add a check in the unreachable area like switch-case
 * statement:
 *
 * <pre>
 *  switch(type) {
 *    case FOO: doSomething(); break;
 *    case BAR: doSomethingElse(); break;
 *    default: goog.assert.fail('Unrecognized type: ' + type);
 *      // We have only 2 types - "default:" section is unreachable code.
 *  }
 * </pre>
 *
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @throws {goog.asserts.AssertionError} Failure.
 */
goog.asserts.fail = function(opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS) {
    goog.asserts.errorHandler_(new goog.asserts.AssertionError(
        'Failure' + (opt_message ? ': ' + opt_message : ''),
        Array.prototype.slice.call(arguments, 1)));
  }
};


/**
 * Checks if the value is a number if goog.asserts.ENABLE_ASSERTS is true.
 * @param {*} value The value to check.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @return {number} The value, guaranteed to be a number when asserts enabled.
 * @throws {goog.asserts.AssertionError} When the value is not a number.
 */
goog.asserts.assertNumber = function(value, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS && !goog.isNumber(value)) {
    goog.asserts.doAssertFailure_('Expected number but got %s: %s.',
        [goog.typeOf(value), value], opt_message,
        Array.prototype.slice.call(arguments, 2));
  }
  return /** @type {number} */ (value);
};


/**
 * Checks if the value is a string if goog.asserts.ENABLE_ASSERTS is true.
 * @param {*} value The value to check.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @return {string} The value, guaranteed to be a string when asserts enabled.
 * @throws {goog.asserts.AssertionError} When the value is not a string.
 */
goog.asserts.assertString = function(value, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS && !goog.isString(value)) {
    goog.asserts.doAssertFailure_('Expected string but got %s: %s.',
        [goog.typeOf(value), value], opt_message,
        Array.prototype.slice.call(arguments, 2));
  }
  return /** @type {string} */ (value);
};


/**
 * Checks if the value is a function if goog.asserts.ENABLE_ASSERTS is true.
 * @param {*} value The value to check.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @return {!Function} The value, guaranteed to be a function when asserts
 *     enabled.
 * @throws {goog.asserts.AssertionError} When the value is not a function.
 */
goog.asserts.assertFunction = function(value, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS && !goog.isFunction(value)) {
    goog.asserts.doAssertFailure_('Expected function but got %s: %s.',
        [goog.typeOf(value), value], opt_message,
        Array.prototype.slice.call(arguments, 2));
  }
  return /** @type {!Function} */ (value);
};


/**
 * Checks if the value is an Object if goog.asserts.ENABLE_ASSERTS is true.
 * @param {*} value The value to check.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @return {!Object} The value, guaranteed to be a non-null object.
 * @throws {goog.asserts.AssertionError} When the value is not an object.
 */
goog.asserts.assertObject = function(value, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS && !goog.isObject(value)) {
    goog.asserts.doAssertFailure_('Expected object but got %s: %s.',
        [goog.typeOf(value), value],
        opt_message, Array.prototype.slice.call(arguments, 2));
  }
  return /** @type {!Object} */ (value);
};


/**
 * Checks if the value is an Array if goog.asserts.ENABLE_ASSERTS is true.
 * @param {*} value The value to check.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @return {!Array.<?>} The value, guaranteed to be a non-null array.
 * @throws {goog.asserts.AssertionError} When the value is not an array.
 */
goog.asserts.assertArray = function(value, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS && !goog.isArray(value)) {
    goog.asserts.doAssertFailure_('Expected array but got %s: %s.',
        [goog.typeOf(value), value], opt_message,
        Array.prototype.slice.call(arguments, 2));
  }
  return /** @type {!Array.<?>} */ (value);
};


/**
 * Checks if the value is a boolean if goog.asserts.ENABLE_ASSERTS is true.
 * @param {*} value The value to check.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @return {boolean} The value, guaranteed to be a boolean when asserts are
 *     enabled.
 * @throws {goog.asserts.AssertionError} When the value is not a boolean.
 */
goog.asserts.assertBoolean = function(value, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS && !goog.isBoolean(value)) {
    goog.asserts.doAssertFailure_('Expected boolean but got %s: %s.',
        [goog.typeOf(value), value], opt_message,
        Array.prototype.slice.call(arguments, 2));
  }
  return /** @type {boolean} */ (value);
};


/**
 * Checks if the value is a DOM Element if goog.asserts.ENABLE_ASSERTS is true.
 * @param {*} value The value to check.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @return {!Element} The value, likely to be a DOM Element when asserts are
 *     enabled.
 * @throws {goog.asserts.AssertionError} When the value is not a boolean.
 */
goog.asserts.assertElement = function(value, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS && (!goog.isObject(value) ||
      value.nodeType != goog.dom.NodeType.ELEMENT)) {
    goog.asserts.doAssertFailure_('Expected Element but got %s: %s.',
        [goog.typeOf(value), value], opt_message,
        Array.prototype.slice.call(arguments, 2));
  }
  return /** @type {!Element} */ (value);
};


/**
 * Checks if the value is an instance of the user-defined type if
 * goog.asserts.ENABLE_ASSERTS is true.
 *
 * The compiler may tighten the type returned by this function.
 *
 * @param {*} value The value to check.
 * @param {function(new: T, ...)} type A user-defined constructor.
 * @param {string=} opt_message Error message in case of failure.
 * @param {...*} var_args The items to substitute into the failure message.
 * @throws {goog.asserts.AssertionError} When the value is not an instance of
 *     type.
 * @return {!T}
 * @template T
 */
goog.asserts.assertInstanceof = function(value, type, opt_message, var_args) {
  if (goog.asserts.ENABLE_ASSERTS && !(value instanceof type)) {
    goog.asserts.doAssertFailure_('instanceof check failed.', null,
        opt_message, Array.prototype.slice.call(arguments, 3));
  }
  return value;
};


/**
 * Checks that no enumerable keys are present in Object.prototype. Such keys
 * would break most code that use {@code for (var ... in ...)} loops.
 */
goog.asserts.assertObjectPrototypeIsIntact = function() {
  for (var key in Object.prototype) {
    goog.asserts.fail(key + ' should not be enumerable in Object.prototype.');
  }
};

//java/com/google/net/rpc3/profiling/jendoscope/web/endo_util.js
// Copyright 2011 Google Inc. All Rights Reserved.

/**
 * @fileoverview This file is a grab-bag of bits and pieces that the Endoscope
 * console uses, mostly convenience classes for manipulating rectangles and
 * utilities for doing smooth transitions between views, colors, etcetera.
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('endo.Point');
goog.provide('endo.Range');
goog.provide('endo.Rect');
goog.provide('endo.util');

/**
 * @type {number}
 */
endo.util.INVALID_CYCLE = -1;

/**
 * @type {number}
 */
endo.util.INVALID_TIME = -1;

/**
 * @type {number}
 */
endo.util.INVALID_ID = -1;

/**
 * @type {string}
 */
endo.util.INVALID_TAG = '';

/**
 * Very simple hash function, produces a 32-bit hash value from a string key.
 * @param {string} key The string to hash.
 * @return {number} The hash of the key.
 */
endo.util.hash = function(key) {
  var result = 0;

  for (var i = 0; i < key.length; ++i) {
    result ^= (key.charCodeAt(i) & 0xff);
    result *= 0x5bd1e995;
    result &= 0xFFFFFFFF;
    result ^= result >> 15;
  }

  return result;
};

/**
 * Converts a HSV color to an RGB '#rrggbb' string.
 * Hue space is divided in to 6 sectors that correspond to red, yellow, green,
 * cyan, blue, and magenta. To convert from HSV space to RGB space, we determine
 * which sector of hue space we're in and then interpolate between the two ends
 * of that sector to get the primary hue in RGB space. We then scale and bias
 * the primary hue to get the final color in RGB space.
 * @param {number} hue Hue.
 * @param {number} saturation Saturation.
 * @param {number} value Value.
 * @return {string} The color string.
 */
endo.util.hsvToRgb = function(hue, saturation, value) {
    var r, g, b;

    // Wrap all hue values (positive and negative) to the range [0,1).
    hue = ((hue % 1) + 1) % 1;

    var i = Math.floor(hue * 6);
    var u = hue * 6 - i;
    var d = 1 - u;

    switch (i % 6) {
        case 0: r = 1, g = u, b = 0; break;
        case 1: r = d, g = 1, b = 0; break;
        case 2: r = 0, g = 1, b = u; break;
        case 3: r = 0, g = d, b = 1; break;
        case 4: r = u, g = 0, b = 1; break;
        case 5: r = 1, g = 0, b = d; break;
    }

    r *= saturation;
    g *= saturation;
    b *= saturation;

    r += (1 - saturation);
    g += (1 - saturation);
    b += (1 - saturation);

    r *= value;
    g *= value;
    b *= value;

    var rs = Math.round(r * 255).toString(16);
    var gs = Math.round(g * 255).toString(16);
    var bs = Math.round(b * 255).toString(16);

    if (rs.length == 1) rs = '0' + rs;
    if (gs.length == 1) gs = '0' + gs;
    if (bs.length == 1) bs = '0' + bs;

    return '#' + rs + gs + bs;
};

/**
 * Converts a hue + frequency into a RGB color that changes over time to give
 * the appearance of a pulsing highlight.
 *
 * @param {number} hue The HSV-space hue to convert to a pulsing color.
 * @param {number} frequency The frequency to pulse.
 * @return {string} The RGB color string.
 */
endo.util.pulseHue = function(hue, frequency) {
  var hsvDim = [hue, 0.45, 0.7];
  var hsvBright = [hue, 0.4, 1.0];
  return endo.util.pulseHSV(hsvDim, hsvBright, frequency);
};

/**
 * Interpolates between two [H,S,V] colors to give the appearance of a pulsing
 * highlight.
 *
 * @param {!Array.<number>} hsvA The first color to interpolate between.
 * @param {!Array.<number>} hsvB The second color to interpolate between.
 * @param {number} frequency The frequency to pulse.
 * @return {string} The RGB color string.
 */
endo.util.pulseHSV = function(hsvA, hsvB, frequency) {
  var time = ((new Date()).getTime() / 1000);
  var sin = Math.sin(time * frequency * 2 * Math.PI);

  var h = endo.util.remap(sin, -1, 1, hsvA[0], hsvB[0]);
  var s = endo.util.remap(sin, -1, 1, hsvA[1], hsvB[1]);
  var v = endo.util.remap(sin, -1, 1, hsvA[2], hsvB[2]);

  return endo.util.hsvToRgb(h, s, v);
};

/**
 * Simple 1D range class.
 * @param {number} min The minimum value in the range.
 * @param {number} max The maximum value in the range.
 * @constructor
 * @struct
 */
endo.Range = function(min, max) {
  this.min = min;
  this.max = max;
};

/**
 * Creates a new empty endo.Range object.
 *
 * @return {!endo.Range} The empty range.
 */
endo.Range.EmptyRange = function() {
  return new endo.Range(Infinity, -Infinity);
};

/**
 * Computes the range that encloses both this range and the given range.
 *
 * @param {!endo.Range} range The range to enclose.
 * @return {!endo.Range} The enclosing range.
 */
endo.Range.prototype.enclose = function(range) {
  var result = new endo.Range(this.min, this.max);

  result.min = Math.min(result.min, range.min);
  result.max = Math.max(result.max, range.max);

  return result;
};

/**
 * Clones this range.
 *
 * @return {!endo.Range} A clone of this range.
 */
endo.Range.prototype.clone = function() {
  return new endo.Range(this.min, this.max);
};

/**
 * Returns true if this range is empty - for the purposes of this system, an
 * empty range is defined as one that does not contain a 1-dimensional set of
 * points.
 *
 * @return {boolean} True if this range is empty.
 */
endo.Range.prototype.isEmpty = function() {
  return this.max <= this.min;
};

/**
 * Returns the width of this range.
 *
 * @return {number} The width of this range.
 */
endo.Range.prototype.getWidth = function() {
  return this.max - this.min;
};

/**
 * Clears this range.
 */
endo.Range.prototype.clear = function() {
  this.min = Infinity;
  this.max = -Infinity;
};

/**
 * Simple 2D point class.
 * @param {number} x The x coordinate of the point.
 * @param {number} y The y coordinate of the point.
 * @constructor
 * @struct
 */
endo.Point = function(x, y) {
  this.x = x;
  this.y = y;
};

/**
 * Sets the (x,y) coordinate of this point.
 *
 * @param {number} x The new x coordinate.
 * @param {number} y The new y coordinate.
 */
endo.Point.prototype.set = function(x, y) {
  this.x = x;
  this.y = y;
};

/**
 * Clones this point.
 * @return {!endo.Point} A clone of this point.
 */
endo.Point.prototype.clone = function() {
  return new endo.Point(this.x, this.y);
};

/**
 * Remap this point from the source rectangle to the dest rectangle.
 * @param {!endo.Rect} srcRect The containing rectangle in the source coordinate
 *     space.
 * @param {!endo.Rect} dstRect The corresponding rectangle in the destination
 *     coordinate space.
 * @return {!endo.Point} The remapped point.
 */
endo.Point.prototype.remap = function(srcRect, dstRect) {
  var result = this.clone();

  // Compute the point's location in normalized rectangle space
  result.x -= srcRect.x0;
  result.y -= srcRect.y0;

  result.x /= srcRect.getWidth();
  result.y /= srcRect.getHeight();

  // and then remap it into the space of dstRect.
  result.x *= dstRect.getWidth();
  result.y *= dstRect.getHeight();

  result.x += dstRect.x0;
  result.y += dstRect.y0;

  return result;
};

/**
 * Simple 2D rectangle class.
 * @param {number} x0 The minimum x coordinate of the rectangle.
 * @param {number} y0 The minimum y coordinate of the rectangle.
 * @param {number} x1 The maximum x coordinate of the rectangle.
 * @param {number} y1 The maximum y coordinate of the rectangle.
 * @constructor
 * @struct
 */
endo.Rect = function(x0, y0, x1, y1) {
  /**
   * @type {number}
   */
  this.x0 = x0;
  /**
   * @type {number}
   */
  this.y0 = y0;
  /**
   * @type {number}
   */
  this.x1 = x1;
  /**
   * @type {number}
   */
  this.y1 = y1;
};

/**
 * Sets (x,y) coordinates of the top left and bottom right of this rectangle.
 *
 * @param {number} x0 The left x coordinate.
 * @param {number} y0 The top y coordinate.
 * @param {number} x1 The right x coordinate.
 * @param {number} y1 The bottom y coordinate.
 */
endo.Rect.prototype.set = function(x0, y0, x1, y1) {
  /**
   * @type {number}
   */
  this.x0 = x0;
  /**
   * @type {number}
   */
  this.y0 = y0;
  /**
   * @type {number}
   */
  this.x1 = x1;
  /**
   * @type {number}
   */
  this.y1 = y1;
};

/**
 * Computes the width of the rectangle.
 * @return {number} The width of the rectangle.
 */
endo.Rect.prototype.getWidth = function() {
  return this.x1 - this.x0;
};

/**
 * Computes the height of the rectangle.
 * @return {number} The height of the rectangle.
 */
endo.Rect.prototype.getHeight = function() {
  return this.y1 - this.y0;
};

/**
 * Computes the x-axis center of the rectangle.
 * @return {number} The x coordinate of the center of the rectangle.
 */
endo.Rect.prototype.getCenterX = function() {
  return (this.x0 + this.x1) / 2;
};

/**
 * Computes the y-axis center of the rectangle.
 * @return {number} The y coordinate of the center of the rectangle.
 */
endo.Rect.prototype.getCenterY = function() {
  return (this.y0 + this.y1) / 2;
};

/**
 * Clones this rectangle.
 * @return {!endo.Rect} A clone of this rectangle.
 */
endo.Rect.prototype.clone = function() {
  return new endo.Rect(this.x0, this.y0, this.x1, this.y1);
};

/**
 * Clips this rectangle so that it lies inside the given rectangle.
 * @param {!endo.Rect} rect The rectangle to clip to.
 */
endo.Rect.prototype.clipTo = function(rect) {
  if (this.x0 < rect.x0) this.x0 = rect.x0;
  if (this.y0 < rect.y0) this.y0 = rect.y0;
  if (this.x1 > rect.x1) this.x1 = rect.x1;
  if (this.y1 > rect.y1) this.y1 = rect.y1;
};

/**
 * Returns true if this rectangle has an area of 0.
 * @return {boolean} True if this rectangle has an area of 0.
 */
endo.Rect.prototype.isEmpty = function() {
  if (this.x1 <= this.x0) return true;
  if (this.y1 <= this.y0) return true;
  return false;
};

/**
 * Returns true if this rectangle overlaps the given rectangle.
 * @param {!endo.Rect} rect The rectangle to check overlap against.
 * @return {boolean} True if this rectangle overlaps the given rectangle.
 */
endo.Rect.prototype.overlaps = function(rect) {
  if (this.x0 > rect.x1) return false;
  if (this.y0 > rect.y1) return false;
  if (this.x1 < rect.x0) return false;
  if (this.y1 < rect.y0) return false;
  return true;
};

/**
 * Returns true if this rectangle contains the given point. Points lying
 * exactly on the boundary of the rectangle are not considered contained.
 * @param {!endo.Point} point The point to test for containment.
 * @return {boolean} True if the rectangle contains the point.
 */
endo.Rect.prototype.contains = function(point) {
  if (this.x0 >= point.x) return false;
  if (this.x1 <= point.x) return false;
  if (this.y0 >= point.y) return false;
  if (this.y1 <= point.y) return false;
  return true;
};

/**
 * Translates this rectangle by the given (x,y) distances.
 * @param {number} x The amount to translate the rectangle along the x axis.
 * @param {number} y The amount to translate the rectangle along the y axis.
 */
endo.Rect.prototype.translate = function(x, y) {
  this.x0 += x;
  this.y0 += y;
  this.x1 += x;
  this.y1 += y;
};

/**
 * Scales this rectangle by the given (x,y) scale factors.
 * @param {number} x The amount to scale this rectangle along the x axis.
 * @param {number} y The amount to scale this rectangle along the y axis.
 */
endo.Rect.prototype.scale = function(x, y) {
  this.x0 *= x;
  this.y0 *= y;
  this.x1 *= x;
  this.y1 *= y;
};

/**
 * Expands the rectangle by the given amount in all directions.
 *
 * @param {number} distance The distance to expand.
 */
endo.Rect.prototype.expand = function(distance) {
  this.x0 -= distance;
  this.x1 += distance;
  this.y0 -= distance;
  this.y1 += distance;
};

/**
 * Contracts the rectangle by the given amount in all directions.
 *
 * @param {number} distance The distance to contract.
 */
endo.Rect.prototype.contract = function(distance) {
  this.x0 += distance;
  this.x1 -= distance;
  this.y0 += distance;
  this.y1 -= distance;
};

/**
 * Scales this rectangle by the given (x,y) scale factors, but do so relative
 * to the given point instead of the origin.
 * @param {number} x The amount to scale this rectangle along the x axis.
 * @param {number} y The amount to scale this rectangle along the y axis.
 * @param {!endo.Point} point The point we're scaling around.
 */
endo.Rect.prototype.scaleRelative = function(x, y, point) {
  this.translate(-point.x, -point.y);
  this.scale(x, y);
  this.translate(point.x, point.y);
};

/**
 * Transforms this rectangle from the coordinate space defined by 'srcRect' to
 * the coordinate space defined by 'dstRect'.
 *
 * @param {!endo.Rect} sourceRect The rectangle defining the source coordinate
 *     space to transform from.
 * @param {!endo.Rect} destRect The rectangle defining the destination
 *     coordinate space to transform to.
 */
endo.Rect.prototype.remap = function(sourceRect, destRect) {
  // Transform the rectangle to a normalized coordinate space,
  this.x0 -= sourceRect.x0;
  this.x1 -= sourceRect.x0;
  this.y0 -= sourceRect.y0;
  this.y1 -= sourceRect.y0;

  this.x0 /= sourceRect.getWidth();
  this.x1 /= sourceRect.getWidth();
  this.y0 /= sourceRect.getHeight();
  this.y1 /= sourceRect.getHeight();

  // and then remap it into the destination space.
  this.x0 *= destRect.getWidth();
  this.x1 *= destRect.getWidth();
  this.y0 *= destRect.getHeight();
  this.y1 *= destRect.getHeight();

  this.x0 += destRect.x0;
  this.x1 += destRect.x0;
  this.y0 += destRect.y0;
  this.y1 += destRect.y0;
};

/**
 * Swaps the min and max values on each axis of this rectangle so that x0 < x1
 * and y0 < y1 - this turns the rectangle "right side out", so to speak.
 */
endo.Rect.prototype.unflip = function() {
  if (this.x0 > this.x1) {
    var t = this.x0;
    this.x0 = this.x1;
    this.x1 = t;
  }

  if (this.y0 > this.y1) {
    var t = this.y0;
    this.y0 = this.y1;
    this.y1 = t;
  }
};

/**
 * Clamps x so that it lies in the range [min,max].
 * @param {number} x The value to clamp.
 * @param {number} min The lower bound of the range to clamp to.
 * @param {number} max The upper bound of the range to clamp to.
 * @return {number} The clamped value of x.
 */
endo.util.clamp = function(x, min, max) {
  if (x < min) return min;
  if (x > max) return max;
  return x;
};

/**
 * Remaps x from the range (min1,max1) to the range (min2,max2) with no
 * clamping or easing.
 * @param {number} x The value to remap.
 * @param {number} min1 The lower bound of the source range.
 * @param {number} max1 The upper bound of the source range.
 * @param {number} min2 The lower bound of the destination range.
 * @param {number} max2 The upper bound of the destination range.
 * @return {number} The remapped value.
 */
endo.util.remap = function(x, min1, max1, min2, max2) {
  x -= min1;
  x /= (max1 - min1);
  x *= (max2 - min2);
  x += min2;
  return x;
};

/**
 * Clamps x to the range (min1,max1) and then remap it to range (min2,max2)
 * with no easing.
 * @param {number} x The value to remap.
 * @param {number} min1 The lower bound of the source range.
 * @param {number} max1 The upper bound of the source range.
 * @param {number} min2 The lower bound of the destination range.
 * @param {number} max2 The upper bound of the destination range.
 * @return {number} The clamped and remapped value.
 */
endo.util.remapClamp = function(x, min1, max1, min2, max2) {
  if (x < min1) return min2;
  if (x > max1) return max2;
  return endo.util.remap(x, min1, max1, min2, max2);
};

/**
 * Remap x such that as x goes from min1 to max1, the output value goes smoothly
 * from min2 to max2 using a cubic ease-in-out curve.
 * @param {number} x The value to remap.
 * @param {number} min1 The lower bound of the source range.
 * @param {number} max1 The upper bound of the source range.
 * @param {number} min2 The lower bound of the destination range.
 * @param {number} max2 The upper bound of the destination range.
 * @return {number} The eased value.
 */
endo.util.remapEase = function(x, min1, max1, min2, max2) {
  if (x < min1) return min2;
  if (x > max1) return max2;

  x -= min1;
  x /= (max1 - min1);
  x = 3 * x * x - 2 * x * x * x;
  x *= (max2 - min2);
  x += min2;

  return x;
};

/**
 * Computes the value of X after a timestep of dt such that X smoothly
 * approaches the goal at a rate of 90% of the way there every 'duration'
 * seconds.
 * Note - In most cases, if X has never been updated before, then we want to
 * snap it to the goal value. Since we don't want to assume that a particular
 * value of X (such as 0) indicates that X hasn't been updated before, we
 * instead adopt the convention that if X is undefined, it gets snapped to the
 * goal automatically.
 * @param {number} x The value we are easing.
 * @param {number} goal The value we are easing towards.
 * @param {number} duration The time it takes the easing to reach 90% of the
 *     goal.
 * @param {number} delta The time interval we're advancing the easing by.
 * @return {number} The eased value.
 */
endo.util.temporalEase = function(x, goal, duration, delta) {
  if (typeof x == 'undefined') return goal;
  var scale = 1.0 - Math.pow(0.1, delta / duration);
  x += (goal - x) * scale;
  return x;
};

/**
 * Applies the above easing function to the corners of a rectangle.
 * @param {!endo.Rect} rect The rectangle we are easing.
 * @param {!endo.Rect} goal The rectangle we are easing towards.
 * @param {number} duration The time it takes the easing to reach 90% of the
 *     goal.
 * @param {number} delta The time interval we're advancing the easing by.
 */
endo.util.temporalEaseRect = function(rect, goal, duration, delta) {
  rect.x0 = endo.util.temporalEase(rect.x0, goal.x0, duration, delta);
  rect.x1 = endo.util.temporalEase(rect.x1, goal.x1, duration, delta);
  rect.y0 = endo.util.temporalEase(rect.y0, goal.y0, duration, delta);
  rect.y1 = endo.util.temporalEase(rect.y1, goal.y1, duration, delta);
};

/**
 * Remaps a time value in the range (timeMin,timeMax) to a screen x coordinate.
 * @param {number} time The time to remap.
 * @param {number} timeMin The minimum time in the time range.
 * @param {number} timeMax The maximum time in the time range.
 * @param {!HTMLCanvasElement} canvas The canvas to remap onto.
 * @return {number} The screen x coordinate.
 */
endo.util.timeToPixel = function(time, timeMin, timeMax, canvas) {
  return endo.util.remap(time, timeMin, timeMax, 0, canvas.width);
};

/**
 * Remaps a screen x coordinate to the time range (timeMin,timeMax).
 * @param {number} pixel The x coodinate in screen space to remap.
 * @param {!HTMLCanvasElement} canvas The canvas to remap onto.
 * @param {number} timeMin The minimum time in the time range.
 * @param {number} timeMax The maximum time in the time range.
 * @return {number} The result in time-space.
 */
endo.util.pixelToTime = function(pixel, canvas, timeMin, timeMax) {
  return endo.util.remap(pixel, 0, canvas.width, timeMin, timeMax);
};

/**
 * Converts a numerical duration in seconds into a string with a suffix that
 * indicates order of magnitude -
 * 'n' -> nanoseconds
 * 'u' -> microseconds
 * 'm' -> milliseconds
 * 's' -> seconds
 * @param {number} duration The time in seconds to prettyprint.
 * @return {string} The prettyprinted duration.
 */
endo.util.prettyprintDuration = function(duration) {
  var suffix = 's';

  if (duration < 1.0) {
    duration *= 1000;
    suffix = 'm';
    if (duration < 1.0) {
      duration *= 1000;
      suffix = 'u';
      if (duration < 1.0) {
        duration *= 1000;
        suffix = 'n';
      }
    }
  }

  return duration.toFixed(2) + suffix;
};

/**
 * Converts a numerical unit into a string with a suffix that indicates order
 * of magnitude -
 * 'n' -> nano
 * 'u' -> micro
 * 'm' -> milli
 * 'K' -> kilo
 * 'M' -> mega
 * 'G' -> giga
 * @param {number} unit The generic unit to prettyprint.
 * @return {string} The prettyprinted unit.
 */
endo.util.prettyprintUnit = function(unit) {
  var suffix = '';
  if (unit < 1.0) {
    unit *= 1000;
    suffix = 'm';
    if (unit < 1.0) {
      unit *= 1000;
      suffix = 'u';
      if (unit < 1.0) {
        unit *= 1000;
        suffix = 'n';
      }
    }
  } else {
    if (unit >= 1000) {
      unit /= 1000;
      suffix = 'K';
      if (unit >= 1000) {
        unit /= 1000;
        suffix = 'M';
        if (unit >= 1000) {
          unit /= 1000;
          suffix = 'G';
        }
      }
    }
  }

  return unit.toFixed(2) + suffix;
};

/**
 * Checks for approximate floating-point equality - two values are considered
 * almost-equal if they're within 0.01% of each other.
 *
 * @param {number} a The first number to check for approximate equality.
 * @param {number} b The second number to check for approximate equality.
 * @return {boolean} True if the two numbers were within 0.01% of each other.
 */
endo.util.almostEqual = function(a, b) {
  if (a == b) return true;

  if (a == 0) return (b == 0);
  if (b == 0) return (a == 0);

  if (a < 0) {
    a = -a;
    b = -b;
  }
  if (b < 0) return false;

  if (a > b) {
    return (a / b) < 1.0001;
  } else {
    return (b / a) < 1.0001;
  }
};

//java/com/google/net/rpc3/profiling/jendoscope/web/endo_atom.js
/**
 * @fileoverview This atom class captures the concept of an atom
 *
 * An atom may represent one of these: subtask start, subtask end, event,
 * tag. It is synthesized because JEndoscope doesn't have this concept.
 * However, such a structure is expected by javascript logic that turns
 * protobuf into internal presentation.
 */

goog.provide('endo.Atom');

goog.require('endo.util');

/**
 * Creates a new, empty Endo atom.
 * @constructor
 * @struct
 */
endo.Atom = function() {
  /**
   * @type {endo.Atom.AtomType}
   */
  this.type = endo.Atom.AtomType.INVALID;
  /**
   * @type {number}
   */
  this.time = 0;
  /**
   * @type {number}
   */
  this.markerId = endo.util.INVALID_ID;
  /**
   * @type {string}
   */
  this.tag = endo.util.INVALID_TAG;
};

/**
 * @enum {number}
 */
endo.Atom.AtomType = {
    INVALID: 0,
    SCOPE_BEGIN: 1,
    SCOPE_END: 2,
    EVENT: 3,
    TAG: 4,
    ERROR: 5
};

//java/com/google/net/rpc3/profiling/jendoscope/web/endo_proxy.js
/**
 * @fileoverview Endo's timeline proxy class, used to represent clusters of
 * spans on the timeline that are individually too small to be worth rendering.
 *
 * Tasks can be so short-lived that they occupy less than a pixel in the
 * timeline view. In order to speed up rendering, we coalesce adjacent short
 * tasks into 'proxies' - simple boxes on the timeline that serve to indicate
 * to the user "there is more data here, zoom in to see details".
 */

goog.provide('endo.Proxy');

goog.require('endo.Rect');
goog.require('goog.asserts');



/**
 * Creates a new, empty proxy rooted at the given task.
 * @param {!endo.Task} task The task we're beginning a proxy for.
 * @param {string} color The color of this proxy.
 * @constructor
 * @struct
 */
endo.Proxy = function(task, color) {
  /**
   * The first task represented by this proxy.
   * @private {!endo.Task}
   */
  this.head_ = task;

  /**
   * The last task represented by this proxy.
   * @private {!endo.Task}
   */
  this.tail_ = task;

  /**
   * The 'granularity' of this proxy, used to decide whether we draw this proxy
   * in place of the things inside it. For tasks, the grain size is the maximum
   * duration of all tasks represented by this proxy, or the maximum duration
   * of the gaps between those tasks.
   * @private {number}
   */
  this.grain_ = task.getDuration();

  /**
   * The color of the box we draw for this proxy.
   * @private {string}
   */
  this.colorFill_ = color;

  /**
   * The number of tasks coalesced into this proxy.
   * @private {number}
   */
  this.taskCount_ = 1;

  /**
   * The screen-space rectangle we draw for this proxy.
   * @private {endo.Rect}
   */
  this.proxyRect_ = new endo.Rect(0, 0, 0, 0);
};

/**
 * @return {!endo.Task}
 */
endo.Proxy.prototype.getTaskHead = function() {
  return this.head_;
};

/**
 * @return {!endo.Task}
 */
endo.Proxy.prototype.getTaskTail = function() {
  return this.tail_;
};

/**
 * @return {number}
 */
endo.Proxy.prototype.getGrain = function() {
  return this.grain_;
};

/**
 * @return {string}
 */
endo.Proxy.prototype.getColorFill = function() {
  return this.colorFill_;
};

/**
 * @return {number}
 */
endo.Proxy.prototype.getTaskCount = function() {
  return this.taskCount_;
};

/**
 * Extends this proxy to the right to enclose the given task. If doing so
 * causes the proxy to enclose tasks of different colors, turns the proxy grey.
 *
 * @param {!endo.Task} task The task we're enclosing.
 * @param {string} color The color of the task we're enclosing.
 */
endo.Proxy.prototype.mergeTask = function(task, color) {
  var gapSize = task.getTimeMin() - this.tail_.getTimeMax();
  this.tail_ = task;
  this.grain_ = Math.max(this.grain_, gapSize);
  this.grain_ = Math.max(this.grain_, task.getDuration());
  if (this.colorFill_ != color) {
    this.colorFill_ = endo.Config.proxyColor;
  }
  this.taskCount_++;
};

/**
 * Extends this proxy to the right to enclose the given proxy. If doing so
 * causes the proxy to enclose tasks of different colors, turns the proxy grey.
 *
 * @param {!endo.Proxy} proxy The proxy we're enclosing.
 */
endo.Proxy.prototype.mergeProxy = function(proxy) {
  var gapSize = proxy.head_.getTimeMin() - this.tail_.getTimeMax();
  this.tail_ = proxy.tail_;
  this.grain_ = Math.max(this.grain_, gapSize);
  this.grain_ = Math.max(this.grain_, proxy.grain_);
  if (this.colorFill_ != proxy.colorFill_) {
    this.colorFill_ = endo.Config.proxyColor;
  }
  this.taskCount_ += proxy.taskCount_;
};

/**
 * Draws the proxy on the timeline.
 *
 * @param {!HTMLCanvasElement} canvas The canvas we're rendering to.
 * @param {!endo.Rect} viewport The viewport we're rendering to.
 * @param {!endo.Rect} clipRect The section of the screen to clip the proxy to.
 */
endo.Proxy.prototype.drawProxyRect = function(canvas, viewport, clipRect) {
  var timeMin = this.head_.getTimeMin();
  var timeMax = this.tail_.getTimeMax();

  var xMin = endo.util.timeToPixel(timeMin, viewport.x0, viewport.x1, canvas);
  var xMax = endo.util.timeToPixel(timeMax, viewport.x0, viewport.x1, canvas);

  var proxyHeight = endo.Config.proxyHeight;

  this.proxyRect_.set(xMin,
                      clipRect.y0,
                      xMax,
                      clipRect.y0 + proxyHeight);

  this.proxyRect_.clipTo(clipRect);

  if (!this.proxyRect_.isEmpty()) {
    endo.draw.drawRect(this.proxyRect_, this.colorFill_, null);
  }
};


//java/com/google/net/rpc3/profiling/jendoscope/web/endo_timeline.js
// Copyright 2011 Google Inc. All Rights Reserved.

/**
 * @fileoverview Endoscope's timeline class, which displays all profiling data
 * retrieved from the Endoscope server in a zoomable, interactive timeline.
 *
 * The most frequently used view (and right now, the only functional view) in
 * the Endoscope console is the timeline view. It visualizes activities in all
 * instrumented threads on the server and allows the user to zoom in/out on
 * individual tasks to see more detail on how long they and their subtasks
 * took.
 *
 * The timeline is the main way that users interact with Endoscope, and a lot
 * of work has been done to make it aesthetically pleasing and comfortable to
 * use. One interesting implementation detail related to this is how the
 * timeline's viewport uses temporal easing to make the interface move smoothly
 * as the user pans the view around (see temporalEaseRect). The 'viewport'
 * member is where the viewport currently is, and 'viewportGoal' is where it's
 * being eased to. When we want the viewport to instantly snap to a location,
 * we set both to the same value.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('endo.Timeline');

goog.require('endo.Point');
goog.require('endo.Proxy');
goog.require('endo.Range');
goog.require('endo.Rect');



/**
 * Constructs an Endoscope timeline view that will render to the given canvas.
 *
 * @param {!HTMLCanvasElement} canvas The canvas to attach the timeline view
 *     to.
 * @constructor
 * @struct
 */
endo.Timeline = function(canvas) {

  /**
   * The canvas this view renders onto.
   * @type {!HTMLCanvasElement}
   * @private
   */
  this.canvas_ = canvas;

  /**
   * The bounding rectangle of the canvas.
   * @type {!endo.Rect}
   * @private
   */
  this.canvasRect_ = new endo.Rect(0, 0, canvas.width, canvas.height);

  /**
   * The viewport for this view.
   * @type {!endo.Rect}
   * @protected
   */
  this.viewport = new endo.Rect(0, 0, 0, 0);

  /**
   * The viewport we're easing towards.
   * @type {!endo.Rect}
   * @protected
   */
  this.viewportGoal = new endo.Rect(0, 0, 0, 0);

  /**
   * The list of saved old viewports.
   * @type {!Array.<!endo.Rect>}
   * @protected
   */
  this.viewportStack = [];

  /**
   * The list of thread objects currently in the viewport - this may not
   * include all the threads in the snapshot if the user is zoomed in.
   * @private {!Array.<!endo.Thread>}
   */
  this.visibleThreads_ = [];

  /**
   * The text labels to draw for the visible thread objects above.
   * @private {!Array.<!string>}
   */
  this.visibleLabels_ = [];

  /**
   * The thread that the user's cursor is hovering above, if any.
   * @private {endo.Thread}
   */
  this.hoverThread_ = null;

  /**
   * The span that the user's cursor is hovering above, if any.
   * @private {endo.Span}
   */
  this.hoverSpan_ = null;

  /**
   * The event that the user's cursor is hovering above, if any.
   * @private {endo.Event}
   */
  this.hoverEvent_ = null;

  /**
   * The event to the left of the user's cursor, if any.
   * @private {endo.Event}
   */
  this.hoverEventLeft_ = null;

  /**
   * The event to the right of the user's cursor, if any.
   * @private {endo.Event}
   */
  this.hoverEventRight_ = null;

  /**
   * The event that is closest to the user's cursor on either side.
   * @private {endo.Event}
   */
  this.hoverEventClosest_ = null;

  /**
   * The time, in seconds, between the event or span start time to the left
   * of the cursor and the time at the user's cursor.
   * @private {number}
   */
  this.hoverIntervalLeft_ = 0;

  /**
   * The time, in seconds, between the time at the user's cursor and the event
   * or span end time to the right of the cursor.
   * @private {number}
   */
  this.hoverIntervalRight_ = 0;

  /**
   * The height, in pixels, that we want to use for rendering spans at the
   * current zoom level.
   * @private {number}
   */
  this.spanHeight_ = 1;
};

/**
 * @return {!HTMLCanvasElement}
 */
endo.Timeline.prototype.getCanvas = function() {
  return this.canvas_;
};

/**
 * Converts a time in view-space to a pixel x coordinate in screen-space.
 *
 * @param {number} time The time to convert.
 * @return {number} The x coordinate in screen-space.
 */
endo.Timeline.prototype.timeToPixel = function(time) {
  return endo.util.timeToPixel(time,
                               this.viewport.x0,
                               this.viewport.x1,
                               this.canvas_);
};

/**
 * Converts a pixel x coordinate in screen-space to a time in view-space.
 *
 * @param {number} pixel The pixel x coordinate to convert.
 * @return {number} The time in view-space.
 */
endo.Timeline.prototype.pixelToTime = function(pixel) {
  return endo.util.pixelToTime(pixel,
                               this.canvas_,
                               this.viewport.x0,
                               this.viewport.x1);
};

/**
 * Resets the timeline view so that it displays a default view of the timeline
 * as specified in the config file. If we have no snapshot, resets the view to
 * show a dummy (-1,1) time range.
 */
endo.Timeline.prototype.resetViewport = function() {
  var rowMinY = -1;
  var rowMaxY = this.canvas_.height / (endo.Config.spanHeightMin + 2);
  rowMaxY /= endo.Config.defaultZoomLevelY;
  var timeMin = -1;
  var timeMax = 1;

  this.viewportStack = [];

  if (endo.network.snapshot) {
    var range = endo.network.snapshot.getTimeRange();
    if (!range.isEmpty()) {
      timeMin = range.min;
      timeMax = range.max;
    }
  }

  var width = timeMax - timeMin;
  var height = rowMaxY - rowMinY;

  var borderX = width * endo.Config.defaultViewPadding;

  var viewport = new endo.Rect(timeMin - borderX, rowMinY,
                               timeMax + borderX, rowMaxY);

  if (this.viewport.getWidth() == 0) {
    this.setViewport(viewport);
  }
  this.setViewportGoal(viewport);
};

/**
 * Clamps a viewport to the min/max sizes allowed for the view as defined in
 * the config file. The viewport size is clamped if it's too large or small
 * while keeping it centered.
 *
 * @param {endo.Rect} viewport The viewport to clamp.
 */
endo.Timeline.prototype.clampViewport = function(viewport) {
  var midX = (viewport.x0 + viewport.x1) / 2;
  var midY = (viewport.y0 + viewport.y1) / 2;

  var width = endo.util.clamp(viewport.getWidth(),
                              endo.Config.minViewWidth,
                              endo.Config.maxViewWidth);

  var pixelsPerRow = this.canvas_.height / viewport.getHeight();

  pixelsPerRow = endo.util.clamp(pixelsPerRow,
                                 endo.Config.minRowPixels,
                                 endo.Config.maxRowPixels);

  var height = this.canvas_.height / pixelsPerRow;

  viewport.x0 = midX - width / 2;
  viewport.x1 = midX + width / 2;
  viewport.y0 = midY - height / 2;
  viewport.y1 = midY + height / 2;
};

/**
 * Pans the viewport in the (x,y) direction given, scaled by the pan speed
 * in the config file.
 *
 * @param {number} x The (dimensionless) x distance to pan.
 * @param {number} y The (dimensionless) y distance to pan.
 */
endo.Timeline.prototype.pan = function(x, y) {
  var viewport = this.getViewport();

  var width = viewport.getWidth();
  var height = viewport.getHeight();

  var speed = endo.Config.panSpeed;

  viewport.x0 += width * x * speed;
  viewport.x1 += width * x * speed;
  viewport.y0 += height * y * speed;
  viewport.y1 += height * y * speed;

  this.setViewportGoal(viewport);
};

/**
 * Zooms in or out on the view while keeping the location under the user's
 * cursor stationary, clamping the view size if the user tries to zoom too far.
 *
 * @param {number} zoomX The (dimensionless) x-axis zoom factor.
 * @param {number} zoomY The (dimensionless) y-axis zoom factor.
 * @param {endo.Point=} opt_eventLocation The screen-space coordinate where the
 *     zoom occurred.
 */
endo.Timeline.prototype.zoom = function(zoomX, zoomY, opt_eventLocation) {
  var canvas = this.canvas_;
  var viewport = this.getViewport();

  // If the event did not occur at a specific spot on the screen, assume it
  // occured in the center of the screen
  var eventLocation = opt_eventLocation ? opt_eventLocation :
      new endo.Point(canvas.width / 2, canvas.height / 2);

  // Transform zoom amount to scale factor
  var scaleX = Math.pow(endo.Config.zoomSpeed, zoomX);
  var scaleY = Math.pow(endo.Config.zoomSpeed, zoomY);

  // Transform the event location from screen-space to graph-space.
  var screenRect = new endo.Rect(0, 0, canvas.width, canvas.height);
  eventLocation = eventLocation.remap(screenRect, viewport);

  // To ensure the view doesn't move when the user tries to zoom in or out too
  // far, we have to convert the zoom amount to width/height, clamp, and then
  // convert back to zoom amount.
  var minHeight = canvas.height / endo.Config.maxRowPixels;
  var maxHeight = canvas.height / endo.Config.minRowPixels;
  var minWidth = endo.Config.minViewWidth;
  var maxWidth = endo.Config.maxViewWidth;
  var newWidth = endo.util.clamp(viewport.getWidth() * scaleX,
                                 minWidth, maxWidth);
  var newHeight = endo.util.clamp(viewport.getHeight() * scaleY,
                                  minHeight, maxHeight);
  scaleX = newWidth / viewport.getWidth();
  scaleY = newHeight / viewport.getHeight();

  // We then scale the viewport relative to where the user clicked, which
  // ensures the position under the mouse cursor does not move.
  viewport.scaleRelative(scaleX, scaleY, eventLocation);
  this.setViewportGoal(viewport);
};

/**
 * Updates the view so that it zooms in on the span that the user double
 * clicked.
 *
 * @param {endo.Point} eventLocation The screen-space location of the double-
 *     click.
 */
endo.Timeline.prototype.doubleclick = function(eventLocation) {
  if (this.hoverSpan_ && !endo.ui.scroll) {
    var range = this.hoverSpan_.getTimeRange();
    var viewport = this.getViewport();
    // The view zooms to enclose the span under the mouse cursor, and centers
    // vertically on the thread the span is in.
    viewport.x0 = range.min - range.getWidth() * 0.2;
    viewport.x1 = range.max + range.getWidth() * 0.2;
    viewport.y0 = this.hoverThread_.getRowIndex();
    viewport.y1 = this.hoverThread_.getRowIndex() + 1;
    this.pushViewportGoal(viewport);
  }
};

/**
 * Gets (by value) the current viewport for the timeline view.
 *
 * @return {!endo.Rect} The current viewport.
 */
endo.Timeline.prototype.getViewport = function() {
  return this.viewport.clone();
};

/**
 * Gets (by value) the current viewport goal for the timeline view.
 *
 * @return {!endo.Rect} The current viewport goal.
 */
endo.Timeline.prototype.getViewportGoal = function() {
  return this.viewportGoal.clone();
};

/**
 * Sets (by value) the current viewport for the timeline view.
 *
 * @param {!endo.Rect} viewport The new viewport.
 */
endo.Timeline.prototype.setViewport = function(viewport) {
  this.viewport = viewport.clone();
  this.clampViewport(this.viewport);
};

/**
 * Sets (by value) the current viewport goal for the timeline view.
 *
 * @param {!endo.Rect} viewport The new viewport goal.
 */
endo.Timeline.prototype.setViewportGoal = function(viewport) {
  this.viewportGoal = viewport.clone();
  this.clampViewport(this.viewportGoal);
};

/**
 * Saves the current viewport on the viewport stack and transitions to
 * the new viewport.
 *
 * @param {!endo.Rect} viewport The new viewport.
 */
endo.Timeline.prototype.pushViewportGoal = function(viewport) {
  this.viewportStack.push(this.viewportGoal.clone());
  this.setViewportGoal(viewport);
};

/**
 * Restores the previous viewport that was pushed onto the viewport stack.
 */
endo.Timeline.prototype.popViewportGoal = function() {
  if (this.viewportStack.length) {
    var viewport = this.viewportStack[this.viewportStack.length - 1];
    this.viewportStack = this.viewportStack.slice(0, -1);
    this.setViewportGoal(viewport);
  } else {
    this.resetViewport();
  }
};

/**
 * Updates the viewport to match the user's selection rectangle.
 */
endo.Timeline.prototype.applySelection = function() {
  if (!endo.ui.selectionRect) { return; }

  var canvas = this.canvas_;
  var screenRect = new endo.Rect(0, 0, canvas.width, canvas.height);

  // Unflip the selection rectangle so that it's oriented correctly no matter
  // which direction the user dragged the mouse.
  var newViewport = endo.ui.selectionRect.clone();
  newViewport.unflip();

  // Transform the selection rectangle to view space and set it as the new
  // viewport.
  newViewport.remap(screenRect, this.viewport);
  this.pushViewportGoal(newViewport);

  endo.ui.selectionRect = null;
};

/**
 * Computes statistics about the selected area of the timeline and displays
 * the results on screen.
 * TODO(aappleby): Not implemented yet.
 */
endo.Timeline.prototype.applyMeasure = function() {
  endo.ui.measureRect = null;
};

/**
 * Advances the timeline view animations by timestep dt and updates all
 * rendering data for all parts of the view.
 * @param {number} dt The timestep to advance our animations by.
 */
endo.Timeline.prototype.update = function(dt) {
  // If the user has auto-scrolling enabled, always snap the right edge of the
  // view so it tracks the current server time.
  if (endo.ui.scroll && endo.network.snapshot) {
    var goal = this.getViewportGoal();
    var width = goal.getWidth();
    goal.x1 = endo.sync.serverTime() +
              width * endo.Config.defaultViewPadding;
    goal.x0 = goal.x1 - width;
    this.setViewportGoal(goal);
  }

  // We want the view to smoothly transition to the goal rectangle instead of
  // snapping, so use temporal easing.
  endo.util.temporalEaseRect(this.viewport,
                             this.viewportGoal,
                             endo.Config.viewEaseTime,
                             dt);

  // Recompute the height of the spans given the current view settings.
  // We want the spans to get wider as the user zooms in, but not excessively
  // so, so we interpolate between the min and max heights as the row size
  // changes.
  var pixelsPerRow = this.canvas_.height / this.viewport.getHeight();

  var pixelsPerRowMin = endo.Config.spanHeightMin * endo.Config.spansPerRowMin;
  var pixelsPerRowMax = endo.Config.spanHeightMax * endo.Config.spansPerRowMax;

  this.spanHeight_ = endo.util.remapClamp(pixelsPerRow,
                                          pixelsPerRowMin,
                                          pixelsPerRowMax,
                                          endo.Config.spanHeightMin,
                                          endo.Config.spanHeightMax);

  // Update all the pieces of this view. Right now, that means all the threads.
  this.updateThreads();
};

/**
 * Updates rendering data for all threads in the snapshot - runs visibility
 * tests, mouse hit tests, and updates for each thread.
 */
endo.Timeline.prototype.updateThreads = function() {
  var canvas = this.canvas_;
  var viewport = this.viewport;
  var snapshot = endo.network.snapshot;

  this.visibleThreads_ = [];
  this.visibleLabels_ = [];
  this.hoverThread_ = null;
  this.hoverSpan_ = null;
  this.hoverEvent_ = null;
  this.hoverEventLeft_ = null;
  this.hoverEventRight_ = null;
  this.hoverEventClosest_ = null;
  this.hoverIntervalLeft_ = 0;
  this.hoverIntervalRight_ = 0;

  if (!snapshot) { return; }

  var rowIndex = 0;
  var threads = snapshot.getThreadList();
  for (var i = 0; i < threads.length; i++) {
    var thread = threads[i];
    this.visibleLabels_.push(thread.getName());

    if (!this.isThreadVisible(thread)) { continue; }
    thread.setRowIndex(rowIndex++);

    // If all tasks in the thread are completely offscreen, don't bother
    // updating it.
    if (thread.getTaskHead().getTimeMin() > viewport.x1) { continue; }
    if (thread.getTaskTail().getTimeMax() < viewport.x0) { continue; }

    // The timeline view is divided into horizontal rows, with one thread
    // rendered in each row. Compute the screen-space Y bounds of the row we're
    // for the current thread.
    var rowMinY = endo.util.remap(thread.getRowIndex(),
                                  viewport.y0, viewport.y1,
                                  0, canvas.height);

    var rowMaxY = endo.util.remap(thread.getRowIndex() + 1,
                                  viewport.y0, viewport.y1,
                                  0, canvas.height);

    // Add one pixel of padding between row border and span rectangles.
    rowMinY += 1;
    rowMaxY -= 1;

    // Skip the thread if it's off the top of the screen.
    if (rowMaxY < 0) { continue; }

    // Stop processing threads if we've gone off the bottom of the screen.
    if (rowMinY > canvas.height) { break; }

    // Otherwise, update the thread's screen rectangle and check for mouse hit.
    thread.setScreenRect(0, rowMinY, canvas.width, rowMaxY);

    if (thread.getScreenRect().contains(endo.ui.mousePos)) {
      this.hoverThread_ = thread;
    }

    // Update our firstVisible_ cursor, which tracks the first task to appear
    // on the screen so we don't have to traverse off-screen tasks before we
    // can start rendering something.
    var cursor = thread.getFirstVisibleTask() ?
                 thread.getFirstVisibleTask() :
                 thread.getTaskHead();

    // Skip forward until we find a task that's on-screen or we hit the tail.
    while (cursor != thread.getTaskTail()) {
      if (cursor.getTimeMax() >= viewport.x0) { break; }
      cursor = cursor.getNext();
    }

    // Skip backwards until we find a task that's off-screen or we hit the head.
    while (cursor != thread.getTaskHead()) {
      if (cursor.getTimeMax() < viewport.x0) {
        // The first visible task is the one after the first invisible task,
        // if there is one.
        if (cursor.getNext()) {
          cursor = cursor.getNext();
        }
        break;
      }
      cursor = cursor.getPrev();
    }

    thread.setFirstVisibleTask(cursor);

    // Add this thread to the list of visible threads and update the thread
    // itself.

    this.visibleThreads_.push(thread);
    this.updateThread(thread);
  }
};


/**
 * Returns true if the thread has any visible tasks.
 * @param {!endo.Thread} thread The thread to check.
 * @return {boolean}
 */
endo.Timeline.prototype.isThreadVisible = function(thread) {
  var tasks = thread.getTaskList();
  for (var i = 0; i < tasks.length; i++) {
    var span = tasks[i].getRootSpan();
    var marker = endo.network.snapshot.getMarker(span.markerId);
    if (marker && marker.visible)
      return true;
  }
  return false;
};


/**
 * Updates rendering data for a thread in the timeline - performs proxy
 * generation, runs visibility tests, and updates all visible, non-proxied
 * tasks in the thread.
 * @param {!endo.Thread} thread The thread to update.
 */
endo.Timeline.prototype.updateThread = function(thread) {
  var canvas = this.canvas_;
  var viewport = this.viewport;
  var snapshot = endo.network.snapshot;

  // Tasks are coalesced into proxies if both the tasks and the spaces between
  // them are smaller than the proxy threshold defined in the config file.
  var pixelsPerSecond = canvas.width / (viewport.x1 - viewport.x0);
  var threshold = endo.Config.proxyThreshold / pixelsPerSecond;

  // Traverse the top-level tasks in the thread, starting with the first one
  // that could appear on-screen and stopping when we go off the screen. Keep
  // track of the most recently seen proxy and try to merge additional tasks
  // and proxies into it as we go.
  var proxy = null;
  var task = thread.getFirstVisibleTask();
  while (task && (task.getTimeMin() < viewport.x1)) {

    // Skip hidden tasks.
    var marker = snapshot.getMarker(task.getMarkerId());
    if (!marker || !marker.visible) {
      task.setVisible(false);
      task = task.getNext();
      continue;
    }
    task.setVisible(true);
    task.setProxied(true);

    // If there's an active proxy but nothing else can be merged into it,
    // (either because it's too large, it's incomplete, or the gap between
    // it and the previous proxy is too lage), discard the active proxy.
    var duration = task.getDuration();
    var unproxyable = !task.isComplete() || (duration >= threshold);
    var gapSize = proxy ?
        task.getTimeMin() - proxy.getTaskTail().getTimeMax() : Infinity;

    if (proxy && (unproxyable || (gapSize > threshold))) {
      proxy = null;
    }

    // If the current task is incomplete or too large to proxy, turn off proxy
    //  drawing for it and update it.
    if (unproxyable) {
      task.setProxied(false);
      this.updateTask(thread, task);
      task = task.getNext();
      continue;
    }

    // The current task is proxy-able. If there is no active proxy, either
    // reuse the one attached to the current task (if it's finely grained
    // enough) or create a new one and attach it to the task.
    if (!proxy) {
      if (!task.getProxy() || (task.getProxy().getGrain() >= threshold)) {
        marker = snapshot.getMarker(task.getMarkerId());
        var color = marker ? marker.colorFill : '#888';
        task.setProxy(new endo.Proxy(task, color));
      }
      proxy = task.getProxy();
    }

    // Merge the proxy attached to the current task (if it's finely grained
    // enough) or just the task into the active proxy.
    var taskProxy = task.getProxy();
    if (taskProxy && (taskProxy.getGrain() < threshold)) {
      proxy.mergeProxy(taskProxy);
    } else {
      marker = snapshot.getMarker(task.getMarkerId());
      var color = marker ? marker.colorFill : '#888';
      proxy.mergeTask(task, color);
    }

    // Skip past all tasks that will be represented by the active proxy.
    task = proxy.getTaskTail().getNext();
  }
};

/**
 * Updates rendering data for a single task in the timeline. Right now we don't
 * have any work to do beyond updating the span tree, so this is mostly a stub.
 *
 * @param {!endo.Thread} thread The parent thread of the task to update.
 * @param {!endo.Task} task The task to update.
 */
endo.Timeline.prototype.updateTask = function(thread, task) {
  var root = task.getRootSpan();
  if (root) {
    this.updateSpanTree(thread, root);
  }
};

/**
 * Recursively updates rendering data for all spans in a tree, stopping when
 * it hits a span whose children aren't visible.
 *
 * @param {!endo.Thread} thread The parent thread for the span we want to draw.
 * @param {!endo.Span} span The span to render.
 */
endo.Timeline.prototype.updateSpanTree = function(thread, span) {
  this.updateSpan(thread, span);

  // Recurse through child spans if they're visible.
  if (!span.childrenVisible) { return; }

  for (var i = 0; i < span.spans.length; ++i) {
    var child = span.spans[i];
    if (child.type == endo.Span.SpanType.SCOPE) {
      this.updateSpanTree(thread, child);
    } else if (child.type == endo.Span.SpanType.HOLE) {
      var subtask = thread.getTask(child.subtaskId);
      if (subtask) {
        var subroot = subtask.getRootSpan();
        if (subroot) {
          this.updateSpanTree(thread, subroot);
        }
      }
    }
  }
};

/**
 * Updates span-related rendering data - screen rectangles, rendering option
 * flags, mouse cursor checks, and color tweaks.
 *
 * @param {!endo.Thread} thread The parent thread of the span to update.
 * @param {!endo.Span} span The span to update.
 */
endo.Timeline.prototype.updateSpan = function(thread, span) {
  var snapshot = endo.network.snapshot;

  // Skip spans associated with missing or invisible markers.
  var marker = snapshot.getMarker(span.markerId);
  if (!marker || !marker.visible) {
    span.visible = false;
    return;
  }

  // Compute the span's screen rectangle.
  var clipRect = thread.getScreenRect();

  var xMin = this.timeToPixel(span.timeMin);
  var xMax = this.timeToPixel(span.timeMax);

  var yMin = Math.round(clipRect.y0) + span.depth * (this.spanHeight_ + 1);
  var yMax = yMin + this.spanHeight_;

  span.screenRect.set(xMin, yMin, xMax, yMax);
  span.screenRect.clipTo(clipRect);

  span.visible = !span.screenRect.isEmpty();
  span.childrenVisible = span.screenRect.getWidth() > 2;

  // Flag the span for detail drawing if it's larger than the threshold in the
  // config file.
  if ((span.screenRect.getHeight() > endo.Config.spanDetailThresholdHeight) &&
      (span.screenRect.getWidth() > endo.Config.spanDetailThresholdWidth)) {
    span.drawDetail = true;
  } else {
    span.drawDetail = false;
  }

  // Check for mouse overlap if the span is larger than the threshold in the
  // config file.
  if ((thread == this.hoverThread_) &&
     (span.screenRect.getHeight() > endo.Config.spanHoverThresholdHeight) &&
     (span.screenRect.getWidth() > endo.Config.spanHoverThresholdWidth) &&
     span.screenRect.contains(endo.ui.mousePos) &&
     !endo.ui.selectionRect &&
     (this.hoverSpan_ == null)) {

    this.hoverSpan_ = span;
    this.updateEvents(span);
  }

  // Update span color. Lost spans are red, hovered spans pulse, and normal
  // spans use their marker's color.

  var lost = !span.complete &&
             (span.timeMax != endo.network.snapshot.getServerTimeEnd());

  if (lost) {
    if (marker.hasMinPeriod) {
      // Lost spans that were generated by a minPeriod-triggered task should
      // not appear on the timeline.
      span.visible = false;
      return;
    } else {
      span.colorFill = '#f00';
      span.colorBorder = '#c00';
    }
  } else if (span == this.hoverSpan_) {
    // The span we're hovering over gets a pulsing color.
    var hsvA = [marker.hue, 0.5, 0.9];
    var hsvB = [marker.hue, 0.4, 1.0];
    span.colorFill = endo.util.pulseHSV(hsvA, hsvB, 1.2);
    span.colorBorder = marker.colorBorder;
  } else {
    // Normal spans use their marker's color.
    // Spans irrelevant to the hovered span will be dimmed. It is done at
    // render time because hovered span may not be available at this time yet.
    span.colorFill = marker.colorFill;
    span.colorBorder = marker.colorBorder;
  }

  if (span.hasError) {
    span.drawDetail = true;
  }
};

/**
 * Finds the closest events under, to the left, and to the right of the mouse
 * cursor.
 *
 * @param {!endo.Span} span The span to search for nearby events.
 */
endo.Timeline.prototype.updateEvents = function(span) {
  this.hoverIntervalLeft_ = span.timeMin;
  this.hoverIntervalRight_ = span.timeMax;

  var closestLeft = Infinity;
  var closestRight = Infinity;
  var closestEvent = Infinity;

  for (var i = 0; i < span.events.length; i++) {
    var event = span.events[i];
    var screenX = this.timeToPixel(event.time);
    var distLeft = endo.ui.mousePos.x - screenX;
    if ((distLeft > 0) && (distLeft < closestLeft)) {
      this.hoverEventLeft_ = event;
      this.hoverIntervalLeft_ = event.time;
      closestLeft = distLeft;
    }
    var distRight = screenX - endo.ui.mousePos.x;
    if ((distRight > 0) && (distRight < closestRight)) {
      this.hoverEventRight_ = event;
      this.hoverIntervalRight_ = event.time;
      closestRight = distRight;
    }
    var dist = Math.abs(screenX - endo.ui.mousePos.x);
    if (dist < closestEvent) {
      this.hoverEventClosest_ = event;
      closestEvent = dist;
    }
  }

  if (this.hoverEventClosest_ &&
      (closestEvent <= endo.Config.eventHoverThreshold)) {
    this.hoverEvent_ = this.hoverEventClosest_;
  }
};

/**
 * Resizes our canvas to fit its parent div and resets our rendering state.
 */
endo.Timeline.prototype.prerender = function() {
  var canvas = this.canvas_;

  var divWidth = canvas.parentNode.clientWidth;
  var divHeight = canvas.parentNode.clientHeight;

  endo.draw.resetCanvas(divWidth, divHeight);

  this.canvasRect_.set(0, 0, canvas.width, canvas.height);
};

/**
 * Renders the timeline view.
 */
endo.Timeline.prototype.render = function() {
  this.prerender();

  var canvas = this.canvas_;
  var viewport = this.viewport;
  var snapshot = endo.network.snapshot;
  var timeRange = snapshot ? snapshot.getTimeRange() : new endo.Range(0, 1);

  // Draw our background grid.
  if (endo.Config.drawBackground) {
    endo.draw.drawGridBackground(viewport,
                                 endo.Config.timelineBackgroundDark,
                                 endo.Config.timelineBackgroundMedium,
                                 timeRange.min, timeRange.max);
    endo.draw.drawGridLinesY(viewport);
  }

  // Draw the hover thread highlight.
  if (this.hoverThread_) {
    var rect = this.hoverThread_.getScreenRect().clone();
    rect.x0++;
    rect.x1--;
    endo.draw.drawRect(rect, null, 'rgba(255,255,255,0.5)');
  }

  // Draw all visible threads.
  for (var i = 0; i < this.visibleThreads_.length; i++) {
    var thread = this.visibleThreads_[i];
    this.drawThread(thread);
  }

  // Draw the hover span.
  if (this.hoverThread_ && this.hoverSpan_) {
    var clipRect = this.hoverThread_.getScreenRect();
    this.drawHoverSpan(this.hoverSpan_, clipRect);
  }

  // Draw the selection rectangle.
  if (endo.ui.selectionRect) {
    endo.draw.drawHighlightRect(endo.ui.selectionRect,
                                endo.Config.selectionHighlight1,
                                endo.Config.selectionHighlight2);
  }

  // Draw the measurement rectangle.
  if (endo.ui.measureRect) {
    endo.draw.drawHighlightRect(endo.ui.measureRect,
                                endo.Config.measureHighlight1,
                                endo.Config.measureHighlight2);
  }

  // Draw the grid labels last so that the span trees don't occlude them.

  var drawBackdrop = true;

  endo.draw.drawGridLabelsY(viewport);
  endo.draw.drawGridLabelsX(viewport, this.visibleLabels_, drawBackdrop);
  endo.draw.drawGridLinesX(viewport);
  this.drawCurrentTimeTick();

  // Print the real-world time under the mouse cursor at the bottom of the
  // timeline along with an indicator line.
  var time = this.pixelToTime(endo.ui.mousePos.x / 2);
  var timeText = new Date(endo.sync.serverTimeBegin * 1000 +
                          time * 1000);
  endo.draw.fillText('#ffffff',
                     timeText.toString(),
                     endo.ui.mousePos.x + endo.Config.textPaddingX,
                     canvas.height - endo.Config.textPaddingX);
  endo.draw.fillRect('rgba(255,255,255,0.3)',
                     endo.ui.mousePos.x, canvas.height - 30,
                     1, 30);

  // Update the info bar at the bottom of the screen to show details about
  // the thread and span being hovered over, if there is one.
  var info = '';
  if (this.hoverThread_) {
    //Info line 1 - thread name, span name, duration, interval, event.
    info += this.hoverThread_.getName();
    if (this.hoverSpan_) {
      // Add the span caption to the info bar.
      info += ' | ';
      info += this.getSpanCaption_(this.hoverSpan_);

      // Add the span tag to the info bar.
      if (this.hoverSpan_.allTags.length > 0) {
        info += ' | ';
        info += 'Tag: ' + this.hoverSpan_.allTags.join(' ');
      }

      // Add the span duration to the info bar.
      info += ' | ';
      info += endo.util.prettyprintDuration(this.hoverSpan_.getDuration());

      // Add the span rate information to the info bar if present.
      var units = this.hoverSpan_.units;
      var rate = this.hoverSpan_.getRate();
      if (units && rate) {
        info += ' | ';
        var rateText = endo.util.prettyprintUnit(units) + ' units, ' +
                       endo.util.prettyprintUnit(rate) + '/s';
        info += rateText;
      }
    }

    // If there's an event on either side of the cursor, show details about
    // the time interval around the cursor on the info bar.
    if (this.hoverEventLeft_ || this.hoverEventRight_) {
      var duration = this.hoverIntervalRight_ - this.hoverIntervalLeft_;
      info += ' | ';
      info += 'Interval: ';
      info += endo.util.prettyprintDuration(duration);
    }

    // If there's an event to the left of the cursor, show its details
    // in the info bar.
    if (this.hoverEventLeft_) {
      info += ' | ';
      info += 'Event: ';

      var leftMarker = snapshot.getMarker(this.hoverEventLeft_.markerId);
      info += leftMarker ? leftMarker.name : '<none>';
    }

    // Info line 2 - marker type, marker location, marker function.
    info += '<br>';
    if (this.hoverSpan_) {
      var marker = snapshot.getMarker(this.hoverSpan_.markerId);
      if (marker) {
        var markerType = endo.Marker.MarkerTypeNames[marker.type];
        info += markerType + ' @ ' + marker.file + ':' +
                marker.line.toString();
        info += ' | ';
        info += marker.functionName;
        info += ' | ';
        info += 'Total: ';
        info += endo.util.prettyprintDuration(marker.totalTaskTime());
        info += ' | ';
        info += 'Average: ';
        // marker.averageTime() may be NaN
        info += endo.util.prettyprintDuration(marker.averageTaskTime());
      }
    }
  }

  document.getElementById('info_popup').innerHTML = info;
};

/**
 * Draws all the tasks and proxies for one thread onto the timeline view.
 *
 * @param {!endo.Thread} thread The thread to draw.
 */
endo.Timeline.prototype.drawThread = function(thread) {
  var canvas = this.canvas_;
  var viewport = this.viewport;

  var task = thread.getFirstVisibleTask();
  while (task && (task.getTimeMin() < viewport.x1)) {
    if (task.isProxied()) {
      var proxy = task.getProxy();
      goog.asserts.assert(proxy);
      proxy.drawProxyRect(canvas, viewport, thread.getScreenRect());
      task = proxy.getTaskTail().getNext();
    } else if (task.isVisible()) {
      var rootSpan = task.getRootSpan();
      if (rootSpan) {
        this.drawSpanTree(thread, rootSpan, thread.getScreenRect());
      }
      task = task.getNext();
    } else {
      task = task.getNext();
    }
  }
};

/**
 * Draws a tick mark at the current server time.
 */
endo.Timeline.prototype.drawCurrentTimeTick = function() {
  var canvas = this.canvas_;
  var viewport = this.viewport;
  var now = endo.sync.serverTime();

  var screenX = endo.util.timeToPixel(now, viewport.x0, viewport.x1, canvas);

  screenX = Math.round(screenX) + 0.5;

  endo.draw.strokeLine('rgba(255,255,128,0.5)',
                       screenX, 0,
                       screenX, 15);
};

/**
 * Recursively draws all spans in a span tree as nested rectangles on the graph,
 * clipping them to fit inside their containing row and skipping spans that are
 * smaller than a few pixels.
 *
 * @param {!endo.Thread} thread The parent thread for the span we want to draw.
 * @param {!endo.Span} span The span to draw.
 * @param {!endo.Rect} clipRect The area of the screen to draw the span tree in.
 */
endo.Timeline.prototype.drawSpanTree = function(thread, span, clipRect) {
  var canvas = this.canvas_;

  // Don't draw the span if it's the hover span - we want to draw last so it
  // won't be occluded.
  if (span.visible &&
     (span != this.hoverSpan_) &&
     this.canvasRect_.overlaps(span.screenRect)) {
    this.drawSpan(span, clipRect);
  }

  // Recurse through child spans.
  if (!span.childrenVisible) { return; }

  for (var i = 0; i < span.spans.length; ++i) {
    var child = span.spans[i];
    if (child.type == endo.Span.SpanType.SCOPE) {
      this.drawSpanTree(thread, child, clipRect);
    } else if (child.type == endo.Span.SpanType.HOLE) {
      var subtask = thread.getTask(child.subtaskId);
      if (subtask) {
        var subroot = subtask.getRootSpan();
        if (subroot) {
          this.drawSpanTree(thread, subroot, clipRect);
        }
      }
    }
  }
};

/**
 * Draws a single span on the timeline, consisting of a background rectangle,
 * detail items (if present), and text describing the original event and its
 * duration.
 *
 * @param {!endo.Span} span The span we want to draw.
 * @param {!endo.Rect} clipRect The area of the screen to draw the span tree in.
 */
endo.Timeline.prototype.drawSpan = function(span, clipRect) {
  // Decide if the current spans should be dimmed
  // (whether it is irrelevant to the current hovered span)
  var dim = false;
  if (this.hoverSpan_ && this.hoverSpan_.allTags.length > 0) {
    dim = true;
    var tags1 = this.hoverSpan_.allTags;
    var tags2 = span.allTags;
    for (var i = 0; i < tags2.length; i++) {
      if (tags1.indexOf(tags2[i]) != -1) {
        dim = false;
        break;
      }
    }
  }

  // Draw the span's rectangle.
  if (dim) {
    // Normal spans irrelevant to hovered span use background color
    endo.draw.drawRect(span.screenRect, endo.Config.spanDimColor,
        span.colorBorder);
  } else {
    // Normal spans relevant to hovered span use original color
    endo.draw.drawRect(span.screenRect, span.colorFill, span.colorBorder);
  }

  // Draw detail items on top of the span's rectangle before we draw the text
  if (span.drawDetail) {
    this.drawEvents(span, clipRect);
  }

  // Draw the span's text last so it's not occluded.
  this.drawTimeRectText(span,
                        this.getSpanCaption_(span),
                        span.getDetails(),
                        span.allTags,
                        span.getDuration(),
                        span.units,
                        span.getRate());
};


/**
 * Builds the caption we want to print in the timeline view from the span's
 * marker's name, along with various identifiers for debugging purposes.
 * @param {!endo.Span} span The span.
 * @return {string} The caption for this span.
 * @private
 */
endo.Timeline.prototype.getSpanCaption_ = function(span) {
  var marker = endo.network.snapshot.getMarker(span.markerId);
  var name = marker ? marker.name : '<unknown>';

  if (endo.Config.debug) {
    if (span.type == endo.Span.SpanType.TASK)
      name = 'T:' + name;
    if (span.type == endo.Span.SpanType.SUBTASK) {
      name = 'S:' + name;
    }
    if (span.type == endo.Span.SpanType.SCOPE)
      name = 'O:' + name;
    if (span.type == endo.Span.SpanType.HOLE)
      name = 'H:' + name;
    if (span.complete == false)
      name = 'I:' + name;
  }
  return name;
};


/**
 * Draws the span the user is hovering the mouse cursor over with additional
 * style elements - a highlight box to set it off from the rest of the
 * timeline, highlighted events, and additional details as necessary.
 *
 * @param {!endo.Span} span The span we want to draw.
 * @param {!endo.Rect} clipRect The area of the screen to draw the span tree in.
 */
endo.Timeline.prototype.drawHoverSpan = function(span, clipRect) {
  var canvas = this.canvas_;
  var viewport = this.viewport;
  var snapshot = endo.network.snapshot;

  // Draw a highlight rect behind the span to set it off from the background.
  var highlightRect = span.screenRect.clone();
  highlightRect.expand(endo.Config.spanHoverBorder);
  endo.draw.drawHighlightRect(highlightRect,
                              endo.Config.spanHoverHighlight1,
                              endo.Config.spanHoverHighlight2);

  // Draw the span's rectangle.
  endo.draw.drawRect(span.screenRect, span.colorFill, span.colorBorder);

  // Draw events on the span and, if the user is hovering over an event
  // interval, draw bars to highlight the interval.
  if (span.drawDetail) {

    this.drawEvents(span, clipRect);

    if (this.hoverEventLeft_ || this.hoverEventRight_) {

      var t0 = this.hoverIntervalLeft_;
      var t1 = this.hoverIntervalRight_;

      var mouseTime = this.pixelToTime(endo.ui.mousePos.x);

      var x0 = this.timeToPixel(this.hoverIntervalLeft_);
      var x1 = this.timeToPixel(this.hoverIntervalRight_);
      var y0 = Math.round(span.screenRect.y0) + this.spanHeight_;
      var y1 = Math.min(y0 + 2, clipRect.y1);

      if (x0 < 0) {
        x0 = 0;
      }
      if (x1 > canvas.width) {
        x1 = canvas.width;
      }

      var rect = new endo.Rect(x0, y0, x1 + 1, y1);
      endo.draw.drawRect(rect, endo.Config.intervalHighlight);

      rect = new endo.Rect(x0, y0 - 10, x0 + 1, y0);
      endo.draw.drawRect(rect, endo.Config.intervalHighlight);
    }
  }

  // If the user is hovering over a span interval, make the text on the span
  // reflect the interval instead of the entire span.
  var caption = this.getSpanCaption_(span);
  var duration = span.getDuration();

  if (this.hoverIntervalLeft_ && this.hoverIntervalRight_) {
    duration = this.hoverIntervalRight_ - this.hoverIntervalLeft_;
  }
  if (this.hoverEventLeft_) {
    var leftMarker = snapshot.getMarker(this.hoverEventLeft_.markerId);
    caption = leftMarker ? leftMarker.name : '<none>';
  }

  // Draw the span's text last so it's not occluded.
  this.drawTimeRectText(span,
                        caption,
                        span.getDetails(),
                        span.allTags,
                        duration,
                        span.units,
                        span.getRate());
};

/**
 * Draws all the bits of text we want to appear on a span so that they're
 * correctly justified in the span and fade in and out as the span changes
 * size.
 *
 * TODO(aappleby): Text metrics should be stored in the task so we don't
 * recalculate them every frame - currently we're wasting ~5% of the CPU.
 *
 * @param {!endo.Span} span The span we're drawing text for.
 * @param {string} captionText The caption for the rectangle.
 * @param {string} detailText The detail text for the span.
 * @param {!Array.<string>} tags The tag for this span
 * @param {number} duration The duration in seconds for the span.
 * @param {number} units The units of work completed in the span.
 * @param {number} rate The rate of work for the span.
 */
endo.Timeline.prototype.drawTimeRectText = function(span,
                                                    captionText,
                                                    detailText,
                                                    tags,
                                                    duration,
                                                    units,
                                                    rate) {
  var canvas = this.canvas_;

  // Update the region of the span we're allowed to draw text in.
  span.textRect.set(span.screenRect.x0 + endo.Config.textPaddingX,
                    span.screenRect.y0,
                    span.screenRect.x1 - endo.Config.textPaddingX,
                    span.screenRect.y1);

  // If the text rect is too small to fit readable text in, don't bother
  // trying to render any.
  if (span.textRect.getWidth() < endo.Config.spanTextFadeMinX) { return; }
  if (span.textRect.getHeight() < endo.Config.spanTextFadeMinY) { return; }

  // If all the text would be off-screen, don't bother trying to render any.
  if (!span.textRect.overlaps(this.canvasRect_)) { return; }

  // OK, we're going to be drawing text, so set up our clipping region.
  endo.draw.clipRect(span.textRect.x0, span.textRect.y0,
                     span.textRect.x1, span.textRect.y1);

  // Compute where the top and bottom of each line of text is.
  var line1Min = span.textRect.y0;
  var line1Max = span.textRect.y0 + endo.Config.spanHeightMin;
  var line2Min = span.textRect.y0 + endo.Config.spanHeightMin;
  var line2Max = span.textRect.y0 + endo.Config.spanHeightMin * 2;

  // Draw the caption centered in the top line.
  if (captionText) {
    var captionWidth = endo.draw.measureText(captionText);
    var x0 = span.textRect.getCenterX() - captionWidth / 2;
    var x1 = span.textRect.getCenterX() + captionWidth / 2;

    span.captionRect.set(x0, line1Min, x1, line1Max);
    span.captionRect.clipTo(span.textRect);
    endo.draw.drawTextInRect(captionText, captionWidth, span.captionRect);
  } else {
    span.captionRect.set(span.textRect.x0, line1Min,
                         span.textRect.x0, line1Max);
  }

  // Draw the duration in the space to the right of the caption.
  if (duration) {
    var durationText = endo.util.prettyprintDuration(duration);
    var durationWidth = endo.draw.measureText(durationText);

    var x0 = Math.max(span.captionRect.x1,
                      span.textRect.x1 - durationWidth);
    var x1 = span.textRect.x1;

    span.durationRect.set(x0, line1Min, x1, line1Max);
    span.durationRect.clipTo(span.textRect);
    endo.draw.drawTextInRect(durationText, durationWidth, span.durationRect);
  } else {
    span.durationRect.set(span.textRect.x1, line1Min,
                          span.textRect.x1, line1Max);
  }

  // Draw the tag in the space to the left of the caption.
  if (tags.length > 0) {
    var tagText = tags.join(' ');
    var tagWidth = endo.draw.measureText(tagText);

    var x0 = span.textRect.x0;
    var x1 = Math.min(span.captionRect.x0,
                      span.textRect.x0 + tagWidth);

    span.tagRect.set(x0, line1Min, x1, line1Max);
    span.tagRect.clipTo(span.textRect);
    endo.draw.clipRect(span.tagRect.x0, span.tagRect.y0,
        span.tagRect.x1, span.tagRect.y1);
    endo.draw.drawTextInRect(tagText, tagWidth, span.tagRect);
    endo.draw.clearClip();
  } else {
    span.tagRect.set(span.textRect.x0, line1Min, span.textRect.x0, line1Max);
  }

  // Draw the detail text on line 2, aligned to the left.
  if (detailText) {
    var detailWidth = endo.draw.measureText(detailText);

    var x0 = span.textRect.x0;
    var x1 = span.textRect.x0 + detailWidth;

    span.detailRect.set(x0, line2Min, x1, line2Max);
    span.detailRect.clipTo(span.textRect);
    endo.draw.drawTextInRect(detailText, detailWidth, span.detailRect);
  } else {
    span.detailRect.set(span.textRect.x0, line2Min,
                        span.textRect.x0, line2Max);
  }

  // Draw the rate text on line 2, in the space to the right of the detail
  // text.
  if (units && rate) {
    var rateText = endo.util.prettyprintUnit(units) + ' : ' +
                   endo.util.prettyprintUnit(rate) + '/s';
    var rateWidth = endo.draw.measureText(rateText);

    var x0 = Math.max(span.detailRect.x1,
                      span.textRect.x1 - rateWidth);
    var x1 = span.textRect.x1;

    span.rateRect.set(x0, line2Min, x1, line2Max);
    span.rateRect.clipTo(span.textRect);
    endo.draw.drawTextInRect(rateText, rateWidth, span.rateRect);
  } else {
    span.rateRect.set(span.textRect.x1, line2Min,
                      span.textRect.x1, line2Max);
  }

  // Done rendering text, clear the clipping region.
  endo.draw.clearClip();
};

/**
 * Draws indicators on a span at the location of each event, skipping events
 * if they're too tightly packed but always drawing error events.
 * @param {!endo.Span} span The span to draw events for.
 * @param {!endo.Rect} clipRect The section of the screen we're drawing.
 */
endo.Timeline.prototype.drawEvents = function(span, clipRect) {
  var snapshot = endo.network.snapshot;

  if (span.events.length == 0) { return; }

  var canvas = this.canvas_;

  var prevScreenX = null;

  for (var i = 0; i < span.events.length; i++) {
    var event = span.events[i];
    var screenX = this.timeToPixel(event.time);
    screenX = Math.round(screenX);
    if (screenX < 0) { continue; }
    if (screenX > canvas.width) { continue; }

    // Skip events if they're packed too close to previous events, unless
    // they're a special event which we always want to draw.
    if (prevScreenX &&
       ((screenX - prevScreenX) < 2) &&
       (event.type != endo.Atom.AtomType.ERROR) &&
       (event != this.hoverEvent_)) continue;
    prevScreenX = screenX;

    var spanMarker = snapshot.getMarker(span.markerId);
    var color = spanMarker ? spanMarker.colorBorder : '#888';
    var eventMarker = snapshot.getMarker(event.markerId);
    if (eventMarker) { color = eventMarker.colorBorder; }

    // ERROR events are always displayed in pulsing yellow.
    if (event.type == endo.Atom.AtomType.ERROR) {
      color = endo.util.pulseHue(endo.Config.HUE_YELLOW, 2);
    }

    var spanHeight = this.spanHeight_;
    var y0 = Math.round(span.screenRect.y0 + 1);
    var y1 = Math.min(span.screenRect.y0 + spanHeight - 2, clipRect.y1);

    endo.draw.fillRect(color, screenX, y0, 2, spanHeight - 1);
  }
};

//java/com/google/net/rpc3/profiling/jendoscope/web/endo_bargraph.js
// Copyright 2011 Google Inc. All Rights Reserved.

/**
 * @fileoverview Endoscope's bargraph view groups tasks by marker and displays
 * real-time bar graphs of how long those tasks are taking to execute.
 *
 * TODO(aappleby): This view is a stub - finish implementing it.
 *
 * TODO(aappleby): This view shouldn't be inheriting directly from timeline,
 * there should be a shared view class that takes care of most of the graph
 * details.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('endo.BarGraph');

goog.require('endo.Range');
goog.require('endo.Rect');
goog.require('endo.Timeline');



/**
 * Endo's bar graph view, shows tasks grouped by marker in bar graph form.
 * @param {!HTMLCanvasElement} canvas The canvas to attach the view to.
 * @constructor
 * @struct
 * @extends {endo.Timeline}
 */
endo.BarGraph = function(canvas) {
  goog.base(this, canvas);
};
goog.inherits(endo.BarGraph, endo.Timeline);


/**
 * Resets the bar graph view so that it displays the first 10 markers in the
 * snapshot with the time range set to 10 seconds and a 1/5th row border on the
 * top and bottom.
 */
endo.BarGraph.prototype.resetViewport = function() {
  var rowMin = 0;
  var rowMax = 10;
  var timeMin = 0;
  var timeMax = 10;
  var borderY = 0.2;  // 1/5th row border on the top and bottom.

  this.viewport.set(timeMin,
                    rowMin - borderY,
                    timeMax,
                    rowMax + borderY);

  this.viewportGoal.set(timeMin,
                        rowMin - borderY,
                        timeMax,
                        rowMax + borderY);
};


/**
 * Advances the viewport animations by timestep dt.
 * @param {number} dt The timestep to advance our animations by.
 */
endo.BarGraph.prototype.update = function(dt) {
  // Snap the left side of the viewport to time 0.
  this.viewport.x1 -= this.viewport.x0;
  this.viewport.x0 = 0;
  this.viewportGoal.x1 -= this.viewportGoal.x0;
  this.viewportGoal.x0 = 0;

  endo.util.temporalEaseRect(this.viewport,
                             this.viewportGoal,
                             endo.Config.viewEaseTime,
                             dt);
};


/**
 * Renders the bar graph view.
 */
endo.BarGraph.prototype.render = function() {
  this.prerender();
  var canvas = this.getCanvas();
  var viewport = this.viewport;
  var snapshot = endo.network.snapshot;

  // Draw our background grid.
  var timeRange = new endo.Range(0, 10);
  var darkGreen = '#002414';
  var mediumGreen = '#003418';

  endo.draw.drawGridBackground(viewport, darkGreen, mediumGreen,
                          timeRange.min, timeRange.max);
  endo.draw.drawGridLinesX(viewport);
  endo.draw.drawGridLinesY(viewport);

  var markerLabels = [];

  if (snapshot) {
    var rowCursor = 0;
    var markers = snapshot.getMarkerList();
    for (var j = 0; j < markers.length; j++) {
      var marker = markers[j];
      if (!marker.visible) continue;
      markerLabels.push(marker.name);

      // Compute the screen-space Y bounds of the row we're drawing spans in.
      var rowMin = endo.util.remap(rowCursor,
                                   viewport.y0, viewport.y1,
                                   0, canvas.height);
      var rowMax = endo.util.remap(rowCursor + 1,
                                   viewport.y0, viewport.y1,
                                   0, canvas.height);

      // Add padding between row border and span rectangles. 15 pixels of
      // padding on the top is enough space to fit the thread label, 3 pixels
      // of paddding on the bottom looks nice.
      rowMin += 15;
      rowMax -= 3;

      // Draw some dummy bars into the bargraph view so we can test rendering
      // speed.
      // TODO(aappleby): Put real data here.
      var rowRect = new endo.Rect(0, rowMin, canvas.width, rowMax);
      var barHeight = 1;

      for (var i = 0; i < 100; i++) {
        // Dummy duration value that draw sine waves.
        var duration = 5 + (Math.sin(i / 3 + endo.sync.unixTime()) + 1) * 1;
        var barYMin = rowMin + i * barHeight;
        var barYMax = barYMin + barHeight;
        var barXMin = endo.util.timeToPixel(0, viewport.x0, viewport.x1,
                                            canvas);
        var barXMax = endo.util.timeToPixel(duration, viewport.x0, viewport.x1,
                                            canvas);
        var barRect = new endo.Rect(barXMin, barYMin, barXMax, barYMax);
        barRect.clipTo(rowRect);
        if (barRect.isEmpty()) break;
        endo.draw.drawBar(barRect, marker.colorFill);
      }

      ++rowCursor;
    }
  }

  // Draw the grid labels last so that the span trees don't occlude them.
  endo.draw.drawGridLabelsY(viewport);
  endo.draw.drawGridLabelsX(viewport, markerLabels, false);
};

//java/com/google/net/rpc3/profiling/jendoscope/web/endo_config.js
// Copyright 2011 Google Inc. All Rights Reserved.

/**
 * @fileoverview All configuration options for the Endoscope client live here.
 *
 * This file declares the 'endo' namespace and defines configuration properties
 * for the rest of the Endoscope client - it _must_ be the first file loaded by
 * the client.
 *
 * Most of the configuration properties defined here are _not_ guaranteed to
 * remain constant - future versions of the client could add a configuration
 * panel that would allow users to customize these properties.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('endo.Config');



/**
 * Global configuration options for Endoscope. All hardcoded magic numbers
 * should be moved here.
 */
endo.Config = {
  // True if we're in debug mode.
  debug: false,

  // Zoom limits for timeline view.
  minRowPixels: 4,
  maxRowPixels: 300,
  minViewWidth: 1.0e-6,
  maxViewWidth: 1.0e6,

  drawBackground: 1,

  // Default zoom & pan speeds
  panSpeed: 0.1,
  zoomSpeed: 1.5,

  // Amount of padding to leave at the sides of the timeline view.
  defaultViewPadding: 0.05,

  // View changes ease towards their goal at a rate of 90% every 80
  // milliseconds.
  viewEaseTime: 0.08,

  // Font used in all spans on the timeline.
  timelineFont: 'bold 11px sans-serif',
  timelineFontHeight: 12,

  // Number of pixels of space to leave on the sides of text
  textPaddingX: 3,

  // Fill colors for the timeline background.
  timelineBackgroundDark: '#001840',
  timelineBackgroundMedium: '#002050',

  // Tasks on the timeline smaller and closer than 2.5 pixels will be merged
  // into a proxy.
  proxyThreshold: 2.5,
  proxyColor: '#AAA',
  proxyHeight: 15,

  // Minimum and maximum height in pixels of a span drawn on the timeline.
  // Spans may be drawn smaller than spanHeightMin if they're clipped.
  spanHeightMin: 15,
  spanHeightMax: 15, // This setting turns off the 2nd line completely.
                     // In JEndoscope, line 2 of a span is not utilized.

  // Minimum and maximum size in pixels during which we fade span text in or
  // out.
  spanTextFadeMinX: 10,
  spanTextFadeMaxX: 40,

  spanTextFadeMinY: 5,
  spanTextFadeMaxY: 11,

  // Minimum and maximum number of spans stacked vertically that we want
  // visible in a timeline row before we clip or clamp.
  spansPerRowMin: 4,
  spansPerRowMax: 10,

  // Minimum size below which we don't draw detail elements on a span.
  spanDetailThresholdWidth: 30,
  spanDetailThresholdHeight: 10,

  // Minimum size below which we don't check for mouse hover on a span.
  spanHoverThresholdWidth: 2,
  spanHoverThresholdHeight: 5,

  // Sizes of the black backdrops underneath the thread names labels & the
  // time labels.
  threadNameBackdropWidth: 120,
  timeBackdropHeight: 14,

  // Color of the thread name labels & how many pixels to offset them from
  // the edges of the backdrop.
  threadNameColor: 'rgba(220,220,128,1.0)',

  // Distance in pixels the mouse can be from an event before we show event
  // details during hover.
  eventHoverThreshold: 12,

  // Size and color of the highlight box we draw around the span under the
  // mouse cursor.
  spanHoverBorder: 4,
  spanHoverHighlight1: 'rgba(255,255,200,0.2)',
  spanHoverHighlight2: 'rgba(255,255,200,0.5)',

  // Color of the dimmed spans. We use it when its tag doesn't match that
  // of the hovered span.
  spanDimColor: '#1d3f73',

  // Color of the selection rectangle.
  selectionHighlight1: 'rgba(200,255,200,0.2)',
  selectionHighlight2: 'rgba(200,255,200,0.5)',

  // Color of the measurement rectangle.
  measureHighlight1: 'rgba(255,0,0,0.2)',
  measureHighlight2: 'rgba(255,0,0,0.5)',

  // Color we use to highlight an event under the mouse cursor.
  intervalHighlight: '#fff',

  // Testing-related options
  createRandomTestEvents: false,

  // Convenience definitions for hues
  HUE_RED: 0 / 12,
  HUE_ORANGE: 1 / 12,
  HUE_YELLOW: 2 / 12,
  HUE_LIME: 3 / 12,
  HUE_GREEN: 4 / 12,
  HUE_MINT: 5 / 12,
  HUE_TEAL: 6 / 12,
  HUE_SKY: 7 / 12,
  HUE_BLUE: 8 / 12,
  HUE_PURPLE: 9 / 12,
  HUE_MAGENTA: 10 / 12,
  HUE_PINK: 11 / 12,

  // Default zoom level on y-axis
  // i.e. how many levels of nested tasks can be seen for each thread
  // i.e. the height of each thread measured by number of spans
  defaultZoomLevelY: 3.5
};

//net/proto2/contrib/js_proto/public/binary_constants.js
/**
 * @fileoverview This file contains constants used by jsproto.BinaryReader and
 * BinaryWriter.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('jsproto.BinaryConstants');


/**
 * Field type codes, taken from proto2/public/wire_format_lite.h.
 * @enum {number}
 */
jsproto.BinaryConstants.FieldType = {
  INVALID: -1,
  DOUBLE: 1,
  FLOAT: 2,
  INT64: 3,
  UINT64: 4,
  INT32: 5,
  FIXED64: 6,
  FIXED32: 7,
  BOOL: 8,
  STRING: 9,
  GROUP: 10,
  MESSAGE: 11,
  BYTES: 12,
  UINT32: 13,
  ENUM: 14,
  SFIXED32: 15,
  SFIXED64: 16,
  SINT32: 17,
  SINT64: 18,

  // Extended types for Javascript

  FHASH64: 30, // 64-bit hash string, fixed-length encoding.
  VHASH64: 31  // 64-bit hash string, varint encoding.
};


/**
 * Wire-format type codes, taken from proto2/public/wire_format_lite.h.
 * @enum {number}
 */
jsproto.BinaryConstants.WireType = {
  INVALID: -1,
  VARINT: 0,
  FIXED64: 1,
  DELIMITED: 2,
  START_GROUP: 3,
  END_GROUP: 4,
  FIXED32: 5
};


/**
 * Translates field type to wire type.
 * @param {jsproto.BinaryConstants.FieldType} fieldType
 * @return {jsproto.BinaryConstants.WireType}
 */
jsproto.BinaryConstants.FieldTypeToWireType = function(fieldType) {
  var fieldTypes = jsproto.BinaryConstants.FieldType;
  var wireTypes = jsproto.BinaryConstants.WireType;
  switch (fieldType) {
    case fieldTypes.INT32:
    case fieldTypes.INT64:
    case fieldTypes.UINT32:
    case fieldTypes.UINT64:
    case fieldTypes.SINT32:
    case fieldTypes.SINT64:
    case fieldTypes.BOOL:
    case fieldTypes.ENUM:
    case fieldTypes.VHASH64:
      return wireTypes.VARINT;

    case fieldTypes.DOUBLE:
    case fieldTypes.FIXED64:
    case fieldTypes.SFIXED64:
    case fieldTypes.FHASH64:
      return wireTypes.FIXED64;

    case fieldTypes.STRING:
    case fieldTypes.MESSAGE:
    case fieldTypes.BYTES:
      return wireTypes.DELIMITED;

    case fieldTypes.FLOAT:
    case fieldTypes.FIXED32:
    case fieldTypes.SFIXED32:
      return wireTypes.FIXED32;

    case fieldTypes.INVALID:
    case fieldTypes.GROUP:
    default:
      return wireTypes.INVALID;
  }
};


/**
 * Flag to indicate a missing field.
 * @const {number}
 */
jsproto.BinaryConstants.INVALID_FIELD_NUMBER = -1;


/**
 * The smallest denormal float32 value.
 * @const {number}
 */
jsproto.BinaryConstants.FLOAT32_EPS = 1.401298464324817e-45;


/**
 * The smallest normal float64 value.
 * @const {number}
 */
jsproto.BinaryConstants.FLOAT32_MIN = 1.1754943508222875e-38;


/**
 * The largest finite float32 value.
 * @const {number}
 */
jsproto.BinaryConstants.FLOAT32_MAX = 3.4028234663852886e+38;


/**
 * The smallest denormal float64 value.
 * @const {number}
 */
jsproto.BinaryConstants.FLOAT64_EPS = 5e-324;


/**
 * The smallest normal float64 value.
 * @const {number}
 */
jsproto.BinaryConstants.FLOAT64_MIN = 2.2250738585072014e-308;


/**
 * The largest finite float64 value.
 * @const {number}
 */
jsproto.BinaryConstants.FLOAT64_MAX = 1.7976931348623157e+308;


/**
 * Convenience constant equal to 2^20.
 * @const {number}
 */
jsproto.BinaryConstants.TWO_TO_20 = 1048576;


/**
 * Convenience constant equal to 2^23.
 * @const {number}
 */
jsproto.BinaryConstants.TWO_TO_23 = 8388608;


/**
 * Convenience constant equal to 2^31.
 * @const {number}
 */
jsproto.BinaryConstants.TWO_TO_31 = 2147483648;


/**
 * Convenience constant equal to 2^32.
 * @const {number}
 */
jsproto.BinaryConstants.TWO_TO_32 = 4294967296;


/**
 * Convenience constant equal to 2^52.
 * @const {number}
 */
jsproto.BinaryConstants.TWO_TO_52 = 4503599627370496;


/**
 * Convenience constant equal to 2^63.
 * @const {number}
 */
jsproto.BinaryConstants.TWO_TO_63 = 9223372036854775808;


/**
 * Convenience constant equal to 2^64.
 * @const {number}
 */
jsproto.BinaryConstants.TWO_TO_64 = 18446744073709551616;


/**
 * Eight-character string of zeros, used as the default 64-bit hash value.
 * @const {string}
 */
jsproto.BinaryConstants.ZERO_HASH = '\0\0\0\0\0\0\0\0';

//net/proto2/contrib/js_proto/public/binary_utils.js
/**
 * @fileoverview This file contains helper code used by jsproto.BinaryReader
 * and BinaryWriter.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('jsproto.utils');

goog.require('goog.string');
goog.require('jsproto.BinaryConstants');


/**
 * Javascript can't natively handle 64-bit data types, so to manipulate them we
 * have to split them into two 32-bit halves and do the math manually.
 *
 * Instead of instantiating and passing small structures around to do this, we
 * instead just use two global temporary values. This one stores the low 32
 * bits of a split value - for example, if the original value was a 64-bit
 * integer, this temporary value will contain the low 32 bits of that integer.
 * If the original value was a double, this temporary value will contain the
 * low 32 bits of the binary representation of that double, etcetera.
 * @type {number}
 */
jsproto.utils.split64Low = 0;


/**
 * And correspondingly, this temporary variable will contain the high 32 bits
 * of whatever value was split.
 * @type {number}
 */
jsproto.utils.split64High = 0;


/**
 * Splits an unsigned Javascript integer into two 32-bit halves and stores it
 * in the temp values above.
 * @param {number} value The number to split.
 */
jsproto.utils.splitUint64 = function(value) {
  // Extract low 32 bits and high 32 bits as unsigned integers.
  var lowBits = value >>> 0;
  var highBits = Math.floor((value - lowBits) /
                            jsproto.BinaryConstants.TWO_TO_32) >>> 0;

  jsproto.utils.split64Low = lowBits;
  jsproto.utils.split64High = highBits;
};


/**
 * Splits a signed Javascript integer into two 32-bit halves and stores it in
 * the temp values above.
 * @param {number} value The number to split.
 */
jsproto.utils.splitInt64 = function(value) {
  // Convert to sign-magnitude representation.
  var sign = (value < 0);
  value = Math.abs(value);

  // Extract low 32 bits and high 32 bits as unsigned integers.
  var lowBits = value >>> 0;
  var highBits = Math.floor((value - lowBits) /
                            jsproto.BinaryConstants.TWO_TO_32);
  highBits = highBits >>> 0;

  // Perform two's complement conversion if the sign bit was set.
  if (sign) {
    highBits = ~highBits >>> 0;
    lowBits = ~lowBits >>> 0;
    lowBits += 1;
    if (lowBits > 0xFFFFFFFF) {
      lowBits = 0;
      highBits++;
      if (highBits > 0xFFFFFFFF) highBits = 0;
    }
  }

  jsproto.utils.split64Low = lowBits;
  jsproto.utils.split64High = highBits;
};


/**
 * Convers a signed Javascript integer into zigzag format, splits it into two
 * 32-bit halves, and stores it in the temp values above.
 * @param {number} value The number to split.
 */
jsproto.utils.splitZigzag64 = function(value) {
  // Convert to sign-magnitude and scale by 2 before we split the value.
  var sign = (value < 0);
  value = Math.abs(value) * 2;

  jsproto.utils.splitUint64(value);
  var lowBits = jsproto.utils.split64Low;
  var highBits = jsproto.utils.split64High;

  // If the value is negative, subtract 1 from the split representation so we
  // don't lose the sign bit due to precision issues.
  if (sign) {
    if (lowBits == 0) {
      if (highBits == 0) {
        lowBits = 0xFFFFFFFF;
        highBits = 0xFFFFFFFF;
      } else {
        highBits--;
        lowBits = 0xFFFFFFFF;
      }
    } else {
      lowBits--;
    }
  }

  jsproto.utils.split64Low = lowBits;
  jsproto.utils.split64High = highBits;
};


/**
 * Converts a floating-point number into 32-bit IEEE representation and stores
 * it in the temp values above.
 * @param {number} value
 */
jsproto.utils.splitFloat32 = function(value) {
  var sign = (value < 0) ? 1 : 0;
  value = sign ? -value : value;
  var exp;
  var mant;

  // Handle zeros.
  if (value === 0) {
    if ((1 / value) > 0) {
      // Positive zero.
      jsproto.utils.split64High = 0;
      jsproto.utils.split64Low = 0x00000000;
    } else {
      // Negative zero.
      jsproto.utils.split64High = 0;
      jsproto.utils.split64Low = 0x80000000;
    }
    return;
  }

  // Handle nans.
  if (isNaN(value)) {
    jsproto.utils.split64High = 0;
    jsproto.utils.split64Low = 0x7FFFFFFF;
    return;
  }

  // Handle infinities.
  if (value > jsproto.BinaryConstants.FLOAT32_MAX) {
    jsproto.utils.split64High = 0;
    jsproto.utils.split64Low = ((sign << 31) | (0x7F800000)) >>> 0;
    return;
  }

  // Handle denormals.
  if (value < jsproto.BinaryConstants.FLOAT32_MIN) {
    // Number is a denormal.
    mant = Math.round(value / Math.pow(2, -149));
    jsproto.utils.split64High = 0;
    jsproto.utils.split64Low = ((sign << 31) | mant) >>> 0;
    return;
  }

  exp = Math.floor(Math.log(value) / Math.LN2);
  mant = value * Math.pow(2, -exp);
  mant = Math.round(mant * jsproto.BinaryConstants.TWO_TO_23) & 0x7FFFFF;

  jsproto.utils.split64High = 0;
  jsproto.utils.split64Low = ((sign << 31) | ((exp + 127) << 23) | mant) >>> 0;
};


/**
 * Converts a floating-point number into 64-bit IEEE representation and stores
 * it in the temp values above.
 * @param {number} value
 */
jsproto.utils.splitFloat64 = function(value) {
  var sign = (value < 0) ? 1 : 0;
  value = sign ? -value : value;

  // Handle zeros.
  if (value === 0) {
    if ((1 / value) > 0) {
      // Positive zero.
      jsproto.utils.split64High = 0x00000000;
      jsproto.utils.split64Low = 0x00000000;
    } else {
      // Negative zero.
      jsproto.utils.split64High = 0x80000000;
      jsproto.utils.split64Low = 0x00000000;
    }
    return;
  }

  // Handle nans.
  if (isNaN(value)) {
    jsproto.utils.split64High = 0x7FFFFFFF;
    jsproto.utils.split64Low = 0xFFFFFFFF;
    return;
  }

  // Handle infinities.
  if (value > jsproto.BinaryConstants.FLOAT64_MAX) {
    jsproto.utils.split64High = ((sign << 31) | (0x7FF00000)) >>> 0;
    jsproto.utils.split64Low = 0;
    return;
  }

  // Handle denormals.
  if (value < jsproto.BinaryConstants.FLOAT64_MIN) {
    // Number is a denormal.
    var mant = value / Math.pow(2, -1074);
    var mantHigh = (mant / jsproto.BinaryConstants.TWO_TO_32);
    jsproto.utils.split64High = ((sign << 31) | mantHigh) >>> 0;
    jsproto.utils.split64Low = (mant >>> 0);
    return;
  }

  var exp = Math.floor(Math.log(value) / Math.LN2);
  if (exp == 1024) exp = 1023;
  var mant = value * Math.pow(2, -exp);

  var mantHigh = (mant * jsproto.BinaryConstants.TWO_TO_20) & 0xFFFFF;
  var mantLow = (mant * jsproto.BinaryConstants.TWO_TO_52) >>> 0;

  jsproto.utils.split64High =
      ((sign << 31) | ((exp + 1023) << 20) | mantHigh) >>> 0;
  jsproto.utils.split64Low = mantLow;
};


/**
 * Converts an 8-character hash string into two 32-bit numbers and stores them
 * in the temp values above.
 * @param {string} hash
 */
jsproto.utils.splitHash64 = function(hash) {
  var a = hash.charCodeAt(0);
  var b = hash.charCodeAt(1);
  var c = hash.charCodeAt(2);
  var d = hash.charCodeAt(3);
  var e = hash.charCodeAt(4);
  var f = hash.charCodeAt(5);
  var g = hash.charCodeAt(6);
  var h = hash.charCodeAt(7);

  jsproto.utils.split64Low = (a + (b << 8) + (c << 16) + (d << 24)) >>> 0;
  jsproto.utils.split64High = (e + (f << 8) + (g << 16) + (h << 24)) >>> 0;
};


/**
 * Joins two 32-bit values into a 64-bit unsigned integer. Precision will be
 * lost if the result is greater than 2^52.
 * @param {number} bitsLow
 * @param {number} bitsHigh
 * @return {number}
 */
jsproto.utils.joinUint64 = function(bitsLow, bitsHigh) {
  return bitsHigh * jsproto.BinaryConstants.TWO_TO_32 + bitsLow;
};


/**
 * Joins two 32-bit values into a 64-bit signed integer. Precision will be lost
 * if the result is greater than 2^52.
 * @param {number} bitsLow
 * @param {number} bitsHigh
 * @return {number}
 */
jsproto.utils.joinInt64 = function(bitsLow, bitsHigh) {
  // If the high bit is set, do a manual two's complement conversion.
  var sign = (bitsHigh & 0x80000000);
  if (sign) {
    bitsLow = (~bitsLow + 1) >>> 0;
    bitsHigh = ~bitsHigh >>> 0;
    if (bitsLow == 0) {
      bitsHigh = (bitsHigh + 1) >>> 0;
    }
  }

  var result = jsproto.utils.joinUint64(bitsLow, bitsHigh);
  return sign ? -result : result;
};


/**
 * Joins two 32-bit values into a 64-bit unsigned integer and applies zigzag
 * decoding. Precision will be lost if the result is greater than 2^52.
 * @param {number} bitsLow
 * @param {number} bitsHigh
 * @return {number}
 */
jsproto.utils.joinZigzag64 = function(bitsLow, bitsHigh) {
  // Extract the sign bit and shift right by one.
  var sign = bitsLow & 1;
  bitsLow = ((bitsLow >>> 1) | (bitsHigh << 31)) >>> 0;
  bitsHigh = bitsHigh >>> 1;

  // Increment the split value if the sign bit was set.
  if (sign) {
    bitsLow = (bitsLow + 1) >>> 0;
    if (bitsLow == 0) {
      bitsHigh = (bitsHigh + 1) >>> 0;
    }
  }

  var result = jsproto.utils.joinUint64(bitsLow, bitsHigh);
  return sign ? -result : result;
};


/**
 * Joins two 32-bit values into a 32-bit IEEE floating point number and
 * converts it back into a Javascript number.
 * @param {number} bitsLow The low 32 bits of the binary number;
 * @param {number} bitsHigh The high 32 bits of the binary number.
 * @return {number}
 */
jsproto.utils.joinFloat32 = function(bitsLow, bitsHigh) {
  var sign = ((bitsLow >> 31) * 2 + 1);
  var exp = (bitsLow >>> 23) & 0xFF;
  var mant = bitsLow & 0x7FFFFF;

  if (exp == 0xFF) {
    if (mant) {
      return NaN;
    } else {
      return sign * Infinity;
    }
  }

  if (exp == 0) {
    // Denormal.
    return sign * Math.pow(2, -149) * mant;
  } else {
    return sign * Math.pow(2, exp - 150) *
           (mant + Math.pow(2, 23));
  }
};


/**
 * Joins two 32-bit values into a 64-bit IEEE floating point number and
 * converts it back into a Javascript number.
 * @param {number} bitsLow The low 32 bits of the binary number;
 * @param {number} bitsHigh The high 32 bits of the binary number.
 * @return {number}
 */
jsproto.utils.joinFloat64 = function(bitsLow, bitsHigh) {
  var sign = ((bitsHigh >> 31) * 2 + 1);
  var exp = (bitsHigh >>> 20) & 0x7FF;
  var mant = jsproto.BinaryConstants.TWO_TO_32 * (bitsHigh & 0xFFFFF) + bitsLow;

  if (exp == 0x7FF) {
    if (mant) {
      return NaN;
    } else {
      return sign * Infinity;
    }
  }

  if (exp == 0) {
    // Denormal.
    return sign * Math.pow(2, -1074) * mant;
  } else {
    return sign * Math.pow(2, exp - 1075) *
           (mant + jsproto.BinaryConstants.TWO_TO_52);
  }
};


/**
 * Joins two 32-bit values into an 8-character hash string.
 * @param {number} bitsLow
 * @param {number} bitsHigh
 * @return {string}
 */
jsproto.utils.joinHash64 = function(bitsLow, bitsHigh) {
  var a = (bitsLow >>> 0) & 0xFF;
  var b = (bitsLow >>> 8) & 0xFF;
  var c = (bitsLow >>> 16) & 0xFF;
  var d = (bitsLow >>> 24) & 0xFF;
  var e = (bitsHigh >>> 0) & 0xFF;
  var f = (bitsHigh >>> 8) & 0xFF;
  var g = (bitsHigh >>> 16) & 0xFF;
  var h = (bitsHigh >>> 24) & 0xFF;

  return String.fromCharCode(a, b, c, d, e, f, g, h);
};


/**
 * Individual digits for number->string conversion.
 * @const {!Array.<number>}
 */
jsproto.utils.DIGITS = [
  '0', '1', '2', '3', '4', '5', '6', '7',
  '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'
];


/**
 * Losslessly converts a 64-bit unsigned integer in 32:32 split representation
 * into a decimal string.
 * @param {number} bitsLow The low 32 bits of the binary number;
 * @param {number} bitsHigh The high 32 bits of the binary number.
 * @return {string} The binary number represented as a string.
 */
jsproto.utils.joinUnsignedDecimalString = function(bitsLow, bitsHigh) {
  // Skip the expensive conversion if the number is small enough to use the
  // built-in conversions.
  if (bitsHigh <= 0x1FFFFF) {
    return '' + (jsproto.BinaryConstants.TWO_TO_32 * bitsHigh + bitsLow);
  }

  // What this code is doing is essentially converting the input number from
  // base-2 to base-1e7, which allows us to represent the 64-bit range with
  // only 3 (very large) digits. Those digits are then trivial to convert to
  // a base-10 string.

  // The magic numbers used here are -
  // 2^24 = 16777216 = (1,6777216) in base-1e7.
  // 2^48 = 281474976710656 = (2,8147497,6710656) in base-1e7.

  // Split 32:32 representation into 16:24:24 representation so our
  // intermediate digits don't overflow.
  var low = bitsLow & 0xFFFFFF;
  var mid = (((bitsLow >>> 24) | (bitsHigh << 8)) >>> 0) & 0xFFFFFF;
  var high = (bitsHigh >> 16) & 0xFFFF;

  // Assemble our three base-1e7 digits, ignoring carries. The maximum
  // value in a digit at this step is representable as a 48-bit integer, which
  // can be stored in a 64-bit floating point number.
  var digitA = low + (mid * 6777216) + (high * 6710656);
  var digitB = mid + (high * 8147497);
  var digitC = (high * 2);

  // Apply carries from A to B and from B to C.
  var base = 10000000;
  if (digitA >= base) {
    digitB += Math.floor(digitA / base);
    digitA %= base;
  }

  if (digitB >= base) {
    digitC += Math.floor(digitB / base);
    digitB %= base;
  }

  // Convert base-1e7 digits to base-10, omitting leading zeroes.
  var table = jsproto.utils.DIGITS;
  var start = false;
  var result = '';

  function emit(digit) {
    var temp = base;
    for (var i = 0; i < 7; i++) {
      temp /= 10;
      var decimalDigit = ((digit / temp) % 10) >>> 0;
      if ((decimalDigit == 0) && !start) continue;
      start = true;
      result += table[decimalDigit];
    }
  }

  if (digitC || start) emit(digitC);
  if (digitB || start) emit(digitB);
  if (digitA || start) emit(digitA);

  return result;
};


/**
 * Losslessly converts a 64-bit signed integer in 32:32 split representation
 * into a decimal string.
 * @param {number} bitsLow The low 32 bits of the binary number;
 * @param {number} bitsHigh The high 32 bits of the binary number.
 * @return {string} The binary number represented as a string.
 */
jsproto.utils.joinSignedDecimalString = function(bitsLow, bitsHigh) {
  // If we're treating the input as a signed value and the high bit is set, do
  // a manual two's complement conversion before the decimal conversion.
  var negative = (bitsHigh & 0x80000000);
  if (negative) {
    bitsLow = (~bitsLow + 1) >>> 0;
    var carry = (bitsLow == 0) ? 1 : 0;
    bitsHigh = (~bitsHigh + carry) >>> 0;
  }

  var result = jsproto.utils.joinUnsignedDecimalString(bitsLow, bitsHigh);
  return negative ? '-' + result : result;
};


/**
 * Convert an 8-character hash string representing either a signed or unsigned
 * 64-bit integer into its decimal representation without losing accuracy.
 * @param {string} hash The hash string to convert.
 * @param {boolean} signed True if we should treat the hash string as encoding
 *     a signed integer.
 * @return {string}
 */
jsproto.utils.hash64ToDecimalString = function(hash, signed) {
  jsproto.utils.splitHash64(hash);
  var bitsLow = jsproto.utils.split64Low;
  var bitsHigh = jsproto.utils.split64High;
  return signed ?
      jsproto.utils.joinSignedDecimalString(bitsLow, bitsHigh) :
      jsproto.utils.joinUnsignedDecimalString(bitsLow, bitsHigh);
};


/**
 * Converts an array of 8-character hash strings into their decimal
 * representations.
 * @param {!Array.<string>} hashes The array of hash strings to convert.
 * @param {boolean} signed True if we should treat the hash string as encoding
 *     a signed integer.
 * @return {!Array.<string>}
 */
jsproto.utils.hash64ArrayToDecimalStrings = function(hashes, signed) {
  var result = new Array(hashes.length);
  for (var i = 0; i < hashes.length; i++) {
    result[i] = jsproto.utils.hash64ToDecimalString(hashes[i], signed);
  }
  return result;
};


/**
 * Converts an 8-character hash string into its hexadecimal representation.
 * @param {string} hash
 * @return {string}
 */
jsproto.utils.hash64ToHexString = function(hash) {
  var temp = new Array(18);
  temp[0] = '0';
  temp[1] = 'x';

  for (var i = 0; i < 8; i++) {
    var c = hash.charCodeAt(7 - i);
    temp[i * 2 + 2] = jsproto.utils.DIGITS[c >> 4];
    temp[i * 2 + 3] = jsproto.utils.DIGITS[c & 0xF];
  }

  var result = temp.join('');
  return result;
};


/**
 * Convert an 8-character hash string representing either a signed or unsigned
 * 64-bit integer into a Javascript number. Will lose accuracy if the result is
 * larger than 2^52.
 * @param {string} hash The hash string to convert.
 * @param {boolean} signed True if the has should be interpreted as a signed
 *     number.
 * @return {number}
 */
jsproto.utils.hash64ToNumber = function(hash, signed) {
  jsproto.utils.splitHash64(hash);
  var bitsLow = jsproto.utils.split64Low;
  var bitsHigh = jsproto.utils.split64High;
  return signed ? jsproto.utils.joinInt64(bitsLow, bitsHigh) :
                  jsproto.utils.joinUint64(bitsLow, bitsHigh);
};


/**
 * Convert a Javascript number into an 8-character hash string. Will lose
 * precision if the value is non-integral or greater than 2^64.
 * @param {number} value The integer to convert.
 * @return {string}
 */
jsproto.utils.numberToHash64 = function(value) {
  jsproto.utils.splitInt64(value);
  return jsproto.utils.joinHash64(jsproto.utils.split64Low,
                                  jsproto.utils.split64High);
};


/**
 * Counts the number of contiguous varints in a buffer.
 * @param {!Uint8Array} buffer The buffer to scan.
 * @param {number} start The starting point in the buffer to scan.
 * @param {number} end The end point in the buffer to scan.
 * @return {number} The number of varints in the buffer.
 */
jsproto.utils.countVarints = function(buffer, start, end) {
  // Count how many high bits of each byte were set in the buffer.
  var count = 0;
  for (var i = start; i < end; i++) {
    count += buffer[i] >> 7;
  }

  // The number of varints in the buffer equals the size of the buffer minus
  // the number of non-terminal bytes in the buffer (those with the high bit
  // set).
  return (end - start) - count;
};


/**
 * Counts the number of contiguous varint fields with the given field number in
 * the buffer.
 * @param {!Uint8Array} buffer The buffer to scan.
 * @param {number} start The starting point in the buffer to scan.
 * @param {number} end The end point in the buffer to scan.
 * @param {number} field The field number to count.
 * @return {number} The number of matching fields in the buffer.
 */
jsproto.utils.countVarintFields = function(buffer, start, end, field) {
  var count = 0;
  var cursor = start;
  var tag = field * 8 + jsproto.BinaryConstants.WireType.VARINT;

  if (tag < 128) {
    // Single-byte field tag, we can use a slightly quicker count.
    while (cursor < end) {
      // Skip the field tag, or exit if we find a non-matching tag.
      if (buffer[cursor++] != tag) return count;

      // Field tag matches, we've found a valid field.
      count++;

      // Skip the varint.
      while (1) {
        var x = buffer[cursor++];
        if ((x & 0x80) == 0) break;
      }
    }
  } else {
    while (cursor < end) {
      // Skip the field tag, or exit if we find a non-matching tag.
      var temp = tag;
      while (temp > 128) {
        if (buffer[cursor] != ((temp & 0x7F) | 0x80)) return count;
        cursor++;
        temp >>= 7;
      }
      if (buffer[cursor++] != temp) return count;

      // Field tag matches, we've found a valid field.
      count++;

      // Skip the varint.
      while (1) {
        var x = buffer[cursor++];
        if ((x & 0x80) == 0) break;
      }
    }
  }
  return count;
};


/**
 * Counts the number of contiguous fixed32 fields with the given tag in the
 * buffer.
 * @param {!Uint8Array} buffer The buffer to scan.
 * @param {number} start The starting point in the buffer to scan.
 * @param {number} end The end point in the buffer to scan.
 * @param {number} tag The tag value to count.
 * @param {number} stride The number of bytes to skip per field.
 * @return {number} The number of fields with a matching tag in the buffer.
 * @private
 */
jsproto.utils.countFixedFields_ =
    function(buffer, start, end, tag, stride) {
  var count = 0;
  var cursor = start;

  if (tag < 128) {
    // Single-byte field tag, we can use a slightly quicker count.
    while (cursor < end) {
      // Skip the field tag, or exit if we find a non-matching tag.
      if (buffer[cursor++] != tag) return count;

      // Field tag matches, we've found a valid field.
      count++;

      // Skip the value.
      cursor += stride;
    }
  } else {
    while (cursor < end) {
      // Skip the field tag, or exit if we find a non-matching tag.
      var temp = tag;
      while (temp > 128) {
        if (buffer[cursor++] != ((temp & 0x7F) | 0x80)) return count;
        temp >>= 7;
      }
      if (buffer[cursor++] != temp) return count;

      // Field tag matches, we've found a valid field.
      count++;

      // Skip the value.
      cursor += stride;
    }
  }
  return count;
};


/**
 * Counts the number of contiguous fixed32 fields with the given field number
 * in the buffer.
 * @param {!Uint8Array} buffer The buffer to scan.
 * @param {number} start The starting point in the buffer to scan.
 * @param {number} end The end point in the buffer to scan.
 * @param {number} field The field number to count.
 * @return {number} The number of matching fields in the buffer.
 */
jsproto.utils.countFixed32Fields = function(buffer, start, end, field) {
  var tag = field * 8 + jsproto.BinaryConstants.WireType.FIXED32;
  return jsproto.utils.countFixedFields_(buffer, start, end, tag, 4);
};


/**
 * Counts the number of contiguous fixed64 fields with the given field number
 * in the buffer.
 * @param {!Uint8Array} buffer The buffer to scan.
 * @param {number} start The starting point in the buffer to scan.
 * @param {number} end The end point in the buffer to scan.
 * @param {number} field The field number to count
 * @return {number} The number of matching fields in the buffer.
 */
jsproto.utils.countFixed64Fields = function(buffer, start, end, field) {
  var tag = field * 8 + jsproto.BinaryConstants.WireType.FIXED64;
  return jsproto.utils.countFixedFields_(buffer, start, end, tag, 8);
};


/**
 * Counts the number of contiguous delimited fields with the given field number
 * in the buffer.
 * @param {!Uint8Array} buffer The buffer to scan.
 * @param {number} start The starting point in the buffer to scan.
 * @param {number} end The end point in the buffer to scan.
 * @param {number} field The field number to count.
 * @return {number} The number of matching fields in the buffer.
 */
jsproto.utils.countDelimitedFields = function(buffer, start, end, field) {
  var count = 0;
  var cursor = start;
  var tag = field * 8 + jsproto.BinaryConstants.WireType.DELIMITED;

  while (cursor < end) {
    // Skip the field tag, or exit if we find a non-matching tag.
    var temp = tag;
    while (temp > 128) {
      if (buffer[cursor++] != ((temp & 0x7F) | 0x80)) return count;
      temp >>= 7;
    }
    if (buffer[cursor++] != temp) return count;

    // Field tag matches, we've found a valid field.
    count++;

    // Decode the length prefix.
    var length = 0;
    var shift = 1;
    while (1) {
      temp = buffer[cursor++];
      length += (temp & 0x7f) * shift;
      shift *= 128;
      if ((temp & 0x80) == 0) break;
    }

    // Advance the cursor past the blob.
    cursor += length;
  }
  return count;
};


/**
 * String-ify bytes for text format. Should be optimized away in non-debug.
 * The returned string uses \xXX escapes for all values and is itself quoted.
 * [1, 31] serializes to '"\x01\x1f"'.
 * @param {Uint8Array} bytes The byte array to serialize.
 * @return {string} Stringified bytes for text format.
 */
jsproto.utils.debugBytesToTextFormat = function(bytes) {
  var s = '"';
  if (bytes) {
    for (var i = 0; i < bytes.length; i++) {
      s += '\\x';
      if (bytes[i] < 16) s += '0';
      s += bytes[i].toString(16);
    }
  }
  return s + '"';
};


/**
 * String-ify a scalar for text format. Should be optimized away in non-debug.
 * @param {string|number|boolean} scalar The scalar to stringify.
 * @return {string} Stringified scalar for text format.
 */
jsproto.utils.debugScalarToTextFormat = function(scalar) {
  if (goog.isString(scalar)) {
    return goog.string.quote(scalar);
  } else {
    return scalar.toString();
  }
};

//net/proto2/contrib/js_proto/public/binary_decoder.js
/**
 * @fileoverview This file contains utilities for decoding primitive values
 * (signed and unsigned integers, varints, booleans, enums, hashes, strings,
 * and raw bytes) embedded in Uint8Arrays into their corresponding Javascript
 * types.
 *
 * Major caveat - Javascript is unable to accurately represent integers larger
 * than 2^53 due to its use of a double-precision floating point format or all
 * numbers. If you need to guarantee that 64-bit values survive with all bits
 * intact, you _must_ read them using one of the Hash64 methods, which return
 * an 8-character string.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('jsproto.BinaryDecoder');

goog.require('goog.asserts');
goog.require('jsproto.utils');



/**
 * BinaryDecoder implements the decoders for all the wire types specified in
 * https://developers.google.com/protocol-buffers/docs/encoding.
 *
 * @param {Uint8Array=} opt_bytes The block of bytes we're reading from.
 * @param {number=} opt_start The optional offset to start reading at.
 * @param {number=} opt_length The optional length of the block to read -
 *     we'll throw an assertion if we go off the end of the block.
 * @param {function(this:jsproto.BinaryDecoder)=} opt_next The optional
 *     function to bind to the next() method.
 * @constructor
 * @struct
 */
jsproto.BinaryDecoder = function(opt_bytes, opt_start, opt_length, opt_next) {
  /**
   * Typed byte-wise view of the source buffer.
   * @private {Uint8Array}
   */
  this.bytes_ = null;

  /**
   * Start point of the block to read.
   * @private {number}
   */
  this.start_ = 0;

  /**
   * End point of the block to read.
   * @private {number}
   */
  this.end_ = 0;

  /**
   * Current read location in bytes_.
   * @private {number}
   */
  this.cursor_ = 0;

  /**
   * Temporary storage for the low 32 bits of 64-bit data types that we're
   * decoding.
   * @private {number}
   */
  this.tempLow_ = 0;

  /**
   * Temporary storage for the high 32 bits of 64-bit data types that we're
   * decoding.
   * @private {number}
   */
  this.tempHigh_ = 0;

  /**
   * Convenience next() function that can be bound to any of the various
   * read* methods so that clients don't have to know what sort of encoding a
   * blob of numbers is using. We always assign this a function in the
   * constructor so we don't get any accidental polymorphism.
   *
   * Note that the 'next' member behaves as if it's a method on BinaryDecoder
   * instances -
   *
   * while (!decoder.atEnd()) {
   *   result.push(decoder.next());
   * }
   *
   * @type {!function(this:jsproto.BinaryDecoder)}
   */
  this.next = goog.isDef(opt_next) ? opt_next : this.readUnsignedVarint32;

  if (opt_bytes) {
    this.setBlock(opt_bytes, opt_start, opt_length);
  }
};


/**
 * Global pool of BinaryDecoder instances.
 * @private {!Array.<!jsproto.BinaryDecoder>}
 */
jsproto.BinaryDecoder.instanceCache_ = [];


/**
 * Number of instances in the cache.
 * @private {number}
 */
jsproto.BinaryDecoder.instanceCount_ = 0;


/**
 * Pops an instance off the instance cache, or creates one if the cache is
 * empty.
 * @param {Uint8Array=} opt_bytes The block of bytes we're reading from.
 * @param {number=} opt_start The optional offset to start reading at.
 * @param {number=} opt_length The optional length of the block to read -
 *     we'll throw an assertion if we go off the end of the block.
 * @param {function(this:jsproto.BinaryDecoder)=} opt_next The optional
 *     function to bind to the next() method.
 * @return {!jsproto.BinaryDecoder}
 */
jsproto.BinaryDecoder.alloc =
    function(opt_bytes, opt_start, opt_length, opt_next) {
  if (jsproto.BinaryDecoder.instanceCount_) {
    jsproto.BinaryDecoder.instanceCount_--;
    var newDecoder = jsproto.BinaryDecoder.instanceCache_[
        jsproto.BinaryDecoder.instanceCount_];
    if (opt_bytes) {
      newDecoder.setBlock(opt_bytes, opt_start, opt_length);
      if (goog.isDef(opt_next)) {
        newDecoder.next = opt_next;
      } else {
        newDecoder.next = newDecoder.readUnsignedVarint32;
      }
    }
    return newDecoder;
  } else {
    return new jsproto.BinaryDecoder(opt_bytes, opt_start, opt_length,
                                     opt_next);
  }
};


/**
 * Puts this instance back in the instance cache.
 */
jsproto.BinaryDecoder.prototype.free = function() {
  this.clear();
  if (jsproto.BinaryDecoder.instanceCount_ < 100) {
    jsproto.BinaryDecoder.instanceCache_[
        jsproto.BinaryDecoder.instanceCount_] = this;
    jsproto.BinaryDecoder.instanceCount_++;
  }
};


/**
 * Makes a copy of this decoder.
 * @return {!jsproto.BinaryDecoder}
 */
jsproto.BinaryDecoder.prototype.clone = function() {
  return jsproto.BinaryDecoder.alloc(this.bytes_,
      this.start_, this.end_ - this.start_, this.next);
};


/**
 * Clears the decoder.
 */
jsproto.BinaryDecoder.prototype.clear = function() {
  this.bytes_ = null;
  this.start_ = 0;
  this.end_ = 0;
  this.cursor_ = 0;
};


/**
 * Returns the raw buffer.
 * @return {Uint8Array} The raw buffer.
 */
jsproto.BinaryDecoder.prototype.getBuffer = function() {
  return this.bytes_;
};


/**
 * Changes the block of bytes we're decoding.
 * @param {!Uint8Array} bytes The block of bytes we're reading from.
 * @param {number=} opt_start The optional offset to start reading at.
 * @param {number=} opt_length The optional length of the block to read -
 *     we'll throw an assertion if we go off the end of the block.
 */
jsproto.BinaryDecoder.prototype.setBlock =
    function(bytes, opt_start, opt_length) {
  this.bytes_ = bytes;
  this.start_ = goog.isDef(opt_start) ? opt_start : 0;
  this.end_ =
      goog.isDef(opt_length) ? this.start_ + opt_length : this.bytes_.length;
  this.cursor_ = this.start_;
};


/**
 * @param {number} end
 */
jsproto.BinaryDecoder.prototype.setEnd = function(end) {
  this.end_ = end;
};


/**
 * Moves the read cursor back to the start of the block.
 */
jsproto.BinaryDecoder.prototype.reset = function() {
  this.cursor_ = this.start_;
};


/**
 * Returns the internal read cursor.
 * @return {number} The internal read cursor.
 */
jsproto.BinaryDecoder.prototype.getCursor = function() {
  return this.cursor_;
};


/**
 * Returns the internal read cursor.
 * @param {number} cursor The new cursor.
 */
jsproto.BinaryDecoder.prototype.setCursor = function(cursor) {
  this.cursor_ = cursor;
};


/**
 * Advances the stream cursor by the given number of bytes.
 * @param {number} count The number of bytes to advance by.
 */
jsproto.BinaryDecoder.prototype.advance = function(count) {
  this.cursor_ += count;
  goog.asserts.assert(this.cursor_ <= this.end_);
};


/**
 * Returns true if this decoder is at the end of the block.
 * @return {boolean}
 */
jsproto.BinaryDecoder.prototype.atEnd = function() {
  return this.cursor_ == this.end_;
};


/**
 * Returns true if this decoder is at the end of the block.
 * @return {boolean}
 */
jsproto.BinaryDecoder.prototype.pastEnd = function() {
  return this.cursor_ > this.end_;
};


/**
 * Reads an unsigned varint from the binary stream and stores it as a split
 * 64-bit integer. Since this does not convert the value to a number, no
 * precision is lost.
 *
 * It's possible for an unsigned varint to be incorrectly encoded - more than
 * 64 bits' worth of data could be present. If this happens, this method will
 * throw an error.
 *
 * Decoding varints requires doing some funny base-128 math - for more
 * details on the format, see
 * https://developers.google.com/protocol-buffers/docs/encoding
 *
 * @private
 */
jsproto.BinaryDecoder.prototype.readSplitVarint64_ = function() {
  var temp;
  var lowBits = 0;
  var highBits = 0;

  // Read the first four bytes of the varint, stopping at the terminator if we
  // see it.
  for (var i = 0; i < 4; i++) {
    temp = this.bytes_[this.cursor_++];
    lowBits |= (temp & 0x7F) << (i * 7);
    if (temp < 128) {
      this.tempLow_ = lowBits >>> 0;
      this.tempHigh_ = 0;
      return;
    }
  }

  // Read the fifth byte, which straddles the low and high dwords.
  temp = this.bytes_[this.cursor_++];
  lowBits |= (temp & 0x7F) << 28;
  highBits |= (temp & 0x7F) >> 4;
  if (temp < 128) {
    this.tempLow_ = lowBits >>> 0;
    this.tempHigh_ = highBits >>> 0;
    return;
  }

  // Read the sixth through tenth byte.
  for (var i = 0; i < 5; i++) {
    temp = this.bytes_[this.cursor_++];
    highBits |= (temp & 0x7F) << (i * 7 + 3);
    if (temp < 128) {
      this.tempLow_ = lowBits >>> 0;
      this.tempHigh_ = highBits >>> 0;
      return;
    }
  }

  // If we did not see the terminator, the encoding was invalid.
  goog.asserts.fail('Failed to read varint, encoding is invalid.');
};


/**
 * Skips over a varint in the block without decoding it.
 */
jsproto.BinaryDecoder.prototype.skipVarint = function() {
  while (this.bytes_[this.cursor_] & 0x80) {
    this.cursor_++;
  }
  this.cursor_++;
};


/**
 * Skips backwards over a varint in the block - to do this correctly, we have
 * to know the value we're skipping backwards over or things are ambiguous.
 * @param {number} value The varint value to unskip.
 */
jsproto.BinaryDecoder.prototype.unskipVarint = function(value) {
  while (value > 128) {
    this.cursor_--;
    value = value >>> 7;
  }
  this.cursor_--;
};


/**
 * Reads a 32-bit varint from the binary stream. Due to a quirk of the encoding
 * format and Javascript's handling of bitwise math, this actually works
 * correctly for both signed and unsigned 32-bit varints.
 *
 * This function is called vastly more frequently than any other in
 * BinaryDecoder, so it has been unrolled and tweaked for performance.
 *
 * If there are more than 32 bits of data in the varint, it _must_ be due to
 * sign-extension. If we're in debug mode and the high 32 bits don't match the
 * expected sign extension, this method will throw an error.
 *
 * Decoding varints requires doing some funny base-128 math - for more
 * details on the format, see
 * https://developers.google.com/protocol-buffers/docs/encoding
 *
 * @return {number} The decoded unsigned 32-bit varint.
 */
jsproto.BinaryDecoder.prototype.readUnsignedVarint32 = function() {
  var temp;
  var bytes = this.bytes_;

  temp = bytes[this.cursor_ + 0];
  var x = (temp & 0x7F);
  if (temp < 128) {
    this.cursor_ += 1;
    goog.asserts.assert(this.cursor_ <= this.end_);
    return x;
  }

  temp = bytes[this.cursor_ + 1];
  x |= (temp & 0x7F) << 7;
  if (temp < 128) {
    this.cursor_ += 2;
    goog.asserts.assert(this.cursor_ <= this.end_);
    return x;
  }

  temp = bytes[this.cursor_ + 2];
  x |= (temp & 0x7F) << 14;
  if (temp < 128) {
    this.cursor_ += 3;
    goog.asserts.assert(this.cursor_ <= this.end_);
    return x;
  }

  temp = bytes[this.cursor_ + 3];
  x |= (temp & 0x7F) << 21;
  if (temp < 128) {
    this.cursor_ += 4;
    goog.asserts.assert(this.cursor_ <= this.end_);
    return x;
  }

  temp = bytes[this.cursor_ + 4];
  x |= (temp & 0x0F) << 28;
  if (temp < 128) {
    // We're reading the high bits of an unsigned varint. The byte we just read
    // also contains bits 33 through 35, which we're going to discard. Those
    // bits _must_ be zero, or the encoding is invalid.
    goog.asserts.assert((temp & 0xF0) == 0);
    this.cursor_ += 5;
    goog.asserts.assert(this.cursor_ <= this.end_);
    return x >>> 0;
  }

  // If we get here, we're reading the sign extension of a negative 32-bit int.
  // We can skip these bytes, as we know in advance that they have to be all
  // 1's if the varint is correctly encoded. Since we also know the value is
  // negative, we don't have to coerce it to unsigned before we return it.

  goog.asserts.assert((temp & 0xF0) == 0xF0);
  goog.asserts.assert(bytes[this.cursor_ + 5] == 0xFF);
  goog.asserts.assert(bytes[this.cursor_ + 6] == 0xFF);
  goog.asserts.assert(bytes[this.cursor_ + 7] == 0xFF);
  goog.asserts.assert(bytes[this.cursor_ + 8] == 0xFF);
  goog.asserts.assert(bytes[this.cursor_ + 9] == 0x01);

  this.cursor_ += 10;
  goog.asserts.assert(this.cursor_ <= this.end_);
  return x;
};


/**
 * The readUnsignedVarint32 above deals with signed 32-bit varints correctly,
 * so this is just an alias.
 *
 * @return {number} The decoded signed 32-bit varint.
 */
jsproto.BinaryDecoder.prototype.readSignedVarint32 =
    jsproto.BinaryDecoder.prototype.readUnsignedVarint32;


/**
 * Reads a signed, zigzag-encoded 32-bit varint from the binary stream.
 *
 * Zigzag encoding is a modification of varint encoding that reduces the
 * storage overhead for small negative integers - for more details on the
 * format, see https://developers.google.com/protocol-buffers/docs/encoding
 *
 * @return {number} The decoded signed, zigzag-encoded 32-bit varint.
 */
jsproto.BinaryDecoder.prototype.readZigzagVarint32 = function() {
  var result = this.readUnsignedVarint32();
  return (result >>> 1) ^ - (result & 1);
};


/**
 * Reads an unsigned 64-bit varint from the binary stream. Note that since
 * Javascript represents all numbers as double-precision floats, there will be
 * precision lost if the absolute value of the varint is larger than 2^53.
 *
 * @return {number} The decoded unsigned varint. Precision will be lost if the
 *     integer exceeds 2^53.
 */
jsproto.BinaryDecoder.prototype.readUnsignedVarint64 = function() {
  this.readSplitVarint64_();
  return jsproto.utils.joinUint64(this.tempLow_, this.tempHigh_);
};


/**
 * Reads a signed 64-bit varint from the binary stream. Note that since
 * Javascript represents all numbers as double-precision floats, there will be
 * precision lost if the absolute value of the varint is larger than 2^53.
 *
 * @return {number} The decoded signed varint. Precision will be lost if the
 *     integer exceeds 2^53.
 */
jsproto.BinaryDecoder.prototype.readSignedVarint64 = function() {
  this.readSplitVarint64_();
  return jsproto.utils.joinInt64(this.tempLow_, this.tempHigh_);
};


/**
 * Reads a signed, zigzag-encoded 64-bit varint from the binary stream. Note
 * that since Javascript represents all numbers as double-precision floats,
 * there will be precision lost if the absolute value of the varint is larger
 * than 2^53.
 *
 * Zigzag encoding is a modification of varint encoding that reduces the
 * storage overhead for small negative integers - for more details on the
 * format, see https://developers.google.com/protocol-buffers/docs/encoding
 *
 * @return {number} The decoded zigzag varint. Precision will be lost if the
 *     integer exceeds 2^53.
 */
jsproto.BinaryDecoder.prototype.readZigzagVarint64 = function() {
  this.readSplitVarint64_();
  return jsproto.utils.joinZigzag64(this.tempLow_, this.tempHigh_);
};


/**
 * Reads a raw unsigned 8-bit integer from the binary stream.
 *
 * @return {number} The unsigned 8-bit integer read from the binary stream.
 */
jsproto.BinaryDecoder.prototype.readUint8 = function() {
  var a = this.bytes_[this.cursor_ + 0];
  this.cursor_ += 1;
  goog.asserts.assert(this.cursor_ <= this.end_);
  return a;
};


/**
 * Reads a raw unsigned 16-bit integer from the binary stream.
 *
 * @return {number} The unsigned 16-bit integer read from the binary stream.
 */
jsproto.BinaryDecoder.prototype.readUint16 = function() {
  var a = this.bytes_[this.cursor_ + 0];
  var b = this.bytes_[this.cursor_ + 1];
  this.cursor_ += 2;
  goog.asserts.assert(this.cursor_ <= this.end_);
  return (a << 0) | (b << 8);
};


/**
 * Reads a raw unsigned 32-bit integer from the binary stream.
 *
 * @return {number} The unsigned 32-bit integer read from the binary stream.
 */
jsproto.BinaryDecoder.prototype.readUint32 = function() {
  var a = this.bytes_[this.cursor_ + 0];
  var b = this.bytes_[this.cursor_ + 1];
  var c = this.bytes_[this.cursor_ + 2];
  var d = this.bytes_[this.cursor_ + 3];
  this.cursor_ += 4;
  goog.asserts.assert(this.cursor_ <= this.end_);
  return ((a << 0) | (b << 8) | (c << 16) | (d << 24)) >>> 0;
};


/**
 * Reads a raw unsigned 64-bit integer from the binary stream. Note that since
 * Javascript represents all numbers as double-precision floats, there will be
 * precision lost if the absolute value of the integer is larger than 2^53.
 *
 * @return {number} The unsigned 64-bit integer read from the binary stream.
 *     Precision will be lost if the integer exceeds 2^53.
 */
jsproto.BinaryDecoder.prototype.readUint64 = function() {
  var bitsLow = this.readUint32();
  var bitsHigh = this.readUint32();
  return jsproto.utils.joinUint64(bitsLow, bitsHigh);
};


/**
 * Reads a raw signed 8-bit integer from the binary stream.
 *
 * @return {number} The signed 8-bit integer read from the binary stream.
 */
jsproto.BinaryDecoder.prototype.readInt8 = function() {
  var a = this.bytes_[this.cursor_ + 0];
  this.cursor_ += 1;
  goog.asserts.assert(this.cursor_ <= this.end_);
  return (a << 24) >> 24;
};


/**
 * Reads a raw signed 16-bit integer from the binary stream.
 *
 * @return {number} The signed 16-bit integer read from the binary stream.
 */
jsproto.BinaryDecoder.prototype.readInt16 = function() {
  var a = this.bytes_[this.cursor_ + 0];
  var b = this.bytes_[this.cursor_ + 1];
  this.cursor_ += 2;
  goog.asserts.assert(this.cursor_ <= this.end_);
  return (((a << 0) | (b << 8)) << 16) >> 16;
};


/**
 * Reads a raw signed 32-bit integer from the binary stream.
 *
 * @return {number} The signed 32-bit integer read from the binary stream.
 */
jsproto.BinaryDecoder.prototype.readInt32 = function() {
  var a = this.bytes_[this.cursor_ + 0];
  var b = this.bytes_[this.cursor_ + 1];
  var c = this.bytes_[this.cursor_ + 2];
  var d = this.bytes_[this.cursor_ + 3];
  this.cursor_ += 4;
  goog.asserts.assert(this.cursor_ <= this.end_);
  return (a << 0) | (b << 8) | (c << 16) | (d << 24);
};


/**
 * Reads a raw signed 64-bit integer from the binary stream. Note that since
 * Javascript represents all numbers as double-precision floats, there will be
 * precision lost if the absolute vlaue of the integer is larger than 2^53.
 *
 * @return {number} The signed 64-bit integer read from the binary stream.
 *     Precision will be lost if the integer exceeds 2^53.
 */
jsproto.BinaryDecoder.prototype.readInt64 = function() {
  var bitsLow = this.readUint32();
  var bitsHigh = this.readUint32();
  return jsproto.utils.joinInt64(bitsLow, bitsHigh);
};


/**
 * Reads a 32-bit floating-point number from the binary stream, using the
 * temporary buffer to realign the data.
 *
 * @return {number} The float read from the binary stream.
 */
jsproto.BinaryDecoder.prototype.readFloat = function() {
  var bitsLow = this.readUint32();
  var bitsHigh = 0;
  return jsproto.utils.joinFloat32(bitsLow, bitsHigh);
};


/**
 * Reads a 64-bit floating-point number from the binary stream, using the
 * temporary buffer to realign the data.
 *
 * @return {number} The double read from the binary stream.
 */
jsproto.BinaryDecoder.prototype.readDouble = function() {
  var bitsLow = this.readUint32();
  var bitsHigh = this.readUint32();
  return jsproto.utils.joinFloat64(bitsLow, bitsHigh);
};


/**
 * Reads a boolean value from the binary stream.
 * @return {boolean} The boolean read from the binary stream.
 */
jsproto.BinaryDecoder.prototype.readBoolean = function() {
  return !!this.bytes_[this.cursor_++];
};


/**
 * Reads an enum value from the binary stream, which are always encoded as
 * signed varints.
 * @return {number} The enum value read from the binary stream.
 */
jsproto.BinaryDecoder.prototype.readEnum = function() {
  return this.readSignedVarint32();
};


/**
 * Reads and parses a UTF-8 encoded unicode string from the stream.
 * The code is inspired by maps.vectortown.parse.StreamedDataViewReader, with
 * the exception that the implementation here does not get confused if it
 * encounters characters longer than three bytes. These characters are ignored
 * though, as they are extremely rare: three UTF-8 bytes cover virtually all
 * characters in common use (http://en.wikipedia.org/wiki/UTF-8).
 * @param {number} length The length of the string to read.
 * @return {string} The decoded string.
 */
jsproto.BinaryDecoder.prototype.readString = function(length) {
  var bytes = this.bytes_;
  var cursor = this.cursor_;
  var end = cursor + length;
  var chars = [];

  while (cursor < end) {
    var c = bytes[cursor++];
    if (c < 128) { // Regular 7-bit ASCII.
      chars.push(c);
    } else if (c < 192) {
      // UTF-8 continuation mark. We are out of sync. This
      // might happen if we attempted to read a character
      // with more than three bytes.
      continue;
    } else if (c < 224) { // UTF-8 with two bytes.
      var c2 = bytes[cursor++];
      chars.push(((c & 31) << 6) | (c2 & 63));
    } else if (c < 240) { // UTF-8 with three bytes.
      var c2 = bytes[cursor++];
      var c3 = bytes[cursor++];
      chars.push(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
    }
  }

  // String.fromCharCode.apply is faster than manually appending characters on
  // Chrome 25+, and generates no additional cons string garbage.
  var result = String.fromCharCode.apply(null, chars);
  this.cursor_ = cursor;
  return result;
};


/**
 * Reads and parses a UTF-8 encoded unicode string (with length prefix) from
 * the stream.
 * @return {string} The decoded string.
 */
jsproto.BinaryDecoder.prototype.readStringWithLength = function() {
  var length = this.readUnsignedVarint32();
  return this.readString(length);
};


/**
 * Reads a block of raw bytes from the binary stream.
 *
 * @param {number} length The number of bytes to read.
 * @return {!Uint8Array} The decoded block of bytes.
 */
jsproto.BinaryDecoder.prototype.readBytes = function(length) {
  var result = new Uint8Array(this.bytes_.buffer,
                              this.bytes_.byteOffset + this.cursor_,
                              length);
  this.cursor_ += length;
  goog.asserts.assert(this.cursor_ <= this.end_);
  return result;
};


/**
 * Reads a 64-bit varint from the stream and returns it as an 8-character
 * Unicode string for use as a hash table key.
 *
 * @return {string} The hash value.
 */
jsproto.BinaryDecoder.prototype.readVarintHash64 = function() {
  this.readSplitVarint64_();
  return jsproto.utils.joinHash64(this.tempLow_, this.tempHigh_);
};


/**
 * Reads a 64-bit fixed-width value from the stream and returns it as an
 * 8-character Unicode string for use as a hash table key.
 *
 * @return {string} The hash value.
 */
jsproto.BinaryDecoder.prototype.readFixedHash64 = function() {
  var bitsLow = this.readUint32();
  var bitsHigh = this.readUint32();
  return jsproto.utils.joinHash64(bitsLow, bitsHigh);
};

//net/proto2/contrib/js_proto/public/binary_reader.js
/**
 * @fileoverview This file contains utilities for converting binary,
 * wire-format protocol buffers into Javascript data structures.
 *
 * jsproto's BinaryReader class wraps the BinaryDecoder class to add methods
 * that understand the protocol buffer syntax and can do the type checking and
 * bookkeeping necessary to parse trees of nested messages.
 *
 * Major caveat - Users of this library _must_ keep their Javascript proto
 * parsing code in sync with the original .proto file - presumably you'll be
 * using the typed jsproto code generator, but if you bypass that you'll need
 * to keep things in sync by hand.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('jsproto.BinaryReader');

goog.require('goog.asserts');
goog.require('jsproto.BinaryConstants');
goog.require('jsproto.BinaryDecoder');



/**
 * BinaryReader implements the decoders for all the wire types specified in
 * https://developers.google.com/protocol-buffers/docs/encoding.
 *
 * @param {Uint8Array=} opt_bytes The block of bytes we're reading from.
 * @param {number=} opt_start The optional offset to start reading at.
 * @param {number=} opt_length The optional length of the block to read -
 *     we'll throw an assertion if we go off the end of the block.
 * @constructor
 * @struct
 */
jsproto.BinaryReader = function(opt_bytes, opt_start, opt_length) {
  /**
   * Wire-format decoder.
   * @private {!jsproto.BinaryDecoder}
   */
  this.decoder_ = jsproto.BinaryDecoder.alloc(opt_bytes, opt_start, opt_length);

  /**
   * Cursor immediately before the field tag.
   * @private {number}
   */
  this.fieldCursor_ = this.decoder_.getCursor();

  /**
   * Field number of the next field in the buffer, filled in by nextField().
   * @private {number}
   */
  this.nextField_ = jsproto.BinaryConstants.INVALID_FIELD_NUMBER;

  /**
   * Wire type of the next proto field in the buffer, filled in by
   * nextField().
   * @private {jsproto.BinaryConstants.WireType}
   */
  this.nextWireType_ = jsproto.BinaryConstants.WireType.INVALID;

  /**
   * User-defined reader callbacks.
   * @private {Object.<string, function(!jsproto.BinaryReader):*>}
   */
  this.readCallbacks_ = null;
};


/**
 * Global pool of BinaryReader instances.
 * @private {!Array.<!jsproto.BinaryReader>}
 */
jsproto.BinaryReader.instanceCache_ = [];


/**
 * Number of instances in the cache.
 * @private {number}
 */
jsproto.BinaryReader.instanceCount_ = 0;


/**
 * Pops an instance off the instance cache, or creates one if the cache is
 * empty.
 * @param {Uint8Array=} opt_bytes The block of bytes we're reading from.
 * @param {number=} opt_start The optional offset to start reading at.
 * @param {number=} opt_length The optional length of the block to read -
 *     we'll throw an assertion if we go off the end of the block.
 * @return {!jsproto.BinaryReader}
 */
jsproto.BinaryReader.alloc =
    function(opt_bytes, opt_start, opt_length) {
  if (jsproto.BinaryReader.instanceCount_) {
    jsproto.BinaryReader.instanceCount_--;
    var newReader = jsproto.BinaryReader.instanceCache_[
        jsproto.BinaryReader.instanceCount_];
    if (opt_bytes) {
      newReader.decoder_.setBlock(opt_bytes, opt_start, opt_length);
    }
    return newReader;
  } else {
    return new jsproto.BinaryReader(opt_bytes, opt_start, opt_length);
  }
};


/**
 * Puts this instance back in the instance cache.
 */
jsproto.BinaryReader.prototype.free = function() {
  this.decoder_.clear();
  this.nextField_ = jsproto.BinaryConstants.INVALID_FIELD_NUMBER;
  this.nextWireType_ = jsproto.BinaryConstants.WireType.INVALID;
  this.readCallbacks_ = null;

  if (jsproto.BinaryReader.instanceCount_ < 100) {
    jsproto.BinaryReader.instanceCache_[
        jsproto.BinaryReader.instanceCount_] = this;
    jsproto.BinaryReader.instanceCount_++;
  }
};


/**
 * Returns the cursor immediately before the current field's tag.
 * @return {number} The internal read cursor.
 */
jsproto.BinaryReader.prototype.getFieldCursor = function() {
  return this.fieldCursor_;
};


/**
 * Returns the internal read cursor.
 * @return {number} The internal read cursor.
 */
jsproto.BinaryReader.prototype.getCursor = function() {
  return this.decoder_.getCursor();
};


/**
 * Returns the raw buffer.
 * @return {Uint8Array} The raw buffer.
 */
jsproto.BinaryReader.prototype.getBuffer = function() {
  return this.decoder_.getBuffer();
};


/**
 * @return {number} The field number of the next field in the buffer, or
 *     INVALID_FIELD_NUMBER if there is no next field.
 */
jsproto.BinaryReader.prototype.getFieldNumber = function() {
  return this.nextField_;
};


/**
 * @return {jsproto.BinaryConstants.WireType} The wire type of the next field
 *     in the stream, or WireType.INVALID if there is no next field.
 */
jsproto.BinaryReader.prototype.getWireType = function() {
  return this.nextWireType_;
};


/**
 * Points this reader at a new block of bytes.
 * @param {!Uint8Array} bytes The block of bytes we're reading from.
 * @param {number} start The offset to start reading at.
 * @param {number} length The length of the block to read.
 */
jsproto.BinaryReader.prototype.setBlock = function(bytes, start, length) {
  this.decoder_.setBlock(bytes, start, length);
  this.nextField_ = jsproto.BinaryConstants.INVALID_FIELD_NUMBER;
  this.nextWireType_ = jsproto.BinaryConstants.WireType.INVALID;
};


/**
 * Rewinds the stream cursor to the beginning of the buffer and resets all
 * internal state.
 */
jsproto.BinaryReader.prototype.reset = function() {
  this.decoder_.reset();
  this.nextField_ = jsproto.BinaryConstants.INVALID_FIELD_NUMBER;
  this.nextWireType_ = jsproto.BinaryConstants.WireType.INVALID;
};


/**
 * Advances the stream cursor by the given number of bytes.
 * @param {number} count The number of bytes to advance by.
 */
jsproto.BinaryReader.prototype.advance = function(count) {
  this.decoder_.advance(count);
};


/**
 * Reads the next field header in the stream if there is one, returns true if
 * we saw a valid field header or false if we've read the whole stream.
 * Throws an error if we encountered a deprecated START_GROUP/END_GROUP field.
 * @return {boolean} True if the stream contains more fields.
 */
jsproto.BinaryReader.prototype.nextField = function() {
  // If we're at the end of the block, there are no more fields.
  if (this.decoder_.atEnd()) {
    return false;
  }

  goog.asserts.assert(!this.decoder_.pastEnd());

  // Otherwise just read the header of the next field.
  this.fieldCursor_ = this.decoder_.getCursor();
  var header = this.decoder_.readUnsignedVarint32();
  this.nextField_ = header >>> 3;
  this.nextWireType_ = /** @type {jsproto.BinaryConstants.WireType} */
      (header & 0x7);

  goog.asserts.assert(
      (this.nextWireType_ != jsproto.BinaryConstants.WireType.START_GROUP) &&
      (this.nextWireType_ != jsproto.BinaryConstants.WireType.END_GROUP));

  return true;
};


/**
 * Winds the reader back to just before this field's header.
 */
jsproto.BinaryReader.prototype.unskipHeader = function() {
  this.decoder_.unskipVarint((this.nextField_ << 3) | this.nextWireType_);
};


/**
 * Skips all contiguous fields whose header matches the one we just read.
 */
jsproto.BinaryReader.prototype.skipMatchingFields = function() {
  var field = this.nextField_;
  this.unskipHeader();

  while (this.nextField() && (this.getFieldNumber() == field)) {
    this.skipField();
  }

  if (!this.decoder_.atEnd()) {
    this.unskipHeader();
  }
};


/**
 * Skips over the next varint field in the binary stream.
 */
jsproto.BinaryReader.prototype.skipVarintField = function() {
  if (this.nextWireType_ != jsproto.BinaryConstants.WireType.VARINT) {
    goog.asserts.fail('Invalid wire type for skipVarintField');
    this.skipField();
    return;
  }

  this.decoder_.skipVarint();
};


/**
 * Skips over the next delimited field in the binary stream.
 */
jsproto.BinaryReader.prototype.skipDelimitedField = function() {
  if (this.nextWireType_ != jsproto.BinaryConstants.WireType.DELIMITED) {
    goog.asserts.fail('Invalid wire type for skipDelimitedField');
    this.skipField();
    return;
  }

  var length = this.decoder_.readUnsignedVarint32();
  this.decoder_.advance(length);
};


/**
 * Skips over the next fixed32 field in the binary stream.
 */
jsproto.BinaryReader.prototype.skipFixed32Field = function() {
  if (this.nextWireType_ != jsproto.BinaryConstants.WireType.FIXED32) {
    goog.asserts.fail('Invalid wire type for skipFixed32Field');
    this.skipField();
    return;
  }

  this.decoder_.advance(4);
};


/**
 * Skips over the next fixed64 field in the binary stream.
 */
jsproto.BinaryReader.prototype.skipFixed64Field = function() {
  if (this.nextWireType_ != jsproto.BinaryConstants.WireType.FIXED64) {
    goog.asserts.fail('Invalid wire type for skipFixed64Field');
    this.skipField();
    return;
  }

  this.decoder_.advance(8);
};


/**
 * Skips over the next field in the binary stream - this is useful if we're
 * decoding a message that contain unknown fields.
 *
 * Throws an error if we encounter the deprecated START_GROUP or END_GROUP wire
 * types.
 */
jsproto.BinaryReader.prototype.skipField = function() {
  switch (this.nextWireType_) {
    case jsproto.BinaryConstants.WireType.VARINT:
      this.skipVarintField();
      break;
    case jsproto.BinaryConstants.WireType.FIXED64:
      this.skipFixed64Field();
      break;
    case jsproto.BinaryConstants.WireType.DELIMITED:
      this.skipDelimitedField();
      break;
    case jsproto.BinaryConstants.WireType.FIXED32:
      this.skipFixed32Field();
      break;
    default:
      goog.asserts.fail('Invalid wire encoding for field.');
  }
};


/**
 * Registers a user-defined read callback.
 * @param {string} callbackName
 * @param {function(!jsproto.BinaryReader):*} callback
 */
jsproto.BinaryReader.prototype.registerReadCallback =
    function(callbackName, callback) {
  if (goog.isNull(this.readCallbacks_)) {
    this.readCallbacks_ = {};
  }
  goog.asserts.assert(!this.readCallbacks_[callbackName]);
  this.readCallbacks_[callbackName] = callback;
};


/**
 * Runs a registered read callback.
 * @param {string} callbackName The name the callback is registered under.
 * @return {*} The value returned by the callback.
 */
jsproto.BinaryReader.prototype.runReadCallback = function(callbackName) {
  goog.asserts.assert(!goog.isNull(this.readCallbacks_));
  var callback = this.readCallbacks_[callbackName];
  goog.asserts.assert(callback);
  return callback(this);
};


/**
 * Reads a field of any valid non-message type from the binary stream.
 * @param {jsproto.BinaryConstants.FieldType} fieldType
 * @return {boolean|number|string|Uint8Array}
 */
jsproto.BinaryReader.prototype.readAny = function(fieldType) {
  this.nextWireType_ = jsproto.BinaryConstants.FieldTypeToWireType(fieldType);
  var fieldTypes = jsproto.BinaryConstants.FieldType;
  switch (fieldType) {
    case fieldTypes.DOUBLE:
      return this.readDouble();
    case fieldTypes.FLOAT:
      return this.readFloat();
    case fieldTypes.INT64:
      return this.readInt64();
    case fieldTypes.UINT64:
      return this.readUint64();
    case fieldTypes.INT32:
      return this.readInt32();
    case fieldTypes.FIXED64:
      return this.readFixed64();
    case fieldTypes.FIXED32:
      return this.readFixed32();
    case fieldTypes.BOOL:
      return this.readBool();
    case fieldTypes.STRING:
      return this.readString();
    case fieldTypes.GROUP:
      goog.asserts.fail('Group field type not supported in readAny()');
    case fieldTypes.MESSAGE:
      goog.asserts.fail('Message field type not supported in readAny()');
    case fieldTypes.BYTES:
      return this.readBytes();
    case fieldTypes.UINT32:
      return this.readUint32();
    case fieldTypes.ENUM:
      return this.readEnum();
    case fieldTypes.SFIXED32:
      return this.readSfixed32();
    case fieldTypes.SFIXED64:
      return this.readSfixed64();
    case fieldTypes.SINT32:
      return this.readSint32();
    case fieldTypes.SINT64:
      return this.readSint64();
    case fieldTypes.FHASH64:
      return this.readFixedHash64();
    case fieldTypes.VHASH64:
      return this.readVarintHash64();
    default:
      goog.asserts.fail('Invalid field type in readAny()');
  }
  return 0;
};


/**
 * Reads an embedded message and returns it as a new BinaryReader, which can
 * then be passed to some other function for parsing.
 * @return {!jsproto.BinaryReader}
 */
jsproto.BinaryReader.prototype.readMessage = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.DELIMITED);

  // Create a new reader pointed at the contents of the message.
  var length = this.decoder_.readUnsignedVarint32();
  var newReader =
      jsproto.BinaryReader.alloc(this.decoder_.getBuffer(),
                                 this.decoder_.getCursor(),
                                 length);

  // All child readers inherit the same set of read callbacks.
  if (this.readCallbacks_) {
    newReader.readCallbacks_ = this.readCallbacks_;
  }

  // Skip this decoder past the message.
  this.decoder_.advance(length);

  return newReader;
};


/**
 * Instead of returning a sub-reader as in the above function, this method just
 * steps into the message. This saves work when traversing down a tree of
 * messages.
 */
jsproto.BinaryReader.prototype.enterMessage = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.DELIMITED);
  var length = this.decoder_.readUnsignedVarint32();
  this.decoder_.setEnd(this.decoder_.getCursor() + length);
};


/**
 * Return a decoder that wraps the current delimited field.
 * @return {!jsproto.BinaryDecoder}
 */
jsproto.BinaryReader.prototype.getFieldDecoder = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.DELIMITED);

  var length = this.decoder_.readUnsignedVarint32();
  var start = this.decoder_.getCursor();
  var end = start + length;

  var innerDecoder = jsproto.BinaryDecoder.alloc(this.decoder_.getBuffer(),
                                                 start, length);
  this.decoder_.setCursor(end);
  return innerDecoder;
};


/**
 * Reads a signed 32-bit integer field from the binary stream, or throws an
 * error if the next field in the stream is not of the correct wire type.
 *
 * @return {number} The value of the signed 32-bit integer field.
 */
jsproto.BinaryReader.prototype.readInt32 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.VARINT);
  return this.decoder_.readSignedVarint32();
};


/**
 * Reads a signed 64-bit integer field from the binary stream, or throws an
 * error if the next field in the stream is not of the correct wire type.
 *
 * @return {number} The value of the signed 64-bit integer field.
 */
jsproto.BinaryReader.prototype.readInt64 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.VARINT);
  return this.decoder_.readSignedVarint64();
};


/**
 * Reads an unsigned 32-bit integer field from the binary stream, or throws an
 * error if the next field in the stream is not of the correct wire type.
 *
 * @return {number} The value of the unsigned 32-bit integer field.
 */
jsproto.BinaryReader.prototype.readUint32 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.VARINT);
  return this.decoder_.readUnsignedVarint32();
};


/**
 * Reads an unsigned 64-bit integer field from the binary stream, or throws an
 * error if the next field in the stream is not of the correct wire type.
 *
 * @return {number} The value of the unsigned 64-bit integer field.
 */
jsproto.BinaryReader.prototype.readUint64 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.VARINT);
  return this.decoder_.readUnsignedVarint64();
};


/**
 * Reads a signed zigzag-encoded 32-bit integer field from the binary stream,
 * or throws an error if the next field in the stream is not of the correct
 * wire type.
 *
 * @return {number} The value of the signed 32-bit integer field.
 */
jsproto.BinaryReader.prototype.readSint32 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.VARINT);
  return this.decoder_.readZigzagVarint32();
};


/**
 * Reads a signed zigzag-encoded 64-bit integer field from the binary stream,
 * or throws an error if the next field in the stream is not of the correct
 * wire type.
 *
 * @return {number} The value of the signed 64-bit integer field.
 */
jsproto.BinaryReader.prototype.readSint64 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.VARINT);
  return this.decoder_.readZigzagVarint64();
};


/**
 * Reads an unsigned 32-bit fixed-length integer fiield from the binary stream,
 * or throws an error if the next field in the stream is not of the correct
 * wire type.
 *
 * @return {number} The value of the double field.
 */
jsproto.BinaryReader.prototype.readFixed32 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.FIXED32);
  return this.decoder_.readUint32();
};


/**
 * Reads an unsigned 64-bit fixed-length integer fiield from the binary stream,
 * or throws an error if the next field in the stream is not of the correct
 * wire type.
 *
 * @return {number} The value of the float field.
 */
jsproto.BinaryReader.prototype.readFixed64 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.FIXED64);
  return this.decoder_.readUint64();
};


/**
 * Reads a signed 32-bit fixed-length integer fiield from the binary stream, or
 * throws an error if the next field in the stream is not of the correct wire
 * type.
 *
 * @return {number} The value of the double field.
 */
jsproto.BinaryReader.prototype.readSfixed32 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.FIXED32);
  return this.decoder_.readInt32();
};


/**
 * Reads a signed 64-bit fixed-length integer fiield from the binary stream, or
 * throws an error if the next field in the stream is not of the correct wire
 * type.
 *
 * @return {number} The value of the float field.
 */
jsproto.BinaryReader.prototype.readSfixed64 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.FIXED64);
  return this.decoder_.readInt64();
};


/**
 * Reads a 32-bit floating-point field from the binary stream, or throws an
 * error if the next field in the stream is not of the correct wire type.
 *
 * @return {number} The value of the float field.
 */
jsproto.BinaryReader.prototype.readFloat = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.FIXED32);
  return this.decoder_.readFloat();
};


/**
 * Reads a 64-bit floating-point field from the binary stream, or throws an
 * error if the next field in the stream is not of the correct wire type.
 *
 * @return {number} The value of the double field.
 */
jsproto.BinaryReader.prototype.readDouble = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.FIXED64);
  return this.decoder_.readDouble();
};


/**
 * Reads a boolean field from the binary stream, or throws an error if the next
 * field in the stream is not of the correct wire type.
 *
 * @return {boolean} The value of the boolean field.
 */
jsproto.BinaryReader.prototype.readBool = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.VARINT);
  return !!this.decoder_.readUnsignedVarint32();
};


/**
 * Reads an enum field from the binary stream, or throws an error if the next
 * field in the stream is not of the correct wire type.
 *
 * @return {number} The value of the enum field.
 */
jsproto.BinaryReader.prototype.readEnum = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.VARINT);
  return this.decoder_.readSignedVarint64();
};


/**
 * Reads a string field from the binary stream, or throws an error if the next
 * field in the stream is not of the correct wire type.
 *
 * @return {string} The value of the string field.
 */
jsproto.BinaryReader.prototype.readString = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.DELIMITED);
  var length = this.decoder_.readUnsignedVarint32();
  return this.decoder_.readString(length);
};


/**
 * Reads a length-prefixed block of bytes from the binary stream, or throws an
 * error if the next field in the stream is not of the correct wire type.
 *
 * @return {!Uint8Array} The block of bytes.
 */
jsproto.BinaryReader.prototype.readBytes = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.DELIMITED);
  var length = this.decoder_.readUnsignedVarint32();
  return this.decoder_.readBytes(length);
};


/**
 * Reads a 64-bit varint or fixed64 field from the stream and returns it as a
 * 8-character Unicode string for use as a hash table key, or throws an error
 * if the next field in the stream is not of the correct wire type.
 *
 * @return {string} The hash value.
 */
jsproto.BinaryReader.prototype.readVarintHash64 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.VARINT);
  return this.decoder_.readVarintHash64();
};


/**
 * Reads a 64-bit varint or fixed64 field from the stream and returns it as a
 * 8-character Unicode string for use as a hash table key, or throws an error
 * if the next field in the stream is not of the correct wire type.
 *
 * @return {string} The hash value.
 */
jsproto.BinaryReader.prototype.readFixedHash64 = function() {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.FIXED64);
  return this.decoder_.readFixedHash64();
};


/**
 * Reads a packed scalar field using the supplied raw reader function.
 * @param {function()} decodeMethod
 * @return {!Array}
 * @private
 */
jsproto.BinaryReader.prototype.readPackedField_ = function(decodeMethod) {
  goog.asserts.assert(
      this.nextWireType_ == jsproto.BinaryConstants.WireType.DELIMITED);
  var length = this.decoder_.readUnsignedVarint32();
  var end = this.decoder_.getCursor() + length;
  var result = [];
  while (this.decoder_.getCursor() < end) {
    // TODO(aappleby): .call is slow
    result.push(decodeMethod.call(this.decoder_));
  }
  return result;
};


/**
 * Reads a packed int32 field, which consists of a length header and a list of
 * signed varints.
 * @return {!Array.<number>}
 */
jsproto.BinaryReader.prototype.readPackedInt32 = function() {
  return this.readPackedField_(this.decoder_.readSignedVarint32);
};


/**
 * Reads a packed int64 field, which consists of a length header and a list of
 * signed varints.
 * @return {!Array.<number>}
 */
jsproto.BinaryReader.prototype.readPackedInt64 = function() {
  return this.readPackedField_(this.decoder_.readSignedVarint64);
};


/**
 * Reads a packed uint32 field, which consists of a length header and a list of
 * unsigned varints.
 * @return {!Array.<number>}
 */
jsproto.BinaryReader.prototype.readPackedUint32 = function() {
  return this.readPackedField_(this.decoder_.readUnsignedVarint32);
};


/**
 * Reads a packed uint64 field, which consists of a length header and a list of
 * unsigned varints.
 * @return {!Array.<number>}
 */
jsproto.BinaryReader.prototype.readPackedUint64 = function() {
  return this.readPackedField_(this.decoder_.readUnsignedVarint64);
};


/**
 * Reads a packed sint32 field, which consists of a length header and a list of
 * zigzag varints.
 * @return {!Array.<number>}
 */
jsproto.BinaryReader.prototype.readPackedSint32 = function() {
  return this.readPackedField_(this.decoder_.readZigzagVarint32);
};


/**
 * Reads a packed sint64 field, which consists of a length header and a list of
 * zigzag varints.
 * @return {!Array.<number>}
 */
jsproto.BinaryReader.prototype.readPackedSint64 = function() {
  return this.readPackedField_(this.decoder_.readZigzagVarint64);
};


/**
 * Reads a packed fixed32 field, which consists of a length header and a list
 * of unsigned 32-bit ints.
 * @return {!Array.<number>}
 */
jsproto.BinaryReader.prototype.readPackedFixed32 = function() {
  return this.readPackedField_(this.decoder_.readUint32);
};


/**
 * Reads a packed fixed64 field, which consists of a length header and a list
 * of unsigned 64-bit ints.
 * @return {!Array.<number>}
 */
jsproto.BinaryReader.prototype.readPackedFixed64 = function() {
  return this.readPackedField_(this.decoder_.readUint64);
};


/**
 * Reads a packed sfixed32 field, which consists of a length header and a list
 * of 32-bit ints.
 * @return {!Array.<number>}
 */
jsproto.BinaryReader.prototype.readPackedSfixed32 = function() {
  return this.readPackedField_(this.decoder_.readInt32);
};


/**
 * Reads a packed sfixed64 field, which consists of a length header and a list
 * of 64-bit ints.
 * @return {!Array.<number>}
 */
jsproto.BinaryReader.prototype.readPackedSfixed64 = function() {
  return this.readPackedField_(this.decoder_.readInt64);
};


/**
 * Reads a packed float field, which consists of a length header and a list of
 * floats.
 * @return {!Array.<number>}
 */
jsproto.BinaryReader.prototype.readPackedFloat = function() {
  return this.readPackedField_(this.decoder_.readFloat);
};


/**
 * Reads a packed double field, which consists of a length header and a list of
 * doubles.
 * @return {!Array.<number>}
 */
jsproto.BinaryReader.prototype.readPackedDouble = function() {
  return this.readPackedField_(this.decoder_.readDouble);
};


/**
 * Reads a packed bool field, which consists of a length header and a list of
 * unsigned varints.
 * @return {!Array.<boolean>}
 */
jsproto.BinaryReader.prototype.readPackedBool = function() {
  return this.readPackedField_(this.decoder_.readBoolean);
};


/**
 * Reads a packed enum field, which consists of a length header and a list of
 * unsigned varints.
 * @return {!Array.<number>}
 */
jsproto.BinaryReader.prototype.readPackedEnum = function() {
  return this.readPackedField_(this.decoder_.readEnum);
};


/**
 * Reads a packed varint hash64 field, which consists of a length header and a
 * list of varint hash64s.
 * @return {!Array.<string>}
 */
jsproto.BinaryReader.prototype.readPackedVarintHash64 = function() {
  return this.readPackedField_(this.decoder_.readVarintHash64);
};


/**
 * Reads a packed fixed hash64 field, which consists of a length header and a
 * list of fixed hash64s.
 * @return {!Array.<string>}
 */
jsproto.BinaryReader.prototype.readPackedFixedHash64 = function() {
  return this.readPackedField_(this.decoder_.readFixedHash64);
};

//net/proto2/contrib/js_proto/public/binary_extension.js
/**
 * @fileoverview Classes to support dynamically registering & materializing
 * extension messages.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('jsproto.AnyFieldType');
goog.provide('jsproto.BinaryExtension');
goog.provide('jsproto.BinaryExtensionType');

goog.require('goog.asserts');
goog.require('jsproto.BinaryConstants');
goog.require('jsproto.BinaryReader');



/**
 * @param {number} field
 * @param {jsproto.BinaryConstants.FieldType} fieldType
 * @param {function():!jsproto.BinaryMessage=} opt_builder
 * @param {function(!jsproto.BinaryMessage)=} opt_recycler
 * @param {function(!jsproto.BinaryMessage, !jsproto.BinaryReader)=} opt_reader
 * @param {function(!jsproto.ConstBinaryMessage, !jsproto.BinaryWriter)=}
 *     opt_writer
 * @param {function(jsproto.BinaryMessage):jsproto.BinaryMessage=} opt_pruner
 * @constructor
 * @struct
 */
jsproto.BinaryExtensionType = function(field, fieldType,
    opt_builder, opt_recycler, opt_reader, opt_writer, opt_pruner) {
  /**
   * The extension's field number, which we use to match this type info with
   * an actual extension object.
   * @const {number}
   */
  this.field = field;

  /**
   * The proto-file interpretation of the field.
   * @const {jsproto.BinaryConstants.FieldType}
   */
  this.fieldType = fieldType;

  /**
   * The optional builder function registered with this extension type.
   * @const {function():!jsproto.BinaryMessage|null}
   */
  this.builder = goog.isDef(opt_builder) ? opt_builder : null;

  /**
   * The optional recycler function registered with this extension type.
   * @const {function(!jsproto.BinaryMessage)|null}
   */
  this.recycler = goog.isDef(opt_recycler) ? opt_recycler : null;

  /**
   * The optional reader registered with this extension type.
   * @const {function(!jsproto.BinaryMessage, !jsproto.BinaryReader)|null}
   */
  this.reader = goog.isDef(opt_reader) ? opt_reader : null;

  /**
   * The optional writer function registered with this extension type.
   * @const {function(!jsproto.ConstBinaryMessage, !jsproto.BinaryWriter)|null}
   */
  this.writer = goog.isDef(opt_writer) ? opt_writer : null;

  /**
   * The optional pruner function registered with this extension type.
   * @const {function(jsproto.BinaryMessage):jsproto.BinaryMessage|null}
   */
  this.pruner = goog.isDef(opt_pruner) ? opt_pruner : null;
};


/**
 * A field in jsproto can be a scalar, a blob of bytes, another proto, or null.
 * @typedef {boolean|number|string|Uint8Array|jsproto.BinaryMessage|null}
 */
jsproto.AnyFieldType;



/**
 * BinaryExtension represents a field whose type was not known at serialization
 * time.
 * @constructor
 * @struct
 */
jsproto.BinaryExtension = function() {
  /**
   * This extension's field number, used to match it up with an extension type
   * above.
   * @type {number}
   */
  this.field = -1;

  /**
   * The wire type this extension was written with.
   * @type {jsproto.BinaryConstants.WireType}
   */
  this.wireType = jsproto.BinaryConstants.WireType.INVALID;

  /**
   * The field block associated with this extension, if present.
   * @type {Uint8Array}
   */
  this.buffer = null;

  /**
   * The starting point in the above field block.
   * @type {?number}
   */
  this.start = null;

  /**
   * The ending point in the above field block.
   * @type {?number}
   */
  this.end = null;

  /**
   * The type information associated with this extension, if we were able to
   * find it.
   * @type {jsproto.BinaryExtensionType}
   */
  this.info = null;

  /**
   * The value of this extension, if we've materialized it.
   * @type {jsproto.AnyFieldType}
   */
  this.value = null;
};


/**
 * @param {number} field The field number for the extension.
 * @param {jsproto.BinaryConstants.WireType} wireType The wire type of the
 *     extension.
 * @param {jsproto.AnyFieldType=} opt_value The optional value to store with
 *     the extension.
 * @param {Uint8Array=} opt_buffer The buffer containing the extension.
 * @param {?number=} opt_start The extension's start point in the buffer.
 * @param {?number=} opt_end The extension's end point in the buffer.
 * @param {jsproto.BinaryExtensionType=} opt_info The type information for the
 *     extension, if it was registered with the base class. If we have no type
 *     information for the extension, it cannot be materialized.
 */
jsproto.BinaryExtension.prototype.init = function(field, wireType, opt_value,
    opt_buffer, opt_start, opt_end, opt_info) {
  this.field = field;
  this.wireType = wireType;

  this.buffer = goog.isDef(opt_buffer) ? opt_buffer : null;
  this.start = goog.isDef(opt_start) ? opt_start : null;
  this.end = goog.isDef(opt_end) ? opt_end : null;

  this.info = goog.isDef(opt_info) ? opt_info : null;
  this.value = goog.isDef(opt_value) ? opt_value : null;

  // Eagerly materialize the extension if we're in debug mode or if it's just a
  // scalar so that we can view the contents of the extension in the debugger.
  if (goog.DEBUG ||
      wireType != jsproto.BinaryConstants.WireType.DELIMITED) {
    this.materialize();
  }
};


/**
 * The default extension pool size.
 * @const {number}
 */
jsproto.BinaryExtension.MAX_POOL_SIZE = 1000;


/**
 * @private {!Array.<jsproto.BinaryExtension>}
 */
jsproto.BinaryExtension.pool_ = [];


/**
 * @private {number}
 */
jsproto.BinaryExtension.poolCursor_ = 0;


/**
 * @param {number} field The field number for the extension.
 * @param {jsproto.BinaryConstants.WireType} wireType The wire type of the
 *     extension.
 * @param {jsproto.AnyFieldType=} opt_value The optional value to store with
 *     the extension.
 * @param {Uint8Array=} opt_buffer The buffer containing the extension.
 * @param {?number=} opt_start The extension's start point in the buffer.
 * @param {?number=} opt_end The extension's end point in the buffer.
 * @param {jsproto.BinaryExtensionType=} opt_info The type information for the
 *     extension, if it was registered with the base class. If we have no type
 *     information for the extension, it cannot be materialized.
 * @return {!jsproto.BinaryExtension}
 */
jsproto.BinaryExtension.alloc = function(field, wireType, opt_value,
                                         opt_buffer, opt_start, opt_end,
                                         opt_info) {
  var result;
  if (jsproto.BinaryExtension.poolCursor_) {
    jsproto.BinaryExtension.poolCursor_--;
    result = jsproto.BinaryExtension.pool_[jsproto.BinaryExtension.poolCursor_];
    jsproto.BinaryExtension.pool_[jsproto.BinaryExtension.poolCursor_] = null;
  } else {
    result = new jsproto.BinaryExtension();
  }
  goog.asserts.assert(result);
  result.init(field, wireType, opt_value,
              opt_buffer, opt_start, opt_end, opt_info);
  return result;
};


/**
 * @param {jsproto.BinaryExtension} extension
 */
jsproto.BinaryExtension.free = function(extension) {
  if (!extension) return;
  extension.recycle();

  extension.field = -1;
  extension.wireType = jsproto.BinaryConstants.WireType.INVALID;
  extension.buffer = null;
  extension.start = null;
  extension.end = null;
  extension.info = null;
  extension.value = null;

  var cursor = jsproto.BinaryExtension.poolCursor_;
  if (cursor < jsproto.BinaryExtension.MAX_POOL_SIZE) {
    jsproto.BinaryExtension.pool_[cursor] = extension;
    jsproto.BinaryExtension.poolCursor_++;
  }
};


/**
 * Gets the raw bytes for the extension.
 * @return {Uint8Array}
 */
jsproto.BinaryExtension.prototype.getBytes = function() {
  if ((this.buffer != null) && (this.start != null) && (this.end != null)) {
    return new Uint8Array(this.buffer.buffer,
                          this.buffer.byteOffset + this.start,
                          this.end - this.start);
  } else {
    return null;
  }
};


/**
 * Materializes a message extension. Does nothing if the extension's field
 * block or type info are missing.
 */
jsproto.BinaryExtension.prototype.materialize = function() {
  if ((this.buffer == null) || (this.start == null) || (this.end == null) ||
      (this.info == null)) {
    return;
  }

  var stream = jsproto.BinaryReader.alloc(this.buffer,
                                          this.start, this.end - this.start);
  stream.nextField();

  if (this.info.builder && this.info.reader) {
    this.value = this.info.builder();
    stream.enterMessage();
    this.info.reader(this.value, stream);
  } else {
    this.value = stream.readAny(this.info.fieldType);
  }

  stream.free();

  this.buffer = null;
  this.start = null;
  this.end = null;
};


/**
 * Dematerializes a message extension. Any changes to the extension proto will
 * be lost!
 */
jsproto.BinaryExtension.prototype.recycle = function() {
  if ((this.buffer == null) || (this.start == null) || (this.end == null) ||
      (this.info == null) || (this.value == null)) {
    return;
  }

  if (this.info.recycler) {
    this.info.recycler(/** @type {!jsproto.BinaryMessage} */(this.value));
  }
  this.value = null;
};


/**
 * Reads an unknown field as an extension, matching it up with extension type
 * information in an extension registry if present.
 * @param {jsproto.BinaryReader} stream
 * @param {Array.<!jsproto.BinaryExtensionType>=} opt_registry The extension
 *     registry
 * @return {jsproto.BinaryExtension}
 */
jsproto.BinaryExtension.readRawExtension = function(stream, opt_registry) {
  var field = stream.getFieldNumber();
  var wireType = stream.getWireType();
  var buffer = stream.getBuffer();

  // Our start/end pair brackets the entire field, including the field tag
  // and the (optional) length value.
  var start = stream.getFieldCursor();
  stream.skipField();
  var end = stream.getCursor();

  goog.asserts.assert(buffer);

  // Find the corresponding extension info, if it was registered.
  var info = null;
  if (opt_registry) {
    for (var i = 0; i < opt_registry.length; i++) {
      var tempInfo = opt_registry[i];
      if (tempInfo.field == field) {
        info = tempInfo;
        break;
      }
    }
  }

  return jsproto.BinaryExtension.alloc(field, wireType, null,
                                       buffer, start, end,
                                       info);
};


/**
 * Writes an opaque extension to the buffer.
 * @param {!jsproto.BinaryExtension} extension
 * @param {!jsproto.BinaryWriter} stream
 */
jsproto.BinaryExtension.writeRawExtension = function(extension, stream) {
  // If we have a pre-serialized field block, just dump it to the writer.
  if ((extension.buffer != null) &&
      (extension.start != null) &&
      (extension.end != null)) {
    stream.rawWriteByteRange(extension.buffer, extension.start, extension.end);
  } else if (extension.value != null && extension.info != null) {
    // Otherwise we use the type info to pick the correct serializer.
    var info = extension.info;
    if (info.fieldType == jsproto.BinaryConstants.FieldType.MESSAGE) {
      if (info.writer) {
        var message =
            /** @type {!jsproto.ConstBinaryMessage} */(extension.value);
        stream.beginSubMessage(extension.field);
        info.writer(message, stream);
        stream.endSubMessage();
      } else {
        goog.asserts.fail('Can\'t serialize extension, writer not found');
      }
    } else {
      var scalarValue = /** @type {boolean|number|string|Uint8Array|null}*/
          (extension.value);
      stream.writeAny(info.fieldType, extension.field, scalarValue);
    }
  }
};


/**
 * Write all the extensions in the array to the buffer sorted by field number.
 * @param {Array.<!jsproto.BinaryExtension>} extensions
 * @param {!jsproto.BinaryWriter} stream
 */
jsproto.BinaryExtension.writeRawExtensions = function(extensions, stream) {
  if (!extensions || !extensions.length) return;

  extensions.sort(function(a, b) {
    return a.field - b.field;
  });

  for (var i = 0; i < extensions.length; i++) {
    jsproto.BinaryExtension.writeRawExtension(extensions[i], stream);
  }
};


/**
 * Helper method, returns true if an extension with the given field number
 * exists.
 * @param {Array.<!jsproto.BinaryExtension>} extensions
 * @param {number} field
 * @return {boolean}
 */
jsproto.BinaryExtension.hasExtension = function(extensions, field) {
  if (!extensions) return false;
  for (var i = 0; i < extensions.length; i++) {
    if (extensions[i].field == field) {
      return true;
    }
  }
  return false;
};


/**
 * Returns the decoded value of the extension, or null if we were unable to
 * decode the extension due to not knowing its type.
 * @param {Array.<!jsproto.BinaryExtension>} extensions
 * @param {number} field
 * @return {jsproto.AnyFieldType}
 */
jsproto.BinaryExtension.getExtension = function(extensions, field) {
  if (!extensions) return null;
  var extension = jsproto.BinaryExtension.getRawExtension(extensions, field);
  if (!extension || !extension.info) {
    return null;
  }

  extension.materialize();
  return extension.value;
};


/**
 * Returns a mutable instance of the given field's extension type, or null if
 * we were unable to determine the extension type.
 * @param {!Array.<!jsproto.BinaryExtension>} extensions
 * @param {Array.<!jsproto.BinaryExtensionType>} registry
 * @param {number} field
 * @return {jsproto.BinaryMessage}
 */
jsproto.BinaryExtension.mutableExtension =
    function(extensions, registry, field) {
  var extension = jsproto.BinaryExtension.getExtension(extensions, field);
  if (extension) return /** @type {jsproto.BinaryMessage} */(extension);

  // Extension not found, let's create one.

  var info = null;
  if (registry) {
    for (var i = 0; i < registry.length; i++) {
      if (registry[i].field == field) {
        info = registry[i];
        break;
      }
    }
  }
  if (!info) {
    goog.asserts.fail("Can't find type info for extension.");
    return null;
  }

  if (info.fieldType != jsproto.BinaryConstants.FieldType.MESSAGE) {
    goog.asserts.fail("Can't create non-message extensions.");
    return null;
  }

  if (info.builder == null) {
    goog.asserts.fail("Extension's builder is missing.");
    return null;
  }

  var instance = info.builder();
  jsproto.BinaryExtension.setExtension(extensions, registry, field, instance);
  return instance;
};


/**
 * Sets a extension by value, creating a new BinaryExtension wrapper object
 * around it if it was not already present on the message.
 * @param {!Array.<!jsproto.BinaryExtension>} extensions
 * @param {Array.<!jsproto.BinaryExtensionType>} registry
 * @param {number} field
 * @param {boolean|number|string|Uint8Array|jsproto.BinaryMessage|null} value
 */
jsproto.BinaryExtension.setExtension =
    function(extensions, registry, field, value) {
  // Find the type info for this extension. If it's not present,
  // we can't create it from an unknown value.
  var info = null;
  if (registry) {
    for (var i = 0; i < registry.length; i++) {
      if (registry[i].field == field) {
        info = registry[i];
        break;
      }
    }
  }
  if (!info) {
    goog.asserts.fail("Can't find type info for extension");
  }

  // Ok, we've got type info. Wrap the unknown value in an extension object.
  var fieldType = info.fieldType;
  var wireType = jsproto.BinaryConstants.FieldTypeToWireType(fieldType);
  var newExtension = jsproto.BinaryExtension.alloc(field, wireType, value,
                                                   null, null, null,
                                                   info);

  // If the extension was already present on the message, replace it with the
  // new one.
  for (var i = 0; i < extensions.length; i++) {
    var extension = extensions[i];
    if (extension && extension.field == field) {
      extensions[i] = newExtension;
      return;
    }
  }

  // Otherwise just append it.
  extensions.push(newExtension);
};


/**
 * Returns the BinaryExtension object associated with the extension field
 * number, if present.
 * @param {!Array.<!jsproto.BinaryExtension>} extensions
 * @param {number} field
 * @return {jsproto.BinaryExtension}
 */
jsproto.BinaryExtension.getRawExtension = function(extensions, field) {
  for (var i = 0; i < extensions.length; i++) {
    if (extensions[i].field == field) {
      return extensions[i];
    }
  }
  return null;
};


/**
 * Adds a BinaryExtension object to the message, replacing any with a matching
 * extension field number if one was already present.
 * @param {!Array.<!jsproto.BinaryExtension>} extensions
 * @param {jsproto.BinaryExtension} value
 */
jsproto.BinaryExtension.setRawExtension = function(extensions, value) {
  for (var i = 0; i < extensions.length; i++) {
    var extension = extensions[i];
    if (extension && extension.field == value.field) {
      extensions[i] = value;
      return;
    }
  }

  extensions.push(value);
};


/**
 * Registers a new extension type with the given class. This is safe to do at
 * runtime as long as all the fields in the message type you're registering
 * share a compatible wire format with the ones in the original extension.
 * Extensions should be registered before any messages using that extension are
 * deserialized - this will be done automatically if the extension field is
 * flagged with the "js_extension" attribute, otherwise you should do it at
 * application startup.
 * @param {!Array.<!jsproto.BinaryExtensionType>} extensionRegistry
 * @param {number} field
 * @param {jsproto.BinaryConstants.FieldType} fieldType
 * @param {function():!jsproto.BinaryMessage=} opt_builder
 * @param {function(!jsproto.BinaryMessage)=} opt_recycler
 * @param {function(!jsproto.BinaryMessage, !jsproto.BinaryReader)=} opt_reader
 * @param {function(!jsproto.ConstBinaryMessage, !jsproto.BinaryWriter)=}
 *     opt_writer
 * @param {function(jsproto.BinaryMessage):jsproto.BinaryMessage=} opt_pruner
 */
jsproto.BinaryExtension.registerExtension =
    function(extensionRegistry, field, fieldType,
        opt_builder, opt_recycler, opt_reader, opt_writer, opt_pruner) {
  var info = new jsproto.BinaryExtensionType(field, fieldType,
      opt_builder, opt_recycler, opt_reader, opt_writer, opt_pruner);

  // Replace existing extension if present, otherwise add it.
  for (var i = 0; i < extensionRegistry.length; i++) {
    if (extensionRegistry[i].field == field) {
      extensionRegistry[i] = info;
      return;
    }
  }
  extensionRegistry.push(info);
};



//net/proto2/contrib/js_proto/public/binary_message.js
/**
 * @fileoverview Base class for code-generated message types.
 *
 * Note - A 'field block' as mentioned here is a block of bytes with the
 * proto-standard field tag and (optional) field length at the beginning of
 * it.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('jsproto.BinaryMessage');
goog.provide('jsproto.ConstBinaryMessage');

goog.require('jsproto.BinaryDecoder');
goog.require('jsproto.BinaryExtension');
goog.require('jsproto.BinaryReader');



/**
 * Base interface class for all const messages. Does __not__ define any
 * methods, as doing so on a widely-used interface defeats dead-code
 * elimination.
 * @interface
 */
jsproto.ConstBinaryMessage = function() {};



/**
 * Base interface class for all messages. Does __not__ define any methods, as
 * doing so on a widely-used interface defeats dead-code elimination.
 * @interface
 * @extends {jsproto.ConstBinaryMessage}
 */
jsproto.BinaryMessage = function() {};


/**
 * Adds a new instance of a message to the array of messages.
 * @param {function():!jsproto.BinaryMessage} builder
 * @param {!Array.<!jsproto.BinaryMessage>} messages
 * @return {!jsproto.BinaryMessage}
 */
jsproto.BinaryMessage.appendInstance = function(builder, messages) {
  var oldLength = messages.length;
  messages.push(builder());
  return messages[oldLength];
};


/**
 * Gets a BinaryReader instance for the given field block. Returns an empty
 * reader if the field block is missing or empty.
 * @param {Uint8Array=} opt_buffer
 * @param {number|null=} opt_start
 * @param {number|null=} opt_end
 * @return {!jsproto.BinaryReader}
 */
jsproto.BinaryMessage.getReader = function(opt_buffer, opt_start, opt_end) {
  if (opt_buffer && (opt_start != null) && (opt_end != null)) {
    var reader = jsproto.BinaryReader.alloc(opt_buffer,
                                            opt_start, opt_end - opt_start);
    return reader;
  } else {
    return jsproto.BinaryReader.alloc();
  }
};


/**
 * Gets a BinaryDecoder instance for the given field block. Returns an empty
 * decoder if the field block is missing or empty.
 * @param {Uint8Array=} opt_buffer
 * @param {number|null=} opt_start
 * @param {number|null=} opt_end
 * @return {!jsproto.BinaryDecoder}
 */
jsproto.BinaryMessage.getDecoder = function(opt_buffer, opt_start, opt_end) {
  if (opt_buffer && (opt_start != null) && (opt_end != null)) {
    var decoder = jsproto.BinaryDecoder.alloc(opt_buffer,
                                              opt_start, opt_end - opt_start);
    // Skip tag.
    decoder.readUnsignedVarint32();
    // Skip length.
    decoder.readUnsignedVarint32();
    return decoder;
  } else {
    return jsproto.BinaryDecoder.alloc();
  }
};


/**
 * Creates an array of numbers (or booleans) from the given field block.
 * Returns null if the field block is missing or empty.
 * @param {!function(this:jsproto.BinaryDecoder)} decodeMethod
 * @param {Uint8Array=} opt_buffer
 * @param {number|null=} opt_start
 * @param {number|null=} opt_end
 * @return {Array.<*>}
 */
jsproto.BinaryMessage.materializeRepeatedScalarField =
    function(decodeMethod, opt_buffer, opt_start, opt_end) {
  if (opt_buffer && (opt_start != null) && (opt_end != null)) {
    var decoder = jsproto.BinaryDecoder.alloc(opt_buffer, opt_start,
                                              opt_end - opt_start);
    decoder.next = decodeMethod;

    var result = [];
    while (!decoder.atEnd()) {
      // Skip tag.
      decoder.readUnsignedVarint32();
      result.push(decoder.next());
    }
    decoder.free();
    return result;
  } else {
    return null;
  }
};


/**
 * Creates an array of numbers (or booleans) from the given field block.
 * Returns null if the field block is missing or empty.
 * @param {!function(this:jsproto.BinaryDecoder)} decodeMethod
 * @param {Uint8Array=} opt_buffer
 * @param {number|null=} opt_start
 * @param {number|null=} opt_end
 * @return {Array.<*>}
 */
jsproto.BinaryMessage.materializePackedScalarField =
    function(decodeMethod, opt_buffer, opt_start, opt_end) {
  if (opt_buffer && (opt_start != null) && (opt_end != null)) {
    var decoder = jsproto.BinaryDecoder.alloc(opt_buffer, opt_start,
                                              opt_end - opt_start);
    decoder.next = decodeMethod;

    // Skip tag.
    decoder.readUnsignedVarint32();
    // Skip length.
    decoder.readUnsignedVarint32();

    var result = [];
    while (!decoder.atEnd()) {
      result.push(decoder.next());
    }
    decoder.free();
    return result;
  } else {
    return null;
  }
};


/**
 * Creates a Uint8Array for the given field block. Returns null if the field
 * block is missing or empty.
 * @param {Uint8Array=} opt_buffer
 * @param {number|null=} opt_start
 * @param {number|null=} opt_end
 * @return {Uint8Array}
 */
jsproto.BinaryMessage.materializeSingularBytesField =
    function(opt_buffer, opt_start, opt_end) {
  if (opt_buffer && (opt_start != null) && (opt_end != null)) {
    var stream = jsproto.BinaryReader.alloc(opt_buffer, opt_start,
                                            opt_end - opt_start);
    stream.nextField();
    var result = stream.readBytes();
    stream.free();
    return result;
  } else {
    return null;
  }
};


/**
 * Creates an array of Uint8Arrays for the given field block. Returns null if
 * the field block is missing or empty.
 * @param {Uint8Array=} opt_buffer
 * @param {number|null=} opt_start
 * @param {number|null=} opt_end
 * @return {Array.<!Uint8Array>}
 */
jsproto.BinaryMessage.materializeRepeatedBytesField =
    function(opt_buffer, opt_start, opt_end) {
  if (opt_buffer && (opt_start != null) && (opt_end != null)) {
    var results = [];
    var stream = jsproto.BinaryReader.alloc(opt_buffer,
                                            opt_start, opt_end - opt_start);
    while (stream.nextField()) {
      results.push(stream.readBytes());
    }
    stream.free();
    return results;
  } else {
    return null;
  }
};


/**
 * Generically converts a field block into a message object. Returns null if
 * the field block is missing or empty.
 * @param {!function():!jsproto.BinaryMessage} builder
 * @param {!function(!jsproto.BinaryMessage, !jsproto.BinaryReader)} reader
 * @param {Uint8Array=} opt_buffer
 * @param {number|null=} opt_start
 * @param {number|null=} opt_end
 * @return {jsproto.BinaryMessage}
 */
jsproto.BinaryMessage.materializeSingularMessageField =
    function(builder, reader, opt_buffer, opt_start, opt_end) {
  if (opt_buffer && (opt_start != null) && (opt_end != null)) {
    var result = builder();
    var stream = jsproto.BinaryReader.alloc(opt_buffer, opt_start,
                                            opt_end - opt_start);
    stream.nextField();
    stream.enterMessage();
    reader(result, stream);
    stream.free();
    return result;
  } else {
    return null;
  }
};


/**
 * Generically converts an array of field blocks into message objects.
 * @param {!function():!jsproto.BinaryMessage} builder
 * @param {!function(!jsproto.BinaryMessage, !jsproto.BinaryReader)} reader
 * @param {Uint8Array=} opt_buffer
 * @param {number|null=} opt_start
 * @param {number|null=} opt_end
 * @return {Array.<!jsproto.BinaryMessage>}
 */
jsproto.BinaryMessage.materializeRepeatedMessageField =
    function(builder, reader, opt_buffer, opt_start, opt_end) {
  if (opt_buffer && (opt_start != null) && (opt_end != null)) {
    var results = [];
    var stream = jsproto.BinaryReader.alloc(opt_buffer,
                                            opt_start, opt_end - opt_start);
    while (stream.nextField()) {
      var result = builder();
      reader(result, stream.readMessage());
      results.push(result);
    }
    stream.free();
    return results;
  } else {
    return null;
  }
};


/**
 * Generically writes a single message or field block.
 * @param {!jsproto.BinaryWriter} stream
 * @param {number} field
 * @param {jsproto.ConstBinaryMessage} message
 * @param {!function(!jsproto.ConstBinaryMessage, !jsproto.BinaryWriter)} writer
 * @param {Uint8Array=} opt_buffer An optional buffer containing a
 *     pre-serialized message.
 * @param {number|null=} opt_start The start position in the above buffer.
 * @param {number|null=} opt_end The end position in the above buffer.
 */
jsproto.BinaryMessage.writeSingularMessageField =
    function(stream, field, message, writer, opt_buffer, opt_start, opt_end) {
  if (message !== null) {
    stream.beginSubMessage(field);
    writer(message, stream);
    stream.endSubMessage();
  } else if (opt_buffer && (opt_start != null) && (opt_end != null)) {
    stream.rawWriteByteRange(opt_buffer, opt_start, opt_end);
  }
};


/**
 * Generically (re)serializes an array of messages or field blocks.
 * @param {!jsproto.BinaryWriter} stream
 * @param {number} field
 * @param {Array.<!jsproto.ConstBinaryMessage>} messages
 * @param {!function(!jsproto.ConstBinaryMessage, !jsproto.BinaryWriter)} writer
 * @param {Uint8Array=} opt_buffer An optional buffer containing pre-serialized
 *     messages.
 * @param {number|null=} opt_start The start position in the above buffer.
 * @param {number|null=} opt_end The end position in the above buffer.
 */
jsproto.BinaryMessage.writeRepeatedMessageField =
    function(stream, field, messages, writer, opt_buffer, opt_start, opt_end) {
  if (messages) {
    for (var i = 0; i < messages.length; i++) {
      stream.beginSubMessage(field);
      writer(messages[i], stream);
      stream.endSubMessage();
    }
  } else if (opt_buffer && (opt_start != null) && (opt_end != null)) {
    stream.rawWriteByteRange(opt_buffer, opt_start, opt_end);
  }
};


/**
 * Recursively removes prunable items from the object/array. Returns null if
 * everything was completely pruned away, otherwise returns the object/array.
 * This function is polymorphic, do not use in hot code.
 * @param {*} variant
 * @param {!function(jsproto.BinaryMessage):jsproto.BinaryMessage=} opt_pruner
 * @return {*}
 */
jsproto.BinaryMessage.pruneVariant = function(variant, opt_pruner) {
  if (variant == null) return null;

  var type = variant.constructor;

  // Scalar types are never pruned, as we dont' know what their default value
  // should be here (that information is only in the codegen'd file).
  if ((type == Boolean) || (type == Number) || (type == String)) {
    return variant;
  }

  // Uint8Arrays are pruned if they're empty.
  if (type == Uint8Array) {
    variant = /** @type {!Uint8Array} */(variant);
    if (variant.length == 0) return null;
    return variant;
  }

  if (type == Array) {
    var array = /** @type {Array} */(variant);
    var cursor = 0;
    for (var i = 0; i < array.length; i++) {
      var temp = jsproto.BinaryMessage.pruneVariant(array[i], opt_pruner);
      if (temp !== null) {
        array[cursor++] = temp;
      }
    }
    if (cursor == 0) return null;
    array.length = cursor;
    return array;
  }

  if (type == jsproto.BinaryExtension) {
    variant = /** @type {jsproto.BinaryExtension} */(variant);
    if ((variant.buffer != null) && (variant.start != null)) {
      return variant;
    }
    if (variant.info && variant.info.pruner) {
      var message = /** @type {jsproto.BinaryMessage} */(variant.value);
      variant.value = /** @type {jsproto.BinaryMessage} */
          (variant.info.pruner(message));
    } else {
      var value = /** @type {boolean|number|string|Uint8Array} */
          (variant.value);
      variant.value = /** @type {boolean|number|string|Uint8Array|null} */
          (jsproto.BinaryMessage.pruneVariant(value));
    }
    return variant.value ? variant : null;
  }

  var message = /** @type {jsproto.BinaryMessage} */(variant);
  return opt_pruner ? opt_pruner(message) : variant;
};

//net/proto2/contrib/js_proto/public/binary_writer.js
/**
 * @fileoverview This file contains utilities for encoding Javascript objects
 * into binary, wire-format protocol buffers (in the form of Uint8Arrays) that
 * a server can consume directly.
 *
 * jsproto's BinaryWriter class defines methods for efficiently encoding
 * Javascript objects into binary, wire-format protocol buffers and supports
 * all the fundamental field types used in protocol buffers.
 *
 * Major caveat 1 - Users of this library _must_ keep their Javascript proto
 * parsing code in sync with the original .proto file - presumably you'll be
 * using the typed jsproto code generator, but if you bypass that you'll need
 * to keep things in sync by hand.
 *
 * Major caveat 2 - Javascript is unable to accurately represent integers
 * larger than 2^53 due to its use of a double-precision floating point format
 * for all numbers. BinaryWriter does not make any special effort to preserve
 * precision for values above this limit - if you need to pass 64-bit integers
 * (hash codes, for example) between the client and server without precision
 * loss, do _not_ use this library.
 *
 * Major caveat 3 - This class uses typed arrays and must not be used on older
 * browsers that do not support them.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('jsproto.BinaryWriter');

goog.require('goog.asserts');
goog.require('jsproto.BinaryConstants');
goog.require('jsproto.utils');



/**
 * BinaryWriter implements encoders for all the wire types specified in
 * https://developers.google.com/protocol-buffers/docs/encoding.
 *
 * @constructor
 * @struct
 */
jsproto.BinaryWriter = function() {
  /**
   * The buffer we're encoding our message (or submessage) into.
   * @private {!Array.<number>}
   */
  this.currentBuffer_ = [];

  /**
   * Starting a submessage pushes the parent message onto this stack, ending a
   * submessage pops the parent message off and appends the submessage to it.
   * @private {!Array.<!Array.<number>>}
   */
  this.bufferStack_ = [];

  /**
   * Starting a submessage pushes the submessage's field number onto this
   * stack, ending a submessage pops it off.
   * @private {!Array.<number>}
   */
  this.fieldStack_ = [];
};


/**
 * Resets the writer, throwing away any accumulated buffers.
 */
jsproto.BinaryWriter.prototype.reset = function() {
  this.currentBuffer_ = [];
  this.bufferStack_ = [];
  this.fieldStack_ = [];
};


/**
 * Converts the encoded data into a Uint8Array.
 * @return {!Uint8Array}
 */
jsproto.BinaryWriter.prototype.getResultBuffer = function() {
  goog.asserts.assert(this.bufferStack_.length == 0);
  return new Uint8Array(this.currentBuffer_);
};


/**
 * Encodes an unsigned 64-bit integer in 32:32 split representation into its
 * wire-format varint representation and stores it in the buffer.
 * @param {number} lowBits The low 32 bits of the int.
 * @param {number} highBits The high 32 bits of the int.
 */
jsproto.BinaryWriter.prototype.rawWriteSplitVarint =
    function(lowBits, highBits) {
  // Break the binary representation into chunks of 7 bits, set the 8th bit
  // in each chunk if it's not the final chunk, and append to the result.
  while (highBits > 0 || lowBits > 127) {
    this.currentBuffer_.push((lowBits & 0x7f) | 0x80);
    lowBits = ((lowBits >>> 7) | (highBits << 25)) >>> 0;
    highBits = highBits >>> 7;
  }
  this.currentBuffer_.push(lowBits);
};


/**
 * Encodes a JavaScript integer into its wire-format varint representation and
 * stores it in the buffer. Due to the way the varint encoding works this
 * behaves correctly for both signed and unsigned integers, though integers
 * that are not representable in 64 bits will still be truncated.
 * @param {number} value The integer to convert.
 */
jsproto.BinaryWriter.prototype.rawWriteVarint = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  jsproto.utils.splitInt64(value);
  this.rawWriteSplitVarint(jsproto.utils.split64Low,
                           jsproto.utils.split64High);
};


/**
 * Encodes a JavaScript integer into its wire-format, zigzag-encoded varint
 * representation and stores it in the buffer. As above, this behaves correctly
 * for both signed and unsigned integers, but integers not representable in 64
 * bits will be truncated.
 * @param {number} value The integer to convert.
 */
jsproto.BinaryWriter.prototype.rawWriteZigzagVarint = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  jsproto.utils.splitZigzag64(value);
  this.rawWriteSplitVarint(jsproto.utils.split64Low,
                           jsproto.utils.split64High);
};


/**
 * Writes a raw 8-bit unsigned integer to the buffer. Numbers outside the range
 * [0,2^8) will be truncated.
 * @param {number} value The value to write.
 */
jsproto.BinaryWriter.prototype.rawWriteUint8 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= 0) && (value < 256));
  this.currentBuffer_.push((value >>> 0) & 0xFF);
};


/**
 * Writes a raw 16-bit unsigned integer to the buffer. Numbers outside the
 * range [0,2^16) will be truncated.
 * @param {number} value The value to write.
 */
jsproto.BinaryWriter.prototype.rawWriteUint16 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= 0) && (value < 65536));
  this.currentBuffer_.push((value >>> 0) & 0xFF);
  this.currentBuffer_.push((value >>> 8) & 0xFF);
};


/**
 * Writes a raw 32-bit unsigned integer to the buffer. Numbers outside the
 * range [0,2^32) will be truncated.
 * @param {number} value The value to write.
 */
jsproto.BinaryWriter.prototype.rawWriteUint32 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= 0) &&
                      (value < jsproto.BinaryConstants.TWO_TO_32));
  this.currentBuffer_.push((value >>> 0) & 0xFF);
  this.currentBuffer_.push((value >>> 8) & 0xFF);
  this.currentBuffer_.push((value >>> 16) & 0xFF);
  this.currentBuffer_.push((value >>> 24) & 0xFF);
};


/**
 * Writes a raw 64-bit unsigned integer to the buffer. Numbers outside the
 * range [0,2^64) will be truncated.
 * @param {number} value The value to write.
 */
jsproto.BinaryWriter.prototype.rawWriteUint64 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= 0) &&
                      (value < jsproto.BinaryConstants.TWO_TO_64));
  jsproto.utils.splitUint64(value);
  this.rawWriteUint32(jsproto.utils.split64Low);
  this.rawWriteUint32(jsproto.utils.split64High);
};


/**
 * Writes a raw 8-bit integer to the buffer. Numbers outside the range
 * [-2^7,2^7) will be truncated.
 * @param {number} value The value to write.
 */
jsproto.BinaryWriter.prototype.rawWriteInt8 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= -128) && (value < 128));
  this.currentBuffer_.push((value >>> 0) & 0xFF);
};


/**
 * Writes a raw 16-bit integer to the buffer. Numbers outside the range
 * [-2^15,2^15) will be truncated.
 * @param {number} value The value to write.
 */
jsproto.BinaryWriter.prototype.rawWriteInt16 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= -32768) && (value < 32768));
  this.currentBuffer_.push((value >>> 0) & 0xFF);
  this.currentBuffer_.push((value >>> 8) & 0xFF);
};


/**
 * Writes a raw 32-bit integer to the buffer. Numbers outside the range
 * [-2^31,2^31) will be truncated.
 * @param {number} value The value to write.
 */
jsproto.BinaryWriter.prototype.rawWriteInt32 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= -jsproto.BinaryConstants.TWO_TO_31) &&
                      (value < jsproto.BinaryConstants.TWO_TO_31));
  this.currentBuffer_.push((value >>> 0) & 0xFF);
  this.currentBuffer_.push((value >>> 8) & 0xFF);
  this.currentBuffer_.push((value >>> 16) & 0xFF);
  this.currentBuffer_.push((value >>> 24) & 0xFF);
};


/**
 * Writes a raw 64-bit integer to the buffer. Numbers outside the range
 * [-2^63,2^63) will be truncated.
 * @param {number} value The value to write.
 */
jsproto.BinaryWriter.prototype.rawWriteInt64 = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= -jsproto.BinaryConstants.TWO_TO_63) &&
                      (value < jsproto.BinaryConstants.TWO_TO_63));
  jsproto.utils.splitInt64(value);
  this.rawWriteUint32(jsproto.utils.split64Low);
  this.rawWriteUint32(jsproto.utils.split64High);
};


/**
 * Writes a raw single-precision floating point value to the buffer. Numbers
 * requiring more than 32 bits of precision will be truncated.
 * @param {number} value The value to write.
 */
jsproto.BinaryWriter.prototype.rawWriteFloat = function(value) {
  jsproto.utils.splitFloat32(value);
  this.rawWriteUint32(jsproto.utils.split64Low);
};


/**
 * Writes a raw double-precision floating point value to the buffer. As this is
 * the native format used by JavaScript, no precision will be lost.
 * @param {number} value The value to write.
 */
jsproto.BinaryWriter.prototype.rawWriteDouble = function(value) {
  jsproto.utils.splitFloat64(value);
  this.rawWriteUint32(jsproto.utils.split64Low);
  this.rawWriteUint32(jsproto.utils.split64High);
};


/**
 * Writes a raw boolean value to the buffer as a varint.
 * @param {boolean} value The value to write.
 */
jsproto.BinaryWriter.prototype.rawWriteBool = function(value) {
  goog.asserts.assert(goog.isBoolean(value));
  this.rawWriteVarint(~~value);
};


/**
 * Writes an raw enum value to the buffer as a varint.
 * @param {number} value The value to write.
 */
jsproto.BinaryWriter.prototype.rawWriteEnum = function(value) {
  goog.asserts.assert(value == Math.floor(value));
  goog.asserts.assert((value >= -jsproto.BinaryConstants.TWO_TO_31) &&
                      (value < jsproto.BinaryConstants.TWO_TO_31));
  this.rawWriteVarint(value);
};


/**
 * Writes a raw string value to the buffer.
 * @param {string} string The string to write.
 */
jsproto.BinaryWriter.prototype.rawWriteUtf8String = function(string) {
  for (var i = 0; i < string.length; i++) {
    this.currentBuffer_.push(string.charCodeAt(i));
  }
};


/**
 * Writes an arbitrary raw byte array to the buffer.
 * @param {!Uint8Array} bytes The array of bytes to write.
 */
jsproto.BinaryWriter.prototype.rawWriteBytes = function(bytes) {
  for (var i = 0; i < bytes.length; i++) {
    this.currentBuffer_.push(bytes[i]);
  }
};


/**
 * Writes an arbitrary raw byte array to the buffer.
 * @param {!Uint8Array} bytes The array of bytes to write.
 * @param {number} start The start of the range to write.
 * @param {number} end The end of the range to write.
 */
jsproto.BinaryWriter.prototype.rawWriteByteRange = function(bytes, start, end) {
  for (var i = start; i < end; i++) {
    this.currentBuffer_.push(bytes[i]);
  }
};


/**
 * Writes a 64-bit hash string (8 characters @ 8 bits of data each) to the
 * buffer as a varint.
 * @param {string} hash The hash to write.
 */
jsproto.BinaryWriter.prototype.rawWriteVarintHash64 = function(hash) {
  jsproto.utils.splitHash64(hash);
  this.rawWriteSplitVarint(jsproto.utils.split64Low,
                           jsproto.utils.split64High);
};


/**
 * Writes a 64-bit hash string (8 characters @ 8 bits of data each) to the
 * buffer as a fixed64.
 * @param {string} hash The hash to write.
 */
jsproto.BinaryWriter.prototype.rawWriteFixedHash64 = function(hash) {
  jsproto.utils.splitHash64(hash);
  this.rawWriteUint32(jsproto.utils.split64Low);
  this.rawWriteUint32(jsproto.utils.split64High);
};


/**
 * Encodes a (field number, wire type) tuple into a wire-format field header
 * and stores it in the buffer as a varint.
 * @param {number} field The field number.
 * @param {number} wireType The wire-type of the field, as specified in the
 *     protocol buffer documentation.
 * @private
 */
jsproto.BinaryWriter.prototype.rawWriteFieldHeader_ =
    function(field, wireType) {
  goog.asserts.assert(field >= 1);
  var x = (field >>> 0) * 8 + wireType;
  this.rawWriteVarint(x);
};


/**
 * Writes an arbitrary byte field to the buffer.
 * @param {number} field The field number.
 * @param {jsproto.BinaryConstants.WireType} wireType The field wire type.
 * @param {Uint8Array=} opt_bytes The array of bytes to write.
 * @param {number|null=} opt_start
 * @param {number|null=} opt_end
 * @private
 */
jsproto.BinaryWriter.prototype.writeByteRange_ =
    function(field, wireType, opt_bytes, opt_start, opt_end) {
  if ((opt_bytes != null) && (opt_start != null) && (opt_end != null)) {
    this.rawWriteFieldHeader_(field, wireType);
    if (wireType == jsproto.BinaryConstants.WireType.DELIMITED) {
      this.rawWriteVarint(opt_end - opt_start);
    }
    for (var i = opt_start; i < opt_end; i++) {
      this.currentBuffer_.push(opt_bytes[i]);
    }
  }
};


/**
 * Writes a field of any valid scalar type to the binary stream.
 * @param {jsproto.BinaryConstants.FieldType} fieldType
 * @param {number} field
 * @param {boolean|number|string|Uint8Array|null} value
 */
jsproto.BinaryWriter.prototype.writeAny = function(fieldType, field, value) {
  var fieldTypes = jsproto.BinaryConstants.FieldType;
  switch (fieldType) {
    case fieldTypes.DOUBLE:
      this.writeDouble(field, /** @type {number} */(value));
      return;
    case fieldTypes.FLOAT:
      this.writeFloat(field, /** @type {number} */(value));
      return;
    case fieldTypes.INT64:
      this.writeInt64(field, /** @type {number} */(value));
      return;
    case fieldTypes.UINT64:
      this.writeUint64(field, /** @type {number} */(value));
      return;
    case fieldTypes.INT32:
      this.writeInt32(field, /** @type {number} */(value));
      return;
    case fieldTypes.FIXED64:
      this.writeFixed64(field, /** @type {number} */(value));
      return;
    case fieldTypes.FIXED32:
      this.writeFixed32(field, /** @type {number} */(value));
      return;
    case fieldTypes.BOOL:
      this.writeBool(field, /** @type {boolean} */(value));
      return;
    case fieldTypes.STRING:
      this.writeString(field, /** @type {string} */(value));
      return;
    case fieldTypes.GROUP:
      goog.asserts.fail('Group field type not supported in writeAny()');
      return;
    case fieldTypes.MESSAGE:
      goog.asserts.fail('Message field type not supported in writeAny()');
      return;
    case fieldTypes.BYTES:
      this.writeBytes(field, /** @type {Uint8Array} */(value));
      return;
    case fieldTypes.UINT32:
      this.writeUint32(field, /** @type {number} */(value));
      return;
    case fieldTypes.ENUM:
      this.writeEnum(field, /** @type {number} */(value));
      return;
    case fieldTypes.SFIXED32:
      this.writeSfixed32(field, /** @type {number} */(value));
      return;
    case fieldTypes.SFIXED64:
      this.writeSfixed64(field, /** @type {number} */(value));
      return;
    case fieldTypes.SINT32:
      this.writeSint32(field, /** @type {number} */(value));
      return;
    case fieldTypes.SINT64:
      this.writeSint64(field, /** @type {number} */(value));
      return;
    case fieldTypes.FHASH64:
      this.writeFixedHash64(field, /** @type {string} */(value));
      return;
    case fieldTypes.VHASH64:
      this.writeVarintHash64(field, /** @type {string} */(value));
      return;
    default:
      goog.asserts.fail('Invalid field type in writeAny()');
      return;
  }
};


/**
 * Writes a varint field to the buffer without range checking.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jsproto.BinaryWriter.prototype.writeVarint = function(field, value) {
  if (value == null) return;
  this.rawWriteFieldHeader_(field, jsproto.BinaryConstants.WireType.VARINT);
  this.rawWriteVarint(value);
};


/**
 * Writes a zigzag varint field to the buffer without range checking.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jsproto.BinaryWriter.prototype.writeZigzagVarint = function(field, value) {
  if (value == null) return;
  this.rawWriteFieldHeader_(field, jsproto.BinaryConstants.WireType.VARINT);
  this.rawWriteZigzagVarint(value);
};


/**
 * Writes an int32 field to the buffer. Numbers outside the range [-2^31,2^31)
 * will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jsproto.BinaryWriter.prototype.writeInt32 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= -jsproto.BinaryConstants.TWO_TO_31) &&
                      (value < jsproto.BinaryConstants.TWO_TO_31));
  this.writeVarint(field, value);
};


/**
 * Writes an int64 field to the buffer. Numbers outside the range [-2^63,2^63)
 * will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jsproto.BinaryWriter.prototype.writeInt64 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= -jsproto.BinaryConstants.TWO_TO_63) &&
                      (value < jsproto.BinaryConstants.TWO_TO_63));
  this.writeVarint(field, value);
};


/**
 * Writes a uint32 field to the buffer. Numbers outside the range [0,2^32)
 * will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jsproto.BinaryWriter.prototype.writeUint32 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= 0) &&
                      (value < jsproto.BinaryConstants.TWO_TO_32));
  this.writeVarint(field, value);
};


/**
 * Writes a uint64 field to the buffer. Numbers outside the range [0,2^64)
 * will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jsproto.BinaryWriter.prototype.writeUint64 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= 0) &&
                      (value < jsproto.BinaryConstants.TWO_TO_64));
  this.writeVarint(field, value);
};


/**
 * Writes a sint32 field to the buffer. Numbers outside the range [-2^31,2^31)
 * will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jsproto.BinaryWriter.prototype.writeSint32 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= -jsproto.BinaryConstants.TWO_TO_31) &&
                      (value < jsproto.BinaryConstants.TWO_TO_31));
  this.writeZigzagVarint(field, value);
};


/**
 * Writes a sint64 field to the buffer. Numbers outside the range [-2^63,2^63)
 * will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jsproto.BinaryWriter.prototype.writeSint64 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= -jsproto.BinaryConstants.TWO_TO_63) &&
                      (value < jsproto.BinaryConstants.TWO_TO_63));
  this.writeZigzagVarint(field, value);
};


/**
 * Writes a fixed32 field to the buffer. Numbers outside the range [0,2^32)
 * will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jsproto.BinaryWriter.prototype.writeFixed32 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= 0) &&
                      (value < jsproto.BinaryConstants.TWO_TO_32));
  this.rawWriteFieldHeader_(field, jsproto.BinaryConstants.WireType.FIXED32);
  this.rawWriteUint32(value);
};


/**
 * Writes a fixed64 field to the buffer. Numbers outside the range [0,2^64)
 * will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jsproto.BinaryWriter.prototype.writeFixed64 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= 0) &&
                      (value < jsproto.BinaryConstants.TWO_TO_64));
  this.rawWriteFieldHeader_(field, jsproto.BinaryConstants.WireType.FIXED64);
  this.rawWriteUint64(value);
};


/**
 * Writes a sfixed32 field to the buffer. Numbers outside the range
 * [-2^31,2^31) will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jsproto.BinaryWriter.prototype.writeSfixed32 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= -jsproto.BinaryConstants.TWO_TO_31) &&
                      (value < jsproto.BinaryConstants.TWO_TO_31));
  this.rawWriteFieldHeader_(field, jsproto.BinaryConstants.WireType.FIXED32);
  this.rawWriteInt32(value);
};


/**
 * Writes a sfixed64 field to the buffer. Numbers outside the range
 * [-2^63,2^63) will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jsproto.BinaryWriter.prototype.writeSfixed64 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= -jsproto.BinaryConstants.TWO_TO_63) &&
                      (value < jsproto.BinaryConstants.TWO_TO_63));
  this.rawWriteFieldHeader_(field, jsproto.BinaryConstants.WireType.FIXED64);
  this.rawWriteInt64(value);
};


/**
 * Writes a single-precision floating point field to the buffer. Numbers
 * requiring more than 32 bits of precision will be truncated.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jsproto.BinaryWriter.prototype.writeFloat = function(field, value) {
  if (value == null) return;
  this.rawWriteFieldHeader_(field, jsproto.BinaryConstants.WireType.FIXED32);
  this.rawWriteFloat(value);
};


/**
 * Writes a double-precision floating point field to the buffer. As this is the
 * native format used by JavaScript, no precision will be lost.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jsproto.BinaryWriter.prototype.writeDouble = function(field, value) {
  if (value == null) return;
  this.rawWriteFieldHeader_(field, jsproto.BinaryConstants.WireType.FIXED64);
  this.rawWriteDouble(value);
};


/**
 * Writes a boolean field to the buffer.
 * @param {number} field The field number.
 * @param {boolean?} value The value to write.
 */
jsproto.BinaryWriter.prototype.writeBool = function(field, value) {
  if (value == null) return;
  goog.asserts.assert(goog.isBoolean(value));
  this.rawWriteFieldHeader_(field, jsproto.BinaryConstants.WireType.VARINT);
  this.rawWriteVarint(~~value);
};


/**
 * Writes an enum field to the buffer.
 * @param {number} field The field number.
 * @param {number?} value The value to write.
 */
jsproto.BinaryWriter.prototype.writeEnum = function(field, value) {
  if (value == null) return;
  goog.asserts.assert((value >= -jsproto.BinaryConstants.TWO_TO_31) &&
                      (value < jsproto.BinaryConstants.TWO_TO_31));
  this.rawWriteFieldHeader_(field, jsproto.BinaryConstants.WireType.VARINT);
  this.rawWriteVarint(value);
};


/**
 * Writes a string field to the buffer.
 * @param {number} field The field number.
 * @param {string?} value The string to write.
 */
jsproto.BinaryWriter.prototype.writeString = function(field, value) {
  if (value == null) return;
  this.rawWriteFieldHeader_(field, jsproto.BinaryConstants.WireType.DELIMITED);

  // Conversion loop swiped from goog.crypt.stringToUtf8ByteArray.
  var bytes = [], p = 0;
  for (var i = 0; i < value.length; i++) {
    var c = value.charCodeAt(i);
    if (c < 128) {
      bytes[p++] = c;
    } else if (c < 2048) {
      bytes[p++] = (c >> 6) | 192;
      bytes[p++] = (c & 63) | 128;
    } else {
      bytes[p++] = (c >> 12) | 224;
      bytes[p++] = ((c >> 6) & 63) | 128;
      bytes[p++] = (c & 63) | 128;
    }
  }

  this.rawWriteVarint(bytes.length);
  this.currentBuffer_ = this.currentBuffer_.concat(bytes);
};


/**
 * Writes an arbitrary byte field to the buffer. Note - to match the behavior
 * of the C++ implementation, empty byte arrays _are_ serialized.
 * @param {number} field The field number.
 * @param {Uint8Array} value The array of bytes to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writeBytes =
    function(field, value, opt_buffer, opt_start, opt_end) {
  if (value != null) {
    this.rawWriteFieldHeader_(field,
        jsproto.BinaryConstants.WireType.DELIMITED);
    this.rawWriteVarint(value.length);
    this.rawWriteBytes(value);
  } else if ((opt_buffer != null) && (opt_start != null) && (opt_end != null)) {
    this.rawWriteByteRange(opt_buffer, opt_start, opt_end);
  }
};


/**
 * Writes a 64-bit hash string field (8 characters @ 8 bits of data each) to
 * the buffer.
 * @param {number} field The field number.
 * @param {string?} value The hash string.
 */
jsproto.BinaryWriter.prototype.writeFixedHash64 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert(value.length == 8);
  this.rawWriteFieldHeader_(field, jsproto.BinaryConstants.WireType.FIXED64);
  this.rawWriteFixedHash64(value);
};


/**
 * Writes a 64-bit hash string field (8 characters @ 8 bits of data each) to
 * the buffer.
 * @param {number} field The field number.
 * @param {string?} value The hash string.
 */
jsproto.BinaryWriter.prototype.writeVarintHash64 = function(field, value) {
  if (value == null) return;
  goog.asserts.assert(value.length == 8);
  this.rawWriteFieldHeader_(field, jsproto.BinaryConstants.WireType.VARINT);
  this.rawWriteVarintHash64(value);
};


/**
 * Writes an array of numbers to the buffer as a repeated varint field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 */
jsproto.BinaryWriter.prototype.writeRepeatedVarints = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeVarint(field, value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a repeated zigzag field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 */
jsproto.BinaryWriter.prototype.writeRepeatedZigzags = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeZigzagVarint(field, value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a repeated 32-bit int field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 */
jsproto.BinaryWriter.prototype.writeRepeatedInt32 =
    jsproto.BinaryWriter.prototype.writeRepeatedVarints;


/**
 * Writes an array of numbers to the buffer as a repeated 64-bit int field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 */
jsproto.BinaryWriter.prototype.writeRepeatedInt64 =
    jsproto.BinaryWriter.prototype.writeRepeatedVarints;


/**
 * Writes an array numbers to the buffer as a repeated unsigned 32-bit int
 *     field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 */
jsproto.BinaryWriter.prototype.writeRepeatedUint32 =
    jsproto.BinaryWriter.prototype.writeRepeatedVarints;


/**
 * Writes an array numbers to the buffer as a repeated unsigned 64-bit int
 *     field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 */
jsproto.BinaryWriter.prototype.writeRepeatedUint64 =
    jsproto.BinaryWriter.prototype.writeRepeatedVarints;


/**
 * Writes an array numbers to the buffer as a repeated signed 32-bit int field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 */
jsproto.BinaryWriter.prototype.writeRepeatedSint32 =
    jsproto.BinaryWriter.prototype.writeRepeatedZigzags;


/**
 * Writes an array numbers to the buffer as a repeated signed 64-bit int field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 */
jsproto.BinaryWriter.prototype.writeRepeatedSint64 =
    jsproto.BinaryWriter.prototype.writeRepeatedZigzags;


/**
 * Writes an array of numbers to the buffer as a repeated fixed32 field. This
 * works for both signed and unsigned fixed32s.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 */
jsproto.BinaryWriter.prototype.writeRepeatedFixed32 = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeFixed32(field, value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a repeated fixed64 field. This
 * works for both signed and unsigned fixed64s.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 */
jsproto.BinaryWriter.prototype.writeRepeatedFixed64 = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeFixed64(field, value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a repeated sfixed32 field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 */
jsproto.BinaryWriter.prototype.writeRepeatedSfixed32 = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeSfixed32(field, value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a repeated sfixed64 field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 */
jsproto.BinaryWriter.prototype.writeRepeatedSfixed64 = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeSfixed64(field, value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a repeated float field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 */
jsproto.BinaryWriter.prototype.writeRepeatedFloat = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeFloat(field, value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a repeated double field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 */
jsproto.BinaryWriter.prototype.writeRepeatedDouble = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeDouble(field, value[i]);
  }
};


/**
 * Writes an array of booleans to the buffer as a repeated bool field.
 * @param {number} field The field number.
 * @param {Array.<boolean>} value The array of ints to write.
 */
jsproto.BinaryWriter.prototype.writeRepeatedBool = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeBool(field, value[i]);
  }
};


/**
 * Writes an array of enums to the buffer as a repeated enum field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 */
jsproto.BinaryWriter.prototype.writeRepeatedEnum = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeEnum(field, value[i]);
  }
};


/**
 * Writes an array of strings to the buffer as a repeated string field.
 * @param {number} field The field number.
 * @param {Array.<string>} value The array of strings to write.
 */
jsproto.BinaryWriter.prototype.writeRepeatedString = function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeString(field, value[i]);
  }
};


/**
 * Writes an array of arbitrary byte fields to the buffer.
 * @param {number} field The field number.
 * @param {Array.<Uint8Array>} value
 *     The arrays of arrays of bytes to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writeRepeatedBytes =
    function(field, value, opt_buffer, opt_start, opt_end) {
  if (value != null) {
    for (var i = 0; i < value.length; i++) {
      this.writeBytes(field, value[i]);
    }
  } else if ((opt_buffer != null) && (opt_start != null) && (opt_end != null)) {
    this.rawWriteByteRange(opt_buffer, opt_start, opt_end);
  }
};


/**
 * Writes a 64-bit hash string field (8 characters @ 8 bits of data each) to
 * the buffer.
 * @param {number} field The field number.
 * @param {Array.<string>} value The array of hashes to write.
 */
jsproto.BinaryWriter.prototype.writeRepeatedFixedHash64 =
    function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeFixedHash64(field, value[i]);
  }
};


/**
 * Writes a repeated 64-bit hash string field (8 characters @ 8 bits of data
 * each) to the buffer.
 * @param {number} field The field number.
 * @param {Array.<string>} value The array of hashes to write.
 */
jsproto.BinaryWriter.prototype.writeRepeatedVarintHash64 =
    function(field, value) {
  if (value == null) return;
  for (var i = 0; i < value.length; i++) {
    this.writeVarintHash64(field, value[i]);
  }
};


/**
 * Writes an array of numbers to the buffer as a packed varint field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writePackedVarints =
    function(field, value, opt_buffer, opt_start, opt_end) {
  if (value != null && value.length) {
    this.beginSubMessage(field);
    for (var i = 0; i < value.length; i++) {
      this.rawWriteVarint(value[i]);
    }
    this.endSubMessage();
  } else if ((opt_buffer != null) && (opt_start != null) && (opt_end != null)) {
    this.rawWriteByteRange(opt_buffer, opt_start, opt_end);
  }
};


/**
 * Writes an array of numbers to the buffer as a packed zigzag field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writePackedZigzags =
    function(field, value, opt_buffer, opt_start, opt_end) {
  if (value != null && value.length) {
    this.beginSubMessage(field);
    for (var i = 0; i < value.length; i++) {
      this.rawWriteZigzagVarint(value[i]);
    }
    this.endSubMessage();
  } else if ((opt_buffer != null) && (opt_start != null) && (opt_end != null)) {
    this.rawWriteByteRange(opt_buffer, opt_start, opt_end);
  }
};


/**
 * Writes an array of numbers to the buffer as a packed 32-bit int field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writePackedInt32 =
    jsproto.BinaryWriter.prototype.writePackedVarints;


/**
 * Writes an array of numbers to the buffer as a packed 64-bit int field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writePackedInt64 =
    jsproto.BinaryWriter.prototype.writePackedVarints;


/**
 * Writes an array numbers to the buffer as a packed unsigned 32-bit int field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writePackedUint32 =
    jsproto.BinaryWriter.prototype.writePackedVarints;


/**
 * Writes an array numbers to the buffer as a packed unsigned 64-bit int field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writePackedUint64 =
    jsproto.BinaryWriter.prototype.writePackedVarints;


/**
 * Writes an array numbers to the buffer as a packed signed 32-bit int field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writePackedSint32 =
    jsproto.BinaryWriter.prototype.writePackedZigzags;


/**
 * Writes an array numbers to the buffer as a packed signed 64-bit int field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writePackedSint64 =
    jsproto.BinaryWriter.prototype.writePackedZigzags;


/**
 * Writes an array of numbers to the buffer as a packed fixed32 field. This
 * works for both signed and unsigned fixed32s.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writePackedFixed32 =
    function(field, value, opt_buffer, opt_start, opt_end) {
  if (value != null && value.length) {
    this.beginSubMessage(field);
    for (var i = 0; i < value.length; i++) {
      this.rawWriteUint32(value[i]);
    }
    this.endSubMessage();
  } else {
    this.writeByteRange_(field, jsproto.BinaryConstants.WireType.DELIMITED,
        opt_buffer, opt_start, opt_end);
  }
};


/**
 * Writes an array of numbers to the buffer as a packed fixed64 field. This
 * works for both signed and unsigned fixed64s.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writePackedFixed64 =
    function(field, value, opt_buffer, opt_start, opt_end) {
  if (value != null && value.length) {
    this.beginSubMessage(field);
    for (var i = 0; i < value.length; i++) {
      this.rawWriteUint64(value[i]);
    }
    this.endSubMessage();
  } else if ((opt_buffer != null) && (opt_start != null) && (opt_end != null)) {
    this.rawWriteByteRange(opt_buffer, opt_start, opt_end);
  }
};


/**
 * Writes an array of numbers to the buffer as a packed sfixed32 field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writePackedSfixed32 =
    function(field, value, opt_buffer, opt_start, opt_end) {
  if (value != null && value.length) {
    this.beginSubMessage(field);
    for (var i = 0; i < value.length; i++) {
      this.rawWriteInt32(value[i]);
    }
    this.endSubMessage();
  } else if ((opt_buffer != null) && (opt_start != null) && (opt_end != null)) {
    this.rawWriteByteRange(opt_buffer, opt_start, opt_end);
  }
};


/**
 * Writes an array of numbers to the buffer as a packed sfixed64 field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writePackedSfixed64 =
    function(field, value, opt_buffer, opt_start, opt_end) {
  if (value != null) {
    this.beginSubMessage(field);
    for (var i = 0; i < value.length; i++) {
      this.rawWriteInt64(value[i]);
    }
    this.endSubMessage();
  } else if ((opt_buffer != null) && (opt_start != null) && (opt_end != null)) {
    this.rawWriteByteRange(opt_buffer, opt_start, opt_end);
  }
};


/**
 * Writes an array of numbers to the buffer as a packed float field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writePackedFloat =
    function(field, value, opt_buffer, opt_start, opt_end) {
  if (value != null && value.length) {
    this.beginSubMessage(field);
    for (var i = 0; i < value.length; i++) {
      this.rawWriteFloat(value[i]);
    }
    this.endSubMessage();
  } else if ((opt_buffer != null) && (opt_start != null) && (opt_end != null)) {
    this.rawWriteByteRange(opt_buffer, opt_start, opt_end);
  }
};


/**
 * Writes an array of numbers to the buffer as a packed double field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writePackedDouble =
    function(field, value, opt_buffer, opt_start, opt_end) {
  if (value != null && value.length) {
    this.beginSubMessage(field);
    for (var i = 0; i < value.length; i++) {
      this.rawWriteDouble(value[i]);
    }
    this.endSubMessage();
  } else if ((opt_buffer != null) && (opt_start != null) && (opt_end != null)) {
    this.rawWriteByteRange(opt_buffer, opt_start, opt_end);
  }
};


/**
 * Writes an array of booleans to the buffer as a packed bool field.
 * @param {number} field The field number.
 * @param {Array.<boolean>} value The array of ints to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writePackedBool =
    function(field, value, opt_buffer, opt_start, opt_end) {
  if (value != null && value.length) {
    this.beginSubMessage(field);
    for (var i = 0; i < value.length; i++) {
      this.rawWriteBool(value[i]);
    }
    this.endSubMessage();
  } else if ((opt_buffer != null) && (opt_start != null) && (opt_end != null)) {
    this.rawWriteByteRange(opt_buffer, opt_start, opt_end);
  }
};


/**
 * Writes an array of enums to the buffer as a packed enum field.
 * @param {number} field The field number.
 * @param {Array.<number>} value The array of ints to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writePackedEnum =
    function(field, value, opt_buffer, opt_start, opt_end) {
  if (value != null && value.length) {
    this.beginSubMessage(field);
    for (var i = 0; i < value.length; i++) {
      this.rawWriteEnum(value[i]);
    }
    this.endSubMessage();
  } else if ((opt_buffer != null) && (opt_start != null) && (opt_end != null)) {
    this.rawWriteByteRange(opt_buffer, opt_start, opt_end);
  }
};


/**
 * Writes a 64-bit hash string field (8 characters @ 8 bits of data each) to
 * the buffer.
 * @param {number} field The field number.
 * @param {Array.<string>} value The array of hashes to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writePackedFixedHash64 =
    function(field, value, opt_buffer, opt_start, opt_end) {
  if (value != null && value.length) {
    this.beginSubMessage(field);
    for (var i = 0; i < value.length; i++) {
      this.rawWriteFixedHash64(value[i]);
    }
    this.endSubMessage();
  } else if ((opt_buffer != null) && (opt_start != null) && (opt_end != null)) {
    this.rawWriteByteRange(opt_buffer, opt_start, opt_end);
  }
};


/**
 * Writes a 64-bit hash string field (8 characters @ 8 bits of data each) to
 * the buffer.
 * @param {number} field The field number.
 * @param {Array.<string>} value The array of hashes to write.
 * @param {Uint8Array=} opt_buffer A buffer containing pre-packed values.
 * @param {number|null=} opt_start The starting point in the above buffer.
 * @param {number|null=} opt_end The ending point in the above buffer.
 */
jsproto.BinaryWriter.prototype.writePackedVarintHash64 =
    function(field, value, opt_buffer, opt_start, opt_end) {
  if (value != null && value.length) {
    this.beginSubMessage(field);
    for (var i = 0; i < value.length; i++) {
      this.rawWriteVarintHash64(value[i]);
    }
    this.endSubMessage();
  } else if ((opt_buffer != null) && (opt_start != null) && (opt_end != null)) {
    this.rawWriteByteRange(opt_buffer, opt_start, opt_end);
  }
};


/**
 * Begins a new sub-message. The client must call endSubMessage() when they're
 * done.
 * @param {number} field The field number of the sub-message.
 */
jsproto.BinaryWriter.prototype.beginSubMessage = function(field) {
  goog.asserts.assert(field >= 0);
  this.bufferStack_.push(this.currentBuffer_);
  this.fieldStack_.push(field);
  this.currentBuffer_ = [];
};


/**
 * Finishes a sub-message and packs it into the parent messages' buffer.
 */
jsproto.BinaryWriter.prototype.endSubMessage = function() {
  goog.asserts.assert(this.bufferStack_.length > 0);
  goog.asserts.assert(this.fieldStack_.length > 0);

  var message = this.currentBuffer_;
  var field = this.fieldStack_.pop();
  this.currentBuffer_ = this.bufferStack_.pop();

  this.rawWriteFieldHeader_(field, jsproto.BinaryConstants.WireType.DELIMITED);
  this.rawWriteVarint(message.length);
  this.currentBuffer_ = this.currentBuffer_.concat(message);
};

//java/com/google/net/rpc3/profiling/jendoscope/web/endo_console.js
// Copyright 2011 Google Inc. All Rights Reserved.

/**
 * @fileoverview Top-level Endoscope client application object.
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('endo.Console');

goog.require('endo.BarGraph');
goog.require('endo.Timeline');
goog.require('jsproto.BinaryReader');



/**
 * The global Endoscope client application object.
 * @constructor
 * @struct
 */
endo.Console = function() {

  /**
   * The main canvas for the console.
   * @private {HTMLCanvasElement}
   */
  this.mainCanvas_ = null;

  /**
   * The number of frames we've rendered so far.
   * @private {number}
   */
  this.frameCount_ = 0;

  /**
   * The time when rendering of the current frame began.
   * @private {number}
   */
  this.frameBegin_ = 0;

  /**
   * The time between when the previous frame began and when the current frame
   * began.
   * @private {number}
   */
  this.frameInterval_ = 0;

  /**
   * The above interval with temporal smoothing applied.
   * @private {number}
   */
  this.smoothInterval_ = 0;

  /**
   * The average time needed to render a frame, with temporal smoothing
   * applied.
   * @private {number}
   */
  this.smoothRender_ = 0;

  /**
   * Our main timeline view.
   * @private {endo.Timeline}
   */
  this.timeline_ = null;

  /**
   * Our main bar graph view.
   * @private {endo.BarGraph}
   */
  this.bargraph_ = null;

  /**
   * Our render callback.
   * @private {function()}
   */
  this.renderCallback_ = goog.bind(this.renderLoop, this);
};


/**
 * The Endoscope console's main render loop.
 */
endo.Console.prototype.renderLoop = function() {

  // Record pre-frame timing.
  ++this.frameCount_;

  var oldbegin = this.frameBegin_ || endo.sync.unixTime();
  this.frameBegin_ = endo.sync.unixTime();
  this.frameInterval_ = (this.frameBegin_ - oldbegin);

  // Update our views.
  endo.view.update(this.frameInterval_);

  // Render our views.
  var renderBegin = endo.sync.unixTime();
  endo.view.render();
  var renderEnd = endo.sync.unixTime();
  var renderDelta = renderEnd - renderBegin;

  // Update our debug performance timers.
  this.smoothInterval_ = endo.util.temporalEase(this.smoothInterval_,
      this.frameInterval_, 1, this.frameInterval_);

  this.smoothRender_ = endo.util.temporalEase(this.smoothRender_,
      renderDelta, 1, this.frameInterval_);

  // Only write the debug performance timers to the page every 15 frames,
  // otherwise they're impossible to read.
  if (this.frameCount_ % 15 == 0) {
    var smoothInterval = (this.smoothInterval_ * 1000).toFixed(2);
    var smoothRender = (this.smoothRender_ * 1000).toFixed(2);

    var delta_text = 'delta time: ' + smoothInterval + 'm';
    var render_text = 'draw time: ' + smoothRender + 'm';

    document.getElementById('delta_time').innerText = delta_text;
    document.getElementById('render_time').innerText = render_text;
  }

  // Update the network status indicator in the bottom right - red for error,
  // green for online, blue for offline.
  var hue = endo.Config.HUE_BLUE;
  if (endo.network.isOnline()) hue = endo.Config.HUE_GREEN;
  if (endo.network.hasError()) hue = endo.Config.HUE_RED;
  document.getElementById('status_icon').style.backgroundColor =
      endo.util.pulseHue(hue, 0.5);

  // Schedule our next render_loop call
  window.webkitRequestAnimationFrame(this.renderCallback_);
};


/**
 * Toggles the application between the timeline view and the bargraph view.
 */
endo.Console.prototype.toggleView = function() {
  endo.view = (endo.view == this.timeline_) ? this.bargraph_ : this.timeline_;
};


/**
 * The Endoscope console's entry point.
 */
endo.Console.prototype.main = function() {
  endo.Config.debug = false;
  this.frameCount_ = 0;
  this.mainCanvas_ = /** @type {!HTMLCanvasElement} */
      (document.getElementById('main_canvas'));

  // Create our views and link them to the document's canvases.
  this.timeline_ = new endo.Timeline(this.mainCanvas_);
  this.bargraph_ = new endo.BarGraph(this.mainCanvas_);

  this.timeline_.resetViewport();
  this.bargraph_.resetViewport();

  // The timeline is our default view.
  endo.view = this.timeline_;

  // Install our interface event handlers.
  endo.ui.install(this, this.mainCanvas_);

  // Set up our network connection with the server.
  endo.network.install();

  // If we're not online, load test data.
  if (!endo.network.isOnline()) {
    var blob = window.atob(endo['testData']);
    var bytes = new Uint8Array(blob.length);
    for (var i = 0; i < blob.length; i++) bytes[i] = blob.charCodeAt(i);

    var reader = jsproto.BinaryReader.alloc(bytes);
    var rawSnapshot = new endo.proto.binary.Snapshot();
    rawSnapshot.readFrom(reader);
    reader.free();

    endo.network.receiveSnapshot(rawSnapshot);
    endo.view.resetViewport();
  }

  // Start the render loop.
  this.renderLoop();
};


/**
 * The global Endoscope application entry point.
 */
endo.main = function() {
  var console = new endo.Console();
  console.main();
};

goog.exportSymbol('endo.main', endo.main);

//java/com/google/net/rpc3/profiling/jendoscope/web/endo_draw.js
// Copyright 2011 Google Inc. All Rights Reserved.

/**
 * @fileoverview Various bits of code for drawing pretty grids and boxes with
 * labels.
 *
 * One bit of weirdness to take note of - The Canvas API ends up spending a
 * non-trivial amount of time parsing the string representations of the colors
 * used by fillStyle and strokeStyle. To reduce that overhead, when possible
 * we cache the current rendering colors on the Javascript side and only
 * modify fillStyle and strokeStyle when the colors actually change.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('endo.draw');

goog.require('endo.Config');



/**
 * Placeholder declaration while we get the rest of the code compiling.
 * TODO(aappleby): Remove this once the view classes compile cleanly.
 * @type {endo.Timeline}
 */
endo.view = null;


/**
 * The current Canvas fill color.
 * @type {?string}
 * @private
 */
endo.draw.currentFill_ = null;


/**
 * The current Canvas stroke color.
 * @type {?string}
 * @private
 */
endo.draw.currentStroke_ = null;


/**
 * Graph settings that show minute/hour/day/week intervals.
 * @const
 */
endo.draw.CLOCK_INTERVALS =
[
 { width: 1, color: '255,255,255' },
 { width: 10, color: '255,255,255' },
 { width: 100, color: '255,255,255' },
 { width: 1000, color: '255,255,255' },
 { width: 10000, color: '255,255,255' },
 { width: 100000, color: '255,255,255' },
 { width: 1 * 1000000, color: '255,255,255' },
 { width: 10 * 1000000, color: '255,255,255' },
 { width: 1 * 60 * 1000000, color: '255,255,255' },
 { width: 10 * 60 * 1000000, color: '255,255,255' },
 { width: 1 * 60 * 60 * 1000000, color: '255,255,255' },
 { width: 24 * 60 * 60 * 1000000, color: '255,255,255' },
 { width: 7 * 24 * 60 * 60 * 1000000, color: '255,255,255' }
];


/**
 * Graph settings for basic power-of-10 intervals.
 * @const
 */
endo.draw.DECIMAL_INTERVALS =
[
  { width: 1, color: '255,255,255' }, // 1 microsecond
  { width: 10, color: '255,255,255' }, // 10 microsecond
  { width: 100, color: '255,255,255' }, // 100 microsecond
  { width: 1000, color: '255,255,255' }, // 1 millisecond
  { width: 10000, color: '255,255,255' }, // 10 millisecond
  { width: 100000, color: '255,255,255' }, // 100 millisecond
  { width: 1000000, color: '255,255,255' }, // 1 second
  { width: 10000000, color: '255,255,255' }, // 10 second
  { width: 100000000, color: '255,255,255' }, // 100 second
  { width: 1000000000, color: '255,255,255' }, // 1K second
  { width: 10000000000, color: '255,255,255' }, // 10K second
  { width: 100000000000, color: '255,255,255' }, // 100K second
  { width: 1000000000000, color: '255,255,255' }, // 1M second
  { width: 10000000000000, color: '255,255,255' } // 10M second
];


/**
 * Resets the rendering context for the current view's canvas to some
 * known-good defaults.
 * @param {number} newWidth The new width of the canvas.
 * @param {number} newHeight The new height of the canvas.
 */
endo.draw.resetCanvas = function(newWidth, newHeight) {
  var canvas = endo.view.getCanvas();

  if ((newWidth != canvas.width) || (newHeight != canvas.height)) {
    canvas.width = newWidth;
    canvas.height = newHeight;
  }

  var context = canvas.getContext('2d');

  context.font = endo.Config.timelineFont;
  context.shadowBlur = 0;
  context.shadowOffsetX = 0;
  context.shadowOffsetY = 0;
  context.lineWidth = 1;
  context.lineCap = 'butt';
  context.lineJoin = 'miter';
  context.miterLimit = 0;

  canvas.context = context;
};

/**
 * Fills a rectangle on the canvas, caching the current fill color to save a
 * few cycles.
 * @param {string} color The fill color.
 * @param {number} x The x coordinate of the top left of the rectangle.
 * @param {number} y The y coordinate of the top left of the rectangle.
 * @param {number} w The width of the rectangle in pixels.
 * @param {number} h The height of the rectangle in pixels.
 */
endo.draw.fillRect = function(color, x, y, w, h) {
  var context = endo.view.getCanvas().context;
  if (endo.draw.currentFill_ != color) {
    endo.draw.currentFill_ = color;
    context.fillStyle = color;
  }
  context.fillRect(x, y, w, h);
};

/**
 * Draws a rectangular outline on the canvas, caching the current fill color to
 * save a few cycles.
 * @param {string} color The fill color.
 * @param {number} x The x coordinate of the top left of the rectangle.
 * @param {number} y The y coordinate of the top left of the rectangle.
 * @param {number} w The width of the rectangle in pixels.
 * @param {number} h The height of the rectangle in pixels.
 */
endo.draw.strokeRect = function(color, x, y, w, h) {
  var context = endo.view.getCanvas().context;
  if (endo.draw.currentStroke_ != color) {
    endo.draw.currentStroke_ = color;
    context.strokeStyle = color;
  }
  context.strokeRect(x, y, w, h);
};

/**
 * Prints text on the canvas, caching the current fill color to save a
 * few cycles.
 * @param {string} color The fill color.
 * @param {string} text The text to print.
 * @param {number} x The x coordinate of the lower left corner of the text.
 * @param {number} y The y coordinate of the lower left corner of the text.
 */
endo.draw.fillText = function(color, text, x, y) {
  var context = endo.view.getCanvas().context;
  if (endo.draw.currentFill_ != color) {
    endo.draw.currentFill_ = color;
    context.fillStyle = color;
  }
  context.fillText(text, x, y);
};

/**
 * Draws a line on the canvas, caching the current fill color to save a
 * few cycles.
 * @param {string} color The fill color.
 * @param {number} x0 The x coordinate of the start point of the line.
 * @param {number} y0 The y coordinate of the start point of the line.
 * @param {number} x1 The x coordinate of the end point of the line.
 * @param {number} y1 The y coordinate of the end point of the line.
 */
endo.draw.strokeLine = function(color, x0, y0, x1, y1) {
  var context = endo.view.getCanvas().context;
  if (endo.draw.currentStroke_ != color) {
    endo.draw.currentStroke_ = color;
    context.strokeStyle = color;
  }
  context.beginPath();
  context.moveTo(x0, y0);
  context.lineTo(x1, y1);
  context.stroke();
};

/**
 * Defines a Canvas clipping rectangle - all draw calls between this clipRect
 * call and clearClip will be clipped to the given rectangle.
 * @param {number} x0 The left boundary of the screen region to clip to.
 * @param {number} y0 The top boundary of the screen region to clip to.
 * @param {number} x1 The right boundary of the screen region to clip to.
 * @param {number} y1 The bottom boundary of the screen region to clip to.
 */
endo.draw.clipRect = function(x0, y0, x1, y1) {
  var context = endo.view.getCanvas().context;
  context.save();
  context.beginPath();
  context.moveTo(x0, y0);
  context.lineTo(x1, y0);
  context.lineTo(x1, y1);
  context.lineTo(x0, y1);
  context.clip();
};

/**
 * Clears the current Canvas clipping rectangle.
 */
endo.draw.clearClip = function() {
  var context = endo.view.getCanvas().context;
  context.restore();
};

/**
 * Draws the background of our zoomable grid, with times outside the range
 * (timeMin, timeMax) in color A and times inside the range in color B.
 * @param {endo.Rect} viewport The graph-space viewport we're rendering.
 * @param {string} colorA The color to use for ouside the time range.
 * @param {string} colorB The color to use for inside the time range.
 * @param {number} timeMin The minimum time of the time range we're rendering.
 * @param {number} timeMax The maximum time of the time range we're rendering.
 */
endo.draw.drawGridBackground = function(viewport,
                                        colorA, colorB,
                                        timeMin, timeMax) {
  var canvas = endo.view.getCanvas();

  var xMin = endo.util.timeToPixel(timeMin, viewport.x0, viewport.x1, canvas);
  var xMax = endo.util.timeToPixel(timeMax, viewport.x0, viewport.x1, canvas);
  xMin = Math.round(xMin);
  xMax = Math.round(xMax);

  if (xMin < 0) xMin = 0;
  if (xMax < 0) xMax = 0;
  if (xMin > canvas.width) xMin = canvas.width;
  if (xMax > canvas.width) xMax = canvas.width;

  // Filling the canvas with magenta helps show cracks between rects, but it
  // also wastes a lot of fillrate.

  if (endo.Config.debug) {
    endo.draw.fillRect('#f0f', 0, 0, canvas.width, canvas.height);
  }

  if (xMin > 0) {
    endo.draw.fillRect(colorA, 0, 0, xMin, canvas.height);
  }

  if (xMax > xMin) {
    endo.draw.fillRect(colorB, xMin, 0, xMax - xMin, canvas.height);
  }

  if (xMax < canvas.width) {
    endo.draw.fillRect(colorA, xMax, 0, canvas.width - xMax, canvas.height);
  }
};

/**
 * Draws horizontal lines between each row of the timeline view.
 * @param {endo.Rect} viewport The graph-space viewport we're rendering.
 */
endo.draw.drawGridLinesX = function(viewport) {
  var canvas = endo.view.getCanvas();
  var context = canvas.context;

  // Horizontal lines fade in as the rows approach 13 pixels high.
  var pixelsPerRow = canvas.height / viewport.getHeight();
  var fade = endo.util.remapClamp(pixelsPerRow, 6, 13, 0, 0.1);
  if (fade <= 0.01) return;

  var begin = Math.ceil(viewport.y0);
  var end = Math.floor(viewport.y1);

  // Update our cached rendering color so that we don't get out of sync.
  var color = 'rgba(255,255,255,' + fade + ')';
  endo.draw.currentStroke_ = color;

  // Draw all the grid lines in one batch.
  context.strokeStyle = 'rgba(255,255,255,' + fade + ')';
  context.beginPath();
  for (var i = begin; i <= end; ++i) {
    var y = endo.util.remap(i, viewport.y0, viewport.y1, 0, canvas.height);
    y = Math.round(y) + 0.5;
    context.moveTo(0, y);
    context.lineTo(canvas.width, y);
  }
  context.stroke();
};

/**
 * Draws one section of vertical lines at each interval boundary in [timeMin,
 * timeMax] in the view, fading them out if they're too densely packed, and then
 * recursing through the sub-intervals if we need to draw them. Time units are
 * in integer microseconds - otherwise numerical precision error does weird
 * things.
 * @private
 * @param {endo.Rect} viewport The graph-space viewport we're rendering.
 * @param {number} timeMin The minimum time of the time range we're rendering.
 * @param {number} timeMax The maximum time of the time range we're rendering.
 * @param {number} whichInterval The index into our interval configuration table
 *     for the interval we're rendering.
 */
endo.draw.drawGridLinesYRecurse_ = function(viewport,
                                       timeMin, timeMax,
                                       whichInterval) {
  var canvas = endo.view.getCanvas();
  var context = canvas.context;

  var interval = endo.draw.DECIMAL_INTERVALS[whichInterval].width;
  var color = endo.draw.DECIMAL_INTERVALS[whichInterval].color;

  var pixelsPerSecond = canvas.width / (viewport.x1 - viewport.x0);
  var pixelsPerInterval = (interval / 1000000) * pixelsPerSecond;

  // Vertical lines fade in as an interval approaches 60 pixels wide.
  var fade = endo.util.remapClamp(pixelsPerInterval, 0, 60, 0, 0.2);

  // Draw vertical lines at the interior divisions between intervals
  color = 'rgba(' + color + ',' + fade + ')';
  endo.draw.currentStroke_ = color;
  context.strokeStyle = color;
  context.beginPath();

  for (var time = timeMin + interval; time < timeMax; time += interval) {
    var seconds = time / 1000000;
    if (seconds < viewport.x0) continue;
    if (seconds > viewport.x1) continue;

    var screenX = endo.util.timeToPixel(seconds, viewport.x0, viewport.x1,
                                        canvas);
    screenX = Math.round(screenX) + 0.5;
    context.moveTo(screenX, 0);
    context.lineTo(screenX, canvas.height);
  }
  context.stroke();

  // If this interval was coarser than we'd like and we're not already at the
  // finest interval, recurse through sub-intervals that overlap the viewport.
  if (pixelsPerInterval < 120) return;
  if (whichInterval == 0) return;

  for (var time = timeMin; time < timeMax; time += interval) {
    var newMin = time;
    var newMax = time + interval;

    if (newMin / 1000000 > viewport.x1) continue;
    if (newMax / 1000000 < viewport.x0) continue;
    endo.draw.drawGridLinesYRecurse_(viewport,
                                     newMin, newMax,
                                     whichInterval - 1);
  }
};

/**
 * Draws vertical lines at each interval boundary in [timeMin,timeMax] in the
 * view. Time units are in integer microseconds - otherwise numerical precision
 * error does weird things.
 * @param {endo.Rect} viewport The graph-space viewport we're rendering.
 */
endo.draw.drawGridLinesY = function(viewport) {
  var intervals = endo.draw.DECIMAL_INTERVALS;
  var maxInterval = intervals[intervals.length - 1].width;
  var timeMin = -maxInterval;
  var timeMax = maxInterval;

  endo.draw.drawGridLinesYRecurse_(viewport,
                                   timeMin, timeMax,
                                   intervals.length - 1);
};

/**
 * Draws the user-provided labels for the horizontal lines on top of an
 * (optional) transparent black overlay so they'll be readable as the timeline
 * passes under them.
 * @param {endo.Rect} viewport The graph-space viewport we're rendering.
 * @param {Array.<string>} labels The user-provided labels for the horizontal
 *     lines.
 * @param {boolean} backdrop True if the labels require a backdrop for
 *     visibility.
 */
endo.draw.drawGridLabelsX = function(viewport, labels, backdrop) {
  var canvas = endo.view.getCanvas();
  var context = canvas.context;

  // Draw the transparent black background for the x axis labels.
  if (backdrop) {
    endo.draw.fillRect('rgba(0,0,0,0.5)',
                  0,
                  endo.Config.timeBackdropHeight,
                  endo.Config.threadNameBackdropWidth,
                  canvas.height);
  }

  // Draw the x axis labels in transparent yellow, clipping them to the
  // backdrop if needed.
  if (backdrop) {
    var backdropWidth = endo.Config.threadNameBackdropWidth -
                        endo.Config.textPaddingX * 2;

    endo.draw.clipRect(endo.Config.textPaddingX,
                       endo.Config.timeBackdropHeight,
                       backdropWidth,
                       canvas.height);
  }

  for (var i = 0; i < labels.length; ++i) {
    var y = endo.util.remap(i, viewport.y0, viewport.y1, 0, canvas.height);
    y = Math.floor(y);
    endo.draw.fillText(endo.Config.threadNameColor,
                       labels[i],
                       endo.Config.textPaddingX,
                       y + endo.Config.timelineFontHeight);
  }

  if (backdrop) {
    endo.draw.clearClip();
  }
};

/**
 * Draws time value labels next to the graph's vertical lines and on top of a
 * transparent black overlay so they'll be readable as the timeline passes
 * under them.
 * @private
 * @param {endo.Rect} viewport The graph-space viewport we're rendering.
 * @param {number} timeMin The minimum time of the time range we're rendering.
 * @param {number} timeMax The maximum time of the time range we're rendering.
 * @param {number} whichInterval The index into our interval configuration table
 *     for the interval we're rendering.
 */
endo.draw.drawGridLabelsYRecurse_ = function(viewport,
                                             timeMin, timeMax,
                                             whichInterval) {
  var canvas = endo.view.getCanvas();
  var context = canvas.context;

  var interval = endo.draw.DECIMAL_INTERVALS[whichInterval].width;
  var color = endo.draw.DECIMAL_INTERVALS[whichInterval].color;

  var pixelsPerSecond = canvas.width / (viewport.x1 - viewport.x0);
  var pixelsPerInterval = (interval / 1000000) * pixelsPerSecond;

  // Don't draw anything if this interval is very narrow.
  if (pixelsPerInterval < 6) return;

  var fade = endo.util.remapClamp(pixelsPerInterval, 30, 100, 0, 1.0);
  if (fade <= 0.01) return;

  // Draw vertical lines at the interior divisions between intervals
  var lineColor = 'rgba(' + color + ',' + fade + ')';
  for (var time = timeMin + interval; time < timeMax; time += interval) {
    var seconds = time / 1000000;
    if (seconds < viewport.x0) continue;
    if (seconds > viewport.x1) continue;

    var screenX = endo.util.timeToPixel(seconds, viewport.x0, viewport.x1,
                                        canvas);
    screenX = Math.round(screenX) + 0.5;

    endo.draw.fillText(lineColor,
                       (time / 1000000).toString(),
                       screenX + 2,
                       endo.Config.timelineFontHeight);
  }

  // If we're not already at the finest interval, recurse through sub-intervals
  // that overlap the viewport.
  if (whichInterval == 0) return;
  for (var time = timeMin; time < timeMax; time += interval) {
    var newMin = time;
    var newMax = time + interval;

    if (newMin / 1000000 > viewport.x1) continue;
    if (newMax / 1000000 < viewport.x0) continue;
    endo.draw.drawGridLabelsYRecurse_(viewport,
                                      newMin, newMax,
                                      whichInterval - 1);
  }
};

/**
 * Draws labels on the vertical lines at each interval boundary in the view.
 * @param {endo.Rect} viewport The graph-space viewport we're rendering.
 */
endo.draw.drawGridLabelsY = function(viewport) {
  var canvas = endo.view.getCanvas();
  var context = canvas.context;

  var intervals = endo.draw.DECIMAL_INTERVALS;
  var maxInterval = intervals[intervals.length - 1].width;
  var timeMin = -maxInterval;
  var timeMax = maxInterval;

  // Draw the transparent black background for the y axis labels.
  endo.draw.fillRect('rgba(0,0,0,0.5)',
                0,
                0,
                canvas.width,
                endo.Config.timeBackdropHeight);

  endo.draw.drawGridLabelsYRecurse_(viewport,
                                   timeMin, timeMax,
                                   intervals.length - 1);
};

/**
 * Draws a rectangle with a 1-pixel border on the canvas using _inclusive_
 * rasterization rules - coordinates are pixel centers, all pixels touched by
 * the rectangle will be drawn, NxM rectangles should always occupy NxM pixels.
 * @param {endo.Rect} srcRect The screen-space rectangle we're drawing.
 * @param {?string=} opt_colorFill The color to use for the body of the
 *     rectangle.
 * @param {?string=} opt_colorBorder The color to use for the outline of the
 *     rectangle.
 */
endo.draw.drawRect = function(srcRect, opt_colorFill, opt_colorBorder) {
  var canvas = endo.view.getCanvas();
  var context = canvas.context;

  // Snap rectangle to pixels
  var x0 = Math.round(srcRect.x0);
  var x1 = Math.round(srcRect.x1);
  var y0 = Math.round(srcRect.y0);
  var y1 = Math.round(srcRect.y1);
  var w = x1 - x0 + 1;
  var h = y1 - y0 + 1;

  if (opt_colorBorder) {
    if ((w < 3) || (h < 3)) {
      endo.draw.fillRect(opt_colorBorder, x0, y0, w, h);
    } else {
      if (opt_colorFill) {
        endo.draw.fillRect(opt_colorFill, x0 + 1, y0 + 1, w - 2, h - 2);
      }
      endo.draw.strokeRect(opt_colorBorder, x0 + 0.5, y0 + 0.5, w - 1, h - 1);
    }
  }
  else if (opt_colorFill) {
    endo.draw.fillRect(opt_colorFill, x0, y0, w, h);
  }
};

/**
 * Draws a double-bordered rectangle on the canvas for use as a highlight or
 * selection rectangle.
 * @param {endo.Rect} rect The screen-space rectangle we're drawing.
 * @param {string} colorFill The color to use for the body of the rectangle.
 * @param {string} colorBorder The color to use for the outline of the
 *     rectangle.
 */
endo.draw.drawHighlightRect = function(rect, colorFill, colorBorder) {
  var canvas = endo.view.getCanvas();
  var context = canvas.context;

  // Draw the rectangle.
  endo.draw.fillRect(colorFill,
                     rect.x0 + 0.5,
                     rect.y0 + 0.5,
                     rect.getWidth(),
                     rect.getHeight());

  // Draw the inner border.
  endo.draw.strokeRect(colorBorder,
                       rect.x0 + 0.5,
                       rect.y0 + 0.5,
                       rect.getWidth(),
                       rect.getHeight());

  // Draw the outer shadow border.
  endo.draw.strokeRect('rgba(0,0,0,0.3)',
                       rect.x0 + 0.5 - 1,
                       rect.y0 + 0.5 - 1,
                       rect.getWidth() + 2,
                       rect.getHeight() + 2);
};

/**
 * Draw a line of text inside the given rectangle, fading it out as the
 * rectangle becomes too small to contain it.
 * @param {string} text The text to render.
 * @param {number} textWidth The width of the text in pixels.
 * @param {endo.Rect} rect The rectangle containing the text.
 */
endo.draw.drawTextInRect = function(text, textWidth, rect) {
  if (text === undefined) return;
  if (text === null) return;
  if (text.length == 0) return;
  if (rect.isEmpty()) return;

  var canvas = endo.view.getCanvas();
  var context = canvas.context;

  var fadeMinX = endo.Config.spanTextFadeMinX;
  var fadeMaxX = endo.Config.spanTextFadeMaxX;

  if (textWidth < fadeMaxX) fadeMaxX = textWidth;

  var fadeX = endo.util.remapEase(rect.getWidth(),
                                  fadeMinX,
                                  fadeMaxX,
                                  0, 1);

  var fadeY = endo.util.remapEase(rect.getHeight(),
                                  endo.Config.spanTextFadeMinY,
                                  endo.Config.spanTextFadeMaxY,
                                  0, 1);

  var textColor = 'rgba(0,0,0,' + (fadeX * fadeY) + ')';
  endo.draw.fillText(textColor,
                     text,
                     rect.x0 + endo.Config.textPaddingX,
                     rect.y0 + endo.Config.timelineFontHeight);
};

/**
 * Computes the width of the rectangle needed to contain the given text,
 * including a few pixels of padding.
 * @param {string} text The text to render.
 * @return {number} The width of the rectangle that can contain the text.
 */
endo.draw.measureText = function(text) {
  if (text === undefined) return 0;
  if (text === null) return 0;
  if (text.length == 0) return 0;

  var canvas = endo.view.getCanvas();
  var context = canvas.context;

  var width = context.measureText(text).width;

  return width + endo.Config.textPaddingX * 2;
};

/**
 * Draws a bar in our bar graph.
 * @param {endo.Rect} srcRect The screen-space rectangle we're drawing.
 * @param {string} color The color to use for the body of the rectangle.
 */
endo.draw.drawBar = function(srcRect, color) {
  var canvas = endo.view.getCanvas();
  var context = canvas.context;

  // Snap rectangle to pixels
  var rect = srcRect.clone();
  rect.x0 = Math.round(rect.x0);
  rect.x1 = Math.round(rect.x1);
  rect.y0 = Math.round(rect.y0);
  rect.y1 = Math.round(rect.y1);

  // Constrain rectangles to be at least 1 pixel wide and 1 pixel high.
  if ((rect.x1 - rect.x0) < 1) rect.x1 = rect.x0 + 1;
  if ((rect.y1 - rect.y0) < 1) rect.y1 = rect.y0 + 1;

  // Draw rectangle body.
  endo.draw.fillRect(color,
                     rect.x0, rect.y0,
                     rect.getWidth(), rect.getHeight());
};

//java/com/google/net/rpc3/profiling/jendoscope/web/endo_event.js
// Copyright 2011 Google Inc. All Rights Reserved.

/**
 * @fileoverview Endoscope's event class represents a single JEndoscope.event or
 * JEndoscope.tag in a single thread.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('endo.Event');



/**
 * Endo's event class.
 * @param {number} markerId The id of the marker that generated this event.
 * @param {endo.Atom.AtomType} type The type of the event.
 * @param {string} tag The user-provided tag
 * @param {number} time The timestamp of the event.
 * @constructor
 * @struct
 */
endo.Event = function(markerId, type, tag, time) {
  /**
   * The numeric ID of the marker that generated this event.
   * @type {number}
   */
  this.markerId = markerId;

  /**
   * The type of this event.
   * @type {endo.Atom.AtomType}
   */
  this.type = type;

  /**
   * The user-provided tag for this task.
   * @type {string}
   */
  this.tag = tag;

  /**
   * The time, in seconds, that this event occurred.
   * @type {number}
   */
  this.time = time;
};


/**
 * Converts a pre-processed atom into an endo.Event object.
 *
 * @param {!endo.Atom} atom The pre-processed atom.
 * @return {endo.Event} The event created from the raw atom.
 */
endo.Event.loadAtom = function(atom) {
  var time = atom.time;
  var event = new endo.Event(atom.markerId, atom.type, atom.tag, time);
  return event;
};

//java/com/google/net/rpc3/profiling/jendoscope/web/endo_marker.js
// Copyright 2011 Google Inc. All Rights Reserved.

/**
 * @fileoverview Endo's marker class represents a single line of code in a
 * single source file.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('endo.Marker');



/**
 * Constructs an Endo marker.
 *
 * @constructor
 * @param {!endo.proto.binary.ConstMarker} rawMarker The raw marker to convert.
 * @struct
 */
endo.Marker = function(rawMarker) {
  /**
   * The user-provided name for this marker - this is generally equal to the
   * last parameter in calls to JEndoscope.xxx on the server side.
   * @type {string}
   */
  this.name = rawMarker.getName();

  /**
   * The numeric type of this marker.
   * @type {endo.proto.binary.Marker.MarkerType}
   */
  this.type = rawMarker.getType();

  /**
   * The global ID of this marker.
   * @type {number}
   */
  this.markerId = rawMarker.getId();

  /**
   * The name of the file containing the JEndoscope call that generated this
   * marker.
   * @type {string}
   */
  this.file = rawMarker.getFileName() || '<unknown>';

  /**
   * The line number in the above file.
   * @type {number}
   */
  this.line = rawMarker.getLine() || 0;

  /**
   * The fully qualified function name containing the JEndoscope call that
   * generated this marker.
   * @type {string}
   */
  this.functionName = rawMarker.getFuncName() || '<unknown>';

  /**
   * Whether the task this marker is associated with has minPeriod defined
   * @type {boolean}
   */
  this.hasMinPeriod = rawMarker.getHasMinPeriod() || false;

  /**
   * True if tasks associated with this marker should be visible on the
   * timeline.
   * @type {boolean}
   */
  this.visible = true;

  // We'd like a unique color per marker, so generate a color based on the
  // hash of the marker name.
  var hash = endo.util.hash(this.name);

  /**
   * The numeric hue, in the range (0,1), to use for all timeline rendering
   * associated with this marker.
   * @type {number}
   */
  this.hue = ((hash >> 16) & 0xFF) / 255;

  /**
   * The fill color to use for spans on the timeline associated with this
   * marker.
   * @type {string}
   */
  this.colorFill = endo.util.hsvToRgb(this.hue, 0.4, 0.90);

  /**
   * The border color to user for spans on the timeline associated with this
   * marker.
   * @type {string}
   */
  this.colorBorder = endo.util.hsvToRgb(this.hue, 0.4, 0.80);

  /**
   * The highlight color to use for spans on the timeline associated with this
   * marker.
   * @type {string}
   */
  this.colorHighlight = endo.util.hsvToRgb(this.hue, 0.5, 1.0);

  /**
   * The color to use for rendering events associated with this marker.
   * @type {string}
   */
  this.colorEvent = endo.util.hsvToRgb(this.hue + 0.1, 0.5, 0.85);

  /**
   * Total time consumed by completed tasks with this Marker.
   * @private {number}
   */
  this.tasksTotalTime_ = 0;

  /**
   * Total number of completed tasks with this Marker.
   * @private {number}
   */
  this.tasksCount_ = 0;
};

/**
 * The descriptive names for each type of Endoscope marker.
 * @type {Object.<endo.proto.binary.Marker.MarkerType, string>}
 */
endo.Marker.MarkerTypeNames = {
  0: 'Invalid',
  1: 'Task',
  2: 'Event',
  3: 'Tag'
};

/**
 * Updates an Endo marker with fresh data from a newer snapshot.
 * TODO(aappleby): This doesn't currently do anything.
 *
 * @param {!endo.Marker} marker The marker from a more recent snapshot.
 */
endo.Marker.prototype.update = function(marker) {
};

/**
 * Add a completed task to the aggregator of this Marker, in order to update
 * the result of averageTime().
 * @param {number} time The time spent by a task, in seconds
 */
endo.Marker.prototype.aggregateTaskTime = function(time) {
  this.tasksTotalTime_ += time;
  this.tasksCount_ += 1;
};

/**
 * The average time of completed tasks with this Marker. User is required to
 * call aggregateTaskTime() for each task.
 * @return {number}
 */
endo.Marker.prototype.averageTaskTime = function() {
  return this.tasksTotalTime_ / this.tasksCount_;
};

/**
 * The total time of completed tasks with this Marker. User is required to
 * call aggregateTaskTime() for each task.
 * @return {number}
 */
endo.Marker.prototype.totalTaskTime = function() {
  return this.tasksTotalTime_;
};

/**
 * Converts a raw marker to an Endo marker.
 *
 * @param {!endo.proto.binary.ConstMarker} rawMarker The raw marker to
 *     convert.
 * @return {!endo.Marker} The converted marker.
 */
endo.Marker.loadRawMarker = function(rawMarker) {
  var marker = new endo.Marker(rawMarker);
  return marker;
};

//java/com/google/net/rpc3/profiling/jendoscope/jendoscope-binary.js

// This file was automatically generated by net/proto2/contrib/js_proto; do not
// modify it directly.
goog.provide('endo.proto.binary.ConstMarker');
goog.provide('endo.proto.binary.ConstSnapshot');
goog.provide('endo.proto.binary.ConstTask');
goog.provide('endo.proto.binary.ConstThread');
goog.provide('endo.proto.binary.Marker');
goog.provide('endo.proto.binary.Marker.MarkerType');
goog.provide('endo.proto.binary.Snapshot');
goog.provide('endo.proto.binary.Task');
goog.provide('endo.proto.binary.Thread');
goog.require('jsproto.AnyFieldType');
goog.require('jsproto.BinaryExtension');
goog.require('jsproto.BinaryMessage');
goog.require('jsproto.ConstBinaryMessage');
////////////////////////////////////////
// Global Enums - these come first as they may be referenced by the classes
// below.
////////////////////////////////////////
/**
 * @extends {jsproto.ConstBinaryMessage}
 * @interface
 */
endo.proto.binary.ConstTask = function() {};
/**
 * Strips all materialized fields off this proto.
 */
endo.proto.binary.ConstTask.prototype.recycleFields = goog.abstractMethod;
/**
 * Returns true if this message has an extension with the given field number.
 * @param {number} field
 * @return {boolean}
 */
endo.proto.binary.ConstTask.prototype.hasExtension = goog.abstractMethod;
/**
 * Retrieves the extension with the given field number.
 * @param {number} field
 * @return {jsproto.AnyFieldType}
 */
endo.proto.binary.ConstTask.prototype.getExtension = goog.abstractMethod;
/**
 * Returns the total number of extensions attached to this message.
 * @return {number}
 */
endo.proto.binary.ConstTask.prototype.getExtensionCount = goog.abstractMethod;
/**
 * Returns the array of 'raw' extensions (known and unknown) associated with
 * this message.
 * @return {!Array.<!jsproto.BinaryExtension>}
 */
endo.proto.binary.ConstTask.prototype.getRawExtensionArray = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstTask.prototype.hasMarkerId = goog.abstractMethod;
/** @return {number} */
endo.proto.binary.ConstTask.prototype.getMarkerId = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstTask.prototype.hasThreadId = goog.abstractMethod;
/** @return {number} */
endo.proto.binary.ConstTask.prototype.getThreadId = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstTask.prototype.hasStartTimeNanos = goog.abstractMethod;
/** @return {number} */
endo.proto.binary.ConstTask.prototype.getStartTimeNanos = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstTask.prototype.hasEndTimeNanos = goog.abstractMethod;
/** @return {number} */
endo.proto.binary.ConstTask.prototype.getEndTimeNanos = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstTask.prototype.hasTag = goog.abstractMethod;
/** @return {string} */
endo.proto.binary.ConstTask.prototype.getTag = goog.abstractMethod;
////////////////////////////////////////
// Constructor
////////////////////////////////////////
/**
 * @implements {endo.proto.binary.ConstTask}
 * @implements {jsproto.BinaryMessage}
 * @constructor
 * @struct
 */
endo.proto.binary.Task = function() {
  /**
   * @private {number|null}
   */
  this.markerId_ = null;
  /**
   * @private {number|null}
   */
  this.threadId_ = null;
  /**
   * @private {number|null}
   */
  this.startTimeNanos_ = null;
  /**
   * @private {number|null}
   */
  this.endTimeNanos_ = null;
  /**
   * @private {string|null}
   */
  this.tag_ = null;
};

/**
 * @extends {jsproto.ConstBinaryMessage}
 * @interface
 */
endo.proto.binary.ConstMarker = function() {};
/**
 * Strips all materialized fields off this proto.
 */
endo.proto.binary.ConstMarker.prototype.recycleFields = goog.abstractMethod;
/**
 * Returns true if this message has an extension with the given field number.
 * @param {number} field
 * @return {boolean}
 */
endo.proto.binary.ConstMarker.prototype.hasExtension = goog.abstractMethod;
/**
 * Retrieves the extension with the given field number.
 * @param {number} field
 * @return {jsproto.AnyFieldType}
 */
endo.proto.binary.ConstMarker.prototype.getExtension = goog.abstractMethod;
/**
 * Returns the total number of extensions attached to this message.
 * @return {number}
 */
endo.proto.binary.ConstMarker.prototype.getExtensionCount = goog.abstractMethod;
/**
 * Returns the array of 'raw' extensions (known and unknown) associated with
 * this message.
 * @return {!Array.<!jsproto.BinaryExtension>}
 */
endo.proto.binary.ConstMarker.prototype.getRawExtensionArray = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstMarker.prototype.hasId = goog.abstractMethod;
/** @return {number} */
endo.proto.binary.ConstMarker.prototype.getId = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstMarker.prototype.hasType = goog.abstractMethod;
/** @return {endo.proto.binary.Marker.MarkerType} */
endo.proto.binary.ConstMarker.prototype.getType = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstMarker.prototype.hasName = goog.abstractMethod;
/** @return {string} */
endo.proto.binary.ConstMarker.prototype.getName = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstMarker.prototype.hasFileName = goog.abstractMethod;
/** @return {string} */
endo.proto.binary.ConstMarker.prototype.getFileName = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstMarker.prototype.hasLine = goog.abstractMethod;
/** @return {number} */
endo.proto.binary.ConstMarker.prototype.getLine = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstMarker.prototype.hasFuncName = goog.abstractMethod;
/** @return {string} */
endo.proto.binary.ConstMarker.prototype.getFuncName = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstMarker.prototype.hasHasMinPeriod = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstMarker.prototype.getHasMinPeriod = goog.abstractMethod;
////////////////////////////////////////
// Constructor
////////////////////////////////////////
/**
 * @implements {endo.proto.binary.ConstMarker}
 * @implements {jsproto.BinaryMessage}
 * @constructor
 * @struct
 */
endo.proto.binary.Marker = function() {
  /**
   * @private {number|null}
   */
  this.id_ = null;
  /**
   * @private {endo.proto.binary.Marker.MarkerType|null}
   */
  this.type_ = null;
  /**
   * @private {string|null}
   */
  this.name_ = null;
  /**
   * @private {string|null}
   */
  this.fileName_ = null;
  /**
   * @private {number|null}
   */
  this.line_ = null;
  /**
   * @private {string|null}
   */
  this.funcName_ = null;
  /**
   * @private {boolean|null}
   */
  this.hasMinPeriod_ = null;
};
/**
 * @enum {number}
 */
endo.proto.binary.Marker.MarkerType = {
  INVALID: 0,
  TASK: 1,
  EVENT: 2,
  TAG: 3
};

/**
 * @extends {jsproto.ConstBinaryMessage}
 * @interface
 */
endo.proto.binary.ConstThread = function() {};
/**
 * Strips all materialized fields off this proto.
 */
endo.proto.binary.ConstThread.prototype.recycleFields = goog.abstractMethod;
/**
 * Returns true if this message has an extension with the given field number.
 * @param {number} field
 * @return {boolean}
 */
endo.proto.binary.ConstThread.prototype.hasExtension = goog.abstractMethod;
/**
 * Retrieves the extension with the given field number.
 * @param {number} field
 * @return {jsproto.AnyFieldType}
 */
endo.proto.binary.ConstThread.prototype.getExtension = goog.abstractMethod;
/**
 * Returns the total number of extensions attached to this message.
 * @return {number}
 */
endo.proto.binary.ConstThread.prototype.getExtensionCount = goog.abstractMethod;
/**
 * Returns the array of 'raw' extensions (known and unknown) associated with
 * this message.
 * @return {!Array.<!jsproto.BinaryExtension>}
 */
endo.proto.binary.ConstThread.prototype.getRawExtensionArray = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstThread.prototype.hasId = goog.abstractMethod;
/** @return {number} */
endo.proto.binary.ConstThread.prototype.getId = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstThread.prototype.hasName = goog.abstractMethod;
/** @return {string} */
endo.proto.binary.ConstThread.prototype.getName = goog.abstractMethod;
////////////////////////////////////////
// Constructor
////////////////////////////////////////
/**
 * @implements {endo.proto.binary.ConstThread}
 * @implements {jsproto.BinaryMessage}
 * @constructor
 * @struct
 */
endo.proto.binary.Thread = function() {
  /**
   * @private {number|null}
   */
  this.id_ = null;
  /**
   * @private {string|null}
   */
  this.name_ = null;
};

/**
 * @extends {jsproto.ConstBinaryMessage}
 * @interface
 */
endo.proto.binary.ConstSnapshot = function() {};
/**
 * Strips all materialized fields off this proto.
 */
endo.proto.binary.ConstSnapshot.prototype.recycleFields = goog.abstractMethod;
/**
 * Returns true if this message has an extension with the given field number.
 * @param {number} field
 * @return {boolean}
 */
endo.proto.binary.ConstSnapshot.prototype.hasExtension = goog.abstractMethod;
/**
 * Retrieves the extension with the given field number.
 * @param {number} field
 * @return {jsproto.AnyFieldType}
 */
endo.proto.binary.ConstSnapshot.prototype.getExtension = goog.abstractMethod;
/**
 * Returns the total number of extensions attached to this message.
 * @return {number}
 */
endo.proto.binary.ConstSnapshot.prototype.getExtensionCount = goog.abstractMethod;
/**
 * Returns the array of 'raw' extensions (known and unknown) associated with
 * this message.
 * @return {!Array.<!jsproto.BinaryExtension>}
 */
endo.proto.binary.ConstSnapshot.prototype.getRawExtensionArray = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstSnapshot.prototype.hasMarker = goog.abstractMethod;
/** @return {number} */
endo.proto.binary.ConstSnapshot.prototype.markerSize = goog.abstractMethod;
/**
 * @param {number} i
 * @return {!endo.proto.binary.ConstMarker}
 */
endo.proto.binary.ConstSnapshot.prototype.getMarker = goog.abstractMethod;
/**
 * @return {!Array.<endo.proto.binary.ConstMarker>}
 */
endo.proto.binary.ConstSnapshot.prototype.getMarkerArray = goog.abstractMethod;
/** */
endo.proto.binary.ConstSnapshot.prototype.recycleMarker = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstSnapshot.prototype.hasTask = goog.abstractMethod;
/** @return {number} */
endo.proto.binary.ConstSnapshot.prototype.taskSize = goog.abstractMethod;
/**
 * @param {number} i
 * @return {!endo.proto.binary.ConstTask}
 */
endo.proto.binary.ConstSnapshot.prototype.getTask = goog.abstractMethod;
/**
 * @return {!Array.<endo.proto.binary.ConstTask>}
 */
endo.proto.binary.ConstSnapshot.prototype.getTaskArray = goog.abstractMethod;
/** */
endo.proto.binary.ConstSnapshot.prototype.recycleTask = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstSnapshot.prototype.hasThread = goog.abstractMethod;
/** @return {number} */
endo.proto.binary.ConstSnapshot.prototype.threadSize = goog.abstractMethod;
/**
 * @param {number} i
 * @return {!endo.proto.binary.ConstThread}
 */
endo.proto.binary.ConstSnapshot.prototype.getThread = goog.abstractMethod;
/**
 * @return {!Array.<endo.proto.binary.ConstThread>}
 */
endo.proto.binary.ConstSnapshot.prototype.getThreadArray = goog.abstractMethod;
/** */
endo.proto.binary.ConstSnapshot.prototype.recycleThread = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstSnapshot.prototype.hasStartTimeMillis = goog.abstractMethod;
/** @return {number} */
endo.proto.binary.ConstSnapshot.prototype.getStartTimeMillis = goog.abstractMethod;
/** @return {boolean} */
endo.proto.binary.ConstSnapshot.prototype.hasDurationNanos = goog.abstractMethod;
/** @return {number} */
endo.proto.binary.ConstSnapshot.prototype.getDurationNanos = goog.abstractMethod;
////////////////////////////////////////
// Constructor
////////////////////////////////////////
/**
 * @implements {endo.proto.binary.ConstSnapshot}
 * @implements {jsproto.BinaryMessage}
 * @constructor
 * @struct
 */
endo.proto.binary.Snapshot = function() {
  /**
   * @private {Array.<!endo.proto.binary.Marker>}
   */
  this.marker_ = null;
  /**
   * @private {Array.<!endo.proto.binary.Task>}
   */
  this.task_ = null;
  /**
   * @private {Array.<!endo.proto.binary.Thread>}
   */
  this.thread_ = null;
  /**
   * @private {number|null}
   */
  this.startTimeMillis_ = null;
  /**
   * @private {number|null}
   */
  this.durationNanos_ = null;
};

////////////////////////////////////////
// Class Descriptors
////////////////////////////////////////
/**
 * @param {!jsproto.BinaryReader} reader
 */
endo.proto.binary.Task.prototype.readFrom = function(reader) {
  if (goog.DEBUG) {
    var lastField = -1;
  }
  while (reader.nextField()) {
    var field = reader.getFieldNumber();
    // In debug builds, we assert that field numbers are monotonically
    // increasing, as we're taking advantage of that property to simplify our
    // decoding.
    if (goog.DEBUG) {
      goog.asserts.assert(lastField <= field);
      lastField = field;
    }
    switch (field) {
    case 1:
      this.markerId_ = /** @type {number} */(reader.readSint32());
      break;
    case 2:
      this.threadId_ = /** @type {number} */(reader.readSint64());
      break;
    case 3:
      this.startTimeNanos_ = /** @type {number} */(reader.readSint64());
      break;
    case 4:
      this.endTimeNanos_ = /** @type {number} */(reader.readSint64());
      break;
    case 5:
      this.tag_ = /** @type {string} */(reader.readString());
      break;
    default:
      reader.skipField();
      break;
    }
  }
};
/**
 * @param {!jsproto.BinaryWriter} writer
 */
endo.proto.binary.Task.prototype.writeTo = function(writer) {
  writer.writeSint32(1, this.markerId_);
  writer.writeSint64(2, this.threadId_);
  writer.writeSint64(3, this.startTimeNanos_);
  writer.writeSint64(4, this.endTimeNanos_);
  writer.writeString(5, this.tag_);
};
/**
 * Converts a proto into a flat array - field N will be copied to array slot
 * N-1 (or slot N if we're in one-based-arrays mode), hash strings will be
 * converted into decimal strings, packed/repeated scalars will be copied, and
 * nested messages will be recursively array-ified.
 * Does not handle byte blob fields.
 * @return {!Array}
 */
endo.proto.binary.Task.prototype.toJspbArray = function() {
  var result = [];
  if (this.markerId_ != null) {
    result[1 + -1] = this.markerId_;;
  }
  if (this.threadId_ != null) {
    result[2 + -1] = this.threadId_;;
  }
  if (this.startTimeNanos_ != null) {
    result[3 + -1] = this.startTimeNanos_;;
  }
  if (this.endTimeNanos_ != null) {
    result[4 + -1] = this.endTimeNanos_;;
  }
  if (this.tag_ != null) {
    result[5 + -1] = this.tag_;;
  }
  return result;
};
/**
 * @param {jsproto.ConstBinaryMessage} other
 * @return {boolean}
 */
endo.proto.binary.Task.prototype.equalsProto = function(other) {
  // TODO(aappleby): implement equality comparison, if needed.
  goog.asserts.fail('Not implemented!');
  return false;
};
if (goog.DEBUG) {
  /**
   * Generate a debug string for this proto that is in proto2 text format.
   * Extensions are not currently handled.
   * @return {string} The debug string.
   */
  endo.proto.binary.Task.prototype.toDebugString = function() {
    return this.toDebugStringInternal_(0);
  };
  /**
   * Helper to generate a debug string for this proto at some indent level. The
   * first line is not indented.
   * @param {number} indentLevel The number of spaces by which to indent lines.
   * @return {string} The debug string.
   */
  endo.proto.binary.Task.prototype.toDebugStringInternal_ = function(indentLevel) {
    var buffer = [];
    var indent = new Array(1 + indentLevel).join(' ');
    if (this.hasMarkerId()) {
      var scalar = jsproto.utils.debugScalarToTextFormat(this.getMarkerId());
      buffer.push('marker_id: ' + scalar);
    }
    if (this.hasThreadId()) {
      var scalar = jsproto.utils.debugScalarToTextFormat(this.getThreadId());
      buffer.push('thread_id: ' + scalar);
    }
    if (this.hasStartTimeNanos()) {
      var scalar = jsproto.utils.debugScalarToTextFormat(this.getStartTimeNanos());
      buffer.push('start_time_nanos: ' + scalar);
    }
    if (this.hasEndTimeNanos()) {
      var scalar = jsproto.utils.debugScalarToTextFormat(this.getEndTimeNanos());
      buffer.push('end_time_nanos: ' + scalar);
    }
    if (this.hasTag()) {
      var scalar = jsproto.utils.debugScalarToTextFormat(this.getTag());
      buffer.push('tag: ' + scalar);
    }
    return buffer.join('\n' + indent);
  };
}
/**
 * Prunes all fields of this message that are equal to their default value.
 * Returns null if all fields were pruned away, otherwise returns 'this'.
 * @return {endo.proto.binary.Task}
 */
endo.proto.binary.Task.prototype.pruneProto = function() {
  var alive = false;
  if (this.markerId_ == 0) {
    this.markerId_ = null;
  }
  alive |= (this.markerId_ != null);
  if (this.threadId_ == 0) {
    this.threadId_ = null;
  }
  alive |= (this.threadId_ != null);
  if (this.startTimeNanos_ == 0) {
    this.startTimeNanos_ = null;
  }
  alive |= (this.startTimeNanos_ != null);
  if (this.endTimeNanos_ == 0) {
    this.endTimeNanos_ = null;
  }
  alive |= (this.endTimeNanos_ != null);
  if (this.tag_ == '') {
    this.tag_ = null;
  }
  alive |= (this.tag_ != null);
  return alive ? this : null;
};
/**
 * Clears all fields of the proto.
 */
endo.proto.binary.Task.prototype.clearProto = function() {
  this.markerId_ = null;
  this.threadId_ = null;
  this.startTimeNanos_ = null;
  this.endTimeNanos_ = null;
  this.tag_ = null;
};
/**
 * Dematerializes all materialized fields in this proto.
 * @override
 */
endo.proto.binary.Task.prototype.recycleFields = function() {
};
/**
 * Returns true if this message has an extension with the given field number.
 * @param {number} field
 * @return {boolean}
 * @override
 */
endo.proto.binary.Task.prototype.hasExtension = function(field) {
  return false;
};
/**
 * Retrieves the extension with the given field number.
 * @param {number} field
 * @return {jsproto.AnyFieldType}
 * @override
 */
endo.proto.binary.Task.prototype.getExtension = function(field) {
  return null;
};
/**
 * Returns the total number of extensions attached to this message.
 * @return {number}
 * @override
 */
endo.proto.binary.Task.prototype.getExtensionCount = function() {
  return 0;
};
/** @return {!Array.<!jsproto.BinaryExtension>} */
endo.proto.binary.Task.prototype.getRawExtensionArray = function() {
  goog.asserts.fail("Type has no extension ranges!");
  return [];
};
/**
 * Gets a mutable copy of an extension in this message, creating it if it
 * isn't present. Only valid for extensions that are messages.
 * @param {number} field
 * @return {jsproto.BinaryMessage}
 */
endo.proto.binary.Task.prototype.mutableExtension = function(field) {
  goog.asserts.fail("Type has no extension ranges!");
  return null;
};
/**
 * Sets an extension in this message.
 * @param {number} field
 * @param {jsproto.AnyFieldType} value
 */
endo.proto.binary.Task.prototype.setExtension = function(field, value) {
  goog.asserts.fail("Type has no extension ranges!");
};
////////////////////////////////////////
// Alloc/Free
////////////////////////////////////////
/**
 * @return {!endo.proto.binary.Task}
 */
endo.proto.binary.Task.alloc = function() {
  return new endo.proto.binary.Task();
};
/**
 * @param {endo.proto.binary.ConstTask} constProto
 */
endo.proto.binary.Task.free = function(constProto) {
  var proto = /** @type {endo.proto.binary.Task} */(constProto);
  if (proto) proto.clearProto();
};
////////////////////////////////////////
// Static Helper Methods
////////////////////////////////////////
/**
 * @return {!jsproto.BinaryMessage}
 */
endo.proto.binary.Task.builder = function() {
  return endo.proto.binary.Task.alloc();
};
/**
 * @param {!jsproto.BinaryMessage} message
 */
endo.proto.binary.Task.recycler = function(message) {
  var message2 = /** @type {!endo.proto.binary.Task} */(message);
  message2.recycleFields();
};
/**
 * @param {!jsproto.BinaryMessage} message
 * @param {!jsproto.BinaryReader} stream
 */
endo.proto.binary.Task.reader = function(message, stream) {
  var message2 = /** @type {!endo.proto.binary.Task} */(message);
  message2.readFrom(stream);
};
/**
 * @param {!jsproto.ConstBinaryMessage} message
 * @param {!jsproto.BinaryWriter} stream
 */
endo.proto.binary.Task.writer = function(message, stream) {
  var message2 = /** @type {!endo.proto.binary.Task} */(message);
  message2.writeTo(stream);
};
/**
 * @param {jsproto.BinaryMessage} message
 * @return {jsproto.BinaryMessage}
 */
endo.proto.binary.Task.pruner = function(message) {
  var message2 = /** @type {!endo.proto.binary.Task} */(message);
  return message2.pruneProto();
};
////////////////////////////////////////
// Class Field Accessors
////////////////////////////////////////
/** @override */
endo.proto.binary.Task.prototype.hasMarkerId = function() {
  return this.markerId_ != null;
};
/** @override */
endo.proto.binary.Task.prototype.getMarkerId = function() {
  if (this.markerId_ != null) {
    return this.markerId_;
  } else {
    return 0;
  }
};
/** */
endo.proto.binary.Task.prototype.clearMarkerId = function() {
  this.markerId_ = null;
};
/** @param {number} value */
endo.proto.binary.Task.prototype.setMarkerId = function(value) {
  this.markerId_ = value;
};
/** @override */
endo.proto.binary.Task.prototype.hasThreadId = function() {
  return this.threadId_ != null;
};
/** @override */
endo.proto.binary.Task.prototype.getThreadId = function() {
  if (this.threadId_ != null) {
    return this.threadId_;
  } else {
    return 0;
  }
};
/** */
endo.proto.binary.Task.prototype.clearThreadId = function() {
  this.threadId_ = null;
};
/** @param {number} value */
endo.proto.binary.Task.prototype.setThreadId = function(value) {
  this.threadId_ = value;
};
/** @override */
endo.proto.binary.Task.prototype.hasStartTimeNanos = function() {
  return this.startTimeNanos_ != null;
};
/** @override */
endo.proto.binary.Task.prototype.getStartTimeNanos = function() {
  if (this.startTimeNanos_ != null) {
    return this.startTimeNanos_;
  } else {
    return 0;
  }
};
/** */
endo.proto.binary.Task.prototype.clearStartTimeNanos = function() {
  this.startTimeNanos_ = null;
};
/** @param {number} value */
endo.proto.binary.Task.prototype.setStartTimeNanos = function(value) {
  this.startTimeNanos_ = value;
};
/** @override */
endo.proto.binary.Task.prototype.hasEndTimeNanos = function() {
  return this.endTimeNanos_ != null;
};
/** @override */
endo.proto.binary.Task.prototype.getEndTimeNanos = function() {
  if (this.endTimeNanos_ != null) {
    return this.endTimeNanos_;
  } else {
    return 0;
  }
};
/** */
endo.proto.binary.Task.prototype.clearEndTimeNanos = function() {
  this.endTimeNanos_ = null;
};
/** @param {number} value */
endo.proto.binary.Task.prototype.setEndTimeNanos = function(value) {
  this.endTimeNanos_ = value;
};
/** @override */
endo.proto.binary.Task.prototype.hasTag = function() {
  return this.tag_ != null;
};
/** @override */
endo.proto.binary.Task.prototype.getTag = function() {
  if (this.tag_ != null) {
    return this.tag_;
  } else {
    return '';
  }
};
/** */
endo.proto.binary.Task.prototype.clearTag = function() {
  this.tag_ = null;
};
/** @param {string} value */
endo.proto.binary.Task.prototype.setTag = function(value) {
  this.tag_ = value;
};
////////////////////////////////////////
// Class Descriptors
////////////////////////////////////////
/**
 * @param {!jsproto.BinaryReader} reader
 */
endo.proto.binary.Marker.prototype.readFrom = function(reader) {
  if (goog.DEBUG) {
    var lastField = -1;
  }
  while (reader.nextField()) {
    var field = reader.getFieldNumber();
    // In debug builds, we assert that field numbers are monotonically
    // increasing, as we're taking advantage of that property to simplify our
    // decoding.
    if (goog.DEBUG) {
      goog.asserts.assert(lastField <= field);
      lastField = field;
    }
    switch (field) {
    case 1:
      this.id_ = /** @type {number} */(reader.readSint32());
      break;
    case 2:
      this.type_ = /** @type {endo.proto.binary.Marker.MarkerType} */(reader.readEnum());
      break;
    case 3:
      this.name_ = /** @type {string} */(reader.readString());
      break;
    case 4:
      this.fileName_ = /** @type {string} */(reader.readString());
      break;
    case 5:
      this.line_ = /** @type {number} */(reader.readSint32());
      break;
    case 6:
      this.funcName_ = /** @type {string} */(reader.readString());
      break;
    case 7:
      this.hasMinPeriod_ = /** @type {boolean} */(reader.readBool());
      break;
    default:
      reader.skipField();
      break;
    }
  }
};
/**
 * @param {!jsproto.BinaryWriter} writer
 */
endo.proto.binary.Marker.prototype.writeTo = function(writer) {
  writer.writeSint32(1, this.id_);
  writer.writeEnum(2, this.type_);
  writer.writeString(3, this.name_);
  writer.writeString(4, this.fileName_);
  writer.writeSint32(5, this.line_);
  writer.writeString(6, this.funcName_);
  writer.writeBool(7, this.hasMinPeriod_);
};
/**
 * Converts a proto into a flat array - field N will be copied to array slot
 * N-1 (or slot N if we're in one-based-arrays mode), hash strings will be
 * converted into decimal strings, packed/repeated scalars will be copied, and
 * nested messages will be recursively array-ified.
 * Does not handle byte blob fields.
 * @return {!Array}
 */
endo.proto.binary.Marker.prototype.toJspbArray = function() {
  var result = [];
  if (this.id_ != null) {
    result[1 + -1] = this.id_;;
  }
  if (this.type_ != null) {
    result[2 + -1] = this.type_;;
  }
  if (this.name_ != null) {
    result[3 + -1] = this.name_;;
  }
  if (this.fileName_ != null) {
    result[4 + -1] = this.fileName_;;
  }
  if (this.line_ != null) {
    result[5 + -1] = this.line_;;
  }
  if (this.funcName_ != null) {
    result[6 + -1] = this.funcName_;;
  }
  if (this.hasMinPeriod_ != null) {
    result[7 + -1] = this.hasMinPeriod_;;
  }
  return result;
};
/**
 * @param {jsproto.ConstBinaryMessage} other
 * @return {boolean}
 */
endo.proto.binary.Marker.prototype.equalsProto = function(other) {
  // TODO(aappleby): implement equality comparison, if needed.
  goog.asserts.fail('Not implemented!');
  return false;
};
if (goog.DEBUG) {
  /**
   * Generate a debug string for this proto that is in proto2 text format.
   * Extensions are not currently handled.
   * @return {string} The debug string.
   */
  endo.proto.binary.Marker.prototype.toDebugString = function() {
    return this.toDebugStringInternal_(0);
  };
  /**
   * Helper to generate a debug string for this proto at some indent level. The
   * first line is not indented.
   * @param {number} indentLevel The number of spaces by which to indent lines.
   * @return {string} The debug string.
   */
  endo.proto.binary.Marker.prototype.toDebugStringInternal_ = function(indentLevel) {
    var buffer = [];
    var indent = new Array(1 + indentLevel).join(' ');
    if (this.hasId()) {
      var scalar = jsproto.utils.debugScalarToTextFormat(this.getId());
      buffer.push('id: ' + scalar);
    }
    if (this.hasType()) {
      var scalar = jsproto.utils.debugScalarToTextFormat(this.getType());
      buffer.push('type: ' + scalar);
    }
    if (this.hasName()) {
      var scalar = jsproto.utils.debugScalarToTextFormat(this.getName());
      buffer.push('name: ' + scalar);
    }
    if (this.hasFileName()) {
      var scalar = jsproto.utils.debugScalarToTextFormat(this.getFileName());
      buffer.push('file_name: ' + scalar);
    }
    if (this.hasLine()) {
      var scalar = jsproto.utils.debugScalarToTextFormat(this.getLine());
      buffer.push('line: ' + scalar);
    }
    if (this.hasFuncName()) {
      var scalar = jsproto.utils.debugScalarToTextFormat(this.getFuncName());
      buffer.push('func_name: ' + scalar);
    }
    if (this.hasHasMinPeriod()) {
      var scalar = jsproto.utils.debugScalarToTextFormat(this.getHasMinPeriod());
      buffer.push('has_min_period: ' + scalar);
    }
    return buffer.join('\n' + indent);
  };
}
/**
 * Prunes all fields of this message that are equal to their default value.
 * Returns null if all fields were pruned away, otherwise returns 'this'.
 * @return {endo.proto.binary.Marker}
 */
endo.proto.binary.Marker.prototype.pruneProto = function() {
  var alive = false;
  if (this.id_ == 0) {
    this.id_ = null;
  }
  alive |= (this.id_ != null);
  if (this.type_ == endo.proto.binary.Marker.MarkerType.INVALID) {
    this.type_ = null;
  }
  alive |= (this.type_ != null);
  if (this.name_ == '') {
    this.name_ = null;
  }
  alive |= (this.name_ != null);
  if (this.fileName_ == '') {
    this.fileName_ = null;
  }
  alive |= (this.fileName_ != null);
  if (this.line_ == 0) {
    this.line_ = null;
  }
  alive |= (this.line_ != null);
  if (this.funcName_ == '') {
    this.funcName_ = null;
  }
  alive |= (this.funcName_ != null);
  if (this.hasMinPeriod_ == false) {
    this.hasMinPeriod_ = null;
  }
  alive |= (this.hasMinPeriod_ != null);
  return alive ? this : null;
};
/**
 * Clears all fields of the proto.
 */
endo.proto.binary.Marker.prototype.clearProto = function() {
  this.id_ = null;
  this.type_ = null;
  this.name_ = null;
  this.fileName_ = null;
  this.line_ = null;
  this.funcName_ = null;
  this.hasMinPeriod_ = null;
};
/**
 * Dematerializes all materialized fields in this proto.
 * @override
 */
endo.proto.binary.Marker.prototype.recycleFields = function() {
};
/**
 * Returns true if this message has an extension with the given field number.
 * @param {number} field
 * @return {boolean}
 * @override
 */
endo.proto.binary.Marker.prototype.hasExtension = function(field) {
  return false;
};
/**
 * Retrieves the extension with the given field number.
 * @param {number} field
 * @return {jsproto.AnyFieldType}
 * @override
 */
endo.proto.binary.Marker.prototype.getExtension = function(field) {
  return null;
};
/**
 * Returns the total number of extensions attached to this message.
 * @return {number}
 * @override
 */
endo.proto.binary.Marker.prototype.getExtensionCount = function() {
  return 0;
};
/** @return {!Array.<!jsproto.BinaryExtension>} */
endo.proto.binary.Marker.prototype.getRawExtensionArray = function() {
  goog.asserts.fail("Type has no extension ranges!");
  return [];
};
/**
 * Gets a mutable copy of an extension in this message, creating it if it
 * isn't present. Only valid for extensions that are messages.
 * @param {number} field
 * @return {jsproto.BinaryMessage}
 */
endo.proto.binary.Marker.prototype.mutableExtension = function(field) {
  goog.asserts.fail("Type has no extension ranges!");
  return null;
};
/**
 * Sets an extension in this message.
 * @param {number} field
 * @param {jsproto.AnyFieldType} value
 */
endo.proto.binary.Marker.prototype.setExtension = function(field, value) {
  goog.asserts.fail("Type has no extension ranges!");
};
////////////////////////////////////////
// Alloc/Free
////////////////////////////////////////
/**
 * @return {!endo.proto.binary.Marker}
 */
endo.proto.binary.Marker.alloc = function() {
  return new endo.proto.binary.Marker();
};
/**
 * @param {endo.proto.binary.ConstMarker} constProto
 */
endo.proto.binary.Marker.free = function(constProto) {
  var proto = /** @type {endo.proto.binary.Marker} */(constProto);
  if (proto) proto.clearProto();
};
////////////////////////////////////////
// Static Helper Methods
////////////////////////////////////////
/**
 * @return {!jsproto.BinaryMessage}
 */
endo.proto.binary.Marker.builder = function() {
  return endo.proto.binary.Marker.alloc();
};
/**
 * @param {!jsproto.BinaryMessage} message
 */
endo.proto.binary.Marker.recycler = function(message) {
  var message2 = /** @type {!endo.proto.binary.Marker} */(message);
  message2.recycleFields();
};
/**
 * @param {!jsproto.BinaryMessage} message
 * @param {!jsproto.BinaryReader} stream
 */
endo.proto.binary.Marker.reader = function(message, stream) {
  var message2 = /** @type {!endo.proto.binary.Marker} */(message);
  message2.readFrom(stream);
};
/**
 * @param {!jsproto.ConstBinaryMessage} message
 * @param {!jsproto.BinaryWriter} stream
 */
endo.proto.binary.Marker.writer = function(message, stream) {
  var message2 = /** @type {!endo.proto.binary.Marker} */(message);
  message2.writeTo(stream);
};
/**
 * @param {jsproto.BinaryMessage} message
 * @return {jsproto.BinaryMessage}
 */
endo.proto.binary.Marker.pruner = function(message) {
  var message2 = /** @type {!endo.proto.binary.Marker} */(message);
  return message2.pruneProto();
};
////////////////////////////////////////
// Class Field Accessors
////////////////////////////////////////
/** @override */
endo.proto.binary.Marker.prototype.hasId = function() {
  return this.id_ != null;
};
/** @override */
endo.proto.binary.Marker.prototype.getId = function() {
  if (this.id_ != null) {
    return this.id_;
  } else {
    return 0;
  }
};
/** */
endo.proto.binary.Marker.prototype.clearId = function() {
  this.id_ = null;
};
/** @param {number} value */
endo.proto.binary.Marker.prototype.setId = function(value) {
  this.id_ = value;
};
/** @override */
endo.proto.binary.Marker.prototype.hasType = function() {
  return this.type_ != null;
};
/** @override */
endo.proto.binary.Marker.prototype.getType = function() {
  if (this.type_ != null) {
    return this.type_;
  } else {
    return endo.proto.binary.Marker.MarkerType.INVALID;
  }
};
/** */
endo.proto.binary.Marker.prototype.clearType = function() {
  this.type_ = null;
};
/** @param {endo.proto.binary.Marker.MarkerType} value */
endo.proto.binary.Marker.prototype.setType = function(value) {
  this.type_ = value;
};
/** @override */
endo.proto.binary.Marker.prototype.hasName = function() {
  return this.name_ != null;
};
/** @override */
endo.proto.binary.Marker.prototype.getName = function() {
  if (this.name_ != null) {
    return this.name_;
  } else {
    return '';
  }
};
/** */
endo.proto.binary.Marker.prototype.clearName = function() {
  this.name_ = null;
};
/** @param {string} value */
endo.proto.binary.Marker.prototype.setName = function(value) {
  this.name_ = value;
};
/** @override */
endo.proto.binary.Marker.prototype.hasFileName = function() {
  return this.fileName_ != null;
};
/** @override */
endo.proto.binary.Marker.prototype.getFileName = function() {
  if (this.fileName_ != null) {
    return this.fileName_;
  } else {
    return '';
  }
};
/** */
endo.proto.binary.Marker.prototype.clearFileName = function() {
  this.fileName_ = null;
};
/** @param {string} value */
endo.proto.binary.Marker.prototype.setFileName = function(value) {
  this.fileName_ = value;
};
/** @override */
endo.proto.binary.Marker.prototype.hasLine = function() {
  return this.line_ != null;
};
/** @override */
endo.proto.binary.Marker.prototype.getLine = function() {
  if (this.line_ != null) {
    return this.line_;
  } else {
    return 0;
  }
};
/** */
endo.proto.binary.Marker.prototype.clearLine = function() {
  this.line_ = null;
};
/** @param {number} value */
endo.proto.binary.Marker.prototype.setLine = function(value) {
  this.line_ = value;
};
/** @override */
endo.proto.binary.Marker.prototype.hasFuncName = function() {
  return this.funcName_ != null;
};
/** @override */
endo.proto.binary.Marker.prototype.getFuncName = function() {
  if (this.funcName_ != null) {
    return this.funcName_;
  } else {
    return '';
  }
};
/** */
endo.proto.binary.Marker.prototype.clearFuncName = function() {
  this.funcName_ = null;
};
/** @param {string} value */
endo.proto.binary.Marker.prototype.setFuncName = function(value) {
  this.funcName_ = value;
};
/** @override */
endo.proto.binary.Marker.prototype.hasHasMinPeriod = function() {
  return this.hasMinPeriod_ != null;
};
/** @override */
endo.proto.binary.Marker.prototype.getHasMinPeriod = function() {
  if (this.hasMinPeriod_ != null) {
    return this.hasMinPeriod_;
  } else {
    return false;
  }
};
/** */
endo.proto.binary.Marker.prototype.clearHasMinPeriod = function() {
  this.hasMinPeriod_ = null;
};
/** @param {boolean} value */
endo.proto.binary.Marker.prototype.setHasMinPeriod = function(value) {
  this.hasMinPeriod_ = value;
};
////////////////////////////////////////
// Class Descriptors
////////////////////////////////////////
/**
 * @param {!jsproto.BinaryReader} reader
 */
endo.proto.binary.Thread.prototype.readFrom = function(reader) {
  if (goog.DEBUG) {
    var lastField = -1;
  }
  while (reader.nextField()) {
    var field = reader.getFieldNumber();
    // In debug builds, we assert that field numbers are monotonically
    // increasing, as we're taking advantage of that property to simplify our
    // decoding.
    if (goog.DEBUG) {
      goog.asserts.assert(lastField <= field);
      lastField = field;
    }
    switch (field) {
    case 1:
      this.id_ = /** @type {number} */(reader.readSint64());
      break;
    case 2:
      this.name_ = /** @type {string} */(reader.readString());
      break;
    default:
      reader.skipField();
      break;
    }
  }
};
/**
 * @param {!jsproto.BinaryWriter} writer
 */
endo.proto.binary.Thread.prototype.writeTo = function(writer) {
  writer.writeSint64(1, this.id_);
  writer.writeString(2, this.name_);
};
/**
 * Converts a proto into a flat array - field N will be copied to array slot
 * N-1 (or slot N if we're in one-based-arrays mode), hash strings will be
 * converted into decimal strings, packed/repeated scalars will be copied, and
 * nested messages will be recursively array-ified.
 * Does not handle byte blob fields.
 * @return {!Array}
 */
endo.proto.binary.Thread.prototype.toJspbArray = function() {
  var result = [];
  if (this.id_ != null) {
    result[1 + -1] = this.id_;;
  }
  if (this.name_ != null) {
    result[2 + -1] = this.name_;;
  }
  return result;
};
/**
 * @param {jsproto.ConstBinaryMessage} other
 * @return {boolean}
 */
endo.proto.binary.Thread.prototype.equalsProto = function(other) {
  // TODO(aappleby): implement equality comparison, if needed.
  goog.asserts.fail('Not implemented!');
  return false;
};
if (goog.DEBUG) {
  /**
   * Generate a debug string for this proto that is in proto2 text format.
   * Extensions are not currently handled.
   * @return {string} The debug string.
   */
  endo.proto.binary.Thread.prototype.toDebugString = function() {
    return this.toDebugStringInternal_(0);
  };
  /**
   * Helper to generate a debug string for this proto at some indent level. The
   * first line is not indented.
   * @param {number} indentLevel The number of spaces by which to indent lines.
   * @return {string} The debug string.
   */
  endo.proto.binary.Thread.prototype.toDebugStringInternal_ = function(indentLevel) {
    var buffer = [];
    var indent = new Array(1 + indentLevel).join(' ');
    if (this.hasId()) {
      var scalar = jsproto.utils.debugScalarToTextFormat(this.getId());
      buffer.push('id: ' + scalar);
    }
    if (this.hasName()) {
      var scalar = jsproto.utils.debugScalarToTextFormat(this.getName());
      buffer.push('name: ' + scalar);
    }
    return buffer.join('\n' + indent);
  };
}
/**
 * Prunes all fields of this message that are equal to their default value.
 * Returns null if all fields were pruned away, otherwise returns 'this'.
 * @return {endo.proto.binary.Thread}
 */
endo.proto.binary.Thread.prototype.pruneProto = function() {
  var alive = false;
  if (this.id_ == 0) {
    this.id_ = null;
  }
  alive |= (this.id_ != null);
  if (this.name_ == '') {
    this.name_ = null;
  }
  alive |= (this.name_ != null);
  return alive ? this : null;
};
/**
 * Clears all fields of the proto.
 */
endo.proto.binary.Thread.prototype.clearProto = function() {
  this.id_ = null;
  this.name_ = null;
};
/**
 * Dematerializes all materialized fields in this proto.
 * @override
 */
endo.proto.binary.Thread.prototype.recycleFields = function() {
};
/**
 * Returns true if this message has an extension with the given field number.
 * @param {number} field
 * @return {boolean}
 * @override
 */
endo.proto.binary.Thread.prototype.hasExtension = function(field) {
  return false;
};
/**
 * Retrieves the extension with the given field number.
 * @param {number} field
 * @return {jsproto.AnyFieldType}
 * @override
 */
endo.proto.binary.Thread.prototype.getExtension = function(field) {
  return null;
};
/**
 * Returns the total number of extensions attached to this message.
 * @return {number}
 * @override
 */
endo.proto.binary.Thread.prototype.getExtensionCount = function() {
  return 0;
};
/** @return {!Array.<!jsproto.BinaryExtension>} */
endo.proto.binary.Thread.prototype.getRawExtensionArray = function() {
  goog.asserts.fail("Type has no extension ranges!");
  return [];
};
/**
 * Gets a mutable copy of an extension in this message, creating it if it
 * isn't present. Only valid for extensions that are messages.
 * @param {number} field
 * @return {jsproto.BinaryMessage}
 */
endo.proto.binary.Thread.prototype.mutableExtension = function(field) {
  goog.asserts.fail("Type has no extension ranges!");
  return null;
};
/**
 * Sets an extension in this message.
 * @param {number} field
 * @param {jsproto.AnyFieldType} value
 */
endo.proto.binary.Thread.prototype.setExtension = function(field, value) {
  goog.asserts.fail("Type has no extension ranges!");
};
////////////////////////////////////////
// Alloc/Free
////////////////////////////////////////
/**
 * @return {!endo.proto.binary.Thread}
 */
endo.proto.binary.Thread.alloc = function() {
  return new endo.proto.binary.Thread();
};
/**
 * @param {endo.proto.binary.ConstThread} constProto
 */
endo.proto.binary.Thread.free = function(constProto) {
  var proto = /** @type {endo.proto.binary.Thread} */(constProto);
  if (proto) proto.clearProto();
};
////////////////////////////////////////
// Static Helper Methods
////////////////////////////////////////
/**
 * @return {!jsproto.BinaryMessage}
 */
endo.proto.binary.Thread.builder = function() {
  return endo.proto.binary.Thread.alloc();
};
/**
 * @param {!jsproto.BinaryMessage} message
 */
endo.proto.binary.Thread.recycler = function(message) {
  var message2 = /** @type {!endo.proto.binary.Thread} */(message);
  message2.recycleFields();
};
/**
 * @param {!jsproto.BinaryMessage} message
 * @param {!jsproto.BinaryReader} stream
 */
endo.proto.binary.Thread.reader = function(message, stream) {
  var message2 = /** @type {!endo.proto.binary.Thread} */(message);
  message2.readFrom(stream);
};
/**
 * @param {!jsproto.ConstBinaryMessage} message
 * @param {!jsproto.BinaryWriter} stream
 */
endo.proto.binary.Thread.writer = function(message, stream) {
  var message2 = /** @type {!endo.proto.binary.Thread} */(message);
  message2.writeTo(stream);
};
/**
 * @param {jsproto.BinaryMessage} message
 * @return {jsproto.BinaryMessage}
 */
endo.proto.binary.Thread.pruner = function(message) {
  var message2 = /** @type {!endo.proto.binary.Thread} */(message);
  return message2.pruneProto();
};
////////////////////////////////////////
// Class Field Accessors
////////////////////////////////////////
/** @override */
endo.proto.binary.Thread.prototype.hasId = function() {
  return this.id_ != null;
};
/** @override */
endo.proto.binary.Thread.prototype.getId = function() {
  if (this.id_ != null) {
    return this.id_;
  } else {
    return 0;
  }
};
/** */
endo.proto.binary.Thread.prototype.clearId = function() {
  this.id_ = null;
};
/** @param {number} value */
endo.proto.binary.Thread.prototype.setId = function(value) {
  this.id_ = value;
};
/** @override */
endo.proto.binary.Thread.prototype.hasName = function() {
  return this.name_ != null;
};
/** @override */
endo.proto.binary.Thread.prototype.getName = function() {
  if (this.name_ != null) {
    return this.name_;
  } else {
    return '';
  }
};
/** */
endo.proto.binary.Thread.prototype.clearName = function() {
  this.name_ = null;
};
/** @param {string} value */
endo.proto.binary.Thread.prototype.setName = function(value) {
  this.name_ = value;
};
////////////////////////////////////////
// Class Descriptors
////////////////////////////////////////
/**
 * @param {!jsproto.BinaryReader} reader
 */
endo.proto.binary.Snapshot.prototype.readFrom = function(reader) {
  if (goog.DEBUG) {
    var lastField = -1;
  }
  while (reader.nextField()) {
    var field = reader.getFieldNumber();
    // In debug builds, we assert that field numbers are monotonically
    // increasing, as we're taking advantage of that property to simplify our
    // decoding.
    if (goog.DEBUG) {
      goog.asserts.assert(lastField <= field);
      lastField = field;
    }
    switch (field) {
    case 1:
      // This block is very hot, factoring it out into a helper method reduces performance.
      this.marker_ = this.marker_ || [];
      var subReader = reader.readMessage();
      var subMessage = endo.proto.binary.Marker.alloc();
      subMessage.readFrom(subReader);
      subReader.free();
      this.marker_.push(subMessage);
      break;
    case 2:
      // This block is very hot, factoring it out into a helper method reduces performance.
      this.task_ = this.task_ || [];
      var subReader = reader.readMessage();
      var subMessage = endo.proto.binary.Task.alloc();
      subMessage.readFrom(subReader);
      subReader.free();
      this.task_.push(subMessage);
      break;
    case 3:
      // This block is very hot, factoring it out into a helper method reduces performance.
      this.thread_ = this.thread_ || [];
      var subReader = reader.readMessage();
      var subMessage = endo.proto.binary.Thread.alloc();
      subMessage.readFrom(subReader);
      subReader.free();
      this.thread_.push(subMessage);
      break;
    case 4:
      this.startTimeMillis_ = /** @type {number} */(reader.readSint64());
      break;
    case 5:
      this.durationNanos_ = /** @type {number} */(reader.readSint64());
      break;
    default:
      reader.skipField();
      break;
    }
  }
};
/**
 * @param {!jsproto.BinaryWriter} writer
 */
endo.proto.binary.Snapshot.prototype.writeTo = function(writer) {
  jsproto.BinaryMessage.writeRepeatedMessageField(
      writer,
      1,
      this.marker_,
      endo.proto.binary.Marker.writer);
  jsproto.BinaryMessage.writeRepeatedMessageField(
      writer,
      2,
      this.task_,
      endo.proto.binary.Task.writer);
  jsproto.BinaryMessage.writeRepeatedMessageField(
      writer,
      3,
      this.thread_,
      endo.proto.binary.Thread.writer);
  writer.writeSint64(4, this.startTimeMillis_);
  writer.writeSint64(5, this.durationNanos_);
};
/**
 * Converts a proto into a flat array - field N will be copied to array slot
 * N-1 (or slot N if we're in one-based-arrays mode), hash strings will be
 * converted into decimal strings, packed/repeated scalars will be copied, and
 * nested messages will be recursively array-ified.
 * Does not handle byte blob fields.
 * @return {!Array}
 */
endo.proto.binary.Snapshot.prototype.toJspbArray = function() {
  var result = [];
  if (this.marker_ != null) {
    var count = this.marker_.length;
    result[1 + -1] = new Array(count);
    for (var i = 0; i < count; i++) {
      result[1 + -1][i] =
          this.marker_[i].toJspbArray();
    }
  }
  if (this.task_ != null) {
    var count = this.task_.length;
    result[2 + -1] = new Array(count);
    for (var i = 0; i < count; i++) {
      result[2 + -1][i] =
          this.task_[i].toJspbArray();
    }
  }
  if (this.thread_ != null) {
    var count = this.thread_.length;
    result[3 + -1] = new Array(count);
    for (var i = 0; i < count; i++) {
      result[3 + -1][i] =
          this.thread_[i].toJspbArray();
    }
  }
  if (this.startTimeMillis_ != null) {
    result[4 + -1] = this.startTimeMillis_;;
  }
  if (this.durationNanos_ != null) {
    result[5 + -1] = this.durationNanos_;;
  }
  return result;
};
/**
 * @param {jsproto.ConstBinaryMessage} other
 * @return {boolean}
 */
endo.proto.binary.Snapshot.prototype.equalsProto = function(other) {
  // TODO(aappleby): implement equality comparison, if needed.
  goog.asserts.fail('Not implemented!');
  return false;
};
if (goog.DEBUG) {
  /**
   * Generate a debug string for this proto that is in proto2 text format.
   * Extensions are not currently handled.
   * @return {string} The debug string.
   */
  endo.proto.binary.Snapshot.prototype.toDebugString = function() {
    return this.toDebugStringInternal_(0);
  };
  /**
   * Helper to generate a debug string for this proto at some indent level. The
   * first line is not indented.
   * @param {number} indentLevel The number of spaces by which to indent lines.
   * @return {string} The debug string.
   */
  endo.proto.binary.Snapshot.prototype.toDebugStringInternal_ = function(indentLevel) {
    var buffer = [];
    var indent = new Array(1 + indentLevel).join(' ');
    if (this.hasMarker()) {
      for (var i = 0; i < this.markerSize(); i++) {
        buffer.push('marker { #' + i);
        buffer.push('  ' + this.getMarker(i).toDebugStringInternal_(indentLevel + 2));
        buffer.push('} #marker ' + i);
      }
    }
    if (this.hasTask()) {
      for (var i = 0; i < this.taskSize(); i++) {
        buffer.push('task { #' + i);
        buffer.push('  ' + this.getTask(i).toDebugStringInternal_(indentLevel + 2));
        buffer.push('} #task ' + i);
      }
    }
    if (this.hasThread()) {
      for (var i = 0; i < this.threadSize(); i++) {
        buffer.push('thread { #' + i);
        buffer.push('  ' + this.getThread(i).toDebugStringInternal_(indentLevel + 2));
        buffer.push('} #thread ' + i);
      }
    }
    if (this.hasStartTimeMillis()) {
      var scalar = jsproto.utils.debugScalarToTextFormat(this.getStartTimeMillis());
      buffer.push('start_time_millis: ' + scalar);
    }
    if (this.hasDurationNanos()) {
      var scalar = jsproto.utils.debugScalarToTextFormat(this.getDurationNanos());
      buffer.push('duration_nanos: ' + scalar);
    }
    return buffer.join('\n' + indent);
  };
}
/**
 * Prunes all fields of this message that are equal to their default value.
 * Returns null if all fields were pruned away, otherwise returns 'this'.
 * @return {endo.proto.binary.Snapshot}
 */
endo.proto.binary.Snapshot.prototype.pruneProto = function() {
  var alive = false;
  this.marker_ = /** @type {Array.<endo.proto.binary.Marker>} */
      (jsproto.BinaryMessage.pruneVariant(this.marker_,
      endo.proto.binary.Marker.pruner));
  alive |= (this.marker_ != null);
  this.task_ = /** @type {Array.<endo.proto.binary.Task>} */
      (jsproto.BinaryMessage.pruneVariant(this.task_,
      endo.proto.binary.Task.pruner));
  alive |= (this.task_ != null);
  this.thread_ = /** @type {Array.<endo.proto.binary.Thread>} */
      (jsproto.BinaryMessage.pruneVariant(this.thread_,
      endo.proto.binary.Thread.pruner));
  alive |= (this.thread_ != null);
  if (this.startTimeMillis_ == 0) {
    this.startTimeMillis_ = null;
  }
  alive |= (this.startTimeMillis_ != null);
  if (this.durationNanos_ == 0) {
    this.durationNanos_ = null;
  }
  alive |= (this.durationNanos_ != null);
  return alive ? this : null;
};
/**
 * Clears all fields of the proto.
 */
endo.proto.binary.Snapshot.prototype.clearProto = function() {
  if (this.marker_) {
    for (var i = 0; i < this.marker_.length; i++) {
      endo.proto.binary.Marker.free(this.marker_[i]);
    }
  }
  this.marker_ = null;
  if (this.task_) {
    for (var i = 0; i < this.task_.length; i++) {
      endo.proto.binary.Task.free(this.task_[i]);
    }
  }
  this.task_ = null;
  if (this.thread_) {
    for (var i = 0; i < this.thread_.length; i++) {
      endo.proto.binary.Thread.free(this.thread_[i]);
    }
  }
  this.thread_ = null;
  this.startTimeMillis_ = null;
  this.durationNanos_ = null;
};
/**
 * Dematerializes all materialized fields in this proto.
 * @override
 */
endo.proto.binary.Snapshot.prototype.recycleFields = function() {
};
/**
 * Returns true if this message has an extension with the given field number.
 * @param {number} field
 * @return {boolean}
 * @override
 */
endo.proto.binary.Snapshot.prototype.hasExtension = function(field) {
  return false;
};
/**
 * Retrieves the extension with the given field number.
 * @param {number} field
 * @return {jsproto.AnyFieldType}
 * @override
 */
endo.proto.binary.Snapshot.prototype.getExtension = function(field) {
  return null;
};
/**
 * Returns the total number of extensions attached to this message.
 * @return {number}
 * @override
 */
endo.proto.binary.Snapshot.prototype.getExtensionCount = function() {
  return 0;
};
/** @return {!Array.<!jsproto.BinaryExtension>} */
endo.proto.binary.Snapshot.prototype.getRawExtensionArray = function() {
  goog.asserts.fail("Type has no extension ranges!");
  return [];
};
/**
 * Gets a mutable copy of an extension in this message, creating it if it
 * isn't present. Only valid for extensions that are messages.
 * @param {number} field
 * @return {jsproto.BinaryMessage}
 */
endo.proto.binary.Snapshot.prototype.mutableExtension = function(field) {
  goog.asserts.fail("Type has no extension ranges!");
  return null;
};
/**
 * Sets an extension in this message.
 * @param {number} field
 * @param {jsproto.AnyFieldType} value
 */
endo.proto.binary.Snapshot.prototype.setExtension = function(field, value) {
  goog.asserts.fail("Type has no extension ranges!");
};
////////////////////////////////////////
// Alloc/Free
////////////////////////////////////////
/**
 * @return {!endo.proto.binary.Snapshot}
 */
endo.proto.binary.Snapshot.alloc = function() {
  return new endo.proto.binary.Snapshot();
};
/**
 * @param {endo.proto.binary.ConstSnapshot} constProto
 */
endo.proto.binary.Snapshot.free = function(constProto) {
  var proto = /** @type {endo.proto.binary.Snapshot} */(constProto);
  if (proto) proto.clearProto();
};
////////////////////////////////////////
// Static Helper Methods
////////////////////////////////////////
/**
 * @return {!jsproto.BinaryMessage}
 */
endo.proto.binary.Snapshot.builder = function() {
  return endo.proto.binary.Snapshot.alloc();
};
/**
 * @param {!jsproto.BinaryMessage} message
 */
endo.proto.binary.Snapshot.recycler = function(message) {
  var message2 = /** @type {!endo.proto.binary.Snapshot} */(message);
  message2.recycleFields();
};
/**
 * @param {!jsproto.BinaryMessage} message
 * @param {!jsproto.BinaryReader} stream
 */
endo.proto.binary.Snapshot.reader = function(message, stream) {
  var message2 = /** @type {!endo.proto.binary.Snapshot} */(message);
  message2.readFrom(stream);
};
/**
 * @param {!jsproto.ConstBinaryMessage} message
 * @param {!jsproto.BinaryWriter} stream
 */
endo.proto.binary.Snapshot.writer = function(message, stream) {
  var message2 = /** @type {!endo.proto.binary.Snapshot} */(message);
  message2.writeTo(stream);
};
/**
 * @param {jsproto.BinaryMessage} message
 * @return {jsproto.BinaryMessage}
 */
endo.proto.binary.Snapshot.pruner = function(message) {
  var message2 = /** @type {!endo.proto.binary.Snapshot} */(message);
  return message2.pruneProto();
};
////////////////////////////////////////
// Class Field Accessors
////////////////////////////////////////
/** @override */
endo.proto.binary.Snapshot.prototype.hasMarker = function() {
  if (this.marker_ != null) return true;
  return false;
};
/** @override */
endo.proto.binary.Snapshot.prototype.markerSize = function() {
  return this.marker_ ? this.marker_.length : 0;
};
/** @override */
endo.proto.binary.Snapshot.prototype.getMarker = function(i) {
  goog.asserts.assert(this.marker_);
  goog.asserts.assert((i >= 0) && (i < this.marker_.length));
  goog.asserts.assert(this.marker_[i] != null);
  return this.marker_[i];
};
/** @override */
endo.proto.binary.Snapshot.prototype.recycleMarker = function() {
};
/** */
endo.proto.binary.Snapshot.prototype.clearMarker = function() {
  this.marker_ = null;
};
/**
 * @return {!Array.<!endo.proto.binary.ConstMarker>}
 * @override
 */
endo.proto.binary.Snapshot.prototype.getMarkerArray = function() {
  goog.asserts.assert(this.marker_);
  return this.marker_;
};
/**
 * @param {number} i
 * @param {!endo.proto.binary.Marker} value
 */
endo.proto.binary.Snapshot.prototype.setMarker = function(i, value) {
  goog.asserts.assert(this.marker_);
  goog.asserts.assert((i >= 0) && (i < this.marker_.length));
  this.marker_[i] = value;
};
/**
 * @param {!Array.<!endo.proto.binary.Marker>} value
 */
endo.proto.binary.Snapshot.prototype.setMarkerArray = function(value) {
  this.marker_ = value;
};
/**
 * @param {number} i
 * @return {!endo.proto.binary.Marker}
 */
endo.proto.binary.Snapshot.prototype.mutableMarker = function(i) {
  goog.asserts.assert(this.marker_);
  goog.asserts.assert((i >= 0) && (i < this.marker_.length));
  var result = this.marker_[i];
  goog.asserts.assert(result);
  return result;
};
/**
 * @return {!endo.proto.binary.Marker}
 */
endo.proto.binary.Snapshot.prototype.addMarker = function() {
  this.marker_ = this.marker_  || [];
  var result = jsproto.BinaryMessage.appendInstance(
      endo.proto.binary.Marker.builder,
      this.marker_);
  return /** @type {!endo.proto.binary.Marker} */(result);
};
/** @override */
endo.proto.binary.Snapshot.prototype.hasTask = function() {
  if (this.task_ != null) return true;
  return false;
};
/** @override */
endo.proto.binary.Snapshot.prototype.taskSize = function() {
  return this.task_ ? this.task_.length : 0;
};
/** @override */
endo.proto.binary.Snapshot.prototype.getTask = function(i) {
  goog.asserts.assert(this.task_);
  goog.asserts.assert((i >= 0) && (i < this.task_.length));
  goog.asserts.assert(this.task_[i] != null);
  return this.task_[i];
};
/** @override */
endo.proto.binary.Snapshot.prototype.recycleTask = function() {
};
/** */
endo.proto.binary.Snapshot.prototype.clearTask = function() {
  this.task_ = null;
};
/**
 * @return {!Array.<!endo.proto.binary.ConstTask>}
 * @override
 */
endo.proto.binary.Snapshot.prototype.getTaskArray = function() {
  goog.asserts.assert(this.task_);
  return this.task_;
};
/**
 * @param {number} i
 * @param {!endo.proto.binary.Task} value
 */
endo.proto.binary.Snapshot.prototype.setTask = function(i, value) {
  goog.asserts.assert(this.task_);
  goog.asserts.assert((i >= 0) && (i < this.task_.length));
  this.task_[i] = value;
};
/**
 * @param {!Array.<!endo.proto.binary.Task>} value
 */
endo.proto.binary.Snapshot.prototype.setTaskArray = function(value) {
  this.task_ = value;
};
/**
 * @param {number} i
 * @return {!endo.proto.binary.Task}
 */
endo.proto.binary.Snapshot.prototype.mutableTask = function(i) {
  goog.asserts.assert(this.task_);
  goog.asserts.assert((i >= 0) && (i < this.task_.length));
  var result = this.task_[i];
  goog.asserts.assert(result);
  return result;
};
/**
 * @return {!endo.proto.binary.Task}
 */
endo.proto.binary.Snapshot.prototype.addTask = function() {
  this.task_ = this.task_  || [];
  var result = jsproto.BinaryMessage.appendInstance(
      endo.proto.binary.Task.builder,
      this.task_);
  return /** @type {!endo.proto.binary.Task} */(result);
};
/** @override */
endo.proto.binary.Snapshot.prototype.hasThread = function() {
  if (this.thread_ != null) return true;
  return false;
};
/** @override */
endo.proto.binary.Snapshot.prototype.threadSize = function() {
  return this.thread_ ? this.thread_.length : 0;
};
/** @override */
endo.proto.binary.Snapshot.prototype.getThread = function(i) {
  goog.asserts.assert(this.thread_);
  goog.asserts.assert((i >= 0) && (i < this.thread_.length));
  goog.asserts.assert(this.thread_[i] != null);
  return this.thread_[i];
};
/** @override */
endo.proto.binary.Snapshot.prototype.recycleThread = function() {
};
/** */
endo.proto.binary.Snapshot.prototype.clearThread = function() {
  this.thread_ = null;
};
/**
 * @return {!Array.<!endo.proto.binary.ConstThread>}
 * @override
 */
endo.proto.binary.Snapshot.prototype.getThreadArray = function() {
  goog.asserts.assert(this.thread_);
  return this.thread_;
};
/**
 * @param {number} i
 * @param {!endo.proto.binary.Thread} value
 */
endo.proto.binary.Snapshot.prototype.setThread = function(i, value) {
  goog.asserts.assert(this.thread_);
  goog.asserts.assert((i >= 0) && (i < this.thread_.length));
  this.thread_[i] = value;
};
/**
 * @param {!Array.<!endo.proto.binary.Thread>} value
 */
endo.proto.binary.Snapshot.prototype.setThreadArray = function(value) {
  this.thread_ = value;
};
/**
 * @param {number} i
 * @return {!endo.proto.binary.Thread}
 */
endo.proto.binary.Snapshot.prototype.mutableThread = function(i) {
  goog.asserts.assert(this.thread_);
  goog.asserts.assert((i >= 0) && (i < this.thread_.length));
  var result = this.thread_[i];
  goog.asserts.assert(result);
  return result;
};
/**
 * @return {!endo.proto.binary.Thread}
 */
endo.proto.binary.Snapshot.prototype.addThread = function() {
  this.thread_ = this.thread_  || [];
  var result = jsproto.BinaryMessage.appendInstance(
      endo.proto.binary.Thread.builder,
      this.thread_);
  return /** @type {!endo.proto.binary.Thread} */(result);
};
/** @override */
endo.proto.binary.Snapshot.prototype.hasStartTimeMillis = function() {
  return this.startTimeMillis_ != null;
};
/** @override */
endo.proto.binary.Snapshot.prototype.getStartTimeMillis = function() {
  if (this.startTimeMillis_ != null) {
    return this.startTimeMillis_;
  } else {
    return 0;
  }
};
/** */
endo.proto.binary.Snapshot.prototype.clearStartTimeMillis = function() {
  this.startTimeMillis_ = null;
};
/** @param {number} value */
endo.proto.binary.Snapshot.prototype.setStartTimeMillis = function(value) {
  this.startTimeMillis_ = value;
};
/** @override */
endo.proto.binary.Snapshot.prototype.hasDurationNanos = function() {
  return this.durationNanos_ != null;
};
/** @override */
endo.proto.binary.Snapshot.prototype.getDurationNanos = function() {
  if (this.durationNanos_ != null) {
    return this.durationNanos_;
  } else {
    return 0;
  }
};
/** */
endo.proto.binary.Snapshot.prototype.clearDurationNanos = function() {
  this.durationNanos_ = null;
};
/** @param {number} value */
endo.proto.binary.Snapshot.prototype.setDurationNanos = function(value) {
  this.durationNanos_ = value;
};
////////////////////////////////////////
// Extensions
////////////////////////////////////////

//java/com/google/net/rpc3/profiling/jendoscope/web/endo_snapshot.js
// Copyright 2011 Google Inc. All Rights Reserved.

/**
 * @fileoverview An Endoscope snapshot is a collection of markers and threads,
 * along with some additional metadata about when the snapshot occurred.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('endo.Snapshot');

goog.require('endo.util');


/**
 * Creates a new Endo snapshot from the raw snapshot.
 * @param {endo.proto.binary.Snapshot=} opt_rawSnapshot The optional raw
 *     snapshot used to initialize this snapshot.
 * @constructor
 * @struct
 */
endo.Snapshot = function(opt_rawSnapshot) {

  /**
   * Maps all global marker IDs to their corresponding marker objects.
   * @private {!Object.<number, !endo.Marker>}
   */
  this.markerMap_ = {};

  /**
   * The list of all markers in the above map.
   * @private {!Array.<!endo.Marker>}
   */
  this.markerList_ = [];

  /**
   * Maps all global thread IDs to their corresponding thread objects.
   * @private {!Object.<number, !endo.Thread>}
   */
  this.threadMap_ = {};

  /**
   * The list of all threads in the above map.
   * @private {!Array.<!endo.Thread>}
   */
  this.threadList_ = [];

  /**
   * The time, in Unix seconds, when this snapshot was created on the client.
   * @private {number}
   */
  this.clientTimeCreated_ = endo.sync.unixTime();

  /**
   * The time, in Unix seconds, of the first event recorded in this snapshot.
   * @private {number}
   */
  this.serverTimeBegin_ = endo.util.INVALID_TIME;

  /**
   * The time, in Unix seconds, of the last event recorded in this snapshot.
   * @private {number}
   */
  this.serverTimeEnd_ = endo.util.INVALID_TIME;

  if (opt_rawSnapshot) {
    this.loadRawSnapshot(opt_rawSnapshot);
  }
};

/**
 * Clears all markers, threads, and metadata from this snapshot.
 */
endo.Snapshot.prototype.clear = function() {
  this.markerMap_ = {};
  this.threadMap_ = {};
  this.clientTimeCreated_ = endo.sync.unixTime();
  this.serverTimeBegin_ = endo.util.INVALID_TIME;
  this.serverTimeEnd_ = endo.util.INVALID_TIME;
};

/**
 * Returns true if this snapshot is empty.
 *
 * @return {boolean} True if this snapshot is empty.
 */
endo.Snapshot.prototype.empty = function() {
  return this.serverTimeBegin_ >= this.serverTimeEnd_;
};

/**
 * @return {!Array.<!endo.Marker>} The list of all markers in the snapshot.
 */
endo.Snapshot.prototype.getMarkerList = function() {
  return this.markerList_;
};

/**
 * Retrieves a single marker by ID.
 *
 * @param {number} markerId The ID of the marker to retrieve.
 * @return {endo.Marker} The marker requested.
 */
endo.Snapshot.prototype.getMarker = function(markerId) {
  return this.markerMap_[markerId];
};

/**
 * Adds or updates a marker in this snapshot with fresh data from a more recent
 * snapshot.
 *
 * @param {number} markerId The ID of the marker to update.
 * @param {!endo.Marker} marker The fresh marker data.
 */
endo.Snapshot.prototype.setMarker = function(markerId, marker) {
  if (this.markerMap_[markerId]) {
    this.markerMap_[markerId].update(marker);
  } else {
    this.markerMap_[markerId] = marker;
  }
};

/**
 * @return {!Array.<!endo.Thread>} The list of all threads in the snapshot.
 */
endo.Snapshot.prototype.getThreadList = function() {
  return this.threadList_;
};

/**
 * Retrives a single thread by ID.
 *
 * @param {number} threadId The ID of the thread to retrieve.
 * @return {endo.Thread} The thread requested.
 */
endo.Snapshot.prototype.getThread = function(threadId) {
  return this.threadMap_[threadId];
};

/**
 * Adds or updates a thread in this snapshot with fresh data from a more recent
 * snapshot.
 *
 * @param {number} threadId The ID of the thread to update.
 * @param {!endo.Thread} thread The fresh thread data.
 */
endo.Snapshot.prototype.setThread = function(threadId, thread) {
  if (this.threadMap_[threadId]) {
    this.threadMap_[threadId].update(thread);
  } else {
    this.threadMap_[threadId] = thread;
    this.threadList_.push(thread);
  }
};

/**
 * @return {number}
 */
endo.Snapshot.prototype.getServerTimeBegin = function() {
  return this.serverTimeBegin_;
};

/**
 * @return {number}
 */
endo.Snapshot.prototype.getServerTimeEnd = function() {
  return this.serverTimeEnd_;
};

/**
 * Returns an endo.Range object representing the server time range covered by
 * this snapshot, or an empty range if the snapshot is empty.
 *
 * @return {endo.Range} The time range.
 */
endo.Snapshot.prototype.getTimeRange = function() {
  var result = endo.Range.EmptyRange();
  var threads = this.getThreadList();
  for (var i = 0; i < threads.length; i++) {
    var thread = threads[i];
    result = result.enclose(thread.getTimeRange());
  }
  return result;
};

/**
 * Converts a raw snapshot object produced from either a deserialized proto or
 * a JSON request into Endoscope client objects and updates all objects in this
 * snapshot with the fresh data.
 *
 * @param {!endo.proto.binary.Snapshot} rawSnapshot The raw snapshot we're
 *     processing.
 */
endo.Snapshot.prototype.loadRawSnapshot = function(rawSnapshot) {
  // Update the snapshot time range to include the new snapshot.
  var nanoBegin = 0;
  var nanoEnd = rawSnapshot.getDurationNanos();
  var timeBegin = rawSnapshot.getStartTimeMillis() / 1000;
  var timeEnd = timeBegin + (nanoEnd - nanoBegin) / 1000000000;
  var timeDelta = timeEnd - timeBegin;

  // The conversion between Unix seconds and the time unit used in snapshot is
  // established here.
  // The 'cycle's provided to sync here is defined as: nanoseconds since
  // JEndoscope initialization.
  endo.sync.update(nanoBegin, nanoEnd, timeBegin, timeEnd);

  this.serverTimeBegin_ = 0;
  if (timeDelta > this.serverTimeEnd_) {
    this.serverTimeEnd_ = timeDelta;
  }

  // Update all marker objects.
  for (var i = 0; i < rawSnapshot.markerSize(); ++i) {
    var rawMarker = rawSnapshot.getMarker(i);
    if (rawMarker) {
      var marker = endo.Marker.loadRawMarker(rawMarker);
      this.setMarker(rawMarker.getId(), marker);
    }
  }

  // Update all thread objects.
  for (var i = 0; i < rawSnapshot.threadSize(); ++i) {
    var rawThread = rawSnapshot.getThread(i);
    if (rawThread) {
      var thread = endo.Thread.loadRawThread(rawThread);
      this.setThread(rawThread.getId(), thread);
    }
  }

  // Update all tasks.
  var allTasks = [];
  var iRoot = rawSnapshot.taskSize() - 1;
  while (iRoot >= 0) {
    var rawTask = rawSnapshot.getTask(iRoot);
    var rootTask = endo.Task.loadRawTask(rawTask, timeDelta);

    var rootComplete = rootTask.isComplete();
    var rootBeginNano = rawTask.getStartTimeNanos();
    if (rootComplete) {
      var rootMarker = this.getMarker(rootTask.getMarkerId());
      var rootBegin = rootTask.getTimeMin();
      var rootEnd = rootTask.getTimeMax();
      rootMarker.aggregateTaskTime(rootEnd - rootBegin);
    }

    var jFirst = iRoot - 1;

    // After this loop, task[jFirst + 1 .. iRoot - 1] in rawSnapshot will be all
    // subtasks of task[iRoot].
    while (jFirst >= 0 &&
        rootBeginNano < rawSnapshot.getTask(jFirst).getStartTimeNanos() &&
        rootTask.getThreadId() == rawSnapshot.getTask(jFirst).getThreadId()) {
      jFirst--;
    }

    // Turn task[jFirst + 1 .. iRoot - 1] into atoms
    var logs = [];
    for (var k = jFirst + 1; k < iRoot; k++) {
      var t = rawSnapshot.getTask(k);
      var mId = t.getMarkerId();
      var m = this.getMarker(mId);
      switch (m.type) {
        case endo.proto.binary.Marker.MarkerType.TASK:
          var complete = (t.getEndTimeNanos() > 0);
          var atom1 = new endo.Atom();
          atom1.type = endo.Atom.AtomType.SCOPE_BEGIN;
          atom1.markerId = mId;
          atom1.time = endo.sync.cycleToTime(t.getStartTimeNanos());
          atom1.tag = t.getTag();
          logs.push(atom1);
          if (complete) {
            var atom2 = new endo.Atom();
            atom2.type = endo.Atom.AtomType.SCOPE_END;
            atom2.time = endo.sync.cycleToTime(t.getEndTimeNanos());
            if (atom2.time <= 0) {
              atom2.time = timeDelta;
            }
            logs.push(atom2);
            if (rootComplete) {
              // we don't count any task that belongs to an incomplete tree.
              m.aggregateTaskTime(atom2.time - atom1.time);
            }
          }
          break;
        case endo.proto.binary.Marker.MarkerType.EVENT:
          var atom = new endo.Atom();
          atom.type = endo.Atom.AtomType.EVENT;
          atom.markerId = mId;
          atom.time = endo.sync.cycleToTime(t.getStartTimeNanos());
          atom.tag = t.getTag();
          logs.push(atom);
          break;
        case endo.proto.binary.Marker.MarkerType.TAG:
          var atom = new endo.Atom();
          atom.type = endo.Atom.AtomType.TAG;
          atom.markerId = mId;
          atom.time = endo.sync.cycleToTime(t.getStartTimeNanos());
          atom.tag = t.getTag();
          logs.push(atom);
          break;
      }
    }
    logs.sort(function(a, b) { return a.time - b.time; });
    var span = endo.Span.loadAtoms(rootTask, logs, timeDelta);
    rootTask.setRootSpan(span);
    allTasks.push(rootTask);

    iRoot = jFirst;
  }

  for (var i = allTasks.length - 1; i >= 0; i--) {
    var task = allTasks[i];
    var thread = this.getThread(task.threadId_);
    thread.setTask(task.taskId_, task);
  }

  // Do post-processing.
  var threads = this.getThreadList();
  for (var i = 0; i < threads.length; i++) {
    threads[i].updateDepths();
  }
};

//java/com/google/net/rpc3/profiling/jendoscope/web/endo_network.js
// Copyright 2011 Google Inc. All Rights Reserved.

/**
 * @fileoverview All Endoscope network-related code goes here.
 *
 * TODO(aappleby): This needs to be turned into an actual object.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('endo.network');

goog.require('endo.Snapshot');
goog.require('endo.proto.binary.Snapshot');
goog.require('jsproto.BinaryReader');



/**
 * True if the Endoscope client needs to request an update from the server.
 * @private {boolean}
 */
endo.network.needsUpdate_ = false;

/**
 * True if the user has requested a fresh Endoscope snapshot from the server.
 * @private {boolean}
 */
endo.network.needsSnapshot_ = false;

/**
 * True if we already have a request for a snapshot or for an update in flight.
 * @private {boolean}
 */
endo.network.requestPending_ = false;

/**
 * True if we're continuously streaming Endoscope updates from the server.
 * @private {boolean}
 */
endo.network.streaming_ = false;

/**
 * True if the network connection between the client and server has
 * encountered an error since the last successful full snapshot.
 * @private {boolean}
 */
endo.network.error_ = false;

/**
 * The time interval, in milliseconds, that we wait between update requests.
 * @private {number}
 */
endo.network.updateInterval_ = 250;

/**
 * The size, in bytes, of the last chunk of data we received from the server.
 * @private {number}
 */
endo.network.lastDataSize_ = 0;

/**
 * True if this client should connect to its server, false if we're running
 * in 'offline' mode for testing.
 * @private {boolean}
 */
endo.network.online_ = true;

/**
 * The global snapshot we received from the server.
 * TODO(aappleby): This needs to not be a global object anymore.
 * @type {endo.Snapshot}
 */
endo.network.snapshot = null;

/**
 * Initializes all network-related variables.
 */
endo.network.install = function() {
  endo.network.clientTimeBegin_ = endo.sync.unixTime();

  // If the Endoscope console was served from a "file://..." URL, we're running
  // in offline mode for testing purposes.
  if (location.protocol == 'file:') {
    endo.network.online_ = false;
  }
};

/**
 * @return {boolean} True if the network is online, false if we're in offline
 *     test mode.
 */
endo.network.isOnline = function() {
  return endo.network.online_;
};

/**
 * @return {boolean} True if the network connection has encountered an error.
 */
endo.network.hasError = function() {
  return endo.network.error_;
};

/**
 * @return {number} The wall-clock time, in Unix seconds, when the server was
 *     started.
 */
endo.network.getServerTimeBegin = function() {
  return endo.sync.serverTimeBegin;
};

/**
 * Starts streaming endoscope data from the server.
 * (Note: Yes, this is technically polling instead of streaming right now, but
 * eventually there will be real stream stuff happening.)
 */
endo.network.startStream = function() {
  endo.network.streaming_ = true;
  endo.network.requestUpdate();
};

/**
 * Stops streaming endoscope data from the server.
 */
endo.network.stopStream = function() {
  endo.network.streaming_ = false;
};

/**
 * Dispatches ajax requests in order by priority if there aren't any already
 * in flight.
 */
endo.network.dispatchRequests = function() {

  // Do nothing if there is already a request in flight.
  if (endo.network.requestPending_) {
    return;
  }

  // If there is nothing in flight, snapshot requests take priority over update
  // requests as they would clobber the update data anyhow.
  if (endo.network.needsSnapshot_) {
    endo.network.asyncFetchProto('proto',
                                 endo.network.receiveSnapshot,
                                 endo.network.receiveSnapshotError);
    return;
  }

  // If there are no snapshots requested, we handle update requests. Updates
  // will be merged with the existing snapshot when received. If we already have
  // a snapshot, then the next update should begin right where the previous one
  // ended. If we don't, we want a full snapshot instead of an update.
  if (endo.network.needsUpdate_) {
    var query = 'proto';
    endo.network.asyncFetchProto(query,
                                 endo.network.receiveUpdate,
                                 endo.network.receiveUpdateError);
    return;
  }
};

/**
 * Convenience function that dispatches an asynchronous network request using
 * the Chrome-specific "responseType = 'arraybuffer'" feature to receive binary
 * protocol buffers from the server. The protocol buffers are then parsed using
 * binary JSPB and sent to the user-specified callbacks.
 *
 * @param {string} theUrl The URL we're requesting asynchronously.
 * @param {function(!endo.proto.binary.Snapshot)} callbackSuccess The
 *     success callback.
 * @param {function()} callbackError The error callback.
 */
endo.network.asyncFetchProto = function(theUrl,
                                         callbackSuccess,
                                         callbackError) {
  endo.network.requestPending_ = true;
  var xhr = new XMLHttpRequest();
  xhr.open('GET', theUrl, true);
  xhr.responseType = 'arraybuffer';

  xhr.onreadystatechange = function(event) {
    if (xhr.readyState != xhr['DONE']) return;

    if (xhr.statusText != 'OK') {
      callbackError();
      endo.network.requestPending_ = false;
      endo.network.dispatchRequests();
      endo.network.error_ = true;
      return;
    }

    endo.network.lastDataSize_ = xhr.response.byteLength;
    var buffer = /** @type {ArrayBuffer} */ (xhr.response);
    if (buffer) {
      var bytes = new Uint8Array(buffer);
      var reader = jsproto.BinaryReader.alloc(bytes);
      var rawSnapshot = new endo.proto.binary.Snapshot();
      rawSnapshot.readFrom(reader);
      reader.free();
      if (rawSnapshot) {
        endo.network.error_ = false;
        callbackSuccess(rawSnapshot);
      } else {
        endo.network.error_ = true;
        callbackError();
      }
    }
    endo.network.requestPending_ = false;
    endo.network.dispatchRequests();
  };

  xhr.send();
};

/**
 * Queues a snapshot request and dispatches any pending requests.
 */
endo.network.requestSnapshot = function() {
  endo.network.needsSnapshot_ = true;
  endo.network.dispatchRequests();
};

/**
 * Handles receiving a snapshot object from the server.
 * @param {!endo.proto.binary.Snapshot} rawSnapshot The raw snapshot we
 *     received from the server.
 */
endo.network.receiveSnapshot = function(rawSnapshot) {
  endo.network.needsSnapshot_ = false;
  document.getElementById('debug2').innerHTML = 'snapshot OK';
  document.getElementById('debug6').innerHTML =
      endo.network.lastDataSize_ + ' bytes';

  endo.network.snapshot = new endo.Snapshot(rawSnapshot);

  // If we're not auto-scrolling, receiving a snapshot always resets the view.
  if (!endo.ui.scroll) endo.view.resetViewport();
};

/**
 * Handles errors during snapshot requests.
 * TODO(aappleby): Give the user some real feedback when errors occur.
 */
endo.network.receiveSnapshotError = function() {
  endo.network.needsSnapshot_ = false;
  document.getElementById('debug2').innerHTML = 'snapshot failed';
};

/**
 * Schedules an update request to happen in the near future.
 */
endo.network.scheduleNextUpdate = function() {
  if (endo.network.streaming_) {
    setTimeout(endo.network.requestUpdate, endo.network.updateInterval_);
  }
};

/**
 * Queues an update request and dispatches any pending requests.
 */
endo.network.requestUpdate = function() {
  endo.network.needsUpdate_ = true;
  endo.network.dispatchRequests();
};

/**
 * Handles receiving an update object from the server and sets a timer to
 * request another update in the near future.
 *
 * @param {!endo.proto.binary.Snapshot} rawUpdate The raw update we
 *     received from the server.
 */
endo.network.receiveUpdate = function(rawUpdate) {
  endo.network.needsUpdate_ = false;
  document.getElementById('debug3').innerHTML = 'update OK';
  document.getElementById('debug6').innerHTML =
      endo.network.lastDataSize_ + ' bytes';

  if (!endo.network.snapshot) {
    endo.network.snapshot = new endo.Snapshot(rawUpdate);
  } else {
    endo.network.snapshot.loadRawSnapshot(rawUpdate);
  }
  endo.network.scheduleNextUpdate();
};

/**
 * Handles errors during update requests.
 * TODO(aappleby): Give the user some real feedback when errors occur.
 */
endo.network.receiveUpdateError = function() {
  endo.network.needsUpdate_ = false;
  document.getElementById('debug3').innerHTML = 'update failed';
  endo.network.scheduleNextUpdate();
};

//java/com/google/net/rpc3/profiling/jendoscope/web/endo_span.js
// Copyright 2011 Google Inc. All Rights Reserved.

/**
 * @fileoverview An Endoscope span represents a single span of time captured by
 *               the server-side instrumentation macros.
 *
 * Endoscope spans can contain both event objects representing actions that
 * happened during that time span and child spans representing finer details.
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('endo.Span');

goog.require('endo.Event');
goog.require('endo.Range');
goog.require('endo.Rect');
goog.require('endo.util');



/**
 * Represents one instrumented time range, possibly with sub-ranges, along with
 * metadata for rendering & metrics.
 * @struct
 * @constructor
 */
endo.Span = function() {
  /**
   * The type of this span.
   * @type {endo.Span.SpanType}
   */
  this.type = endo.Span.SpanType.INVALID;

  /**
   * The tag we'll draw for this span on the timeline.
   * @type {string}
   */
  this.tag = endo.util.INVALID_TAG;

  /**
   * The array that include all the tags associated, including the tasks' own
   * tag and all tags of its Events and Tags.
   * @type {!Array.<string>}
   */
  this.allTags = [];

  /**
   * The numeric ID of the marker that created this span.
   * @type {number}
   */
  this.markerId = endo.util.INVALID_ID;

  /**
   * True if we've received both the 'begin' and the 'end' event for this span
   * from the server.
   * @type {boolean}
   */
  this.complete = false;

  /**
   * True if we've received the 'begin' event for this span, we have _not_
   * received the 'end' event for this span, and the server has forgotten about
   * this span due to recycling of its internal buffers.
   * @type {boolean}
   */
  this.lost = false;

  /**
   * The time, in seconds since server start, that the event which created this
   * span began.
   * @type {number}
   */
  this.timeMin = 0;

  /**
   * The time, in seconds since server start, that the event which created this
   * span ended.
   * @type {number}
   */
  this.timeMax = 0;

  /**
   * The total time encompassed by this span, including time spent in child
   * spans.
   * @type {number}
   */
  this.timeInc = 0;

  /**
   * The total time encompassed by this span, minus time spent in child spans.
   * @type {number}
   */
  this.timeEx = 0;

  /**
   * The number of units of work associated with this span.
   * @type {number}
   */
  this.units = 0;

  /**
   * The nesting depth of this span.
   * @type {number}
   */
  this.depth = 0;

  /**
   * True if we should draw this span on the timeline.
   * @type {boolean}
   */
  this.visible = true;

  /**
   * True if we should draw the children of this span on the timeline.
   * @type {boolean}
   */
  this.childrenVisible = true;

  /**
   * True if the span is large enough on screen to draw additional details
   * (text, event markers) inside it.
   * @type {boolean}
   */
  this.drawDetail = true;

  /**
   * The global ID of the subtask associated with this span, if there is one.
   * @type {number}
   */
  this.subtaskId = endo.util.INVALID_ID;

  /**
   * The children of this span.
   * @type {!Array.<!endo.Span>}
   */
  this.spans = [];
  this.spans.back = function() {
    return this[this.length - 1];
  };

  /**
   * The events that occurred during this span.
   * @type {!Array.<!endo.Event>}
   */
  this.events = [];

  /**
   * The screen-space location of this span.
   * @type {!endo.Rect}
   */
  this.screenRect = new endo.Rect(0, 0, 0, 0);

  /**
   * The clip rectangle used for rendering all the text inside this span.
   * @type {!endo.Rect}
   */
  this.textRect = new endo.Rect(0, 0, 0, 0);

  /**
   * The clip rectangle representing the position of the span caption
   * (description, etc.) inside this span.
   * @type {!endo.Rect}
   */
  this.captionRect = new endo.Rect(0, 0, 0, 0);

  /**
   * The clip rectangle representing the position of the duration text inside
   * this span.
   * @type {!endo.Rect}
   */
  this.durationRect = new endo.Rect(0, 0, 0, 0);

  /**
   * The clip rectangle representing the position of the tag text inside
   * this span.
   * @type {!endo.Rect}
   */
  this.tagRect = new endo.Rect(0, 0, 0, 0);

  /**
   * The clip rectangle representing the position of detail text inside this
   * span.
   * @type {!endo.Rect}
   */
  this.detailRect = new endo.Rect(0, 0, 0, 0);

  /**
   * The clip rectangle representing the position of the user-specified rate
   * text in this span.
   * @type {!endo.Rect}
   */
  this.rateRect = new endo.Rect(0, 0, 0, 0);

  /**
   * True if there's an error event in this span.
   * @type {boolean}
   */
  this.hasError = false;

  /**
   * The fill color used for rendering this span.
   * @type {string}
   */
  this.colorFill = '#fff';

  /**
   * The border color used for rendering this span.
   * @type {string}
   */
  this.colorBorder = '#000';
};

/**
 * Enum for span types - these are only used internally and do not need to be
 * kept in sync with the server code.
 *
 * @enum {number}
 */
endo.Span.SpanType = {
    // Invalid spans are invalid.
    INVALID: 0,

    // Task spans are created by JEndoscope.task calls. They denote the root
    // tasks.
    TASK: 1,

    // A span created by an JEndoscope.task call that occured while another
    // JEndoscope.task was still active is assigned type 'subtask'. They are
    // essentially the same as tasks from the user's perspective, but do not
    // appear in the list of top-level spans in a thread.
    SUBTASK: 2,

    // Scope represent a plain block of time and do not have additional data
    // attached. This type of span is not created by JEndoscope.
    SCOPE: 3,

    // Holes represent time in a parent task that are accounted for in the
    // child subtask. Holes are not rendered directly, instead we render the
    // child subtask in the space they would have occupied.
    HOLE: 4
};

/**
 * Returns the detail text to display in the bottom left of this span.
 *
 * @return {string} The detail text, or the empty string if we have nothing to
 *     display.
 */
endo.Span.prototype.getDetails = function() {
  if (this.allTags.length > 0) {
    return 'tag: ' + this.allTags.join(' ');
  } else {
    return '';
  }
};

/**
 * Computes the duration (in seconds) of this span.
 *
 * @return {number} The duration of this span.
 */
endo.Span.prototype.getDuration = function() {
  return this.timeMax - this.timeMin;
};

/**
 * Returns the work rate (in generic 'units per second') performed by this
 * span.
 *
 * @return {number} The work rate for this span.
 */
endo.Span.prototype.getRate = function() {
  return this.units / this.getDuration();
};

/**
 * Gets the time range for this span as a Range object.
 *
 * @return {endo.Range} The time range for this span, as a Range object.
 */
endo.Span.prototype.getTimeRange = function() {
  return new endo.Range(this.timeMin, this.timeMax);
};

/**
 * Spans need to know both their "inclusive" duration (end time minus begin
 * time), and also their "exclusive" duration (inclusive duration, minus the
 * inclusive durations of all their child spans). Calculate this recursively.
 * TODO(aappleby): The inclusive and exclusive times are not currently
 * displayed on the timeline.
 *
 * @return {number} The inclusive time for this span.
 */
endo.Span.prototype.computeDurations = function() {
  this.timeInc = this.timeMax - this.timeMin;
  this.timeEx = this.timeInc;

  for (var i = 0; i < this.spans.length; ++i) {
    this.timeEx -= this.spans[i].computeDurations();
  }

  return this.timeInc;
};

/**
 * Creates random events at random intervals inside this span - used only for
 * offline testing.
 *
 * @param {number} markerId The ID of the marker to use as the source of these
 *     imaginary events.
 */
endo.Span.prototype.createRandomEvents = function(markerId) {
  var eventCount = 10;
  for (var j = 1; j < eventCount; j++) {
    var duration = this.timeMax - this.timeMin;
    var interval = duration / eventCount;
    var time = this.timeMin + interval * j;
    time += (Math.random() * interval - interval / 2);
    var atom_type = endo.Atom.AtomType.EVENT;
    var event = new endo.Event(markerId, atom_type, endo.util.INVALID_TAG,
        time);
    this.events.push(event);
  }
};

/**
 * A raw Endo task is an unstructured list of Endo atoms. Walk through that list
 * and construct the corresponding hierarchical tree of spans that generated it.
 * If the spans were not completed when the snapshot was taken, close them at
 * timeEnd.
 *
 * @param {!endo.Task} rootTask The root task of the span tree
 * @param {Array.<endo.Atom>} atoms The synthesized atoms of the span
 * @param {number} timeEnd The end time, in seconds since server start, of the
 *     parent snapshot.
 * @return {!endo.Span} The span tree generated from this raw task.
 */
endo.Span.loadAtoms = function(rootTask, atoms, timeEnd) {
  var spanStack = [];
  spanStack.back = function() {
    return spanStack[spanStack.length - 1];
  };

  // Create the root span from the data stored in the task.
  var rootSpan = new endo.Span();
  rootSpan.type = endo.Span.SpanType.TASK;
  rootSpan.markerId = rootTask.getMarkerId();
  rootSpan.tag = rootTask.getTag();
  if (rootSpan.tag != endo.util.INVALID_TAG) {
    // do not check duplicity because it's the first
    rootSpan.allTags.push(rootSpan.tag);
  }
  rootSpan.complete = rootTask.isComplete();
  rootSpan.timeMin = rootTask.getTimeMin();
  rootSpan.timeMax = rootTask.getTimeMax();

  spanStack.push(rootSpan);

  // Create child spans for everything in the log.
  for (var i = 0; i < atoms.length; ++i) {
    var atom = atoms[i];

    switch (atom.type) {
      case endo.Atom.AtomType.SCOPE_BEGIN:
        // When we start a span, we create a new span object on the top of the
        // span stack.
        var span = new endo.Span();
        var parent = spanStack.back();

        span.type = endo.Span.SpanType.SCOPE;

        span.markerId = atom.markerId;
        span.complete = false;
        span.timeMin = atom.time;
        span.timeMax = timeEnd;
        span.tag = atom.tag;
        if (atom.tag != endo.util.INVALID_TAG) {
          // do not check duplicity because it's the first
          span.allTags.push(atom.tag);
        }

        parent.spans.push(span);
        spanStack.push(span);
        break;
      case endo.Atom.AtomType.SCOPE_END:
        // When we complete a span, it gets popped off the span stack.
        var top = spanStack.back();

        top.complete = true;
        top.timeMax = atom.time;

        if (endo.Config.createRandomTestEvents) {
          top.createRandomEvents(top.markerId);
        }

        spanStack.pop();
        break;
      case endo.Atom.AtomType.EVENT:
        // Load events and add them to the span on top of the stack.
        var event = endo.Event.loadAtom(atom);
        var span = spanStack.back();
        span.events.push(event);
        if (event.tag != endo.util.INVALID_TAG &&
            span.allTags.indexOf(event.tag) == -1) {
          span.allTags.push(event.tag);
        }
        break;
      case endo.Atom.AtomType.TAG:
        var tag = atom.tag;
        var span = spanStack.back();
        if (tag != endo.util.INVALID_TAG && span.allTags.indexOf(tag) == -1) {
          span.allTags.push(tag);
        }
        break;
    }
  }

  return rootSpan;
};

//java/com/google/net/rpc3/profiling/jendoscope/web/endo_sync.js
// Copyright 2011 Google Inc. All Rights Reserved.

/**
 * @fileoverview All Endoscope time unit conversion code goes here.
 *
 * This file is used to convert an arbitary time unit ('cycle') to Unix seconds.
 * In the context of this file, 'cycle' is considered an arbitary time unit.
 * It may be the same as CPU cycles since boot, but it may also be other units,
 * like nanoseconds since program start.
 *
 * Two 'cycle's along with their corresponding time in Unix seconds needs to be
 * supplied to this file. It is assumed that the relationship between the
 * arbitary time unit and Unix second can be expressed by a linear function.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('endo.sync');

goog.require('endo.util');



/**
 * The wall-clock time, in Unix seconds, when the client started.
 * @type {number}
 */
endo.sync.clientTimeBegin = ((new Date()).getTime() / 1000);

/**
 * The wall-clock time, in Unix seconds, when we last received a sync packet
 * from the server.
 * @type {number}
 */
endo.sync.clientTimeSync = endo.util.INVALID_TIME;


/**
 * The wall-clock time, in Unix seconds, whe the server started.
 * @type {number}
 */
endo.sync.serverTimeBegin = endo.util.INVALID_TIME;


/**
 * The wall-clock time, in Unix seconds, when the server last generated a sync
 * packet.
 * @type {number}
 */
endo.sync.serverTimeSync = endo.util.INVALID_TIME;


/**
 * The cycle time, in cycles since power on, when the server started.
 * @type {number}
 */
endo.sync.serverCycleBegin = endo.util.INVALID_CYCLE;


/**
 * The cycle time, in cycles since power on, when the server last generated a
 * sync packet.
 * @type {number}
 */
endo.sync.serverCycleSync = endo.util.INVALID_CYCLE;


/**
 * The time, in seconds, between when the server generated a sync packet and
 * when the client received it.
 * @type {number}
 */
endo.sync.serverTimeSkew = endo.util.INVALID_TIME;


/**
 * Supply two 'cycle's along with their corresponding Unix time.
 *
 * The relationship between 'cycle' and Unix time is assumed to be linear.
 * With the 2 pairs of input and this assumption, any 'cycle' can
 * subsequently be converted to Unix time.
 *
 * @param {number} cycleBegin
 * @param {number} cycleSync
 * @param {number} timeBegin
 * @param {number} timeSync
 */
endo.sync.update = function(cycleBegin, cycleSync, timeBegin, timeSync) {
  endo.sync.clientTimeSync = endo.sync.unixTime();

  endo.sync.serverCycleBegin = cycleBegin;
  endo.sync.serverCycleSync = cycleSync;
  endo.sync.serverTimeBegin = timeBegin;
  endo.sync.serverTimeSync = timeSync;

  endo.sync.serverTimeSkew = endo.sync.serverTimeSync -
                              endo.sync.clientTimeSync;
};


/**
 * Converts server cycle time to seconds since server start.
 *
 * @param {number} cycle The server cycle to convert.
 * @return {number} The time in seconds since server start.
 */
endo.sync.cycleToTime = function(cycle) {
  var serverTimeDelta = endo.sync.serverTimeSync -
                        endo.sync.serverTimeBegin;

  return endo.util.remap(cycle,
                         endo.sync.serverCycleBegin,
                         endo.sync.serverCycleSync,
                         0,
                         serverTimeDelta);
};


/**
 * Computes the current wall-clock time, in Unix seconds.
 * @return {number} The current Unix time, in seconds.
 */
endo.sync.unixTime = function() {
  return ((new Date()).getTime() / 1000);
};


/**
 * Computes the time in seconds since client start.
 * @return {number} The time in seconds since server start.
 */
endo.sync.clientTime = function() {
  return endo.sync.unixTime() - endo.sync.clientTimeBegin;
};


/**
 * Computes the time in seconds since server start.
 * @return {number} The time in seconds since server start.
 */
endo.sync.serverTime = function() {
  return endo.sync.unixTime() -
         endo.sync.serverTimeBegin +
         endo.sync.serverTimeSkew;
};

//java/com/google/net/rpc3/profiling/jendoscope/web/endo_task.js
// Copyright 2011 Google Inc. All Rights Reserved.

/**
 * @fileoverview Endoscope's task class is responsible for holding the tree of
 * Spans generated from the raw task logs and for allowing quick traversal
 * along a row of the timeline for rendering. There's also a bit of an
 * envelope-letter pattern going on here, as a Task can represent an incomplete
 * JEndoscope call on the server - once the server completes the task, the
 * contents of this Task get swapped out via update() without breaking the
 * prev/next links used for traversal and rendering.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('endo.Task');



/**
 * Constructs an empty task.
 *
 * @constructor
 * @struct
 */
endo.Task = function() {
  /**
   * The global ID of this task.
   * @private {number}
   */
  this.taskId_ = endo.util.INVALID_ID;

  /**
   * The global ID of this task's parent task, or 0 if there isn't one.
   * @private {number}
   */
  this.parentId_ = endo.util.INVALID_ID;

  /**
   * The global ID of the marker that generated this task.
   * @private {number}
   */
  this.markerId_ = endo.util.INVALID_ID;

  /**
   * The global ID of the thread that generated this task.
   * @private {number}
   */
  this.threadId_ = endo.util.INVALID_ID;

  /**
   * The user-provided tag for this task.
   * @private {string}
   */
  this.tag_ = endo.util.INVALID_TAG;

  /**
   * The time, in seconds since server start, when this task began.
   * @private {number}
   */
  this.timeMin_ = 0;

  /**
   * The time, in seconds since server start, when this task ended.
   * @private {number}
   */
  this.timeMax_ = 0;

  /**
   * True if this task has completed on the server.
   * @private {boolean}
   */
  this.complete_ = false;

  /**
   * The root of the span tree associated with this task.
   * @private {endo.Span}
   */
  this.rootSpan_ = null;

  /**
   * True if this task is visible on the timeline.
   * @private {boolean}
   */
  this.visible_ = true;

  /**
   * True if this task is represented by a proxy rectangle on the timeline.
   * @private {boolean}
   */
  this.proxied_ = false;

  /**
   * The proxy object attached to this task.
   * @private {endo.Proxy}
   */
  this.proxy_ = null;

  /**
   * The task following this one in the task's thread.
   * @private {endo.Task}
   */
  this.next_ = null;

  /**
   * The task preceding this one in the task's thread.
   * @private {endo.Task}
   */
  this.prev_ = null;

  /**
   * The nesting level of this task.
   * @private {number}
   */
  this.depth_ = 0;
};

/**
 * @return {number}
 */
endo.Task.prototype.getTaskId = function() {
  return this.taskId_;
};

/**
 * @return {number}
 */
endo.Task.prototype.getParentId = function() {
  return this.parentId_;
};

/**
 * @return {number}
 */
endo.Task.prototype.getMarkerId = function() {
  return this.markerId_;
};

/**
 * @return {string}
 */
endo.Task.prototype.getTag = function() {
  return this.tag_;
};

/**
 * @return {number}
 */
endo.Task.prototype.getThreadId = function() {
  return this.threadId_;
};

/**
 * @return {number}
 */
endo.Task.prototype.getTimeMin = function() {
  return this.timeMin_;
};

/**
 * @return {number}
 */
endo.Task.prototype.getTimeMax = function() {
  return this.timeMax_;
};

/**
 * @return {boolean} True if this task has completed on the server.
 */
endo.Task.prototype.isComplete = function() {
  return this.complete_;
};

/**
 * @param {boolean} complete True if this task has completed on the server.
 */
endo.Task.prototype.setComplete = function(complete) {
  this.complete_ = complete;
};

/**
 * @return {endo.Span}
 */
endo.Task.prototype.getRootSpan = function() {
  return this.rootSpan_;
};

/**
 * @param {endo.Span} rootSpan
 */
endo.Task.prototype.setRootSpan = function(rootSpan) {
  this.rootSpan_ = rootSpan;
};

/**
 * @return {boolean} True if this task is visible on the timeline.
 */
endo.Task.prototype.isVisible = function() {
  return this.visible_;
};

/**
 * @param {boolean} visible True if this task should be visible on the
 *     timeline.
 */
endo.Task.prototype.setVisible = function(visible) {
  this.visible_ = visible;
};

/**
 * @return {boolean} True if this task is represented by a proxy rectangle.
 */
endo.Task.prototype.isProxied = function() {
  return this.proxied_;
};

/**
 * @param {boolean} proxied True if this task should be represented by a proxy
 *     rectangle on the timeline.
 */
endo.Task.prototype.setProxied = function(proxied) {
  this.proxied_ = proxied;
};

/**
 * @return {endo.Proxy} This task's proxy.
 */
endo.Task.prototype.getProxy = function() {
  return this.proxy_;
};

/**
 * @param {endo.Proxy} proxy The new proxy.
 */
endo.Task.prototype.setProxy = function(proxy) {
  this.proxy_ = proxy;
};

/**
 * @return {number} The nesting depth of this task - this will be 0 for top-
 * level tasks and will increase by one for each level of nesting.
 */
endo.Task.prototype.getDepth = function() {
  return this.depth_;
};

/**
 * @param {number} depth The new nesting depth of this task.
 */
endo.Task.prototype.setDepth = function(depth) {
  this.depth_ = depth;
};

/**
 * Updates this task with fresh data from a newer snapshot.
 * @param {endo.Task} task The fresh task.
 */
endo.Task.prototype.update = function(task) {
  this.taskId_ = task.taskId_;
  this.parentId_ = task.parentId_;
  this.markerId_ = task.markerId_;
  this.tag_ = task.tag_;
  this.complete_ = task.complete_;
  this.timeMin_ = task.timeMin_;
  this.timeMax_ = task.timeMax_;
  this.rootSpan_ = task.rootSpan_;
};

/**
 * Retrieves the next task in this task's doubly-linked list.
 * @return {endo.Task} The next task in the list.
 */
endo.Task.prototype.getNext = function() {
  return this.next_;
};

/**
 * Splice 'newTask' into this task list, immediately after this node.
 * @param {!endo.Task} newTask The task to splice.
 */
endo.Task.prototype.spliceNext = function(newTask) {
  var oldNext = this.next_;

  if (newTask.next_) newTask.next_.prev_ = newTask.prev_;
  if (newTask.prev_) newTask.prev_.next_ = newTask.next_;

  this.next_ = newTask;
  newTask.prev_ = this;
  newTask.next_ = oldNext;
  if (oldNext) oldNext.prev_ = newTask;
};

/**
 * Retrieves the previous task in this task's doubly-linked list.
 * @return {endo.Task} The previous task in the list.
 */
endo.Task.prototype.getPrev = function() {
  return this.prev_;
};

/**
 * Splice 'newTask' into this task list, immediately before this node.
 * @param {!endo.Task} newTask The task to splice.
 */
endo.Task.prototype.splicePrev = function(newTask) {
  var oldPrev = this.prev_;

  if (newTask.next_) newTask.next_.prev_ = newTask.prev_;
  if (newTask.prev_) newTask.prev_.next_ = newTask.next_;

  if (oldPrev) oldPrev.next_ = newTask;
  newTask.prev_ = oldPrev;
  newTask.next_ = this;
  this.prev_ = newTask;
};

/**
 * Gets a Range object representing the time range of this task.
 *
 * @return {!endo.Range} The time range of this task.
 */
endo.Task.prototype.getTimeRange = function() {
  return new endo.Range(this.timeMin_, this.timeMax_);
};

/**
 * Gets the duration (in seconds) of this task.
 *
 * @return {number} The duration of this task.
 */
endo.Task.prototype.getDuration = function() {
  return this.timeMax_ - this.timeMin_;
};

/**
 * Converts a raw task into an Endo task. If the task was not complete when the
 * snapshot was taken, the task is clipped to the snapshot end time.
 *
 * @param {!endo.proto.binary.ConstTask} rawTask The raw task to convert.
 * @param {number} timeEnd The end time, in seconds since server start, of the
 *     parent snapshot.
 * @return {!endo.Task} The converted task.
 */
endo.Task.loadRawTask = function(rawTask, timeEnd) {
  var complete = (rawTask.getEndTimeNanos() > 0);
  var rootBeginNano = rawTask.getStartTimeNanos();
  var rootBegin = endo.sync.cycleToTime(rootBeginNano);
  var rootEnd = complete ?
                endo.sync.cycleToTime(rawTask.getEndTimeNanos()) :
                timeEnd;

  var task = new endo.Task();

  task.taskId_ = rootBeginNano; // use start time in nanos as task id
  task.threadId_ = rawTask.getThreadId();
  task.parentId_ = endo.util.INVALID_ID;
  task.markerId_ = rawTask.getMarkerId();
  task.tag_ = rawTask.getTag();
  task.complete_ = complete;
  task.timeMin_ = rootBegin;
  task.timeMax_ = rootEnd;
  // task.rootSpan_ is set by endo_snapshot later

  return task;
};


//java/com/google/net/rpc3/profiling/jendoscope/web/endo_testdata.js
// Copyright 2011 Google Inc. All Rights Reserved.

/**
 * @fileoverview This file contains an Endoscope snapshot that has been base64
 *               encoded. It is used by the protocol buffer decoder test suite
 *               and during development to exercise the UI rendering.
 *
 * To regenerate, run JEndoscope example locally and copy the output of
 *
 * wget -qO- localhost:20001/jendoz/proto | base64 |
 *   sed "s/$/\' +/;s/^/'/"
 *
 * into this file.
 */

goog.provide('endo.testData');



/**
 * This is a binary JEndoPB.Snapshot that has been Base64 encoded.
 */
endo.testData =
'CnIIChABGgV0YXNrMiIVSkVuZG9Vc2VyRXhhbXBsZS5qYXZhKHoySmNvbS9nb29nbGUvbmV0L3Jw' +
'YzMvcHJvZmlsaW5nL2plbmRvc2NvcGUvZXhhbXBsZS9KRW5kb1VzZXJFeGFtcGxlJFdvcmsucnVu' +
'OAAKcwgMEAEaBXRhc2szIhVKRW5kb1VzZXJFeGFtcGxlLmphdmEoggEySmNvbS9nb29nbGUvbmV0' +
'L3JwYzMvcHJvZmlsaW5nL2plbmRvc2NvcGUvZXhhbXBsZS9KRW5kb1VzZXJFeGFtcGxlJFdvcmsu' +
'cnVuOAAKcAgEEAIaA2V2MSIVSkVuZG9Vc2VyRXhhbXBsZS5qYXZhKGYySmNvbS9nb29nbGUvbmV0' +
'L3JwYzMvcHJvZmlsaW5nL2plbmRvc2NvcGUvZXhhbXBsZS9KRW5kb1VzZXJFeGFtcGxlJFdvcmsu' +
'cnVuOAAKcggIEAEaBXRhc2sxIhVKRW5kb1VzZXJFeGFtcGxlLmphdmEodjJKY29tL2dvb2dsZS9u' +
'ZXQvcnBjMy9wcm9maWxpbmcvamVuZG9zY29wZS9leGFtcGxlL0pFbmRvVXNlckV4YW1wbGUkV29y' +
'ay5ydW44AApwCAYQAhoDZXYyIhVKRW5kb1VzZXJFeGFtcGxlLmphdmEobDJKY29tL2dvb2dsZS9u' +
'ZXQvcnBjMy9wcm9maWxpbmcvamVuZG9zY29wZS9leGFtcGxlL0pFbmRvVXNlckV4YW1wbGUkV29y' +
'ay5ydW44AApxCAIQARoEdGFzayIVSkVuZG9Vc2VyRXhhbXBsZS5qYXZhKFwySmNvbS9nb29nbGUv' +
'bmV0L3JwYzMvcHJvZmlsaW5nL2plbmRvc2NvcGUvZXhhbXBsZS9KRW5kb1VzZXJFeGFtcGxlJFdv' +
'cmsucnVuOAASEggCECYYvP6p6wIg5vKNpQooBBISCAIQKBjqhJvRBSDg4P2KDSgEEhIIAhAuGNjf' +
'ibcIILjQ7PAPKAoSEggCECYYmNfqnAsgpvfK1hIoBBISCAoQJBjqi+X2ASCMv8qwCSgCEhIIDBAk' +
'GPS9j58LIOzQ8dgSKAISEQgIECQYqKijCCDSwbHHFCgCEhEIAhAkGKa9swUgxN6zxxQoAhISCAIQ' +
'LhjspqboECDW4oaiGCgSEhIIAhAmGI6OhM4TIPSs5YcbKBASEggCECQYgI7isxYghLTE7R0oEhIS' +
'CAIQLhjChb6ZGSDuqZ/TICgQEhIIAhAmGIrUm/8bIO6u/LgjKAwSEggEECgYnvv8nRYgnvv8nRYo' +
'BhISCAYQKBiqzuHXHSCqzuHXHSgGEhIIAhAoGIKOx4IOIOrHwpElKAASEggCECQY8rz75B4g7NPf' +
'niYoCBISCAIQLhiKpdjKISCGg7mEKSgEEhIIAhAmGMzat7AkIJiLmOorKAoSEggCECgYvIuRlicg' +
'8rfxzy4oEBISCAIQLhi26M7hLCCOu6+bNCgQEhIIAhAmGOyIqccvIL7ViYE3KBISEggCEC4Y7Jjj' +
'kjUghJfCzDwoCBISCAoQJBjGsa3qKyCAq46kMygCEhIIDBAkGJzez5I1IPSUtMw8KAISEggIECQY' +
'nPPu+ykgzIj1uj4oAhISCAIQJBiO6e37KSCaive6PigCEhIIAhAuGLyrxcw8IO7spYZEKAoSEggC' +
'ECQYvob6uj4gisHZ9EUoChISCAoQKBjY2cabNCDqiqfVOygCEhIIDBAoGPrx58M9IPiNyf1EKAIS' +
'EggIECgYiMqJrTIgpPeJ7EYoAhISCAIQKBjow4itMiDc94vsRigCEhIIAhAuGP7VqoZEINqiisBL' +
'KAwSEggKECYYqrn/5jkgjJThoEEoAhISCAwQJhjcsaKPQyCm+YPJSigCEhIICBAmGMC5wPg3IOD6' +
'xLdMKAISEggCECYY/La/+Dcg5vTGt0woAhISCAIQJBjc6970RSComL6uTSgMEhIIAhAoGMTzjuxG' +
'IIaJ76VOKA4SEggCEC4Y/JaPwEsgpu3v+VIoBhISCAIQJhi86sm3TCDe+qnxUygKEhIIAhAkGK7z' +
'm6ZOIJ7h+99VKBASEggCECgYkJf+i1EgwJPfxVgoDhISCAIQJBiqy5a9WSDesPf2YCgKEhIIAhAo' +
'GPqQ8qJcIM6l09xjKAQSEggKEC4Y7N+U4FUgtrT1mV0oAhISCAwQLhiajLeIXyDQwZnCZigCEhII' +
'CBAuGJTX1vFTIIb92rBoKAISEggCEC4YxpfW8VMgnI3dsGgoAhISCAIQJBiuqvz2YCD+7t2waCgI' +
'EhIIAhAoGKzb29xjIPCevJZrKBASEggEECYY7I+ckV4g7I+ckV4oBBISCAYQJhiWlYDLZSCWlYDL' +
'ZSgKEhIIAhAmGNzvuNdWIMze4YRtKAASEggCECQYkKjisGgg5OXD6m8oCBISCAIQKBjg+8CWayCS' +
'7aDQcigKEhIIAhAmGLLSooVtIOjIg790KAYSEggCECQY+sL86m8grqncpHcoChISCAIQKBjs/dTQ' +
'ciC64rWKeigQEhIIChAuGIDSo59qINCdh9lxKAISEggMEC4YpPXIx3MgopCrgXsoAhISCAgQLhjy' +
'qOGwaCCQmezvfCgCEhIIAhAuGK7d4LBoIL6Z7u98KAISEggCECYYzry3tnUgvLWW8HwoCBISCAIQ' +
'JBj875WceCC03vbVfygMEhMIAhAoGNDT8IF7IJK21buCASgKEhMIAhAuGMiy0Od9IMTKsaGFASgO' +
'EhQIAhAmGLCRqc2AASCunYmHiAEoChIUCAIQKBjmpeWYhgEgjqLG0o0BKAwSFAgCEC4YoMXD/ogB' +
'ILCqpLiQASgEEhQIAhAmGMbMoeSLASCAm4KekwEoBhIUCAIQLhjYltyvkQEgwLO96ZgBKA4SFAgE' +
'ECQY+Nrq7IoBIPja6uyKASgOEhQIBhAkGPC3zaaSASDwt82mkgEoEhIUCAIQJBje34izgwEg+Pyt' +
'4JkBKAASFAgCEC4Y1JXC6ZgBIJTToqOgASgEEhQIAhAkGLz88eCZASCw1NCaoQEoBBIUCAoQKBiE' +
'3r24kAEgkt+e8pcBKAISFAgMECgY1oLf4JkBIKTCwJqhASgCEhQICBAoGLSsgMqOASDG5PyIowEo' +
'AhIUCAIQKBjGm//JjgEg1q7+iKMBKAISFAgCEC4Ylseno6ABIO6biN2nASgOEhQIAhAkGPKL1Jqh' +
'ASCqyLTUqAEoDhIUCAQQJhic3pzPmwEgnN6cz5sBKA4SFAgGECYYrpf/iKMBIK6X/4ijASgKEhQI' +
'AhAmGNiiupWUASCau9/CqgEoABIUCAIQKBjal4GJowEgko7iwqoBKBASFAgCEC4Y9omN3acBIKj3' +
'75avASgOEhQIAhAkGOSoudSoASCI5pmOsAEoCBIUCAIQJhji9ZbDqgEgkIf3/LEBKBASFAgCECgY' +
'pK/6qK0BIKyz3eK0ASgKEhQIAhAuGMLs1I6wASC297TItwEoDBIUCAIQJBjqg7b0sgEgzrmZrroB' +
'KAYSFAgCECYY5tuQ2rUBIJjs8ZO9ASgOEhQIAhAuGJjRzaW7ASCWrq7fwgEoDhIUCAIQJBjsoa2L' +
'vgEgtvSUxcUBKA4SFAgCEC4YltTm1sMBIJ65xpDLASgSEhQIChAoGLipr666ASCCiJDowQEoAhIU' +
'CAwQKBjm0dHWwwEgsvS1kMsBKAISFAgIECgYqvjxv7gBIMy+9/7MASgCEhQIAhAoGLrm8L+4ASDg' +
'x/n+zAEoAhIUCAIQJBjkmcu8xgEgguir9s0BKAQSFAgCEC4Y5tTKkMsBILDhq8rSASgQEhQIAhAo' +
'GOrK/P7MASDwh9241AEoDhIUCAoQJhiQ3sbfwgEgloKnmcoBKAISFAgMECYY5q7oh8wBIICSysHT' +
'ASgCEhQICBAmGN7AifHAASDq2o2w1QEoAhIUCAIQJhjYqYjxwAEgiuiPsNUBKAISFAgCECQYgvLg' +
'7c4BIJCfwafWASgKEhQIAhAoGOCslrnUASCAtPjy2wEoEBIUCAIQJBigh8uE2gEgqK6qvuEBKAoS' +
'FAgCECgYwqGu6twBIMa2jqTkASgMEhQIBBAuGIrEk4TaASCKxJOE2gEoEhIUCAYQLhjqj/e94QEg' +
'6o/3veEBKAASFAgCEC4YrJuxytIBIPiw2PfoASgAEhQIAhAoGNqbk6TkASD0yvXd6wEoEBIUCAQQ' +
'Jhjqv+LY3gEg6r/i2N4BKAISFAgGECYY0JHFkuYBINCRxZLmASgQEhQIAhAmGOi8gJ/XASD0zaXM' +
'7QEoABIUCAIQLhiy/N336AEg6rS9sfABKBISFAgCECgYuMH63esBILjn2pfzASgEEhQIAhAuGOic' +
'wrHwASCm0qLr9wEoDBIUCAQQJBjqmo746AEg6pqO+OgBKAYSFAgGECQYtJTusfABILSU7rHwASgI' +
'EhQIAhAkGMzMrb7hASDSnM3r9wEoABIUCAIQKBiKw9+X8wEgkOu80foBKA4SFAgCEC4Yysyn6/cB' +
'ILrOiqX/ASgKEhQIAhAkGKyFz+v3ASDesq2l/wEoEBIUCAIQKBjC1sHR+gEg8Pqhi4ICKAQSFAgE' +
'ECYYnrOOhvUBIJ6zjob1ASgIEhQIBhAmGLDB8L/8ASCwwfC//AEoEhIUCAIQJhiuo6rM7QEgyqjQ' +
'+YMCKAASFAgCEC4Y2taPpf8BIMb/796GAigSEhQIAhAkGJbhr6X/ASDgxI7fhgIoBBIUCAIQKBji' +
'5KaLggIgioeHxYkCKAQSFAgCECYY1ta0+oMCIO6dlbSLAigKEhQIAhAuGMD7j+CGAiCUz++ZjgIo' +
'ChIUCAIQKBjilsmrjAIghJKs5ZMCKBASFAgCECYY3M2lkY8CIPy5hsuWAigKEhQIAhAuGK7CgfeR' +
'AiD8gOKwmQIoBBIUCAIQKBj02OLclAIg5LjClpwCKA4SFAgKECQY7vGntIsCIK7EiO6SAigCEhQI' +
'DBAkGOzTydyUAiDU26+WnAIoAhIUCAgQJBiurenFiQIgtr3yhJ4CKAISFAgCECQYzsvoxYkCIIa0' +
'9ISeAigCEhQIAhAmGISMwMKXAiCchqH8ngIoBhIUCAIQLhj+7KComgIgitqF4qECKAwSFAgCECgY' +
'2Kv/jZ0CIKjG4MekAigKEhQIAhAkGPKx3fOfAiD+0r2tpwIoChIUCAIQKBiW7vmkqAIg5uDa3q8C' +
'KAQSFAgCECQYwO3XiqsCIPLRuMSyAigEEhQIChAmGKz7+cekAiC8zNqBrAIoAhIUCAwQJhiShpzw' +
'rQIguIj+qbUCKAISFAgIECYYnou52aICIJ6ov5i3AigCEhQIAhAmGIqCuNmiAiCQncGYtwIoAhIU' +
'CAQQLhi8l/74rAIgvJf++KwCKAASFAgGEC4Y8pngsrQCIPKZ4LK0AigMEhQIAhAuGMT2mr+lAiCG' +
'5r/suwIoABIUCAIQJhjOo8SYtwIg8KCl0r4CKA4SFAgKECgY0q+hzbECIIaMg4e5AigCEhQIDBAo' +
'GPDhxvW6AiCG5aivwgIoAhIUCAgQKBiY9d/erwIgipHsncQCKAISFAgCECgYiL7f3q8CIJiO7p3E' +
'AigCEhQIAhAmGJ6Eq9K+AiCUxo2MxgIoCBIUCAoQJBj+sfyytAIgysfb7LsCKAISFAgMECQY1Iee' +
'270CILyq/5TFAigCEhQICBAkGKj3vcSyAiDav8CDxwIoAhIUCAIQJBjGuL3EsgIgtrfCg8cCKAIS' +
'FAgCECgYlKLxncQCIITW09fLAigKEhQIAhAmGMKrkozGAiDM8fHFzQIoEBIUCAIQJBiiwcWDxwIg' +
'qvWkvc4CKBISFAgEEC4YnLCmpsMCIJywpqbDAigQEhQIBhAuGL69iODKAiC+vYjgygIoChIUCAIQ' +
'Lhj458TsuwIgkP/omdICKAASFAgCECgYouTY18sCIKKnu5HTAigOEhQIAhAmGJbg9sXNAiDMqdn/' +
'1AIoEBIUCAIQLhj++e2Z0gIg5JrO09kCKAwSFAgCECYYjJPe/9QCIO7cvrncAigOEhQIAhAuGOSn' +
'09PZAiCG2LWN4QIoEhIUCAIQJhjg+MO53AIglrCj8+MCKAQSFAgEECQYmuCM99UCIJrgjPfVAigE' +
'EhQIBhAkGOj677DdAiDo+u+w3QIoCBIUCAIQJBi6hqq9zgIg6PjP6uQCKAASFAgKECgYvID+/9QC' +
'IOie3rncAigCEhQIDBAoGJSRoKjeAiDWq4Hi5QIoAhIUCAgQKBjm0MCR0wIgvL/C0OcCKAISFAgC' +
'ECgYzpfAkdMCIMi8xNDnAigCEhQIAhAuGK7Vuo3hAiCsmZ3H6AIoDhIUCAIQJhioqKjz4wIgpqqI' +
'resCKA4SFAgCECQY9v/V6uQCILyltqTsAigKEhQIAhAoGNrSx9DnAiCokqeK7wIoEhIUCAIQLhje' +
'/qHH6AIg2KyEgfACKAwSFAgCECYYtoiNresCIITO7ObyAigIEhQIAhAkGKbzu6TsAiD+xJ7e8wIo' +
'ChIUCAIQKBiij6yK7wIgwLGQxPYCKAoSFAgCEC4YrpTK+fACILrvqbP4AigQEhQIAhAmGJKnpd/z' +
'AiDi84WZ+wIoEhIUCAIQJBjgnv/E9gIg1KLf/v0CKAoSFAgCECgYqtvdqvkCIOS9vuSAAygQEhQI' +
'AhAuGMKht5D8AiDSspfKgwMoDhIUCAIQJBicrPLbgQMg5NHSlYkDKBASFAgCECQYyPGNjYoDIP6G' +
'7saRAygSEhQIBBAmGM6k+q+GAyDOpPqvhgMoCBIUCAYQJhju/NzpjQMg7vzc6Y0DKAwSFAgCECYY' +
'krOX9v4CIMy6vaOVAygAEhQIChAoGIqbk7CGAyCG6fLpjQMoAhIUCAwQKBjisbLYjwMg4KSVkpcD' +
'KAISFAgIECgYqODUwYQDILqV1YCZAygCEhQIAhAoGIbQ08GEAyCYpNeAmQMoAhIUCAIQJhjsqMKj' +
'lQMg4Mmi3ZwDKAQSFAgEEC4Y3vyR4Y4DIN78keGOAygIEhQIBhAuGPyK9ZqWAyD8ivWalgMoBBIU' +
'CAIQLhio/K2nhwMgmtTV1J0DKAASFAgCECgYsNjagJkDINqEvbqgAygGEhQIChAkGJaas7WTAyDc' +
'l5XvmgMoAhIUCAwQJBi8xdTdnAMghJK1l6QDKAISFAgIECQYvsrzxpEDIJrP9IWmAygCEhQIAhAk' +
'GN6O88aRAyC2nfWFpgMoAhIUCAIQKBjM7MG6oAMg1LGg9KcDKAYSFAgCECQY6tT3haYDINim17+t' +
'AygGEhQIAhAoGNT7o/SnAyCmn4OurwMoDhIUCAoQJhiG9OjLngMgrqvMhaYDKAISFAgMECYY9omM' +
'9KcDIJLf7a2vAygCEhQICBAmGMrsp92cAyDYqa6csQMoAhIUCAIQJhjytafdnAMgip+wnLEDKAIS' +
'FAgEEC4Y/vy9jqUDIP78vY6lAygIEhQIBhAuGIrJoMisAyCKyaDIrAMoAhIUCAIQLhiMttvUnQMg' +
'tM6AgrQDKAASFAgCECQY0KHcv60DIJzHvPm0AygKEhQIAhAoGMjqhq6vAyDA7ubntgMoDBIUCAIQ' +
'LhiOtIWCtAMg5Kzlu7sDKAYSFAgCECQYusHB+bQDIN7voLO8AygKEhQIAhAuGJym6ru7AyDS9cz1' +
'wgMoBBIUCAIQJBiKxaSzvAMgtuqD7cMDKA4SFAgKECYYwoTzirMDINr61MS6AygCEhQIDBAmGLq9' +
'lrO8AyD40ffswwMoAhIUCAgQJhjg5rOcsQMgrKS428UDKAISFAgCECYYiq6znLEDIMCdutvFAygC' +
'EhQIAhAuGJDv0fXCAyD8hLKvygMoBhIUCAIQJBjQpIftwwMg6oDnpssDKAoSFAgKECgYxsar1rgD' +
'ILC0jZDAAygCEhQIDBAoGOKUzv7BAyD+kbC4yQMoAhIUCAgQKBjYnOzntgMgqpjwpssDKAISFAgC' +
'ECgYuuDr57YDILb18KbLAygCEhQIAhAmGJagvdvFAyCUv52VzQMoBBIUCAIQLhiQgLevygMg7saW' +
'6dEDKAgSFAgCECQYpLDspssDIIaJz+DSAygGEhQIAhAoGLzD8qbLAyCMxdHg0gMoBBIUCAIQJhiw' +
'naKVzQMg5siCz9QDKAwSFAgCEC4YvN6b6dEDINad/qLZAygMEhQIAhAkGJbn0+DSAyCArbOa2gMo' +
'EBIUCAIQKBjy7dzh0gMgss27m9oDKBASFAgCECYYkpi8x9UDIKr6nIHdAygGEhQIAhAuGOKlg6PZ' +
'AyCq0OPc4AMoEBIUCAIQJhjUuLDe4AMg8reTmOgDKAwSFAgCEC4Y2M2OxOMDINbl7v3qAygMEg8I' +
'ChAmGJLe1YbqAyAAKAISDwgIECYYjNaYmOgDIAAoAhIPCAIQJhjqnZiY6AMgACgCEhQIBBAkGOSe' +
'28ziAyDkntvM4gMoDBIUCAYQJBjqgr2G6gMg6oK9huoDKAwSDwgCECQY3KD4ktsDIAAoABIUCAoQ' +
'KBiswZPn3wMggP30oOcDKAISDwgMECgYwPy1j+kDIAAoAhIPCAgQKBjoiNT43QMgACgCEg8IAhAo' +
'GOaH0/jdAyAAKAISDwgIEC4YzP3z/eoDIAAoAhIPCAIQLhjOwvP96gMgACgCGhMIKBIPcG9vbC0y' +
'LXRocmVhZC0zGhMILhIPcG9vbC0yLXRocmVhZC00GhMIJBIPcG9vbC0yLXRocmVhZC0xGhMIJhIP' +
'cG9vbC0yLXRocmVhZC0yII6y/tv1USik68Cb7AM=' +
'';

//java/com/google/net/rpc3/profiling/jendoscope/web/endo_thread.js
/**
 * @fileoverview Endo's thread class stores all tasks and subtasks that occured
 * in a particular thread. Note that top-level tasks in a thread are tracked in
 * two separate data structures - a dictionary for random lookup via task ID
 * and a doubly-linked list for efficient traversal during rendering.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('endo.Thread');

goog.require('endo.Range');
goog.require('endo.Rect');



/**
 * Creates a new, empty Endo thread.
 * @param {string} name The name of the thread.
 * @param {number} id The unique ID of the thread.
 * @constructor
 * @struct
 */
endo.Thread = function(name, id) {
  /**
   * The user-specified name of this thread.
   * @private {string}
   */
  this.name_ = name;

  /**
   * The global ID of this thread.
   * @private {number}
   */
  this.threadId_ = id;

  /**
   * The first task that occurred in this thread.
   * @private {endo.Task}
   */
  this.taskHead_ = null;

  /**
   * The last task that occurred in this thread.
   * @private {endo.Task}
   */
  this.taskTail_ = null;

  /**
   * The first task in this thread that should be visible on screen - this may
   * not be the first in the thread if the user has zoomed in deeply. Tracking
   * the first visible task saves us a lot of redundant list traversal work
   * each frame.
   * @private {endo.Task}
   */
  this.firstVisible_ = null;

  /**
   * Maps from global task ID to the tasks that occurred in this thread.
   * @private {!Object.<number, !endo.Task>}
   */
  this.taskMap_ = {};

  /**
   * The list of tasks in the above map.
   * @private {!Array.<!endo.Task>}
   */
  this.taskList_ = [];

  /**
   * The horizontal 'row' of the timeline that this thread will be drawn in.
   * @private {number}
   */
  this.rowIndex_ = -1;

  /**
   * The screen-space clip rectangle used for rendering this thread.
   * @private {!endo.Rect}
   */
  this.screenRect_ = new endo.Rect(0, 0, 0, 0);
};

/**
 * @return {string}
 */
endo.Thread.prototype.getName = function() {
  return this.name_;
};

/**
 * @return {!Array.<!endo.Task>}
 */
endo.Thread.prototype.getTaskList = function() {
  return this.taskList_;
};

/**
 * @return {number}
 */
endo.Thread.prototype.getRowIndex = function() {
  return this.rowIndex_;
};

/**
 * @param {number} rowIndex
 */
endo.Thread.prototype.setRowIndex = function(rowIndex) {
  this.rowIndex_ = rowIndex;
};

/**
 * @return {!endo.Rect}
 */
endo.Thread.prototype.getScreenRect = function() {
  return this.screenRect_;
};

/**
 * @param {number} x
 * @param {number} y
 * @param {number} width
 * @param {number} height
 */
endo.Thread.prototype.setScreenRect = function(x, y, width, height) {
  this.screenRect_.set(x, y, width, height);
};

/**
 * Updates this thread with fresh data from a newer snapshot.
 * @param {endo.Thread} thread The fresh thread.
 */
endo.Thread.prototype.update = function(thread) {
};

/**
 * @return {endo.Task} The head of the list of tasks.
 */
endo.Thread.prototype.getTaskHead = function() {
  return this.taskHead_;
};

/**
 * @return {endo.Task} The tail of the list of tasks.
 */
endo.Thread.prototype.getTaskTail = function() {
  return this.taskTail_;
};

/**
 * @return {endo.Task} The first visible task in this thread.
 */
endo.Thread.prototype.getFirstVisibleTask = function() {
  return this.firstVisible_;
};

/**
 * @param {endo.Task} task
 */
endo.Thread.prototype.setFirstVisibleTask = function(task) {
  this.firstVisible_ = task;
};

/**
 * Retrieves a single task by ID.
 * @param {number} taskId The ID of the task to retrieve.
 * @return {endo.Task} The task requested.
 */
endo.Thread.prototype.getTask = function(taskId) {
  return this.taskMap_[taskId];
};

/**
 * Adds or replaces a task in a thread with a fresh task from a more recent
 * snapshot.
 * @param {number} taskId The ID of the task to fetch.
 * @param {!endo.Task} newTask The fresh task.
 */
endo.Thread.prototype.setTask = function(taskId, newTask) {
  // If we already have a task with that ID, replace its contents with that
  // of the new task.
  if (this.taskMap_[taskId]) {
    this.taskMap_[taskId].update(newTask);
    return;
  }

  // Otherwise we're adding a new task, which requires some bookkeeping.
  // First, add it to the task map & task list.
  this.taskMap_[taskId] = newTask;
  this.taskList_.push(newTask);

  // If it's a subtask, it doesn't go on the doubly-linked list.
  if (newTask.getParentId() != endo.util.INVALID_ID) return;

  // Otherwise if this is the first task, just add it.
  if (!this.taskHead_) {
    this.taskHead_ = newTask;
    this.taskTail_ = newTask;
    return;
  }

  // If the task has a higher ID than the tail, it becomes the new tail.
  if (newTask.getTaskId() > this.taskTail_.getTaskId()) {
    this.taskTail_.spliceNext(newTask);
    this.taskTail_ = newTask;
    return;
  }

  // If the task has a lower ID than the head, it becomes the new head.
  if (newTask.getTaskId() < this.taskHead_.getTaskId()) {
    this.taskHead_.splicePrev(newTask);
    this.taskHead_ = newTask;
    return;
  }

  // We need to scan for the insertion point - since we're likely to be adding
  // data near the end of our log, we start the scan at the tail and work
  // backwards.

  // (Note - We generally won't need to do this, as the snapshots and updates
  // currently always keep tasks in sorted order per thread. I can't guarantee
  // that future updates won't break that, though...)

  var cursor = this.taskTail_;

  while (cursor) {
    if (cursor.getTaskId() < newTask.getTaskId()) {
      // Splicing task into list.
      cursor.spliceNext(newTask);
      return;
    }
    cursor = cursor.getPrev();
  }

  throw 'endo.Thread.prototype.setTask - Failed to find a valid insertion ' +
        'point for a new task.';
};

/**
 * Computes the enclosing time range for all tasks and subtasks in this thread.
 * @return {!endo.Range} The enclosing time range.
 */
endo.Thread.prototype.getTimeRange = function() {
  if (this.taskHead_ && this.taskTail_) {
    return new endo.Range(this.taskHead_.getTimeMin(),
                          this.taskTail_.getTimeMax());
  } else {
    return endo.Range.EmptyRange();
  }
};


/**
 * Updates the nesting depths for all tasks in this thread.
 */
endo.Thread.prototype.updateDepths = function() {
  var task = this.getTaskHead();
  while (task) {
    task.setDepth(0);
    this.updateSpanDepths(task.getRootSpan(), 0);
    task = task.getNext();
  }
};


/**
 * Recursively computes the nesting depth for the given span and all spans
 * beneath it. The renderer uses the depth to determine the vertical
 * positioning of the span on the timeline, but since it only needs to be
 * computed once we do it here instead of in the renderer.
 * @param {endo.Span} span The span to update.
 * @param {number} depth The nesting depth for this span.
 */
endo.Thread.prototype.updateSpanDepths = function(span, depth) {
  span.depth = depth;

  if (span.type == endo.Span.SpanType.HOLE) {
    var subtask = this.getTask(span.subtaskId);
    if (subtask) {
      subtask.setDepth(depth);
      this.updateSpanDepths(subtask.getRootSpan(), depth);
    }
  }

  for (var i = 0; i < span.spans.length; i++) {
    this.updateSpanDepths(span.spans[i], depth + 1);
  }
};


/**
 * Converts a raw thread into an Endo thread.
 * @param {!endo.proto.binary.ConstThread} rawThread The raw thread to convert.
 * @return {!endo.Thread} The converted thread.
 */
endo.Thread.loadRawThread = function(rawThread) {
  var thread = new endo.Thread(rawThread.getName(),
                               rawThread.getId());
  return thread;
};

//java/com/google/net/rpc3/profiling/jendoscope/web/endo_ui.js
// Copyright 2011 Google Inc. All Rights Reserved.

/**
 * @fileoverview Code to connect up and handle mouse clicks and button press
 * events lives here.
 *
 * TODO(aappleby): This should be turned into a real object.
 *
 * @author aappleby@google.com (Austin Appleby)
 */

goog.provide('endo.ui');

goog.require('endo.Point');
goog.require('endo.Rect');



/**
 * The parent Endoscope console object that this UI is connected to.
 * @type {endo.Console}
 */
endo.ui.console = null;

/**
 * True if the UI should be auto-scrolling.
 * @type {boolean}
 */
endo.ui.scroll = false;

/**
 * True if the UI should be streaming updates from the server.
 * @type {boolean}
 */
endo.ui.stream = false;

/**
 * True if the user is currently dragging the mouse.
 * @type {boolean}
 */
endo.ui.dragging = false;

/**
 * True if the user is currently selecting something.
 * @type {boolean}
 */
endo.ui.selecting = false;

/**
 * True if the user is currently measuring something.
 * @type {boolean}
 */
endo.ui.measuring = false;

/**
 * True if the help popup is visible.
 * @type {boolean}
 */
endo.ui.helpVisible = false;

/**
 * True if the debug popup is visible.
 * @type {boolean}
 */
endo.ui.debugVisible = false;

/**
 * The mouse coordinate as an endo.Point.
 * @type {!endo.Point}
 */
endo.ui.mousePos = new endo.Point(0, 0);

/**
 * The screen-space rectangle that the user has selected with the mouse.
 * @type {endo.Rect}
 */
endo.ui.selectionRect = null;

/**
 * The screen-space rectangle that the user is measuring with the mouse.
 * @type {endo.Rect}
 */
endo.ui.measureRect = null;

/**
 * Initializes all UI-related variables and wire up all events.
 * @param {!endo.Console} console The main console.
 * @param {!HTMLCanvasElement} canvas The canvas to attach events to.
 */
endo.ui.install = function(console, canvas) {

  endo.ui.console = console;

  document.getElementById('load_snapshot').onclick =
      endo.network.requestSnapshot;
  document.getElementById('reset_view').onclick = endo.ui.resetViewport;
  document.getElementById('toggle_help').onclick = endo.ui.toggleHelp;
  document.getElementById('toggle_scroll').onclick = endo.ui.toggleScroll;
  document.getElementById('toggle_stream').onclick = endo.ui.toggleStream;
  document.getElementById('toggle_view').onclick = endo.ui.toggleView;

  document.onkeydown = endo.ui.eventKeydown;

  canvas.onmousewheel = endo.ui.eventMouseWheel;
  canvas.onmousedown = endo.ui.eventMouseDown;
  canvas.onmousemove = endo.ui.eventMouseMove;
  canvas.onmouseup = endo.ui.eventMouseUp;

  canvas.ondblclick = function(event) {
    var eventLocation = new endo.Point(event.offsetX, event.offsetY);
    if (endo.view.doubleclick) endo.view.doubleclick(eventLocation);
  };
};

/**
 * Handles 'reset view' button event.
 */
endo.ui.resetViewport = function() {
  endo.view.resetViewport();
  if (endo.ui.scroll) endo.ui.toggleScroll();
};

/**
 * Toggles the help popup.
 * TODO(aappleby): the help popup needs more actual help in it.
 */
endo.ui.toggleHelp = function() {
  endo.ui.helpVisible = !endo.ui.helpVisible;
  var popup = document.getElementById('help_popup');
  popup.style.display = endo.ui.helpVisible ? 'block' : 'none';
};

/**
 * Toggles the debug popup.
 */
endo.ui.toggleDebug = function() {
  endo.ui.debugVisible = !endo.ui.debugVisible;
  var popup = document.getElementById('debug_popup');
  popup.style.display = endo.ui.debugVisible ? 'block' : 'none';
};

/**
 * Toggles scrolling.
 */
endo.ui.toggleScroll = function() {
  endo.ui.scroll = !endo.ui.scroll;
  var button = document.getElementById('toggle_scroll');
  button.innerText = endo.ui.scroll ? 'Stop Scroll' : 'Start Scroll';
};

/**
 * Toggles streaming updates.
 */
endo.ui.toggleStream = function() {
  endo.ui.stream = !endo.ui.stream;
  var button = document.getElementById('toggle_stream');
  button.innerText = endo.ui.stream ? 'Stop Stream' : 'Start Stream';

  if (endo.ui.stream) {
    endo.network.startStream();
  } else {
    endo.network.stopStream();
  }
};

/**
 * Toggles main view between timeline and bargraph modes.
 */
endo.ui.toggleView = function() {
  endo.ui.console.toggleView();
};

/**
 * Cancels the mouse event so that it isn't accidentally handled by other
 * widgets on the page.
 * (Note - I copied this out of some example code, not entirely sure why it
 * works...)
 * @param {Object} event The mouse event that we're cancelling.
 */
endo.ui.cancelEvent = function(event)
{
  event = event ? event : window.event;
  if (event.stopPropagation) event.stopPropagation();
  if (event.preventDefault) event.preventDefault();
  event.cancelBubble = true;
  event.cancel = true;
  event.returnValue = false;
};

/**
 * Zooms the view in and out when the user moves the mouse wheel.
 * @param {Object} event The mouse wheel event.
 */
endo.ui.eventMouseWheel = function(event) {
  // Compute the zoom amounts in each axis. If shift or alt was held down, zoom
  // vertically. If shift was not held down or alt was held down, zoom
  // horizontally.

  var zoomX = 0.0;
  var zoomY = 0.0;

  if (event.shiftKey || event.altKey) {
    var dir = event.wheelDelta ? event.wheelDelta : -event.detail;
    zoomY = (dir > 0) ? -1.0 : 1.0;
  }

  if (!event.shiftKey || event.altKey) {
    var dir = event.wheelDelta ? event.wheelDelta : -event.detail;
    zoomX = (dir > 0) ? -1.0 : 1.0;
  }

  var eventLocation = new endo.Point(event.offsetX, event.offsetY);
  if (endo.view.zoom) endo.view.zoom(zoomX, zoomY, eventLocation);

  endo.ui.cancelEvent(event);
};

/**
 * Records where the mouse was clicked so we can support click-and-dragging.
 * @param {Object} event The mouse button down event.
 */
endo.ui.eventMouseDown = function(event) {
  var eventPos = new endo.Point(event.offsetX, event.offsetY);

  // If the control key was held down, the user is selecting a region of the
  // timeline.
  if (event.shiftKey) {
    endo.ui.selecting = true;
    endo.ui.markX1 = eventPos.x;
    endo.ui.markY1 = eventPos.y;
  } else if (event.ctrlKey) {
    endo.ui.measuring = true;
    endo.ui.measureX1 = eventPos.x;
    endo.ui.measureY1 = eventPos.y;
  } else {
    // Otherwise we're dragging the timeline.
    endo.ui.dragging = true;
    endo.ui.dragX = eventPos.x;
    endo.ui.dragY = eventPos.y;
  }

  endo.ui.cancelEvent(event);
};

/**
 * Moves the timeline view around when the user clicks and drags the mouse in
 * the timeline view.
 * @param {Object} event The mouse movement event.
 */
endo.ui.eventMouseMove = function(event) {
  var eventPos = new endo.Point(event.offsetX, event.offsetY);
  var viewport = endo.view.getViewport();

  endo.ui.mousePos = new endo.Point(event.offsetX, event.offsetY);

  if (endo.ui.dragging) {
    // Compute motion since last drag event.
    var deltaX = eventPos.x - endo.ui.dragX;
    var deltaY = eventPos.y - endo.ui.dragY;
    endo.ui.dragX = eventPos.x;
    endo.ui.dragY = eventPos.y;

    // Scale motion to viewport.
    deltaX = (deltaX / event.target.width) * viewport.getWidth();
    deltaY = (deltaY / event.target.height) * viewport.getHeight();

    // Translate viewport goal to match motion.
    var newGoal = endo.view.getViewportGoal();
    newGoal.translate(-deltaX, -deltaY);
    endo.view.setViewportGoal(newGoal);
  }

  if (endo.ui.selecting && !endo.ui.scroll) {
    endo.ui.markX2 = eventPos.x;
    endo.ui.markY2 = eventPos.y;
    endo.ui.selectionRect = new endo.Rect(endo.ui.markX1,
                                          endo.ui.markY1,
                                          endo.ui.markX2,
                                          endo.ui.markY2);
  } else {
    endo.ui.selectionRect = null;
  }

  // TODO(aappleby): Measuring a section of the timeline isn't fully
  // implemented yet.
  if (endo.ui.measuring && !endo.ui.scroll) {
    endo.ui.measureX2 = eventPos.x;
    endo.ui.measureY2 = eventPos.y;
    endo.ui.measureRect = new endo.Rect(endo.ui.measureX1,
                                        endo.ui.measureY1,
                                        endo.ui.measureX2,
                                        endo.ui.measureY2);
  } else {
    endo.ui.measureRect = null;
  }

  endo.ui.cancelEvent(event);
};

/**
 * Stops dragging when the user releases the mouse button.
 * @param {Object} event The mouse button up event.
 */
endo.ui.eventMouseUp = function(event) {
  if (endo.ui.dragging) {
    endo.ui.dragging = false;
  }
  if (endo.ui.selecting) {
    if (endo.view.applySelection) endo.view.applySelection();
    endo.ui.selecting = false;
  }
  if (endo.ui.measuring) {
    if (endo.view.applyMeasure) endo.view.applyMeasure();
    endo.ui.measuring = false;
  }
  endo.ui.cancelEvent(event);
};

/**
 * Translates a JQuery keypress event into a text description of the key
 * pressed, including the set of meta keys (ctrl/alt/shift) pressed along with
 * it. Meta keys are added to the description in ctrl -> alt -> shift order, so
 * pressing "shift + alt + ctrl + R" will generate the description
 * "ctrl-alt-shift-r".
 *
 * @param {!KeyboardEvent} event The keypress event to translate.
 * @return {?string} A text description of the keypress event.
 */
endo.ui.translateKeyCode = function(event) {
  var meta = '';
  if (event.shiftKey) meta = 'shift-' + meta;
  if (event.altKey) meta = 'alt-' + meta;
  if (event.ctrlKey) meta = 'ctrl-' + meta;

  // Convert text keys to their lower-case string equivalent.
  if ((event.keyCode >= 65) && (event.keyCode <= 90)) {
    return meta + String.fromCharCode(event.keyCode).toLowerCase();
  }

  // Convert number keys on the keypad to 'numX'.
  if ((event.keyCode >= 96) && (event.keyCode <= 105)) {
    return meta + 'num' + (event.keyCode - 96).toString();
  }

  // Convert other interesting keys to a descriptive name.
  switch (event.keyCode) {
    case 8: return meta + 'backspace';
    case 27: return meta + 'escape';
    case 32: return meta + 'space';
    case 37: return meta + 'left';
    case 38: return meta + 'up';
    case 39: return meta + 'right';
    case 40: return meta + 'down';
    case 106: return meta + 'num*';
    case 107: return meta + 'num+';
    case 109: return meta + 'num-';
    case 110: return meta + 'num.';
    case 111: return meta + 'num/';
    case 192: return meta + '`';
    case 219: return meta + '[';
    case 221: return meta + ']';
  }

  return null;
};

/**
 * Handles keypress events.
 *
 * @param {!KeyboardEvent} event The key press event.
 */
endo.ui.eventKeydown = function(event) {
  var key = endo.ui.translateKeyCode(event);

  if (!key) return;

  // Do _not_ hook key combinations that Chrome uses for navigation.
  if (key == 'alt-left') return;
  if (key == 'alt-right') return;
  if (key == 'ctrl-l') return;
  if (key == 'ctrl-r') return;

  endo.ui.cancelEvent(event);

  switch (key) {
    case 'escape':
    case 'backspace':
      if (endo.view.popViewportGoal) endo.view.popViewportGoal(); break;

    case 's':
      endo.ui.toggleStream(); break;

    case 'space':
      endo.ui.toggleScroll(); break;

    case 'v':
      endo.ui.resetViewport(); break;

    case 'ctrl-v':
      endo.ui.toggleView(); break;

    case 'h':
      endo.ui.toggleHelp(); break;

    case '`':
      endo.ui.toggleDebug(); break;

    case 'num8':
    case 'up':
      endo.view.pan(0, -1); break;

    case 'num2':
    case 'down':
      endo.view.pan(0, 1); break;

    case 'num4':
    case 'left':
      endo.view.pan(-1, 0); break;

    case 'num6':
    case 'right':
      endo.view.pan(1, 0); break;

    case 'num7':
    case '[':
    case 'shift-left':
      endo.view.zoom(1, 0); break;

    case 'num9':
    case ']':
    case 'shift-right':
      endo.view.zoom(-1, 0); break;

    case 'num3':
    case 'shift-up':
      endo.view.zoom(0, -1); break;

    case 'num1':
    case 'shift-down':
      endo.view.zoom(0, 1); break;

    case 'num+':
    case 'num5':
      endo.view.zoom(-1, -1); break;

    case 'num-':
    case 'num0':
      endo.view.zoom(1, 1); break;
  }
};

